(herald wrap-decrypt (bound 8))

(comment "CPSA 4.3.0")
(comment "All input read from tst/wrap_decrypt.lsp")

(defprotocol wrap-decrypt basic
  (defrole make
    (vars (k skey) (old mesg) (lk locn))
    (trace (load lk old) (stor lk (cat k "init")) (send (hash k)))
    (uniq-gen k))
  (defrole set-wrap
    (vars (k skey) (cur mesg) (lk locn))
    (trace (load lk (cat k cur)) (stor lk (cat k "wrap")))
    (gen-st (cat k cur))
    (facts (neq cur "wrap")))
  (defrole set-decrypt
    (vars (k skey) (cur mesg) (lk locn))
    (trace (load lk (cat k cur)) (stor lk (cat k "decrypt")))
    (gen-st (cat k cur))
    (facts (neq cur "decrypt")))
  (defrole wrap
    (vars (k0 k1 skey) (lk locn) (cur mesg))
    (trace (recv (hash k0)) (recv (hash k1)) (load lk (cat k0 cur))
      (load lk (cat k1 "wrap")) (send (enc k0 k1)))
    (gen-st (cat k0 cur))
    (gen-st (cat k1 "wrap")))
  (defrole decrypt
    (vars (x mesg) (k skey) (lk locn))
    (trace (recv (enc x k)) (recv (hash k)) (load lk (cat k "decrypt"))
      (send x))
    (gen-st (cat k "decrypt")))
  (defrule cakeRule
    (forall ((z0 z1 z2 strd) (i0 i1 i2 indx))
      (implies
        (and (trans z0 i0) (trans z1 i1) (leads-to z0 i0 z1 i1)
          (leads-to z0 i0 z2 i2) (prec z1 i1 z2 i2))
        (false))))
  (defrule no-interruption
    (forall ((z0 z1 z2 strd) (i0 i1 i2 indx))
      (implies
        (and (leads-to z0 i0 z2 i2) (trans z1 i1)
          (same-locn z0 i0 z1 i1) (prec z0 i0 z1 i1) (prec z1 i1 z2 i2))
        (false))))
  (defrule neqRl_mesg
    (forall ((x mesg)) (implies (fact neq x x) (false))))
  (defrule neqRl_strd
    (forall ((x strd)) (implies (fact neq x x) (false))))
  (defrule neqRl_indx
    (forall ((x indx)) (implies (fact neq x x) (false))))
  (defrule scissorsRule
    (forall ((z0 z1 z2 strd) (i0 i1 i2 indx))
      (implies
        (and (trans z0 i0) (trans z1 i1) (trans z2 i2)
          (leads-to z0 i0 z1 i1) (leads-to z0 i0 z2 i2))
        (and (= z1 z2) (= i1 i2)))))
  (defrule trRl_make-at-1
    (forall ((z strd)) (implies (p "make" z 2) (trans z 1))))
  (defrule trRl_make-at-0
    (forall ((z strd)) (implies (p "make" z 1) (trans z 0))))
  (defrule fact-set-wrap-neq0
    (forall ((z strd) (cur mesg))
      (implies
        (and (p "set-wrap" z 1) (p "set-wrap" "cur" z cur))
        (fact neq cur "wrap"))))
  (defrule gen-st-set-wrap-0
    (forall ((z strd) (cur mesg) (k skey))
      (implies
        (and (p "set-wrap" z 1) (p "set-wrap" "cur" z cur)
          (p "set-wrap" "k" z k))
        (gen-st (cat k cur)))))
  (defrule trRl_set-wrap-at-1
    (forall ((z strd)) (implies (p "set-wrap" z 2) (trans z 1))))
  (defrule trRl_set-wrap-at-0
    (forall ((z strd)) (implies (p "set-wrap" z 1) (trans z 0))))
  (defrule fact-set-decrypt-neq0
    (forall ((z strd) (cur mesg))
      (implies
        (and (p "set-decrypt" z 1) (p "set-decrypt" "cur" z cur))
        (fact neq cur "decrypt"))))
  (defrule gen-st-set-decrypt-0
    (forall ((z strd) (cur mesg) (k skey))
      (implies
        (and (p "set-decrypt" z 1) (p "set-decrypt" "cur" z cur)
          (p "set-decrypt" "k" z k))
        (gen-st (cat k cur)))))
  (defrule trRl_set-decrypt-at-1
    (forall ((z strd)) (implies (p "set-decrypt" z 2) (trans z 1))))
  (defrule trRl_set-decrypt-at-0
    (forall ((z strd)) (implies (p "set-decrypt" z 1) (trans z 0))))
  (defrule gen-st-wrap-1
    (forall ((z strd) (cur mesg) (k0 skey))
      (implies
        (and (p "wrap" z 3) (p "wrap" "cur" z cur) (p "wrap" "k0" z k0))
        (gen-st (cat k0 cur)))))
  (defrule gen-st-wrap-0
    (forall ((z strd) (k1 skey))
      (implies
        (and (p "wrap" z 2) (p "wrap" "k1" z k1))
        (gen-st (cat k1 "wrap")))))
  (defrule gen-st-decrypt-0
    (forall ((z strd) (k skey))
      (implies
        (and (p "decrypt" z 1) (p "decrypt" "k" z k))
        (gen-st (cat k "decrypt")))))
  (defrule shearsRule
    (forall ((z0 z1 z2 strd) (i0 i1 i2 indx))
      (implies
        (and (trans z0 i0) (trans z1 i1) (trans z2 i2)
          (leads-to z0 i0 z1 i1) (same-locn z0 i0 z2 i2)
          (prec z0 i0 z2 i2))
        (or (and (= z1 z2) (= i1 i2)) (prec z1 i1 z2 i2)))))
  (defrule invShearsRule
    (forall ((z0 z1 z2 strd) (i0 i1 i2 indx))
      (implies
        (and (trans z0 i0) (trans z1 i1) (same-locn z0 i0 z1 i1)
          (leads-to z1 i1 z2 i2) (prec z0 i0 z2 i2))
        (or (and (= z0 z1) (= i0 i1)) (prec z0 i0 z1 i1))))))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k kp skey)
    (lk lk-0 lk-1 lk-2 lk-3 lk-4 locn))
  (deflistener k)
  (defstrand decrypt 4 (x k) (k kp) (lk lk))
  (defstrand make 2 (old old) (k kp) (lk lk-0))
  (defstrand set-decrypt 1 (cur "wrap") (k kp) (lk lk-1))
  (defstrand set-wrap 1 (cur "init") (k kp) (lk lk-2))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 kp) (lk lk-3))
  (defstrand make 2 (old old-0) (k k) (lk lk-4))
  (traces ((recv k) (send k))
    ((recv (enc k kp)) (recv (hash kp)) (load lk (cat pt kp "decrypt"))
      (send k))
    ((load lk-0 (cat pt-0 old)) (stor lk-0 (cat pt-1 kp "init")))
    ((load lk-1 (cat pt-2 kp "wrap")))
    ((load lk-2 (cat pt-3 kp "init")))
    ((recv (hash k)) (recv (hash kp)) (load lk-3 (cat pt-4 k "init"))
      (load lk-3 (cat pt-5 kp "wrap")) (send (enc k kp)))
    ((load lk-4 (cat pt-6 old-0)) (stor lk-4 (cat pt-7 k "init"))))
  (label 0)
  (realized)
  (origs (pt-7 (6 1)) (pt-1 (2 1)))
  (comment "Not closed under rules"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k kp skey)
    (lk lk-0 lk-1 lk-2 lk-3 lk-4 locn))
  (deflistener k)
  (defstrand decrypt 4 (x k) (k kp) (lk lk))
  (defstrand make 2 (old old) (k kp) (lk lk-0))
  (defstrand set-decrypt 1 (cur "wrap") (k kp) (lk lk-1))
  (defstrand set-wrap 1 (cur "init") (k kp) (lk lk-2))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 kp) (lk lk-3))
  (defstrand make 2 (old old-0) (k k) (lk lk-4))
  (genStV (cat k "init") (cat kp "decrypt") (cat kp "init")
    (cat kp "wrap"))
  (facts (neq "init" "decrypt") (neq "wrap" "decrypt")
    (neq "init" "wrap") (neq "wrap" "wrap"))
  (rule gen-st-decrypt-0 gen-st-wrap-1 fact-set-decrypt-neq0
    gen-st-set-wrap-0 fact-set-wrap-neq0 trRl_make-at-0 trRl_make-at-1)
  (traces ((recv k) (send k))
    ((recv (enc k kp)) (recv (hash kp)) (load lk (cat pt kp "decrypt"))
      (send k))
    ((load lk-0 (cat pt-0 old)) (stor lk-0 (cat pt-1 kp "init")))
    ((load lk-1 (cat pt-2 kp "wrap")))
    ((load lk-2 (cat pt-3 kp "init")))
    ((recv (hash k)) (recv (hash kp)) (load lk-3 (cat pt-4 k "init"))
      (load lk-3 (cat pt-5 kp "wrap")) (send (enc k kp)))
    ((load lk-4 (cat pt-6 old-0)) (stor lk-4 (cat pt-7 k "init"))))
  (label 1)
  (parent 0)
  (unrealized (1 2) (3 0) (4 0) (5 2) (5 3))
  (dead)
  (origs (pt-7 (6 1)) (pt-1 (2 1)))
  (comment "empty cohort"))

(comment "Nothing left to do")

(defprotocol wrap-decrypt basic
  (defrole make
    (vars (k skey) (old mesg) (lk locn))
    (trace (load lk old) (stor lk (cat k "init")) (send (hash k)))
    (uniq-gen k))
  (defrole set-wrap
    (vars (k skey) (cur mesg) (lk locn))
    (trace (load lk (cat k cur)) (stor lk (cat k "wrap")))
    (gen-st (cat k cur))
    (facts (neq cur "wrap")))
  (defrole set-decrypt
    (vars (k skey) (cur mesg) (lk locn))
    (trace (load lk (cat k cur)) (stor lk (cat k "decrypt")))
    (gen-st (cat k cur))
    (facts (neq cur "decrypt")))
  (defrole wrap
    (vars (k0 k1 skey) (lk locn) (cur mesg))
    (trace (recv (hash k0)) (recv (hash k1)) (load lk (cat k0 cur))
      (load lk (cat k1 "wrap")) (send (enc k0 k1)))
    (gen-st (cat k0 cur))
    (gen-st (cat k1 "wrap")))
  (defrole decrypt
    (vars (x mesg) (k skey) (lk locn))
    (trace (recv (enc x k)) (recv (hash k)) (load lk (cat k "decrypt"))
      (send x))
    (gen-st (cat k "decrypt")))
  (defrule cakeRule
    (forall ((z0 z1 z2 strd) (i0 i1 i2 indx))
      (implies
        (and (trans z0 i0) (trans z1 i1) (leads-to z0 i0 z1 i1)
          (leads-to z0 i0 z2 i2) (prec z1 i1 z2 i2))
        (false))))
  (defrule no-interruption
    (forall ((z0 z1 z2 strd) (i0 i1 i2 indx))
      (implies
        (and (leads-to z0 i0 z2 i2) (trans z1 i1)
          (same-locn z0 i0 z1 i1) (prec z0 i0 z1 i1) (prec z1 i1 z2 i2))
        (false))))
  (defrule neqRl_mesg
    (forall ((x mesg)) (implies (fact neq x x) (false))))
  (defrule neqRl_strd
    (forall ((x strd)) (implies (fact neq x x) (false))))
  (defrule neqRl_indx
    (forall ((x indx)) (implies (fact neq x x) (false))))
  (defrule scissorsRule
    (forall ((z0 z1 z2 strd) (i0 i1 i2 indx))
      (implies
        (and (trans z0 i0) (trans z1 i1) (trans z2 i2)
          (leads-to z0 i0 z1 i1) (leads-to z0 i0 z2 i2))
        (and (= z1 z2) (= i1 i2)))))
  (defrule trRl_make-at-1
    (forall ((z strd)) (implies (p "make" z 2) (trans z 1))))
  (defrule trRl_make-at-0
    (forall ((z strd)) (implies (p "make" z 1) (trans z 0))))
  (defrule fact-set-wrap-neq0
    (forall ((z strd) (cur mesg))
      (implies
        (and (p "set-wrap" z 1) (p "set-wrap" "cur" z cur))
        (fact neq cur "wrap"))))
  (defrule gen-st-set-wrap-0
    (forall ((z strd) (cur mesg) (k skey))
      (implies
        (and (p "set-wrap" z 1) (p "set-wrap" "cur" z cur)
          (p "set-wrap" "k" z k))
        (gen-st (cat k cur)))))
  (defrule trRl_set-wrap-at-1
    (forall ((z strd)) (implies (p "set-wrap" z 2) (trans z 1))))
  (defrule trRl_set-wrap-at-0
    (forall ((z strd)) (implies (p "set-wrap" z 1) (trans z 0))))
  (defrule fact-set-decrypt-neq0
    (forall ((z strd) (cur mesg))
      (implies
        (and (p "set-decrypt" z 1) (p "set-decrypt" "cur" z cur))
        (fact neq cur "decrypt"))))
  (defrule gen-st-set-decrypt-0
    (forall ((z strd) (cur mesg) (k skey))
      (implies
        (and (p "set-decrypt" z 1) (p "set-decrypt" "cur" z cur)
          (p "set-decrypt" "k" z k))
        (gen-st (cat k cur)))))
  (defrule trRl_set-decrypt-at-1
    (forall ((z strd)) (implies (p "set-decrypt" z 2) (trans z 1))))
  (defrule trRl_set-decrypt-at-0
    (forall ((z strd)) (implies (p "set-decrypt" z 1) (trans z 0))))
  (defrule gen-st-wrap-1
    (forall ((z strd) (cur mesg) (k0 skey))
      (implies
        (and (p "wrap" z 3) (p "wrap" "cur" z cur) (p "wrap" "k0" z k0))
        (gen-st (cat k0 cur)))))
  (defrule gen-st-wrap-0
    (forall ((z strd) (k1 skey))
      (implies
        (and (p "wrap" z 2) (p "wrap" "k1" z k1))
        (gen-st (cat k1 "wrap")))))
  (defrule gen-st-decrypt-0
    (forall ((z strd) (k skey))
      (implies
        (and (p "decrypt" z 1) (p "decrypt" "k" z k))
        (gen-st (cat k "decrypt")))))
  (defrule shearsRule
    (forall ((z0 z1 z2 strd) (i0 i1 i2 indx))
      (implies
        (and (trans z0 i0) (trans z1 i1) (trans z2 i2)
          (leads-to z0 i0 z1 i1) (same-locn z0 i0 z2 i2)
          (prec z0 i0 z2 i2))
        (or (and (= z1 z2) (= i1 i2)) (prec z1 i1 z2 i2)))))
  (defrule invShearsRule
    (forall ((z0 z1 z2 strd) (i0 i1 i2 indx))
      (implies
        (and (trans z0 i0) (trans z1 i1) (same-locn z0 i0 z1 i1)
          (leads-to z1 i1 z2 i2) (prec z0 i0 z2 i2))
        (or (and (= z0 z1) (= i0 i1)) (prec z0 i0 z1 i1))))))

(defskeleton wrap-decrypt
  (vars (k skey))
  (deflistener k)
  (pen-non-orig k)
  (traces ((recv k) (send k)))
  (label 2)
  (unrealized (0 0))
  (origs)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old mesg) (pt pt-0 pval) (k skey) (lk locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (precedes ((1 1) (0 0)))
  (pen-non-orig k)
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation nonce-test (added-strand make 2) k (0 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init"))))
  (label 3)
  (parent 2)
  (unrealized (0 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old mesg) (pt pt-0 pt-1 pval) (k skey) (lk locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule fact-set-decrypt-neq0 trRl_set-wrap-at-1 gen-st-set-wrap-0
    fact-set-wrap-neq0 trRl_make-at-0)
  (operation nonce-test (added-strand set-wrap 2) k (0 0)
    (ch-msg lk (cat pt-0 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap"))))
  (label 4)
  (parent 3)
  (unrealized (0 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old mesg) (pt pt-0 pt-1 pval) (k skey) (lk locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-decrypt-at-1 fact-set-decrypt-neq0 gen-st-set-wrap-0
    fact-set-wrap-neq0 trRl_make-at-0)
  (operation nonce-test (added-strand set-decrypt 2) k (0 0)
    (ch-msg lk (cat pt-0 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt"))))
  (label 5)
  (parent 3)
  (unrealized (0 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old mesg) (pt pt-0 pt-1 pval) (k k1 skey) (lk locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (2 2)) ((2 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap"))
  (rule gen-st-wrap-0 gen-st-wrap-1)
  (operation nonce-test (added-strand wrap 5) k (0 0)
    (ch-msg lk (cat pt-0 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k1 "wrap")) (send (enc k k1))))
  (label 6)
  (parent 3)
  (unrealized (2 0) (2 3))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old mesg) (pt pt-0 pt-1 pt-2 pval) (k skey) (lk locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "wrap") (k k) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (3 0)) ((3 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "wrap" "decrypt") (neq "wrap" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-decrypt-at-1 fact-set-decrypt-neq0 gen-st-set-wrap-0
    fact-set-wrap-neq0 trRl_make-at-0)
  (operation nonce-test (added-strand set-decrypt 2) k (0 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk (cat pt-1 k "wrap")) (stor lk (cat pt-2 k "decrypt"))))
  (label 7)
  (parent 4)
  (unrealized (0 0))
  (dead)
  (comment "empty cohort"))

(defskeleton wrap-decrypt
  (vars (old mesg) (pt pt-0 pt-1 pt-2 pval) (k skey) (lk locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand set-wrap 2 (cur "wrap") (k k) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (3 0)) ((3 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "wrap" "decrypt") (neq "wrap" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule fact-set-decrypt-neq0 trRl_set-wrap-at-1 gen-st-set-wrap-0
    fact-set-wrap-neq0 trRl_make-at-0)
  (operation nonce-test (added-strand set-wrap 2) k (0 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk (cat pt-1 k "wrap")) (stor lk (cat pt-2 k "wrap"))))
  (label 8)
  (parent 4)
  (unrealized (0 0))
  (dead)
  (comment "empty cohort"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pval) (k skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (0 0)) ((3 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation nonce-test (added-strand make 2) k (0 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))))
  (label 9)
  (parent 4)
  (unrealized (0 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old mesg) (pt pt-0 pt-1 pt-2 pval) (k k1 skey) (lk locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (3 2)) ((3 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0 gen-st-wrap-1)
  (operation nonce-test (added-strand wrap 5) k (0 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1))))
  (label 10)
  (parent 4)
  (unrealized (3 0) (3 3))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old mesg) (pt pt-0 pt-1 pt-2 pval) (k k1 skey) (lk locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (3 2)) ((2 1) (0 0)) ((3 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1))))
  (label 11)
  (parent 4)
  (unrealized (3 0) (3 3))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old mesg) (pt pt-0 pt-1 pt-2 pval) (k skey) (lk locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand set-wrap 2 (cur "decrypt") (k k) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (3 0)) ((3 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init"))
  (facts (neq "decrypt" "decrypt") (neq "decrypt" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule fact-set-decrypt-neq0 trRl_set-wrap-at-1 gen-st-set-wrap-0
    fact-set-wrap-neq0 trRl_make-at-0)
  (operation nonce-test (added-strand set-wrap 2) k (0 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "decrypt")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((load lk (cat pt-1 k "decrypt")) (stor lk (cat pt-2 k "wrap"))))
  (label 12)
  (parent 5)
  (unrealized (0 0))
  (dead)
  (comment "empty cohort"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pval) (k skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (0 0)) ((3 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation nonce-test (added-strand make 2) k (0 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "decrypt")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))))
  (label 13)
  (parent 5)
  (seen 21)
  (unrealized (0 0))
  (comment "6 in cohort - 5 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old mesg) (pt pt-0 pt-1 pt-2 pval) (k skey) (lk locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "decrypt") (k k) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (3 0)) ((3 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init"))
  (facts (neq "decrypt" "decrypt") (neq "decrypt" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-decrypt-at-1 fact-set-decrypt-neq0 gen-st-set-wrap-0
    fact-set-wrap-neq0 trRl_make-at-0)
  (operation nonce-test (added-strand set-decrypt 2) k (0 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "decrypt")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((load lk (cat pt-1 k "decrypt")) (stor lk (cat pt-2 k "decrypt"))))
  (label 14)
  (parent 5)
  (unrealized (0 0))
  (dead)
  (comment "empty cohort"))

(defskeleton wrap-decrypt
  (vars (old mesg) (pt pt-0 pt-1 pt-2 pval) (k k1 skey) (lk locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (3 2)) ((3 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0 gen-st-wrap-1)
  (operation nonce-test (added-strand wrap 5) k (0 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "decrypt")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "decrypt"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1))))
  (label 15)
  (parent 5)
  (unrealized (3 0) (3 3))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old mesg) (pt pt-0 pt-1 pt-2 pval) (k k1 skey) (lk locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (3 2)) ((2 1) (0 0)) ((3 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "decrypt")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1))))
  (label 16)
  (parent 5)
  (unrealized (3 0) (3 3))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old mesg) (pt pt-0 pt-1 pval) (k k1 skey) (lk locn))
  (deflistener k)
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old) (k k) (lk lk))
  (precedes ((1 4) (0 0)) ((2 2) (1 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap"))
  (operation encryption-test (displaced 1 3 make 3) (hash k) (2 0))
  (traces ((recv k) (send k))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt k1 "wrap")) (send (enc k k1)))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init"))
      (send (hash k))))
  (label 17)
  (parent 6)
  (seen 43)
  (unrealized (1 3))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (precedes ((1 1) (2 2)) ((2 4) (0 0)) ((3 2) (2 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation encryption-test (added-strand make 3) (hash k) (2 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k))))
  (label 18)
  (parent 6)
  (seen 44)
  (unrealized (2 3))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old mesg) (pt pt-0 pt-1 pval) (k k1 skey) (lk locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (precedes ((1 1) (2 2)) ((2 4) (0 0)) ((3 1) (2 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap"))
  (operation encryption-test (added-listener k) (hash k) (2 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k)))
  (label 19)
  (parent 6)
  (unrealized (2 3) (3 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "wrap") (k k) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (4 0)) ((3 1) (0 0)) ((4 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "wrap" "decrypt") (neq "wrap" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-decrypt-at-1 fact-set-decrypt-neq0 gen-st-set-wrap-0
    fact-set-wrap-neq0 trRl_make-at-0)
  (operation nonce-test (added-strand set-decrypt 2) k (0 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk (cat pt-1 k "wrap")) (stor lk (cat pt-4 k "decrypt"))))
  (label 20)
  (parent 9)
  (unrealized (0 0))
  (dead)
  (comment "empty cohort"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (0 0)) ((3 1) (4 0)) ((4 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-decrypt-at-1 trRl_make-at-0)
  (operation nonce-test (added-strand set-decrypt 2) k (0 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init"))
      (stor lk-0 (cat pt-4 k "decrypt"))))
  (label 21)
  (parent 9)
  (unrealized (0 0))
  (comment "9 in cohort - 9 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "wrap") (k k) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (4 0)) ((3 1) (0 0)) ((4 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "wrap" "decrypt") (neq "wrap" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule fact-set-decrypt-neq0 trRl_set-wrap-at-1 gen-st-set-wrap-0
    fact-set-wrap-neq0 trRl_make-at-0)
  (operation nonce-test (added-strand set-wrap 2) k (0 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk (cat pt-1 k "wrap")) (stor lk (cat pt-4 k "wrap"))))
  (label 22)
  (parent 9)
  (unrealized (0 0))
  (dead)
  (comment "empty cohort"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (0 0)) ((3 1) (4 2)) ((4 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-4 k1 "wrap")) (send (enc k k1))))
  (label 23)
  (parent 9)
  (unrealized (4 0) (4 3))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (4 2)) ((3 1) (0 0)) ((4 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0 gen-st-wrap-1)
  (operation nonce-test (added-strand wrap 5) k (0 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1))))
  (label 24)
  (parent 9)
  (seen 27)
  (unrealized (4 0) (4 3))
  (comment "4 in cohort - 3 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (4 2)) ((2 1) (0 0)) ((3 1) (0 0))
    ((4 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1))))
  (label 25)
  (parent 9)
  (seen 30)
  (unrealized (4 0) (4 3))
  (comment "4 in cohort - 3 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old mesg) (pt pt-0 pt-1 pt-2 pval) (k k1 skey) (lk locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old) (k k) (lk lk))
  (precedes ((1 1) (2 2)) ((2 4) (0 0)) ((3 1) (1 0)) ((3 2) (2 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 1 4 make 3) (hash k) (3 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt-2 k "init")) (stor lk (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt k "wrap"))
      (load lk (cat pt-0 k1 "wrap")) (send (enc k k1)))
    ((load lk (cat pt-1 old)) (stor lk (cat pt-2 k "init"))
      (send (hash k))))
  (label 26)
  (parent 10)
  (seen 66 67)
  (unrealized (2 3))
  (comment "4 in cohort - 2 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (3 2)) ((3 4) (0 0)) ((4 2) (3 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation encryption-test (added-strand make 3) (hash k) (3 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init"))
      (send (hash k))))
  (label 27)
  (parent 10)
  (seen 68 69)
  (unrealized (3 3))
  (comment "4 in cohort - 2 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old mesg) (pt pt-0 pt-1 pt-2 pval) (k k1 skey) (lk locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (precedes ((1 1) (2 0)) ((2 1) (3 2)) ((3 4) (0 0)) ((4 1) (3 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (added-listener k) (hash k) (3 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k)))
  (label 28)
  (parent 10)
  (unrealized (3 3) (4 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old mesg) (pt pt-0 pt-1 pt-2 pval) (k k1 skey) (lk locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old) (k k) (lk lk))
  (precedes ((1 1) (0 0)) ((2 4) (0 0)) ((3 1) (1 0)) ((3 2) (2 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 1 4 make 3) (hash k) (3 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt-2 k "init")) (stor lk (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-2 k "init"))
      (load lk (cat pt-0 k1 "wrap")) (send (enc k k1)))
    ((load lk (cat pt-1 old)) (stor lk (cat pt-2 k "init"))
      (send (hash k))))
  (label 29)
  (parent 11)
  (seen 72 73)
  (unrealized (2 3))
  (comment "7 in cohort - 2 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((1 1) (3 2)) ((2 1) (0 0)) ((3 4) (0 0))
    ((4 2) (3 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation encryption-test (added-strand make 3) (hash k) (3 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init"))
      (send (hash k))))
  (label 30)
  (parent 11)
  (seen 74 75)
  (unrealized (3 3))
  (comment "7 in cohort - 2 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old mesg) (pt pt-0 pt-1 pt-2 pval) (k k1 skey) (lk locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (precedes ((1 1) (2 0)) ((1 1) (3 2)) ((2 1) (0 0)) ((3 4) (0 0))
    ((4 1) (3 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (added-listener k) (hash k) (3 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k)))
  (label 31)
  (parent 11)
  (unrealized (3 3) (4 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "decrypt") (k k) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (4 0)) ((3 1) (0 0)) ((4 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init"))
  (facts (neq "decrypt" "decrypt") (neq "decrypt" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule fact-set-decrypt-neq0 trRl_set-wrap-at-1 gen-st-set-wrap-0
    fact-set-wrap-neq0 trRl_make-at-0)
  (operation nonce-test (added-strand set-wrap 2) k (0 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "decrypt"))
    (ch-msg lk-0 (cat pt-3 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk (cat pt-1 k "decrypt")) (stor lk (cat pt-4 k "wrap"))))
  (label 32)
  (parent 13)
  (unrealized (0 0))
  (dead)
  (comment "empty cohort"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (0 0)) ((3 1) (4 2)) ((4 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "decrypt"))
    (ch-msg lk-0 (cat pt-3 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-4 k1 "wrap")) (send (enc k k1))))
  (label 33)
  (parent 13)
  (unrealized (4 0) (4 3))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "decrypt") (k k) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (4 0)) ((3 1) (0 0)) ((4 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init"))
  (facts (neq "decrypt" "decrypt") (neq "decrypt" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-decrypt-at-1 fact-set-decrypt-neq0 gen-st-set-wrap-0
    fact-set-wrap-neq0 trRl_make-at-0)
  (operation nonce-test (added-strand set-decrypt 2) k (0 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "decrypt"))
    (ch-msg lk-0 (cat pt-3 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk (cat pt-1 k "decrypt")) (stor lk (cat pt-4 k "decrypt"))))
  (label 34)
  (parent 13)
  (unrealized (0 0))
  (dead)
  (comment "empty cohort"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (4 2)) ((3 1) (0 0)) ((4 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0 gen-st-wrap-1)
  (operation nonce-test (added-strand wrap 5) k (0 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "decrypt"))
    (ch-msg lk-0 (cat pt-3 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "decrypt"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1))))
  (label 35)
  (parent 13)
  (seen 38)
  (unrealized (4 0) (4 3))
  (comment "4 in cohort - 3 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (4 2)) ((2 1) (0 0)) ((3 1) (0 0))
    ((4 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "decrypt"))
    (ch-msg lk-0 (cat pt-3 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1))))
  (label 36)
  (parent 13)
  (seen 41)
  (unrealized (4 0) (4 3))
  (comment "4 in cohort - 3 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old mesg) (pt pt-0 pt-1 pt-2 pval) (k k1 skey) (lk locn))
  (deflistener k)
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old) (k k) (lk lk))
  (precedes ((1 1) (2 2)) ((2 4) (0 0)) ((3 1) (1 0)) ((3 2) (2 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 1 4 make 3) (hash k) (3 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt-2 k "init")) (stor lk (cat pt k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt k "decrypt"))
      (load lk (cat pt-0 k1 "wrap")) (send (enc k k1)))
    ((load lk (cat pt-1 old)) (stor lk (cat pt-2 k "init"))
      (send (hash k))))
  (label 37)
  (parent 15)
  (seen 88)
  (unrealized (2 3))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (3 2)) ((3 4) (0 0)) ((4 2) (3 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation encryption-test (added-strand make 3) (hash k) (3 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "decrypt"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init"))
      (send (hash k))))
  (label 38)
  (parent 15)
  (seen 89)
  (unrealized (3 3))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old mesg) (pt pt-0 pt-1 pt-2 pval) (k k1 skey) (lk locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (precedes ((1 1) (2 0)) ((2 1) (3 2)) ((3 4) (0 0)) ((4 1) (3 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (added-listener k) (hash k) (3 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "decrypt"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k)))
  (label 39)
  (parent 15)
  (unrealized (3 3) (4 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old mesg) (pt pt-0 pt-1 pt-2 pval) (k k1 skey) (lk locn))
  (deflistener k)
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old) (k k) (lk lk))
  (precedes ((1 1) (0 0)) ((2 4) (0 0)) ((3 1) (1 0)) ((3 2) (2 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 1 4 make 3) (hash k) (3 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt-2 k "init")) (stor lk (cat pt k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-2 k "init"))
      (load lk (cat pt-0 k1 "wrap")) (send (enc k k1)))
    ((load lk (cat pt-1 old)) (stor lk (cat pt-2 k "init"))
      (send (hash k))))
  (label 40)
  (parent 16)
  (seen 92)
  (unrealized (2 3))
  (comment "4 in cohort - 1 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((1 1) (3 2)) ((2 1) (0 0)) ((3 4) (0 0))
    ((4 2) (3 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation encryption-test (added-strand make 3) (hash k) (3 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init"))
      (send (hash k))))
  (label 41)
  (parent 16)
  (seen 93)
  (unrealized (3 3))
  (comment "4 in cohort - 1 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old mesg) (pt pt-0 pt-1 pt-2 pval) (k k1 skey) (lk locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (precedes ((1 1) (2 0)) ((1 1) (3 2)) ((2 1) (0 0)) ((3 4) (0 0))
    ((4 1) (3 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (added-listener k) (hash k) (3 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k)))
  (label 42)
  (parent 16)
  (unrealized (3 3) (4 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old cur mesg) (pt pt-0 pt-1 pt-2 pval) (k k1 skey) (lk locn))
  (deflistener k)
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur cur) (k k1) (lk lk))
  (precedes ((1 4) (0 0)) ((2 1) (3 0)) ((2 2) (1 0)) ((3 1) (1 3)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 cur) (cat k1 "wrap"))
  (facts (neq cur "decrypt") (neq cur "wrap"))
  (rule invShearsRule fact-set-decrypt-neq0 trRl_set-wrap-at-1
    gen-st-set-wrap-0 fact-set-wrap-neq0 trRl_make-at-0)
  (operation channel-test (added-strand set-wrap 2)
    (ch-msg lk (cat pt k1 "wrap")) (1 3))
  (traces ((recv k) (send k))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt k1 "wrap")) (send (enc k k1)))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init"))
      (send (hash k)))
    ((load lk (cat pt-2 k1 cur)) (stor lk (cat pt k1 "wrap"))))
  (label 43)
  (parent 17)
  (seen 72 96)
  (unrealized (3 0))
  (comment "4 in cohort - 1 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 cur mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur cur) (k k1) (lk lk))
  (precedes ((1 1) (2 2)) ((1 1) (4 0)) ((2 4) (0 0)) ((3 2) (2 0))
    ((4 1) (2 3)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 cur) (cat k1 "wrap"))
  (facts (neq cur "decrypt") (neq cur "wrap"))
  (rule invShearsRule fact-set-decrypt-neq0 trRl_set-wrap-at-1
    gen-st-set-wrap-0 fact-set-wrap-neq0 trRl_make-at-0)
  (operation channel-test (added-strand set-wrap 2)
    (ch-msg lk (cat pt-1 k1 "wrap")) (2 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk (cat pt-4 k1 cur)) (stor lk (cat pt-1 k1 "wrap"))))
  (label 44)
  (parent 18)
  (seen 72 74 97)
  (unrealized (4 0))
  (comment "6 in cohort - 1 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old mesg) (pt pt-0 pt-1 pval) (k k1 skey) (lk locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (precedes ((1 1) (3 0)) ((2 4) (0 0)) ((3 1) (2 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap"))
  (operation nonce-test (displaced 4 1 make 2) k (3 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k)))
  (label 45)
  (parent 19)
  (unrealized (2 3) (3 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (precedes ((1 1) (2 2)) ((2 4) (0 0)) ((3 1) (2 0)) ((4 1) (3 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation nonce-test (added-strand make 2) k (3 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))))
  (label 46)
  (parent 19)
  (seen 98)
  (unrealized (2 3) (3 0))
  (comment "4 in cohort - 3 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "decrypt") (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (0 0)) ((3 1) (4 0)) ((4 1) (5 0))
    ((5 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init"))
  (facts (neq "decrypt" "decrypt") (neq "decrypt" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-decrypt-at-1 fact-set-decrypt-neq0 gen-st-set-wrap-0
    fact-set-wrap-neq0 trRl_make-at-0)
  (operation nonce-test (added-strand set-decrypt 2) k (0 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init"))
    (ch-msg lk-0 (cat pt-4 k "decrypt")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((load lk-0 (cat pt-4 k "decrypt"))
      (stor lk-0 (cat pt-5 k "decrypt"))))
  (label 47)
  (parent 21)
  (unrealized (0 0))
  (dead)
  (comment "empty cohort"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "wrap") (k k) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (5 0)) ((3 1) (4 0)) ((4 1) (0 0))
    ((5 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "wrap" "decrypt") (neq "wrap" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule fact-set-decrypt-neq0 trRl_set-wrap-at-1 gen-st-set-wrap-0
    fact-set-wrap-neq0 trRl_make-at-0)
  (operation nonce-test (added-strand set-wrap 2) k (0 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init"))
    (ch-msg lk-0 (cat pt-4 k "decrypt")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((load lk (cat pt-1 k "wrap")) (stor lk (cat pt-5 k "wrap"))))
  (label 48)
  (parent 21)
  (unrealized (0 0))
  (dead)
  (comment "empty cohort"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((2 1) (0 0)) ((3 1) (4 0)) ((4 1) (0 0))
    ((5 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation nonce-test (added-strand make 2) k (0 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init"))
    (ch-msg lk-0 (cat pt-4 k "decrypt")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init"))))
  (label 49)
  (parent 21)
  (unrealized (0 0))
  (comment "9 in cohort - 9 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "decrypt") (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (0 0)) ((3 1) (4 0)) ((4 1) (5 0))
    ((5 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init"))
  (facts (neq "decrypt" "decrypt") (neq "decrypt" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule fact-set-decrypt-neq0 trRl_set-wrap-at-1 gen-st-set-wrap-0
    fact-set-wrap-neq0 trRl_make-at-0)
  (operation nonce-test (added-strand set-wrap 2) k (0 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init"))
    (ch-msg lk-0 (cat pt-4 k "decrypt")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((load lk-0 (cat pt-4 k "decrypt"))
      (stor lk-0 (cat pt-5 k "wrap"))))
  (label 50)
  (parent 21)
  (unrealized (0 0))
  (dead)
  (comment "empty cohort"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (0 0)) ((3 1) (4 0)) ((4 1) (5 2))
    ((5 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0 gen-st-wrap-1)
  (operation nonce-test (added-strand wrap 5) k (0 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init"))
    (ch-msg lk-0 (cat pt-4 k "decrypt")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-4 k "decrypt"))
      (load lk-0 (cat pt-5 k1 "wrap")) (send (enc k k1))))
  (label 51)
  (parent 21)
  (unrealized (5 0) (5 3))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "wrap") (k k) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (5 0)) ((3 1) (4 0)) ((4 1) (0 0))
    ((5 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "wrap" "decrypt") (neq "wrap" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-decrypt-at-1 fact-set-decrypt-neq0 gen-st-set-wrap-0
    fact-set-wrap-neq0 trRl_make-at-0)
  (operation nonce-test (added-strand set-decrypt 2) k (0 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init"))
    (ch-msg lk-0 (cat pt-4 k "decrypt")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((load lk (cat pt-1 k "wrap")) (stor lk (cat pt-5 k "decrypt"))))
  (label 52)
  (parent 21)
  (unrealized (0 0))
  (dead)
  (comment "empty cohort"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (0 0)) ((3 1) (4 0)) ((3 1) (5 2))
    ((4 1) (0 0)) ((5 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init"))
    (ch-msg lk-0 (cat pt-4 k "decrypt")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-5 k1 "wrap")) (send (enc k k1))))
  (label 53)
  (parent 21)
  (unrealized (5 0) (5 3))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (5 2)) ((3 1) (4 0)) ((4 1) (0 0))
    ((5 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0 gen-st-wrap-1)
  (operation nonce-test (added-strand wrap 5) k (0 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init"))
    (ch-msg lk-0 (cat pt-4 k "decrypt")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-5 k1 "wrap")) (send (enc k k1))))
  (label 54)
  (parent 21)
  (unrealized (5 0) (5 3))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (5 2)) ((2 1) (0 0)) ((3 1) (4 0))
    ((4 1) (0 0)) ((5 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init"))
    (ch-msg lk-0 (cat pt-4 k "decrypt")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-5 k1 "wrap")) (send (enc k k1))))
  (label 55)
  (parent 21)
  (unrealized (5 0) (5 3))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (0 0)) ((3 4) (0 0)) ((4 2) (3 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 3 5 make 3) (hash k) (4 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-4 k "init"))
      (load lk-0 (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init"))
      (send (hash k))))
  (label 56)
  (parent 23)
  (seen 72 128 129)
  (unrealized (3 3))
  (comment "8 in cohort - 2 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (precedes ((1 1) (0 0)) ((2 1) (3 2)) ((3 4) (0 0)) ((4 1) (1 0))
    ((4 2) (3 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 1 5 make 3) (hash k) (4 0))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init"))
      (send (hash k))))
  (label 57)
  (parent 23)
  (seen 72 130)
  (unrealized (3 3))
  (comment "7 in cohort - 1 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((2 1) (0 0)) ((3 1) (4 2)) ((4 4) (0 0))
    ((5 2) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation encryption-test (added-strand make 3) (hash k) (4 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init"))
      (send (hash k))))
  (label 58)
  (parent 23)
  (seen 74 131 132)
  (unrealized (4 3))
  (comment "8 in cohort - 2 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (deflistener k)
  (precedes ((1 1) (2 0)) ((2 1) (0 0)) ((3 1) (4 2)) ((4 4) (0 0))
    ((5 1) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (added-listener k) (hash k) (4 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k)))
  (label 59)
  (parent 23)
  (unrealized (4 3) (5 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (precedes ((1 1) (3 2)) ((2 1) (0 0)) ((3 4) (0 0)) ((4 1) (1 0))
    ((4 2) (3 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 1 5 make 3) (hash k) (4 0))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt k "wrap"))
      (load lk-0 (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init"))
      (send (hash k))))
  (label 60)
  (parent 24)
  (seen 136 137)
  (unrealized (3 3))
  (comment "4 in cohort - 2 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((2 1) (4 2)) ((3 1) (0 0)) ((4 4) (0 0))
    ((5 2) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation encryption-test (added-strand make 3) (hash k) (4 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init"))
      (send (hash k))))
  (label 61)
  (parent 24)
  (seen 138 139)
  (unrealized (4 3))
  (comment "4 in cohort - 2 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (precedes ((1 1) (2 0)) ((2 1) (4 2)) ((3 1) (0 0)) ((4 4) (0 0))
    ((5 1) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (added-listener k) (hash k) (4 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k)))
  (label 62)
  (parent 24)
  (seen 71)
  (unrealized (4 3) (5 0))
  (comment "3 in cohort - 2 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (precedes ((1 1) (0 0)) ((2 1) (0 0)) ((3 4) (0 0)) ((4 1) (1 0))
    ((4 2) (3 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 1 5 make 3) (hash k) (4 0))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-4 k "init"))
      (load lk-0 (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init"))
      (send (hash k))))
  (label 63)
  (parent 25)
  (seen 142 143)
  (unrealized (3 3))
  (comment "7 in cohort - 2 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((1 1) (4 2)) ((2 1) (0 0)) ((3 1) (0 0))
    ((4 4) (0 0)) ((5 2) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation encryption-test (added-strand make 3) (hash k) (4 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init"))
      (send (hash k))))
  (label 64)
  (parent 25)
  (seen 144 145)
  (unrealized (4 3))
  (comment "7 in cohort - 2 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (precedes ((1 1) (2 0)) ((1 1) (4 2)) ((2 1) (0 0)) ((3 1) (0 0))
    ((4 4) (0 0)) ((5 1) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (added-listener k) (hash k) (4 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k)))
  (label 65)
  (parent 25)
  (seen 77)
  (unrealized (4 3) (5 0))
  (comment "3 in cohort - 2 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old mesg) (pt pt-0 pt-1 pval) (k skey) (lk locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old) (k k) (lk lk))
  (precedes ((1 1) (2 2)) ((2 4) (0 0)) ((3 1) (1 0)) ((3 2) (2 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation channel-test (displaced 4 1 set-wrap 2)
    (ch-msg lk (cat pt-2 k1 "wrap")) (2 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt k "wrap"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init"))
      (send (hash k))))
  (label 66)
  (parent 26)
  (seen 136)
  (unrealized (0 0))
  (comment "5 in cohort - 4 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old cur mesg) (pt pt-0 pt-1 pt-2 pt-3 pval) (k k1 skey)
    (lk locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur cur) (k k1) (lk lk))
  (precedes ((1 1) (2 2)) ((1 1) (4 0)) ((2 4) (0 0)) ((3 1) (1 0))
    ((3 2) (2 0)) ((4 1) (2 3)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 cur) (cat k1 "wrap"))
  (facts (neq cur "decrypt") (neq cur "wrap") (neq "init" "decrypt")
    (neq "init" "wrap"))
  (rule invShearsRule fact-set-decrypt-neq0 trRl_set-wrap-at-1
    gen-st-set-wrap-0 fact-set-wrap-neq0 trRl_make-at-0)
  (operation channel-test (added-strand set-wrap 2)
    (ch-msg lk (cat pt-0 k1 "wrap")) (2 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt-2 k "init")) (stor lk (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt k "wrap"))
      (load lk (cat pt-0 k1 "wrap")) (send (enc k k1)))
    ((load lk (cat pt-1 old)) (stor lk (cat pt-2 k "init"))
      (send (hash k)))
    ((load lk (cat pt-3 k1 cur)) (stor lk (cat pt-0 k1 "wrap"))))
  (label 67)
  (parent 26)
  (seen 152)
  (unrealized (4 0))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pval) (k skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (3 2)) ((3 4) (0 0)) ((4 2) (3 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation channel-test (displaced 5 2 set-wrap 2)
    (ch-msg lk (cat pt-4 k1 "wrap")) (3 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k))))
  (label 68)
  (parent 27)
  (seen 138)
  (unrealized (0 0))
  (comment "8 in cohort - 7 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 cur mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur cur) (k k1) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (3 2)) ((2 1) (5 0)) ((3 4) (0 0))
    ((4 2) (3 0)) ((5 1) (3 3)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 cur) (cat k1 "wrap"))
  (facts (neq cur "decrypt") (neq cur "wrap") (neq "init" "decrypt")
    (neq "init" "wrap"))
  (rule invShearsRule fact-set-decrypt-neq0 trRl_set-wrap-at-1
    gen-st-set-wrap-0 fact-set-wrap-neq0 trRl_make-at-0)
  (operation channel-test (added-strand set-wrap 2)
    (ch-msg lk (cat pt-2 k1 "wrap")) (3 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init"))
      (send (hash k)))
    ((load lk (cat pt-5 k1 cur)) (stor lk (cat pt-2 k1 "wrap"))))
  (label 69)
  (parent 27)
  (seen 160)
  (unrealized (5 0))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old mesg) (pt pt-0 pt-1 pt-2 pval) (k k1 skey) (lk locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (precedes ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (3 2)) ((3 4) (0 0))
    ((4 1) (3 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation nonce-test (displaced 5 1 make 2) k (4 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k)))
  (label 70)
  (parent 28)
  (seen 161)
  (unrealized (3 3) (4 0))
  (comment "4 in cohort - 3 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (3 2)) ((3 4) (0 0)) ((4 1) (3 0))
    ((5 1) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation nonce-test (added-strand make 2) k (4 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init"))))
  (label 71)
  (parent 28)
  (seen 162)
  (unrealized (3 3) (4 0))
  (comment "4 in cohort - 3 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old mesg) (pt pt-0 pt-1 pval) (k skey) (lk locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old) (k k) (lk lk))
  (precedes ((1 1) (2 3)) ((2 4) (0 0)) ((3 1) (1 0)) ((3 2) (2 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation channel-test (displaced 4 1 set-wrap 2)
    (ch-msg lk (cat pt-2 k1 "wrap")) (2 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init"))
      (send (hash k))))
  (label 72)
  (parent 29)
  (seen 142)
  (unrealized (0 0))
  (comment "5 in cohort - 4 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old cur mesg) (pt pt-0 pt-1 pt-2 pt-3 pval) (k k1 skey)
    (lk locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur cur) (k k1) (lk lk))
  (precedes ((1 1) (4 0)) ((2 4) (0 0)) ((3 1) (1 0)) ((3 2) (2 0))
    ((4 1) (2 3)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 cur) (cat k1 "wrap"))
  (facts (neq cur "decrypt") (neq cur "wrap") (neq "init" "decrypt")
    (neq "init" "wrap"))
  (rule shearsRule invShearsRule fact-set-decrypt-neq0
    trRl_set-wrap-at-1 gen-st-set-wrap-0 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation channel-test (added-strand set-wrap 2)
    (ch-msg lk (cat pt-0 k1 "wrap")) (2 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt-2 k "init")) (stor lk (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-2 k "init"))
      (load lk (cat pt-0 k1 "wrap")) (send (enc k k1)))
    ((load lk (cat pt-1 old)) (stor lk (cat pt-2 k "init"))
      (send (hash k)))
    ((load lk (cat pt-3 k1 cur)) (stor lk (cat pt-0 k1 "wrap"))))
  (label 73)
  (parent 29)
  (seen 171)
  (unrealized (4 0))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pval) (k skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((1 1) (3 2)) ((2 1) (3 3)) ((3 4) (0 0))
    ((4 2) (3 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation channel-test (displaced 5 2 set-wrap 2)
    (ch-msg lk (cat pt-4 k1 "wrap")) (3 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k))))
  (label 74)
  (parent 30)
  (seen 144)
  (unrealized (0 0))
  (comment "8 in cohort - 7 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 cur mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur cur) (k k1) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (3 2)) ((2 1) (5 0)) ((3 4) (0 0))
    ((4 2) (3 0)) ((5 1) (3 3)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 cur) (cat k1 "wrap"))
  (facts (neq cur "decrypt") (neq cur "wrap") (neq "init" "decrypt")
    (neq "init" "wrap"))
  (rule shearsRule invShearsRule fact-set-decrypt-neq0
    trRl_set-wrap-at-1 gen-st-set-wrap-0 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation channel-test (added-strand set-wrap 2)
    (ch-msg lk (cat pt-2 k1 "wrap")) (3 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init"))
      (send (hash k)))
    ((load lk (cat pt-5 k1 cur)) (stor lk (cat pt-2 k1 "wrap"))))
  (label 75)
  (parent 30)
  (seen 179)
  (unrealized (5 0))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old mesg) (pt pt-0 pt-1 pt-2 pval) (k k1 skey) (lk locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (precedes ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (0 0)) ((3 4) (0 0))
    ((4 1) (3 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation nonce-test (displaced 5 1 make 2) k (4 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k)))
  (label 76)
  (parent 31)
  (unrealized (3 3) (4 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((1 1) (3 2)) ((2 1) (0 0)) ((3 4) (0 0))
    ((4 1) (3 0)) ((5 1) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation nonce-test (added-strand make 2) k (4 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init"))))
  (label 77)
  (parent 31)
  (seen 180)
  (unrealized (3 3) (4 0))
  (comment "4 in cohort - 3 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (0 0)) ((3 4) (0 0)) ((4 2) (3 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 3 5 make 3) (hash k) (4 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-4 k "init"))
      (load lk-0 (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init"))
      (send (hash k))))
  (label 78)
  (parent 33)
  (seen 185)
  (unrealized (3 3))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (precedes ((1 1) (0 0)) ((2 1) (3 2)) ((3 4) (0 0)) ((4 1) (1 0))
    ((4 2) (3 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 1 5 make 3) (hash k) (4 0))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt k "decrypt")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init"))
      (send (hash k))))
  (label 79)
  (parent 33)
  (seen 186)
  (unrealized (3 3))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((2 1) (0 0)) ((3 1) (4 2)) ((4 4) (0 0))
    ((5 2) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation encryption-test (added-strand make 3) (hash k) (4 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init"))
      (send (hash k))))
  (label 80)
  (parent 33)
  (seen 187)
  (unrealized (4 3))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (deflistener k)
  (precedes ((1 1) (2 0)) ((2 1) (0 0)) ((3 1) (4 2)) ((4 4) (0 0))
    ((5 1) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (added-listener k) (hash k) (4 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k)))
  (label 81)
  (parent 33)
  (unrealized (4 3) (5 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (precedes ((1 1) (3 2)) ((2 1) (0 0)) ((3 4) (0 0)) ((4 1) (1 0))
    ((4 2) (3 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 1 5 make 3) (hash k) (4 0))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt k "decrypt")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt k "decrypt"))
      (load lk-0 (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init"))
      (send (hash k))))
  (label 82)
  (parent 35)
  (seen 191)
  (unrealized (3 3))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((2 1) (4 2)) ((3 1) (0 0)) ((4 4) (0 0))
    ((5 2) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation encryption-test (added-strand make 3) (hash k) (4 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "decrypt"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init"))
      (send (hash k))))
  (label 83)
  (parent 35)
  (seen 192)
  (unrealized (4 3))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (precedes ((1 1) (2 0)) ((2 1) (4 2)) ((3 1) (0 0)) ((4 4) (0 0))
    ((5 1) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (added-listener k) (hash k) (4 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "decrypt"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k)))
  (label 84)
  (parent 35)
  (seen 91)
  (unrealized (4 3) (5 0))
  (comment "3 in cohort - 2 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (precedes ((1 1) (0 0)) ((2 1) (0 0)) ((3 4) (0 0)) ((4 1) (1 0))
    ((4 2) (3 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 1 5 make 3) (hash k) (4 0))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt k "decrypt")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-4 k "init"))
      (load lk-0 (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init"))
      (send (hash k))))
  (label 85)
  (parent 36)
  (seen 195)
  (unrealized (3 3))
  (comment "4 in cohort - 1 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((1 1) (4 2)) ((2 1) (0 0)) ((3 1) (0 0))
    ((4 4) (0 0)) ((5 2) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation encryption-test (added-strand make 3) (hash k) (4 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init"))
      (send (hash k))))
  (label 86)
  (parent 36)
  (seen 196)
  (unrealized (4 3))
  (comment "4 in cohort - 1 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (precedes ((1 1) (2 0)) ((1 1) (4 2)) ((2 1) (0 0)) ((3 1) (0 0))
    ((4 4) (0 0)) ((5 1) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (added-listener k) (hash k) (4 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k)))
  (label 87)
  (parent 36)
  (seen 95)
  (unrealized (4 3) (5 0))
  (comment "3 in cohort - 2 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old cur mesg) (pt pt-0 pt-1 pt-2 pt-3 pval) (k k1 skey)
    (lk locn))
  (deflistener k)
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur cur) (k k1) (lk lk))
  (precedes ((1 1) (2 2)) ((1 1) (4 0)) ((2 4) (0 0)) ((3 1) (1 0))
    ((3 2) (2 0)) ((4 1) (2 3)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k1 cur) (cat k1 "wrap"))
  (facts (neq cur "decrypt") (neq cur "wrap") (neq "init" "decrypt")
    (neq "init" "wrap"))
  (rule invShearsRule fact-set-decrypt-neq0 trRl_set-wrap-at-1
    gen-st-set-wrap-0 fact-set-wrap-neq0 trRl_make-at-0)
  (operation channel-test (added-strand set-wrap 2)
    (ch-msg lk (cat pt-0 k1 "wrap")) (2 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt-2 k "init")) (stor lk (cat pt k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt k "decrypt"))
      (load lk (cat pt-0 k1 "wrap")) (send (enc k k1)))
    ((load lk (cat pt-1 old)) (stor lk (cat pt-2 k "init"))
      (send (hash k)))
    ((load lk (cat pt-3 k1 cur)) (stor lk (cat pt-0 k1 "wrap"))))
  (label 88)
  (parent 37)
  (seen 199)
  (unrealized (4 0))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 cur mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur cur) (k k1) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (3 2)) ((2 1) (5 0)) ((3 4) (0 0))
    ((4 2) (3 0)) ((5 1) (3 3)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k1 cur) (cat k1 "wrap"))
  (facts (neq cur "decrypt") (neq cur "wrap") (neq "init" "decrypt")
    (neq "init" "wrap"))
  (rule invShearsRule fact-set-decrypt-neq0 trRl_set-wrap-at-1
    gen-st-set-wrap-0 fact-set-wrap-neq0 trRl_make-at-0)
  (operation channel-test (added-strand set-wrap 2)
    (ch-msg lk (cat pt-2 k1 "wrap")) (3 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "decrypt"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init"))
      (send (hash k)))
    ((load lk (cat pt-5 k1 cur)) (stor lk (cat pt-2 k1 "wrap"))))
  (label 89)
  (parent 38)
  (seen 200)
  (unrealized (5 0))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old mesg) (pt pt-0 pt-1 pt-2 pval) (k k1 skey) (lk locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (precedes ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (3 2)) ((3 4) (0 0))
    ((4 1) (3 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation nonce-test (displaced 5 1 make 2) k (4 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "decrypt"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k)))
  (label 90)
  (parent 39)
  (seen 202)
  (unrealized (3 3) (4 0))
  (comment "4 in cohort - 3 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (3 2)) ((3 4) (0 0)) ((4 1) (3 0))
    ((5 1) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation nonce-test (added-strand make 2) k (4 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "decrypt"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init"))))
  (label 91)
  (parent 39)
  (seen 201)
  (unrealized (3 3) (4 0))
  (comment "4 in cohort - 3 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old cur mesg) (pt pt-0 pt-1 pt-2 pt-3 pval) (k k1 skey)
    (lk locn))
  (deflistener k)
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur cur) (k k1) (lk lk))
  (precedes ((1 1) (4 0)) ((2 4) (0 0)) ((3 1) (1 0)) ((3 2) (2 0))
    ((4 1) (2 3)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 cur) (cat k1 "wrap"))
  (facts (neq cur "decrypt") (neq cur "wrap") (neq "init" "decrypt")
    (neq "init" "wrap"))
  (rule shearsRule invShearsRule fact-set-decrypt-neq0
    trRl_set-wrap-at-1 gen-st-set-wrap-0 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation channel-test (added-strand set-wrap 2)
    (ch-msg lk (cat pt-0 k1 "wrap")) (2 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt-2 k "init")) (stor lk (cat pt k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-2 k "init"))
      (load lk (cat pt-0 k1 "wrap")) (send (enc k k1)))
    ((load lk (cat pt-1 old)) (stor lk (cat pt-2 k "init"))
      (send (hash k)))
    ((load lk (cat pt-3 k1 cur)) (stor lk (cat pt-0 k1 "wrap"))))
  (label 92)
  (parent 40)
  (seen 207)
  (unrealized (4 0))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 cur mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur cur) (k k1) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (3 2)) ((2 1) (5 0)) ((3 4) (0 0))
    ((4 2) (3 0)) ((5 1) (3 3)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 cur) (cat k1 "wrap"))
  (facts (neq cur "decrypt") (neq cur "wrap") (neq "init" "decrypt")
    (neq "init" "wrap"))
  (rule shearsRule invShearsRule fact-set-decrypt-neq0
    trRl_set-wrap-at-1 gen-st-set-wrap-0 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation channel-test (added-strand set-wrap 2)
    (ch-msg lk (cat pt-2 k1 "wrap")) (3 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init"))
      (send (hash k)))
    ((load lk (cat pt-5 k1 cur)) (stor lk (cat pt-2 k1 "wrap"))))
  (label 93)
  (parent 41)
  (seen 208)
  (unrealized (5 0))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old mesg) (pt pt-0 pt-1 pt-2 pval) (k k1 skey) (lk locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (precedes ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (0 0)) ((3 4) (0 0))
    ((4 1) (3 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation nonce-test (displaced 5 1 make 2) k (4 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k)))
  (label 94)
  (parent 42)
  (unrealized (3 3) (4 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((1 1) (3 2)) ((2 1) (0 0)) ((3 4) (0 0))
    ((4 1) (3 0)) ((5 1) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation nonce-test (added-strand make 2) k (4 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init"))))
  (label 95)
  (parent 42)
  (seen 209)
  (unrealized (3 3) (4 0))
  (comment "4 in cohort - 3 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pval) (k k1 skey)
    (lk locn))
  (deflistener k)
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k1) (lk lk))
  (defstrand make 2 (old old-0) (k k1) (lk lk))
  (precedes ((1 4) (0 0)) ((2 1) (4 0)) ((2 2) (1 0)) ((3 1) (1 3))
    ((4 1) (3 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule invShearsRule trRl_make-at-0 trRl_make-at-1)
  (operation channel-test (added-strand make 2)
    (ch-msg lk (cat pt-2 k1 "init")) (3 0))
  (traces ((recv k) (send k))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt k1 "wrap")) (send (enc k k1)))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init"))
      (send (hash k)))
    ((load lk (cat pt-2 k1 "init")) (stor lk (cat pt k1 "wrap")))
    ((load lk (cat pt-3 old-0)) (stor lk (cat pt-2 k1 "init"))))
  (label 96)
  (parent 43)
  (realized)
  (shape)
  (maps ((0) ((k k))))
  (origs (pt-2 (4 1)) (pt (3 1)) (pt-1 (2 1))))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k1) (lk lk))
  (defstrand make 2 (old old-1) (k k1) (lk lk))
  (precedes ((1 1) (2 2)) ((1 1) (5 0)) ((2 4) (0 0)) ((3 2) (2 0))
    ((4 1) (2 3)) ((5 1) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule invShearsRule trRl_make-at-0 trRl_make-at-1)
  (operation channel-test (added-strand make 2)
    (ch-msg lk (cat pt-4 k1 "init")) (4 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk (cat pt-4 k1 "init")) (stor lk (cat pt-1 k1 "wrap")))
    ((load lk (cat pt-5 old-1)) (stor lk (cat pt-4 k1 "init"))))
  (label 97)
  (parent 44)
  (realized)
  (shape)
  (maps ((0) ((k k))))
  (origs (pt-4 (5 1)) (pt-1 (4 1)) (pt-3 (3 1)) (pt-0 (1 1))))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (precedes ((1 1) (3 0)) ((2 4) (0 0)) ((3 1) (2 0)) ((4 1) (3 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation nonce-test (added-strand make 2) k (3 0)
    (ch-msg lk (cat pt-0 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))))
  (label 98)
  (parent 45)
  (unrealized (2 3) (3 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old mesg) (pt pt-0 pt-1 pt-2 pval) (k k1 k1-0 skey) (lk locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk))
  (precedes ((1 1) (4 2)) ((2 4) (0 0)) ((3 1) (2 0)) ((4 4) (3 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (3 0)
    (ch-msg lk (cat pt-0 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((recv (hash k)) (recv (hash k1-0)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-2 k1-0 "wrap")) (send (enc k k1-0))))
  (label 99)
  (parent 45)
  (unrealized (2 3) (4 0) (4 3))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (precedes ((1 1) (2 2)) ((2 4) (0 0)) ((3 1) (2 0)) ((4 1) (5 0))
    ((5 1) (3 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule fact-set-decrypt-neq0 trRl_set-wrap-at-1 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation nonce-test (added-strand set-wrap 2) k (3 0)
    (ch-msg lk-0 (cat pt-3 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "wrap"))))
  (label 100)
  (parent 46)
  (seen 215)
  (unrealized (2 3) (3 0))
  (comment "6 in cohort - 5 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (precedes ((1 1) (2 2)) ((2 4) (0 0)) ((3 1) (2 0)) ((4 1) (5 0))
    ((5 1) (3 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-decrypt-at-1 fact-set-decrypt-neq0 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation nonce-test (added-strand set-decrypt 2) k (3 0)
    (ch-msg lk-0 (cat pt-3 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init"))
      (stor lk-0 (cat pt-4 k "decrypt"))))
  (label 101)
  (parent 46)
  (seen 214)
  (unrealized (2 3) (3 0))
  (comment "6 in cohort - 5 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk-0))
  (precedes ((1 1) (2 2)) ((2 4) (0 0)) ((3 1) (2 0)) ((4 1) (5 2))
    ((5 4) (3 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (3 0)
    (ch-msg lk-0 (cat pt-3 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-4 k1-0 "wrap")) (send (enc k k1-0))))
  (label 102)
  (parent 46)
  (unrealized (2 3) (5 0) (5 3))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (defstrand set-decrypt 2 (cur "wrap") (k k) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (6 0)) ((3 1) (4 0)) ((4 1) (0 0))
    ((5 1) (0 0)) ((6 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "wrap" "decrypt") (neq "wrap" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-decrypt-at-1 fact-set-decrypt-neq0 gen-st-set-wrap-0
    fact-set-wrap-neq0 trRl_make-at-0)
  (operation nonce-test (added-strand set-decrypt 2) k (0 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init"))
    (ch-msg lk-0 (cat pt-4 k "decrypt"))
    (ch-msg lk-1 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init")))
    ((load lk (cat pt-1 k "wrap")) (stor lk (cat pt-7 k "decrypt"))))
  (label 103)
  (parent 49)
  (unrealized (0 0))
  (dead)
  (comment "empty cohort"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (defstrand set-decrypt 2 (cur "decrypt") (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (0 0)) ((3 1) (4 0)) ((4 1) (6 0))
    ((5 1) (0 0)) ((6 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init"))
  (facts (neq "decrypt" "decrypt") (neq "decrypt" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-decrypt-at-1 fact-set-decrypt-neq0 gen-st-set-wrap-0
    fact-set-wrap-neq0 trRl_make-at-0)
  (operation nonce-test (added-strand set-decrypt 2) k (0 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init"))
    (ch-msg lk-0 (cat pt-4 k "decrypt"))
    (ch-msg lk-1 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init")))
    ((load lk-0 (cat pt-4 k "decrypt"))
      (stor lk-0 (cat pt-7 k "decrypt"))))
  (label 104)
  (parent 49)
  (unrealized (0 0))
  (dead)
  (comment "empty cohort"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (defstrand set-wrap 2 (cur "wrap") (k k) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (6 0)) ((3 1) (4 0)) ((4 1) (0 0))
    ((5 1) (0 0)) ((6 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "wrap" "decrypt") (neq "wrap" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule fact-set-decrypt-neq0 trRl_set-wrap-at-1 gen-st-set-wrap-0
    fact-set-wrap-neq0 trRl_make-at-0)
  (operation nonce-test (added-strand set-wrap 2) k (0 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init"))
    (ch-msg lk-0 (cat pt-4 k "decrypt"))
    (ch-msg lk-1 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init")))
    ((load lk (cat pt-1 k "wrap")) (stor lk (cat pt-7 k "wrap"))))
  (label 105)
  (parent 49)
  (unrealized (0 0))
  (dead)
  (comment "empty cohort"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (defstrand set-wrap 2 (cur "decrypt") (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (0 0)) ((3 1) (4 0)) ((4 1) (6 0))
    ((5 1) (0 0)) ((6 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init"))
  (facts (neq "decrypt" "decrypt") (neq "decrypt" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule fact-set-decrypt-neq0 trRl_set-wrap-at-1 gen-st-set-wrap-0
    fact-set-wrap-neq0 trRl_make-at-0)
  (operation nonce-test (added-strand set-wrap 2) k (0 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init"))
    (ch-msg lk-0 (cat pt-4 k "decrypt"))
    (ch-msg lk-1 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init")))
    ((load lk-0 (cat pt-4 k "decrypt"))
      (stor lk-0 (cat pt-7 k "wrap"))))
  (label 106)
  (parent 49)
  (unrealized (0 0))
  (dead)
  (comment "empty cohort"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (0 0)) ((3 1) (4 0)) ((4 1) (6 2))
    ((5 1) (0 0)) ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0 gen-st-wrap-1)
  (operation nonce-test (added-strand wrap 5) k (0 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init"))
    (ch-msg lk-0 (cat pt-4 k "decrypt"))
    (ch-msg lk-1 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-4 k "decrypt"))
      (load lk-0 (cat pt-7 k1 "wrap")) (send (enc k k1))))
  (label 107)
  (parent 49)
  (seen 114)
  (unrealized (6 0) (6 3))
  (comment "5 in cohort - 4 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-1))
  (precedes ((1 1) (2 0)) ((2 1) (0 0)) ((3 1) (4 0)) ((4 1) (0 0))
    ((5 1) (6 2)) ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init"))
    (ch-msg lk-0 (cat pt-4 k "decrypt"))
    (ch-msg lk-1 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk-1 (cat pt-6 k "init"))
      (load lk-1 (cat pt-7 k1 "wrap")) (send (enc k k1))))
  (label 108)
  (parent 49)
  (unrealized (6 0) (6 3))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (0 0)) ((3 1) (4 0)) ((3 1) (6 2))
    ((4 1) (0 0)) ((5 1) (0 0)) ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init"))
    (ch-msg lk-0 (cat pt-4 k "decrypt"))
    (ch-msg lk-1 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-7 k1 "wrap")) (send (enc k k1))))
  (label 109)
  (parent 49)
  (seen 118)
  (unrealized (6 0) (6 3))
  (comment "5 in cohort - 4 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (6 2)) ((3 1) (4 0)) ((4 1) (0 0))
    ((5 1) (0 0)) ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0 gen-st-wrap-1)
  (operation nonce-test (added-strand wrap 5) k (0 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init"))
    (ch-msg lk-0 (cat pt-4 k "decrypt"))
    (ch-msg lk-1 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-7 k1 "wrap")) (send (enc k k1))))
  (label 110)
  (parent 49)
  (seen 122)
  (unrealized (6 0) (6 3))
  (comment "5 in cohort - 4 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (6 2)) ((2 1) (0 0)) ((3 1) (4 0))
    ((4 1) (0 0)) ((5 1) (0 0)) ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init"))
    (ch-msg lk-0 (cat pt-4 k "decrypt"))
    (ch-msg lk-1 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-7 k1 "wrap")) (send (enc k k1))))
  (label 111)
  (parent 49)
  (seen 126)
  (unrealized (6 0) (6 3))
  (comment "5 in cohort - 4 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (0 0)) ((3 1) (4 2)) ((4 4) (0 0))
    ((5 1) (3 0)) ((5 2) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 3 6 make 3) (hash k) (5 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-5 k "init")) (stor lk-0 (cat pt-2 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-2 k "decrypt"))
      (load lk-0 (cat pt-3 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-4 old-0)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k))))
  (label 112)
  (parent 51)
  (seen 256 257 258 259)
  (unrealized (4 3))
  (comment "14 in cohort - 4 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (precedes ((1 1) (0 0)) ((2 1) (3 0)) ((3 1) (4 2)) ((4 4) (0 0))
    ((5 1) (1 0)) ((5 2) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 1 6 make 3) (hash k) (5 0))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-5 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-2 k "decrypt"))
      (load lk (cat pt-3 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-4 old-0)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k))))
  (label 113)
  (parent 51)
  (seen 260 262)
  (unrealized (4 3))
  (comment "5 in cohort - 3 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1) (lk lk-0))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((2 1) (0 0)) ((3 1) (4 0)) ((4 1) (5 2))
    ((5 4) (0 0)) ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation encryption-test (added-strand make 3) (hash k) (5 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-4 k "decrypt"))
      (load lk-0 (cat pt-5 k1 "wrap")) (send (enc k k1)))
    ((load lk-1 (cat pt-6 old-1)) (stor lk-1 (cat pt-7 k "init"))
      (send (hash k))))
  (label 114)
  (parent 51)
  (seen 263 264 265 266)
  (unrealized (5 3))
  (comment "14 in cohort - 4 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1) (lk lk-0))
  (deflistener k)
  (precedes ((1 1) (2 0)) ((2 1) (0 0)) ((3 1) (4 0)) ((4 1) (5 2))
    ((5 4) (0 0)) ((6 1) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (added-listener k) (hash k) (5 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-4 k "decrypt"))
      (load lk-0 (cat pt-5 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k)))
  (label 115)
  (parent 51)
  (unrealized (5 3) (6 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (0 0)) ((3 1) (0 0)) ((4 4) (0 0))
    ((5 1) (3 0)) ((5 2) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 3 6 make 3) (hash k) (5 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-5 k "init")) (stor lk-0 (cat pt-2 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-5 k "init"))
      (load lk-0 (cat pt-3 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-4 old-0)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k))))
  (label 116)
  (parent 53)
  (seen 270 271 272)
  (unrealized (4 3))
  (comment "14 in cohort - 3 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (precedes ((1 1) (0 0)) ((2 1) (3 0)) ((2 1) (4 2)) ((3 1) (0 0))
    ((4 4) (0 0)) ((5 1) (1 0)) ((5 2) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 1 6 make 3) (hash k) (5 0))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-5 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt-3 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-4 old-0)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k))))
  (label 117)
  (parent 53)
  (seen 273)
  (unrealized (4 3))
  (comment "4 in cohort - 1 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((2 1) (0 0)) ((3 1) (4 0)) ((3 1) (5 2))
    ((4 1) (0 0)) ((5 4) (0 0)) ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation encryption-test (added-strand make 3) (hash k) (5 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-5 k1 "wrap")) (send (enc k k1)))
    ((load lk-1 (cat pt-6 old-1)) (stor lk-1 (cat pt-7 k "init"))
      (send (hash k))))
  (label 118)
  (parent 53)
  (seen 274 275 276)
  (unrealized (5 3))
  (comment "14 in cohort - 3 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (deflistener k)
  (precedes ((1 1) (2 0)) ((2 1) (0 0)) ((3 1) (4 0)) ((3 1) (5 2))
    ((4 1) (0 0)) ((5 4) (0 0)) ((6 1) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (added-listener k) (hash k) (5 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-5 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k)))
  (label 119)
  (parent 53)
  (unrealized (5 3) (6 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (4 2)) ((3 1) (0 0)) ((4 4) (0 0))
    ((5 1) (3 0)) ((5 2) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 3 6 make 3) (hash k) (5 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-5 k "init")) (stor lk-0 (cat pt-2 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-3 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-4 old-0)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k))))
  (label 120)
  (parent 54)
  (seen 153 280)
  (unrealized (4 3))
  (comment "4 in cohort - 1 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (precedes ((1 1) (4 2)) ((2 1) (3 0)) ((3 1) (0 0)) ((4 4) (0 0))
    ((5 1) (1 0)) ((5 2) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 1 6 make 3) (hash k) (5 0))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-5 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt k "wrap"))
      (load lk-0 (cat pt-3 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-4 old-0)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k))))
  (label 121)
  (parent 54)
  (seen 281 282)
  (unrealized (4 3))
  (comment "4 in cohort - 2 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((2 1) (5 2)) ((3 1) (4 0)) ((4 1) (0 0))
    ((5 4) (0 0)) ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation encryption-test (added-strand make 3) (hash k) (5 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-5 k1 "wrap")) (send (enc k k1)))
    ((load lk-1 (cat pt-6 old-1)) (stor lk-1 (cat pt-7 k "init"))
      (send (hash k))))
  (label 122)
  (parent 54)
  (seen 283 284)
  (unrealized (5 3))
  (comment "4 in cohort - 2 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (precedes ((1 1) (2 0)) ((2 1) (5 2)) ((3 1) (4 0)) ((4 1) (0 0))
    ((5 4) (0 0)) ((6 1) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (added-listener k) (hash k) (5 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-5 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k)))
  (label 123)
  (parent 54)
  (unrealized (5 3) (6 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((1 1) (4 2)) ((2 1) (0 0)) ((3 1) (0 0))
    ((4 4) (0 0)) ((5 1) (3 0)) ((5 2) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 3 6 make 3) (hash k) (5 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-5 k "init")) (stor lk-0 (cat pt-2 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-3 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-4 old-0)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k))))
  (label 124)
  (parent 55)
  (seen 172 288)
  (unrealized (4 3))
  (comment "7 in cohort - 1 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (precedes ((1 1) (0 0)) ((2 1) (3 0)) ((3 1) (0 0)) ((4 4) (0 0))
    ((5 1) (1 0)) ((5 2) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 1 6 make 3) (hash k) (5 0))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-5 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-5 k "init"))
      (load lk-0 (cat pt-3 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-4 old-0)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k))))
  (label 125)
  (parent 55)
  (seen 289 290)
  (unrealized (4 3))
  (comment "7 in cohort - 2 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((1 1) (5 2)) ((2 1) (0 0)) ((3 1) (4 0))
    ((4 1) (0 0)) ((5 4) (0 0)) ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation encryption-test (added-strand make 3) (hash k) (5 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-5 k1 "wrap")) (send (enc k k1)))
    ((load lk-1 (cat pt-6 old-1)) (stor lk-1 (cat pt-7 k "init"))
      (send (hash k))))
  (label 126)
  (parent 55)
  (seen 291 292)
  (unrealized (5 3))
  (comment "7 in cohort - 2 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (precedes ((1 1) (2 0)) ((1 1) (5 2)) ((2 1) (0 0)) ((3 1) (4 0))
    ((4 1) (0 0)) ((5 4) (0 0)) ((6 1) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (added-listener k) (hash k) (5 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-5 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k)))
  (label 127)
  (parent 55)
  (unrealized (5 3) (6 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pval) (k skey)
    (lk locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (3 3)) ((3 4) (0 0)) ((4 1) (1 0))
    ((4 2) (3 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule invShearsRule)
  (operation channel-test (displaced 5 2 set-wrap 2)
    (ch-msg lk-0 (cat pt-4 k1 "wrap")) (3 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init"))
      (send (hash k))))
  (label 128)
  (parent 56)
  (seen 72 270 298)
  (unrealized (0 0))
  (comment "19 in cohort - 7 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 cur mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur cur) (k k1) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (0 0)) ((3 4) (0 0)) ((4 1) (5 0))
    ((4 2) (3 0)) ((5 1) (3 3)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 cur) (cat k1 "wrap"))
  (facts (neq cur "decrypt") (neq cur "wrap") (neq "init" "decrypt")
    (neq "init" "wrap"))
  (rule invShearsRule fact-set-decrypt-neq0 trRl_set-wrap-at-1
    gen-st-set-wrap-0 fact-set-wrap-neq0 trRl_make-at-0)
  (operation channel-test (added-strand set-wrap 2)
    (ch-msg lk-0 (cat pt-2 k1 "wrap")) (3 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-4 k "init"))
      (load lk-0 (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-5 k1 cur)) (stor lk-0 (cat pt-2 k1 "wrap"))))
  (label 129)
  (parent 56)
  (seen 72 128 303 304)
  (unrealized (5 0))
  (comment "7 in cohort - 2 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 cur mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur cur) (k k1) (lk lk))
  (precedes ((1 1) (0 0)) ((2 1) (3 2)) ((2 1) (5 0)) ((3 4) (0 0))
    ((4 1) (1 0)) ((4 2) (3 0)) ((5 1) (3 3)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 cur) (cat k1 "wrap"))
  (facts (neq cur "decrypt") (neq cur "wrap") (neq "init" "decrypt")
    (neq "init" "wrap"))
  (rule invShearsRule fact-set-decrypt-neq0 trRl_set-wrap-at-1
    gen-st-set-wrap-0 fact-set-wrap-neq0 trRl_make-at-0)
  (operation channel-test (added-strand set-wrap 2)
    (ch-msg lk (cat pt-2 k1 "wrap")) (3 3))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init"))
      (send (hash k)))
    ((load lk (cat pt-5 k1 cur)) (stor lk (cat pt-2 k1 "wrap"))))
  (label 130)
  (parent 57)
  (seen 72 174 305)
  (unrealized (5 0))
  (comment "6 in cohort - 1 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (4 3)) ((3 1) (1 0)) ((3 1) (4 2))
    ((4 4) (0 0)) ((5 2) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule invShearsRule)
  (operation channel-test (displaced 6 2 set-wrap 2)
    (ch-msg lk-1 (cat pt-6 k1 "wrap")) (4 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-4 old-1)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k))))
  (label 131)
  (parent 58)
  (seen 74 274 310)
  (unrealized (0 0))
  (comment "22 in cohort - 10 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 cur mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand set-wrap 2 (cur cur) (k k1) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (0 0)) ((3 1) (4 2)) ((3 1) (6 0))
    ((4 4) (0 0)) ((5 2) (4 0)) ((6 1) (4 3)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 cur) (cat k1 "wrap"))
  (facts (neq cur "decrypt") (neq cur "wrap") (neq "init" "decrypt")
    (neq "init" "wrap"))
  (rule invShearsRule fact-set-decrypt-neq0 trRl_set-wrap-at-1
    gen-st-set-wrap-0 fact-set-wrap-neq0 trRl_make-at-0)
  (operation channel-test (added-strand set-wrap 2)
    (ch-msg lk-0 (cat pt-4 k1 "wrap")) (4 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-7 k1 cur)) (stor lk-0 (cat pt-4 k1 "wrap"))))
  (label 132)
  (parent 58)
  (seen 74 131 304 316 317)
  (unrealized (6 0))
  (comment "9 in cohort - 2 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (deflistener k)
  (precedes ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (0 0)) ((3 1) (4 2))
    ((4 4) (0 0)) ((5 1) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation nonce-test (displaced 6 1 make 2) k (5 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k)))
  (label 133)
  (parent 59)
  (seen 100)
  (unrealized (4 3) (5 0))
  (comment "5 in cohort - 3 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (deflistener k)
  (precedes ((1 1) (2 0)) ((2 1) (0 0)) ((3 1) (5 0)) ((4 4) (0 0))
    ((5 1) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation nonce-test (displaced 6 3 make 2) k (5 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k)))
  (label 134)
  (parent 59)
  (seen 319)
  (unrealized (4 3) (5 0))
  (comment "3 in cohort - 2 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (deflistener k)
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((2 1) (0 0)) ((3 1) (4 2)) ((4 4) (0 0))
    ((5 1) (4 0)) ((6 1) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation nonce-test (added-strand make 2) k (5 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init"))))
  (label 135)
  (parent 59)
  (seen 318 321)
  (unrealized (4 3) (5 0))
  (comment "5 in cohort - 3 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pval) (k skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (precedes ((1 1) (3 2)) ((2 1) (0 0)) ((3 4) (0 0)) ((4 1) (1 0))
    ((4 2) (3 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation channel-test (displaced 5 1 set-wrap 2)
    (ch-msg lk-0 (cat pt-4 k1 "wrap")) (3 3))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt k "wrap"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k))))
  (label 136)
  (parent 60)
  (seen 281)
  (unrealized (0 0))
  (comment "7 in cohort - 6 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 cur mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur cur) (k k1) (lk lk-0))
  (precedes ((1 1) (3 2)) ((1 1) (5 0)) ((2 1) (0 0)) ((3 4) (0 0))
    ((4 1) (1 0)) ((4 2) (3 0)) ((5 1) (3 3)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 cur) (cat k1 "wrap"))
  (facts (neq cur "decrypt") (neq cur "wrap") (neq "init" "decrypt")
    (neq "init" "wrap"))
  (rule invShearsRule fact-set-decrypt-neq0 trRl_set-wrap-at-1
    gen-st-set-wrap-0 fact-set-wrap-neq0 trRl_make-at-0)
  (operation channel-test (added-strand set-wrap 2)
    (ch-msg lk-0 (cat pt-2 k1 "wrap")) (3 3))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt k "wrap"))
      (load lk-0 (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-5 k1 cur)) (stor lk-0 (cat pt-2 k1 "wrap"))))
  (label 137)
  (parent 60)
  (seen 332 333)
  (unrealized (5 0))
  (comment "4 in cohort - 2 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k skey) (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((2 1) (4 2)) ((3 1) (0 0)) ((4 4) (0 0))
    ((5 2) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation channel-test (displaced 6 2 set-wrap 2)
    (ch-msg lk (cat pt-6 k1 "wrap")) (4 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-1 (cat pt-4 old-1)) (stor lk-1 (cat pt-5 k "init"))
      (send (hash k))))
  (label 138)
  (parent 61)
  (seen 283)
  (unrealized (0 0))
  (comment "10 in cohort - 9 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 cur mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand set-wrap 2 (cur cur) (k k1) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (4 2)) ((2 1) (6 0)) ((3 1) (0 0))
    ((4 4) (0 0)) ((5 2) (4 0)) ((6 1) (4 3)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 cur) (cat k1 "wrap"))
  (facts (neq cur "decrypt") (neq cur "wrap") (neq "init" "decrypt")
    (neq "init" "wrap"))
  (rule invShearsRule fact-set-decrypt-neq0 trRl_set-wrap-at-1
    gen-st-set-wrap-0 fact-set-wrap-neq0 trRl_make-at-0)
  (operation channel-test (added-strand set-wrap 2)
    (ch-msg lk (cat pt-4 k1 "wrap")) (4 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init"))
      (send (hash k)))
    ((load lk (cat pt-7 k1 cur)) (stor lk (cat pt-4 k1 "wrap"))))
  (label 139)
  (parent 61)
  (seen 343 344)
  (unrealized (6 0))
  (comment "4 in cohort - 2 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (precedes ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (4 2)) ((3 1) (0 0))
    ((4 4) (0 0)) ((5 1) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation nonce-test (displaced 6 1 make 2) k (5 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k)))
  (label 140)
  (parent 62)
  (seen 162 345)
  (unrealized (4 3) (5 0))
  (comment "5 in cohort - 3 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((2 1) (4 2)) ((3 1) (0 0)) ((4 4) (0 0))
    ((5 1) (4 0)) ((6 1) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation nonce-test (added-strand make 2) k (5 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init"))))
  (label 141)
  (parent 62)
  (seen 346)
  (unrealized (4 3) (5 0))
  (comment "4 in cohort - 3 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pval) (k skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (precedes ((1 1) (3 3)) ((2 1) (0 0)) ((3 4) (0 0)) ((4 1) (1 0))
    ((4 2) (3 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation channel-test (displaced 5 1 set-wrap 2)
    (ch-msg lk-0 (cat pt-4 k1 "wrap")) (3 3))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k))))
  (label 142)
  (parent 63)
  (seen 289 304)
  (unrealized (0 0))
  (comment "7 in cohort - 5 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 cur mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur cur) (k k1) (lk lk-0))
  (precedes ((1 1) (5 0)) ((2 1) (0 0)) ((3 4) (0 0)) ((4 1) (1 0))
    ((4 2) (3 0)) ((5 1) (3 3)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 cur) (cat k1 "wrap"))
  (facts (neq cur "decrypt") (neq cur "wrap") (neq "init" "decrypt")
    (neq "init" "wrap"))
  (rule shearsRule invShearsRule fact-set-decrypt-neq0
    trRl_set-wrap-at-1 gen-st-set-wrap-0 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation channel-test (added-strand set-wrap 2)
    (ch-msg lk-0 (cat pt-2 k1 "wrap")) (3 3))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-4 k "init"))
      (load lk-0 (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-5 k1 cur)) (stor lk-0 (cat pt-2 k1 "wrap"))))
  (label 143)
  (parent 63)
  (seen 298 356)
  (unrealized (5 0))
  (comment "4 in cohort - 1 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k skey) (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((1 1) (4 2)) ((2 1) (4 3)) ((3 1) (0 0))
    ((4 4) (0 0)) ((5 2) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation channel-test (displaced 6 2 set-wrap 2)
    (ch-msg lk (cat pt-6 k1 "wrap")) (4 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-1 (cat pt-4 old-1)) (stor lk-1 (cat pt-5 k "init"))
      (send (hash k))))
  (label 144)
  (parent 64)
  (seen 291 316)
  (unrealized (0 0))
  (comment "10 in cohort - 8 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 cur mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand set-wrap 2 (cur cur) (k k1) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (4 2)) ((2 1) (6 0)) ((3 1) (0 0))
    ((4 4) (0 0)) ((5 2) (4 0)) ((6 1) (4 3)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 cur) (cat k1 "wrap"))
  (facts (neq cur "decrypt") (neq cur "wrap") (neq "init" "decrypt")
    (neq "init" "wrap"))
  (rule shearsRule invShearsRule fact-set-decrypt-neq0
    trRl_set-wrap-at-1 gen-st-set-wrap-0 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation channel-test (added-strand set-wrap 2)
    (ch-msg lk (cat pt-4 k1 "wrap")) (4 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init"))
      (send (hash k)))
    ((load lk (cat pt-7 k1 cur)) (stor lk (cat pt-4 k1 "wrap"))))
  (label 145)
  (parent 64)
  (seen 310 365)
  (unrealized (6 0))
  (comment "4 in cohort - 1 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (precedes ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (0 0)) ((3 1) (0 0))
    ((4 4) (0 0)) ((5 1) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation nonce-test (displaced 6 1 make 2) k (5 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k)))
  (label 146)
  (parent 65)
  (seen 180)
  (unrealized (4 3) (5 0))
  (comment "3 in cohort - 2 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((1 1) (4 2)) ((2 1) (0 0)) ((3 1) (0 0))
    ((4 4) (0 0)) ((5 1) (4 0)) ((6 1) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation nonce-test (added-strand make 2) k (5 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init"))))
  (label 147)
  (parent 65)
  (seen 366)
  (unrealized (4 3) (5 0))
  (comment "4 in cohort - 3 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old mesg) (pt pt-0 pt-1 pt-2 pval) (k skey) (lk locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "wrap") (k k) (lk lk))
  (precedes ((1 1) (2 2)) ((1 1) (4 0)) ((2 4) (0 0)) ((3 1) (1 0))
    ((3 2) (2 0)) ((4 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "wrap" "decrypt") (neq "wrap" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule fact-set-decrypt-neq0 trRl_set-wrap-at-1 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation nonce-test (added-strand set-wrap 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt k "wrap")) (ch-msg lk (cat pt-1 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt k "wrap"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init"))
      (send (hash k)))
    ((load lk (cat pt k "wrap")) (stor lk (cat pt-2 k "wrap"))))
  (label 148)
  (parent 66)
  (unrealized (0 0))
  (dead)
  (comment "empty cohort"))

(defskeleton wrap-decrypt
  (vars (old mesg) (pt pt-0 pt-1 pt-2 pval) (k skey) (lk locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "wrap") (k k) (lk lk))
  (precedes ((1 1) (2 2)) ((1 1) (4 0)) ((2 4) (0 0)) ((3 1) (1 0))
    ((3 2) (2 0)) ((4 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "wrap" "decrypt") (neq "wrap" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-decrypt-at-1 fact-set-decrypt-neq0 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation nonce-test (added-strand set-decrypt 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt k "wrap")) (ch-msg lk (cat pt-1 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt k "wrap"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init"))
      (send (hash k)))
    ((load lk (cat pt k "wrap")) (stor lk (cat pt-2 k "decrypt"))))
  (label 149)
  (parent 66)
  (unrealized (0 0))
  (dead)
  (comment "empty cohort"))

(defskeleton wrap-decrypt
  (vars (old mesg) (pt pt-0 pt-1 pt-2 pval) (k k1 skey) (lk locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (2 2)) ((2 4) (0 0)) ((3 1) (1 0)) ((3 1) (4 2))
    ((3 2) (2 0)) ((4 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt k "wrap")) (ch-msg lk (cat pt-1 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt k "wrap"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1))))
  (label 150)
  (parent 66)
  (unrealized (4 0) (4 3))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old mesg) (pt pt-0 pt-1 pt-2 pval) (k k1 skey) (lk locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (2 2)) ((1 1) (4 2)) ((2 4) (0 0)) ((3 1) (1 0))
    ((3 2) (2 0)) ((4 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt k "wrap")) (ch-msg lk (cat pt-1 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt k "wrap"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt k "wrap"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1))))
  (label 151)
  (parent 66)
  (unrealized (4 0) (4 3))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k k1 skey)
    (lk locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k1) (lk lk))
  (defstrand make 2 (old old-0) (k k1) (lk lk))
  (precedes ((1 1) (2 2)) ((1 1) (5 0)) ((2 4) (0 0)) ((3 1) (1 0))
    ((3 2) (2 0)) ((4 1) (2 3)) ((5 1) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule invShearsRule trRl_make-at-0 trRl_make-at-1)
  (operation channel-test (added-strand make 2)
    (ch-msg lk (cat pt-3 k1 "init")) (4 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt-2 k "init")) (stor lk (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt k "wrap"))
      (load lk (cat pt-0 k1 "wrap")) (send (enc k k1)))
    ((load lk (cat pt-1 old)) (stor lk (cat pt-2 k "init"))
      (send (hash k)))
    ((load lk (cat pt-3 k1 "init")) (stor lk (cat pt-0 k1 "wrap")))
    ((load lk (cat pt-4 old-0)) (stor lk (cat pt-3 k1 "init"))))
  (label 152)
  (parent 67)
  (realized)
  (shape)
  (maps ((0) ((k k))))
  (origs (pt-3 (5 1)) (pt-0 (4 1)) (pt-2 (3 1)) (pt (1 1))))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (3 2)) ((3 4) (0 0)) ((4 1) (5 0))
    ((4 2) (3 0)) ((5 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-decrypt-at-1 trRl_make-at-0)
  (operation nonce-test (added-strand set-decrypt 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-3 k "init"))
      (stor lk-0 (cat pt-4 k "decrypt"))))
  (label 153)
  (parent 68)
  (seen 334)
  (unrealized (0 0))
  (comment "10 in cohort - 9 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "wrap") (k k) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (3 2)) ((2 1) (5 0)) ((3 4) (0 0))
    ((4 2) (3 0)) ((5 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "wrap" "decrypt") (neq "wrap" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule fact-set-decrypt-neq0 trRl_set-wrap-at-1 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation nonce-test (added-strand set-wrap 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk (cat pt-1 k "wrap")) (stor lk (cat pt-4 k "wrap"))))
  (label 154)
  (parent 68)
  (unrealized (0 0))
  (dead)
  (comment "empty cohort"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (3 2)) ((3 4) (0 0)) ((4 1) (5 0))
    ((4 2) (3 0)) ((5 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-wrap-at-1 trRl_make-at-0)
  (operation nonce-test (added-strand set-wrap 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "wrap"))))
  (label 155)
  (parent 68)
  (seen 335)
  (unrealized (0 0))
  (comment "9 in cohort - 8 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "wrap") (k k) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (3 2)) ((2 1) (5 0)) ((3 4) (0 0))
    ((4 2) (3 0)) ((5 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "wrap" "decrypt") (neq "wrap" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-decrypt-at-1 fact-set-decrypt-neq0 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation nonce-test (added-strand set-decrypt 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk (cat pt-1 k "wrap")) (stor lk (cat pt-4 k "decrypt"))))
  (label 156)
  (parent 68)
  (unrealized (0 0))
  (dead)
  (comment "empty cohort"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (3 2)) ((3 4) (0 0)) ((4 1) (5 2))
    ((4 2) (3 0)) ((5 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-4 k1 "wrap")) (send (enc k k1))))
  (label 157)
  (parent 68)
  (unrealized (5 0) (5 3))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (3 2)) ((2 1) (5 2)) ((3 4) (0 0))
    ((4 2) (3 0)) ((5 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1))))
  (label 158)
  (parent 68)
  (unrealized (5 0) (5 3))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (5 2)) ((2 1) (3 2)) ((3 4) (0 0))
    ((4 2) (3 0)) ((5 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1))))
  (label 159)
  (parent 68)
  (unrealized (5 0) (5 3))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k1) (lk lk))
  (defstrand make 2 (old old-1) (k k1) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (3 2)) ((2 1) (6 0)) ((3 4) (0 0))
    ((4 2) (3 0)) ((5 1) (3 3)) ((6 1) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule invShearsRule trRl_make-at-0 trRl_make-at-1)
  (operation channel-test (added-strand make 2)
    (ch-msg lk (cat pt-5 k1 "init")) (5 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init"))
      (send (hash k)))
    ((load lk (cat pt-5 k1 "init")) (stor lk (cat pt-2 k1 "wrap")))
    ((load lk (cat pt-6 old-1)) (stor lk (cat pt-5 k1 "init"))))
  (label 160)
  (parent 69)
  (realized)
  (shape)
  (maps ((0) ((k k))))
  (origs (pt-5 (6 1)) (pt-2 (5 1)) (pt-4 (4 1)) (pt-1 (2 1))
    (pt-0 (1 1))))

(defskeleton wrap-decrypt
  (vars (old mesg) (pt pt-0 pt-1 pt-2 pval) (k k1 skey) (lk locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (precedes ((1 1) (2 0)) ((2 1) (4 0)) ((3 4) (0 0)) ((4 1) (3 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation nonce-test (displaced 5 2 set-wrap 2) k (4 0)
    (ch-msg lk (cat pt-0 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k)))
  (label 161)
  (parent 70)
  (unrealized (3 3) (4 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (3 2)) ((3 4) (0 0))
    ((4 1) (3 0)) ((5 1) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation nonce-test (added-strand make 2) k (4 0)
    (ch-msg lk (cat pt-0 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init"))))
  (label 162)
  (parent 70)
  (seen 406)
  (unrealized (3 3) (4 0))
  (comment "6 in cohort - 4 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old mesg) (pt pt-0 pt-1 pt-2 pt-3 pval) (k k1 k1-0 skey)
    (lk locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (5 2)) ((2 1) (3 2)) ((3 4) (0 0))
    ((4 1) (3 0)) ((5 4) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap")
    (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (4 0)
    (ch-msg lk (cat pt-0 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((recv (hash k)) (recv (hash k1-0)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-3 k1-0 "wrap")) (send (enc k k1-0))))
  (label 163)
  (parent 70)
  (unrealized (3 3) (5 0) (5 3))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (3 2)) ((3 4) (0 0)) ((4 1) (3 0))
    ((5 1) (6 0)) ((6 1) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-wrap-at-1 trRl_make-at-0)
  (operation nonce-test (added-strand set-wrap 2) k (4 0)
    (ch-msg lk-0 (cat pt-4 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt-5 k "wrap"))))
  (label 164)
  (parent 71)
  (seen 409)
  (unrealized (3 3) (4 0))
  (comment "6 in cohort - 5 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (3 2)) ((3 4) (0 0)) ((4 1) (3 0))
    ((5 1) (6 0)) ((6 1) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-decrypt-at-1 trRl_make-at-0)
  (operation nonce-test (added-strand set-decrypt 2) k (4 0)
    (ch-msg lk-0 (cat pt-4 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((load lk-0 (cat pt-4 k "init"))
      (stor lk-0 (cat pt-5 k "decrypt"))))
  (label 165)
  (parent 71)
  (seen 410)
  (unrealized (3 3) (4 0))
  (comment "6 in cohort - 5 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (3 2)) ((3 4) (0 0)) ((4 1) (3 0))
    ((5 1) (6 2)) ((6 4) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap")
    (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (4 0)
    (ch-msg lk-0 (cat pt-4 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-0 (cat pt-4 k "init"))
      (load lk-0 (cat pt-5 k1-0 "wrap")) (send (enc k k1-0))))
  (label 166)
  (parent 71)
  (unrealized (3 3) (6 0) (6 3))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old mesg) (pt pt-0 pt-1 pt-2 pval) (k skey) (lk locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "wrap") (k k) (lk lk))
  (precedes ((1 1) (2 3)) ((1 1) (4 0)) ((2 4) (0 0)) ((3 1) (1 0))
    ((3 2) (2 0)) ((4 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "wrap" "decrypt") (neq "wrap" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule fact-set-decrypt-neq0 trRl_set-wrap-at-1 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation nonce-test (added-strand set-wrap 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt k "wrap")) (ch-msg lk (cat pt-1 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init"))
      (send (hash k)))
    ((load lk (cat pt k "wrap")) (stor lk (cat pt-2 k "wrap"))))
  (label 167)
  (parent 72)
  (unrealized (0 0))
  (dead)
  (comment "empty cohort"))

(defskeleton wrap-decrypt
  (vars (old mesg) (pt pt-0 pt-1 pt-2 pval) (k skey) (lk locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "wrap") (k k) (lk lk))
  (precedes ((1 1) (2 3)) ((1 1) (4 0)) ((2 4) (0 0)) ((3 1) (1 0))
    ((3 2) (2 0)) ((4 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "wrap" "decrypt") (neq "wrap" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-decrypt-at-1 fact-set-decrypt-neq0 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation nonce-test (added-strand set-decrypt 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt k "wrap")) (ch-msg lk (cat pt-1 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init"))
      (send (hash k)))
    ((load lk (cat pt k "wrap")) (stor lk (cat pt-2 k "decrypt"))))
  (label 168)
  (parent 72)
  (unrealized (0 0))
  (dead)
  (comment "empty cohort"))

(defskeleton wrap-decrypt
  (vars (old mesg) (pt pt-0 pt-1 pt-2 pval) (k k1 skey) (lk locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (2 3)) ((2 4) (0 0)) ((3 1) (1 0)) ((3 1) (4 2))
    ((3 2) (2 0)) ((4 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt k "wrap")) (ch-msg lk (cat pt-1 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1))))
  (label 169)
  (parent 72)
  (unrealized (4 0) (4 3))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old mesg) (pt pt-0 pt-1 pt-2 pval) (k k1 skey) (lk locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (2 3)) ((1 1) (4 2)) ((2 4) (0 0)) ((3 1) (1 0))
    ((3 2) (2 0)) ((4 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt k "wrap")) (ch-msg lk (cat pt-1 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt k "wrap"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1))))
  (label 170)
  (parent 72)
  (unrealized (4 0) (4 3))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k k1 skey)
    (lk locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k1) (lk lk))
  (defstrand make 2 (old old-0) (k k1) (lk lk))
  (precedes ((1 1) (5 0)) ((2 4) (0 0)) ((3 1) (1 0)) ((3 2) (2 0))
    ((4 1) (2 3)) ((5 1) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule invShearsRule trRl_make-at-0 trRl_make-at-1)
  (operation channel-test (added-strand make 2)
    (ch-msg lk (cat pt-3 k1 "init")) (4 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt-2 k "init")) (stor lk (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-2 k "init"))
      (load lk (cat pt-0 k1 "wrap")) (send (enc k k1)))
    ((load lk (cat pt-1 old)) (stor lk (cat pt-2 k "init"))
      (send (hash k)))
    ((load lk (cat pt-3 k1 "init")) (stor lk (cat pt-0 k1 "wrap")))
    ((load lk (cat pt-4 old-0)) (stor lk (cat pt-3 k1 "init"))))
  (label 171)
  (parent 73)
  (seen 96)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((1 1) (3 2)) ((2 1) (3 3)) ((3 4) (0 0))
    ((4 1) (5 0)) ((4 2) (3 0)) ((5 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-decrypt-at-1 trRl_make-at-0)
  (operation nonce-test (added-strand set-decrypt 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-3 k "init"))
      (stor lk-0 (cat pt-4 k "decrypt"))))
  (label 172)
  (parent 74)
  (seen 357)
  (unrealized (0 0))
  (comment "10 in cohort - 9 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "wrap") (k k) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (3 2)) ((2 1) (3 3)) ((2 1) (5 0))
    ((3 4) (0 0)) ((4 2) (3 0)) ((5 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "wrap" "decrypt") (neq "wrap" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule fact-set-decrypt-neq0 trRl_set-wrap-at-1 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation nonce-test (added-strand set-wrap 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk (cat pt-1 k "wrap")) (stor lk (cat pt-4 k "wrap"))))
  (label 173)
  (parent 74)
  (unrealized (0 0))
  (dead)
  (comment "empty cohort"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((1 1) (3 2)) ((2 1) (3 3)) ((3 4) (0 0))
    ((4 1) (5 0)) ((4 2) (3 0)) ((5 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-wrap-at-1 trRl_make-at-0)
  (operation nonce-test (added-strand set-wrap 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "wrap"))))
  (label 174)
  (parent 74)
  (seen 358)
  (unrealized (0 0))
  (comment "9 in cohort - 8 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "wrap") (k k) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (3 2)) ((2 1) (3 3)) ((2 1) (5 0))
    ((3 4) (0 0)) ((4 2) (3 0)) ((5 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "wrap" "decrypt") (neq "wrap" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-decrypt-at-1 fact-set-decrypt-neq0 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation nonce-test (added-strand set-decrypt 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk (cat pt-1 k "wrap")) (stor lk (cat pt-4 k "decrypt"))))
  (label 175)
  (parent 74)
  (unrealized (0 0))
  (dead)
  (comment "empty cohort"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (precedes ((1 1) (2 0)) ((1 1) (3 2)) ((2 1) (3 3)) ((3 4) (0 0))
    ((4 1) (5 2)) ((4 2) (3 0)) ((5 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-4 k1 "wrap")) (send (enc k k1))))
  (label 176)
  (parent 74)
  (unrealized (5 0) (5 3))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (3 2)) ((2 1) (3 3)) ((2 1) (5 2))
    ((3 4) (0 0)) ((4 2) (3 0)) ((5 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1))))
  (label 177)
  (parent 74)
  (unrealized (5 0) (5 3))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (3 2)) ((1 1) (5 2)) ((2 1) (3 3))
    ((3 4) (0 0)) ((4 2) (3 0)) ((5 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1))))
  (label 178)
  (parent 74)
  (unrealized (5 0) (5 3))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k1) (lk lk))
  (defstrand make 2 (old old-1) (k k1) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (3 2)) ((2 1) (6 0)) ((3 4) (0 0))
    ((4 2) (3 0)) ((5 1) (3 3)) ((6 1) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule invShearsRule trRl_make-at-0 trRl_make-at-1)
  (operation channel-test (added-strand make 2)
    (ch-msg lk (cat pt-5 k1 "init")) (5 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init"))
      (send (hash k)))
    ((load lk (cat pt-5 k1 "init")) (stor lk (cat pt-2 k1 "wrap")))
    ((load lk (cat pt-6 old-1)) (stor lk (cat pt-5 k1 "init"))))
  (label 179)
  (parent 75)
  (seen 97)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (0 0)) ((3 4) (0 0))
    ((4 1) (3 0)) ((5 1) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation nonce-test (added-strand make 2) k (4 0)
    (ch-msg lk (cat pt-0 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init"))))
  (label 180)
  (parent 76)
  (unrealized (3 3) (4 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old mesg) (pt pt-0 pt-1 pt-2 pt-3 pval) (k k1 k1-0 skey)
    (lk locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (5 2)) ((2 1) (0 0)) ((3 4) (0 0))
    ((4 1) (3 0)) ((5 4) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (4 0)
    (ch-msg lk (cat pt-0 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((recv (hash k)) (recv (hash k1-0)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-3 k1-0 "wrap")) (send (enc k k1-0))))
  (label 181)
  (parent 76)
  (unrealized (3 3) (5 0) (5 3))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((1 1) (3 2)) ((2 1) (0 0)) ((3 4) (0 0))
    ((4 1) (3 0)) ((5 1) (6 0)) ((6 1) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-wrap-at-1 trRl_make-at-0)
  (operation nonce-test (added-strand set-wrap 2) k (4 0)
    (ch-msg lk-0 (cat pt-4 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt-5 k "wrap"))))
  (label 182)
  (parent 77)
  (seen 465)
  (unrealized (3 3) (4 0))
  (comment "6 in cohort - 5 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((1 1) (3 2)) ((2 1) (0 0)) ((3 4) (0 0))
    ((4 1) (3 0)) ((5 1) (6 0)) ((6 1) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-decrypt-at-1 trRl_make-at-0)
  (operation nonce-test (added-strand set-decrypt 2) k (4 0)
    (ch-msg lk-0 (cat pt-4 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((load lk-0 (cat pt-4 k "init"))
      (stor lk-0 (cat pt-5 k "decrypt"))))
  (label 183)
  (parent 77)
  (seen 466)
  (unrealized (3 3) (4 0))
  (comment "6 in cohort - 5 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk-0))
  (precedes ((1 1) (2 0)) ((1 1) (3 2)) ((2 1) (0 0)) ((3 4) (0 0))
    ((4 1) (3 0)) ((5 1) (6 2)) ((6 4) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (4 0)
    (ch-msg lk-0 (cat pt-4 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-0 (cat pt-4 k "init"))
      (load lk-0 (cat pt-5 k1-0 "wrap")) (send (enc k k1-0))))
  (label 184)
  (parent 77)
  (unrealized (3 3) (6 0) (6 3))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 cur mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur cur) (k k1) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (0 0)) ((3 4) (0 0)) ((4 1) (5 0))
    ((4 2) (3 0)) ((5 1) (3 3)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 cur) (cat k1 "wrap"))
  (facts (neq cur "decrypt") (neq cur "wrap") (neq "init" "decrypt")
    (neq "init" "wrap"))
  (rule invShearsRule fact-set-decrypt-neq0 trRl_set-wrap-at-1
    gen-st-set-wrap-0 fact-set-wrap-neq0 trRl_make-at-0)
  (operation channel-test (added-strand set-wrap 2)
    (ch-msg lk-0 (cat pt-2 k1 "wrap")) (3 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-4 k "init"))
      (load lk-0 (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-5 k1 cur)) (stor lk-0 (cat pt-2 k1 "wrap"))))
  (label 185)
  (parent 78)
  (seen 289 486)
  (unrealized (5 0))
  (comment "4 in cohort - 1 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 cur mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur cur) (k k1) (lk lk))
  (precedes ((1 1) (0 0)) ((2 1) (3 2)) ((2 1) (5 0)) ((3 4) (0 0))
    ((4 1) (1 0)) ((4 2) (3 0)) ((5 1) (3 3)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 cur) (cat k1 "wrap"))
  (facts (neq cur "decrypt") (neq cur "wrap") (neq "init" "decrypt")
    (neq "init" "wrap"))
  (rule invShearsRule fact-set-decrypt-neq0 trRl_set-wrap-at-1
    gen-st-set-wrap-0 fact-set-wrap-neq0 trRl_make-at-0)
  (operation channel-test (added-strand set-wrap 2)
    (ch-msg lk (cat pt-2 k1 "wrap")) (3 3))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt k "decrypt")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init"))
      (send (hash k)))
    ((load lk (cat pt-5 k1 cur)) (stor lk (cat pt-2 k1 "wrap"))))
  (label 186)
  (parent 79)
  (seen 172 487)
  (unrealized (5 0))
  (comment "4 in cohort - 1 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 cur mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand set-wrap 2 (cur cur) (k k1) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (0 0)) ((3 1) (4 2)) ((3 1) (6 0))
    ((4 4) (0 0)) ((5 2) (4 0)) ((6 1) (4 3)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 cur) (cat k1 "wrap"))
  (facts (neq cur "decrypt") (neq cur "wrap") (neq "init" "decrypt")
    (neq "init" "wrap"))
  (rule invShearsRule fact-set-decrypt-neq0 trRl_set-wrap-at-1
    gen-st-set-wrap-0 fact-set-wrap-neq0 trRl_make-at-0)
  (operation channel-test (added-strand set-wrap 2)
    (ch-msg lk-0 (cat pt-4 k1 "wrap")) (4 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-7 k1 cur)) (stor lk-0 (cat pt-4 k1 "wrap"))))
  (label 187)
  (parent 80)
  (seen 289 291 488)
  (unrealized (6 0))
  (comment "6 in cohort - 1 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (deflistener k)
  (precedes ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (0 0)) ((3 1) (4 2))
    ((4 4) (0 0)) ((5 1) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation nonce-test (displaced 6 1 make 2) k (5 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k)))
  (label 188)
  (parent 81)
  (seen 101)
  (unrealized (4 3) (5 0))
  (comment "5 in cohort - 3 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (deflistener k)
  (precedes ((1 1) (2 0)) ((2 1) (0 0)) ((3 1) (5 0)) ((4 4) (0 0))
    ((5 1) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation nonce-test (displaced 6 3 make 2) k (5 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k)))
  (label 189)
  (parent 81)
  (seen 490)
  (unrealized (4 3) (5 0))
  (comment "3 in cohort - 2 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (deflistener k)
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((2 1) (0 0)) ((3 1) (4 2)) ((4 4) (0 0))
    ((5 1) (4 0)) ((6 1) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation nonce-test (added-strand make 2) k (5 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init"))))
  (label 190)
  (parent 81)
  (seen 489 492)
  (unrealized (4 3) (5 0))
  (comment "5 in cohort - 3 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 cur mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur cur) (k k1) (lk lk-0))
  (precedes ((1 1) (3 2)) ((1 1) (5 0)) ((2 1) (0 0)) ((3 4) (0 0))
    ((4 1) (1 0)) ((4 2) (3 0)) ((5 1) (3 3)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k1 cur) (cat k1 "wrap"))
  (facts (neq cur "decrypt") (neq cur "wrap") (neq "init" "decrypt")
    (neq "init" "wrap"))
  (rule invShearsRule fact-set-decrypt-neq0 trRl_set-wrap-at-1
    gen-st-set-wrap-0 fact-set-wrap-neq0 trRl_make-at-0)
  (operation channel-test (added-strand set-wrap 2)
    (ch-msg lk-0 (cat pt-2 k1 "wrap")) (3 3))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt k "decrypt")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt k "decrypt"))
      (load lk-0 (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-5 k1 cur)) (stor lk-0 (cat pt-2 k1 "wrap"))))
  (label 191)
  (parent 82)
  (seen 256 497)
  (unrealized (5 0))
  (comment "4 in cohort - 1 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 cur mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand set-wrap 2 (cur cur) (k k1) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (4 2)) ((2 1) (6 0)) ((3 1) (0 0))
    ((4 4) (0 0)) ((5 2) (4 0)) ((6 1) (4 3)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k1 cur) (cat k1 "wrap"))
  (facts (neq cur "decrypt") (neq cur "wrap") (neq "init" "decrypt")
    (neq "init" "wrap"))
  (rule invShearsRule fact-set-decrypt-neq0 trRl_set-wrap-at-1
    gen-st-set-wrap-0 fact-set-wrap-neq0 trRl_make-at-0)
  (operation channel-test (added-strand set-wrap 2)
    (ch-msg lk (cat pt-4 k1 "wrap")) (4 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "decrypt"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init"))
      (send (hash k)))
    ((load lk (cat pt-7 k1 cur)) (stor lk (cat pt-4 k1 "wrap"))))
  (label 192)
  (parent 83)
  (seen 263 498)
  (unrealized (6 0))
  (comment "4 in cohort - 1 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (precedes ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (4 2)) ((3 1) (0 0))
    ((4 4) (0 0)) ((5 1) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation nonce-test (displaced 6 1 make 2) k (5 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "decrypt"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k)))
  (label 193)
  (parent 84)
  (seen 201 500)
  (unrealized (4 3) (5 0))
  (comment "5 in cohort - 3 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((2 1) (4 2)) ((3 1) (0 0)) ((4 4) (0 0))
    ((5 1) (4 0)) ((6 1) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation nonce-test (added-strand make 2) k (5 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "decrypt"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init"))))
  (label 194)
  (parent 84)
  (seen 499)
  (unrealized (4 3) (5 0))
  (comment "4 in cohort - 3 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 cur mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur cur) (k k1) (lk lk-0))
  (precedes ((1 1) (5 0)) ((2 1) (0 0)) ((3 4) (0 0)) ((4 1) (1 0))
    ((4 2) (3 0)) ((5 1) (3 3)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 cur) (cat k1 "wrap"))
  (facts (neq cur "decrypt") (neq cur "wrap") (neq "init" "decrypt")
    (neq "init" "wrap"))
  (rule shearsRule invShearsRule fact-set-decrypt-neq0
    trRl_set-wrap-at-1 gen-st-set-wrap-0 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation channel-test (added-strand set-wrap 2)
    (ch-msg lk-0 (cat pt-2 k1 "wrap")) (3 3))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt k "decrypt")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-4 k "init"))
      (load lk-0 (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-5 k1 cur)) (stor lk-0 (cat pt-2 k1 "wrap"))))
  (label 195)
  (parent 85)
  (seen 270 505)
  (unrealized (5 0))
  (comment "4 in cohort - 1 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 cur mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand set-wrap 2 (cur cur) (k k1) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (4 2)) ((2 1) (6 0)) ((3 1) (0 0))
    ((4 4) (0 0)) ((5 2) (4 0)) ((6 1) (4 3)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 cur) (cat k1 "wrap"))
  (facts (neq cur "decrypt") (neq cur "wrap") (neq "init" "decrypt")
    (neq "init" "wrap"))
  (rule shearsRule invShearsRule fact-set-decrypt-neq0
    trRl_set-wrap-at-1 gen-st-set-wrap-0 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation channel-test (added-strand set-wrap 2)
    (ch-msg lk (cat pt-4 k1 "wrap")) (4 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init"))
      (send (hash k)))
    ((load lk (cat pt-7 k1 cur)) (stor lk (cat pt-4 k1 "wrap"))))
  (label 196)
  (parent 86)
  (seen 274 506)
  (unrealized (6 0))
  (comment "4 in cohort - 1 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (precedes ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (0 0)) ((3 1) (0 0))
    ((4 4) (0 0)) ((5 1) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation nonce-test (displaced 6 1 make 2) k (5 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k)))
  (label 197)
  (parent 87)
  (seen 209)
  (unrealized (4 3) (5 0))
  (comment "3 in cohort - 2 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((1 1) (4 2)) ((2 1) (0 0)) ((3 1) (0 0))
    ((4 4) (0 0)) ((5 1) (4 0)) ((6 1) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation nonce-test (added-strand make 2) k (5 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init"))))
  (label 198)
  (parent 87)
  (seen 507)
  (unrealized (4 3) (5 0))
  (comment "4 in cohort - 3 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k k1 skey)
    (lk locn))
  (deflistener k)
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k1) (lk lk))
  (defstrand make 2 (old old-0) (k k1) (lk lk))
  (precedes ((1 1) (2 2)) ((1 1) (5 0)) ((2 4) (0 0)) ((3 1) (1 0))
    ((3 2) (2 0)) ((4 1) (2 3)) ((5 1) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k1 "init")
    (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule invShearsRule trRl_make-at-0 trRl_make-at-1)
  (operation channel-test (added-strand make 2)
    (ch-msg lk (cat pt-3 k1 "init")) (4 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt-2 k "init")) (stor lk (cat pt k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt k "decrypt"))
      (load lk (cat pt-0 k1 "wrap")) (send (enc k k1)))
    ((load lk (cat pt-1 old)) (stor lk (cat pt-2 k "init"))
      (send (hash k)))
    ((load lk (cat pt-3 k1 "init")) (stor lk (cat pt-0 k1 "wrap")))
    ((load lk (cat pt-4 old-0)) (stor lk (cat pt-3 k1 "init"))))
  (label 199)
  (parent 88)
  (realized)
  (shape)
  (maps ((0) ((k k))))
  (origs (pt-3 (5 1)) (pt-0 (4 1)) (pt-2 (3 1)) (pt (1 1))))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k1) (lk lk))
  (defstrand make 2 (old old-1) (k k1) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (3 2)) ((2 1) (6 0)) ((3 4) (0 0))
    ((4 2) (3 0)) ((5 1) (3 3)) ((6 1) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k1 "init")
    (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule invShearsRule trRl_make-at-0 trRl_make-at-1)
  (operation channel-test (added-strand make 2)
    (ch-msg lk (cat pt-5 k1 "init")) (5 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "decrypt"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init"))
      (send (hash k)))
    ((load lk (cat pt-5 k1 "init")) (stor lk (cat pt-2 k1 "wrap")))
    ((load lk (cat pt-6 old-1)) (stor lk (cat pt-5 k1 "init"))))
  (label 200)
  (parent 89)
  (realized)
  (shape)
  (maps ((0) ((k k))))
  (origs (pt-5 (6 1)) (pt-2 (5 1)) (pt-4 (4 1)) (pt-1 (2 1))
    (pt-0 (1 1))))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (3 2)) ((3 4) (0 0))
    ((4 1) (3 0)) ((5 1) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation nonce-test (added-strand make 2) k (4 0)
    (ch-msg lk (cat pt-0 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "decrypt"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init"))))
  (label 201)
  (parent 90)
  (seen 514)
  (unrealized (3 3) (4 0))
  (comment "6 in cohort - 5 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old mesg) (pt pt-0 pt-1 pt-2 pval) (k k1 skey) (lk locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (precedes ((1 1) (2 0)) ((2 1) (4 0)) ((3 4) (0 0)) ((4 1) (3 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation nonce-test (displaced 5 2 set-decrypt 2) k (4 0)
    (ch-msg lk (cat pt-0 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "decrypt"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k)))
  (label 202)
  (parent 90)
  (seen 514)
  (unrealized (3 3) (4 0))
  (comment "3 in cohort - 2 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old mesg) (pt pt-0 pt-1 pt-2 pt-3 pval) (k k1 k1-0 skey)
    (lk locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (5 2)) ((2 1) (3 2)) ((3 4) (0 0))
    ((4 1) (3 0)) ((5 4) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k1 "wrap")
    (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (4 0)
    (ch-msg lk (cat pt-0 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "decrypt"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((recv (hash k)) (recv (hash k1-0)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-3 k1-0 "wrap")) (send (enc k k1-0))))
  (label 203)
  (parent 90)
  (unrealized (3 3) (5 0) (5 3))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (3 2)) ((3 4) (0 0)) ((4 1) (3 0))
    ((5 1) (6 0)) ((6 1) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-wrap-at-1 trRl_make-at-0)
  (operation nonce-test (added-strand set-wrap 2) k (4 0)
    (ch-msg lk-0 (cat pt-4 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "decrypt"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt-5 k "wrap"))))
  (label 204)
  (parent 91)
  (seen 513)
  (unrealized (3 3) (4 0))
  (comment "6 in cohort - 5 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (3 2)) ((3 4) (0 0)) ((4 1) (3 0))
    ((5 1) (6 0)) ((6 1) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-decrypt-at-1 trRl_make-at-0)
  (operation nonce-test (added-strand set-decrypt 2) k (4 0)
    (ch-msg lk-0 (cat pt-4 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "decrypt"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((load lk-0 (cat pt-4 k "init"))
      (stor lk-0 (cat pt-5 k "decrypt"))))
  (label 205)
  (parent 91)
  (seen 512)
  (unrealized (3 3) (4 0))
  (comment "6 in cohort - 5 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (3 2)) ((3 4) (0 0)) ((4 1) (3 0))
    ((5 1) (6 2)) ((6 4) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k1 "wrap")
    (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (4 0)
    (ch-msg lk-0 (cat pt-4 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "decrypt"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-0 (cat pt-4 k "init"))
      (load lk-0 (cat pt-5 k1-0 "wrap")) (send (enc k k1-0))))
  (label 206)
  (parent 91)
  (unrealized (3 3) (6 0) (6 3))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k k1 skey)
    (lk locn))
  (deflistener k)
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k1) (lk lk))
  (defstrand make 2 (old old-0) (k k1) (lk lk))
  (precedes ((1 1) (5 0)) ((2 4) (0 0)) ((3 1) (1 0)) ((3 2) (2 0))
    ((4 1) (2 3)) ((5 1) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule invShearsRule trRl_make-at-0 trRl_make-at-1)
  (operation channel-test (added-strand make 2)
    (ch-msg lk (cat pt-3 k1 "init")) (4 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt-2 k "init")) (stor lk (cat pt k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-2 k "init"))
      (load lk (cat pt-0 k1 "wrap")) (send (enc k k1)))
    ((load lk (cat pt-1 old)) (stor lk (cat pt-2 k "init"))
      (send (hash k)))
    ((load lk (cat pt-3 k1 "init")) (stor lk (cat pt-0 k1 "wrap")))
    ((load lk (cat pt-4 old-0)) (stor lk (cat pt-3 k1 "init"))))
  (label 207)
  (parent 92)
  (seen 96)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k1) (lk lk))
  (defstrand make 2 (old old-1) (k k1) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (3 2)) ((2 1) (6 0)) ((3 4) (0 0))
    ((4 2) (3 0)) ((5 1) (3 3)) ((6 1) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule invShearsRule trRl_make-at-0 trRl_make-at-1)
  (operation channel-test (added-strand make 2)
    (ch-msg lk (cat pt-5 k1 "init")) (5 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init"))
      (send (hash k)))
    ((load lk (cat pt-5 k1 "init")) (stor lk (cat pt-2 k1 "wrap")))
    ((load lk (cat pt-6 old-1)) (stor lk (cat pt-5 k1 "init"))))
  (label 208)
  (parent 93)
  (seen 97)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (0 0)) ((3 4) (0 0))
    ((4 1) (3 0)) ((5 1) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation nonce-test (added-strand make 2) k (4 0)
    (ch-msg lk (cat pt-0 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init"))))
  (label 209)
  (parent 94)
  (unrealized (3 3) (4 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old mesg) (pt pt-0 pt-1 pt-2 pt-3 pval) (k k1 k1-0 skey)
    (lk locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (5 2)) ((2 1) (0 0)) ((3 4) (0 0))
    ((4 1) (3 0)) ((5 4) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (4 0)
    (ch-msg lk (cat pt-0 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((recv (hash k)) (recv (hash k1-0)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-3 k1-0 "wrap")) (send (enc k k1-0))))
  (label 210)
  (parent 94)
  (unrealized (3 3) (5 0) (5 3))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((1 1) (3 2)) ((2 1) (0 0)) ((3 4) (0 0))
    ((4 1) (3 0)) ((5 1) (6 0)) ((6 1) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-wrap-at-1 trRl_make-at-0)
  (operation nonce-test (added-strand set-wrap 2) k (4 0)
    (ch-msg lk-0 (cat pt-4 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt-5 k "wrap"))))
  (label 211)
  (parent 95)
  (seen 537)
  (unrealized (3 3) (4 0))
  (comment "6 in cohort - 5 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((1 1) (3 2)) ((2 1) (0 0)) ((3 4) (0 0))
    ((4 1) (3 0)) ((5 1) (6 0)) ((6 1) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-decrypt-at-1 trRl_make-at-0)
  (operation nonce-test (added-strand set-decrypt 2) k (4 0)
    (ch-msg lk-0 (cat pt-4 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((load lk-0 (cat pt-4 k "init"))
      (stor lk-0 (cat pt-5 k "decrypt"))))
  (label 212)
  (parent 95)
  (seen 536)
  (unrealized (3 3) (4 0))
  (comment "6 in cohort - 5 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk-0))
  (precedes ((1 1) (2 0)) ((1 1) (3 2)) ((2 1) (0 0)) ((3 4) (0 0))
    ((4 1) (3 0)) ((5 1) (6 2)) ((6 4) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (4 0)
    (ch-msg lk-0 (cat pt-4 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-0 (cat pt-4 k "init"))
      (load lk-0 (cat pt-5 k1-0 "wrap")) (send (enc k k1-0))))
  (label 213)
  (parent 95)
  (unrealized (3 3) (6 0) (6 3))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (precedes ((1 1) (3 0)) ((2 4) (0 0)) ((3 1) (2 0)) ((4 1) (5 0))
    ((5 1) (3 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-decrypt-at-1 fact-set-decrypt-neq0 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation nonce-test (added-strand set-decrypt 2) k (3 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk-0 (cat pt-3 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init"))
      (stor lk-0 (cat pt-4 k "decrypt"))))
  (label 214)
  (parent 98)
  (unrealized (2 3) (3 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (precedes ((1 1) (3 0)) ((2 4) (0 0)) ((3 1) (2 0)) ((4 1) (5 0))
    ((5 1) (3 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule fact-set-decrypt-neq0 trRl_set-wrap-at-1 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation nonce-test (added-strand set-wrap 2) k (3 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk-0 (cat pt-3 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "wrap"))))
  (label 215)
  (parent 98)
  (unrealized (2 3) (3 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk-0))
  (precedes ((1 1) (3 0)) ((2 4) (0 0)) ((3 1) (2 0)) ((4 1) (5 2))
    ((5 4) (3 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (3 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk-0 (cat pt-3 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-4 k1-0 "wrap")) (send (enc k k1-0))))
  (label 216)
  (parent 98)
  (seen 232)
  (unrealized (2 3) (5 0) (5 3))
  (comment "4 in cohort - 3 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk))
  (precedes ((1 1) (5 2)) ((2 4) (0 0)) ((3 1) (2 0)) ((4 1) (3 0))
    ((5 4) (3 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (3 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk-0 (cat pt-3 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-4 k1-0 "wrap")) (send (enc k k1-0))))
  (label 217)
  (parent 98)
  (seen 219)
  (unrealized (2 3) (5 0) (5 3))
  (comment "4 in cohort - 3 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old mesg) (pt pt-0 pt-1 pt-2 pval) (k k1 k1-0 skey) (lk locn))
  (deflistener k)
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk))
  (defstrand make 3 (old old) (k k) (lk lk))
  (precedes ((1 4) (0 0)) ((2 1) (1 0)) ((3 4) (2 0)) ((4 2) (3 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 "wrap"))
  (operation encryption-test (displaced 1 5 make 3) (hash k) (4 0))
  (traces ((recv k) (send k))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-2 k "init"))
      (load lk (cat pt k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((recv (hash k)) (recv (hash k1-0)) (load lk (cat pt-2 k "init"))
      (load lk (cat pt-0 k1-0 "wrap")) (send (enc k k1-0)))
    ((load lk (cat pt-1 old)) (stor lk (cat pt-2 k "init"))
      (send (hash k))))
  (label 218)
  (parent 99)
  (unrealized (1 3) (3 3))
  (dead)
  (comment "empty cohort"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (precedes ((1 1) (4 2)) ((2 4) (0 0)) ((3 1) (2 0)) ((4 4) (3 0))
    ((5 2) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation encryption-test (added-strand make 3) (hash k) (4 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((recv (hash k)) (recv (hash k1-0)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-2 k1-0 "wrap")) (send (enc k k1-0)))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init"))
      (send (hash k))))
  (label 219)
  (parent 99)
  (unrealized (2 3) (4 3))
  (dead)
  (comment "empty cohort"))

(defskeleton wrap-decrypt
  (vars (old mesg) (pt pt-0 pt-1 pt-2 pval) (k k1 k1-0 skey) (lk locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk))
  (deflistener k)
  (precedes ((1 1) (4 2)) ((2 4) (0 0)) ((3 1) (2 0)) ((4 4) (3 0))
    ((5 1) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 "wrap"))
  (operation encryption-test (added-listener k) (hash k) (4 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((recv (hash k)) (recv (hash k1-0)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-2 k1-0 "wrap")) (send (enc k k1-0)))
    ((recv k) (send k)))
  (label 220)
  (parent 99)
  (unrealized (2 3) (4 3) (5 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "wrap") (k k) (lk lk-0))
  (precedes ((1 1) (2 2)) ((2 4) (0 0)) ((3 1) (2 0)) ((4 1) (5 0))
    ((5 1) (6 0)) ((6 1) (3 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "wrap" "decrypt") (neq "wrap" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule fact-set-decrypt-neq0 trRl_set-wrap-at-1 gen-st-set-wrap-0
    fact-set-wrap-neq0 trRl_make-at-0)
  (operation nonce-test (added-strand set-wrap 2) k (3 0)
    (ch-msg lk-0 (cat pt-3 k "init")) (ch-msg lk-0 (cat pt-4 k "wrap")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "wrap")))
    ((load lk-0 (cat pt-4 k "wrap")) (stor lk-0 (cat pt-5 k "wrap"))))
  (label 221)
  (parent 100)
  (unrealized (2 3) (3 0))
  (dead)
  (comment "empty cohort"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (2 2)) ((2 4) (0 0)) ((3 1) (2 0)) ((4 1) (5 0))
    ((5 1) (3 0)) ((6 1) (3 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation nonce-test (added-strand make 2) k (3 0)
    (ch-msg lk-0 (cat pt-3 k "init")) (ch-msg lk-0 (cat pt-4 k "wrap")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "wrap")))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init"))))
  (label 222)
  (parent 100)
  (seen 564)
  (unrealized (2 3) (3 0))
  (comment "7 in cohort - 6 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "wrap") (k k) (lk lk-0))
  (precedes ((1 1) (2 2)) ((2 4) (0 0)) ((3 1) (2 0)) ((4 1) (5 0))
    ((5 1) (6 0)) ((6 1) (3 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "wrap" "decrypt") (neq "wrap" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-decrypt-at-1 fact-set-decrypt-neq0 gen-st-set-wrap-0
    fact-set-wrap-neq0 trRl_make-at-0)
  (operation nonce-test (added-strand set-decrypt 2) k (3 0)
    (ch-msg lk-0 (cat pt-3 k "init")) (ch-msg lk-0 (cat pt-4 k "wrap")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "wrap")))
    ((load lk-0 (cat pt-4 k "wrap"))
      (stor lk-0 (cat pt-5 k "decrypt"))))
  (label 223)
  (parent 100)
  (unrealized (2 3) (3 0))
  (dead)
  (comment "empty cohort"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1-0) (lk lk-0))
  (precedes ((1 1) (2 2)) ((2 4) (0 0)) ((3 1) (2 0)) ((4 1) (5 0))
    ((5 1) (6 2)) ((6 4) (3 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap")
    (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0 gen-st-wrap-1)
  (operation nonce-test (added-strand wrap 5) k (3 0)
    (ch-msg lk-0 (cat pt-3 k "init")) (ch-msg lk-0 (cat pt-4 k "wrap")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "wrap")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-0 (cat pt-4 k "wrap"))
      (load lk-0 (cat pt-5 k1-0 "wrap")) (send (enc k k1-0))))
  (label 224)
  (parent 100)
  (unrealized (2 3) (6 0) (6 3))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk-0))
  (precedes ((1 1) (2 2)) ((2 4) (0 0)) ((3 1) (2 0)) ((4 1) (5 0))
    ((4 1) (6 2)) ((5 1) (3 0)) ((6 4) (3 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (3 0)
    (ch-msg lk-0 (cat pt-3 k "init")) (ch-msg lk-0 (cat pt-4 k "wrap")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "wrap")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-5 k1-0 "wrap")) (send (enc k k1-0))))
  (label 225)
  (parent 100)
  (unrealized (2 3) (6 0) (6 3))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "decrypt") (k k) (lk lk-0))
  (precedes ((1 1) (2 2)) ((2 4) (0 0)) ((3 1) (2 0)) ((4 1) (5 0))
    ((5 1) (6 0)) ((6 1) (3 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k1 "wrap"))
  (facts (neq "decrypt" "decrypt") (neq "decrypt" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule fact-set-decrypt-neq0 trRl_set-wrap-at-1 gen-st-set-wrap-0
    fact-set-wrap-neq0 trRl_make-at-0)
  (operation nonce-test (added-strand set-wrap 2) k (3 0)
    (ch-msg lk-0 (cat pt-3 k "init"))
    (ch-msg lk-0 (cat pt-4 k "decrypt")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((load lk-0 (cat pt-4 k "decrypt"))
      (stor lk-0 (cat pt-5 k "wrap"))))
  (label 226)
  (parent 101)
  (unrealized (2 3) (3 0))
  (dead)
  (comment "empty cohort"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (2 2)) ((2 4) (0 0)) ((3 1) (2 0)) ((4 1) (5 0))
    ((5 1) (3 0)) ((6 1) (3 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation nonce-test (added-strand make 2) k (3 0)
    (ch-msg lk-0 (cat pt-3 k "init"))
    (ch-msg lk-0 (cat pt-4 k "decrypt")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init"))))
  (label 227)
  (parent 101)
  (seen 557 578)
  (unrealized (2 3) (3 0))
  (comment "7 in cohort - 5 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "decrypt") (k k) (lk lk-0))
  (precedes ((1 1) (2 2)) ((2 4) (0 0)) ((3 1) (2 0)) ((4 1) (5 0))
    ((5 1) (6 0)) ((6 1) (3 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k1 "wrap"))
  (facts (neq "decrypt" "decrypt") (neq "decrypt" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-decrypt-at-1 fact-set-decrypt-neq0 gen-st-set-wrap-0
    fact-set-wrap-neq0 trRl_make-at-0)
  (operation nonce-test (added-strand set-decrypt 2) k (3 0)
    (ch-msg lk-0 (cat pt-3 k "init"))
    (ch-msg lk-0 (cat pt-4 k "decrypt")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((load lk-0 (cat pt-4 k "decrypt"))
      (stor lk-0 (cat pt-5 k "decrypt"))))
  (label 228)
  (parent 101)
  (unrealized (2 3) (3 0))
  (dead)
  (comment "empty cohort"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1-0) (lk lk-0))
  (precedes ((1 1) (2 2)) ((2 4) (0 0)) ((3 1) (2 0)) ((4 1) (5 0))
    ((5 1) (6 2)) ((6 4) (3 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k1 "wrap")
    (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0 gen-st-wrap-1)
  (operation nonce-test (added-strand wrap 5) k (3 0)
    (ch-msg lk-0 (cat pt-3 k "init"))
    (ch-msg lk-0 (cat pt-4 k "decrypt")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k1-0))
      (load lk-0 (cat pt-4 k "decrypt"))
      (load lk-0 (cat pt-5 k1-0 "wrap")) (send (enc k k1-0))))
  (label 229)
  (parent 101)
  (unrealized (2 3) (6 0) (6 3))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk-0))
  (precedes ((1 1) (2 2)) ((2 4) (0 0)) ((3 1) (2 0)) ((4 1) (5 0))
    ((4 1) (6 2)) ((5 1) (3 0)) ((6 4) (3 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (3 0)
    (ch-msg lk-0 (cat pt-3 k "init"))
    (ch-msg lk-0 (cat pt-4 k "decrypt")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-5 k1-0 "wrap")) (send (enc k k1-0))))
  (label 230)
  (parent 101)
  (unrealized (2 3) (6 0) (6 3))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (precedes ((1 1) (2 2)) ((2 4) (0 0)) ((3 1) (2 0)) ((4 4) (3 0))
    ((5 2) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 "wrap"))
  (operation encryption-test (displaced 4 6 make 3) (hash k) (5 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-0 (cat pt-4 k "init"))
      (load lk-0 (cat pt-2 k1-0 "wrap")) (send (enc k k1-0)))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init"))
      (send (hash k))))
  (label 231)
  (parent 102)
  (seen 604)
  (unrealized (2 3) (4 3))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (precedes ((1 4) (0 0)) ((2 1) (1 0)) ((3 1) (4 2)) ((4 4) (2 0))
    ((5 2) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 "wrap"))
  (operation encryption-test (displaced 1 6 make 3) (hash k) (5 0))
  (traces ((recv k) (send k))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-4 k "init"))
      (load lk-0 (cat pt k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt-2 k1-0 "wrap")) (send (enc k k1-0)))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init"))
      (send (hash k))))
  (label 232)
  (parent 102)
  (seen 605)
  (unrealized (1 3) (4 3))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 k1-0 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk-0))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (2 2)) ((2 4) (0 0)) ((3 1) (2 0)) ((4 1) (5 2))
    ((5 4) (3 0)) ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation encryption-test (added-strand make 3) (hash k) (5 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-4 k1-0 "wrap")) (send (enc k k1-0)))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init"))
      (send (hash k))))
  (label 233)
  (parent 102)
  (seen 606)
  (unrealized (2 3) (5 3))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk-0))
  (deflistener k)
  (precedes ((1 1) (2 2)) ((2 4) (0 0)) ((3 1) (2 0)) ((4 1) (5 2))
    ((5 4) (3 0)) ((6 1) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 "wrap"))
  (operation encryption-test (added-listener k) (hash k) (5 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-4 k1-0 "wrap")) (send (enc k k1-0)))
    ((recv k) (send k)))
  (label 234)
  (parent 102)
  (unrealized (2 3) (5 3) (6 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-1))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1) (lk lk-1))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((2 1) (0 0)) ((3 1) (5 2)) ((4 1) (0 0))
    ((5 4) (0 0)) ((6 1) (3 0)) ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 3 7 make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-1 (cat pt-7 k "init")) (stor lk-1 (cat pt-2 k "decrypt")))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk-1 (cat pt-2 k "decrypt"))
      (load lk-1 (cat pt-5 k1 "wrap")) (send (enc k k1)))
    ((load lk-1 (cat pt-6 old-1)) (stor lk-1 (cat pt-7 k "init"))
      (send (hash k))))
  (label 235)
  (parent 107)
  (seen 610 611 612 613)
  (unrealized (5 3))
  (comment "14 in cohort - 4 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-1))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (0 0)) ((2 1) (3 0)) ((3 1) (5 2)) ((4 1) (0 0))
    ((5 4) (0 0)) ((6 1) (1 0)) ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 1 7 make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk-1 (cat pt-7 k "init")) (stor lk-1 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-2 k "decrypt"))
      (load lk (cat pt-5 k1 "wrap")) (send (enc k k1)))
    ((load lk-1 (cat pt-6 old-1)) (stor lk-1 (cat pt-7 k "init"))
      (send (hash k))))
  (label 236)
  (parent 107)
  (seen 614 616)
  (unrealized (5 3))
  (comment "5 in cohort - 3 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-1))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((2 1) (0 0)) ((3 1) (0 0)) ((4 1) (5 2))
    ((5 4) (0 0)) ((6 1) (3 0)) ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 3 7 make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-1 (cat pt-7 k "init")) (stor lk-1 (cat pt-2 k "decrypt")))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-4 k "init"))
      (load lk-0 (cat pt-5 k1 "wrap")) (send (enc k k1)))
    ((load lk-1 (cat pt-6 old-1)) (stor lk-1 (cat pt-7 k "init"))
      (send (hash k))))
  (label 239)
  (parent 108)
  (seen 172 306 617)
  (unrealized (5 3))
  (comment "8 in cohort - 1 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-1))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (0 0)) ((2 1) (3 0)) ((3 1) (0 0)) ((4 1) (5 2))
    ((5 4) (0 0)) ((6 1) (1 0)) ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 1 7 make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk-1 (cat pt-7 k "init")) (stor lk-1 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-4 k "init"))
      (load lk-0 (cat pt-5 k1 "wrap")) (send (enc k k1)))
    ((load lk-1 (cat pt-6 old-1)) (stor lk-1 (cat pt-7 k "init"))
      (send (hash k))))
  (label 240)
  (parent 108)
  (seen 289 618)
  (unrealized (5 3))
  (comment "7 in cohort - 1 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-1))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((2 1) (0 0)) ((3 1) (4 0)) ((4 1) (0 0))
    ((5 4) (0 0)) ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 5 7 make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk-1 (cat pt-7 k "init"))
      (load lk-1 (cat pt-5 k1 "wrap")) (send (enc k k1)))
    ((load lk-1 (cat pt-6 old-1)) (stor lk-1 (cat pt-7 k "init"))
      (send (hash k))))
  (label 241)
  (parent 108)
  (seen 289 619 620)
  (unrealized (5 3))
  (comment "8 in cohort - 2 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-1))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-1))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((2 1) (0 0)) ((3 1) (0 0)) ((4 1) (0 0))
    ((5 4) (0 0)) ((6 1) (3 0)) ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 3 7 make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-1 (cat pt-7 k "init")) (stor lk-1 (cat pt-2 k "decrypt")))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk-1 (cat pt-7 k "init"))
      (load lk-1 (cat pt-5 k1 "wrap")) (send (enc k k1)))
    ((load lk-1 (cat pt-6 old-1)) (stor lk-1 (cat pt-7 k "init"))
      (send (hash k))))
  (label 244)
  (parent 109)
  (seen 621 622 623)
  (unrealized (5 3))
  (comment "14 in cohort - 3 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-1))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (0 0)) ((2 1) (3 0)) ((2 1) (5 2)) ((3 1) (0 0))
    ((4 1) (0 0)) ((5 4) (0 0)) ((6 1) (1 0)) ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 1 7 make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk-1 (cat pt-7 k "init")) (stor lk-1 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt-5 k1 "wrap")) (send (enc k k1)))
    ((load lk-1 (cat pt-6 old-1)) (stor lk-1 (cat pt-7 k "init"))
      (send (hash k))))
  (label 245)
  (parent 109)
  (seen 624)
  (unrealized (5 3))
  (comment "4 in cohort - 1 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-1))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((2 1) (5 2)) ((3 1) (0 0)) ((4 1) (0 0))
    ((5 4) (0 0)) ((6 1) (3 0)) ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 3 7 make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-1 (cat pt-7 k "init")) (stor lk-1 (cat pt-2 k "decrypt")))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-5 k1 "wrap")) (send (enc k k1)))
    ((load lk-1 (cat pt-6 old-1)) (stor lk-1 (cat pt-7 k "init"))
      (send (hash k))))
  (label 248)
  (parent 110)
  (seen 334 625)
  (unrealized (5 3))
  (comment "4 in cohort - 1 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-1))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk-1))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (5 2)) ((2 1) (3 0)) ((3 1) (0 0)) ((4 1) (0 0))
    ((5 4) (0 0)) ((6 1) (1 0)) ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 1 7 make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk-1 (cat pt-7 k "init")) (stor lk-1 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk-1 (cat pt k "wrap"))
      (load lk-1 (cat pt-5 k1 "wrap")) (send (enc k k1)))
    ((load lk-1 (cat pt-6 old-1)) (stor lk-1 (cat pt-7 k "init"))
      (send (hash k))))
  (label 249)
  (parent 110)
  (seen 626 627)
  (unrealized (5 3))
  (comment "4 in cohort - 2 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-1))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((1 1) (5 2)) ((2 1) (0 0)) ((3 1) (0 0))
    ((4 1) (0 0)) ((5 4) (0 0)) ((6 1) (3 0)) ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 3 7 make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-1 (cat pt-7 k "init")) (stor lk-1 (cat pt-2 k "decrypt")))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-5 k1 "wrap")) (send (enc k k1)))
    ((load lk-1 (cat pt-6 old-1)) (stor lk-1 (cat pt-7 k "init"))
      (send (hash k))))
  (label 252)
  (parent 111)
  (seen 357 628)
  (unrealized (5 3))
  (comment "7 in cohort - 1 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-1))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-1))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (0 0)) ((2 1) (3 0)) ((3 1) (0 0)) ((4 1) (0 0))
    ((5 4) (0 0)) ((6 1) (1 0)) ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 1 7 make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk-1 (cat pt-7 k "init")) (stor lk-1 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk-1 (cat pt-7 k "init"))
      (load lk-1 (cat pt-5 k1 "wrap")) (send (enc k k1)))
    ((load lk-1 (cat pt-6 old-1)) (stor lk-1 (cat pt-7 k "init"))
      (send (hash k))))
  (label 253)
  (parent 111)
  (seen 629 630)
  (unrealized (5 3))
  (comment "7 in cohort - 2 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k skey)
    (lk locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (4 3)) ((3 1) (1 0)) ((3 1) (4 2))
    ((4 4) (0 0)) ((5 1) (3 0)) ((5 2) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule invShearsRule shearsRule)
  (operation channel-test (displaced 6 2 set-wrap 2)
    (ch-msg lk-0 (cat pt-5 k1 "wrap")) (4 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk (cat pt-4 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-2 k "decrypt"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-3 old-0)) (stor lk (cat pt-4 k "init"))
      (send (hash k))))
  (label 256)
  (parent 112)
  (seen 610)
  (unrealized (0 0))
  (comment "8 in cohort - 7 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k skey)
    (lk locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (4 3)) ((3 1) (2 0)) ((3 1) (4 2))
    ((4 4) (0 0)) ((5 1) (1 0)) ((5 1) (3 0)) ((5 2) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule invShearsRule shearsRule)
  (operation channel-test (displaced 6 2 set-wrap 2)
    (ch-msg lk-0 (cat pt-5 k1 "wrap")) (4 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk (cat pt-4 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-2 k "decrypt"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-3 old-0)) (stor lk (cat pt-4 k "init"))
      (send (hash k))))
  (label 257)
  (parent 112)
  (seen 256 610 632 634 635 636 637)
  (unrealized (0 0))
  (comment "16 in cohort - 0 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k skey)
    (lk locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (4 3)) ((3 1) (2 0)) ((3 1) (4 2))
    ((4 4) (0 0)) ((5 1) (3 0)) ((5 2) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule invShearsRule)
  (operation channel-test (displaced 6 2 set-wrap 2)
    (ch-msg lk-0 (cat pt-5 k1 "wrap")) (4 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk (cat pt-4 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-2 k "decrypt"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-3 old-0)) (stor lk (cat pt-4 k "init"))
      (send (hash k))))
  (label 258)
  (parent 112)
  (seen 256 610 632 634 635 636 637)
  (unrealized (0 0))
  (comment "16 in cohort - 0 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 cur mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur cur) (k k1) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (0 0)) ((3 1) (4 2)) ((3 1) (6 0))
    ((4 4) (0 0)) ((5 1) (3 0)) ((5 2) (4 0)) ((6 1) (4 3)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k1 cur) (cat k1 "wrap"))
  (facts (neq cur "decrypt") (neq cur "wrap") (neq "init" "decrypt")
    (neq "init" "wrap"))
  (rule invShearsRule fact-set-decrypt-neq0 trRl_set-wrap-at-1
    gen-st-set-wrap-0 fact-set-wrap-neq0 trRl_make-at-0)
  (operation channel-test (added-strand set-wrap 2)
    (ch-msg lk-0 (cat pt-3 k1 "wrap")) (4 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-5 k "init")) (stor lk-0 (cat pt-2 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-2 k "decrypt"))
      (load lk-0 (cat pt-3 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-4 old-0)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-6 k1 cur)) (stor lk-0 (cat pt-3 k1 "wrap"))))
  (label 259)
  (parent 112)
  (seen 256 638)
  (unrealized (6 0))
  (comment "4 in cohort - 1 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k skey)
    (lk locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (precedes ((1 1) (4 3)) ((2 1) (3 0)) ((2 1) (5 0)) ((3 1) (1 0))
    ((3 1) (4 2)) ((4 4) (0 0)) ((5 1) (1 0)) ((5 2) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule invShearsRule)
  (operation channel-test (displaced 6 1 set-wrap 2)
    (ch-msg lk-0 (cat pt-5 k1 "wrap")) (4 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt-4 k "init")) (stor lk (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-2 k "decrypt"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-3 old-0)) (stor lk (cat pt-4 k "init"))
      (send (hash k))))
  (label 260)
  (parent 113)
  (unrealized (0 0))
  (dead)
  (comment "empty cohort"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k skey)
    (lk locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (precedes ((1 1) (4 3)) ((2 1) (3 0)) ((3 1) (1 0)) ((3 1) (4 2))
    ((4 4) (0 0)) ((5 1) (1 0)) ((5 2) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule invShearsRule)
  (operation channel-test (displaced 6 1 set-wrap 2)
    (ch-msg lk-0 (cat pt-5 k1 "wrap")) (4 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt-4 k "init")) (stor lk (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-2 k "decrypt"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-3 old-0)) (stor lk (cat pt-4 k "init"))
      (send (hash k))))
  (label 261)
  (parent 113)
  (unrealized (0 0))
  (dead)
  (comment "empty cohort"))

(defskeleton wrap-decrypt
  (vars (old old-0 cur mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur cur) (k k1) (lk lk))
  (precedes ((1 1) (0 0)) ((2 1) (3 0)) ((3 1) (4 2)) ((3 1) (6 0))
    ((4 4) (0 0)) ((5 1) (1 0)) ((5 2) (4 0)) ((6 1) (4 3)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k1 cur) (cat k1 "wrap"))
  (facts (neq cur "decrypt") (neq cur "wrap") (neq "init" "decrypt")
    (neq "init" "wrap"))
  (rule invShearsRule fact-set-decrypt-neq0 trRl_set-wrap-at-1
    gen-st-set-wrap-0 fact-set-wrap-neq0 trRl_make-at-0)
  (operation channel-test (added-strand set-wrap 2)
    (ch-msg lk (cat pt-3 k1 "wrap")) (4 3))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-5 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-2 k "decrypt"))
      (load lk (cat pt-3 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-4 old-0)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k)))
    ((load lk (cat pt-6 k1 cur)) (stor lk (cat pt-3 k1 "wrap"))))
  (label 262)
  (parent 113)
  (seen 639)
  (unrealized (6 0))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (5 3)) ((3 1) (4 0)) ((4 1) (1 0))
    ((4 1) (5 2)) ((5 4) (0 0)) ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule invShearsRule shearsRule)
  (operation channel-test (displaced 7 2 set-wrap 2)
    (ch-msg lk-1 (cat pt-7 k1 "wrap")) (5 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init")))
    ((load lk (cat pt-3 k "init")) (stor lk (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-4 k "decrypt"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-5 old-1)) (stor lk-0 (cat pt-6 k "init"))
      (send (hash k))))
  (label 263)
  (parent 114)
  (unrealized (0 0))
  (comment "11 in cohort - 11 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (5 3)) ((3 1) (1 0)) ((3 1) (4 0))
    ((4 1) (2 0)) ((4 1) (5 2)) ((5 4) (0 0)) ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule invShearsRule shearsRule)
  (operation channel-test (displaced 7 2 set-wrap 2)
    (ch-msg lk-1 (cat pt-7 k1 "wrap")) (5 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init")))
    ((load lk (cat pt-3 k "init")) (stor lk (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-4 k "decrypt"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-5 old-1)) (stor lk-0 (cat pt-6 k "init"))
      (send (hash k))))
  (label 264)
  (parent 114)
  (seen 263 640 642 643 644 646 647 648 649 650)
  (unrealized (0 0))
  (comment "22 in cohort - 0 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (5 3)) ((3 1) (4 0)) ((4 1) (2 0))
    ((4 1) (5 2)) ((5 4) (0 0)) ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule invShearsRule)
  (operation channel-test (displaced 7 2 set-wrap 2)
    (ch-msg lk-1 (cat pt-7 k1 "wrap")) (5 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init")))
    ((load lk (cat pt-3 k "init")) (stor lk (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-4 k "decrypt"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-5 old-1)) (stor lk-0 (cat pt-6 k "init"))
      (send (hash k))))
  (label 265)
  (parent 114)
  (seen 263 640 642 643 644 646 647 648 649 650)
  (unrealized (0 0))
  (comment "22 in cohort - 0 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1) (lk lk-0))
  (deflistener k)
  (precedes ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (0 0)) ((3 1) (4 0))
    ((4 1) (5 2)) ((5 4) (0 0)) ((6 1) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation nonce-test (displaced 7 1 make 2) k (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-4 k "decrypt"))
      (load lk-0 (cat pt-5 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k)))
  (label 267)
  (parent 115)
  (seen 204)
  (unrealized (5 3) (6 0))
  (comment "5 in cohort - 3 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1) (lk lk-0))
  (deflistener k)
  (precedes ((1 1) (2 0)) ((2 1) (0 0)) ((3 1) (4 0)) ((3 1) (6 0))
    ((4 1) (5 2)) ((5 4) (0 0)) ((6 1) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation nonce-test (displaced 7 3 make 2) k (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-4 k "decrypt"))
      (load lk-0 (cat pt-5 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k)))
  (label 268)
  (parent 115)
  (seen 652 655)
  (unrealized (5 3) (6 0))
  (comment "5 in cohort - 3 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k skey)
    (lk locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (4 3)) ((3 1) (1 0)) ((4 4) (0 0))
    ((5 1) (3 0)) ((5 2) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule invShearsRule shearsRule)
  (operation channel-test (displaced 6 2 set-wrap 2)
    (ch-msg lk-0 (cat pt-5 k1 "wrap")) (4 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk (cat pt-4 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-4 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-3 old-0)) (stor lk (cat pt-4 k "init"))
      (send (hash k))))
  (label 270)
  (parent 116)
  (seen 621)
  (unrealized (0 0))
  (comment "8 in cohort - 7 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k skey)
    (lk locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (4 3)) ((3 1) (2 0)) ((4 4) (0 0))
    ((5 1) (1 0)) ((5 1) (3 0)) ((5 2) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule invShearsRule shearsRule)
  (operation channel-test (displaced 6 2 set-wrap 2)
    (ch-msg lk-0 (cat pt-5 k1 "wrap")) (4 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk (cat pt-4 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-4 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-3 old-0)) (stor lk (cat pt-4 k "init"))
      (send (hash k))))
  (label 271)
  (parent 116)
  (seen 270 621 658 660 661 662 663)
  (unrealized (0 0))
  (comment "16 in cohort - 0 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 cur mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur cur) (k k1) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (0 0)) ((3 1) (6 0)) ((4 4) (0 0))
    ((5 1) (3 0)) ((5 2) (4 0)) ((6 1) (4 3)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 cur) (cat k1 "wrap"))
  (facts (neq cur "decrypt") (neq cur "wrap") (neq "init" "decrypt")
    (neq "init" "wrap"))
  (rule shearsRule invShearsRule fact-set-decrypt-neq0
    trRl_set-wrap-at-1 gen-st-set-wrap-0 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation channel-test (added-strand set-wrap 2)
    (ch-msg lk-0 (cat pt-3 k1 "wrap")) (4 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-5 k "init")) (stor lk-0 (cat pt-2 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-5 k "init"))
      (load lk-0 (cat pt-3 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-4 old-0)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-6 k1 cur)) (stor lk-0 (cat pt-3 k1 "wrap"))))
  (label 272)
  (parent 116)
  (seen 270 664)
  (unrealized (6 0))
  (comment "4 in cohort - 1 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 cur mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur cur) (k k1) (lk lk))
  (precedes ((1 1) (0 0)) ((2 1) (3 0)) ((2 1) (4 2)) ((3 1) (6 0))
    ((4 4) (0 0)) ((5 1) (1 0)) ((5 2) (4 0)) ((6 1) (4 3)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 cur) (cat k1 "wrap"))
  (facts (neq cur "decrypt") (neq cur "wrap") (neq "init" "decrypt")
    (neq "init" "wrap"))
  (rule shearsRule invShearsRule fact-set-decrypt-neq0
    trRl_set-wrap-at-1 gen-st-set-wrap-0 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation channel-test (added-strand set-wrap 2)
    (ch-msg lk (cat pt-3 k1 "wrap")) (4 3))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-5 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt-3 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-4 old-0)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k)))
    ((load lk (cat pt-6 k1 cur)) (stor lk (cat pt-3 k1 "wrap"))))
  (label 273)
  (parent 117)
  (seen 665)
  (unrealized (6 0))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (5 3)) ((3 1) (4 0)) ((3 1) (5 2))
    ((4 1) (1 0)) ((5 4) (0 0)) ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule invShearsRule shearsRule)
  (operation channel-test (displaced 7 2 set-wrap 2)
    (ch-msg lk-1 (cat pt-7 k1 "wrap")) (5 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init")))
    ((load lk (cat pt-3 k "init")) (stor lk (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-5 old-1)) (stor lk-0 (cat pt-6 k "init"))
      (send (hash k))))
  (label 274)
  (parent 118)
  (unrealized (0 0))
  (comment "11 in cohort - 11 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (5 3)) ((3 1) (1 0)) ((3 1) (4 0))
    ((3 1) (5 2)) ((4 1) (2 0)) ((5 4) (0 0)) ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule invShearsRule shearsRule)
  (operation channel-test (displaced 7 2 set-wrap 2)
    (ch-msg lk-1 (cat pt-7 k1 "wrap")) (5 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init")))
    ((load lk (cat pt-3 k "init")) (stor lk (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-5 old-1)) (stor lk-0 (cat pt-6 k "init"))
      (send (hash k))))
  (label 275)
  (parent 118)
  (seen 274 666 668 669 670 672 673 674 675 676)
  (unrealized (0 0))
  (comment "22 in cohort - 0 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (deflistener k)
  (precedes ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (0 0)) ((3 1) (4 0))
    ((3 1) (5 2)) ((4 1) (0 0)) ((5 4) (0 0)) ((6 1) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation nonce-test (displaced 7 1 make 2) k (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-5 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k)))
  (label 277)
  (parent 119)
  (seen 211)
  (unrealized (5 3) (6 0))
  (comment "5 in cohort - 3 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (deflistener k)
  (precedes ((1 1) (2 0)) ((2 1) (0 0)) ((3 1) (4 0)) ((3 1) (6 0))
    ((4 1) (0 0)) ((5 4) (0 0)) ((6 1) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation nonce-test (displaced 7 3 make 2) k (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-5 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k)))
  (label 278)
  (parent 119)
  (seen 678)
  (unrealized (5 3) (6 0))
  (comment "3 in cohort - 2 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 cur mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur cur) (k k1) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (4 2)) ((2 1) (6 0)) ((3 1) (0 0))
    ((4 4) (0 0)) ((5 1) (3 0)) ((5 2) (4 0)) ((6 1) (4 3)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 cur) (cat k1 "wrap"))
  (facts (neq cur "decrypt") (neq cur "wrap") (neq "init" "decrypt")
    (neq "init" "wrap"))
  (rule invShearsRule fact-set-decrypt-neq0 trRl_set-wrap-at-1
    gen-st-set-wrap-0 fact-set-wrap-neq0 trRl_make-at-0)
  (operation channel-test (added-strand set-wrap 2)
    (ch-msg lk (cat pt-3 k1 "wrap")) (4 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-5 k "init")) (stor lk-0 (cat pt-2 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-3 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-4 old-0)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k)))
    ((load lk (cat pt-6 k1 cur)) (stor lk (cat pt-3 k1 "wrap"))))
  (label 280)
  (parent 120)
  (seen 682)
  (unrealized (6 0))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (precedes ((1 1) (4 2)) ((2 1) (3 0)) ((3 1) (0 0)) ((4 4) (0 0))
    ((5 1) (1 0)) ((5 2) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation channel-test (displaced 6 1 set-wrap 2)
    (ch-msg lk-0 (cat pt-5 k1 "wrap")) (4 3))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt k "wrap"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init"))
      (send (hash k))))
  (label 281)
  (parent 121)
  (seen 626)
  (unrealized (0 0))
  (comment "10 in cohort - 9 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 cur mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur cur) (k k1) (lk lk-0))
  (precedes ((1 1) (4 2)) ((1 1) (6 0)) ((2 1) (3 0)) ((3 1) (0 0))
    ((4 4) (0 0)) ((5 1) (1 0)) ((5 2) (4 0)) ((6 1) (4 3)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 cur) (cat k1 "wrap"))
  (facts (neq cur "decrypt") (neq cur "wrap") (neq "init" "decrypt")
    (neq "init" "wrap"))
  (rule invShearsRule fact-set-decrypt-neq0 trRl_set-wrap-at-1
    gen-st-set-wrap-0 fact-set-wrap-neq0 trRl_make-at-0)
  (operation channel-test (added-strand set-wrap 2)
    (ch-msg lk-0 (cat pt-3 k1 "wrap")) (4 3))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-5 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt k "wrap"))
      (load lk-0 (cat pt-3 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-4 old-0)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-6 k1 cur)) (stor lk-0 (cat pt-3 k1 "wrap"))))
  (label 282)
  (parent 121)
  (seen 692)
  (unrealized (6 0))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((2 1) (5 2)) ((3 1) (4 0)) ((4 1) (0 0))
    ((5 4) (0 0)) ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation channel-test (displaced 7 2 set-wrap 2)
    (ch-msg lk (cat pt-7 k1 "wrap")) (5 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init"))
      (send (hash k))))
  (label 283)
  (parent 122)
  (unrealized (0 0))
  (comment "13 in cohort - 13 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (precedes ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (5 2)) ((3 1) (4 0))
    ((4 1) (0 0)) ((5 4) (0 0)) ((6 1) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation nonce-test (displaced 7 1 make 2) k (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-5 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k)))
  (label 285)
  (parent 123)
  (seen 706)
  (unrealized (5 3) (6 0))
  (comment "5 in cohort - 4 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (precedes ((1 1) (2 0)) ((2 1) (5 2)) ((3 1) (4 0)) ((3 1) (6 0))
    ((4 1) (0 0)) ((5 4) (0 0)) ((6 1) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation nonce-test (displaced 7 3 make 2) k (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-5 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k)))
  (label 286)
  (parent 123)
  (seen 165 708)
  (unrealized (5 3) (6 0))
  (comment "5 in cohort - 2 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 cur mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur cur) (k k1) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (4 2)) ((2 1) (6 0)) ((3 1) (0 0))
    ((4 4) (0 0)) ((5 1) (3 0)) ((5 2) (4 0)) ((6 1) (4 3)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 cur) (cat k1 "wrap"))
  (facts (neq cur "decrypt") (neq cur "wrap") (neq "init" "decrypt")
    (neq "init" "wrap"))
  (rule shearsRule invShearsRule fact-set-decrypt-neq0
    trRl_set-wrap-at-1 gen-st-set-wrap-0 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation channel-test (added-strand set-wrap 2)
    (ch-msg lk (cat pt-3 k1 "wrap")) (4 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-5 k "init")) (stor lk-0 (cat pt-2 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-3 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-4 old-0)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k)))
    ((load lk (cat pt-6 k1 cur)) (stor lk (cat pt-3 k1 "wrap"))))
  (label 288)
  (parent 124)
  (seen 712)
  (unrealized (6 0))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (precedes ((1 1) (4 3)) ((2 1) (3 0)) ((3 1) (0 0)) ((4 4) (0 0))
    ((5 1) (1 0)) ((5 2) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation channel-test (displaced 6 1 set-wrap 2)
    (ch-msg lk-0 (cat pt-5 k1 "wrap")) (4 3))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt-4 k "init"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init"))
      (send (hash k))))
  (label 289)
  (parent 125)
  (seen 629)
  (unrealized (0 0))
  (comment "10 in cohort - 9 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 cur mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur cur) (k k1) (lk lk-0))
  (precedes ((1 1) (6 0)) ((2 1) (3 0)) ((3 1) (0 0)) ((4 4) (0 0))
    ((5 1) (1 0)) ((5 2) (4 0)) ((6 1) (4 3)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 cur) (cat k1 "wrap"))
  (facts (neq cur "decrypt") (neq cur "wrap") (neq "init" "decrypt")
    (neq "init" "wrap"))
  (rule shearsRule invShearsRule fact-set-decrypt-neq0
    trRl_set-wrap-at-1 gen-st-set-wrap-0 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation channel-test (added-strand set-wrap 2)
    (ch-msg lk-0 (cat pt-3 k1 "wrap")) (4 3))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-5 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-5 k "init"))
      (load lk-0 (cat pt-3 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-4 old-0)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-6 k1 cur)) (stor lk-0 (cat pt-3 k1 "wrap"))))
  (label 290)
  (parent 125)
  (seen 722)
  (unrealized (6 0))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((1 1) (5 2)) ((2 1) (5 3)) ((3 1) (4 0))
    ((4 1) (0 0)) ((5 4) (0 0)) ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation channel-test (displaced 7 2 set-wrap 2)
    (ch-msg lk (cat pt-7 k1 "wrap")) (5 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init"))
      (send (hash k))))
  (label 291)
  (parent 126)
  (unrealized (0 0))
  (comment "13 in cohort - 13 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (precedes ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (0 0)) ((3 1) (4 0))
    ((4 1) (0 0)) ((5 4) (0 0)) ((6 1) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation nonce-test (displaced 7 1 make 2) k (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-5 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k)))
  (label 293)
  (parent 127)
  (unrealized (5 3) (6 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (precedes ((1 1) (2 0)) ((1 1) (5 2)) ((2 1) (0 0)) ((3 1) (4 0))
    ((3 1) (6 0)) ((4 1) (0 0)) ((5 4) (0 0)) ((6 1) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation nonce-test (displaced 7 3 make 2) k (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-5 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k)))
  (label 294)
  (parent 127)
  (seen 183 737)
  (unrealized (5 3) (6 0))
  (comment "5 in cohort - 2 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k skey)
    (lk locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "wrap") (k k) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (3 3)) ((2 1) (5 0)) ((3 4) (0 0))
    ((4 1) (1 0)) ((4 2) (3 0)) ((5 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "wrap" "decrypt") (neq "wrap" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule fact-set-decrypt-neq0 trRl_set-wrap-at-1 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation nonce-test (added-strand set-wrap 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk (cat pt-3 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk (cat pt-1 k "wrap")) (stor lk (cat pt-4 k "wrap"))))
  (label 296)
  (parent 128)
  (unrealized (0 0))
  (dead)
  (comment "empty cohort"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand make 2 (old old-1) (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (3 3)) ((3 4) (0 0)) ((4 1) (1 0))
    ((4 2) (3 0)) ((5 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation nonce-test (added-strand make 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk (cat pt-3 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-4 old-1)) (stor lk-0 (cat pt-5 k "init"))))
  (label 297)
  (parent 128)
  (seen 142 619 621 742)
  (unrealized (0 0))
  (comment "21 in cohort - 8 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k skey)
    (lk locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (3 3)) ((3 4) (0 0)) ((4 1) (5 0))
    ((4 2) (3 0)) ((5 1) (1 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule invShearsRule shearsRule trRl_set-wrap-at-1 trRl_make-at-0)
  (operation nonce-test (added-strand set-wrap 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk (cat pt-3 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk (cat pt-3 k "init")) (stor lk (cat pt-4 k "wrap"))))
  (label 298)
  (parent 128)
  (seen 742)
  (unrealized (0 0))
  (comment "7 in cohort - 6 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k skey)
    (lk locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "wrap") (k k) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (3 3)) ((2 1) (5 0)) ((3 4) (0 0))
    ((4 1) (1 0)) ((4 2) (3 0)) ((5 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "wrap" "decrypt") (neq "wrap" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-decrypt-at-1 fact-set-decrypt-neq0 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation nonce-test (added-strand set-decrypt 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk (cat pt-3 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk (cat pt-1 k "wrap")) (stor lk (cat pt-4 k "decrypt"))))
  (label 299)
  (parent 128)
  (unrealized (0 0))
  (dead)
  (comment "empty cohort"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k k1 skey)
    (lk locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (3 3)) ((3 4) (0 0)) ((4 1) (1 0))
    ((4 1) (5 2)) ((4 2) (3 0)) ((5 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk (cat pt-3 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1))))
  (label 300)
  (parent 128)
  (unrealized (5 0) (5 3))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k k1 skey)
    (lk locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (3 3)) ((2 1) (5 2)) ((3 4) (0 0))
    ((4 1) (1 0)) ((4 2) (3 0)) ((5 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk (cat pt-3 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1))))
  (label 301)
  (parent 128)
  (unrealized (5 0) (5 3))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k k1 skey)
    (lk locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (5 2)) ((2 1) (3 3)) ((3 4) (0 0))
    ((4 1) (1 0)) ((4 2) (3 0)) ((5 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk (cat pt-3 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1))))
  (label 302)
  (parent 128)
  (unrealized (5 0) (5 3))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k1) (lk lk-0))
  (defstrand make 2 (old old-1) (k k1) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (0 0)) ((3 4) (0 0)) ((4 1) (6 0))
    ((4 2) (3 0)) ((5 1) (3 3)) ((6 1) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule invShearsRule trRl_make-at-0 trRl_make-at-1)
  (operation channel-test (added-strand make 2)
    (ch-msg lk-0 (cat pt-5 k1 "init")) (5 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-4 k "init"))
      (load lk-0 (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-5 k1 "init")) (stor lk-0 (cat pt-2 k1 "wrap")))
    ((load lk-0 (cat pt-6 old-1)) (stor lk-0 (cat pt-5 k1 "init"))))
  (label 303)
  (parent 129)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (0 0)) ((3 4) (0 0)) ((4 1) (5 0))
    ((4 2) (3 0)) ((5 1) (3 3)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule invShearsRule trRl_make-at-0 trRl_make-at-1)
  (operation channel-test (added-strand make 2)
    (ch-msg lk-0 (cat pt-4 k "init")) (5 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt-4 k "init"))
      (load lk-0 (cat pt-2 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt-2 k "wrap"))))
  (label 304)
  (parent 129)
  (unrealized (0 0))
  (comment "9 in cohort - 9 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k1) (lk lk))
  (defstrand make 2 (old old-1) (k k1) (lk lk))
  (precedes ((1 1) (0 0)) ((2 1) (3 2)) ((2 1) (6 0)) ((3 4) (0 0))
    ((4 1) (1 0)) ((4 2) (3 0)) ((5 1) (3 3)) ((6 1) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule invShearsRule trRl_make-at-0 trRl_make-at-1)
  (operation channel-test (added-strand make 2)
    (ch-msg lk (cat pt-5 k1 "init")) (5 0))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init"))
      (send (hash k)))
    ((load lk (cat pt-5 k1 "init")) (stor lk (cat pt-2 k1 "wrap")))
    ((load lk (cat pt-6 old-1)) (stor lk (cat pt-5 k1 "init"))))
  (label 305)
  (parent 130)
  (seen 97)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (4 3)) ((3 1) (1 0)) ((3 1) (4 2))
    ((4 4) (0 0)) ((5 1) (6 0)) ((5 2) (4 0)) ((6 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-decrypt-at-1 trRl_make-at-0)
  (operation nonce-test (added-strand set-decrypt 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk (cat pt-3 k "init")) (ch-msg lk-0 (cat pt-5 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-4 old-1)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-5 k "init"))
      (stor lk-0 (cat pt-6 k "decrypt"))))
  (label 306)
  (parent 131)
  (seen 172 666 780)
  (unrealized (0 0))
  (comment "24 in cohort - 12 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "wrap") (k k) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (4 3)) ((2 1) (6 0)) ((3 1) (1 0))
    ((3 1) (4 2)) ((4 4) (0 0)) ((5 2) (4 0)) ((6 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "wrap" "decrypt") (neq "wrap" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule fact-set-decrypt-neq0 trRl_set-wrap-at-1 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation nonce-test (added-strand set-wrap 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk (cat pt-3 k "init")) (ch-msg lk-0 (cat pt-5 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-4 old-1)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k)))
    ((load lk (cat pt-1 k "wrap")) (stor lk (cat pt-6 k "wrap"))))
  (label 307)
  (parent 131)
  (unrealized (0 0))
  (dead)
  (comment "empty cohort"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 old-2 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-0))
  (defstrand make 2 (old old-2) (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((2 1) (4 3)) ((3 1) (1 0)) ((3 1) (4 2))
    ((4 4) (0 0)) ((5 2) (4 0)) ((6 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation nonce-test (added-strand make 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk (cat pt-3 k "init")) (ch-msg lk-0 (cat pt-5 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-4 old-1)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k)))
    ((load lk-1 (cat pt-6 old-2)) (stor lk-1 (cat pt-7 k "init"))))
  (label 308)
  (parent 131)
  (seen 144 669 779 792)
  (unrealized (0 0))
  (comment "24 in cohort - 11 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (4 3)) ((3 1) (1 0)) ((3 1) (4 2))
    ((4 4) (0 0)) ((5 1) (6 0)) ((5 2) (4 0)) ((6 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-wrap-at-1 trRl_make-at-0)
  (operation nonce-test (added-strand set-wrap 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk (cat pt-3 k "init")) (ch-msg lk-0 (cat pt-5 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-4 old-1)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-5 k "init")) (stor lk-0 (cat pt-6 k "wrap"))))
  (label 309)
  (parent 131)
  (seen 174 668 791 802)
  (unrealized (0 0))
  (comment "23 in cohort - 10 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (4 3)) ((3 1) (4 2)) ((3 1) (6 0))
    ((4 4) (0 0)) ((5 2) (4 0)) ((6 1) (1 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule invShearsRule shearsRule trRl_set-wrap-at-1 trRl_make-at-0)
  (operation nonce-test (added-strand set-wrap 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk (cat pt-3 k "init")) (ch-msg lk-0 (cat pt-5 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-4 old-1)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k)))
    ((load lk (cat pt-3 k "init")) (stor lk (cat pt-6 k "wrap"))))
  (label 310)
  (parent 131)
  (seen 780 792 802)
  (unrealized (0 0))
  (comment "10 in cohort - 7 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "wrap") (k k) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (4 3)) ((2 1) (6 0)) ((3 1) (1 0))
    ((3 1) (4 2)) ((4 4) (0 0)) ((5 2) (4 0)) ((6 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "wrap" "decrypt") (neq "wrap" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-decrypt-at-1 fact-set-decrypt-neq0 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation nonce-test (added-strand set-decrypt 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk (cat pt-3 k "init")) (ch-msg lk-0 (cat pt-5 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-4 old-1)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k)))
    ((load lk (cat pt-1 k "wrap")) (stor lk (cat pt-6 k "decrypt"))))
  (label 311)
  (parent 131)
  (unrealized (0 0))
  (dead)
  (comment "empty cohort"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (4 3)) ((3 1) (1 0)) ((3 1) (4 2))
    ((3 1) (6 2)) ((4 4) (0 0)) ((5 2) (4 0)) ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk (cat pt-3 k "init")) (ch-msg lk-0 (cat pt-5 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-4 old-1)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-6 k1 "wrap")) (send (enc k k1))))
  (label 312)
  (parent 131)
  (unrealized (6 0) (6 3))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (4 3)) ((3 1) (1 0)) ((3 1) (4 2))
    ((4 4) (0 0)) ((5 1) (6 2)) ((5 2) (4 0)) ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk (cat pt-3 k "init")) (ch-msg lk-0 (cat pt-5 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-4 old-1)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-5 k "init"))
      (load lk-0 (cat pt-6 k1 "wrap")) (send (enc k k1))))
  (label 313)
  (parent 131)
  (unrealized (6 0) (6 3))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (4 3)) ((2 1) (6 2)) ((3 1) (1 0))
    ((3 1) (4 2)) ((4 4) (0 0)) ((5 2) (4 0)) ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk (cat pt-3 k "init")) (ch-msg lk-0 (cat pt-5 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-4 old-1)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-6 k1 "wrap")) (send (enc k k1))))
  (label 314)
  (parent 131)
  (unrealized (6 0) (6 3))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (6 2)) ((2 1) (4 3)) ((3 1) (1 0))
    ((3 1) (4 2)) ((4 4) (0 0)) ((5 2) (4 0)) ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk (cat pt-3 k "init")) (ch-msg lk-0 (cat pt-5 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-4 old-1)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-6 k1 "wrap")) (send (enc k k1))))
  (label 315)
  (parent 131)
  (unrealized (6 0) (6 3))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (0 0)) ((3 1) (4 2)) ((3 1) (6 0))
    ((4 4) (0 0)) ((5 2) (4 0)) ((6 1) (4 3)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation channel-test (displaced 7 3 make 2)
    (ch-msg lk-0 (cat pt-7 k1 "init")) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-4 k "wrap")) (send (enc k k)))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "wrap"))))
  (label 316)
  (parent 132)
  (unrealized (0 0))
  (comment "12 in cohort - 12 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (deflistener k)
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (0 0)) ((3 1) (4 2))
    ((4 4) (0 0)) ((5 1) (4 0)) ((6 1) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation nonce-test (added-strand make 2) k (5 0)
    (ch-msg lk (cat pt-0 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init"))))
  (label 318)
  (parent 133)
  (seen 222)
  (unrealized (4 3) (5 0))
  (comment "7 in cohort - 5 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (deflistener k)
  (precedes ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (0 0)) ((3 1) (5 0))
    ((4 4) (0 0)) ((5 1) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation nonce-test (displaced 6 3 make 2) k (5 0)
    (ch-msg lk (cat pt-0 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k)))
  (label 319)
  (parent 133)
  (seen 215 847)
  (unrealized (4 3) (5 0))
  (comment "5 in cohort - 2 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (deflistener k)
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (6 2)) ((2 1) (0 0)) ((3 1) (4 2))
    ((4 4) (0 0)) ((5 1) (4 0)) ((6 4) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (5 0)
    (ch-msg lk (cat pt-0 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((recv (hash k)) (recv (hash k1-0)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-5 k1-0 "wrap")) (send (enc k k1-0))))
  (label 320)
  (parent 133)
  (unrealized (4 3) (6 0) (6 3))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (deflistener k)
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((2 1) (0 0)) ((3 1) (5 0)) ((4 4) (0 0))
    ((5 1) (4 0)) ((6 1) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation nonce-test (added-strand make 2) k (5 0)
    (ch-msg lk-0 (cat pt-3 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init"))))
  (label 321)
  (parent 134)
  (seen 847)
  (unrealized (4 3) (5 0))
  (comment "5 in cohort - 4 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (deflistener k)
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (0 0)) ((3 1) (6 2)) ((4 4) (0 0))
    ((5 1) (4 0)) ((6 4) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (5 0)
    (ch-msg lk-0 (cat pt-3 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-5 k1-0 "wrap")) (send (enc k k1-0))))
  (label 322)
  (parent 134)
  (unrealized (4 3) (6 0) (6 3))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (precedes ((1 1) (3 2)) ((2 1) (5 0)) ((3 4) (0 0)) ((4 1) (1 0))
    ((4 2) (3 0)) ((5 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-wrap-at-1 trRl_make-at-0)
  (operation nonce-test (added-strand set-wrap 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-1 k "init")) (ch-msg lk-0 (cat pt k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init")))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt k "wrap"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-4 k "wrap"))))
  (label 326)
  (parent 136)
  (unrealized (0 0))
  (comment "9 in cohort - 9 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "wrap") (k k) (lk lk-0))
  (precedes ((1 1) (3 2)) ((1 1) (5 0)) ((2 1) (0 0)) ((3 4) (0 0))
    ((4 1) (1 0)) ((4 2) (3 0)) ((5 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "wrap" "decrypt") (neq "wrap" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule fact-set-decrypt-neq0 trRl_set-wrap-at-1 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation nonce-test (added-strand set-wrap 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-1 k "init")) (ch-msg lk-0 (cat pt k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init")))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt k "wrap"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt k "wrap")) (stor lk-0 (cat pt-4 k "wrap"))))
  (label 327)
  (parent 136)
  (unrealized (0 0))
  (dead)
  (comment "empty cohort"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "wrap") (k k) (lk lk-0))
  (precedes ((1 1) (3 2)) ((1 1) (5 0)) ((2 1) (0 0)) ((3 4) (0 0))
    ((4 1) (1 0)) ((4 2) (3 0)) ((5 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "wrap" "decrypt") (neq "wrap" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-decrypt-at-1 fact-set-decrypt-neq0 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation nonce-test (added-strand set-decrypt 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-1 k "init")) (ch-msg lk-0 (cat pt k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init")))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt k "wrap"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt k "wrap")) (stor lk-0 (cat pt-4 k "decrypt"))))
  (label 328)
  (parent 136)
  (unrealized (0 0))
  (dead)
  (comment "empty cohort"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (precedes ((1 1) (3 2)) ((2 1) (0 0)) ((3 4) (0 0)) ((4 1) (1 0))
    ((4 1) (5 2)) ((4 2) (3 0)) ((5 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-1 k "init")) (ch-msg lk-0 (cat pt k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init")))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt k "wrap"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-4 k1 "wrap")) (send (enc k k1))))
  (label 329)
  (parent 136)
  (seen 372)
  (unrealized (5 0) (5 3))
  (comment "4 in cohort - 3 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk-0))
  (precedes ((1 1) (3 2)) ((1 1) (5 2)) ((2 1) (0 0)) ((3 4) (0 0))
    ((4 1) (1 0)) ((4 2) (3 0)) ((5 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-1 k "init")) (ch-msg lk-0 (cat pt k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init")))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt k "wrap"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt k "wrap"))
      (load lk-0 (cat pt-4 k1 "wrap")) (send (enc k k1))))
  (label 330)
  (parent 136)
  (seen 375)
  (unrealized (5 0) (5 3))
  (comment "4 in cohort - 3 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (3 2)) ((2 1) (5 2)) ((3 4) (0 0)) ((4 1) (1 0))
    ((4 2) (3 0)) ((5 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-1 k "init")) (ch-msg lk-0 (cat pt k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init")))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt k "wrap"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1))))
  (label 331)
  (parent 136)
  (unrealized (5 0) (5 3))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k1) (lk lk-0))
  (defstrand make 2 (old old-1) (k k1) (lk lk-0))
  (precedes ((1 1) (3 2)) ((1 1) (6 0)) ((2 1) (0 0)) ((3 4) (0 0))
    ((4 1) (1 0)) ((4 2) (3 0)) ((5 1) (3 3)) ((6 1) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule invShearsRule trRl_make-at-0 trRl_make-at-1)
  (operation channel-test (added-strand make 2)
    (ch-msg lk-0 (cat pt-5 k1 "init")) (5 0))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt k "wrap"))
      (load lk-0 (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-5 k1 "init")) (stor lk-0 (cat pt-2 k1 "wrap")))
    ((load lk-0 (cat pt-6 old-1)) (stor lk-0 (cat pt-5 k1 "init"))))
  (label 332)
  (parent 137)
  (seen 152)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k skey)
    (lk locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (3 2)) ((2 1) (5 0)) ((3 4) (0 0))
    ((4 1) (1 0)) ((4 2) (3 0)) ((5 1) (3 3)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule invShearsRule)
  (operation channel-test (displaced 6 2 make 2)
    (ch-msg lk-0 (cat pt-5 k1 "init")) (5 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt-4 k "init")) (stor lk (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt k "wrap"))
      (load lk (cat pt-2 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-3 old-0)) (stor lk (cat pt-4 k "init"))
      (send (hash k)))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-2 k "wrap"))))
  (label 333)
  (parent 137)
  (unrealized (0 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((2 1) (4 2)) ((3 1) (0 0)) ((4 4) (0 0))
    ((5 1) (6 0)) ((5 2) (4 0)) ((6 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-decrypt-at-1 trRl_make-at-0)
  (operation nonce-test (added-strand set-decrypt 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init")) (ch-msg lk-1 (cat pt-5 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-1 (cat pt-4 old-1)) (stor lk-1 (cat pt-5 k "init"))
      (send (hash k)))
    ((load lk-1 (cat pt-5 k "init"))
      (stor lk-1 (cat pt-6 k "decrypt"))))
  (label 334)
  (parent 138)
  (seen 694)
  (unrealized (0 0))
  (comment "12 in cohort - 11 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((2 1) (4 2)) ((3 1) (0 0)) ((4 4) (0 0))
    ((5 1) (6 0)) ((5 2) (4 0)) ((6 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-wrap-at-1 trRl_make-at-0)
  (operation nonce-test (added-strand set-wrap 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init")) (ch-msg lk-1 (cat pt-5 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-1 (cat pt-4 old-1)) (stor lk-1 (cat pt-5 k "init"))
      (send (hash k)))
    ((load lk-1 (cat pt-5 k "init")) (stor lk-1 (cat pt-6 k "wrap"))))
  (label 335)
  (parent 138)
  (seen 696)
  (unrealized (0 0))
  (comment "11 in cohort - 10 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (4 2)) ((3 1) (6 0)) ((4 4) (0 0))
    ((5 2) (4 0)) ((6 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-wrap-at-1 trRl_make-at-0)
  (operation nonce-test (added-strand set-wrap 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init")) (ch-msg lk-1 (cat pt-5 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-1 (cat pt-4 old-1)) (stor lk-1 (cat pt-5 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-6 k "wrap"))))
  (label 336)
  (parent 138)
  (seen 895 905)
  (unrealized (0 0))
  (comment "12 in cohort - 10 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand set-wrap 2 (cur "wrap") (k k) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (4 2)) ((2 1) (6 0)) ((3 1) (0 0))
    ((4 4) (0 0)) ((5 2) (4 0)) ((6 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "wrap" "decrypt") (neq "wrap" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule fact-set-decrypt-neq0 trRl_set-wrap-at-1 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation nonce-test (added-strand set-wrap 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init")) (ch-msg lk-1 (cat pt-5 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-1 (cat pt-4 old-1)) (stor lk-1 (cat pt-5 k "init"))
      (send (hash k)))
    ((load lk (cat pt-1 k "wrap")) (stor lk (cat pt-6 k "wrap"))))
  (label 337)
  (parent 138)
  (unrealized (0 0))
  (dead)
  (comment "empty cohort"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-1))
  (precedes ((1 1) (2 0)) ((2 1) (4 2)) ((3 1) (0 0)) ((4 4) (0 0))
    ((5 1) (6 2)) ((5 2) (4 0)) ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init")) (ch-msg lk-1 (cat pt-5 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-1 (cat pt-4 old-1)) (stor lk-1 (cat pt-5 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk-1 (cat pt-5 k "init"))
      (load lk-1 (cat pt-6 k1 "wrap")) (send (enc k k1))))
  (label 338)
  (parent 138)
  (seen 396)
  (unrealized (6 0) (6 3))
  (comment "5 in cohort - 4 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand set-decrypt 2 (cur "wrap") (k k) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (4 2)) ((2 1) (6 0)) ((3 1) (0 0))
    ((4 4) (0 0)) ((5 2) (4 0)) ((6 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "wrap" "decrypt") (neq "wrap" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-decrypt-at-1 fact-set-decrypt-neq0 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation nonce-test (added-strand set-decrypt 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init")) (ch-msg lk-1 (cat pt-5 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-1 (cat pt-4 old-1)) (stor lk-1 (cat pt-5 k "init"))
      (send (hash k)))
    ((load lk (cat pt-1 k "wrap")) (stor lk (cat pt-6 k "decrypt"))))
  (label 339)
  (parent 138)
  (unrealized (0 0))
  (dead)
  (comment "empty cohort"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (4 2)) ((3 1) (6 2)) ((4 4) (0 0))
    ((5 2) (4 0)) ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init")) (ch-msg lk-1 (cat pt-5 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-1 (cat pt-4 old-1)) (stor lk-1 (cat pt-5 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-6 k1 "wrap")) (send (enc k k1))))
  (label 340)
  (parent 138)
  (unrealized (6 0) (6 3))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (4 2)) ((2 1) (6 2)) ((3 1) (0 0))
    ((4 4) (0 0)) ((5 2) (4 0)) ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init")) (ch-msg lk-1 (cat pt-5 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-1 (cat pt-4 old-1)) (stor lk-1 (cat pt-5 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-6 k1 "wrap")) (send (enc k k1))))
  (label 341)
  (parent 138)
  (seen 400)
  (unrealized (6 0) (6 3))
  (comment "5 in cohort - 4 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (6 2)) ((2 1) (4 2)) ((3 1) (0 0))
    ((4 4) (0 0)) ((5 2) (4 0)) ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init")) (ch-msg lk-1 (cat pt-5 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-1 (cat pt-4 old-1)) (stor lk-1 (cat pt-5 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-6 k1 "wrap")) (send (enc k k1))))
  (label 342)
  (parent 138)
  (seen 404)
  (unrealized (6 0) (6 3))
  (comment "5 in cohort - 4 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (3 0)) ((2 1) (4 2)) ((3 1) (6 0))
    ((4 4) (0 0)) ((5 2) (4 0)) ((6 1) (4 3)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule invShearsRule)
  (operation channel-test (displaced 7 3 make 2)
    (ch-msg lk-1 (cat pt-7 k1 "init")) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-4 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-5 old-1)) (stor lk-0 (cat pt-6 k "init"))
      (send (hash k)))
    ((load lk (cat pt-3 k "init")) (stor lk (cat pt-4 k "wrap"))))
  (label 343)
  (parent 139)
  (unrealized (0 0))
  (comment "10 in cohort - 10 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (precedes ((1 1) (2 0)) ((2 1) (5 0)) ((3 1) (0 0)) ((4 4) (0 0))
    ((5 1) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation nonce-test (displaced 6 2 set-wrap 2) k (5 0)
    (ch-msg lk (cat pt-0 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k)))
  (label 345)
  (parent 140)
  (seen 406)
  (unrealized (4 3) (5 0))
  (comment "4 in cohort - 3 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (4 2)) ((3 1) (0 0))
    ((4 4) (0 0)) ((5 1) (4 0)) ((6 1) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation nonce-test (added-strand make 2) k (5 0)
    (ch-msg lk (cat pt-0 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init"))))
  (label 346)
  (parent 140)
  (seen 950)
  (unrealized (4 3) (5 0))
  (comment "6 in cohort - 4 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (6 2)) ((2 1) (4 2)) ((3 1) (0 0))
    ((4 4) (0 0)) ((5 1) (4 0)) ((6 4) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap")
    (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (5 0)
    (ch-msg lk (cat pt-0 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((recv (hash k)) (recv (hash k1-0)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-5 k1-0 "wrap")) (send (enc k k1-0))))
  (label 347)
  (parent 140)
  (seen 414)
  (unrealized (4 3) (6 0) (6 3))
  (comment "4 in cohort - 3 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "wrap") (k k) (lk lk-0))
  (precedes ((1 1) (3 3)) ((1 1) (5 0)) ((2 1) (0 0)) ((3 4) (0 0))
    ((4 1) (1 0)) ((4 2) (3 0)) ((5 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "wrap" "decrypt") (neq "wrap" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule fact-set-decrypt-neq0 trRl_set-wrap-at-1 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation nonce-test (added-strand set-wrap 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-1 k "init")) (ch-msg lk-0 (cat pt k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init")))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt k "wrap")) (stor lk-0 (cat pt-4 k "wrap"))))
  (label 351)
  (parent 142)
  (unrealized (0 0))
  (dead)
  (comment "empty cohort"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "wrap") (k k) (lk lk-0))
  (precedes ((1 1) (3 3)) ((1 1) (5 0)) ((2 1) (0 0)) ((3 4) (0 0))
    ((4 1) (1 0)) ((4 2) (3 0)) ((5 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "wrap" "decrypt") (neq "wrap" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-decrypt-at-1 fact-set-decrypt-neq0 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation nonce-test (added-strand set-decrypt 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-1 k "init")) (ch-msg lk-0 (cat pt k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init")))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt k "wrap")) (stor lk-0 (cat pt-4 k "decrypt"))))
  (label 352)
  (parent 142)
  (unrealized (0 0))
  (dead)
  (comment "empty cohort"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (precedes ((1 1) (3 3)) ((2 1) (0 0)) ((3 4) (0 0)) ((4 1) (1 0))
    ((4 1) (5 2)) ((4 2) (3 0)) ((5 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-1 k "init")) (ch-msg lk-0 (cat pt k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init")))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-4 k1 "wrap")) (send (enc k k1))))
  (label 353)
  (parent 142)
  (seen 431)
  (unrealized (5 0) (5 3))
  (comment "4 in cohort - 3 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk-0))
  (precedes ((1 1) (3 3)) ((1 1) (5 2)) ((2 1) (0 0)) ((3 4) (0 0))
    ((4 1) (1 0)) ((4 2) (3 0)) ((5 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-1 k "init")) (ch-msg lk-0 (cat pt k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init")))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt k "wrap"))
      (load lk-0 (cat pt-4 k1 "wrap")) (send (enc k k1))))
  (label 354)
  (parent 142)
  (seen 434)
  (unrealized (5 0) (5 3))
  (comment "4 in cohort - 3 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (3 3)) ((2 1) (5 2)) ((3 4) (0 0)) ((4 1) (1 0))
    ((4 2) (3 0)) ((5 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-1 k "init")) (ch-msg lk-0 (cat pt k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init")))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1))))
  (label 355)
  (parent 142)
  (unrealized (5 0) (5 3))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k1) (lk lk-0))
  (defstrand make 2 (old old-1) (k k1) (lk lk-0))
  (precedes ((1 1) (6 0)) ((2 1) (0 0)) ((3 4) (0 0)) ((4 1) (1 0))
    ((4 2) (3 0)) ((5 1) (3 3)) ((6 1) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule invShearsRule trRl_make-at-0 trRl_make-at-1)
  (operation channel-test (added-strand make 2)
    (ch-msg lk-0 (cat pt-5 k1 "init")) (5 0))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-4 k "init"))
      (load lk-0 (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-5 k1 "init")) (stor lk-0 (cat pt-2 k1 "wrap")))
    ((load lk-0 (cat pt-6 old-1)) (stor lk-0 (cat pt-5 k1 "init"))))
  (label 356)
  (parent 143)
  (seen 766)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((1 1) (4 2)) ((2 1) (4 3)) ((3 1) (0 0))
    ((4 4) (0 0)) ((5 1) (6 0)) ((5 2) (4 0)) ((6 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-decrypt-at-1 trRl_make-at-0)
  (operation nonce-test (added-strand set-decrypt 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init")) (ch-msg lk-1 (cat pt-5 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-1 (cat pt-4 old-1)) (stor lk-1 (cat pt-5 k "init"))
      (send (hash k)))
    ((load lk-1 (cat pt-5 k "init"))
      (stor lk-1 (cat pt-6 k "decrypt"))))
  (label 357)
  (parent 144)
  (seen 724 836)
  (unrealized (0 0))
  (comment "12 in cohort - 10 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((1 1) (4 2)) ((2 1) (4 3)) ((3 1) (0 0))
    ((4 4) (0 0)) ((5 1) (6 0)) ((5 2) (4 0)) ((6 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-wrap-at-1 trRl_make-at-0)
  (operation nonce-test (added-strand set-wrap 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init")) (ch-msg lk-1 (cat pt-5 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-1 (cat pt-4 old-1)) (stor lk-1 (cat pt-5 k "init"))
      (send (hash k)))
    ((load lk-1 (cat pt-5 k "init")) (stor lk-1 (cat pt-6 k "wrap"))))
  (label 358)
  (parent 144)
  (seen 726 839)
  (unrealized (0 0))
  (comment "11 in cohort - 9 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand set-wrap 2 (cur "wrap") (k k) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (4 2)) ((2 1) (4 3)) ((2 1) (6 0))
    ((3 1) (0 0)) ((4 4) (0 0)) ((5 2) (4 0)) ((6 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "wrap" "decrypt") (neq "wrap" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule fact-set-decrypt-neq0 trRl_set-wrap-at-1 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation nonce-test (added-strand set-wrap 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init")) (ch-msg lk-1 (cat pt-5 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-1 (cat pt-4 old-1)) (stor lk-1 (cat pt-5 k "init"))
      (send (hash k)))
    ((load lk (cat pt-1 k "wrap")) (stor lk (cat pt-6 k "wrap"))))
  (label 359)
  (parent 144)
  (unrealized (0 0))
  (dead)
  (comment "empty cohort"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-1))
  (precedes ((1 1) (2 0)) ((1 1) (4 2)) ((2 1) (4 3)) ((3 1) (0 0))
    ((4 4) (0 0)) ((5 1) (6 2)) ((5 2) (4 0)) ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init")) (ch-msg lk-1 (cat pt-5 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-1 (cat pt-4 old-1)) (stor lk-1 (cat pt-5 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk-1 (cat pt-5 k "init"))
      (load lk-1 (cat pt-6 k1 "wrap")) (send (enc k k1))))
  (label 360)
  (parent 144)
  (seen 455)
  (unrealized (6 0) (6 3))
  (comment "5 in cohort - 4 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand set-decrypt 2 (cur "wrap") (k k) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (4 2)) ((2 1) (4 3)) ((2 1) (6 0))
    ((3 1) (0 0)) ((4 4) (0 0)) ((5 2) (4 0)) ((6 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "wrap" "decrypt") (neq "wrap" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-decrypt-at-1 fact-set-decrypt-neq0 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation nonce-test (added-strand set-decrypt 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init")) (ch-msg lk-1 (cat pt-5 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-1 (cat pt-4 old-1)) (stor lk-1 (cat pt-5 k "init"))
      (send (hash k)))
    ((load lk (cat pt-1 k "wrap")) (stor lk (cat pt-6 k "decrypt"))))
  (label 361)
  (parent 144)
  (unrealized (0 0))
  (dead)
  (comment "empty cohort"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (precedes ((1 1) (2 0)) ((1 1) (4 2)) ((2 1) (4 3)) ((3 1) (6 2))
    ((4 4) (0 0)) ((5 2) (4 0)) ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init")) (ch-msg lk-1 (cat pt-5 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-1 (cat pt-4 old-1)) (stor lk-1 (cat pt-5 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-6 k1 "wrap")) (send (enc k k1))))
  (label 362)
  (parent 144)
  (unrealized (6 0) (6 3))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (4 2)) ((2 1) (4 3)) ((2 1) (6 2))
    ((3 1) (0 0)) ((4 4) (0 0)) ((5 2) (4 0)) ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init")) (ch-msg lk-1 (cat pt-5 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-1 (cat pt-4 old-1)) (stor lk-1 (cat pt-5 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-6 k1 "wrap")) (send (enc k k1))))
  (label 363)
  (parent 144)
  (seen 459)
  (unrealized (6 0) (6 3))
  (comment "5 in cohort - 4 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (4 2)) ((1 1) (6 2)) ((2 1) (4 3))
    ((3 1) (0 0)) ((4 4) (0 0)) ((5 2) (4 0)) ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init")) (ch-msg lk-1 (cat pt-5 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-1 (cat pt-4 old-1)) (stor lk-1 (cat pt-5 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-6 k1 "wrap")) (send (enc k k1))))
  (label 364)
  (parent 144)
  (seen 463)
  (unrealized (6 0) (6 3))
  (comment "5 in cohort - 4 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (0 0)) ((3 1) (0 0))
    ((4 4) (0 0)) ((5 1) (4 0)) ((6 1) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation nonce-test (added-strand make 2) k (5 0)
    (ch-msg lk (cat pt-0 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init"))))
  (label 366)
  (parent 146)
  (unrealized (4 3) (5 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (6 2)) ((2 1) (0 0)) ((3 1) (0 0))
    ((4 4) (0 0)) ((5 1) (4 0)) ((6 4) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (5 0)
    (ch-msg lk (cat pt-0 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((recv (hash k)) (recv (hash k1-0)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-5 k1-0 "wrap")) (send (enc k k1-0))))
  (label 367)
  (parent 146)
  (seen 470)
  (unrealized (4 3) (6 0) (6 3))
  (comment "4 in cohort - 3 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old mesg) (pt pt-0 pt-1 pt-2 pval) (k k1 skey) (lk locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (2 2)) ((2 4) (0 0)) ((3 1) (1 0)) ((3 2) (2 0))
    ((3 2) (4 0)) ((4 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 5 3 make 3) (hash k) (4 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt k "wrap"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1))))
  (label 371)
  (parent 150)
  (seen 1013 1014)
  (unrealized (4 3))
  (comment "7 in cohort - 2 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (precedes ((1 1) (2 2)) ((2 4) (0 0)) ((3 1) (1 0)) ((3 1) (4 2))
    ((3 2) (2 0)) ((4 4) (0 0)) ((5 2) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation encryption-test (added-strand make 3) (hash k) (4 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt k "wrap"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init"))
      (send (hash k))))
  (label 372)
  (parent 150)
  (seen 1015 1016)
  (unrealized (4 3))
  (comment "7 in cohort - 2 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old mesg) (pt pt-0 pt-1 pt-2 pval) (k k1 skey) (lk locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (precedes ((1 1) (2 2)) ((2 4) (0 0)) ((3 1) (1 0)) ((3 1) (4 2))
    ((3 2) (2 0)) ((4 4) (0 0)) ((5 1) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (added-listener k) (hash k) (4 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt k "wrap"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k)))
  (label 373)
  (parent 150)
  (unrealized (4 3) (5 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old mesg) (pt pt-0 pt-1 pt-2 pval) (k k1 skey) (lk locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (2 2)) ((1 1) (4 2)) ((2 4) (0 0)) ((3 1) (1 0))
    ((3 2) (2 0)) ((3 2) (4 0)) ((4 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 5 3 make 3) (hash k) (4 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt k "wrap"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt k "wrap"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1))))
  (label 374)
  (parent 151)
  (seen 66)
  (unrealized (4 3))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (precedes ((1 1) (2 2)) ((1 1) (4 2)) ((2 4) (0 0)) ((3 1) (1 0))
    ((3 2) (2 0)) ((4 4) (0 0)) ((5 2) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation encryption-test (added-strand make 3) (hash k) (4 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt k "wrap"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt k "wrap"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init"))
      (send (hash k))))
  (label 375)
  (parent 151)
  (seen 1019 1020)
  (unrealized (4 3))
  (comment "4 in cohort - 2 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old mesg) (pt pt-0 pt-1 pt-2 pval) (k k1 skey) (lk locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (precedes ((1 1) (2 2)) ((1 1) (4 2)) ((2 4) (0 0)) ((3 1) (1 0))
    ((3 2) (2 0)) ((4 4) (0 0)) ((5 1) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (added-listener k) (hash k) (4 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt k "wrap"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt k "wrap"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k)))
  (label 376)
  (parent 151)
  (unrealized (4 3) (5 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "decrypt") (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (3 2)) ((3 4) (0 0)) ((4 1) (5 0))
    ((4 2) (3 0)) ((5 1) (6 0)) ((6 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k "wrap"))
  (facts (neq "decrypt" "decrypt") (neq "decrypt" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-decrypt-at-1 fact-set-decrypt-neq0 gen-st-set-wrap-0
    fact-set-wrap-neq0 trRl_make-at-0)
  (operation nonce-test (added-strand set-decrypt 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init"))
    (ch-msg lk-0 (cat pt-4 k "decrypt")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((load lk-0 (cat pt-4 k "decrypt"))
      (stor lk-0 (cat pt-5 k "decrypt"))))
  (label 377)
  (parent 153)
  (unrealized (0 0))
  (dead)
  (comment "empty cohort"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "wrap") (k k) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (3 2)) ((2 1) (6 0)) ((3 4) (0 0))
    ((4 1) (5 0)) ((4 2) (3 0)) ((5 1) (0 0)) ((6 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "wrap" "decrypt") (neq "wrap" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule fact-set-decrypt-neq0 trRl_set-wrap-at-1 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation nonce-test (added-strand set-wrap 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init"))
    (ch-msg lk-0 (cat pt-4 k "decrypt")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((load lk (cat pt-1 k "wrap")) (stor lk (cat pt-5 k "wrap"))))
  (label 378)
  (parent 153)
  (unrealized (0 0))
  (dead)
  (comment "empty cohort"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "decrypt") (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (3 2)) ((3 4) (0 0)) ((4 1) (5 0))
    ((4 2) (3 0)) ((5 1) (6 0)) ((6 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k "wrap"))
  (facts (neq "decrypt" "decrypt") (neq "decrypt" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule fact-set-decrypt-neq0 trRl_set-wrap-at-1 gen-st-set-wrap-0
    fact-set-wrap-neq0 trRl_make-at-0)
  (operation nonce-test (added-strand set-wrap 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init"))
    (ch-msg lk-0 (cat pt-4 k "decrypt")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((load lk-0 (cat pt-4 k "decrypt"))
      (stor lk-0 (cat pt-5 k "wrap"))))
  (label 379)
  (parent 153)
  (unrealized (0 0))
  (dead)
  (comment "empty cohort"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (3 2)) ((3 4) (0 0)) ((4 1) (5 0))
    ((4 2) (3 0)) ((5 1) (6 2)) ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k "wrap")
    (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0 gen-st-wrap-1)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init"))
    (ch-msg lk-0 (cat pt-4 k "decrypt")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-4 k "decrypt"))
      (load lk-0 (cat pt-5 k1 "wrap")) (send (enc k k1))))
  (label 380)
  (parent 153)
  (unrealized (6 0) (6 3))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "wrap") (k k) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (3 2)) ((2 1) (6 0)) ((3 4) (0 0))
    ((4 1) (5 0)) ((4 2) (3 0)) ((5 1) (0 0)) ((6 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "wrap" "decrypt") (neq "wrap" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-decrypt-at-1 fact-set-decrypt-neq0 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation nonce-test (added-strand set-decrypt 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init"))
    (ch-msg lk-0 (cat pt-4 k "decrypt")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((load lk (cat pt-1 k "wrap")) (stor lk (cat pt-5 k "decrypt"))))
  (label 381)
  (parent 153)
  (unrealized (0 0))
  (dead)
  (comment "empty cohort"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (3 2)) ((2 1) (6 2)) ((3 4) (0 0))
    ((4 1) (5 0)) ((4 2) (3 0)) ((5 1) (0 0)) ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init"))
    (ch-msg lk-0 (cat pt-4 k "decrypt")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-5 k1 "wrap")) (send (enc k k1))))
  (label 382)
  (parent 153)
  (unrealized (6 0) (6 3))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (3 2)) ((3 4) (0 0)) ((4 1) (5 0))
    ((4 1) (6 2)) ((4 2) (3 0)) ((5 1) (0 0)) ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init"))
    (ch-msg lk-0 (cat pt-4 k "decrypt")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-5 k1 "wrap")) (send (enc k k1))))
  (label 383)
  (parent 153)
  (unrealized (6 0) (6 3))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (6 2)) ((2 1) (3 2)) ((3 4) (0 0))
    ((4 1) (5 0)) ((4 2) (3 0)) ((5 1) (0 0)) ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init"))
    (ch-msg lk-0 (cat pt-4 k "decrypt")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-5 k1 "wrap")) (send (enc k k1))))
  (label 384)
  (parent 153)
  (unrealized (6 0) (6 3))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand decrypt 4 (x k) (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (3 2)) ((3 4) (0 0)) ((4 1) (5 0))
    ((4 2) (3 0)) ((5 1) (6 2)) ((6 3) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-decrypt-0)
  (operation nonce-test (added-strand decrypt 4) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init"))
    (ch-msg lk-0 (cat pt-4 k "decrypt")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((recv (enc k k)) (recv (hash k)) (load lk-0 (cat pt-4 k "decrypt"))
      (send k)))
  (label 385)
  (parent 153)
  (unrealized (6 0) (6 1))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "wrap") (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (3 2)) ((3 4) (0 0)) ((4 1) (5 0))
    ((4 2) (3 0)) ((5 1) (6 0)) ((6 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "wrap" "decrypt") (neq "wrap" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-decrypt-at-1 fact-set-decrypt-neq0 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation nonce-test (added-strand set-decrypt 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init")) (ch-msg lk-0 (cat pt-4 k "wrap")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "wrap")))
    ((load lk-0 (cat pt-4 k "wrap"))
      (stor lk-0 (cat pt-5 k "decrypt"))))
  (label 386)
  (parent 155)
  (unrealized (0 0))
  (dead)
  (comment "empty cohort"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "wrap") (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (3 2)) ((3 4) (0 0)) ((4 1) (5 0))
    ((4 2) (3 0)) ((5 1) (6 0)) ((6 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "wrap" "decrypt") (neq "wrap" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule fact-set-decrypt-neq0 trRl_set-wrap-at-1 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation nonce-test (added-strand set-wrap 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init")) (ch-msg lk-0 (cat pt-4 k "wrap")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "wrap")))
    ((load lk-0 (cat pt-4 k "wrap")) (stor lk-0 (cat pt-5 k "wrap"))))
  (label 387)
  (parent 155)
  (unrealized (0 0))
  (dead)
  (comment "empty cohort"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "wrap") (k k) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (3 2)) ((2 1) (6 0)) ((3 4) (0 0))
    ((4 1) (5 0)) ((4 2) (3 0)) ((5 1) (0 0)) ((6 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "wrap" "decrypt") (neq "wrap" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule fact-set-decrypt-neq0 trRl_set-wrap-at-1 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation nonce-test (added-strand set-wrap 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init")) (ch-msg lk-0 (cat pt-4 k "wrap")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "wrap")))
    ((load lk (cat pt-1 k "wrap")) (stor lk (cat pt-5 k "wrap"))))
  (label 388)
  (parent 155)
  (unrealized (0 0))
  (dead)
  (comment "empty cohort"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (3 2)) ((3 4) (0 0)) ((4 1) (5 0))
    ((4 2) (3 0)) ((5 1) (6 2)) ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init")) (ch-msg lk-0 (cat pt-4 k "wrap")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-4 k "wrap"))
      (load lk-0 (cat pt-5 k1 "wrap")) (send (enc k k1))))
  (label 389)
  (parent 155)
  (unrealized (6 0) (6 3))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "wrap") (k k) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (3 2)) ((2 1) (6 0)) ((3 4) (0 0))
    ((4 1) (5 0)) ((4 2) (3 0)) ((5 1) (0 0)) ((6 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "wrap" "decrypt") (neq "wrap" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-decrypt-at-1 fact-set-decrypt-neq0 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation nonce-test (added-strand set-decrypt 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init")) (ch-msg lk-0 (cat pt-4 k "wrap")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "wrap")))
    ((load lk (cat pt-1 k "wrap")) (stor lk (cat pt-5 k "decrypt"))))
  (label 390)
  (parent 155)
  (unrealized (0 0))
  (dead)
  (comment "empty cohort"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (3 2)) ((3 4) (0 0)) ((4 1) (5 0))
    ((4 1) (6 2)) ((4 2) (3 0)) ((5 1) (0 0)) ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init")) (ch-msg lk-0 (cat pt-4 k "wrap")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-5 k1 "wrap")) (send (enc k k1))))
  (label 391)
  (parent 155)
  (unrealized (6 0) (6 3))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (3 2)) ((2 1) (6 2)) ((3 4) (0 0))
    ((4 1) (5 0)) ((4 2) (3 0)) ((5 1) (0 0)) ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init")) (ch-msg lk-0 (cat pt-4 k "wrap")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-5 k1 "wrap")) (send (enc k k1))))
  (label 392)
  (parent 155)
  (unrealized (6 0) (6 3))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (6 2)) ((2 1) (3 2)) ((3 4) (0 0))
    ((4 1) (5 0)) ((4 2) (3 0)) ((5 1) (0 0)) ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init")) (ch-msg lk-0 (cat pt-4 k "wrap")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-5 k1 "wrap")) (send (enc k k1))))
  (label 393)
  (parent 155)
  (unrealized (6 0) (6 3))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (3 2)) ((3 4) (0 0)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 6 4 make 3) (hash k) (5 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-4 k1 "wrap")) (send (enc k k1))))
  (label 394)
  (parent 157)
  (seen 1013 1058 1059 1060)
  (unrealized (5 3))
  (comment "16 in cohort - 3 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (precedes ((1 1) (2 2)) ((2 4) (0 0)) ((3 1) (4 2)) ((3 2) (2 0))
    ((4 4) (0 0)) ((5 1) (1 0)) ((5 2) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 1 6 make 3) (hash k) (5 0))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt k "wrap"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init"))
      (send (hash k))))
  (label 395)
  (parent 157)
  (seen 1013 1061 1062)
  (unrealized (4 3))
  (comment "13 in cohort - 2 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((2 1) (3 2)) ((3 4) (0 0)) ((4 1) (5 2))
    ((4 2) (3 0)) ((5 4) (0 0)) ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation encryption-test (added-strand make 3) (hash k) (5 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init"))
      (send (hash k))))
  (label 396)
  (parent 157)
  (seen 1015 1063 1064 1065)
  (unrealized (5 3))
  (comment "16 in cohort - 3 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (deflistener k)
  (precedes ((1 1) (2 0)) ((2 1) (3 2)) ((3 4) (0 0)) ((4 1) (5 2))
    ((4 2) (3 0)) ((5 4) (0 0)) ((6 1) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (added-listener k) (hash k) (5 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k)))
  (label 397)
  (parent 157)
  (unrealized (5 3) (6 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (3 2)) ((2 1) (5 2)) ((3 4) (0 0))
    ((4 2) (3 0)) ((4 2) (5 0)) ((5 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 6 4 make 3) (hash k) (5 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1))))
  (label 398)
  (parent 158)
  (seen 68)
  (unrealized (5 3))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (precedes ((1 1) (2 2)) ((1 1) (4 2)) ((2 4) (0 0)) ((3 2) (2 0))
    ((4 4) (0 0)) ((5 1) (1 0)) ((5 2) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 1 6 make 3) (hash k) (5 0))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt k "wrap"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt k "wrap"))
      (load lk-0 (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init"))
      (send (hash k))))
  (label 399)
  (parent 158)
  (seen 1019 1069)
  (unrealized (4 3))
  (comment "4 in cohort - 1 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((2 1) (3 2)) ((2 1) (5 2)) ((3 4) (0 0))
    ((4 2) (3 0)) ((5 4) (0 0)) ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation encryption-test (added-strand make 3) (hash k) (5 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init"))
      (send (hash k))))
  (label 400)
  (parent 158)
  (seen 1070)
  (unrealized (5 3))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (precedes ((1 1) (2 0)) ((2 1) (3 2)) ((2 1) (5 2)) ((3 4) (0 0))
    ((4 2) (3 0)) ((5 4) (0 0)) ((6 1) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (added-listener k) (hash k) (5 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k)))
  (label 401)
  (parent 158)
  (unrealized (5 3) (6 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (5 2)) ((2 1) (3 2)) ((3 4) (0 0))
    ((4 2) (3 0)) ((4 2) (5 0)) ((5 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 6 4 make 3) (hash k) (5 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1))))
  (label 402)
  (parent 159)
  (seen 1074 1075)
  (unrealized (5 3))
  (comment "7 in cohort - 2 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (precedes ((1 1) (2 2)) ((2 4) (0 0)) ((3 2) (2 0)) ((4 4) (0 0))
    ((5 1) (1 0)) ((5 2) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 1 6 make 3) (hash k) (5 0))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt k "wrap"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-4 k "init"))
      (load lk-0 (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init"))
      (send (hash k))))
  (label 403)
  (parent 159)
  (seen 1076 1077)
  (unrealized (4 3))
  (comment "7 in cohort - 2 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((1 1) (5 2)) ((2 1) (3 2)) ((3 4) (0 0))
    ((4 2) (3 0)) ((5 4) (0 0)) ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation encryption-test (added-strand make 3) (hash k) (5 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init"))
      (send (hash k))))
  (label 404)
  (parent 159)
  (seen 1078 1079)
  (unrealized (5 3))
  (comment "7 in cohort - 2 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (precedes ((1 1) (2 0)) ((1 1) (5 2)) ((2 1) (3 2)) ((3 4) (0 0))
    ((4 2) (3 0)) ((5 4) (0 0)) ((6 1) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (added-listener k) (hash k) (5 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k)))
  (label 405)
  (parent 159)
  (unrealized (5 3) (6 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (4 0)) ((3 4) (0 0)) ((4 1) (3 0))
    ((5 1) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation nonce-test (added-strand make 2) k (4 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init"))))
  (label 406)
  (parent 161)
  (unrealized (3 3) (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old mesg) (pt pt-0 pt-1 pt-2 pt-3 pval) (k k1 k1-0 skey)
    (lk locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1-0) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (5 2)) ((3 4) (0 0)) ((4 1) (3 0))
    ((5 4) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap")
    (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (4 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((recv (hash k)) (recv (hash k1-0)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-3 k1-0 "wrap")) (send (enc k k1-0))))
  (label 407)
  (parent 161)
  (unrealized (3 3) (5 0) (5 3))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old mesg) (pt pt-0 pt-1 pt-2 pt-3 pval) (k k1 k1-0 skey)
    (lk locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (5 2)) ((2 1) (4 0)) ((3 4) (0 0))
    ((4 1) (3 0)) ((5 4) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap")
    (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (4 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((recv (hash k)) (recv (hash k1-0)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-3 k1-0 "wrap")) (send (enc k k1-0))))
  (label 408)
  (parent 161)
  (unrealized (3 3) (5 0) (5 3))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (3 2)) ((3 4) (0 0))
    ((4 1) (3 0)) ((5 1) (6 0)) ((6 1) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-wrap-at-1 trRl_make-at-0)
  (operation nonce-test (added-strand set-wrap 2) k (4 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk-0 (cat pt-4 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt-5 k "wrap"))))
  (label 409)
  (parent 162)
  (seen 1084)
  (unrealized (3 3) (4 0))
  (comment "8 in cohort - 6 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (3 2)) ((3 4) (0 0))
    ((4 1) (3 0)) ((5 1) (6 0)) ((6 1) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-decrypt-at-1 trRl_make-at-0)
  (operation nonce-test (added-strand set-decrypt 2) k (4 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk-0 (cat pt-4 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((load lk-0 (cat pt-4 k "init"))
      (stor lk-0 (cat pt-5 k "decrypt"))))
  (label 410)
  (parent 162)
  (seen 1083)
  (unrealized (3 3) (4 0))
  (comment "8 in cohort - 6 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk-0))
  (precedes ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (3 2)) ((3 4) (0 0))
    ((4 1) (3 0)) ((5 1) (6 2)) ((6 4) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap")
    (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (4 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk-0 (cat pt-4 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-0 (cat pt-4 k "init"))
      (load lk-0 (cat pt-5 k1-0 "wrap")) (send (enc k k1-0))))
  (label 411)
  (parent 162)
  (seen 427)
  (unrealized (3 3) (6 0) (6 3))
  (comment "4 in cohort - 3 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (6 2)) ((2 1) (3 2)) ((3 4) (0 0))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 4) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap")
    (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (4 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk-0 (cat pt-4 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-5 k1-0 "wrap")) (send (enc k k1-0))))
  (label 412)
  (parent 162)
  (seen 414)
  (unrealized (3 3) (6 0) (6 3))
  (comment "4 in cohort - 3 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old mesg) (pt pt-0 pt-1 pt-2 pt-3 pval) (k k1 k1-0 skey)
    (lk locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk))
  (defstrand make 3 (old old) (k k) (lk lk))
  (precedes ((1 1) (2 2)) ((2 4) (0 0)) ((3 1) (2 0)) ((4 4) (3 0))
    ((5 1) (1 0)) ((5 2) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap")
    (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 1 6 make 3) (hash k) (5 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt-3 k "init")) (stor lk (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt k "wrap"))
      (load lk (cat pt-0 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((recv (hash k)) (recv (hash k1-0)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k1-0 "wrap")) (send (enc k k1-0)))
    ((load lk (cat pt-2 old)) (stor lk (cat pt-3 k "init"))
      (send (hash k))))
  (label 413)
  (parent 163)
  (seen 1112)
  (unrealized (2 3) (4 3))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((1 1) (5 2)) ((2 1) (3 2)) ((3 4) (0 0))
    ((4 1) (3 0)) ((5 4) (4 0)) ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap")
    (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation encryption-test (added-strand make 3) (hash k) (5 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((recv (hash k)) (recv (hash k1-0)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-3 k1-0 "wrap")) (send (enc k k1-0)))
    ((load lk-0 (cat pt-4 old-0)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k))))
  (label 414)
  (parent 163)
  (seen 1113)
  (unrealized (3 3) (5 3))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old mesg) (pt pt-0 pt-1 pt-2 pt-3 pval) (k k1 k1-0 skey)
    (lk locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk))
  (deflistener k)
  (precedes ((1 1) (2 0)) ((1 1) (5 2)) ((2 1) (3 2)) ((3 4) (0 0))
    ((4 1) (3 0)) ((5 4) (4 0)) ((6 1) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap")
    (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (added-listener k) (hash k) (5 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((recv (hash k)) (recv (hash k1-0)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-3 k1-0 "wrap")) (send (enc k k1-0)))
    ((recv k) (send k)))
  (label 415)
  (parent 163)
  (unrealized (3 3) (5 3) (6 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (3 2)) ((3 4) (0 0)) ((4 1) (3 0))
    ((5 4) (4 0)) ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap")
    (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 5 7 make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-0 (cat pt-5 k "init"))
      (load lk-0 (cat pt-3 k1-0 "wrap")) (send (enc k k1-0)))
    ((load lk-0 (cat pt-4 old-0)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k))))
  (label 426)
  (parent 166)
  (seen 1112 1116 1117)
  (unrealized (3 3) (5 3))
  (comment "14 in cohort - 2 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk-0))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (precedes ((1 1) (2 2)) ((2 4) (0 0)) ((3 1) (2 0)) ((4 1) (5 2))
    ((5 4) (3 0)) ((6 1) (1 0)) ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap")
    (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 1 7 make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-5 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt k "wrap"))
      (load lk-0 (cat pt-0 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk (cat pt-1 old)) (stor lk (cat pt-2 k "init")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk (cat pt-2 k "init"))
      (load lk (cat pt-3 k1-0 "wrap")) (send (enc k k1-0)))
    ((load lk-0 (cat pt-4 old-0)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k))))
  (label 427)
  (parent 166)
  (seen 1112 1118)
  (unrealized (2 3) (5 3))
  (comment "11 in cohort - 1 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old mesg) (pt pt-0 pt-1 pt-2 pval) (k k1 skey) (lk locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (2 3)) ((2 4) (0 0)) ((3 1) (1 0)) ((3 2) (2 0))
    ((3 2) (4 0)) ((4 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 5 3 make 3) (hash k) (4 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1))))
  (label 430)
  (parent 169)
  (seen 72)
  (unrealized (4 3))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (precedes ((1 1) (2 3)) ((2 4) (0 0)) ((3 1) (1 0)) ((3 1) (4 2))
    ((3 2) (2 0)) ((4 4) (0 0)) ((5 2) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation encryption-test (added-strand make 3) (hash k) (4 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init"))
      (send (hash k))))
  (label 431)
  (parent 169)
  (seen 1119 1120)
  (unrealized (4 3))
  (comment "7 in cohort - 2 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old mesg) (pt pt-0 pt-1 pt-2 pval) (k k1 skey) (lk locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (precedes ((1 1) (2 3)) ((2 4) (0 0)) ((3 1) (1 0)) ((3 1) (4 2))
    ((3 2) (2 0)) ((4 4) (0 0)) ((5 1) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (added-listener k) (hash k) (4 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k)))
  (label 432)
  (parent 169)
  (unrealized (4 3) (5 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old mesg) (pt pt-0 pt-1 pt-2 pval) (k k1 skey) (lk locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (2 3)) ((1 1) (4 2)) ((2 4) (0 0)) ((3 1) (1 0))
    ((3 2) (2 0)) ((3 2) (4 0)) ((4 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 5 3 make 3) (hash k) (4 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt k "wrap"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1))))
  (label 433)
  (parent 170)
  (seen 1013 1123)
  (unrealized (4 3))
  (comment "4 in cohort - 1 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (precedes ((1 1) (2 3)) ((1 1) (4 2)) ((2 4) (0 0)) ((3 1) (1 0))
    ((3 2) (2 0)) ((4 4) (0 0)) ((5 2) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation encryption-test (added-strand make 3) (hash k) (4 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt k "wrap"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init"))
      (send (hash k))))
  (label 434)
  (parent 170)
  (seen 1076 1124)
  (unrealized (4 3))
  (comment "4 in cohort - 1 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old mesg) (pt pt-0 pt-1 pt-2 pval) (k k1 skey) (lk locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (precedes ((1 1) (2 3)) ((1 1) (4 2)) ((2 4) (0 0)) ((3 1) (1 0))
    ((3 2) (2 0)) ((4 4) (0 0)) ((5 1) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (added-listener k) (hash k) (4 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt k "wrap"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k)))
  (label 435)
  (parent 170)
  (unrealized (4 3) (5 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "decrypt") (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((1 1) (3 2)) ((2 1) (3 3)) ((3 4) (0 0))
    ((4 1) (5 0)) ((4 2) (3 0)) ((5 1) (6 0)) ((6 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k "wrap"))
  (facts (neq "decrypt" "decrypt") (neq "decrypt" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-decrypt-at-1 fact-set-decrypt-neq0 gen-st-set-wrap-0
    fact-set-wrap-neq0 trRl_make-at-0)
  (operation nonce-test (added-strand set-decrypt 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init"))
    (ch-msg lk-0 (cat pt-4 k "decrypt")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((load lk-0 (cat pt-4 k "decrypt"))
      (stor lk-0 (cat pt-5 k "decrypt"))))
  (label 436)
  (parent 172)
  (unrealized (0 0))
  (dead)
  (comment "empty cohort"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "wrap") (k k) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (3 2)) ((2 1) (3 3)) ((2 1) (6 0))
    ((3 4) (0 0)) ((4 1) (5 0)) ((4 2) (3 0)) ((5 1) (0 0))
    ((6 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "wrap" "decrypt") (neq "wrap" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule fact-set-decrypt-neq0 trRl_set-wrap-at-1 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation nonce-test (added-strand set-wrap 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init"))
    (ch-msg lk-0 (cat pt-4 k "decrypt")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((load lk (cat pt-1 k "wrap")) (stor lk (cat pt-5 k "wrap"))))
  (label 437)
  (parent 172)
  (unrealized (0 0))
  (dead)
  (comment "empty cohort"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "decrypt") (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((1 1) (3 2)) ((2 1) (3 3)) ((3 4) (0 0))
    ((4 1) (5 0)) ((4 2) (3 0)) ((5 1) (6 0)) ((6 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k "wrap"))
  (facts (neq "decrypt" "decrypt") (neq "decrypt" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule fact-set-decrypt-neq0 trRl_set-wrap-at-1 gen-st-set-wrap-0
    fact-set-wrap-neq0 trRl_make-at-0)
  (operation nonce-test (added-strand set-wrap 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init"))
    (ch-msg lk-0 (cat pt-4 k "decrypt")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((load lk-0 (cat pt-4 k "decrypt"))
      (stor lk-0 (cat pt-5 k "wrap"))))
  (label 438)
  (parent 172)
  (unrealized (0 0))
  (dead)
  (comment "empty cohort"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1) (lk lk-0))
  (precedes ((1 1) (2 0)) ((1 1) (3 2)) ((2 1) (3 3)) ((3 4) (0 0))
    ((4 1) (5 0)) ((4 2) (3 0)) ((5 1) (6 2)) ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k "wrap")
    (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0 gen-st-wrap-1)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init"))
    (ch-msg lk-0 (cat pt-4 k "decrypt")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-4 k "decrypt"))
      (load lk-0 (cat pt-5 k1 "wrap")) (send (enc k k1))))
  (label 439)
  (parent 172)
  (unrealized (6 0) (6 3))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "wrap") (k k) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (3 2)) ((2 1) (3 3)) ((2 1) (6 0))
    ((3 4) (0 0)) ((4 1) (5 0)) ((4 2) (3 0)) ((5 1) (0 0))
    ((6 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "wrap" "decrypt") (neq "wrap" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-decrypt-at-1 fact-set-decrypt-neq0 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation nonce-test (added-strand set-decrypt 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init"))
    (ch-msg lk-0 (cat pt-4 k "decrypt")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((load lk (cat pt-1 k "wrap")) (stor lk (cat pt-5 k "decrypt"))))
  (label 440)
  (parent 172)
  (unrealized (0 0))
  (dead)
  (comment "empty cohort"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (3 2)) ((2 1) (3 3)) ((2 1) (6 2))
    ((3 4) (0 0)) ((4 1) (5 0)) ((4 2) (3 0)) ((5 1) (0 0))
    ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init"))
    (ch-msg lk-0 (cat pt-4 k "decrypt")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-5 k1 "wrap")) (send (enc k k1))))
  (label 441)
  (parent 172)
  (unrealized (6 0) (6 3))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (precedes ((1 1) (2 0)) ((1 1) (3 2)) ((2 1) (3 3)) ((3 4) (0 0))
    ((4 1) (5 0)) ((4 1) (6 2)) ((4 2) (3 0)) ((5 1) (0 0))
    ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init"))
    (ch-msg lk-0 (cat pt-4 k "decrypt")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-5 k1 "wrap")) (send (enc k k1))))
  (label 442)
  (parent 172)
  (unrealized (6 0) (6 3))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (3 2)) ((1 1) (6 2)) ((2 1) (3 3))
    ((3 4) (0 0)) ((4 1) (5 0)) ((4 2) (3 0)) ((5 1) (0 0))
    ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init"))
    (ch-msg lk-0 (cat pt-4 k "decrypt")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-5 k1 "wrap")) (send (enc k k1))))
  (label 443)
  (parent 172)
  (unrealized (6 0) (6 3))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand decrypt 4 (x k) (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((1 1) (3 2)) ((2 1) (3 3)) ((3 4) (0 0))
    ((4 1) (5 0)) ((4 2) (3 0)) ((5 1) (6 2)) ((6 3) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-decrypt-0)
  (operation nonce-test (added-strand decrypt 4) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init"))
    (ch-msg lk-0 (cat pt-4 k "decrypt")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((recv (enc k k)) (recv (hash k)) (load lk-0 (cat pt-4 k "decrypt"))
      (send k)))
  (label 444)
  (parent 172)
  (unrealized (6 0) (6 1))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "wrap") (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((1 1) (3 2)) ((2 1) (3 3)) ((3 4) (0 0))
    ((4 1) (5 0)) ((4 2) (3 0)) ((5 1) (6 0)) ((6 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "wrap" "decrypt") (neq "wrap" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-decrypt-at-1 fact-set-decrypt-neq0 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation nonce-test (added-strand set-decrypt 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init")) (ch-msg lk-0 (cat pt-4 k "wrap")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "wrap")))
    ((load lk-0 (cat pt-4 k "wrap"))
      (stor lk-0 (cat pt-5 k "decrypt"))))
  (label 445)
  (parent 174)
  (unrealized (0 0))
  (dead)
  (comment "empty cohort"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "wrap") (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((1 1) (3 2)) ((2 1) (3 3)) ((3 4) (0 0))
    ((4 1) (5 0)) ((4 2) (3 0)) ((5 1) (6 0)) ((6 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "wrap" "decrypt") (neq "wrap" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule fact-set-decrypt-neq0 trRl_set-wrap-at-1 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation nonce-test (added-strand set-wrap 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init")) (ch-msg lk-0 (cat pt-4 k "wrap")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "wrap")))
    ((load lk-0 (cat pt-4 k "wrap")) (stor lk-0 (cat pt-5 k "wrap"))))
  (label 446)
  (parent 174)
  (unrealized (0 0))
  (dead)
  (comment "empty cohort"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "wrap") (k k) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (3 2)) ((2 1) (3 3)) ((2 1) (6 0))
    ((3 4) (0 0)) ((4 1) (5 0)) ((4 2) (3 0)) ((5 1) (0 0))
    ((6 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "wrap" "decrypt") (neq "wrap" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule fact-set-decrypt-neq0 trRl_set-wrap-at-1 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation nonce-test (added-strand set-wrap 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init")) (ch-msg lk-0 (cat pt-4 k "wrap")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "wrap")))
    ((load lk (cat pt-1 k "wrap")) (stor lk (cat pt-5 k "wrap"))))
  (label 447)
  (parent 174)
  (unrealized (0 0))
  (dead)
  (comment "empty cohort"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk-0))
  (precedes ((1 1) (2 0)) ((1 1) (3 2)) ((2 1) (3 3)) ((3 4) (0 0))
    ((4 1) (5 0)) ((4 2) (3 0)) ((5 1) (6 2)) ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init")) (ch-msg lk-0 (cat pt-4 k "wrap")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-4 k "wrap"))
      (load lk-0 (cat pt-5 k1 "wrap")) (send (enc k k1))))
  (label 448)
  (parent 174)
  (unrealized (6 0) (6 3))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "wrap") (k k) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (3 2)) ((2 1) (3 3)) ((2 1) (6 0))
    ((3 4) (0 0)) ((4 1) (5 0)) ((4 2) (3 0)) ((5 1) (0 0))
    ((6 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "wrap" "decrypt") (neq "wrap" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-decrypt-at-1 fact-set-decrypt-neq0 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation nonce-test (added-strand set-decrypt 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init")) (ch-msg lk-0 (cat pt-4 k "wrap")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "wrap")))
    ((load lk (cat pt-1 k "wrap")) (stor lk (cat pt-5 k "decrypt"))))
  (label 449)
  (parent 174)
  (unrealized (0 0))
  (dead)
  (comment "empty cohort"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (precedes ((1 1) (2 0)) ((1 1) (3 2)) ((2 1) (3 3)) ((3 4) (0 0))
    ((4 1) (5 0)) ((4 1) (6 2)) ((4 2) (3 0)) ((5 1) (0 0))
    ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init")) (ch-msg lk-0 (cat pt-4 k "wrap")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-5 k1 "wrap")) (send (enc k k1))))
  (label 450)
  (parent 174)
  (unrealized (6 0) (6 3))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (3 2)) ((2 1) (3 3)) ((2 1) (6 2))
    ((3 4) (0 0)) ((4 1) (5 0)) ((4 2) (3 0)) ((5 1) (0 0))
    ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init")) (ch-msg lk-0 (cat pt-4 k "wrap")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-5 k1 "wrap")) (send (enc k k1))))
  (label 451)
  (parent 174)
  (unrealized (6 0) (6 3))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (3 2)) ((1 1) (6 2)) ((2 1) (3 3))
    ((3 4) (0 0)) ((4 1) (5 0)) ((4 2) (3 0)) ((5 1) (0 0))
    ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init")) (ch-msg lk-0 (cat pt-4 k "wrap")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-5 k1 "wrap")) (send (enc k k1))))
  (label 452)
  (parent 174)
  (unrealized (6 0) (6 3))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (precedes ((1 1) (2 0)) ((1 1) (3 2)) ((2 1) (3 3)) ((3 4) (0 0))
    ((4 2) (3 0)) ((4 2) (5 0)) ((5 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 6 4 make 3) (hash k) (5 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-4 k1 "wrap")) (send (enc k k1))))
  (label 453)
  (parent 176)
  (seen 72 1162 1164)
  (unrealized (5 3))
  (comment "16 in cohort - 3 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (precedes ((1 1) (2 3)) ((2 4) (0 0)) ((3 1) (4 2)) ((3 2) (2 0))
    ((4 4) (0 0)) ((5 1) (1 0)) ((5 1) (2 2)) ((5 2) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 1 6 make 3) (hash k) (5 0))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt-4 k "init"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init"))
      (send (hash k))))
  (label 454)
  (parent 176)
  (seen 72 1166)
  (unrealized (4 3))
  (comment "14 in cohort - 2 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((1 1) (3 2)) ((2 1) (3 3)) ((3 4) (0 0))
    ((4 1) (5 2)) ((4 2) (3 0)) ((5 4) (0 0)) ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation encryption-test (added-strand make 3) (hash k) (5 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init"))
      (send (hash k))))
  (label 455)
  (parent 176)
  (seen 1119 1167 1169)
  (unrealized (5 3))
  (comment "16 in cohort - 3 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (deflistener k)
  (precedes ((1 1) (2 0)) ((1 1) (3 2)) ((2 1) (3 3)) ((3 4) (0 0))
    ((4 1) (5 2)) ((4 2) (3 0)) ((5 4) (0 0)) ((6 1) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (added-listener k) (hash k) (5 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k)))
  (label 456)
  (parent 176)
  (unrealized (5 3) (6 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (3 2)) ((2 1) (3 3)) ((2 1) (5 2))
    ((3 4) (0 0)) ((4 2) (3 0)) ((4 2) (5 0)) ((5 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 6 4 make 3) (hash k) (5 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1))))
  (label 457)
  (parent 177)
  (seen 1074 1173)
  (unrealized (5 3))
  (comment "4 in cohort - 1 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (precedes ((1 1) (2 3)) ((1 1) (4 2)) ((2 4) (0 0)) ((3 2) (2 0))
    ((4 4) (0 0)) ((5 1) (1 0)) ((5 1) (2 2)) ((5 2) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 1 6 make 3) (hash k) (5 0))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt-4 k "init"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt k "wrap"))
      (load lk-0 (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init"))
      (send (hash k))))
  (label 458)
  (parent 177)
  (seen 1015 1174)
  (unrealized (4 3))
  (comment "4 in cohort - 1 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((1 1) (3 2)) ((2 1) (3 3)) ((2 1) (5 2))
    ((3 4) (0 0)) ((4 2) (3 0)) ((5 4) (0 0)) ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation encryption-test (added-strand make 3) (hash k) (5 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init"))
      (send (hash k))))
  (label 459)
  (parent 177)
  (seen 1078 1175)
  (unrealized (5 3))
  (comment "4 in cohort - 1 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (precedes ((1 1) (2 0)) ((1 1) (3 2)) ((2 1) (3 3)) ((2 1) (5 2))
    ((3 4) (0 0)) ((4 2) (3 0)) ((5 4) (0 0)) ((6 1) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (added-listener k) (hash k) (5 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k)))
  (label 460)
  (parent 177)
  (unrealized (5 3) (6 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (3 2)) ((1 1) (5 2)) ((2 1) (3 3))
    ((3 4) (0 0)) ((4 2) (3 0)) ((4 2) (5 0)) ((5 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 6 4 make 3) (hash k) (5 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1))))
  (label 461)
  (parent 178)
  (seen 74)
  (unrealized (5 3))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (precedes ((1 1) (2 3)) ((2 4) (0 0)) ((3 2) (2 0)) ((4 4) (0 0))
    ((5 1) (1 0)) ((5 1) (2 2)) ((5 2) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 1 6 make 3) (hash k) (5 0))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt-4 k "init"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-4 k "init"))
      (load lk-0 (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init"))
      (send (hash k))))
  (label 462)
  (parent 178)
  (seen 1119 1179)
  (unrealized (4 3))
  (comment "7 in cohort - 1 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((1 1) (3 2)) ((1 1) (5 2)) ((2 1) (3 3))
    ((3 4) (0 0)) ((4 2) (3 0)) ((5 4) (0 0)) ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation encryption-test (added-strand make 3) (hash k) (5 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init"))
      (send (hash k))))
  (label 463)
  (parent 178)
  (seen 1180)
  (unrealized (5 3))
  (comment "4 in cohort - 1 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (precedes ((1 1) (2 0)) ((1 1) (3 2)) ((1 1) (5 2)) ((2 1) (3 3))
    ((3 4) (0 0)) ((4 2) (3 0)) ((5 4) (0 0)) ((6 1) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (added-listener k) (hash k) (5 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k)))
  (label 464)
  (parent 178)
  (unrealized (5 3) (6 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (0 0)) ((3 4) (0 0))
    ((4 1) (3 0)) ((5 1) (6 0)) ((6 1) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-wrap-at-1 trRl_make-at-0)
  (operation nonce-test (added-strand set-wrap 2) k (4 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk-0 (cat pt-4 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt-5 k "wrap"))))
  (label 465)
  (parent 180)
  (unrealized (3 3) (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (0 0)) ((3 4) (0 0))
    ((4 1) (3 0)) ((5 1) (6 0)) ((6 1) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-decrypt-at-1 trRl_make-at-0)
  (operation nonce-test (added-strand set-decrypt 2) k (4 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk-0 (cat pt-4 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((load lk-0 (cat pt-4 k "init"))
      (stor lk-0 (cat pt-5 k "decrypt"))))
  (label 466)
  (parent 180)
  (unrealized (3 3) (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk-0))
  (precedes ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (0 0)) ((3 4) (0 0))
    ((4 1) (3 0)) ((5 1) (6 2)) ((6 4) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (4 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk-0 (cat pt-4 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-0 (cat pt-4 k "init"))
      (load lk-0 (cat pt-5 k1-0 "wrap")) (send (enc k k1-0))))
  (label 467)
  (parent 180)
  (seen 483)
  (unrealized (3 3) (6 0) (6 3))
  (comment "4 in cohort - 3 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (6 2)) ((2 1) (0 0)) ((3 4) (0 0))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 4) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (4 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk-0 (cat pt-4 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-5 k1-0 "wrap")) (send (enc k k1-0))))
  (label 468)
  (parent 180)
  (seen 470)
  (unrealized (3 3) (6 0) (6 3))
  (comment "4 in cohort - 3 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old mesg) (pt pt-0 pt-1 pt-2 pt-3 pval) (k k1 k1-0 skey)
    (lk locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk))
  (defstrand make 3 (old old) (k k) (lk lk))
  (precedes ((1 1) (0 0)) ((2 4) (0 0)) ((3 1) (2 0)) ((4 4) (3 0))
    ((5 1) (1 0)) ((5 2) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 1 6 make 3) (hash k) (5 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt-3 k "init")) (stor lk (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-0 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((recv (hash k)) (recv (hash k1-0)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k1-0 "wrap")) (send (enc k k1-0)))
    ((load lk (cat pt-2 old)) (stor lk (cat pt-3 k "init"))
      (send (hash k))))
  (label 469)
  (parent 181)
  (unrealized (2 3) (4 3))
  (dead)
  (comment "empty cohort"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((1 1) (5 2)) ((2 1) (0 0)) ((3 4) (0 0))
    ((4 1) (3 0)) ((5 4) (4 0)) ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation encryption-test (added-strand make 3) (hash k) (5 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((recv (hash k)) (recv (hash k1-0)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-3 k1-0 "wrap")) (send (enc k k1-0)))
    ((load lk-0 (cat pt-4 old-0)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k))))
  (label 470)
  (parent 181)
  (unrealized (3 3) (5 3))
  (dead)
  (comment "empty cohort"))

(defskeleton wrap-decrypt
  (vars (old mesg) (pt pt-0 pt-1 pt-2 pt-3 pval) (k k1 k1-0 skey)
    (lk locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk))
  (deflistener k)
  (precedes ((1 1) (2 0)) ((1 1) (5 2)) ((2 1) (0 0)) ((3 4) (0 0))
    ((4 1) (3 0)) ((5 4) (4 0)) ((6 1) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (added-listener k) (hash k) (5 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((recv (hash k)) (recv (hash k1-0)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-3 k1-0 "wrap")) (send (enc k k1-0)))
    ((recv k) (send k)))
  (label 471)
  (parent 181)
  (unrealized (3 3) (5 3) (6 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((1 1) (3 2)) ((2 1) (0 0)) ((3 4) (0 0))
    ((4 1) (3 0)) ((5 4) (4 0)) ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 5 7 make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-0 (cat pt-5 k "init"))
      (load lk-0 (cat pt-3 k1-0 "wrap")) (send (enc k k1-0)))
    ((load lk-0 (cat pt-4 old-0)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k))))
  (label 482)
  (parent 184)
  (seen 1204)
  (unrealized (3 3) (5 3))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (precedes ((1 1) (0 0)) ((2 4) (0 0)) ((3 1) (2 0)) ((4 1) (5 2))
    ((5 4) (3 0)) ((6 1) (1 0)) ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 1 7 make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-5 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-5 k "init"))
      (load lk-0 (cat pt-0 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk (cat pt-1 old)) (stor lk (cat pt-2 k "init")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk (cat pt-2 k "init"))
      (load lk (cat pt-3 k1-0 "wrap")) (send (enc k k1-0)))
    ((load lk-0 (cat pt-4 old-0)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k))))
  (label 483)
  (parent 184)
  (seen 1205)
  (unrealized (2 3) (5 3))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k1) (lk lk-0))
  (defstrand make 2 (old old-1) (k k1) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (0 0)) ((3 4) (0 0)) ((4 1) (6 0))
    ((4 2) (3 0)) ((5 1) (3 3)) ((6 1) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule invShearsRule trRl_make-at-0 trRl_make-at-1)
  (operation channel-test (added-strand make 2)
    (ch-msg lk-0 (cat pt-5 k1 "init")) (5 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-4 k "init"))
      (load lk-0 (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-5 k1 "init")) (stor lk-0 (cat pt-2 k1 "wrap")))
    ((load lk-0 (cat pt-6 old-1)) (stor lk-0 (cat pt-5 k1 "init"))))
  (label 486)
  (parent 185)
  (seen 766)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k1) (lk lk))
  (defstrand make 2 (old old-1) (k k1) (lk lk))
  (precedes ((1 1) (0 0)) ((2 1) (3 2)) ((2 1) (6 0)) ((3 4) (0 0))
    ((4 1) (1 0)) ((4 2) (3 0)) ((5 1) (3 3)) ((6 1) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule invShearsRule trRl_make-at-0 trRl_make-at-1)
  (operation channel-test (added-strand make 2)
    (ch-msg lk (cat pt-5 k1 "init")) (5 0))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt k "decrypt")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init"))
      (send (hash k)))
    ((load lk (cat pt-5 k1 "init")) (stor lk (cat pt-2 k1 "wrap")))
    ((load lk (cat pt-6 old-1)) (stor lk (cat pt-5 k1 "init"))))
  (label 487)
  (parent 186)
  (seen 97)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (deflistener k)
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (0 0)) ((3 1) (4 2))
    ((4 4) (0 0)) ((5 1) (4 0)) ((6 1) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation nonce-test (added-strand make 2) k (5 0)
    (ch-msg lk (cat pt-0 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init"))))
  (label 489)
  (parent 188)
  (seen 227)
  (unrealized (4 3) (5 0))
  (comment "7 in cohort - 5 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (deflistener k)
  (precedes ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (0 0)) ((3 1) (5 0))
    ((4 4) (0 0)) ((5 1) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation nonce-test (displaced 6 3 make 2) k (5 0)
    (ch-msg lk (cat pt-0 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k)))
  (label 490)
  (parent 188)
  (seen 214 1207)
  (unrealized (4 3) (5 0))
  (comment "5 in cohort - 2 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (deflistener k)
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (6 2)) ((2 1) (0 0)) ((3 1) (4 2))
    ((4 4) (0 0)) ((5 1) (4 0)) ((6 4) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (5 0)
    (ch-msg lk (cat pt-0 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((recv (hash k)) (recv (hash k1-0)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-5 k1-0 "wrap")) (send (enc k k1-0))))
  (label 491)
  (parent 188)
  (unrealized (4 3) (6 0) (6 3))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (deflistener k)
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((2 1) (0 0)) ((3 1) (5 0)) ((4 4) (0 0))
    ((5 1) (4 0)) ((6 1) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation nonce-test (added-strand make 2) k (5 0)
    (ch-msg lk-0 (cat pt-3 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init"))))
  (label 492)
  (parent 189)
  (seen 1207)
  (unrealized (4 3) (5 0))
  (comment "5 in cohort - 4 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (deflistener k)
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (0 0)) ((3 1) (6 2)) ((4 4) (0 0))
    ((5 1) (4 0)) ((6 4) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (5 0)
    (ch-msg lk-0 (cat pt-3 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-5 k1-0 "wrap")) (send (enc k k1-0))))
  (label 493)
  (parent 189)
  (unrealized (4 3) (6 0) (6 3))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k1) (lk lk-0))
  (defstrand make 2 (old old-1) (k k1) (lk lk-0))
  (precedes ((1 1) (3 2)) ((1 1) (6 0)) ((2 1) (0 0)) ((3 4) (0 0))
    ((4 1) (1 0)) ((4 2) (3 0)) ((5 1) (3 3)) ((6 1) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k1 "init")
    (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule invShearsRule trRl_make-at-0 trRl_make-at-1)
  (operation channel-test (added-strand make 2)
    (ch-msg lk-0 (cat pt-5 k1 "init")) (5 0))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt k "decrypt")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt k "decrypt"))
      (load lk-0 (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-5 k1 "init")) (stor lk-0 (cat pt-2 k1 "wrap")))
    ((load lk-0 (cat pt-6 old-1)) (stor lk-0 (cat pt-5 k1 "init"))))
  (label 497)
  (parent 191)
  (seen 199)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (4 2)) ((3 1) (0 0))
    ((4 4) (0 0)) ((5 1) (4 0)) ((6 1) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation nonce-test (added-strand make 2) k (5 0)
    (ch-msg lk (cat pt-0 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "decrypt"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init"))))
  (label 499)
  (parent 193)
  (seen 1227)
  (unrealized (4 3) (5 0))
  (comment "6 in cohort - 5 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (precedes ((1 1) (2 0)) ((2 1) (5 0)) ((3 1) (0 0)) ((4 4) (0 0))
    ((5 1) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation nonce-test (displaced 6 2 set-decrypt 2) k (5 0)
    (ch-msg lk (cat pt-0 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "decrypt"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k)))
  (label 500)
  (parent 193)
  (seen 514 1227)
  (unrealized (4 3) (5 0))
  (comment "4 in cohort - 2 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (6 2)) ((2 1) (4 2)) ((3 1) (0 0))
    ((4 4) (0 0)) ((5 1) (4 0)) ((6 4) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k1 "wrap")
    (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (5 0)
    (ch-msg lk (cat pt-0 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "decrypt"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((recv (hash k)) (recv (hash k1-0)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-5 k1-0 "wrap")) (send (enc k k1-0))))
  (label 501)
  (parent 193)
  (seen 520)
  (unrealized (4 3) (6 0) (6 3))
  (comment "4 in cohort - 3 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k1) (lk lk-0))
  (defstrand make 2 (old old-1) (k k1) (lk lk-0))
  (precedes ((1 1) (6 0)) ((2 1) (0 0)) ((3 4) (0 0)) ((4 1) (1 0))
    ((4 2) (3 0)) ((5 1) (3 3)) ((6 1) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule invShearsRule trRl_make-at-0 trRl_make-at-1)
  (operation channel-test (added-strand make 2)
    (ch-msg lk-0 (cat pt-5 k1 "init")) (5 0))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt k "decrypt")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-4 k "init"))
      (load lk-0 (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-5 k1 "init")) (stor lk-0 (cat pt-2 k1 "wrap")))
    ((load lk-0 (cat pt-6 old-1)) (stor lk-0 (cat pt-5 k1 "init"))))
  (label 505)
  (parent 195)
  (seen 766)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (0 0)) ((3 1) (0 0))
    ((4 4) (0 0)) ((5 1) (4 0)) ((6 1) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation nonce-test (added-strand make 2) k (5 0)
    (ch-msg lk (cat pt-0 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init"))))
  (label 507)
  (parent 197)
  (unrealized (4 3) (5 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (6 2)) ((2 1) (0 0)) ((3 1) (0 0))
    ((4 4) (0 0)) ((5 1) (4 0)) ((6 4) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (5 0)
    (ch-msg lk (cat pt-0 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((recv (hash k)) (recv (hash k1-0)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-5 k1-0 "wrap")) (send (enc k k1-0))))
  (label 508)
  (parent 197)
  (seen 541)
  (unrealized (4 3) (6 0) (6 3))
  (comment "4 in cohort - 3 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (3 2)) ((3 4) (0 0))
    ((4 1) (3 0)) ((5 1) (6 0)) ((6 1) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-decrypt-at-1 trRl_make-at-0)
  (operation nonce-test (added-strand set-decrypt 2) k (4 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk-0 (cat pt-4 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "decrypt"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((load lk-0 (cat pt-4 k "init"))
      (stor lk-0 (cat pt-5 k "decrypt"))))
  (label 512)
  (parent 201)
  (seen 1245)
  (unrealized (3 3) (4 0))
  (comment "8 in cohort - 7 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (3 2)) ((3 4) (0 0))
    ((4 1) (3 0)) ((5 1) (6 0)) ((6 1) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-wrap-at-1 trRl_make-at-0)
  (operation nonce-test (added-strand set-wrap 2) k (4 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk-0 (cat pt-4 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "decrypt"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt-5 k "wrap"))))
  (label 513)
  (parent 201)
  (seen 1252)
  (unrealized (3 3) (4 0))
  (comment "8 in cohort - 7 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (4 0)) ((3 4) (0 0)) ((4 1) (3 0))
    ((5 1) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule scissorsRule trRl_set-decrypt-at-1 trRl_make-at-0)
  (operation nonce-test (added-strand set-decrypt 2) k (4 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk-0 (cat pt-4 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "decrypt"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init"))))
  (label 514)
  (parent 201)
  (seen 1245 1252)
  (unrealized (3 3) (4 0))
  (comment "5 in cohort - 3 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk-0))
  (precedes ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (3 2)) ((3 4) (0 0))
    ((4 1) (3 0)) ((5 1) (6 2)) ((6 4) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k1 "wrap")
    (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (4 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk-0 (cat pt-4 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "decrypt"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-0 (cat pt-4 k "init"))
      (load lk-0 (cat pt-5 k1-0 "wrap")) (send (enc k k1-0))))
  (label 515)
  (parent 201)
  (seen 533)
  (unrealized (3 3) (6 0) (6 3))
  (comment "4 in cohort - 3 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (6 2)) ((2 1) (3 2)) ((3 4) (0 0))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 4) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k1 "wrap")
    (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (4 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk-0 (cat pt-4 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "decrypt"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-5 k1-0 "wrap")) (send (enc k k1-0))))
  (label 516)
  (parent 201)
  (seen 520)
  (unrealized (3 3) (6 0) (6 3))
  (comment "4 in cohort - 3 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old mesg) (pt pt-0 pt-1 pt-2 pt-3 pval) (k k1 k1-0 skey)
    (lk locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1-0) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (5 2)) ((3 4) (0 0)) ((4 1) (3 0))
    ((5 4) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k1 "wrap")
    (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (4 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "decrypt")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "decrypt"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((recv (hash k)) (recv (hash k1-0)) (load lk (cat pt-1 k "decrypt"))
      (load lk (cat pt-3 k1-0 "wrap")) (send (enc k k1-0))))
  (label 517)
  (parent 202)
  (unrealized (3 3) (5 0) (5 3))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old mesg) (pt pt-0 pt-1 pt-2 pt-3 pval) (k k1 k1-0 skey)
    (lk locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (5 2)) ((2 1) (4 0)) ((3 4) (0 0))
    ((4 1) (3 0)) ((5 4) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k1 "wrap")
    (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (4 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "decrypt")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "decrypt"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((recv (hash k)) (recv (hash k1-0)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-3 k1-0 "wrap")) (send (enc k k1-0))))
  (label 518)
  (parent 202)
  (unrealized (3 3) (5 0) (5 3))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old mesg) (pt pt-0 pt-1 pt-2 pt-3 pval) (k k1 k1-0 skey)
    (lk locn))
  (deflistener k)
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk))
  (defstrand make 3 (old old) (k k) (lk lk))
  (precedes ((1 1) (2 2)) ((2 4) (0 0)) ((3 1) (2 0)) ((4 4) (3 0))
    ((5 1) (1 0)) ((5 2) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k1 "wrap")
    (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 1 6 make 3) (hash k) (5 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt-3 k "init")) (stor lk (cat pt k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt k "decrypt"))
      (load lk (cat pt-0 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((recv (hash k)) (recv (hash k1-0)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k1-0 "wrap")) (send (enc k k1-0)))
    ((load lk (cat pt-2 old)) (stor lk (cat pt-3 k "init"))
      (send (hash k))))
  (label 519)
  (parent 203)
  (unrealized (2 3) (4 3))
  (dead)
  (comment "empty cohort"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((1 1) (5 2)) ((2 1) (3 2)) ((3 4) (0 0))
    ((4 1) (3 0)) ((5 4) (4 0)) ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k1 "wrap")
    (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation encryption-test (added-strand make 3) (hash k) (5 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "decrypt"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((recv (hash k)) (recv (hash k1-0)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-3 k1-0 "wrap")) (send (enc k k1-0)))
    ((load lk-0 (cat pt-4 old-0)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k))))
  (label 520)
  (parent 203)
  (unrealized (3 3) (5 3))
  (dead)
  (comment "empty cohort"))

(defskeleton wrap-decrypt
  (vars (old mesg) (pt pt-0 pt-1 pt-2 pt-3 pval) (k k1 k1-0 skey)
    (lk locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk))
  (deflistener k)
  (precedes ((1 1) (2 0)) ((1 1) (5 2)) ((2 1) (3 2)) ((3 4) (0 0))
    ((4 1) (3 0)) ((5 4) (4 0)) ((6 1) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k1 "wrap")
    (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (added-listener k) (hash k) (5 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "decrypt"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((recv (hash k)) (recv (hash k1-0)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-3 k1-0 "wrap")) (send (enc k k1-0)))
    ((recv k) (send k)))
  (label 521)
  (parent 203)
  (unrealized (3 3) (5 3) (6 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (3 2)) ((3 4) (0 0)) ((4 1) (3 0))
    ((5 4) (4 0)) ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k1 "wrap")
    (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 5 7 make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "decrypt"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-0 (cat pt-5 k "init"))
      (load lk-0 (cat pt-3 k1-0 "wrap")) (send (enc k k1-0)))
    ((load lk-0 (cat pt-4 old-0)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k))))
  (label 532)
  (parent 206)
  (seen 1273)
  (unrealized (3 3) (5 3))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1) (lk lk-0))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (precedes ((1 1) (2 2)) ((2 4) (0 0)) ((3 1) (2 0)) ((4 1) (5 2))
    ((5 4) (3 0)) ((6 1) (1 0)) ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k1 "wrap")
    (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 1 7 make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-5 k "init")) (stor lk-0 (cat pt k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt k "decrypt"))
      (load lk-0 (cat pt-0 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk (cat pt-1 old)) (stor lk (cat pt-2 k "init")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk (cat pt-2 k "init"))
      (load lk (cat pt-3 k1-0 "wrap")) (send (enc k k1-0)))
    ((load lk-0 (cat pt-4 old-0)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k))))
  (label 533)
  (parent 206)
  (seen 1274)
  (unrealized (2 3) (5 3))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (0 0)) ((3 4) (0 0))
    ((4 1) (3 0)) ((5 1) (6 0)) ((6 1) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-decrypt-at-1 trRl_make-at-0)
  (operation nonce-test (added-strand set-decrypt 2) k (4 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk-0 (cat pt-4 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((load lk-0 (cat pt-4 k "init"))
      (stor lk-0 (cat pt-5 k "decrypt"))))
  (label 536)
  (parent 209)
  (unrealized (3 3) (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (0 0)) ((3 4) (0 0))
    ((4 1) (3 0)) ((5 1) (6 0)) ((6 1) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-wrap-at-1 trRl_make-at-0)
  (operation nonce-test (added-strand set-wrap 2) k (4 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk-0 (cat pt-4 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt-5 k "wrap"))))
  (label 537)
  (parent 209)
  (unrealized (3 3) (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk-0))
  (precedes ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (0 0)) ((3 4) (0 0))
    ((4 1) (3 0)) ((5 1) (6 2)) ((6 4) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (4 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk-0 (cat pt-4 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-0 (cat pt-4 k "init"))
      (load lk-0 (cat pt-5 k1-0 "wrap")) (send (enc k k1-0))))
  (label 538)
  (parent 209)
  (seen 554)
  (unrealized (3 3) (6 0) (6 3))
  (comment "4 in cohort - 3 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (6 2)) ((2 1) (0 0)) ((3 4) (0 0))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 4) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (4 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk-0 (cat pt-4 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-5 k1-0 "wrap")) (send (enc k k1-0))))
  (label 539)
  (parent 209)
  (seen 541)
  (unrealized (3 3) (6 0) (6 3))
  (comment "4 in cohort - 3 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old mesg) (pt pt-0 pt-1 pt-2 pt-3 pval) (k k1 k1-0 skey)
    (lk locn))
  (deflistener k)
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk))
  (defstrand make 3 (old old) (k k) (lk lk))
  (precedes ((1 1) (0 0)) ((2 4) (0 0)) ((3 1) (2 0)) ((4 4) (3 0))
    ((5 1) (1 0)) ((5 2) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 1 6 make 3) (hash k) (5 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt-3 k "init")) (stor lk (cat pt k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-0 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((recv (hash k)) (recv (hash k1-0)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k1-0 "wrap")) (send (enc k k1-0)))
    ((load lk (cat pt-2 old)) (stor lk (cat pt-3 k "init"))
      (send (hash k))))
  (label 540)
  (parent 210)
  (unrealized (2 3) (4 3))
  (dead)
  (comment "empty cohort"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((1 1) (5 2)) ((2 1) (0 0)) ((3 4) (0 0))
    ((4 1) (3 0)) ((5 4) (4 0)) ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation encryption-test (added-strand make 3) (hash k) (5 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((recv (hash k)) (recv (hash k1-0)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-3 k1-0 "wrap")) (send (enc k k1-0)))
    ((load lk-0 (cat pt-4 old-0)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k))))
  (label 541)
  (parent 210)
  (unrealized (3 3) (5 3))
  (dead)
  (comment "empty cohort"))

(defskeleton wrap-decrypt
  (vars (old mesg) (pt pt-0 pt-1 pt-2 pt-3 pval) (k k1 k1-0 skey)
    (lk locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk))
  (deflistener k)
  (precedes ((1 1) (2 0)) ((1 1) (5 2)) ((2 1) (0 0)) ((3 4) (0 0))
    ((4 1) (3 0)) ((5 4) (4 0)) ((6 1) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (added-listener k) (hash k) (5 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((recv (hash k)) (recv (hash k1-0)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-3 k1-0 "wrap")) (send (enc k k1-0)))
    ((recv k) (send k)))
  (label 542)
  (parent 210)
  (unrealized (3 3) (5 3) (6 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((1 1) (3 2)) ((2 1) (0 0)) ((3 4) (0 0))
    ((4 1) (3 0)) ((5 4) (4 0)) ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 5 7 make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-0 (cat pt-5 k "init"))
      (load lk-0 (cat pt-3 k1-0 "wrap")) (send (enc k k1-0)))
    ((load lk-0 (cat pt-4 old-0)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k))))
  (label 553)
  (parent 213)
  (seen 1295)
  (unrealized (3 3) (5 3))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (precedes ((1 1) (0 0)) ((2 4) (0 0)) ((3 1) (2 0)) ((4 1) (5 2))
    ((5 4) (3 0)) ((6 1) (1 0)) ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 1 7 make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-5 k "init")) (stor lk-0 (cat pt k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-5 k "init"))
      (load lk-0 (cat pt-0 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk (cat pt-1 old)) (stor lk (cat pt-2 k "init")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk (cat pt-2 k "init"))
      (load lk (cat pt-3 k1-0 "wrap")) (send (enc k k1-0)))
    ((load lk-0 (cat pt-4 old-0)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k))))
  (label 554)
  (parent 213)
  (seen 1296)
  (unrealized (2 3) (5 3))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (3 0)) ((2 4) (0 0)) ((3 1) (2 0)) ((4 1) (5 0))
    ((5 1) (3 0)) ((6 1) (3 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation nonce-test (added-strand make 2) k (3 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk-0 (cat pt-3 k "init"))
    (ch-msg lk-0 (cat pt-4 k "decrypt")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init"))))
  (label 557)
  (parent 214)
  (unrealized (2 3) (3 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "decrypt") (k k) (lk lk-0))
  (precedes ((1 1) (3 0)) ((2 4) (0 0)) ((3 1) (2 0)) ((4 1) (5 0))
    ((5 1) (6 0)) ((6 1) (3 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k1 "wrap"))
  (facts (neq "decrypt" "decrypt") (neq "decrypt" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule fact-set-decrypt-neq0 trRl_set-wrap-at-1 gen-st-set-wrap-0
    fact-set-wrap-neq0 trRl_make-at-0)
  (operation nonce-test (added-strand set-wrap 2) k (3 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk-0 (cat pt-3 k "init"))
    (ch-msg lk-0 (cat pt-4 k "decrypt")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((load lk-0 (cat pt-4 k "decrypt"))
      (stor lk-0 (cat pt-5 k "wrap"))))
  (label 558)
  (parent 214)
  (unrealized (2 3) (3 0))
  (dead)
  (comment "empty cohort"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "decrypt") (k k) (lk lk-0))
  (precedes ((1 1) (3 0)) ((2 4) (0 0)) ((3 1) (2 0)) ((4 1) (5 0))
    ((5 1) (6 0)) ((6 1) (3 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k1 "wrap"))
  (facts (neq "decrypt" "decrypt") (neq "decrypt" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-decrypt-at-1 fact-set-decrypt-neq0 gen-st-set-wrap-0
    fact-set-wrap-neq0 trRl_make-at-0)
  (operation nonce-test (added-strand set-decrypt 2) k (3 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk-0 (cat pt-3 k "init"))
    (ch-msg lk-0 (cat pt-4 k "decrypt")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((load lk-0 (cat pt-4 k "decrypt"))
      (stor lk-0 (cat pt-5 k "decrypt"))))
  (label 559)
  (parent 214)
  (unrealized (2 3) (3 0))
  (dead)
  (comment "empty cohort"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1-0) (lk lk-0))
  (precedes ((1 1) (3 0)) ((2 4) (0 0)) ((3 1) (2 0)) ((4 1) (5 0))
    ((5 1) (6 2)) ((6 4) (3 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k1 "wrap")
    (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0 gen-st-wrap-1)
  (operation nonce-test (added-strand wrap 5) k (3 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk-0 (cat pt-3 k "init"))
    (ch-msg lk-0 (cat pt-4 k "decrypt")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k1-0))
      (load lk-0 (cat pt-4 k "decrypt"))
      (load lk-0 (cat pt-5 k1-0 "wrap")) (send (enc k k1-0))))
  (label 560)
  (parent 214)
  (seen 597)
  (unrealized (2 3) (6 0) (6 3))
  (comment "4 in cohort - 3 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk-0))
  (precedes ((1 1) (3 0)) ((2 4) (0 0)) ((3 1) (2 0)) ((4 1) (5 0))
    ((4 1) (6 2)) ((5 1) (3 0)) ((6 4) (3 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (3 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk-0 (cat pt-3 k "init"))
    (ch-msg lk-0 (cat pt-4 k "decrypt")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-5 k1-0 "wrap")) (send (enc k k1-0))))
  (label 561)
  (parent 214)
  (seen 601)
  (unrealized (2 3) (6 0) (6 3))
  (comment "4 in cohort - 3 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk))
  (precedes ((1 1) (6 2)) ((2 4) (0 0)) ((3 1) (2 0)) ((4 1) (5 0))
    ((5 1) (3 0)) ((6 4) (3 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (3 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk-0 (cat pt-3 k "init"))
    (ch-msg lk-0 (cat pt-4 k "decrypt")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-5 k1-0 "wrap")) (send (enc k k1-0))))
  (label 562)
  (parent 214)
  (unrealized (2 3) (6 0) (6 3))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "wrap") (k k) (lk lk-0))
  (precedes ((1 1) (3 0)) ((2 4) (0 0)) ((3 1) (2 0)) ((4 1) (5 0))
    ((5 1) (6 0)) ((6 1) (3 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "wrap" "decrypt") (neq "wrap" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-decrypt-at-1 fact-set-decrypt-neq0 gen-st-set-wrap-0
    fact-set-wrap-neq0 trRl_make-at-0)
  (operation nonce-test (added-strand set-decrypt 2) k (3 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk-0 (cat pt-3 k "init"))
    (ch-msg lk-0 (cat pt-4 k "wrap")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "wrap")))
    ((load lk-0 (cat pt-4 k "wrap"))
      (stor lk-0 (cat pt-5 k "decrypt"))))
  (label 563)
  (parent 215)
  (unrealized (2 3) (3 0))
  (dead)
  (comment "empty cohort"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (3 0)) ((2 4) (0 0)) ((3 1) (2 0)) ((4 1) (5 0))
    ((5 1) (3 0)) ((6 1) (3 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation nonce-test (added-strand make 2) k (3 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk-0 (cat pt-3 k "init"))
    (ch-msg lk-0 (cat pt-4 k "wrap")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "wrap")))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init"))))
  (label 564)
  (parent 215)
  (seen 1298)
  (unrealized (2 3) (3 0))
  (comment "7 in cohort - 6 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "wrap") (k k) (lk lk-0))
  (precedes ((1 1) (3 0)) ((2 4) (0 0)) ((3 1) (2 0)) ((4 1) (5 0))
    ((5 1) (6 0)) ((6 1) (3 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "wrap" "decrypt") (neq "wrap" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule fact-set-decrypt-neq0 trRl_set-wrap-at-1 gen-st-set-wrap-0
    fact-set-wrap-neq0 trRl_make-at-0)
  (operation nonce-test (added-strand set-wrap 2) k (3 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk-0 (cat pt-3 k "init"))
    (ch-msg lk-0 (cat pt-4 k "wrap")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "wrap")))
    ((load lk-0 (cat pt-4 k "wrap")) (stor lk-0 (cat pt-5 k "wrap"))))
  (label 565)
  (parent 215)
  (unrealized (2 3) (3 0))
  (dead)
  (comment "empty cohort"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1-0) (lk lk-0))
  (precedes ((1 1) (3 0)) ((2 4) (0 0)) ((3 1) (2 0)) ((4 1) (5 0))
    ((5 1) (6 2)) ((6 4) (3 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap")
    (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0 gen-st-wrap-1)
  (operation nonce-test (added-strand wrap 5) k (3 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk-0 (cat pt-3 k "init"))
    (ch-msg lk-0 (cat pt-4 k "wrap")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "wrap")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-0 (cat pt-4 k "wrap"))
      (load lk-0 (cat pt-5 k1-0 "wrap")) (send (enc k k1-0))))
  (label 566)
  (parent 215)
  (seen 584)
  (unrealized (2 3) (6 0) (6 3))
  (comment "4 in cohort - 3 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk-0))
  (precedes ((1 1) (3 0)) ((2 4) (0 0)) ((3 1) (2 0)) ((4 1) (5 0))
    ((4 1) (6 2)) ((5 1) (3 0)) ((6 4) (3 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (3 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk-0 (cat pt-3 k "init"))
    (ch-msg lk-0 (cat pt-4 k "wrap")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "wrap")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-5 k1-0 "wrap")) (send (enc k k1-0))))
  (label 567)
  (parent 215)
  (seen 588)
  (unrealized (2 3) (6 0) (6 3))
  (comment "4 in cohort - 3 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk))
  (precedes ((1 1) (6 2)) ((2 4) (0 0)) ((3 1) (2 0)) ((4 1) (5 0))
    ((5 1) (3 0)) ((6 4) (3 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (3 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk-0 (cat pt-3 k "init"))
    (ch-msg lk-0 (cat pt-4 k "wrap")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "wrap")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-5 k1-0 "wrap")) (send (enc k k1-0))))
  (label 568)
  (parent 215)
  (unrealized (2 3) (6 0) (6 3))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (precedes ((1 1) (3 0)) ((2 4) (0 0)) ((3 1) (2 0)) ((4 4) (3 0))
    ((5 2) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 "wrap"))
  (operation encryption-test (displaced 4 6 make 3) (hash k) (5 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-0 (cat pt-4 k "init"))
      (load lk-0 (cat pt-2 k1-0 "wrap")) (send (enc k k1-0)))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init"))
      (send (hash k))))
  (label 569)
  (parent 216)
  (seen 1330)
  (unrealized (2 3) (4 3))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 k1-0 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk-0))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (3 0)) ((2 4) (0 0)) ((3 1) (2 0)) ((4 1) (5 2))
    ((5 4) (3 0)) ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation encryption-test (added-strand make 3) (hash k) (5 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-4 k1-0 "wrap")) (send (enc k k1-0)))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init"))
      (send (hash k))))
  (label 570)
  (parent 216)
  (seen 1331)
  (unrealized (2 3) (5 3))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk-0))
  (deflistener k)
  (precedes ((1 1) (3 0)) ((2 4) (0 0)) ((3 1) (2 0)) ((4 1) (5 2))
    ((5 4) (3 0)) ((6 1) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 "wrap"))
  (operation encryption-test (added-listener k) (hash k) (5 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-4 k1-0 "wrap")) (send (enc k k1-0)))
    ((recv k) (send k)))
  (label 571)
  (parent 216)
  (seen 607)
  (unrealized (2 3) (5 3) (6 0))
  (comment "3 in cohort - 2 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (precedes ((1 4) (0 0)) ((2 1) (1 0)) ((3 1) (2 0)) ((4 4) (2 0))
    ((5 2) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 "wrap"))
  (operation encryption-test (displaced 1 6 make 3) (hash k) (5 0))
  (traces ((recv k) (send k))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-4 k "init"))
      (load lk-0 (cat pt k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-0 (cat pt-4 k "init"))
      (load lk-0 (cat pt-2 k1-0 "wrap")) (send (enc k k1-0)))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init"))
      (send (hash k))))
  (label 572)
  (parent 217)
  (unrealized (1 3) (4 3))
  (dead)
  (comment "empty cohort"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 k1-0 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (5 2)) ((2 4) (0 0)) ((3 1) (2 0)) ((4 1) (3 0))
    ((5 4) (3 0)) ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation encryption-test (added-strand make 3) (hash k) (5 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-4 k1-0 "wrap")) (send (enc k k1-0)))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init"))
      (send (hash k))))
  (label 573)
  (parent 217)
  (unrealized (2 3) (5 3))
  (dead)
  (comment "empty cohort"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk))
  (deflistener k)
  (precedes ((1 1) (5 2)) ((2 4) (0 0)) ((3 1) (2 0)) ((4 1) (3 0))
    ((5 4) (3 0)) ((6 1) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 "wrap"))
  (operation encryption-test (added-listener k) (hash k) (5 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-4 k1-0 "wrap")) (send (enc k k1-0)))
    ((recv k) (send k)))
  (label 574)
  (parent 217)
  (seen 576)
  (unrealized (2 3) (5 3) (6 0))
  (comment "3 in cohort - 2 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old mesg) (pt pt-0 pt-1 pt-2 pval) (k k1 k1-0 skey) (lk locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk))
  (deflistener k)
  (precedes ((1 1) (5 0)) ((2 4) (0 0)) ((3 1) (2 0)) ((4 4) (3 0))
    ((5 1) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 "wrap"))
  (operation nonce-test (displaced 6 1 make 2) k (5 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((recv (hash k)) (recv (hash k1-0)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-2 k1-0 "wrap")) (send (enc k k1-0)))
    ((recv k) (send k)))
  (label 575)
  (parent 220)
  (unrealized (2 3) (4 3) (5 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (precedes ((1 1) (4 2)) ((2 4) (0 0)) ((3 1) (2 0)) ((4 4) (3 0))
    ((5 1) (4 0)) ((6 1) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation nonce-test (added-strand make 2) k (5 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((recv (hash k)) (recv (hash k1-0)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-2 k1-0 "wrap")) (send (enc k k1-0)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init"))))
  (label 576)
  (parent 220)
  (seen 1336)
  (unrealized (2 3) (4 3) (5 0))
  (comment "4 in cohort - 3 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1-0) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (precedes ((1 1) (2 2)) ((2 4) (0 0)) ((3 1) (2 0)) ((4 1) (5 2))
    ((5 4) (3 0)) ((6 1) (4 0)) ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap")
    (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 4 7 make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-5 k "init")) (stor lk-0 (cat pt-2 k "wrap")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-0 (cat pt-2 k "wrap"))
      (load lk-0 (cat pt-3 k1-0 "wrap")) (send (enc k k1-0)))
    ((load lk-0 (cat pt-4 old-0)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k))))
  (label 583)
  (parent 224)
  (seen 1341 1342)
  (unrealized (2 3) (5 3))
  (comment "4 in cohort - 2 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1-0) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (precedes ((1 4) (0 0)) ((2 1) (1 0)) ((3 1) (4 0)) ((4 1) (5 2))
    ((5 4) (2 0)) ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap")
    (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 1 7 make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-5 k "init"))
      (load lk-0 (cat pt k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-2 k "wrap")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk (cat pt-2 k "wrap"))
      (load lk (cat pt-3 k1-0 "wrap")) (send (enc k k1-0)))
    ((load lk-0 (cat pt-4 old-0)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k))))
  (label 584)
  (parent 224)
  (seen 1343 1344)
  (unrealized (1 3) (5 3))
  (comment "4 in cohort - 2 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (precedes ((1 1) (2 2)) ((2 4) (0 0)) ((3 1) (2 0)) ((4 1) (3 0))
    ((5 4) (3 0)) ((6 1) (4 0)) ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 4 7 make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-5 k "init")) (stor lk-0 (cat pt-2 k "wrap")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-0 (cat pt-5 k "init"))
      (load lk-0 (cat pt-3 k1-0 "wrap")) (send (enc k k1-0)))
    ((load lk-0 (cat pt-4 old-0)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k))))
  (label 587)
  (parent 225)
  (seen 1345 1346)
  (unrealized (2 3) (5 3))
  (comment "7 in cohort - 2 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (precedes ((1 4) (0 0)) ((2 1) (1 0)) ((3 1) (4 0)) ((3 1) (5 2))
    ((4 1) (2 0)) ((5 4) (2 0)) ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 1 7 make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-5 k "init"))
      (load lk-0 (cat pt k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-2 k "wrap")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt-3 k1-0 "wrap")) (send (enc k k1-0)))
    ((load lk-0 (cat pt-4 old-0)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k))))
  (label 588)
  (parent 225)
  (seen 1347 1348)
  (unrealized (1 3) (5 3))
  (comment "7 in cohort - 2 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1-0) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (precedes ((1 1) (2 2)) ((2 4) (0 0)) ((3 1) (2 0)) ((4 1) (5 2))
    ((5 4) (3 0)) ((6 1) (4 0)) ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k1 "wrap")
    (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 4 7 make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-5 k "init")) (stor lk-0 (cat pt-2 k "decrypt")))
    ((recv (hash k)) (recv (hash k1-0))
      (load lk-0 (cat pt-2 k "decrypt"))
      (load lk-0 (cat pt-3 k1-0 "wrap")) (send (enc k k1-0)))
    ((load lk-0 (cat pt-4 old-0)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k))))
  (label 596)
  (parent 229)
  (seen 1349)
  (unrealized (2 3) (5 3))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1-0) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (precedes ((1 4) (0 0)) ((2 1) (1 0)) ((3 1) (4 0)) ((4 1) (5 2))
    ((5 4) (2 0)) ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k1 "wrap")
    (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 1 7 make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-5 k "init"))
      (load lk-0 (cat pt k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk (cat pt-2 k "decrypt"))
      (load lk (cat pt-3 k1-0 "wrap")) (send (enc k k1-0)))
    ((load lk-0 (cat pt-4 old-0)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k))))
  (label 597)
  (parent 229)
  (seen 1350)
  (unrealized (1 3) (5 3))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (precedes ((1 1) (2 2)) ((2 4) (0 0)) ((3 1) (2 0)) ((4 1) (3 0))
    ((5 4) (3 0)) ((6 1) (4 0)) ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 4 7 make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-5 k "init")) (stor lk-0 (cat pt-2 k "decrypt")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-0 (cat pt-5 k "init"))
      (load lk-0 (cat pt-3 k1-0 "wrap")) (send (enc k k1-0)))
    ((load lk-0 (cat pt-4 old-0)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k))))
  (label 600)
  (parent 230)
  (seen 1351)
  (unrealized (2 3) (5 3))
  (comment "4 in cohort - 1 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (precedes ((1 4) (0 0)) ((2 1) (1 0)) ((3 1) (4 0)) ((3 1) (5 2))
    ((4 1) (2 0)) ((5 4) (2 0)) ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 1 7 make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-5 k "init"))
      (load lk-0 (cat pt k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt-3 k1-0 "wrap")) (send (enc k k1-0)))
    ((load lk-0 (cat pt-4 old-0)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k))))
  (label 601)
  (parent 230)
  (seen 1352)
  (unrealized (1 3) (5 3))
  (comment "4 in cohort - 1 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 cur mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur cur) (k k1-0) (lk lk-0))
  (precedes ((1 1) (2 2)) ((2 4) (0 0)) ((3 1) (2 0)) ((4 4) (3 0))
    ((5 1) (6 0)) ((5 2) (4 0)) ((6 1) (4 3)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 cur)
    (cat k1-0 "wrap"))
  (facts (neq cur "decrypt") (neq cur "wrap"))
  (rule invShearsRule fact-set-decrypt-neq0 trRl_set-wrap-at-1
    gen-st-set-wrap-0 fact-set-wrap-neq0 trRl_make-at-0)
  (operation channel-test (added-strand set-wrap 2)
    (ch-msg lk-0 (cat pt-2 k1-0 "wrap")) (4 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-0 (cat pt-4 k "init"))
      (load lk-0 (cat pt-2 k1-0 "wrap")) (send (enc k k1-0)))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-5 k1-0 cur))
      (stor lk-0 (cat pt-2 k1-0 "wrap"))))
  (label 604)
  (parent 231)
  (seen 1345 1353)
  (unrealized (2 3) (6 0))
  (comment "4 in cohort - 1 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 cur mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur cur) (k k1-0) (lk lk))
  (precedes ((1 4) (0 0)) ((2 1) (1 0)) ((3 1) (4 2)) ((3 1) (6 0))
    ((4 4) (2 0)) ((5 2) (4 0)) ((6 1) (4 3)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 cur)
    (cat k1-0 "wrap"))
  (facts (neq cur "decrypt") (neq cur "wrap"))
  (rule invShearsRule fact-set-decrypt-neq0 trRl_set-wrap-at-1
    gen-st-set-wrap-0 fact-set-wrap-neq0 trRl_make-at-0)
  (operation channel-test (added-strand set-wrap 2)
    (ch-msg lk (cat pt-2 k1-0 "wrap")) (4 3))
  (traces ((recv k) (send k))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-4 k "init"))
      (load lk-0 (cat pt k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt-2 k1-0 "wrap")) (send (enc k k1-0)))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init"))
      (send (hash k)))
    ((load lk (cat pt-5 k1-0 cur)) (stor lk (cat pt-2 k1-0 "wrap"))))
  (label 605)
  (parent 232)
  (seen 1347 1354)
  (unrealized (1 3) (6 0))
  (comment "4 in cohort - 1 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk-0))
  (deflistener k)
  (precedes ((1 1) (6 0)) ((2 4) (0 0)) ((3 1) (2 0)) ((4 1) (5 2))
    ((5 4) (3 0)) ((6 1) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 "wrap"))
  (operation nonce-test (displaced 7 1 make 2) k (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-4 k1-0 "wrap")) (send (enc k k1-0)))
    ((recv k) (send k)))
  (label 607)
  (parent 234)
  (unrealized (2 3) (5 3) (6 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk-0))
  (deflistener k)
  (precedes ((1 1) (2 2)) ((2 4) (0 0)) ((3 1) (2 0)) ((4 1) (6 0))
    ((5 4) (3 0)) ((6 1) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 "wrap"))
  (operation nonce-test (displaced 7 4 make 2) k (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-4 k1-0 "wrap")) (send (enc k k1-0)))
    ((recv k) (send k)))
  (label 608)
  (parent 234)
  (seen 1356)
  (unrealized (2 3) (5 3) (6 0))
  (comment "3 in cohort - 2 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (5 3)) ((3 1) (1 0)) ((3 1) (5 2))
    ((4 1) (0 0)) ((5 4) (0 0)) ((6 1) (3 0)) ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule invShearsRule shearsRule)
  (operation channel-test (displaced 7 2 set-wrap 2)
    (ch-msg lk-1 (cat pt-7 k1 "wrap")) (5 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk (cat pt-6 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-2 k "decrypt"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-5 old-1)) (stor lk (cat pt-6 k "init"))
      (send (hash k))))
  (label 610)
  (parent 235)
  (unrealized (0 0))
  (comment "10 in cohort - 10 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (5 3)) ((3 1) (2 0)) ((3 1) (5 2))
    ((4 1) (0 0)) ((5 4) (0 0)) ((6 1) (1 0)) ((6 1) (3 0))
    ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule invShearsRule shearsRule)
  (operation channel-test (displaced 7 2 set-wrap 2)
    (ch-msg lk-1 (cat pt-7 k1 "wrap")) (5 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk (cat pt-6 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-2 k "decrypt"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-5 old-1)) (stor lk (cat pt-6 k "init"))
      (send (hash k))))
  (label 611)
  (parent 235)
  (seen 610 1360 1361 1363 1364 1366 1367 1368 1369)
  (unrealized (0 0))
  (comment "20 in cohort - 0 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (5 3)) ((3 1) (2 0)) ((3 1) (5 2))
    ((4 1) (0 0)) ((5 4) (0 0)) ((6 1) (3 0)) ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule invShearsRule)
  (operation channel-test (displaced 7 2 set-wrap 2)
    (ch-msg lk-1 (cat pt-7 k1 "wrap")) (5 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk (cat pt-6 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-2 k "decrypt"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-5 old-1)) (stor lk (cat pt-6 k "init"))
      (send (hash k))))
  (label 612)
  (parent 235)
  (seen 610 1360 1361 1363 1364 1366 1367 1368 1369)
  (unrealized (0 0))
  (comment "20 in cohort - 0 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-1) (k k) (lk lk-0))
  (precedes ((1 1) (5 3)) ((2 1) (3 0)) ((2 1) (6 0)) ((3 1) (1 0))
    ((3 1) (5 2)) ((4 1) (0 0)) ((5 4) (0 0)) ((6 1) (1 0))
    ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule invShearsRule)
  (operation channel-test (displaced 7 1 set-wrap 2)
    (ch-msg lk-1 (cat pt-7 k1 "wrap")) (5 3))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-6 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((load lk-0 (cat pt-0 old)) (stor lk-0 (cat pt-1 k "init")))
    ((load lk-0 (cat pt-1 k "init")) (stor lk-0 (cat pt-2 k "decrypt")))
    ((load lk (cat pt-3 old-0)) (stor lk (cat pt-4 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt-2 k "decrypt"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-5 old-1)) (stor lk-0 (cat pt-6 k "init"))
      (send (hash k))))
  (label 614)
  (parent 236)
  (unrealized (0 0))
  (dead)
  (comment "empty cohort"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-1) (k k) (lk lk-0))
  (precedes ((1 1) (5 3)) ((2 1) (3 0)) ((3 1) (1 0)) ((3 1) (5 2))
    ((4 1) (0 0)) ((5 4) (0 0)) ((6 1) (1 0)) ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule invShearsRule)
  (operation channel-test (displaced 7 1 set-wrap 2)
    (ch-msg lk-1 (cat pt-7 k1 "wrap")) (5 3))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-6 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((load lk-0 (cat pt-0 old)) (stor lk-0 (cat pt-1 k "init")))
    ((load lk-0 (cat pt-1 k "init")) (stor lk-0 (cat pt-2 k "decrypt")))
    ((load lk (cat pt-3 old-0)) (stor lk (cat pt-4 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt-2 k "decrypt"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-5 old-1)) (stor lk-0 (cat pt-6 k "init"))
      (send (hash k))))
  (label 615)
  (parent 236)
  (unrealized (0 0))
  (dead)
  (comment "empty cohort"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (5 3)) ((3 1) (4 0)) ((4 1) (0 0))
    ((5 4) (0 0)) ((6 1) (1 0)) ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule invShearsRule)
  (operation channel-test (displaced 7 2 set-wrap 2)
    (ch-msg lk-1 (cat pt-7 k1 "wrap")) (5 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-6 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-5 old-1)) (stor lk (cat pt-6 k "init"))
      (send (hash k))))
  (label 619)
  (parent 241)
  (seen 289 1370 1375)
  (unrealized (0 0))
  (comment "24 in cohort - 13 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (5 3)) ((3 1) (1 0)) ((4 1) (0 0))
    ((5 4) (0 0)) ((6 1) (3 0)) ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule invShearsRule shearsRule)
  (operation channel-test (displaced 7 2 set-wrap 2)
    (ch-msg lk-1 (cat pt-7 k1 "wrap")) (5 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk (cat pt-6 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-6 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-5 old-1)) (stor lk (cat pt-6 k "init"))
      (send (hash k))))
  (label 621)
  (parent 244)
  (seen 1370)
  (unrealized (0 0))
  (comment "10 in cohort - 9 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (5 3)) ((3 1) (2 0)) ((4 1) (0 0))
    ((5 4) (0 0)) ((6 1) (1 0)) ((6 1) (3 0)) ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule invShearsRule shearsRule)
  (operation channel-test (displaced 7 2 set-wrap 2)
    (ch-msg lk-1 (cat pt-7 k1 "wrap")) (5 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk (cat pt-6 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-6 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-5 old-1)) (stor lk (cat pt-6 k "init"))
      (send (hash k))))
  (label 622)
  (parent 244)
  (seen 621 1370 1383 1385 1386 1388 1389 1390 1391)
  (unrealized (0 0))
  (comment "20 in cohort - 0 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-1))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk-1))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (5 2)) ((2 1) (3 0)) ((3 1) (0 0)) ((4 1) (0 0))
    ((5 4) (0 0)) ((6 1) (1 0)) ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation channel-test (displaced 7 1 set-wrap 2)
    (ch-msg lk-1 (cat pt-7 k1 "wrap")) (5 3))
  (traces ((recv k) (send k))
    ((load lk-1 (cat pt-6 k "init")) (stor lk-1 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk-1 (cat pt k "wrap"))
      (load lk-1 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init"))
      (send (hash k))))
  (label 626)
  (parent 249)
  (unrealized (0 0))
  (comment "11 in cohort - 11 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-1))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk-1))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (5 3)) ((2 1) (3 0)) ((3 1) (0 0)) ((4 1) (0 0))
    ((5 4) (0 0)) ((6 1) (1 0)) ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation channel-test (displaced 7 1 set-wrap 2)
    (ch-msg lk-1 (cat pt-7 k1 "wrap")) (5 3))
  (traces ((recv k) (send k))
    ((load lk-1 (cat pt-6 k "init")) (stor lk-1 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk-1 (cat pt-6 k "init"))
      (load lk-1 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init"))
      (send (hash k))))
  (label 629)
  (parent 253)
  (unrealized (0 0))
  (comment "11 in cohort - 11 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k skey) (lk locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "wrap") (k k) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (4 3)) ((2 1) (6 0)) ((3 1) (1 0))
    ((3 1) (4 2)) ((4 4) (0 0)) ((5 1) (3 0)) ((5 2) (4 0))
    ((6 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k "wrap"))
  (facts (neq "wrap" "decrypt") (neq "wrap" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule fact-set-decrypt-neq0 trRl_set-wrap-at-1 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation nonce-test (added-strand set-wrap 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk (cat pt-2 k "decrypt")) (ch-msg lk (cat pt-4 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk (cat pt-4 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-2 k "decrypt"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-3 old-0)) (stor lk (cat pt-4 k "init"))
      (send (hash k)))
    ((load lk (cat pt-1 k "wrap")) (stor lk (cat pt-5 k "wrap"))))
  (label 631)
  (parent 256)
  (unrealized (0 0))
  (dead)
  (comment "empty cohort"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (4 3)) ((3 1) (1 0)) ((3 1) (4 2))
    ((4 4) (0 0)) ((5 1) (3 0)) ((5 1) (6 2)) ((5 2) (4 0))
    ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k "wrap")
    (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk (cat pt-2 k "decrypt")) (ch-msg lk (cat pt-4 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk (cat pt-4 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-2 k "decrypt"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-3 old-0)) (stor lk (cat pt-4 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-4 k "init"))
      (load lk (cat pt-5 k1 "wrap")) (send (enc k k1))))
  (label 632)
  (parent 256)
  (unrealized (6 0) (6 3))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k skey) (lk locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "wrap") (k k) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (4 3)) ((2 1) (6 0)) ((3 1) (1 0))
    ((3 1) (4 2)) ((4 4) (0 0)) ((5 1) (3 0)) ((5 2) (4 0))
    ((6 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k "wrap"))
  (facts (neq "wrap" "decrypt") (neq "wrap" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-decrypt-at-1 fact-set-decrypt-neq0 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation nonce-test (added-strand set-decrypt 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk (cat pt-2 k "decrypt")) (ch-msg lk (cat pt-4 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk (cat pt-4 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-2 k "decrypt"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-3 old-0)) (stor lk (cat pt-4 k "init"))
      (send (hash k)))
    ((load lk (cat pt-1 k "wrap")) (stor lk (cat pt-5 k "decrypt"))))
  (label 633)
  (parent 256)
  (unrealized (0 0))
  (dead)
  (comment "empty cohort"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (4 3)) ((2 1) (6 2)) ((3 1) (1 0))
    ((3 1) (4 2)) ((4 4) (0 0)) ((5 1) (3 0)) ((5 2) (4 0))
    ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k "wrap")
    (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk (cat pt-2 k "decrypt")) (ch-msg lk (cat pt-4 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk (cat pt-4 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-2 k "decrypt"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-3 old-0)) (stor lk (cat pt-4 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-5 k1 "wrap")) (send (enc k k1))))
  (label 634)
  (parent 256)
  (unrealized (6 0) (6 3))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (4 3)) ((3 1) (1 0)) ((3 1) (4 2))
    ((3 1) (6 2)) ((4 4) (0 0)) ((5 1) (3 0)) ((5 2) (4 0))
    ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k "wrap")
    (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk (cat pt-2 k "decrypt")) (ch-msg lk (cat pt-4 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk (cat pt-4 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-2 k "decrypt"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-3 old-0)) (stor lk (cat pt-4 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-2 k "decrypt"))
      (load lk (cat pt-5 k1 "wrap")) (send (enc k k1))))
  (label 635)
  (parent 256)
  (unrealized (6 0) (6 3))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (6 2)) ((2 1) (4 3)) ((3 1) (1 0))
    ((3 1) (4 2)) ((4 4) (0 0)) ((5 1) (3 0)) ((5 2) (4 0))
    ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k "wrap")
    (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk (cat pt-2 k "decrypt")) (ch-msg lk (cat pt-4 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk (cat pt-4 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-2 k "decrypt"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-3 old-0)) (stor lk (cat pt-4 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-5 k1 "wrap")) (send (enc k k1))))
  (label 636)
  (parent 256)
  (unrealized (6 0) (6 3))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k skey)
    (lk locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand decrypt 4 (x k) (k k) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (4 3)) ((3 1) (1 0)) ((3 1) (4 2))
    ((3 1) (6 2)) ((4 4) (0 0)) ((5 1) (3 0)) ((5 2) (4 0))
    ((6 3) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation nonce-test (added-strand decrypt 4) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk (cat pt-2 k "decrypt")) (ch-msg lk (cat pt-4 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk (cat pt-4 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-2 k "decrypt"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-3 old-0)) (stor lk (cat pt-4 k "init"))
      (send (hash k)))
    ((recv (enc k k)) (recv (hash k)) (load lk (cat pt-2 k "decrypt"))
      (send k)))
  (label 637)
  (parent 256)
  (unrealized (6 0) (6 1))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1) (lk lk-0))
  (deflistener k)
  (precedes ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (0 0)) ((3 1) (4 0))
    ((3 1) (6 0)) ((4 1) (5 2)) ((5 4) (0 0)) ((6 1) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation nonce-test (displaced 7 3 make 2) k (6 0)
    (ch-msg lk (cat pt-0 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-4 k "decrypt"))
      (load lk-0 (cat pt-5 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k)))
  (label 652)
  (parent 267)
  (seen 513 1431)
  (unrealized (5 3) (6 0))
  (comment "7 in cohort - 4 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1) (lk lk-0))
  (deflistener k)
  (precedes ((1 1) (2 0)) ((2 1) (0 0)) ((3 1) (4 0)) ((4 1) (6 0))
    ((5 4) (0 0)) ((6 1) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation nonce-test (displaced 7 4 set-decrypt 2) k (6 0)
    (ch-msg lk-0 (cat pt-3 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-4 k "decrypt"))
      (load lk-0 (cat pt-5 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k)))
  (label 655)
  (parent 268)
  (seen 1431)
  (unrealized (5 3) (6 0))
  (comment "4 in cohort - 3 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k skey) (lk locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "wrap") (k k) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (4 3)) ((2 1) (6 0)) ((3 1) (1 0))
    ((4 4) (0 0)) ((5 1) (3 0)) ((5 2) (4 0)) ((6 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "wrap" "decrypt") (neq "wrap" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule fact-set-decrypt-neq0 trRl_set-wrap-at-1 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation nonce-test (added-strand set-wrap 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk (cat pt-2 k "decrypt")) (ch-msg lk (cat pt-4 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk (cat pt-4 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-4 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-3 old-0)) (stor lk (cat pt-4 k "init"))
      (send (hash k)))
    ((load lk (cat pt-1 k "wrap")) (stor lk (cat pt-5 k "wrap"))))
  (label 657)
  (parent 270)
  (unrealized (0 0))
  (dead)
  (comment "empty cohort"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (4 3)) ((3 1) (1 0)) ((4 4) (0 0))
    ((5 1) (3 0)) ((5 1) (6 2)) ((5 2) (4 0)) ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk (cat pt-2 k "decrypt")) (ch-msg lk (cat pt-4 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk (cat pt-4 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-4 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-3 old-0)) (stor lk (cat pt-4 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-4 k "init"))
      (load lk (cat pt-5 k1 "wrap")) (send (enc k k1))))
  (label 658)
  (parent 270)
  (unrealized (6 0) (6 3))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k skey) (lk locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "wrap") (k k) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (4 3)) ((2 1) (6 0)) ((3 1) (1 0))
    ((4 4) (0 0)) ((5 1) (3 0)) ((5 2) (4 0)) ((6 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "wrap" "decrypt") (neq "wrap" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-decrypt-at-1 fact-set-decrypt-neq0 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation nonce-test (added-strand set-decrypt 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk (cat pt-2 k "decrypt")) (ch-msg lk (cat pt-4 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk (cat pt-4 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-4 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-3 old-0)) (stor lk (cat pt-4 k "init"))
      (send (hash k)))
    ((load lk (cat pt-1 k "wrap")) (stor lk (cat pt-5 k "decrypt"))))
  (label 659)
  (parent 270)
  (unrealized (0 0))
  (dead)
  (comment "empty cohort"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (4 3)) ((2 1) (6 2)) ((3 1) (1 0))
    ((4 4) (0 0)) ((5 1) (3 0)) ((5 2) (4 0)) ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk (cat pt-2 k "decrypt")) (ch-msg lk (cat pt-4 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk (cat pt-4 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-4 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-3 old-0)) (stor lk (cat pt-4 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-5 k1 "wrap")) (send (enc k k1))))
  (label 660)
  (parent 270)
  (unrealized (6 0) (6 3))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (4 3)) ((3 1) (1 0)) ((3 1) (6 2))
    ((4 4) (0 0)) ((5 1) (3 0)) ((5 2) (4 0)) ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k "wrap")
    (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0 gen-st-wrap-1)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk (cat pt-2 k "decrypt")) (ch-msg lk (cat pt-4 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk (cat pt-4 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-4 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-3 old-0)) (stor lk (cat pt-4 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-2 k "decrypt"))
      (load lk (cat pt-5 k1 "wrap")) (send (enc k k1))))
  (label 661)
  (parent 270)
  (unrealized (6 0) (6 3))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (6 2)) ((2 1) (4 3)) ((3 1) (1 0))
    ((4 4) (0 0)) ((5 1) (3 0)) ((5 2) (4 0)) ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk (cat pt-2 k "decrypt")) (ch-msg lk (cat pt-4 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk (cat pt-4 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-4 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-3 old-0)) (stor lk (cat pt-4 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-5 k1 "wrap")) (send (enc k k1))))
  (label 662)
  (parent 270)
  (unrealized (6 0) (6 3))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k skey)
    (lk locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand decrypt 4 (x k) (k k) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (4 3)) ((3 1) (1 0)) ((3 1) (6 2))
    ((4 4) (0 0)) ((5 1) (3 0)) ((5 2) (4 0)) ((6 3) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-decrypt-0)
  (operation nonce-test (added-strand decrypt 4) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk (cat pt-2 k "decrypt")) (ch-msg lk (cat pt-4 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk (cat pt-4 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-4 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-3 old-0)) (stor lk (cat pt-4 k "init"))
      (send (hash k)))
    ((recv (enc k k)) (recv (hash k)) (load lk (cat pt-2 k "decrypt"))
      (send k)))
  (label 663)
  (parent 270)
  (unrealized (6 0) (6 1))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (deflistener k)
  (precedes ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (0 0)) ((3 1) (4 0))
    ((3 1) (6 0)) ((4 1) (0 0)) ((5 4) (0 0)) ((6 1) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation nonce-test (displaced 7 3 make 2) k (6 0)
    (ch-msg lk (cat pt-0 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-5 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k)))
  (label 678)
  (parent 277)
  (seen 537)
  (unrealized (5 3) (6 0))
  (comment "5 in cohort - 3 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "decrypt") (k k) (lk lk))
  (precedes ((1 1) (4 2)) ((2 1) (3 0)) ((3 1) (6 0)) ((4 4) (0 0))
    ((5 1) (1 0)) ((5 2) (4 0)) ((6 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k "wrap"))
  (facts (neq "decrypt" "decrypt") (neq "decrypt" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule fact-set-decrypt-neq0 trRl_set-wrap-at-1 gen-st-set-wrap-0
    fact-set-wrap-neq0 trRl_make-at-0)
  (operation nonce-test (added-strand set-wrap 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-1 k "init")) (ch-msg lk (cat pt-2 k "decrypt"))
    (ch-msg lk-0 (cat pt k "wrap")) (ch-msg lk-0 (cat pt-4 k "init")))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt k "wrap"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init"))
      (send (hash k)))
    ((load lk (cat pt-2 k "decrypt")) (stor lk (cat pt-5 k "wrap"))))
  (label 683)
  (parent 281)
  (unrealized (0 0))
  (dead)
  (comment "empty cohort"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "wrap") (k k) (lk lk-0))
  (precedes ((1 1) (4 2)) ((1 1) (6 0)) ((2 1) (3 0)) ((3 1) (0 0))
    ((4 4) (0 0)) ((5 1) (1 0)) ((5 2) (4 0)) ((6 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "wrap" "decrypt") (neq "wrap" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule fact-set-decrypt-neq0 trRl_set-wrap-at-1 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation nonce-test (added-strand set-wrap 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-1 k "init")) (ch-msg lk (cat pt-2 k "decrypt"))
    (ch-msg lk-0 (cat pt k "wrap")) (ch-msg lk-0 (cat pt-4 k "init")))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt k "wrap"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt k "wrap")) (stor lk-0 (cat pt-5 k "wrap"))))
  (label 684)
  (parent 281)
  (unrealized (0 0))
  (dead)
  (comment "empty cohort"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (precedes ((1 1) (4 2)) ((2 1) (3 0)) ((3 1) (0 0)) ((4 4) (0 0))
    ((5 1) (1 0)) ((5 1) (6 2)) ((5 2) (4 0)) ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-1 k "init")) (ch-msg lk (cat pt-2 k "decrypt"))
    (ch-msg lk-0 (cat pt k "wrap")) (ch-msg lk-0 (cat pt-4 k "init")))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt k "wrap"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-4 k "init"))
      (load lk-0 (cat pt-5 k1 "wrap")) (send (enc k k1))))
  (label 685)
  (parent 281)
  (unrealized (6 0) (6 3))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "decrypt") (k k) (lk lk))
  (precedes ((1 1) (4 2)) ((2 1) (3 0)) ((3 1) (6 0)) ((4 4) (0 0))
    ((5 1) (1 0)) ((5 2) (4 0)) ((6 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k "wrap"))
  (facts (neq "decrypt" "decrypt") (neq "decrypt" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-decrypt-at-1 fact-set-decrypt-neq0 gen-st-set-wrap-0
    fact-set-wrap-neq0 trRl_make-at-0)
  (operation nonce-test (added-strand set-decrypt 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-1 k "init")) (ch-msg lk (cat pt-2 k "decrypt"))
    (ch-msg lk-0 (cat pt k "wrap")) (ch-msg lk-0 (cat pt-4 k "init")))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt k "wrap"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init"))
      (send (hash k)))
    ((load lk (cat pt-2 k "decrypt")) (stor lk (cat pt-5 k "decrypt"))))
  (label 686)
  (parent 281)
  (unrealized (0 0))
  (dead)
  (comment "empty cohort"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "wrap") (k k) (lk lk-0))
  (precedes ((1 1) (4 2)) ((1 1) (6 0)) ((2 1) (3 0)) ((3 1) (0 0))
    ((4 4) (0 0)) ((5 1) (1 0)) ((5 2) (4 0)) ((6 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "wrap" "decrypt") (neq "wrap" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-decrypt-at-1 fact-set-decrypt-neq0 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation nonce-test (added-strand set-decrypt 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-1 k "init")) (ch-msg lk (cat pt-2 k "decrypt"))
    (ch-msg lk-0 (cat pt k "wrap")) (ch-msg lk-0 (cat pt-4 k "init")))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt k "wrap"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt k "wrap")) (stor lk-0 (cat pt-5 k "decrypt"))))
  (label 687)
  (parent 281)
  (unrealized (0 0))
  (dead)
  (comment "empty cohort"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (4 2)) ((2 1) (3 0)) ((3 1) (6 2)) ((4 4) (0 0))
    ((5 1) (1 0)) ((5 2) (4 0)) ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k "wrap")
    (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0 gen-st-wrap-1)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-1 k "init")) (ch-msg lk (cat pt-2 k "decrypt"))
    (ch-msg lk-0 (cat pt k "wrap")) (ch-msg lk-0 (cat pt-4 k "init")))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt k "wrap"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-2 k "decrypt"))
      (load lk (cat pt-5 k1 "wrap")) (send (enc k k1))))
  (label 688)
  (parent 281)
  (unrealized (6 0) (6 3))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk-0))
  (precedes ((1 1) (4 2)) ((1 1) (6 2)) ((2 1) (3 0)) ((3 1) (0 0))
    ((4 4) (0 0)) ((5 1) (1 0)) ((5 2) (4 0)) ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-1 k "init")) (ch-msg lk (cat pt-2 k "decrypt"))
    (ch-msg lk-0 (cat pt k "wrap")) (ch-msg lk-0 (cat pt-4 k "init")))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt k "wrap"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt k "wrap"))
      (load lk-0 (cat pt-5 k1 "wrap")) (send (enc k k1))))
  (label 689)
  (parent 281)
  (unrealized (6 0) (6 3))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (4 2)) ((2 1) (3 0)) ((2 1) (6 2)) ((3 1) (0 0))
    ((4 4) (0 0)) ((5 1) (1 0)) ((5 2) (4 0)) ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-1 k "init")) (ch-msg lk (cat pt-2 k "decrypt"))
    (ch-msg lk-0 (cat pt k "wrap")) (ch-msg lk-0 (cat pt-4 k "init")))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt k "wrap"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt-5 k1 "wrap")) (send (enc k k1))))
  (label 690)
  (parent 281)
  (unrealized (6 0) (6 3))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand decrypt 4 (x k) (k k) (lk lk))
  (precedes ((1 1) (4 2)) ((2 1) (3 0)) ((3 1) (6 2)) ((4 4) (0 0))
    ((5 1) (1 0)) ((5 2) (4 0)) ((6 3) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-decrypt-0)
  (operation nonce-test (added-strand decrypt 4) k (0 0) (enc k k)
    (ch-msg lk (cat pt-1 k "init")) (ch-msg lk (cat pt-2 k "decrypt"))
    (ch-msg lk-0 (cat pt k "wrap")) (ch-msg lk-0 (cat pt-4 k "init")))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt k "wrap"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init"))
      (send (hash k)))
    ((recv (enc k k)) (recv (hash k)) (load lk (cat pt-2 k "decrypt"))
      (send k)))
  (label 691)
  (parent 281)
  (unrealized (6 0) (6 1))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (precedes ((1 1) (2 0)) ((2 1) (6 0)) ((3 1) (4 0)) ((4 1) (0 0))
    ((5 4) (0 0)) ((6 1) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation nonce-test (displaced 7 2 set-wrap 2) k (6 0)
    (ch-msg lk (cat pt-0 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-5 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k)))
  (label 706)
  (parent 285)
  (unrealized (5 3) (6 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (precedes ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (5 2)) ((3 1) (4 0))
    ((3 1) (6 0)) ((4 1) (0 0)) ((5 4) (0 0)) ((6 1) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation nonce-test (displaced 7 3 make 2) k (6 0)
    (ch-msg lk (cat pt-0 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-5 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k)))
  (label 708)
  (parent 285)
  (seen 410 1475)
  (unrealized (5 3) (6 0))
  (comment "7 in cohort - 3 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "decrypt") (k k) (lk lk))
  (precedes ((1 1) (4 3)) ((2 1) (3 0)) ((3 1) (6 0)) ((4 4) (0 0))
    ((5 1) (1 0)) ((5 2) (4 0)) ((6 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k "wrap"))
  (facts (neq "decrypt" "decrypt") (neq "decrypt" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule fact-set-decrypt-neq0 trRl_set-wrap-at-1 gen-st-set-wrap-0
    fact-set-wrap-neq0 trRl_make-at-0)
  (operation nonce-test (added-strand set-wrap 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-1 k "init")) (ch-msg lk (cat pt-2 k "decrypt"))
    (ch-msg lk-0 (cat pt k "wrap")) (ch-msg lk-0 (cat pt-4 k "init")))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt-4 k "init"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init"))
      (send (hash k)))
    ((load lk (cat pt-2 k "decrypt")) (stor lk (cat pt-5 k "wrap"))))
  (label 713)
  (parent 289)
  (unrealized (0 0))
  (dead)
  (comment "empty cohort"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "wrap") (k k) (lk lk-0))
  (precedes ((1 1) (4 3)) ((1 1) (6 0)) ((2 1) (3 0)) ((3 1) (0 0))
    ((4 4) (0 0)) ((5 1) (1 0)) ((5 2) (4 0)) ((6 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "wrap" "decrypt") (neq "wrap" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule fact-set-decrypt-neq0 trRl_set-wrap-at-1 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation nonce-test (added-strand set-wrap 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-1 k "init")) (ch-msg lk (cat pt-2 k "decrypt"))
    (ch-msg lk-0 (cat pt k "wrap")) (ch-msg lk-0 (cat pt-4 k "init")))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt-4 k "init"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt k "wrap")) (stor lk-0 (cat pt-5 k "wrap"))))
  (label 714)
  (parent 289)
  (unrealized (0 0))
  (dead)
  (comment "empty cohort"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (precedes ((1 1) (4 3)) ((2 1) (3 0)) ((3 1) (0 0)) ((4 4) (0 0))
    ((5 1) (1 0)) ((5 1) (6 2)) ((5 2) (4 0)) ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-1 k "init")) (ch-msg lk (cat pt-2 k "decrypt"))
    (ch-msg lk-0 (cat pt k "wrap")) (ch-msg lk-0 (cat pt-4 k "init")))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt-4 k "init"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-4 k "init"))
      (load lk-0 (cat pt-5 k1 "wrap")) (send (enc k k1))))
  (label 715)
  (parent 289)
  (unrealized (6 0) (6 3))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "decrypt") (k k) (lk lk))
  (precedes ((1 1) (4 3)) ((2 1) (3 0)) ((3 1) (6 0)) ((4 4) (0 0))
    ((5 1) (1 0)) ((5 2) (4 0)) ((6 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k "wrap"))
  (facts (neq "decrypt" "decrypt") (neq "decrypt" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-decrypt-at-1 fact-set-decrypt-neq0 gen-st-set-wrap-0
    fact-set-wrap-neq0 trRl_make-at-0)
  (operation nonce-test (added-strand set-decrypt 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-1 k "init")) (ch-msg lk (cat pt-2 k "decrypt"))
    (ch-msg lk-0 (cat pt k "wrap")) (ch-msg lk-0 (cat pt-4 k "init")))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt-4 k "init"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init"))
      (send (hash k)))
    ((load lk (cat pt-2 k "decrypt")) (stor lk (cat pt-5 k "decrypt"))))
  (label 716)
  (parent 289)
  (unrealized (0 0))
  (dead)
  (comment "empty cohort"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "wrap") (k k) (lk lk-0))
  (precedes ((1 1) (4 3)) ((1 1) (6 0)) ((2 1) (3 0)) ((3 1) (0 0))
    ((4 4) (0 0)) ((5 1) (1 0)) ((5 2) (4 0)) ((6 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "wrap" "decrypt") (neq "wrap" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-decrypt-at-1 fact-set-decrypt-neq0 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation nonce-test (added-strand set-decrypt 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-1 k "init")) (ch-msg lk (cat pt-2 k "decrypt"))
    (ch-msg lk-0 (cat pt k "wrap")) (ch-msg lk-0 (cat pt-4 k "init")))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt-4 k "init"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt k "wrap")) (stor lk-0 (cat pt-5 k "decrypt"))))
  (label 717)
  (parent 289)
  (unrealized (0 0))
  (dead)
  (comment "empty cohort"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (4 3)) ((2 1) (3 0)) ((3 1) (6 2)) ((4 4) (0 0))
    ((5 1) (1 0)) ((5 2) (4 0)) ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k "wrap")
    (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0 gen-st-wrap-1)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-1 k "init")) (ch-msg lk (cat pt-2 k "decrypt"))
    (ch-msg lk-0 (cat pt k "wrap")) (ch-msg lk-0 (cat pt-4 k "init")))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt-4 k "init"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-2 k "decrypt"))
      (load lk (cat pt-5 k1 "wrap")) (send (enc k k1))))
  (label 718)
  (parent 289)
  (unrealized (6 0) (6 3))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk-0))
  (precedes ((1 1) (4 3)) ((1 1) (6 2)) ((2 1) (3 0)) ((3 1) (0 0))
    ((4 4) (0 0)) ((5 1) (1 0)) ((5 2) (4 0)) ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-1 k "init")) (ch-msg lk (cat pt-2 k "decrypt"))
    (ch-msg lk-0 (cat pt k "wrap")) (ch-msg lk-0 (cat pt-4 k "init")))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt-4 k "init"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt k "wrap"))
      (load lk-0 (cat pt-5 k1 "wrap")) (send (enc k k1))))
  (label 719)
  (parent 289)
  (unrealized (6 0) (6 3))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (4 3)) ((2 1) (3 0)) ((2 1) (6 2)) ((3 1) (0 0))
    ((4 4) (0 0)) ((5 1) (1 0)) ((5 2) (4 0)) ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-1 k "init")) (ch-msg lk (cat pt-2 k "decrypt"))
    (ch-msg lk-0 (cat pt k "wrap")) (ch-msg lk-0 (cat pt-4 k "init")))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt-4 k "init"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt-5 k1 "wrap")) (send (enc k k1))))
  (label 720)
  (parent 289)
  (unrealized (6 0) (6 3))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand decrypt 4 (x k) (k k) (lk lk))
  (precedes ((1 1) (4 3)) ((2 1) (3 0)) ((3 1) (6 2)) ((4 4) (0 0))
    ((5 1) (1 0)) ((5 2) (4 0)) ((6 3) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-decrypt-0)
  (operation nonce-test (added-strand decrypt 4) k (0 0) (enc k k)
    (ch-msg lk (cat pt-1 k "init")) (ch-msg lk (cat pt-2 k "decrypt"))
    (ch-msg lk-0 (cat pt k "wrap")) (ch-msg lk-0 (cat pt-4 k "init")))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt-4 k "init"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init"))
      (send (hash k)))
    ((recv (enc k k)) (recv (hash k)) (load lk (cat pt-2 k "decrypt"))
      (send k)))
  (label 721)
  (parent 289)
  (unrealized (6 0) (6 1))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (precedes ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (0 0)) ((3 1) (4 0))
    ((3 1) (6 0)) ((4 1) (0 0)) ((5 4) (0 0)) ((6 1) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation nonce-test (displaced 7 3 make 2) k (6 0)
    (ch-msg lk (cat pt-0 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-5 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k)))
  (label 737)
  (parent 293)
  (seen 466)
  (unrealized (5 3) (6 0))
  (comment "5 in cohort - 3 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand make 2 (old old-1) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (3 3)) ((3 4) (0 0)) ((4 1) (1 0))
    ((4 2) (3 0)) ((5 1) (6 0)) ((6 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-wrap-at-1 trRl_make-at-0)
  (operation nonce-test (added-strand set-wrap 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk (cat pt-3 k "init")) (ch-msg lk-0 (cat pt-5 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-4 old-1)) (stor lk-0 (cat pt-5 k "init")))
    ((load lk-0 (cat pt-5 k "init")) (stor lk-0 (cat pt-6 k "wrap"))))
  (label 741)
  (parent 297)
  (seen 304 1383 1508)
  (unrealized (0 0))
  (comment "23 in cohort - 11 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand make 2 (old old-1) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (3 3)) ((3 4) (0 0)) ((4 1) (6 0))
    ((4 2) (3 0)) ((5 1) (0 0)) ((6 1) (1 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule invShearsRule shearsRule trRl_set-wrap-at-1 trRl_make-at-0)
  (operation nonce-test (added-strand set-wrap 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk (cat pt-3 k "init")) (ch-msg lk-0 (cat pt-5 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-4 old-1)) (stor lk-0 (cat pt-5 k "init")))
    ((load lk (cat pt-3 k "init")) (stor lk (cat pt-6 k "wrap"))))
  (label 742)
  (parent 297)
  (seen 1375 1508)
  (unrealized (0 0))
  (comment "9 in cohort - 7 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand make 2 (old old-1) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "wrap") (k k) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (3 3)) ((2 1) (6 0)) ((3 4) (0 0))
    ((4 1) (1 0)) ((4 2) (3 0)) ((5 1) (0 0)) ((6 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "wrap" "decrypt") (neq "wrap" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule fact-set-decrypt-neq0 trRl_set-wrap-at-1 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation nonce-test (added-strand set-wrap 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk (cat pt-3 k "init")) (ch-msg lk-0 (cat pt-5 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-4 old-1)) (stor lk-0 (cat pt-5 k "init")))
    ((load lk (cat pt-1 k "wrap")) (stor lk (cat pt-6 k "wrap"))))
  (label 743)
  (parent 297)
  (unrealized (0 0))
  (dead)
  (comment "empty cohort"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand make 2 (old old-1) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (3 3)) ((3 4) (0 0)) ((4 1) (1 0))
    ((4 2) (3 0)) ((5 1) (6 2)) ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk (cat pt-3 k "init")) (ch-msg lk-0 (cat pt-5 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-4 old-1)) (stor lk-0 (cat pt-5 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-5 k "init"))
      (load lk-0 (cat pt-6 k1 "wrap")) (send (enc k k1))))
  (label 744)
  (parent 297)
  (unrealized (6 0) (6 3))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand make 2 (old old-1) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "wrap") (k k) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (3 3)) ((2 1) (6 0)) ((3 4) (0 0))
    ((4 1) (1 0)) ((4 2) (3 0)) ((5 1) (0 0)) ((6 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "wrap" "decrypt") (neq "wrap" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-decrypt-at-1 fact-set-decrypt-neq0 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation nonce-test (added-strand set-decrypt 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk (cat pt-3 k "init")) (ch-msg lk-0 (cat pt-5 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-4 old-1)) (stor lk-0 (cat pt-5 k "init")))
    ((load lk (cat pt-1 k "wrap")) (stor lk (cat pt-6 k "decrypt"))))
  (label 745)
  (parent 297)
  (unrealized (0 0))
  (dead)
  (comment "empty cohort"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand make 2 (old old-1) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (3 3)) ((3 4) (0 0)) ((4 1) (1 0))
    ((4 1) (6 2)) ((4 2) (3 0)) ((5 1) (0 0)) ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk (cat pt-3 k "init")) (ch-msg lk-0 (cat pt-5 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-4 old-1)) (stor lk-0 (cat pt-5 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-6 k1 "wrap")) (send (enc k k1))))
  (label 746)
  (parent 297)
  (seen 756)
  (unrealized (6 0) (6 3))
  (comment "4 in cohort - 3 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand make 2 (old old-1) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (3 3)) ((2 1) (6 2)) ((3 4) (0 0))
    ((4 1) (1 0)) ((4 2) (3 0)) ((5 1) (0 0)) ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk (cat pt-3 k "init")) (ch-msg lk-0 (cat pt-5 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-4 old-1)) (stor lk-0 (cat pt-5 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-6 k1 "wrap")) (send (enc k k1))))
  (label 747)
  (parent 297)
  (seen 760)
  (unrealized (6 0) (6 3))
  (comment "5 in cohort - 4 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand make 2 (old old-1) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (6 2)) ((2 1) (3 3)) ((3 4) (0 0))
    ((4 1) (1 0)) ((4 2) (3 0)) ((5 1) (0 0)) ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk (cat pt-3 k "init")) (ch-msg lk-0 (cat pt-5 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-4 old-1)) (stor lk-0 (cat pt-5 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-6 k1 "wrap")) (send (enc k k1))))
  (label 748)
  (parent 297)
  (seen 764)
  (unrealized (6 0) (6 3))
  (comment "5 in cohort - 4 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k skey) (lk locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand set-wrap 2 (cur "wrap") (k k) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (3 3)) ((2 1) (6 0)) ((3 4) (0 0))
    ((4 1) (5 0)) ((4 2) (3 0)) ((5 1) (1 0)) ((6 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "wrap" "decrypt") (neq "wrap" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule fact-set-decrypt-neq0 trRl_set-wrap-at-1 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation nonce-test (added-strand set-wrap 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk (cat pt-3 k "init")) (ch-msg lk (cat pt-4 k "wrap")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk (cat pt-3 k "init")) (stor lk (cat pt-4 k "wrap")))
    ((load lk (cat pt-1 k "wrap")) (stor lk (cat pt-5 k "wrap"))))
  (label 749)
  (parent 298)
  (unrealized (0 0))
  (dead)
  (comment "empty cohort"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k skey) (lk locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "wrap") (k k) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (3 3)) ((2 1) (6 0)) ((3 4) (0 0))
    ((4 1) (5 0)) ((4 2) (3 0)) ((5 1) (1 0)) ((6 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "wrap" "decrypt") (neq "wrap" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-decrypt-at-1 fact-set-decrypt-neq0 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation nonce-test (added-strand set-decrypt 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk (cat pt-3 k "init")) (ch-msg lk (cat pt-4 k "wrap")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk (cat pt-3 k "init")) (stor lk (cat pt-4 k "wrap")))
    ((load lk (cat pt-1 k "wrap")) (stor lk (cat pt-5 k "decrypt"))))
  (label 750)
  (parent 298)
  (unrealized (0 0))
  (dead)
  (comment "empty cohort"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (3 3)) ((3 4) (0 0)) ((4 1) (5 0))
    ((4 1) (6 2)) ((4 2) (3 0)) ((5 1) (1 0)) ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk (cat pt-3 k "init")) (ch-msg lk (cat pt-4 k "wrap")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk (cat pt-3 k "init")) (stor lk (cat pt-4 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-5 k1 "wrap")) (send (enc k k1))))
  (label 751)
  (parent 298)
  (unrealized (6 0) (6 3))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (3 3)) ((3 4) (0 0)) ((4 1) (5 0))
    ((4 2) (3 0)) ((5 1) (1 0)) ((5 1) (6 2)) ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk (cat pt-3 k "init")) (ch-msg lk (cat pt-4 k "wrap")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk (cat pt-3 k "init")) (stor lk (cat pt-4 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-4 k "wrap"))
      (load lk (cat pt-5 k1 "wrap")) (send (enc k k1))))
  (label 752)
  (parent 298)
  (unrealized (6 0) (6 3))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (3 3)) ((2 1) (6 2)) ((3 4) (0 0))
    ((4 1) (5 0)) ((4 2) (3 0)) ((5 1) (1 0)) ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk (cat pt-3 k "init")) (ch-msg lk (cat pt-4 k "wrap")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk (cat pt-3 k "init")) (stor lk (cat pt-4 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-5 k1 "wrap")) (send (enc k k1))))
  (label 753)
  (parent 298)
  (unrealized (6 0) (6 3))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (6 2)) ((2 1) (3 3)) ((3 4) (0 0))
    ((4 1) (5 0)) ((4 2) (3 0)) ((5 1) (1 0)) ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk (cat pt-3 k "init")) (ch-msg lk (cat pt-4 k "wrap")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk (cat pt-3 k "init")) (stor lk (cat pt-4 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-5 k1 "wrap")) (send (enc k k1))))
  (label 754)
  (parent 298)
  (unrealized (6 0) (6 3))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k k1 skey)
    (lk locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (3 3)) ((3 4) (0 0)) ((4 1) (1 0))
    ((4 2) (3 0)) ((4 2) (5 0)) ((5 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 6 4 make 3) (hash k) (5 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1))))
  (label 755)
  (parent 300)
  (seen 128)
  (unrealized (5 3))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (3 3)) ((3 4) (0 0)) ((4 1) (1 0))
    ((4 1) (5 2)) ((4 2) (3 0)) ((5 4) (0 0)) ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation encryption-test (added-strand make 3) (hash k) (5 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-5 old-1)) (stor lk-0 (cat pt-6 k "init"))
      (send (hash k))))
  (label 756)
  (parent 300)
  (seen 1553)
  (unrealized (5 3))
  (comment "3 in cohort - 2 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k k1 skey)
    (lk locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (precedes ((1 1) (2 0)) ((2 1) (3 3)) ((3 4) (0 0)) ((4 1) (1 0))
    ((4 1) (5 2)) ((4 2) (3 0)) ((5 4) (0 0)) ((6 1) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (added-listener k) (hash k) (5 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k)))
  (label 757)
  (parent 300)
  (unrealized (5 3) (6 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k k1 skey)
    (lk locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (3 3)) ((2 1) (5 2)) ((3 4) (0 0))
    ((4 1) (1 0)) ((4 2) (3 0)) ((4 2) (5 0)) ((5 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 6 4 make 3) (hash k) (5 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1))))
  (label 758)
  (parent 301)
  (seen 1058 1557)
  (unrealized (5 3))
  (comment "11 in cohort - 2 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k k1 skey)
    (lk locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (precedes ((1 1) (2 3)) ((1 1) (4 2)) ((2 4) (0 0)) ((3 1) (5 0))
    ((3 2) (2 0)) ((4 4) (0 0)) ((5 1) (1 0)) ((5 2) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 1 6 make 3) (hash k) (5 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt-4 k "init")) (stor lk (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt k "wrap"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((load lk (cat pt-3 old-0)) (stor lk (cat pt-4 k "init"))
      (send (hash k))))
  (label 759)
  (parent 301)
  (seen 1558 1560)
  (unrealized (4 3))
  (comment "11 in cohort - 3 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (3 3)) ((2 1) (5 2)) ((3 4) (0 0))
    ((4 1) (1 0)) ((4 2) (3 0)) ((5 4) (0 0)) ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation encryption-test (added-strand make 3) (hash k) (5 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-5 old-1)) (stor lk-0 (cat pt-6 k "init"))
      (send (hash k))))
  (label 760)
  (parent 301)
  (seen 1561 1563)
  (unrealized (5 3))
  (comment "11 in cohort - 3 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k k1 skey)
    (lk locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (precedes ((1 1) (2 0)) ((2 1) (3 3)) ((2 1) (5 2)) ((3 4) (0 0))
    ((4 1) (1 0)) ((4 2) (3 0)) ((5 4) (0 0)) ((6 1) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (added-listener k) (hash k) (5 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k)))
  (label 761)
  (parent 301)
  (unrealized (5 3) (6 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k k1 skey)
    (lk locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (5 2)) ((2 1) (3 3)) ((3 4) (0 0))
    ((4 1) (1 0)) ((4 2) (3 0)) ((4 2) (5 0)) ((5 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 6 4 make 3) (hash k) (5 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1))))
  (label 762)
  (parent 302)
  (seen 1162 1568)
  (unrealized (5 3))
  (comment "12 in cohort - 3 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k k1 skey)
    (lk locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (precedes ((1 1) (2 3)) ((2 4) (0 0)) ((3 1) (5 0)) ((3 2) (2 0))
    ((4 4) (0 0)) ((5 1) (1 0)) ((5 2) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 1 6 make 3) (hash k) (5 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt-4 k "init")) (stor lk (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-4 k "init"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((load lk (cat pt-3 old-0)) (stor lk (cat pt-4 k "init"))
      (send (hash k))))
  (label 763)
  (parent 302)
  (seen 1569 1572)
  (unrealized (4 3))
  (comment "12 in cohort - 4 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((1 1) (5 2)) ((2 1) (3 3)) ((3 4) (0 0))
    ((4 1) (1 0)) ((4 2) (3 0)) ((5 4) (0 0)) ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation encryption-test (added-strand make 3) (hash k) (5 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-5 old-1)) (stor lk-0 (cat pt-6 k "init"))
      (send (hash k))))
  (label 764)
  (parent 302)
  (seen 1573 1576)
  (unrealized (5 3))
  (comment "12 in cohort - 4 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k k1 skey)
    (lk locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (precedes ((1 1) (2 0)) ((1 1) (5 2)) ((2 1) (3 3)) ((3 4) (0 0))
    ((4 1) (1 0)) ((4 2) (3 0)) ((5 4) (0 0)) ((6 1) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (added-listener k) (hash k) (5 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k)))
  (label 765)
  (parent 302)
  (unrealized (5 3) (6 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k1) (lk lk-0))
  (defstrand make 2 (old old-1) (k k1) (lk lk-0))
  (precedes ((1 1) (0 0)) ((2 4) (0 0)) ((3 1) (5 0)) ((3 2) (2 0))
    ((4 1) (2 3)) ((5 1) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation generalization deleted (2 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-1 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-4 k1 "init")) (stor lk-0 (cat pt-1 k1 "wrap")))
    ((load lk-0 (cat pt-5 old-1)) (stor lk-0 (cat pt-4 k1 "init"))))
  (label 766)
  (parent 303)
  (seen 96)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "wrap") (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (0 0)) ((3 4) (0 0)) ((4 1) (5 0))
    ((4 2) (3 0)) ((5 1) (3 3)) ((5 1) (6 0)) ((6 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "wrap" "decrypt") (neq "wrap" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-decrypt-at-1 fact-set-decrypt-neq0 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation nonce-test (added-strand set-decrypt 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-2 k "wrap")) (ch-msg lk-0 (cat pt-4 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt-4 k "init"))
      (load lk-0 (cat pt-2 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt-2 k "wrap")))
    ((load lk-0 (cat pt-2 k "wrap"))
      (stor lk-0 (cat pt-5 k "decrypt"))))
  (label 767)
  (parent 304)
  (unrealized (0 0))
  (dead)
  (comment "empty cohort"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "wrap") (k k) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (6 0)) ((3 4) (0 0)) ((4 1) (5 0))
    ((4 2) (3 0)) ((5 1) (3 3)) ((6 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "wrap" "decrypt") (neq "wrap" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule fact-set-decrypt-neq0 trRl_set-wrap-at-1 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation nonce-test (added-strand set-wrap 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-2 k "wrap")) (ch-msg lk-0 (cat pt-4 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt-4 k "init"))
      (load lk-0 (cat pt-2 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt-2 k "wrap")))
    ((load lk (cat pt-1 k "wrap")) (stor lk (cat pt-5 k "wrap"))))
  (label 768)
  (parent 304)
  (unrealized (0 0))
  (dead)
  (comment "empty cohort"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((2 1) (0 0)) ((3 4) (0 0)) ((4 1) (5 0))
    ((4 2) (3 0)) ((5 1) (3 3)) ((6 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation nonce-test (added-strand make 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-2 k "wrap")) (ch-msg lk-0 (cat pt-4 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt-4 k "init"))
      (load lk-0 (cat pt-2 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt-2 k "wrap")))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init"))))
  (label 769)
  (parent 304)
  (seen 1406)
  (unrealized (0 0))
  (comment "10 in cohort - 9 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "wrap") (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (0 0)) ((3 4) (0 0)) ((4 1) (5 0))
    ((4 2) (3 0)) ((5 1) (3 3)) ((5 1) (6 0)) ((6 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "wrap" "decrypt") (neq "wrap" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule fact-set-decrypt-neq0 trRl_set-wrap-at-1 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation nonce-test (added-strand set-wrap 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-2 k "wrap")) (ch-msg lk-0 (cat pt-4 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt-4 k "init"))
      (load lk-0 (cat pt-2 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt-2 k "wrap")))
    ((load lk-0 (cat pt-2 k "wrap")) (stor lk-0 (cat pt-5 k "wrap"))))
  (label 770)
  (parent 304)
  (unrealized (0 0))
  (dead)
  (comment "empty cohort"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (0 0)) ((3 4) (0 0)) ((4 1) (5 0))
    ((4 1) (6 2)) ((4 2) (3 0)) ((5 1) (3 3)) ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-2 k "wrap")) (ch-msg lk-0 (cat pt-4 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt-4 k "init"))
      (load lk-0 (cat pt-2 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt-2 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-4 k "init"))
      (load lk-0 (cat pt-5 k1 "wrap")) (send (enc k k1))))
  (label 771)
  (parent 304)
  (unrealized (6 0) (6 3))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "wrap") (k k) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (6 0)) ((3 4) (0 0)) ((4 1) (5 0))
    ((4 2) (3 0)) ((5 1) (3 3)) ((6 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "wrap" "decrypt") (neq "wrap" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-decrypt-at-1 fact-set-decrypt-neq0 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation nonce-test (added-strand set-decrypt 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-2 k "wrap")) (ch-msg lk-0 (cat pt-4 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt-4 k "init"))
      (load lk-0 (cat pt-2 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt-2 k "wrap")))
    ((load lk (cat pt-1 k "wrap")) (stor lk (cat pt-5 k "decrypt"))))
  (label 772)
  (parent 304)
  (unrealized (0 0))
  (dead)
  (comment "empty cohort"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (0 0)) ((3 4) (0 0)) ((4 1) (5 0))
    ((4 2) (3 0)) ((5 1) (3 3)) ((5 1) (6 2)) ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-2 k "wrap")) (ch-msg lk-0 (cat pt-4 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt-4 k "init"))
      (load lk-0 (cat pt-2 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt-2 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-2 k "wrap"))
      (load lk-0 (cat pt-5 k1 "wrap")) (send (enc k k1))))
  (label 773)
  (parent 304)
  (unrealized (6 0) (6 3))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (6 2)) ((3 4) (0 0)) ((4 1) (5 0))
    ((4 2) (3 0)) ((5 1) (3 3)) ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-2 k "wrap")) (ch-msg lk-0 (cat pt-4 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt-4 k "init"))
      (load lk-0 (cat pt-2 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt-2 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-5 k1 "wrap")) (send (enc k k1))))
  (label 774)
  (parent 304)
  (unrealized (6 0) (6 3))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (6 2)) ((2 1) (0 0)) ((3 4) (0 0))
    ((4 1) (5 0)) ((4 2) (3 0)) ((5 1) (3 3)) ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-2 k "wrap")) (ch-msg lk-0 (cat pt-4 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt-4 k "init"))
      (load lk-0 (cat pt-2 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt-2 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-5 k1 "wrap")) (send (enc k k1))))
  (label 775)
  (parent 304)
  (unrealized (6 0) (6 3))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (defstrand make 3 (old old-1) (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (3 3)) ((3 4) (0 0)) ((4 2) (3 0))
    ((5 4) (0 0)) ((6 1) (1 0)) ((6 1) (3 2)) ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 3 7 make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt old)) (stor lk-0 (cat pt-0 k "init")))
    ((load lk-0 (cat pt-0 k "init")) (stor lk-0 (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt-6 k "init"))
      (load lk-0 (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-6 k "init"))
      (load lk-0 (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-5 old-1)) (stor lk-0 (cat pt-6 k "init"))
      (send (hash k))))
  (label 816)
  (parent 312)
  (seen 1552 1604)
  (unrealized (5 3))
  (comment "3 in cohort - 1 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (4 3)) ((3 1) (1 0)) ((3 1) (4 2))
    ((3 1) (6 2)) ((4 4) (0 0)) ((5 2) (4 0)) ((5 2) (6 0))
    ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 7 5 make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-4 old-1)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-6 k1 "wrap")) (send (enc k k1))))
  (label 817)
  (parent 312)
  (seen 131)
  (unrealized (6 3))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (3 3)) ((3 4) (0 0)) ((4 1) (5 2))
    ((4 2) (3 0)) ((5 4) (0 0)) ((6 1) (1 0)) ((6 1) (3 2))
    ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 3 7 make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt old)) (stor lk-0 (cat pt-0 k "init")))
    ((load lk-0 (cat pt-0 k "init")) (stor lk-0 (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt-6 k "init"))
      (load lk-0 (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-5 old-1)) (stor lk-0 (cat pt-6 k "init"))
      (send (hash k))))
  (label 820)
  (parent 313)
  (seen 128 1605)
  (unrealized (5 3))
  (comment "8 in cohort - 1 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-0))
  (precedes ((1 1) (3 3)) ((2 1) (3 2)) ((2 1) (6 0)) ((3 4) (0 0))
    ((4 1) (5 2)) ((4 2) (3 0)) ((5 4) (0 0)) ((6 1) (1 0))
    ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 1 7 make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-6 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((load lk-0 (cat pt-0 old)) (stor lk-0 (cat pt-1 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt-1 k "init"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-5 old-1)) (stor lk-0 (cat pt-6 k "init"))
      (send (hash k))))
  (label 821)
  (parent 313)
  (seen 1606)
  (unrealized (5 3))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (4 3)) ((3 1) (1 0)) ((3 1) (4 2))
    ((4 4) (0 0)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 7 5 make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-4 old-1)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-5 k "init"))
      (load lk-0 (cat pt-6 k1 "wrap")) (send (enc k k1))))
  (label 822)
  (parent 313)
  (seen 128 1608 1609)
  (unrealized (6 3))
  (comment "8 in cohort - 3 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk-0))
  (defstrand make 3 (old old-1) (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (3 3)) ((2 1) (5 2)) ((3 4) (0 0))
    ((4 2) (3 0)) ((5 4) (0 0)) ((6 1) (1 0)) ((6 1) (3 2))
    ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 3 7 make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt old)) (stor lk-0 (cat pt-0 k "init")))
    ((load lk-0 (cat pt-0 k "init")) (stor lk-0 (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt-6 k "init"))
      (load lk-0 (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-1 k "wrap"))
      (load lk-0 (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-5 old-1)) (stor lk-0 (cat pt-6 k "init"))
      (send (hash k))))
  (label 825)
  (parent 314)
  (seen 1063 1611)
  (unrealized (5 3))
  (comment "11 in cohort - 2 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk-0))
  (defstrand make 3 (old old-1) (k k) (lk lk-0))
  (precedes ((1 1) (3 3)) ((1 1) (5 2)) ((2 1) (3 2)) ((2 1) (6 0))
    ((3 4) (0 0)) ((4 2) (3 0)) ((5 4) (0 0)) ((6 1) (1 0))
    ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 1 7 make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-6 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((load lk-0 (cat pt-0 old)) (stor lk-0 (cat pt-1 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt-1 k "init"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt k "wrap"))
      (load lk-0 (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-5 old-1)) (stor lk-0 (cat pt-6 k "init"))
      (send (hash k))))
  (label 826)
  (parent 314)
  (seen 1612 1614)
  (unrealized (5 3))
  (comment "11 in cohort - 3 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (4 3)) ((2 1) (6 2)) ((3 1) (1 0))
    ((3 1) (4 2)) ((4 4) (0 0)) ((5 2) (4 0)) ((5 2) (6 0))
    ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 7 5 make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-4 old-1)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-6 k1 "wrap")) (send (enc k k1))))
  (label 827)
  (parent 314)
  (seen 1615 1617)
  (unrealized (6 3))
  (comment "11 in cohort - 3 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (defstrand make 3 (old old-1) (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((1 1) (5 2)) ((2 1) (3 3)) ((3 4) (0 0))
    ((4 2) (3 0)) ((5 4) (0 0)) ((6 1) (1 0)) ((6 1) (3 2))
    ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 3 7 make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt old)) (stor lk-0 (cat pt-0 k "init")))
    ((load lk-0 (cat pt-0 k "init")) (stor lk-0 (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt-6 k "init"))
      (load lk-0 (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-0 k "init"))
      (load lk-0 (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-5 old-1)) (stor lk-0 (cat pt-6 k "init"))
      (send (hash k))))
  (label 830)
  (parent 315)
  (seen 1167 1620)
  (unrealized (5 3))
  (comment "12 in cohort - 3 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (defstrand make 3 (old old-1) (k k) (lk lk-0))
  (precedes ((1 1) (3 3)) ((2 1) (3 2)) ((2 1) (6 0)) ((3 4) (0 0))
    ((4 2) (3 0)) ((5 4) (0 0)) ((6 1) (1 0)) ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 1 7 make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-6 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((load lk-0 (cat pt-0 old)) (stor lk-0 (cat pt-1 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt-1 k "init"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-6 k "init"))
      (load lk-0 (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-5 old-1)) (stor lk-0 (cat pt-6 k "init"))
      (send (hash k))))
  (label 831)
  (parent 315)
  (seen 1621 1624)
  (unrealized (5 3))
  (comment "12 in cohort - 4 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (6 2)) ((2 1) (4 3)) ((3 1) (1 0))
    ((3 1) (4 2)) ((4 4) (0 0)) ((5 2) (4 0)) ((5 2) (6 0))
    ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 7 5 make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-4 old-1)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-6 k1 "wrap")) (send (enc k k1))))
  (label 832)
  (parent 315)
  (seen 1625 1628)
  (unrealized (6 3))
  (comment "12 in cohort - 4 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (deflistener k)
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (0 0)) ((3 1) (5 0))
    ((4 4) (0 0)) ((5 1) (4 0)) ((6 1) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation nonce-test (displaced 7 3 make 2) k (5 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk-1 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init"))))
  (label 847)
  (parent 318)
  (seen 564)
  (unrealized (4 3) (5 0))
  (comment "7 in cohort - 5 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (deflistener k)
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk-0))
  (precedes ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (0 0)) ((3 1) (6 2))
    ((4 4) (0 0)) ((5 1) (4 0)) ((6 4) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (5 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk-0 (cat pt-3 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-5 k1-0 "wrap")) (send (enc k k1-0))))
  (label 852)
  (parent 319)
  (seen 863)
  (unrealized (4 3) (6 0) (6 3))
  (comment "4 in cohort - 3 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (deflistener k)
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (6 2)) ((2 1) (0 0)) ((3 1) (5 0))
    ((4 4) (0 0)) ((5 1) (4 0)) ((6 4) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (5 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk-0 (cat pt-3 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((recv (hash k)) (recv (hash k1-0)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-5 k1-0 "wrap")) (send (enc k k1-0))))
  (label 853)
  (parent 319)
  (seen 854)
  (unrealized (4 3) (6 0) (6 3))
  (comment "4 in cohort - 3 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (deflistener k)
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((1 1) (5 2)) ((2 1) (0 0)) ((3 4) (0 0))
    ((4 1) (3 0)) ((5 4) (4 0)) ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 3 7 make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-5 k "init"))
      (load lk-0 (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((recv (hash k)) (recv (hash k1-0)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-3 k1-0 "wrap")) (send (enc k k1-0)))
    ((load lk-0 (cat pt-4 old-0)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k))))
  (label 854)
  (parent 320)
  (seen 1347 1348)
  (unrealized (3 3) (5 3))
  (comment "7 in cohort - 0 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (precedes ((1 1) (0 0)) ((2 1) (3 2)) ((3 4) (0 0)) ((4 1) (3 0))
    ((5 4) (4 0)) ((6 1) (1 0)) ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 1 7 make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-5 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-0 (cat pt-5 k "init"))
      (load lk-0 (cat pt-3 k1-0 "wrap")) (send (enc k k1-0)))
    ((load lk-0 (cat pt-4 old-0)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k))))
  (label 855)
  (parent 320)
  (seen 1345 1346)
  (unrealized (3 3) (5 3))
  (comment "7 in cohort - 0 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (deflistener k)
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (0 0)) ((3 4) (0 0)) ((4 1) (3 0))
    ((5 4) (4 0)) ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 3 7 make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-5 k "init"))
      (load lk-0 (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-0 (cat pt-5 k "init"))
      (load lk-0 (cat pt-3 k1-0 "wrap")) (send (enc k k1-0)))
    ((load lk-0 (cat pt-4 old-0)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k))))
  (label 862)
  (parent 322)
  (unrealized (3 3) (5 3))
  (dead)
  (comment "empty cohort"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (precedes ((1 1) (0 0)) ((2 1) (5 2)) ((3 4) (0 0)) ((4 1) (3 0))
    ((5 4) (4 0)) ((6 1) (1 0)) ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 1 7 make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-5 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((recv (hash k)) (recv (hash k1-0)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt-3 k1-0 "wrap")) (send (enc k k1-0)))
    ((load lk-0 (cat pt-4 old-0)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k))))
  (label 863)
  (parent 322)
  (unrealized (3 3) (5 3))
  (dead)
  (comment "empty cohort"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "wrap") (k k) (lk lk-0))
  (precedes ((1 1) (3 2)) ((1 1) (6 0)) ((2 1) (5 0)) ((3 4) (0 0))
    ((4 1) (1 0)) ((4 2) (3 0)) ((5 1) (0 0)) ((6 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "wrap" "decrypt") (neq "wrap" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-decrypt-at-1 fact-set-decrypt-neq0 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation nonce-test (added-strand set-decrypt 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-1 k "init")) (ch-msg lk (cat pt-4 k "wrap"))
    (ch-msg lk-0 (cat pt k "wrap")) (ch-msg lk-0 (cat pt-3 k "init")))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt k "wrap"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-4 k "wrap")))
    ((load lk-0 (cat pt k "wrap")) (stor lk-0 (cat pt-5 k "decrypt"))))
  (label 866)
  (parent 326)
  (unrealized (0 0))
  (dead)
  (comment "empty cohort"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand set-wrap 2 (cur "wrap") (k k) (lk lk))
  (precedes ((1 1) (3 2)) ((2 1) (5 0)) ((3 4) (0 0)) ((4 1) (1 0))
    ((4 2) (3 0)) ((5 1) (6 0)) ((6 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "wrap" "decrypt") (neq "wrap" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule fact-set-decrypt-neq0 trRl_set-wrap-at-1 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation nonce-test (added-strand set-wrap 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-1 k "init")) (ch-msg lk (cat pt-4 k "wrap"))
    (ch-msg lk-0 (cat pt k "wrap")) (ch-msg lk-0 (cat pt-3 k "init")))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt k "wrap"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-4 k "wrap")))
    ((load lk (cat pt-4 k "wrap")) (stor lk (cat pt-5 k "wrap"))))
  (label 867)
  (parent 326)
  (unrealized (0 0))
  (dead)
  (comment "empty cohort"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (3 2)) ((2 1) (5 0)) ((3 4) (0 0)) ((4 1) (1 0))
    ((4 2) (3 0)) ((5 1) (0 0)) ((6 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation nonce-test (added-strand make 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-1 k "init")) (ch-msg lk (cat pt-4 k "wrap"))
    (ch-msg lk-0 (cat pt k "wrap")) (ch-msg lk-0 (cat pt-3 k "init")))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt k "wrap"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-4 k "wrap")))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init"))))
  (label 868)
  (parent 326)
  (seen 1395)
  (unrealized (0 0))
  (comment "10 in cohort - 9 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand set-wrap 2 (cur "wrap") (k k) (lk lk-0))
  (precedes ((1 1) (3 2)) ((1 1) (6 0)) ((2 1) (5 0)) ((3 4) (0 0))
    ((4 1) (1 0)) ((4 2) (3 0)) ((5 1) (0 0)) ((6 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "wrap" "decrypt") (neq "wrap" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule fact-set-decrypt-neq0 trRl_set-wrap-at-1 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation nonce-test (added-strand set-wrap 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-1 k "init")) (ch-msg lk (cat pt-4 k "wrap"))
    (ch-msg lk-0 (cat pt k "wrap")) (ch-msg lk-0 (cat pt-3 k "init")))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt k "wrap"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-4 k "wrap")))
    ((load lk-0 (cat pt k "wrap")) (stor lk-0 (cat pt-5 k "wrap"))))
  (label 869)
  (parent 326)
  (unrealized (0 0))
  (dead)
  (comment "empty cohort"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (precedes ((1 1) (3 2)) ((2 1) (5 0)) ((3 4) (0 0)) ((4 1) (1 0))
    ((4 1) (6 2)) ((4 2) (3 0)) ((5 1) (0 0)) ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-1 k "init")) (ch-msg lk (cat pt-4 k "wrap"))
    (ch-msg lk-0 (cat pt k "wrap")) (ch-msg lk-0 (cat pt-3 k "init")))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt k "wrap"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-4 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-5 k1 "wrap")) (send (enc k k1))))
  (label 870)
  (parent 326)
  (unrealized (6 0) (6 3))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "wrap") (k k) (lk lk))
  (precedes ((1 1) (3 2)) ((2 1) (5 0)) ((3 4) (0 0)) ((4 1) (1 0))
    ((4 2) (3 0)) ((5 1) (6 0)) ((6 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "wrap" "decrypt") (neq "wrap" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-decrypt-at-1 fact-set-decrypt-neq0 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation nonce-test (added-strand set-decrypt 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-1 k "init")) (ch-msg lk (cat pt-4 k "wrap"))
    (ch-msg lk-0 (cat pt k "wrap")) (ch-msg lk-0 (cat pt-3 k "init")))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt k "wrap"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-4 k "wrap")))
    ((load lk (cat pt-4 k "wrap")) (stor lk (cat pt-5 k "decrypt"))))
  (label 871)
  (parent 326)
  (unrealized (0 0))
  (dead)
  (comment "empty cohort"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk-0))
  (precedes ((1 1) (3 2)) ((1 1) (6 2)) ((2 1) (5 0)) ((3 4) (0 0))
    ((4 1) (1 0)) ((4 2) (3 0)) ((5 1) (0 0)) ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-1 k "init")) (ch-msg lk (cat pt-4 k "wrap"))
    (ch-msg lk-0 (cat pt k "wrap")) (ch-msg lk-0 (cat pt-3 k "init")))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt k "wrap"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-4 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt k "wrap"))
      (load lk-0 (cat pt-5 k1 "wrap")) (send (enc k k1))))
  (label 872)
  (parent 326)
  (unrealized (6 0) (6 3))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (3 2)) ((2 1) (5 0)) ((3 4) (0 0)) ((4 1) (1 0))
    ((4 2) (3 0)) ((5 1) (6 2)) ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-1 k "init")) (ch-msg lk (cat pt-4 k "wrap"))
    (ch-msg lk-0 (cat pt k "wrap")) (ch-msg lk-0 (cat pt-3 k "init")))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt k "wrap"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-4 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-4 k "wrap"))
      (load lk (cat pt-5 k1 "wrap")) (send (enc k k1))))
  (label 873)
  (parent 326)
  (unrealized (6 0) (6 3))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (3 2)) ((2 1) (5 0)) ((2 1) (6 2)) ((3 4) (0 0))
    ((4 1) (1 0)) ((4 2) (3 0)) ((5 1) (0 0)) ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-1 k "init")) (ch-msg lk (cat pt-4 k "wrap"))
    (ch-msg lk-0 (cat pt k "wrap")) (ch-msg lk-0 (cat pt-3 k "init")))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt k "wrap"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-4 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt-5 k1 "wrap")) (send (enc k k1))))
  (label 874)
  (parent 326)
  (unrealized (6 0) (6 3))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (precedes ((1 1) (3 2)) ((2 1) (0 0)) ((3 4) (0 0)) ((4 1) (1 0))
    ((4 2) (3 0)) ((4 2) (5 0)) ((5 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 6 4 make 3) (hash k) (5 0))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt k "wrap"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-4 k1 "wrap")) (send (enc k k1))))
  (label 875)
  (parent 329)
  (seen 1665 1666)
  (unrealized (5 3))
  (comment "7 in cohort - 2 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (3 2)) ((2 1) (0 0)) ((3 4) (0 0)) ((4 1) (1 0))
    ((4 1) (5 2)) ((4 2) (3 0)) ((5 4) (0 0)) ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation encryption-test (added-strand make 3) (hash k) (5 0))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt k "wrap"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init"))
      (send (hash k))))
  (label 876)
  (parent 329)
  (seen 1667 1668)
  (unrealized (5 3))
  (comment "7 in cohort - 2 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (deflistener k)
  (precedes ((1 1) (3 2)) ((2 1) (0 0)) ((3 4) (0 0)) ((4 1) (1 0))
    ((4 1) (5 2)) ((4 2) (3 0)) ((5 4) (0 0)) ((6 1) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (added-listener k) (hash k) (5 0))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt k "wrap"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k)))
  (label 877)
  (parent 329)
  (seen 1018)
  (unrealized (5 3) (6 0))
  (comment "3 in cohort - 2 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk-0))
  (precedes ((1 1) (3 2)) ((1 1) (5 2)) ((2 1) (0 0)) ((3 4) (0 0))
    ((4 1) (1 0)) ((4 2) (3 0)) ((4 2) (5 0)) ((5 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 6 4 make 3) (hash k) (5 0))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt k "wrap"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt k "wrap"))
      (load lk-0 (cat pt-4 k1 "wrap")) (send (enc k k1))))
  (label 878)
  (parent 330)
  (seen 136)
  (unrealized (5 3))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk-0))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (3 2)) ((1 1) (5 2)) ((2 1) (0 0)) ((3 4) (0 0))
    ((4 1) (1 0)) ((4 2) (3 0)) ((5 4) (0 0)) ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation encryption-test (added-strand make 3) (hash k) (5 0))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt k "wrap"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt k "wrap"))
      (load lk-0 (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init"))
      (send (hash k))))
  (label 879)
  (parent 330)
  (seen 1671 1672)
  (unrealized (5 3))
  (comment "4 in cohort - 2 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk-0))
  (deflistener k)
  (precedes ((1 1) (3 2)) ((1 1) (5 2)) ((2 1) (0 0)) ((3 4) (0 0))
    ((4 1) (1 0)) ((4 2) (3 0)) ((5 4) (0 0)) ((6 1) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (added-listener k) (hash k) (5 0))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt k "wrap"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt k "wrap"))
      (load lk-0 (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k)))
  (label 880)
  (parent 330)
  (seen 1022)
  (unrealized (5 3) (6 0))
  (comment "3 in cohort - 2 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (3 2)) ((2 1) (5 2)) ((3 4) (0 0)) ((4 1) (1 0))
    ((4 2) (3 0)) ((4 2) (5 0)) ((5 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 6 4 make 3) (hash k) (5 0))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt k "wrap"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1))))
  (label 881)
  (parent 331)
  (seen 1013 1675)
  (unrealized (5 3))
  (comment "7 in cohort - 1 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (precedes ((1 1) (2 2)) ((2 4) (0 0)) ((3 1) (1 0)) ((3 2) (2 0))
    ((4 4) (0 0)) ((5 2) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 2 6 make 3) (hash k) (5 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt k "wrap"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-4 k "init"))
      (load lk-0 (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init"))
      (send (hash k))))
  (label 882)
  (parent 331)
  (seen 1013 1558 1676)
  (unrealized (4 3))
  (comment "8 in cohort - 1 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (3 2)) ((2 1) (5 2)) ((3 4) (0 0)) ((4 1) (1 0))
    ((4 2) (3 0)) ((5 4) (0 0)) ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation encryption-test (added-strand make 3) (hash k) (5 0))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt k "wrap"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init"))
      (send (hash k))))
  (label 883)
  (parent 331)
  (seen 1015 1612 1677)
  (unrealized (5 3))
  (comment "8 in cohort - 1 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (precedes ((1 1) (3 2)) ((2 1) (5 2)) ((3 4) (0 0)) ((4 1) (1 0))
    ((4 2) (3 0)) ((5 4) (0 0)) ((6 1) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (added-listener k) (hash k) (5 0))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt k "wrap"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k)))
  (label 884)
  (parent 331)
  (unrealized (5 3) (6 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-1) (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((1 1) (3 2)) ((2 1) (5 0)) ((3 4) (0 0))
    ((4 1) (1 0)) ((4 2) (3 0)) ((5 1) (3 3)) ((6 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation nonce-test (added-strand make 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt k "wrap")) (ch-msg lk (cat pt-1 k "init"))
    (ch-msg lk (cat pt-2 k "wrap")) (ch-msg lk (cat pt-4 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt-4 k "init")) (stor lk (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt k "wrap"))
      (load lk (cat pt-2 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-3 old-0)) (stor lk (cat pt-4 k "init"))
      (send (hash k)))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-2 k "wrap")))
    ((load lk-0 (cat pt-5 old-1)) (stor lk-0 (cat pt-6 k "init"))))
  (label 885)
  (parent 333)
  (unrealized (0 0))
  (comment "9 in cohort - 9 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k skey) (lk locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand set-wrap 2 (cur "wrap") (k k) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (3 2)) ((2 1) (5 0)) ((3 4) (0 0))
    ((4 1) (1 0)) ((4 2) (3 0)) ((5 1) (3 3)) ((5 1) (6 0))
    ((6 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "wrap" "decrypt") (neq "wrap" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule fact-set-decrypt-neq0 trRl_set-wrap-at-1 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation nonce-test (added-strand set-wrap 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt k "wrap")) (ch-msg lk (cat pt-1 k "init"))
    (ch-msg lk (cat pt-2 k "wrap")) (ch-msg lk (cat pt-4 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt-4 k "init")) (stor lk (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt k "wrap"))
      (load lk (cat pt-2 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-3 old-0)) (stor lk (cat pt-4 k "init"))
      (send (hash k)))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-2 k "wrap")))
    ((load lk (cat pt-2 k "wrap")) (stor lk (cat pt-5 k "wrap"))))
  (label 886)
  (parent 333)
  (unrealized (0 0))
  (dead)
  (comment "empty cohort"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k skey) (lk locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "wrap") (k k) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (3 2)) ((2 1) (5 0)) ((3 4) (0 0))
    ((4 1) (1 0)) ((4 2) (3 0)) ((5 1) (3 3)) ((5 1) (6 0))
    ((6 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "wrap" "decrypt") (neq "wrap" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-decrypt-at-1 fact-set-decrypt-neq0 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation nonce-test (added-strand set-decrypt 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt k "wrap")) (ch-msg lk (cat pt-1 k "init"))
    (ch-msg lk (cat pt-2 k "wrap")) (ch-msg lk (cat pt-4 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt-4 k "init")) (stor lk (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt k "wrap"))
      (load lk (cat pt-2 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-3 old-0)) (stor lk (cat pt-4 k "init"))
      (send (hash k)))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-2 k "wrap")))
    ((load lk (cat pt-2 k "wrap")) (stor lk (cat pt-5 k "decrypt"))))
  (label 887)
  (parent 333)
  (unrealized (0 0))
  (dead)
  (comment "empty cohort"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (3 2)) ((2 1) (5 0)) ((3 4) (0 0))
    ((4 1) (1 0)) ((4 2) (3 0)) ((5 1) (3 3)) ((5 1) (6 2))
    ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt k "wrap")) (ch-msg lk (cat pt-1 k "init"))
    (ch-msg lk (cat pt-2 k "wrap")) (ch-msg lk (cat pt-4 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt-4 k "init")) (stor lk (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt k "wrap"))
      (load lk (cat pt-2 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-3 old-0)) (stor lk (cat pt-4 k "init"))
      (send (hash k)))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-2 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-2 k "wrap"))
      (load lk (cat pt-5 k1 "wrap")) (send (enc k k1))))
  (label 888)
  (parent 333)
  (unrealized (6 0) (6 3))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (3 2)) ((2 1) (5 0)) ((3 4) (0 0))
    ((4 1) (1 0)) ((4 1) (6 2)) ((4 2) (3 0)) ((5 1) (3 3))
    ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt k "wrap")) (ch-msg lk (cat pt-1 k "init"))
    (ch-msg lk (cat pt-2 k "wrap")) (ch-msg lk (cat pt-4 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt-4 k "init")) (stor lk (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt k "wrap"))
      (load lk (cat pt-2 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-3 old-0)) (stor lk (cat pt-4 k "init"))
      (send (hash k)))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-2 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-4 k "init"))
      (load lk (cat pt-5 k1 "wrap")) (send (enc k k1))))
  (label 889)
  (parent 333)
  (unrealized (6 0) (6 3))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (3 2)) ((2 1) (5 0)) ((2 1) (6 2))
    ((3 4) (0 0)) ((4 1) (1 0)) ((4 2) (3 0)) ((5 1) (3 3))
    ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt k "wrap")) (ch-msg lk (cat pt-1 k "init"))
    (ch-msg lk (cat pt-2 k "wrap")) (ch-msg lk (cat pt-4 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt-4 k "init")) (stor lk (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt k "wrap"))
      (load lk (cat pt-2 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-3 old-0)) (stor lk (cat pt-4 k "init"))
      (send (hash k)))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-2 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt-5 k1 "wrap")) (send (enc k k1))))
  (label 890)
  (parent 333)
  (unrealized (6 0) (6 3))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (3 2)) ((1 1) (6 2)) ((2 1) (5 0))
    ((3 4) (0 0)) ((4 1) (1 0)) ((4 2) (3 0)) ((5 1) (3 3))
    ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt k "wrap")) (ch-msg lk (cat pt-1 k "init"))
    (ch-msg lk (cat pt-2 k "wrap")) (ch-msg lk (cat pt-4 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt-4 k "init")) (stor lk (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt k "wrap"))
      (load lk (cat pt-2 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-3 old-0)) (stor lk (cat pt-4 k "init"))
      (send (hash k)))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-2 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt k "wrap"))
      (load lk (cat pt-5 k1 "wrap")) (send (enc k k1))))
  (label 891)
  (parent 333)
  (unrealized (6 0) (6 3))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-1))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk-1))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (3 2)) ((2 1) (0 0)) ((3 4) (0 0)) ((4 1) (5 2))
    ((4 2) (3 0)) ((5 4) (0 0)) ((6 1) (1 0)) ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 1 7 make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk-1 (cat pt-6 k "init")) (stor lk-1 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk-1 (cat pt k "wrap"))
      (load lk-1 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init"))
      (send (hash k))))
  (label 923)
  (parent 338)
  (seen 1665 1704 1705)
  (unrealized (5 3))
  (comment "13 in cohort - 2 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-1))
  (precedes ((1 1) (2 0)) ((2 1) (4 2)) ((3 1) (0 0)) ((4 4) (0 0))
    ((5 2) (4 0)) ((5 2) (6 0)) ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 7 5 make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-1 (cat pt-4 old-1)) (stor lk-1 (cat pt-5 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk-1 (cat pt-5 k "init"))
      (load lk-1 (cat pt-6 k1 "wrap")) (send (enc k k1))))
  (label 924)
  (parent 338)
  (seen 1665 1706 1707 1708)
  (unrealized (6 3))
  (comment "16 in cohort - 3 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-1))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((2 1) (3 2)) ((3 4) (0 0)) ((4 2) (3 0))
    ((5 4) (0 0)) ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 3 7 make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk-1 (cat pt-6 k "init"))
      (load lk-1 (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init"))
      (send (hash k))))
  (label 927)
  (parent 340)
  (seen 1076 1561 1709)
  (unrealized (5 3))
  (comment "8 in cohort - 1 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-1))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk-1))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (3 2)) ((2 1) (5 2)) ((3 4) (0 0)) ((4 2) (3 0))
    ((5 4) (0 0)) ((6 1) (1 0)) ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 1 7 make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk-1 (cat pt-6 k "init")) (stor lk-1 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk-1 (cat pt k "wrap"))
      (load lk-1 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init"))
      (send (hash k))))
  (label 928)
  (parent 340)
  (seen 1076 1710)
  (unrealized (5 3))
  (comment "7 in cohort - 1 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (4 2)) ((3 1) (6 2)) ((4 4) (0 0))
    ((5 2) (4 0)) ((5 2) (6 0)) ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 7 5 make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-1 (cat pt-4 old-1)) (stor lk-1 (cat pt-5 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-6 k1 "wrap")) (send (enc k k1))))
  (label 929)
  (parent 340)
  (seen 1074 1615 1711)
  (unrealized (6 3))
  (comment "8 in cohort - 1 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-1))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk-1))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk-1))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (3 2)) ((1 1) (5 2)) ((2 1) (0 0)) ((3 4) (0 0))
    ((4 2) (3 0)) ((5 4) (0 0)) ((6 1) (1 0)) ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 1 7 make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk-1 (cat pt-6 k "init")) (stor lk-1 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk-1 (cat pt k "wrap"))
      (load lk-1 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk-1 (cat pt k "wrap"))
      (load lk-1 (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init"))
      (send (hash k))))
  (label 932)
  (parent 341)
  (seen 1671 1712)
  (unrealized (5 3))
  (comment "4 in cohort - 1 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (4 2)) ((2 1) (6 2)) ((3 1) (0 0))
    ((4 4) (0 0)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 7 5 make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-1 (cat pt-4 old-1)) (stor lk-1 (cat pt-5 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-6 k1 "wrap")) (send (enc k k1))))
  (label 933)
  (parent 341)
  (seen 138)
  (unrealized (6 3))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-1))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk-1))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-1))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (3 2)) ((2 1) (0 0)) ((3 4) (0 0)) ((4 2) (3 0))
    ((5 4) (0 0)) ((6 1) (1 0)) ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 1 7 make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk-1 (cat pt-6 k "init")) (stor lk-1 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk-1 (cat pt k "wrap"))
      (load lk-1 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk-1 (cat pt-6 k "init"))
      (load lk-1 (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init"))
      (send (hash k))))
  (label 936)
  (parent 342)
  (seen 1713 1714)
  (unrealized (5 3))
  (comment "7 in cohort - 2 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (6 2)) ((2 1) (4 2)) ((3 1) (0 0))
    ((4 4) (0 0)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 7 5 make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-1 (cat pt-4 old-1)) (stor lk-1 (cat pt-5 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-6 k1 "wrap")) (send (enc k k1))))
  (label 937)
  (parent 342)
  (seen 1715 1716)
  (unrealized (6 3))
  (comment "7 in cohort - 2 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((2 1) (5 0)) ((3 1) (0 0)) ((4 4) (0 0))
    ((5 1) (4 0)) ((6 1) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation nonce-test (added-strand make 2) k (5 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init"))))
  (label 950)
  (parent 345)
  (unrealized (4 3) (5 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1-0) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (6 2)) ((3 1) (0 0)) ((4 4) (0 0))
    ((5 1) (4 0)) ((6 4) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap")
    (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (5 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((recv (hash k)) (recv (hash k1-0)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-5 k1-0 "wrap")) (send (enc k k1-0))))
  (label 951)
  (parent 345)
  (seen 1089)
  (unrealized (4 3) (6 0) (6 3))
  (comment "4 in cohort - 3 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (6 2)) ((2 1) (5 0)) ((3 1) (0 0))
    ((4 4) (0 0)) ((5 1) (4 0)) ((6 4) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap")
    (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (5 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((recv (hash k)) (recv (hash k1-0)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-5 k1-0 "wrap")) (send (enc k k1-0))))
  (label 952)
  (parent 345)
  (seen 1092)
  (unrealized (4 3) (6 0) (6 3))
  (comment "4 in cohort - 3 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk-0))
  (deflistener k)
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (precedes ((1 1) (3 2)) ((2 1) (0 0)) ((3 4) (0 0)) ((4 1) (3 0))
    ((5 4) (4 0)) ((6 1) (1 0)) ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap")
    (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 1 7 make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-5 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt k "wrap"))
      (load lk-0 (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-0 (cat pt-5 k "init"))
      (load lk-0 (cat pt-3 k1-0 "wrap")) (send (enc k k1-0)))
    ((load lk-0 (cat pt-4 old-0)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k))))
  (label 957)
  (parent 347)
  (seen 1728)
  (unrealized (3 3) (5 3))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (precedes ((1 1) (3 3)) ((2 1) (0 0)) ((3 4) (0 0)) ((4 1) (1 0))
    ((4 2) (3 0)) ((4 2) (5 0)) ((5 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 6 4 make 3) (hash k) (5 0))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-4 k1 "wrap")) (send (enc k k1))))
  (label 960)
  (parent 353)
  (seen 142)
  (unrealized (5 3))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (3 3)) ((2 1) (0 0)) ((3 4) (0 0)) ((4 1) (1 0))
    ((4 1) (5 2)) ((4 2) (3 0)) ((5 4) (0 0)) ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation encryption-test (added-strand make 3) (hash k) (5 0))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init"))
      (send (hash k))))
  (label 961)
  (parent 353)
  (seen 1729 1730)
  (unrealized (5 3))
  (comment "7 in cohort - 2 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (deflistener k)
  (precedes ((1 1) (3 3)) ((2 1) (0 0)) ((3 4) (0 0)) ((4 1) (1 0))
    ((4 1) (5 2)) ((4 2) (3 0)) ((5 4) (0 0)) ((6 1) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (added-listener k) (hash k) (5 0))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k)))
  (label 962)
  (parent 353)
  (seen 1122)
  (unrealized (5 3) (6 0))
  (comment "3 in cohort - 2 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk-0))
  (precedes ((1 1) (3 3)) ((1 1) (5 2)) ((2 1) (0 0)) ((3 4) (0 0))
    ((4 1) (1 0)) ((4 2) (3 0)) ((4 2) (5 0)) ((5 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 6 4 make 3) (hash k) (5 0))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt k "wrap"))
      (load lk-0 (cat pt-4 k1 "wrap")) (send (enc k k1))))
  (label 963)
  (parent 354)
  (seen 1665 1733)
  (unrealized (5 3))
  (comment "4 in cohort - 1 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk-0))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (3 3)) ((1 1) (5 2)) ((2 1) (0 0)) ((3 4) (0 0))
    ((4 1) (1 0)) ((4 2) (3 0)) ((5 4) (0 0)) ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation encryption-test (added-strand make 3) (hash k) (5 0))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt k "wrap"))
      (load lk-0 (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init"))
      (send (hash k))))
  (label 964)
  (parent 354)
  (seen 1713 1734)
  (unrealized (5 3))
  (comment "4 in cohort - 1 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk-0))
  (deflistener k)
  (precedes ((1 1) (3 3)) ((1 1) (5 2)) ((2 1) (0 0)) ((3 4) (0 0))
    ((4 1) (1 0)) ((4 2) (3 0)) ((5 4) (0 0)) ((6 1) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (added-listener k) (hash k) (5 0))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt k "wrap"))
      (load lk-0 (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k)))
  (label 965)
  (parent 354)
  (seen 1126)
  (unrealized (5 3) (6 0))
  (comment "3 in cohort - 2 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (3 3)) ((2 1) (5 2)) ((3 4) (0 0)) ((4 1) (1 0))
    ((4 2) (3 0)) ((4 2) (5 0)) ((5 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 6 4 make 3) (hash k) (5 0))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1))))
  (label 966)
  (parent 355)
  (seen 72 1737)
  (unrealized (5 3))
  (comment "7 in cohort - 1 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (precedes ((1 1) (2 3)) ((2 4) (0 0)) ((3 1) (1 0)) ((3 2) (2 0))
    ((4 4) (0 0)) ((5 2) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 2 6 make 3) (hash k) (5 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-4 k "init"))
      (load lk-0 (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init"))
      (send (hash k))))
  (label 967)
  (parent 355)
  (seen 72 1569 1738)
  (unrealized (4 3))
  (comment "8 in cohort - 1 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (3 3)) ((2 1) (5 2)) ((3 4) (0 0)) ((4 1) (1 0))
    ((4 2) (3 0)) ((5 4) (0 0)) ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation encryption-test (added-strand make 3) (hash k) (5 0))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init"))
      (send (hash k))))
  (label 968)
  (parent 355)
  (seen 1119 1621 1739)
  (unrealized (5 3))
  (comment "8 in cohort - 1 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (precedes ((1 1) (3 3)) ((2 1) (5 2)) ((3 4) (0 0)) ((4 1) (1 0))
    ((4 2) (3 0)) ((5 4) (0 0)) ((6 1) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (added-listener k) (hash k) (5 0))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k)))
  (label 969)
  (parent 355)
  (unrealized (5 3) (6 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-1))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk-1))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (3 3)) ((2 1) (0 0)) ((3 4) (0 0)) ((4 1) (5 2))
    ((4 2) (3 0)) ((5 4) (0 0)) ((6 1) (1 0)) ((6 1) (3 2))
    ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 1 7 make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk-1 (cat pt-6 k "init")) (stor lk-1 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk-1 (cat pt-6 k "init"))
      (load lk-1 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init"))
      (send (hash k))))
  (label 989)
  (parent 360)
  (seen 142 1744)
  (unrealized (5 3))
  (comment "14 in cohort - 2 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-1))
  (precedes ((1 1) (2 0)) ((1 1) (4 2)) ((2 1) (4 3)) ((3 1) (0 0))
    ((4 4) (0 0)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 7 5 make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-1 (cat pt-4 old-1)) (stor lk-1 (cat pt-5 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk-1 (cat pt-5 k "init"))
      (load lk-1 (cat pt-6 k1 "wrap")) (send (enc k k1))))
  (label 990)
  (parent 360)
  (seen 142 1745 1747)
  (unrealized (6 3))
  (comment "16 in cohort - 3 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-1))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((1 1) (3 2)) ((2 1) (3 3)) ((3 4) (0 0))
    ((4 2) (3 0)) ((5 4) (0 0)) ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 3 7 make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk-1 (cat pt-6 k "init"))
      (load lk-1 (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init"))
      (send (hash k))))
  (label 993)
  (parent 362)
  (seen 1119 1573 1748)
  (unrealized (5 3))
  (comment "8 in cohort - 1 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-1))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk-1))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (3 3)) ((2 1) (5 2)) ((3 4) (0 0)) ((4 2) (3 0))
    ((5 4) (0 0)) ((6 1) (1 0)) ((6 1) (3 2)) ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 1 7 make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk-1 (cat pt-6 k "init")) (stor lk-1 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk-1 (cat pt-6 k "init"))
      (load lk-1 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init"))
      (send (hash k))))
  (label 994)
  (parent 362)
  (seen 1119 1749)
  (unrealized (5 3))
  (comment "7 in cohort - 1 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (precedes ((1 1) (2 0)) ((1 1) (4 2)) ((2 1) (4 3)) ((3 1) (6 2))
    ((4 4) (0 0)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 7 5 make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-1 (cat pt-4 old-1)) (stor lk-1 (cat pt-5 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-6 k1 "wrap")) (send (enc k k1))))
  (label 995)
  (parent 362)
  (seen 74 1625 1750)
  (unrealized (6 3))
  (comment "8 in cohort - 1 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-1))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk-1))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk-1))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (3 3)) ((1 1) (5 2)) ((2 1) (0 0)) ((3 4) (0 0))
    ((4 2) (3 0)) ((5 4) (0 0)) ((6 1) (1 0)) ((6 1) (3 2))
    ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 1 7 make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk-1 (cat pt-6 k "init")) (stor lk-1 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk-1 (cat pt-6 k "init"))
      (load lk-1 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk-1 (cat pt k "wrap"))
      (load lk-1 (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init"))
      (send (hash k))))
  (label 998)
  (parent 363)
  (seen 1667 1751)
  (unrealized (5 3))
  (comment "4 in cohort - 1 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (4 2)) ((2 1) (4 3)) ((2 1) (6 2))
    ((3 1) (0 0)) ((4 4) (0 0)) ((5 2) (4 0)) ((5 2) (6 0))
    ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 7 5 make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-1 (cat pt-4 old-1)) (stor lk-1 (cat pt-5 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-6 k1 "wrap")) (send (enc k k1))))
  (label 999)
  (parent 363)
  (seen 1715 1752)
  (unrealized (6 3))
  (comment "4 in cohort - 1 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-1))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk-1))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-1))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (3 3)) ((2 1) (0 0)) ((3 4) (0 0)) ((4 2) (3 0))
    ((5 4) (0 0)) ((6 1) (1 0)) ((6 1) (3 2)) ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 1 7 make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk-1 (cat pt-6 k "init")) (stor lk-1 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk-1 (cat pt-6 k "init"))
      (load lk-1 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk-1 (cat pt-6 k "init"))
      (load lk-1 (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init"))
      (send (hash k))))
  (label 1002)
  (parent 364)
  (seen 1729 1753)
  (unrealized (5 3))
  (comment "7 in cohort - 1 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (4 2)) ((1 1) (6 2)) ((2 1) (4 3))
    ((3 1) (0 0)) ((4 4) (0 0)) ((5 2) (4 0)) ((5 2) (6 0))
    ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 7 5 make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-1 (cat pt-4 old-1)) (stor lk-1 (cat pt-5 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-6 k1 "wrap")) (send (enc k k1))))
  (label 1003)
  (parent 364)
  (seen 144)
  (unrealized (6 3))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (deflistener k)
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (precedes ((1 1) (0 0)) ((2 1) (0 0)) ((3 4) (0 0)) ((4 1) (3 0))
    ((5 4) (4 0)) ((6 1) (1 0)) ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 1 7 make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-5 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-5 k "init"))
      (load lk-0 (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-0 (cat pt-5 k "init"))
      (load lk-0 (cat pt-3 k1-0 "wrap")) (send (enc k k1-0)))
    ((load lk-0 (cat pt-4 old-0)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k))))
  (label 1010)
  (parent 367)
  (unrealized (3 3) (5 3))
  (dead)
  (comment "empty cohort"))

(defskeleton wrap-decrypt
  (vars (old mesg) (pt pt-0 pt-1 pval) (k skey) (lk locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (precedes ((1 1) (2 2)) ((1 1) (4 3)) ((2 4) (0 0)) ((3 1) (1 0))
    ((3 2) (2 0)) ((3 2) (4 0)) ((4 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation channel-test (displaced 5 1 set-wrap 2)
    (ch-msg lk (cat pt-2 k1 "wrap")) (4 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt k "wrap"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt k "wrap")) (send (enc k k))))
  (label 1013)
  (parent 371)
  (seen 1665)
  (unrealized (0 0))
  (comment "5 in cohort - 4 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old cur mesg) (pt pt-0 pt-1 pt-2 pt-3 pval) (k k1 skey)
    (lk locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (defstrand set-wrap 2 (cur cur) (k k1) (lk lk))
  (precedes ((1 1) (2 2)) ((1 1) (5 0)) ((2 4) (0 0)) ((3 1) (1 0))
    ((3 2) (2 0)) ((3 2) (4 0)) ((4 4) (0 0)) ((5 1) (4 3)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 cur) (cat k1 "wrap"))
  (facts (neq cur "decrypt") (neq cur "wrap") (neq "init" "decrypt")
    (neq "init" "wrap"))
  (rule shearsRule invShearsRule fact-set-decrypt-neq0
    trRl_set-wrap-at-1 gen-st-set-wrap-0 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation channel-test (added-strand set-wrap 2)
    (ch-msg lk (cat pt-2 k1 "wrap")) (4 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt k "wrap"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((load lk (cat pt-3 k1 cur)) (stor lk (cat pt-2 k1 "wrap"))))
  (label 1014)
  (parent 371)
  (seen 1758)
  (unrealized (5 0))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pval) (k skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (precedes ((1 1) (2 2)) ((1 1) (4 3)) ((2 4) (0 0)) ((3 1) (1 0))
    ((3 1) (4 2)) ((3 2) (2 0)) ((4 4) (0 0)) ((5 2) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation channel-test (displaced 6 1 set-wrap 2)
    (ch-msg lk (cat pt-4 k1 "wrap")) (4 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt k "wrap"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k))))
  (label 1015)
  (parent 372)
  (seen 1667)
  (unrealized (0 0))
  (comment "8 in cohort - 7 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 cur mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur cur) (k k1) (lk lk))
  (precedes ((1 1) (2 2)) ((1 1) (6 0)) ((2 4) (0 0)) ((3 1) (1 0))
    ((3 1) (4 2)) ((3 2) (2 0)) ((4 4) (0 0)) ((5 2) (4 0))
    ((6 1) (4 3)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 cur) (cat k1 "wrap"))
  (facts (neq cur "decrypt") (neq cur "wrap") (neq "init" "decrypt")
    (neq "init" "wrap"))
  (rule shearsRule invShearsRule fact-set-decrypt-neq0
    trRl_set-wrap-at-1 gen-st-set-wrap-0 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation channel-test (added-strand set-wrap 2)
    (ch-msg lk (cat pt-2 k1 "wrap")) (4 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt k "wrap"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init"))
      (send (hash k)))
    ((load lk (cat pt-5 k1 cur)) (stor lk (cat pt-2 k1 "wrap"))))
  (label 1016)
  (parent 372)
  (seen 1766)
  (unrealized (6 0))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old mesg) (pt pt-0 pt-1 pt-2 pval) (k k1 skey) (lk locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (precedes ((1 1) (2 2)) ((2 4) (0 0)) ((3 1) (1 0)) ((3 1) (5 0))
    ((3 2) (2 0)) ((4 4) (0 0)) ((5 1) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation nonce-test (displaced 6 3 make 2) k (5 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt k "wrap"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k)))
  (label 1017)
  (parent 373)
  (unrealized (4 3) (5 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (precedes ((1 1) (2 2)) ((2 4) (0 0)) ((3 1) (1 0)) ((3 1) (4 2))
    ((3 2) (2 0)) ((4 4) (0 0)) ((5 1) (4 0)) ((6 1) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation nonce-test (added-strand make 2) k (5 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt k "wrap"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init"))))
  (label 1018)
  (parent 373)
  (seen 1767)
  (unrealized (4 3) (5 0))
  (comment "4 in cohort - 3 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pval) (k skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (precedes ((1 1) (2 2)) ((1 1) (4 2)) ((2 4) (0 0)) ((3 1) (1 0))
    ((3 2) (2 0)) ((4 4) (0 0)) ((5 2) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation channel-test (displaced 6 1 set-wrap 2)
    (ch-msg lk (cat pt-4 k1 "wrap")) (4 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt k "wrap"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt k "wrap"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k))))
  (label 1019)
  (parent 375)
  (seen 1671)
  (unrealized (0 0))
  (comment "8 in cohort - 7 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 cur mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur cur) (k k1) (lk lk))
  (precedes ((1 1) (2 2)) ((1 1) (4 2)) ((1 1) (6 0)) ((2 4) (0 0))
    ((3 1) (1 0)) ((3 2) (2 0)) ((4 4) (0 0)) ((5 2) (4 0))
    ((6 1) (4 3)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 cur) (cat k1 "wrap"))
  (facts (neq cur "decrypt") (neq cur "wrap") (neq "init" "decrypt")
    (neq "init" "wrap"))
  (rule invShearsRule fact-set-decrypt-neq0 trRl_set-wrap-at-1
    gen-st-set-wrap-0 fact-set-wrap-neq0 trRl_make-at-0)
  (operation channel-test (added-strand set-wrap 2)
    (ch-msg lk (cat pt-2 k1 "wrap")) (4 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt k "wrap"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt k "wrap"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init"))
      (send (hash k)))
    ((load lk (cat pt-5 k1 cur)) (stor lk (cat pt-2 k1 "wrap"))))
  (label 1020)
  (parent 375)
  (seen 1779)
  (unrealized (6 0))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old mesg) (pt pt-0 pt-1 pt-2 pval) (k k1 skey) (lk locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (precedes ((1 1) (2 2)) ((1 1) (4 2)) ((2 4) (0 0)) ((3 1) (1 0))
    ((3 1) (5 0)) ((3 2) (2 0)) ((4 4) (0 0)) ((5 1) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation nonce-test (displaced 6 3 make 2) k (5 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt k "wrap"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt k "wrap"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k)))
  (label 1021)
  (parent 376)
  (seen 1780)
  (unrealized (4 3) (5 0))
  (comment "4 in cohort - 3 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (precedes ((1 1) (2 2)) ((1 1) (4 2)) ((2 4) (0 0)) ((3 1) (1 0))
    ((3 2) (2 0)) ((4 4) (0 0)) ((5 1) (4 0)) ((6 1) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation nonce-test (added-strand make 2) k (5 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt k "wrap"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt k "wrap"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init"))))
  (label 1022)
  (parent 376)
  (seen 1781)
  (unrealized (4 3) (5 0))
  (comment "4 in cohort - 3 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (3 2)) ((3 4) (0 0)) ((4 1) (5 0))
    ((4 2) (3 0)) ((4 2) (6 0)) ((5 1) (6 2)) ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k "wrap")
    (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 7 4 make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-4 k "decrypt"))
      (load lk-0 (cat pt-5 k1 "wrap")) (send (enc k k1))))
  (label 1023)
  (parent 380)
  (seen 1786 1787 1788 1789 1790)
  (unrealized (6 3))
  (comment "18 in cohort - 5 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (precedes ((1 1) (2 2)) ((2 4) (0 0)) ((3 1) (4 0)) ((3 2) (2 0))
    ((4 1) (5 2)) ((5 4) (0 0)) ((6 1) (1 0)) ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k "wrap")
    (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 1 7 make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-5 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt k "wrap"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init"))
      (send (hash k)))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-2 k "decrypt"))
      (load lk (cat pt-3 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-4 old-0)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k))))
  (label 1024)
  (parent 380)
  (seen 1791 1792 1793 1794)
  (unrealized (5 3))
  (comment "11 in cohort - 4 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (3 2)) ((2 1) (6 2)) ((3 4) (0 0))
    ((4 1) (5 0)) ((4 2) (3 0)) ((4 2) (6 0)) ((5 1) (0 0))
    ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 7 4 make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-5 k1 "wrap")) (send (enc k k1))))
  (label 1027)
  (parent 382)
  (seen 153)
  (unrealized (6 3))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (precedes ((1 1) (2 2)) ((1 1) (5 2)) ((2 4) (0 0)) ((3 1) (4 0))
    ((3 2) (2 0)) ((4 1) (0 0)) ((5 4) (0 0)) ((6 1) (1 0))
    ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 1 7 make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-5 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt k "wrap"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init"))
      (send (hash k)))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt k "wrap"))
      (load lk-0 (cat pt-3 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-4 old-0)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k))))
  (label 1028)
  (parent 382)
  (seen 1772 1795)
  (unrealized (5 3))
  (comment "4 in cohort - 1 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (3 2)) ((3 4) (0 0)) ((4 1) (5 0))
    ((4 2) (3 0)) ((4 2) (6 0)) ((5 1) (0 0)) ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 7 4 make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-5 k1 "wrap")) (send (enc k k1))))
  (label 1031)
  (parent 383)
  (seen 1796 1797 1798 1799)
  (unrealized (6 3))
  (comment "18 in cohort - 4 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (precedes ((1 1) (2 2)) ((2 4) (0 0)) ((3 1) (4 0)) ((3 1) (5 2))
    ((3 2) (2 0)) ((4 1) (0 0)) ((5 4) (0 0)) ((6 1) (1 0))
    ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 1 7 make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-5 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt k "wrap"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init"))
      (send (hash k)))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt-3 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-4 old-0)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k))))
  (label 1032)
  (parent 383)
  (seen 1800)
  (unrealized (5 3))
  (comment "4 in cohort - 1 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (6 2)) ((2 1) (3 2)) ((3 4) (0 0))
    ((4 1) (5 0)) ((4 2) (3 0)) ((4 2) (6 0)) ((5 1) (0 0))
    ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 7 4 make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-5 k1 "wrap")) (send (enc k k1))))
  (label 1035)
  (parent 384)
  (seen 1801 1802)
  (unrealized (6 3))
  (comment "7 in cohort - 2 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (precedes ((1 1) (2 2)) ((2 4) (0 0)) ((3 1) (4 0)) ((3 2) (2 0))
    ((4 1) (0 0)) ((5 4) (0 0)) ((6 1) (1 0)) ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 1 7 make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-5 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt k "wrap"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init"))
      (send (hash k)))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-5 k "init"))
      (load lk-0 (cat pt-3 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-4 old-0)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k))))
  (label 1036)
  (parent 384)
  (seen 1803 1804)
  (unrealized (5 3))
  (comment "7 in cohort - 2 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand decrypt 4 (x k) (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (3 2)) ((3 4) (6 0)) ((4 1) (5 0))
    ((4 2) (3 0)) ((5 1) (6 2)) ((6 3) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 7 3 wrap 5) (enc k k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((recv (enc k k)) (recv (hash k)) (load lk-0 (cat pt-4 k "decrypt"))
      (send k)))
  (label 1039)
  (parent 385)
  (realized)
  (shape)
  (maps ((0) ((k k))))
  (origs (pt-4 (5 1)) (pt-1 (2 1)) (pt-3 (4 1)) (pt-0 (1 1))))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (3 2)) ((3 4) (0 0)) ((4 1) (5 0))
    ((4 2) (3 0)) ((4 2) (6 0)) ((5 1) (6 2)) ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 7 4 make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-4 k "wrap"))
      (load lk-0 (cat pt-5 k1 "wrap")) (send (enc k k1))))
  (label 1042)
  (parent 389)
  (seen 66 1805 1806 1807 1808 1809 1810)
  (unrealized (6 3))
  (comment "21 in cohort - 6 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (precedes ((1 1) (2 2)) ((2 4) (0 0)) ((3 1) (4 0)) ((3 2) (2 0))
    ((4 1) (5 2)) ((5 4) (0 0)) ((6 1) (1 0)) ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 1 7 make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-5 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt k "wrap"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init"))
      (send (hash k)))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-2 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-2 k "wrap"))
      (load lk (cat pt-3 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-4 old-0)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k))))
  (label 1043)
  (parent 389)
  (seen 66 1811 1812 1813 1814)
  (unrealized (5 3))
  (comment "12 in cohort - 4 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (3 2)) ((3 4) (0 0)) ((4 1) (5 0))
    ((4 2) (3 0)) ((4 2) (6 0)) ((5 1) (0 0)) ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 7 4 make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-5 k1 "wrap")) (send (enc k k1))))
  (label 1046)
  (parent 391)
  (seen 1013 1815 1816 1817 1818 1819)
  (unrealized (6 3))
  (comment "23 in cohort - 5 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (precedes ((1 1) (2 2)) ((2 4) (0 0)) ((3 1) (4 0)) ((3 1) (5 2))
    ((3 2) (2 0)) ((4 1) (0 0)) ((5 4) (0 0)) ((6 1) (1 0))
    ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 1 7 make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-5 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt k "wrap"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init"))
      (send (hash k)))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-2 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt-3 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-4 old-0)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k))))
  (label 1047)
  (parent 391)
  (seen 1013 1821 1822)
  (unrealized (5 3))
  (comment "15 in cohort - 3 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (3 2)) ((2 1) (6 2)) ((3 4) (0 0))
    ((4 1) (5 0)) ((4 2) (3 0)) ((4 2) (6 0)) ((5 1) (0 0))
    ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 7 4 make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-5 k1 "wrap")) (send (enc k k1))))
  (label 1050)
  (parent 392)
  (seen 155)
  (unrealized (6 3))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (precedes ((1 1) (2 2)) ((1 1) (5 2)) ((2 4) (0 0)) ((3 1) (4 0))
    ((3 2) (2 0)) ((4 1) (0 0)) ((5 4) (0 0)) ((6 1) (1 0))
    ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 1 7 make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-5 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt k "wrap"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init"))
      (send (hash k)))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-2 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt k "wrap"))
      (load lk-0 (cat pt-3 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-4 old-0)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k))))
  (label 1051)
  (parent 392)
  (seen 66 1774 1824 1825)
  (unrealized (5 3))
  (comment "10 in cohort - 3 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (6 2)) ((2 1) (3 2)) ((3 4) (0 0))
    ((4 1) (5 0)) ((4 2) (3 0)) ((4 2) (6 0)) ((5 1) (0 0))
    ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 7 4 make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-5 k1 "wrap")) (send (enc k k1))))
  (label 1054)
  (parent 393)
  (seen 1013 1826 1827 1828 1830)
  (unrealized (6 3))
  (comment "18 in cohort - 5 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (precedes ((1 1) (2 2)) ((2 4) (0 0)) ((3 1) (4 0)) ((3 2) (2 0))
    ((4 1) (0 0)) ((5 4) (0 0)) ((6 1) (1 0)) ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 1 7 make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-5 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt k "wrap"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init"))
      (send (hash k)))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-2 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-5 k "init"))
      (load lk-0 (cat pt-3 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-4 old-0)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k))))
  (label 1055)
  (parent 393)
  (seen 1013 1831 1832 1833 1834)
  (unrealized (5 3))
  (comment "17 in cohort - 4 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pval) (k skey)
    (lk locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (3 2)) ((2 1) (5 3)) ((3 4) (0 0))
    ((4 1) (1 0)) ((4 2) (3 0)) ((4 2) (5 0)) ((5 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule invShearsRule)
  (operation channel-test (displaced 6 2 set-wrap 2)
    (ch-msg lk-0 (cat pt-4 k1 "wrap")) (5 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k))))
  (label 1058)
  (parent 394)
  (seen 1013 1706 1796 1815)
  (unrealized (0 0))
  (comment "19 in cohort - 5 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pval) (k skey)
    (lk locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (3 2)) ((2 1) (5 3)) ((3 4) (0 0))
    ((4 1) (2 0)) ((4 2) (3 0)) ((4 2) (5 0)) ((5 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule invShearsRule)
  (operation channel-test (displaced 6 2 set-wrap 2)
    (ch-msg lk-0 (cat pt-4 k1 "wrap")) (5 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k))))
  (label 1059)
  (parent 394)
  (seen 1013 1058 1665 1706 1756 1757 1796 1815 1837 1838 1839)
  (unrealized (0 0))
  (comment "24 in cohort - 0 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 cur mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (defstrand set-wrap 2 (cur cur) (k k1) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (3 2)) ((3 4) (0 0)) ((4 1) (6 0))
    ((4 2) (3 0)) ((4 2) (5 0)) ((5 4) (0 0)) ((6 1) (5 3)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 cur) (cat k1 "wrap"))
  (facts (neq cur "decrypt") (neq cur "wrap") (neq "init" "decrypt")
    (neq "init" "wrap"))
  (rule invShearsRule fact-set-decrypt-neq0 trRl_set-wrap-at-1
    gen-st-set-wrap-0 fact-set-wrap-neq0 trRl_make-at-0)
  (operation channel-test (added-strand set-wrap 2)
    (ch-msg lk-0 (cat pt-4 k1 "wrap")) (5 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-5 k1 cur)) (stor lk-0 (cat pt-4 k1 "wrap"))))
  (label 1060)
  (parent 394)
  (seen 1013 1058 1818 1840)
  (unrealized (6 0))
  (comment "7 in cohort - 1 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pval) (k skey)
    (lk locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (precedes ((1 1) (2 2)) ((1 1) (4 3)) ((2 4) (0 0)) ((3 1) (1 0))
    ((3 1) (4 2)) ((3 2) (2 0)) ((4 4) (0 0)) ((5 1) (1 0))
    ((5 2) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule invShearsRule)
  (operation channel-test (displaced 6 1 set-wrap 2)
    (ch-msg lk-0 (cat pt-4 k1 "wrap")) (4 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt-3 k "init")) (stor lk (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt k "wrap"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init"))
      (send (hash k))))
  (label 1061)
  (parent 395)
  (seen 1013 1665 1756 1757)
  (unrealized (0 0))
  (comment "11 in cohort - 0 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 cur mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur cur) (k k1) (lk lk))
  (precedes ((1 1) (2 2)) ((2 4) (0 0)) ((3 1) (4 2)) ((3 1) (6 0))
    ((3 2) (2 0)) ((4 4) (0 0)) ((5 1) (1 0)) ((5 2) (4 0))
    ((6 1) (4 3)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 cur) (cat k1 "wrap"))
  (facts (neq cur "decrypt") (neq cur "wrap") (neq "init" "decrypt")
    (neq "init" "wrap"))
  (rule invShearsRule fact-set-decrypt-neq0 trRl_set-wrap-at-1
    gen-st-set-wrap-0 fact-set-wrap-neq0 trRl_make-at-0)
  (operation channel-test (added-strand set-wrap 2)
    (ch-msg lk (cat pt-2 k1 "wrap")) (4 3))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt k "wrap"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init"))
      (send (hash k)))
    ((load lk (cat pt-5 k1 cur)) (stor lk (cat pt-2 k1 "wrap"))))
  (label 1062)
  (parent 395)
  (seen 1013 1821 1841)
  (unrealized (6 0))
  (comment "6 in cohort - 1 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (3 2)) ((2 1) (5 3)) ((3 4) (0 0))
    ((4 1) (1 0)) ((4 1) (5 2)) ((4 2) (3 0)) ((5 4) (0 0))
    ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule invShearsRule)
  (operation channel-test (displaced 7 2 set-wrap 2)
    (ch-msg lk-1 (cat pt-6 k1 "wrap")) (5 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-4 old-1)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k))))
  (label 1063)
  (parent 396)
  (seen 1015 1846 1848)
  (unrealized (0 0))
  (comment "22 in cohort - 11 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (3 2)) ((2 1) (5 3)) ((3 4) (0 0))
    ((4 1) (2 0)) ((4 1) (5 2)) ((4 2) (3 0)) ((5 4) (0 0))
    ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule invShearsRule)
  (operation channel-test (displaced 7 2 set-wrap 2)
    (ch-msg lk-1 (cat pt-6 k1 "wrap")) (5 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-4 old-1)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k))))
  (label 1064)
  (parent 396)
  (seen 1015 1063 1667 1759 1761 1763 1764 1765 1842 1844 1845 1846 1848
    1849 1850 1851 1852)
  (unrealized (0 0))
  (comment "33 in cohort - 0 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (deflistener k)
  (precedes ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (3 2)) ((3 4) (0 0))
    ((4 1) (5 2)) ((4 2) (3 0)) ((5 4) (0 0)) ((6 1) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation nonce-test (displaced 7 1 make 2) k (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k)))
  (label 1066)
  (parent 397)
  (seen 1853)
  (unrealized (5 3) (6 0))
  (comment "5 in cohort - 4 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (deflistener k)
  (precedes ((1 1) (2 0)) ((2 1) (3 2)) ((3 4) (0 0)) ((4 1) (6 0))
    ((4 2) (3 0)) ((5 4) (0 0)) ((6 1) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation nonce-test (displaced 7 4 make 2) k (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k)))
  (label 1067)
  (parent 397)
  (seen 1855)
  (unrealized (5 3) (6 0))
  (comment "3 in cohort - 2 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 cur mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur cur) (k k1) (lk lk-0))
  (precedes ((1 1) (2 2)) ((1 1) (4 2)) ((1 1) (6 0)) ((2 4) (0 0))
    ((3 2) (2 0)) ((4 4) (0 0)) ((5 1) (1 0)) ((5 2) (4 0))
    ((6 1) (4 3)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 cur) (cat k1 "wrap"))
  (facts (neq cur "decrypt") (neq cur "wrap") (neq "init" "decrypt")
    (neq "init" "wrap"))
  (rule invShearsRule fact-set-decrypt-neq0 trRl_set-wrap-at-1
    gen-st-set-wrap-0 fact-set-wrap-neq0 trRl_make-at-0)
  (operation channel-test (added-strand set-wrap 2)
    (ch-msg lk-0 (cat pt-2 k1 "wrap")) (4 3))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt k "wrap"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt k "wrap"))
      (load lk-0 (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-5 k1 cur)) (stor lk-0 (cat pt-2 k1 "wrap"))))
  (label 1069)
  (parent 399)
  (seen 1859)
  (unrealized (6 0))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (precedes ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (3 2)) ((2 1) (5 2))
    ((3 4) (0 0)) ((4 2) (3 0)) ((5 4) (0 0)) ((6 1) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation nonce-test (displaced 7 1 make 2) k (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k)))
  (label 1071)
  (parent 401)
  (seen 1860)
  (unrealized (5 3) (6 0))
  (comment "5 in cohort - 4 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (precedes ((1 1) (2 0)) ((2 1) (3 2)) ((2 1) (5 2)) ((3 4) (0 0))
    ((4 1) (6 0)) ((4 2) (3 0)) ((5 4) (0 0)) ((6 1) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation nonce-test (displaced 7 4 make 2) k (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k)))
  (label 1072)
  (parent 401)
  (seen 1862)
  (unrealized (5 3) (6 0))
  (comment "5 in cohort - 4 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pval) (k skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (5 2)) ((2 1) (3 2)) ((2 1) (5 3))
    ((3 4) (0 0)) ((4 2) (3 0)) ((4 2) (5 0)) ((5 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation channel-test (displaced 6 2 set-wrap 2)
    (ch-msg lk (cat pt-4 k1 "wrap")) (5 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k))))
  (label 1074)
  (parent 402)
  (seen 1715 1801 1826)
  (unrealized (0 0))
  (comment "8 in cohort - 5 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 cur mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (defstrand set-wrap 2 (cur cur) (k k1) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (5 2)) ((2 1) (3 2)) ((2 1) (6 0))
    ((3 4) (0 0)) ((4 2) (3 0)) ((4 2) (5 0)) ((5 4) (0 0))
    ((6 1) (5 3)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 cur) (cat k1 "wrap"))
  (facts (neq cur "decrypt") (neq cur "wrap") (neq "init" "decrypt")
    (neq "init" "wrap"))
  (rule shearsRule invShearsRule fact-set-decrypt-neq0
    trRl_set-wrap-at-1 gen-st-set-wrap-0 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation channel-test (added-strand set-wrap 2)
    (ch-msg lk (cat pt-4 k1 "wrap")) (5 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((load lk (cat pt-5 k1 cur)) (stor lk (cat pt-4 k1 "wrap"))))
  (label 1075)
  (parent 402)
  (seen 1873)
  (unrealized (6 0))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pval) (k skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (precedes ((1 1) (2 2)) ((1 1) (4 3)) ((2 4) (0 0)) ((3 2) (2 0))
    ((4 4) (0 0)) ((5 1) (1 0)) ((5 2) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation channel-test (displaced 6 1 set-wrap 2)
    (ch-msg lk-0 (cat pt-4 k1 "wrap")) (4 3))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt k "wrap"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k))))
  (label 1076)
  (parent 403)
  (seen 1713 1803 1831)
  (unrealized (0 0))
  (comment "8 in cohort - 5 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 cur mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur cur) (k k1) (lk lk-0))
  (precedes ((1 1) (2 2)) ((1 1) (6 0)) ((2 4) (0 0)) ((3 2) (2 0))
    ((4 4) (0 0)) ((5 1) (1 0)) ((5 2) (4 0)) ((6 1) (4 3)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 cur) (cat k1 "wrap"))
  (facts (neq cur "decrypt") (neq cur "wrap") (neq "init" "decrypt")
    (neq "init" "wrap"))
  (rule shearsRule invShearsRule fact-set-decrypt-neq0
    trRl_set-wrap-at-1 gen-st-set-wrap-0 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation channel-test (added-strand set-wrap 2)
    (ch-msg lk-0 (cat pt-2 k1 "wrap")) (4 3))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt k "wrap"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-4 k "init"))
      (load lk-0 (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-5 k1 cur)) (stor lk-0 (cat pt-2 k1 "wrap"))))
  (label 1077)
  (parent 403)
  (seen 1879)
  (unrealized (6 0))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k skey) (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((1 1) (5 2)) ((2 1) (3 2)) ((2 1) (5 3))
    ((3 4) (0 0)) ((4 2) (3 0)) ((5 4) (0 0)) ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation channel-test (displaced 7 2 set-wrap 2)
    (ch-msg lk (cat pt-6 k1 "wrap")) (5 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-1 (cat pt-4 old-1)) (stor lk-1 (cat pt-5 k "init"))
      (send (hash k))))
  (label 1078)
  (parent 404)
  (unrealized (0 0))
  (comment "11 in cohort - 11 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (precedes ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (3 2)) ((3 4) (0 0))
    ((4 2) (3 0)) ((5 4) (0 0)) ((6 1) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation nonce-test (displaced 7 1 make 2) k (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k)))
  (label 1080)
  (parent 405)
  (unrealized (5 3) (6 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (precedes ((1 1) (2 0)) ((1 1) (5 2)) ((2 1) (3 2)) ((3 4) (0 0))
    ((4 1) (6 0)) ((4 2) (3 0)) ((5 4) (0 0)) ((6 1) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation nonce-test (displaced 7 4 make 2) k (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k)))
  (label 1081)
  (parent 405)
  (seen 1892)
  (unrealized (5 3) (6 0))
  (comment "5 in cohort - 4 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (4 0)) ((3 4) (0 0)) ((4 1) (3 0))
    ((5 1) (6 0)) ((6 1) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-decrypt-at-1 trRl_make-at-0)
  (operation nonce-test (added-strand set-decrypt 2) k (4 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-4 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((load lk-0 (cat pt-4 k "init"))
      (stor lk-0 (cat pt-5 k "decrypt"))))
  (label 1083)
  (parent 406)
  (unrealized (3 3) (4 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (4 0)) ((3 4) (0 0)) ((4 1) (3 0))
    ((5 1) (6 0)) ((6 1) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-wrap-at-1 trRl_make-at-0)
  (operation nonce-test (added-strand set-wrap 2) k (4 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-4 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt-5 k "wrap"))))
  (label 1084)
  (parent 406)
  (unrealized (3 3) (4 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (4 0)) ((3 4) (0 0)) ((4 1) (3 0))
    ((5 1) (6 2)) ((6 4) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap")
    (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (4 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-4 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-0 (cat pt-4 k "init"))
      (load lk-0 (cat pt-5 k1-0 "wrap")) (send (enc k k1-0))))
  (label 1085)
  (parent 406)
  (unrealized (3 3) (6 0) (6 3))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1-0) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (6 2)) ((3 4) (0 0)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 4) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap")
    (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (4 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-4 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-5 k1-0 "wrap")) (send (enc k k1-0))))
  (label 1086)
  (parent 406)
  (seen 1089)
  (unrealized (3 3) (6 0) (6 3))
  (comment "4 in cohort - 3 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (6 2)) ((2 1) (4 0)) ((3 4) (0 0))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 4) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap")
    (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (4 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-4 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-5 k1-0 "wrap")) (send (enc k k1-0))))
  (label 1087)
  (parent 406)
  (seen 1092)
  (unrealized (3 3) (6 0) (6 3))
  (comment "4 in cohort - 3 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old mesg) (pt pt-0 pt-1 pt-2 pt-3 pval) (k k1 k1-0 skey)
    (lk locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1-0) (lk lk))
  (defstrand make 3 (old old) (k k) (lk lk))
  (precedes ((1 1) (4 2)) ((2 4) (0 0)) ((3 1) (2 0)) ((4 4) (3 0))
    ((5 1) (1 0)) ((5 2) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap")
    (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 1 6 make 3) (hash k) (5 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt-3 k "init")) (stor lk (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt k "wrap"))
      (load lk (cat pt-0 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((recv (hash k)) (recv (hash k1-0)) (load lk (cat pt k "wrap"))
      (load lk (cat pt-1 k1-0 "wrap")) (send (enc k k1-0)))
    ((load lk (cat pt-2 old)) (stor lk (cat pt-3 k "init"))
      (send (hash k))))
  (label 1088)
  (parent 407)
  (seen 1922)
  (unrealized (2 3) (4 3))
  (comment "3 in cohort - 1 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1-0) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (5 2)) ((3 4) (0 0)) ((4 1) (3 0))
    ((5 4) (4 0)) ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap")
    (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation encryption-test (added-strand make 3) (hash k) (5 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((recv (hash k)) (recv (hash k1-0)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-3 k1-0 "wrap")) (send (enc k k1-0)))
    ((load lk-0 (cat pt-4 old-0)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k))))
  (label 1089)
  (parent 407)
  (seen 1923)
  (unrealized (3 3) (5 3))
  (comment "3 in cohort - 1 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old mesg) (pt pt-0 pt-1 pt-2 pt-3 pval) (k k1 k1-0 skey)
    (lk locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1-0) (lk lk))
  (deflistener k)
  (precedes ((1 1) (2 0)) ((2 1) (5 2)) ((3 4) (0 0)) ((4 1) (3 0))
    ((5 4) (4 0)) ((6 1) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap")
    (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (added-listener k) (hash k) (5 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((recv (hash k)) (recv (hash k1-0)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-3 k1-0 "wrap")) (send (enc k k1-0)))
    ((recv k) (send k)))
  (label 1090)
  (parent 407)
  (unrealized (3 3) (5 3) (6 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old mesg) (pt pt-0 pt-1 pt-2 pt-3 pval) (k k1 k1-0 skey)
    (lk locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk))
  (defstrand make 3 (old old) (k k) (lk lk))
  (precedes ((1 1) (3 0)) ((2 4) (0 0)) ((3 1) (2 0)) ((4 4) (3 0))
    ((5 1) (1 0)) ((5 2) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap")
    (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 1 6 make 3) (hash k) (5 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt-3 k "init")) (stor lk (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt k "wrap"))
      (load lk (cat pt-0 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((recv (hash k)) (recv (hash k1-0)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k1-0 "wrap")) (send (enc k k1-0)))
    ((load lk (cat pt-2 old)) (stor lk (cat pt-3 k "init"))
      (send (hash k))))
  (label 1091)
  (parent 408)
  (seen 1112)
  (unrealized (2 3) (4 3))
  (comment "2 in cohort - 0 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((1 1) (5 2)) ((2 1) (4 0)) ((3 4) (0 0))
    ((4 1) (3 0)) ((5 4) (4 0)) ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap")
    (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation encryption-test (added-strand make 3) (hash k) (5 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((recv (hash k)) (recv (hash k1-0)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-3 k1-0 "wrap")) (send (enc k k1-0)))
    ((load lk-0 (cat pt-4 old-0)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k))))
  (label 1092)
  (parent 408)
  (seen 1113)
  (unrealized (3 3) (5 3))
  (comment "2 in cohort - 0 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old mesg) (pt pt-0 pt-1 pt-2 pt-3 pval) (k k1 k1-0 skey)
    (lk locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk))
  (deflistener k)
  (precedes ((1 1) (2 0)) ((1 1) (5 2)) ((2 1) (4 0)) ((3 4) (0 0))
    ((4 1) (3 0)) ((5 4) (4 0)) ((6 1) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap")
    (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (added-listener k) (hash k) (5 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((recv (hash k)) (recv (hash k1-0)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-3 k1-0 "wrap")) (send (enc k k1-0)))
    ((recv k) (send k)))
  (label 1093)
  (parent 408)
  (unrealized (3 3) (5 3) (6 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (3 2)) ((3 4) (0 0))
    ((4 1) (3 0)) ((5 4) (4 0)) ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap")
    (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 5 7 make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-0 (cat pt-5 k "init"))
      (load lk-0 (cat pt-3 k1-0 "wrap")) (send (enc k k1-0)))
    ((load lk-0 (cat pt-4 old-0)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k))))
  (label 1106)
  (parent 411)
  (seen 1112 1116 1928)
  (unrealized (3 3) (5 3))
  (comment "14 in cohort - 1 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk-0))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (precedes ((1 1) (2 2)) ((2 4) (0 0)) ((3 1) (2 0)) ((4 1) (3 0))
    ((5 4) (3 0)) ((6 1) (1 0)) ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap")
    (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 1 7 make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-5 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt k "wrap"))
      (load lk-0 (cat pt-0 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk (cat pt-1 old)) (stor lk (cat pt-2 k "init")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-0 (cat pt-5 k "init"))
      (load lk-0 (cat pt-3 k1-0 "wrap")) (send (enc k k1-0)))
    ((load lk-0 (cat pt-4 old-0)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k))))
  (label 1109)
  (parent 412)
  (seen 1929)
  (unrealized (2 3) (5 3))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old mesg) (pt pt-0 pt-1 pt-2 pval) (k k1 skey) (lk locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old) (k k) (lk lk))
  (precedes ((1 1) (4 3)) ((2 4) (0 0)) ((3 1) (2 0)) ((4 4) (3 0))
    ((5 1) (1 0)) ((5 2) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation channel-test (displaced 6 1 set-wrap 2)
    (ch-msg lk (cat pt-3 k1-0 "wrap")) (4 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt-2 k "init")) (stor lk (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt k "wrap"))
      (load lk (cat pt-0 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-2 k "init"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-1 old)) (stor lk (cat pt-2 k "init"))
      (send (hash k))))
  (label 1112)
  (parent 413)
  (seen 1929)
  (unrealized (2 3) (3 0))
  (comment "3 in cohort - 2 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((1 1) (5 2)) ((2 1) (5 3)) ((3 4) (0 0))
    ((4 1) (3 0)) ((5 4) (4 0)) ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation channel-test (displaced 7 2 set-wrap 2)
    (ch-msg lk (cat pt-5 k1-0 "wrap")) (5 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init"))
      (send (hash k))))
  (label 1113)
  (parent 414)
  (unrealized (3 3) (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old mesg) (pt pt-0 pt-1 pt-2 pt-3 pval) (k k1 k1-0 skey)
    (lk locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk))
  (deflistener k)
  (precedes ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (3 2)) ((3 4) (0 0))
    ((4 1) (3 0)) ((5 4) (4 0)) ((6 1) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap")
    (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation nonce-test (displaced 7 1 make 2) k (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((recv (hash k)) (recv (hash k1-0)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-3 k1-0 "wrap")) (send (enc k k1-0)))
    ((recv k) (send k)))
  (label 1114)
  (parent 415)
  (unrealized (3 3) (5 3) (6 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k k1 skey)
    (lk locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (5 3)) ((3 4) (0 0)) ((4 1) (3 0))
    ((5 4) (4 0)) ((6 1) (1 0)) ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule invShearsRule)
  (operation channel-test (displaced 7 2 set-wrap 2)
    (ch-msg lk-0 (cat pt-5 k1-0 "wrap")) (5 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-4 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-3 old-0)) (stor lk (cat pt-4 k "init"))
      (send (hash k))))
  (label 1116)
  (parent 426)
  (seen 1112 1940 1942)
  (unrealized (3 3) (4 0))
  (comment "17 in cohort - 6 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pval) (k skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (precedes ((1 1) (2 3)) ((1 1) (4 3)) ((2 4) (0 0)) ((3 1) (1 0))
    ((3 1) (4 2)) ((3 2) (2 0)) ((4 4) (0 0)) ((5 2) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation channel-test (displaced 6 1 set-wrap 2)
    (ch-msg lk (cat pt-4 k1 "wrap")) (4 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k))))
  (label 1119)
  (parent 431)
  (seen 1729)
  (unrealized (0 0))
  (comment "8 in cohort - 7 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 cur mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur cur) (k k1) (lk lk))
  (precedes ((1 1) (2 3)) ((1 1) (6 0)) ((2 4) (0 0)) ((3 1) (1 0))
    ((3 1) (4 2)) ((3 2) (2 0)) ((4 4) (0 0)) ((5 2) (4 0))
    ((6 1) (4 3)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 cur) (cat k1 "wrap"))
  (facts (neq cur "decrypt") (neq cur "wrap") (neq "init" "decrypt")
    (neq "init" "wrap"))
  (rule shearsRule invShearsRule fact-set-decrypt-neq0
    trRl_set-wrap-at-1 gen-st-set-wrap-0 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation channel-test (added-strand set-wrap 2)
    (ch-msg lk (cat pt-2 k1 "wrap")) (4 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init"))
      (send (hash k)))
    ((load lk (cat pt-5 k1 cur)) (stor lk (cat pt-2 k1 "wrap"))))
  (label 1120)
  (parent 431)
  (seen 1953)
  (unrealized (6 0))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old mesg) (pt pt-0 pt-1 pt-2 pval) (k k1 skey) (lk locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (precedes ((1 1) (2 3)) ((2 4) (0 0)) ((3 1) (1 0)) ((3 1) (5 0))
    ((3 2) (2 0)) ((4 4) (0 0)) ((5 1) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation nonce-test (displaced 6 3 make 2) k (5 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k)))
  (label 1121)
  (parent 432)
  (unrealized (4 3) (5 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (precedes ((1 1) (2 3)) ((2 4) (0 0)) ((3 1) (1 0)) ((3 1) (4 2))
    ((3 2) (2 0)) ((4 4) (0 0)) ((5 1) (4 0)) ((6 1) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation nonce-test (added-strand make 2) k (5 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init"))))
  (label 1122)
  (parent 432)
  (seen 1954)
  (unrealized (4 3) (5 0))
  (comment "4 in cohort - 3 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old cur mesg) (pt pt-0 pt-1 pt-2 pt-3 pval) (k k1 skey)
    (lk locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (defstrand set-wrap 2 (cur cur) (k k1) (lk lk))
  (precedes ((1 1) (2 3)) ((1 1) (4 2)) ((1 1) (5 0)) ((2 4) (0 0))
    ((3 1) (1 0)) ((3 2) (2 0)) ((3 2) (4 0)) ((4 4) (0 0))
    ((5 1) (4 3)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 cur) (cat k1 "wrap"))
  (facts (neq cur "decrypt") (neq cur "wrap") (neq "init" "decrypt")
    (neq "init" "wrap"))
  (rule invShearsRule fact-set-decrypt-neq0 trRl_set-wrap-at-1
    gen-st-set-wrap-0 fact-set-wrap-neq0 trRl_make-at-0)
  (operation channel-test (added-strand set-wrap 2)
    (ch-msg lk (cat pt-2 k1 "wrap")) (4 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt k "wrap"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((load lk (cat pt-3 k1 cur)) (stor lk (cat pt-2 k1 "wrap"))))
  (label 1123)
  (parent 433)
  (seen 1959)
  (unrealized (5 0))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 cur mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur cur) (k k1) (lk lk))
  (precedes ((1 1) (2 3)) ((1 1) (4 2)) ((1 1) (6 0)) ((2 4) (0 0))
    ((3 1) (1 0)) ((3 2) (2 0)) ((4 4) (0 0)) ((5 2) (4 0))
    ((6 1) (4 3)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 cur) (cat k1 "wrap"))
  (facts (neq cur "decrypt") (neq cur "wrap") (neq "init" "decrypt")
    (neq "init" "wrap"))
  (rule invShearsRule fact-set-decrypt-neq0 trRl_set-wrap-at-1
    gen-st-set-wrap-0 fact-set-wrap-neq0 trRl_make-at-0)
  (operation channel-test (added-strand set-wrap 2)
    (ch-msg lk (cat pt-2 k1 "wrap")) (4 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt k "wrap"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init"))
      (send (hash k)))
    ((load lk (cat pt-5 k1 cur)) (stor lk (cat pt-2 k1 "wrap"))))
  (label 1124)
  (parent 434)
  (seen 1960)
  (unrealized (6 0))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old mesg) (pt pt-0 pt-1 pt-2 pval) (k k1 skey) (lk locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (precedes ((1 1) (2 3)) ((1 1) (4 2)) ((2 4) (0 0)) ((3 1) (1 0))
    ((3 1) (5 0)) ((3 2) (2 0)) ((4 4) (0 0)) ((5 1) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation nonce-test (displaced 6 3 make 2) k (5 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt k "wrap"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k)))
  (label 1125)
  (parent 435)
  (seen 1961)
  (unrealized (4 3) (5 0))
  (comment "4 in cohort - 3 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (precedes ((1 1) (2 3)) ((1 1) (4 2)) ((2 4) (0 0)) ((3 1) (1 0))
    ((3 2) (2 0)) ((4 4) (0 0)) ((5 1) (4 0)) ((6 1) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation nonce-test (added-strand make 2) k (5 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt k "wrap"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init"))))
  (label 1126)
  (parent 435)
  (seen 1962)
  (unrealized (4 3) (5 0))
  (comment "4 in cohort - 3 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1) (lk lk-0))
  (precedes ((1 1) (2 0)) ((1 1) (3 2)) ((2 1) (3 3)) ((3 4) (0 0))
    ((4 1) (5 0)) ((4 2) (3 0)) ((4 2) (6 0)) ((5 1) (6 2))
    ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k "wrap")
    (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 7 4 make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-4 k "decrypt"))
      (load lk-0 (cat pt-5 k1 "wrap")) (send (enc k k1))))
  (label 1127)
  (parent 439)
  (seen 1967 1968 1969 1971 1972)
  (unrealized (6 3))
  (comment "22 in cohort - 6 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (precedes ((1 1) (2 3)) ((2 4) (0 0)) ((3 1) (4 0)) ((3 2) (2 0))
    ((4 1) (5 2)) ((5 4) (0 0)) ((6 1) (1 0)) ((6 1) (2 2))
    ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k "wrap")
    (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 1 7 make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-5 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt-5 k "init"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init"))
      (send (hash k)))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-2 k "decrypt"))
      (load lk (cat pt-3 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-4 old-0)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k))))
  (label 1128)
  (parent 439)
  (seen 1973 1975 1976 1977)
  (unrealized (5 3))
  (comment "15 in cohort - 5 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (3 2)) ((2 1) (3 3)) ((2 1) (6 2))
    ((3 4) (0 0)) ((4 1) (5 0)) ((4 2) (3 0)) ((4 2) (6 0))
    ((5 1) (0 0)) ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 7 4 make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-5 k1 "wrap")) (send (enc k k1))))
  (label 1131)
  (parent 441)
  (seen 1801 1978)
  (unrealized (6 3))
  (comment "4 in cohort - 1 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (precedes ((1 1) (2 3)) ((1 1) (5 2)) ((2 4) (0 0)) ((3 1) (4 0))
    ((3 2) (2 0)) ((4 1) (0 0)) ((5 4) (0 0)) ((6 1) (1 0))
    ((6 1) (2 2)) ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 1 7 make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-5 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt-5 k "init"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init"))
      (send (hash k)))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt k "wrap"))
      (load lk-0 (cat pt-3 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-4 old-0)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k))))
  (label 1132)
  (parent 441)
  (seen 1759 1979)
  (unrealized (5 3))
  (comment "4 in cohort - 1 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (precedes ((1 1) (2 0)) ((1 1) (3 2)) ((2 1) (3 3)) ((3 4) (0 0))
    ((4 1) (5 0)) ((4 2) (3 0)) ((4 2) (6 0)) ((5 1) (0 0))
    ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 7 4 make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-5 k1 "wrap")) (send (enc k k1))))
  (label 1135)
  (parent 442)
  (seen 1980 1981 1982 1984)
  (unrealized (6 3))
  (comment "22 in cohort - 5 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (precedes ((1 1) (2 3)) ((2 4) (0 0)) ((3 1) (4 0)) ((3 1) (5 2))
    ((3 2) (2 0)) ((4 1) (0 0)) ((5 4) (0 0)) ((6 1) (1 0))
    ((6 1) (2 2)) ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 1 7 make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-5 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt-5 k "init"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init"))
      (send (hash k)))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt-3 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-4 old-0)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k))))
  (label 1136)
  (parent 442)
  (seen 1985)
  (unrealized (5 3))
  (comment "4 in cohort - 1 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (3 2)) ((1 1) (6 2)) ((2 1) (3 3))
    ((3 4) (0 0)) ((4 1) (5 0)) ((4 2) (3 0)) ((4 2) (6 0))
    ((5 1) (0 0)) ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 7 4 make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-5 k1 "wrap")) (send (enc k k1))))
  (label 1139)
  (parent 443)
  (seen 172)
  (unrealized (6 3))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (precedes ((1 1) (2 3)) ((2 4) (0 0)) ((3 1) (4 0)) ((3 2) (2 0))
    ((4 1) (0 0)) ((5 4) (0 0)) ((6 1) (1 0)) ((6 1) (2 2))
    ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 1 7 make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-5 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt-5 k "init"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init"))
      (send (hash k)))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-5 k "init"))
      (load lk-0 (cat pt-3 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-4 old-0)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k))))
  (label 1140)
  (parent 443)
  (seen 1946 1986)
  (unrealized (5 3))
  (comment "7 in cohort - 1 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand decrypt 4 (x k) (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((1 1) (3 2)) ((2 1) (3 3)) ((3 4) (6 0))
    ((4 1) (5 0)) ((4 2) (3 0)) ((5 1) (6 2)) ((6 3) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 7 3 wrap 5) (enc k k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((recv (enc k k)) (recv (hash k)) (load lk-0 (cat pt-4 k "decrypt"))
      (send k)))
  (label 1143)
  (parent 444)
  (realized)
  (shape)
  (maps ((0) ((k k))))
  (origs (pt-4 (5 1)) (pt-1 (2 1)) (pt-3 (4 1)) (pt-0 (1 1))))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk-0))
  (precedes ((1 1) (2 0)) ((1 1) (3 2)) ((2 1) (3 3)) ((3 4) (0 0))
    ((4 1) (5 0)) ((4 2) (3 0)) ((4 2) (6 0)) ((5 1) (6 2))
    ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 7 4 make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-4 k "wrap"))
      (load lk-0 (cat pt-5 k1 "wrap")) (send (enc k k1))))
  (label 1146)
  (parent 448)
  (seen 1013 1987 1988 1989 1991 1992 1993)
  (unrealized (6 3))
  (comment "25 in cohort - 7 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (precedes ((1 1) (2 3)) ((2 4) (0 0)) ((3 1) (4 0)) ((3 2) (2 0))
    ((4 1) (5 2)) ((5 4) (0 0)) ((6 1) (1 0)) ((6 1) (2 2))
    ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 1 7 make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-5 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt-5 k "init"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init"))
      (send (hash k)))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-2 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-2 k "wrap"))
      (load lk (cat pt-3 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-4 old-0)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k))))
  (label 1147)
  (parent 448)
  (seen 1013 1821 1994 1996 1997)
  (unrealized (5 3))
  (comment "12 in cohort - 4 not yet seen"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (precedes ((1 1) (2 0)) ((1 1) (3 2)) ((2 1) (3 3)) ((3 4) (0 0))
    ((4 1) (5 0)) ((4 2) (3 0)) ((4 2) (6 0)) ((5 1) (0 0))
    ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 7 4 make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-5 k1 "wrap")) (send (enc k k1))))
  (label 1150)
  (parent 450)
  (seen 72 1998 1999 2000 2002 2003)
  (unrealized (6 3))
  (comment "27 in cohort - 6 not yet seen"))

(comment "Step limit exceeded--aborting run")

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (precedes ((1 1) (2 3)) ((2 4) (0 0)) ((3 1) (4 0)) ((3 1) (5 2))
    ((3 2) (2 0)) ((4 1) (0 0)) ((5 4) (0 0)) ((6 1) (1 0))
    ((6 1) (2 2)) ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 1 7 make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-5 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt-5 k "init"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init"))
      (send (hash k)))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-2 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt-3 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-4 old-0)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k))))
  (label 1151)
  (parent 450)
  (unrealized (5 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((1 1) (3 2)) ((2 1) (3 3)) ((3 4) (0 0))
    ((4 1) (5 0)) ((4 1) (6 2)) ((4 2) (3 0)) ((5 1) (0 0))
    ((6 4) (0 0)) ((7 2) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation encryption-test (added-strand make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-5 k1 "wrap")) (send (enc k k1)))
    ((load lk-1 (cat pt-6 old-1)) (stor lk-1 (cat pt-7 k "init"))
      (send (hash k))))
  (label 1152)
  (parent 450)
  (unrealized (6 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (deflistener k)
  (precedes ((1 1) (2 0)) ((1 1) (3 2)) ((2 1) (3 3)) ((3 4) (0 0))
    ((4 1) (5 0)) ((4 1) (6 2)) ((4 2) (3 0)) ((5 1) (0 0))
    ((6 4) (0 0)) ((7 1) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (added-listener k) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-5 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k)))
  (label 1153)
  (parent 450)
  (unrealized (6 3) (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (3 2)) ((2 1) (3 3)) ((2 1) (6 2))
    ((3 4) (0 0)) ((4 1) (5 0)) ((4 2) (3 0)) ((4 2) (6 0))
    ((5 1) (0 0)) ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 7 4 make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-5 k1 "wrap")) (send (enc k k1))))
  (label 1154)
  (parent 451)
  (unrealized (6 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (precedes ((1 1) (2 3)) ((1 1) (5 2)) ((2 4) (0 0)) ((3 1) (4 0))
    ((3 2) (2 0)) ((4 1) (0 0)) ((5 4) (0 0)) ((6 1) (1 0))
    ((6 1) (2 2)) ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 1 7 make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-5 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt-5 k "init"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init"))
      (send (hash k)))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-2 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt k "wrap"))
      (load lk-0 (cat pt-3 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-4 old-0)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k))))
  (label 1155)
  (parent 451)
  (unrealized (5 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((1 1) (3 2)) ((2 1) (3 3)) ((2 1) (6 2))
    ((3 4) (0 0)) ((4 1) (5 0)) ((4 2) (3 0)) ((5 1) (0 0))
    ((6 4) (0 0)) ((7 2) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation encryption-test (added-strand make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-5 k1 "wrap")) (send (enc k k1)))
    ((load lk-1 (cat pt-6 old-1)) (stor lk-1 (cat pt-7 k "init"))
      (send (hash k))))
  (label 1156)
  (parent 451)
  (unrealized (6 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (precedes ((1 1) (2 0)) ((1 1) (3 2)) ((2 1) (3 3)) ((2 1) (6 2))
    ((3 4) (0 0)) ((4 1) (5 0)) ((4 2) (3 0)) ((5 1) (0 0))
    ((6 4) (0 0)) ((7 1) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (added-listener k) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-5 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k)))
  (label 1157)
  (parent 451)
  (unrealized (6 3) (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (3 2)) ((1 1) (6 2)) ((2 1) (3 3))
    ((3 4) (0 0)) ((4 1) (5 0)) ((4 2) (3 0)) ((4 2) (6 0))
    ((5 1) (0 0)) ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 7 4 make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-5 k1 "wrap")) (send (enc k k1))))
  (label 1158)
  (parent 452)
  (unrealized (6 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (precedes ((1 1) (2 3)) ((2 4) (0 0)) ((3 1) (4 0)) ((3 2) (2 0))
    ((4 1) (0 0)) ((5 4) (0 0)) ((6 1) (1 0)) ((6 1) (2 2))
    ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 1 7 make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-5 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt-5 k "init"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init"))
      (send (hash k)))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-2 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-5 k "init"))
      (load lk-0 (cat pt-3 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-4 old-0)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k))))
  (label 1159)
  (parent 452)
  (unrealized (5 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((1 1) (3 2)) ((1 1) (6 2)) ((2 1) (3 3))
    ((3 4) (0 0)) ((4 1) (5 0)) ((4 2) (3 0)) ((5 1) (0 0))
    ((6 4) (0 0)) ((7 2) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation encryption-test (added-strand make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-5 k1 "wrap")) (send (enc k k1)))
    ((load lk-1 (cat pt-6 old-1)) (stor lk-1 (cat pt-7 k "init"))
      (send (hash k))))
  (label 1160)
  (parent 452)
  (unrealized (6 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (precedes ((1 1) (2 0)) ((1 1) (3 2)) ((1 1) (6 2)) ((2 1) (3 3))
    ((3 4) (0 0)) ((4 1) (5 0)) ((4 2) (3 0)) ((5 1) (0 0))
    ((6 4) (0 0)) ((7 1) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (added-listener k) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-5 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k)))
  (label 1161)
  (parent 452)
  (unrealized (6 3) (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pval) (k skey)
    (lk locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (3 2)) ((2 1) (3 3)) ((2 1) (5 3))
    ((3 4) (0 0)) ((4 1) (1 0)) ((4 2) (3 0)) ((4 2) (5 0))
    ((5 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule invShearsRule)
  (operation channel-test (displaced 6 2 set-wrap 2)
    (ch-msg lk-0 (cat pt-4 k1 "wrap")) (5 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k))))
  (label 1162)
  (parent 453)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pval) (k skey)
    (lk locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (3 2)) ((2 1) (3 3)) ((2 1) (5 3))
    ((3 4) (0 0)) ((4 1) (2 0)) ((4 2) (3 0)) ((4 2) (5 0))
    ((5 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule invShearsRule)
  (operation channel-test (displaced 6 2 set-wrap 2)
    (ch-msg lk-0 (cat pt-4 k1 "wrap")) (5 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k))))
  (label 1163)
  (parent 453)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 cur mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (defstrand set-wrap 2 (cur cur) (k k1) (lk lk-0))
  (precedes ((1 1) (2 0)) ((1 1) (3 2)) ((2 1) (3 3)) ((3 4) (0 0))
    ((4 1) (6 0)) ((4 2) (3 0)) ((4 2) (5 0)) ((5 4) (0 0))
    ((6 1) (5 3)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 cur) (cat k1 "wrap"))
  (facts (neq cur "decrypt") (neq cur "wrap") (neq "init" "decrypt")
    (neq "init" "wrap"))
  (rule invShearsRule fact-set-decrypt-neq0 trRl_set-wrap-at-1
    gen-st-set-wrap-0 fact-set-wrap-neq0 trRl_make-at-0)
  (operation channel-test (added-strand set-wrap 2)
    (ch-msg lk-0 (cat pt-4 k1 "wrap")) (5 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-5 k1 cur)) (stor lk-0 (cat pt-4 k1 "wrap"))))
  (label 1164)
  (parent 453)
  (unrealized (6 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pval) (k skey)
    (lk locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (precedes ((1 1) (2 3)) ((1 1) (4 3)) ((2 4) (0 0)) ((3 1) (1 0))
    ((3 1) (4 2)) ((3 2) (2 0)) ((4 4) (0 0)) ((5 1) (1 0))
    ((5 1) (2 2)) ((5 2) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule invShearsRule)
  (operation channel-test (displaced 6 1 set-wrap 2)
    (ch-msg lk-0 (cat pt-4 k1 "wrap")) (4 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt-3 k "init")) (stor lk (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init"))
      (send (hash k))))
  (label 1165)
  (parent 454)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 cur mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur cur) (k k1) (lk lk))
  (precedes ((1 1) (2 3)) ((2 4) (0 0)) ((3 1) (4 2)) ((3 1) (6 0))
    ((3 2) (2 0)) ((4 4) (0 0)) ((5 1) (1 0)) ((5 1) (2 2))
    ((5 2) (4 0)) ((6 1) (4 3)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 cur) (cat k1 "wrap"))
  (facts (neq cur "decrypt") (neq cur "wrap") (neq "init" "decrypt")
    (neq "init" "wrap"))
  (rule invShearsRule fact-set-decrypt-neq0 trRl_set-wrap-at-1
    gen-st-set-wrap-0 fact-set-wrap-neq0 trRl_make-at-0)
  (operation channel-test (added-strand set-wrap 2)
    (ch-msg lk (cat pt-2 k1 "wrap")) (4 3))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt-4 k "init"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init"))
      (send (hash k)))
    ((load lk (cat pt-5 k1 cur)) (stor lk (cat pt-2 k1 "wrap"))))
  (label 1166)
  (parent 454)
  (unrealized (6 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((1 1) (3 2)) ((2 1) (3 3)) ((2 1) (5 3))
    ((3 4) (0 0)) ((4 1) (1 0)) ((4 1) (5 2)) ((4 2) (3 0))
    ((5 4) (0 0)) ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule invShearsRule)
  (operation channel-test (displaced 7 2 set-wrap 2)
    (ch-msg lk-1 (cat pt-6 k1 "wrap")) (5 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-4 old-1)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k))))
  (label 1167)
  (parent 455)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((1 1) (3 2)) ((2 1) (3 3)) ((2 1) (5 3))
    ((3 4) (0 0)) ((4 1) (2 0)) ((4 1) (5 2)) ((4 2) (3 0))
    ((5 4) (0 0)) ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule invShearsRule)
  (operation channel-test (displaced 7 2 set-wrap 2)
    (ch-msg lk-1 (cat pt-6 k1 "wrap")) (5 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-4 old-1)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k))))
  (label 1168)
  (parent 455)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 cur mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand set-wrap 2 (cur cur) (k k1) (lk lk-0))
  (precedes ((1 1) (2 0)) ((1 1) (3 2)) ((2 1) (3 3)) ((3 4) (0 0))
    ((4 1) (5 2)) ((4 1) (7 0)) ((4 2) (3 0)) ((5 4) (0 0))
    ((6 2) (5 0)) ((7 1) (5 3)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 cur) (cat k1 "wrap"))
  (facts (neq cur "decrypt") (neq cur "wrap") (neq "init" "decrypt")
    (neq "init" "wrap"))
  (rule invShearsRule fact-set-decrypt-neq0 trRl_set-wrap-at-1
    gen-st-set-wrap-0 fact-set-wrap-neq0 trRl_make-at-0)
  (operation channel-test (added-strand set-wrap 2)
    (ch-msg lk-0 (cat pt-4 k1 "wrap")) (5 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-7 k1 cur)) (stor lk-0 (cat pt-4 k1 "wrap"))))
  (label 1169)
  (parent 455)
  (unrealized (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (deflistener k)
  (precedes ((1 1) (2 0)) ((1 1) (3 2)) ((1 1) (6 0)) ((2 1) (3 3))
    ((3 4) (0 0)) ((4 1) (5 2)) ((4 2) (3 0)) ((5 4) (0 0))
    ((6 1) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation nonce-test (displaced 7 1 make 2) k (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k)))
  (label 1170)
  (parent 456)
  (unrealized (5 3) (6 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (deflistener k)
  (precedes ((1 1) (2 0)) ((1 1) (3 2)) ((2 1) (3 3)) ((3 4) (0 0))
    ((4 1) (6 0)) ((4 2) (3 0)) ((5 4) (0 0)) ((6 1) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation nonce-test (displaced 7 4 make 2) k (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k)))
  (label 1171)
  (parent 456)
  (unrealized (5 3) (6 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (deflistener k)
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((1 1) (3 2)) ((2 1) (3 3)) ((3 4) (0 0))
    ((4 1) (5 2)) ((4 2) (3 0)) ((5 4) (0 0)) ((6 1) (5 0))
    ((7 1) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation nonce-test (added-strand make 2) k (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init"))))
  (label 1172)
  (parent 456)
  (unrealized (5 3) (6 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 cur mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (defstrand set-wrap 2 (cur cur) (k k1) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (3 2)) ((2 1) (3 3)) ((2 1) (5 2))
    ((2 1) (6 0)) ((3 4) (0 0)) ((4 2) (3 0)) ((4 2) (5 0))
    ((5 4) (0 0)) ((6 1) (5 3)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 cur) (cat k1 "wrap"))
  (facts (neq cur "decrypt") (neq cur "wrap") (neq "init" "decrypt")
    (neq "init" "wrap"))
  (rule invShearsRule fact-set-decrypt-neq0 trRl_set-wrap-at-1
    gen-st-set-wrap-0 fact-set-wrap-neq0 trRl_make-at-0)
  (operation channel-test (added-strand set-wrap 2)
    (ch-msg lk (cat pt-4 k1 "wrap")) (5 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((load lk (cat pt-5 k1 cur)) (stor lk (cat pt-4 k1 "wrap"))))
  (label 1173)
  (parent 457)
  (unrealized (6 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 cur mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur cur) (k k1) (lk lk-0))
  (precedes ((1 1) (2 3)) ((1 1) (4 2)) ((1 1) (6 0)) ((2 4) (0 0))
    ((3 2) (2 0)) ((4 4) (0 0)) ((5 1) (1 0)) ((5 1) (2 2))
    ((5 2) (4 0)) ((6 1) (4 3)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 cur) (cat k1 "wrap"))
  (facts (neq cur "decrypt") (neq cur "wrap") (neq "init" "decrypt")
    (neq "init" "wrap"))
  (rule invShearsRule fact-set-decrypt-neq0 trRl_set-wrap-at-1
    gen-st-set-wrap-0 fact-set-wrap-neq0 trRl_make-at-0)
  (operation channel-test (added-strand set-wrap 2)
    (ch-msg lk-0 (cat pt-2 k1 "wrap")) (4 3))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt-4 k "init"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt k "wrap"))
      (load lk-0 (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-5 k1 cur)) (stor lk-0 (cat pt-2 k1 "wrap"))))
  (label 1174)
  (parent 458)
  (unrealized (6 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 cur mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand set-wrap 2 (cur cur) (k k1) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (3 2)) ((2 1) (3 3)) ((2 1) (5 2))
    ((2 1) (7 0)) ((3 4) (0 0)) ((4 2) (3 0)) ((5 4) (0 0))
    ((6 2) (5 0)) ((7 1) (5 3)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 cur) (cat k1 "wrap"))
  (facts (neq cur "decrypt") (neq cur "wrap") (neq "init" "decrypt")
    (neq "init" "wrap"))
  (rule invShearsRule fact-set-decrypt-neq0 trRl_set-wrap-at-1
    gen-st-set-wrap-0 fact-set-wrap-neq0 trRl_make-at-0)
  (operation channel-test (added-strand set-wrap 2)
    (ch-msg lk (cat pt-4 k1 "wrap")) (5 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init"))
      (send (hash k)))
    ((load lk (cat pt-7 k1 cur)) (stor lk (cat pt-4 k1 "wrap"))))
  (label 1175)
  (parent 459)
  (unrealized (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (precedes ((1 1) (2 0)) ((1 1) (3 2)) ((1 1) (6 0)) ((2 1) (3 3))
    ((2 1) (5 2)) ((3 4) (0 0)) ((4 2) (3 0)) ((5 4) (0 0))
    ((6 1) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation nonce-test (displaced 7 1 make 2) k (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k)))
  (label 1176)
  (parent 460)
  (unrealized (5 3) (6 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (precedes ((1 1) (2 0)) ((1 1) (3 2)) ((2 1) (3 3)) ((2 1) (5 2))
    ((3 4) (0 0)) ((4 1) (6 0)) ((4 2) (3 0)) ((5 4) (0 0))
    ((6 1) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation nonce-test (displaced 7 4 make 2) k (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k)))
  (label 1177)
  (parent 460)
  (unrealized (5 3) (6 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((1 1) (3 2)) ((2 1) (3 3)) ((2 1) (5 2))
    ((3 4) (0 0)) ((4 2) (3 0)) ((5 4) (0 0)) ((6 1) (5 0))
    ((7 1) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation nonce-test (added-strand make 2) k (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init"))))
  (label 1178)
  (parent 460)
  (unrealized (5 3) (6 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 cur mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur cur) (k k1) (lk lk-0))
  (precedes ((1 1) (2 3)) ((1 1) (6 0)) ((2 4) (0 0)) ((3 2) (2 0))
    ((4 4) (0 0)) ((5 1) (1 0)) ((5 1) (2 2)) ((5 2) (4 0))
    ((6 1) (4 3)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 cur) (cat k1 "wrap"))
  (facts (neq cur "decrypt") (neq cur "wrap") (neq "init" "decrypt")
    (neq "init" "wrap"))
  (rule shearsRule invShearsRule fact-set-decrypt-neq0
    trRl_set-wrap-at-1 gen-st-set-wrap-0 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation channel-test (added-strand set-wrap 2)
    (ch-msg lk-0 (cat pt-2 k1 "wrap")) (4 3))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt-4 k "init"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-4 k "init"))
      (load lk-0 (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-5 k1 cur)) (stor lk-0 (cat pt-2 k1 "wrap"))))
  (label 1179)
  (parent 462)
  (unrealized (6 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 cur mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand set-wrap 2 (cur cur) (k k1) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (3 2)) ((1 1) (5 2)) ((2 1) (3 3))
    ((2 1) (7 0)) ((3 4) (0 0)) ((4 2) (3 0)) ((5 4) (0 0))
    ((6 2) (5 0)) ((7 1) (5 3)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 cur) (cat k1 "wrap"))
  (facts (neq cur "decrypt") (neq cur "wrap") (neq "init" "decrypt")
    (neq "init" "wrap"))
  (rule shearsRule invShearsRule fact-set-decrypt-neq0
    trRl_set-wrap-at-1 gen-st-set-wrap-0 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation channel-test (added-strand set-wrap 2)
    (ch-msg lk (cat pt-4 k1 "wrap")) (5 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init"))
      (send (hash k)))
    ((load lk (cat pt-7 k1 cur)) (stor lk (cat pt-4 k1 "wrap"))))
  (label 1180)
  (parent 463)
  (unrealized (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (precedes ((1 1) (2 0)) ((1 1) (3 2)) ((1 1) (6 0)) ((2 1) (3 3))
    ((3 4) (0 0)) ((4 2) (3 0)) ((5 4) (0 0)) ((6 1) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation nonce-test (displaced 7 1 make 2) k (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k)))
  (label 1181)
  (parent 464)
  (unrealized (5 3) (6 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (precedes ((1 1) (2 0)) ((1 1) (3 2)) ((1 1) (5 2)) ((2 1) (3 3))
    ((3 4) (0 0)) ((4 1) (6 0)) ((4 2) (3 0)) ((5 4) (0 0))
    ((6 1) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation nonce-test (displaced 7 4 make 2) k (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k)))
  (label 1182)
  (parent 464)
  (unrealized (5 3) (6 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((1 1) (3 2)) ((1 1) (5 2)) ((2 1) (3 3))
    ((3 4) (0 0)) ((4 2) (3 0)) ((5 4) (0 0)) ((6 1) (5 0))
    ((7 1) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation nonce-test (added-strand make 2) k (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init"))))
  (label 1183)
  (parent 464)
  (unrealized (5 3) (6 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "wrap") (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (0 0)) ((3 4) (0 0))
    ((4 1) (3 0)) ((5 1) (6 0)) ((6 1) (7 0)) ((7 1) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "wrap" "decrypt") (neq "wrap" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-decrypt-at-1 fact-set-decrypt-neq0 gen-st-set-wrap-0
    fact-set-wrap-neq0 trRl_make-at-0)
  (operation nonce-test (added-strand set-decrypt 2) k (4 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk-0 (cat pt-4 k "init"))
    (ch-msg lk-0 (cat pt-5 k "wrap")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt-5 k "wrap")))
    ((load lk-0 (cat pt-5 k "wrap"))
      (stor lk-0 (cat pt-6 k "decrypt"))))
  (label 1184)
  (parent 465)
  (unrealized (3 3) (4 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (0 0)) ((3 4) (0 0))
    ((4 1) (3 0)) ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation nonce-test (added-strand make 2) k (4 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk-0 (cat pt-4 k "init"))
    (ch-msg lk-0 (cat pt-5 k "wrap")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt-5 k "wrap")))
    ((load lk-1 (cat pt-6 old-1)) (stor lk-1 (cat pt-7 k "init"))))
  (label 1185)
  (parent 465)
  (unrealized (3 3) (4 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "wrap") (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (0 0)) ((3 4) (0 0))
    ((4 1) (3 0)) ((5 1) (6 0)) ((6 1) (7 0)) ((7 1) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "wrap" "decrypt") (neq "wrap" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule fact-set-decrypt-neq0 trRl_set-wrap-at-1 gen-st-set-wrap-0
    fact-set-wrap-neq0 trRl_make-at-0)
  (operation nonce-test (added-strand set-wrap 2) k (4 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk-0 (cat pt-4 k "init"))
    (ch-msg lk-0 (cat pt-5 k "wrap")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt-5 k "wrap")))
    ((load lk-0 (cat pt-5 k "wrap")) (stor lk-0 (cat pt-6 k "wrap"))))
  (label 1186)
  (parent 465)
  (unrealized (3 3) (4 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1-0) (lk lk-0))
  (precedes ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (0 0)) ((3 4) (0 0))
    ((4 1) (3 0)) ((5 1) (6 0)) ((6 1) (7 2)) ((7 4) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap")
    (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0 gen-st-wrap-1)
  (operation nonce-test (added-strand wrap 5) k (4 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk-0 (cat pt-4 k "init"))
    (ch-msg lk-0 (cat pt-5 k "wrap")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt-5 k "wrap")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-0 (cat pt-5 k "wrap"))
      (load lk-0 (cat pt-6 k1-0 "wrap")) (send (enc k k1-0))))
  (label 1187)
  (parent 465)
  (unrealized (3 3) (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk-0))
  (precedes ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (0 0)) ((3 4) (0 0))
    ((4 1) (3 0)) ((5 1) (6 0)) ((5 1) (7 2)) ((6 1) (4 0))
    ((7 4) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (4 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk-0 (cat pt-4 k "init"))
    (ch-msg lk-0 (cat pt-5 k "wrap")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt-5 k "wrap")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-0 (cat pt-4 k "init"))
      (load lk-0 (cat pt-6 k1-0 "wrap")) (send (enc k k1-0))))
  (label 1188)
  (parent 465)
  (unrealized (3 3) (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (7 2)) ((2 1) (0 0)) ((3 4) (0 0))
    ((4 1) (3 0)) ((5 1) (6 0)) ((6 1) (4 0)) ((7 4) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (4 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk-0 (cat pt-4 k "init"))
    (ch-msg lk-0 (cat pt-5 k "wrap")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt-5 k "wrap")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-6 k1-0 "wrap")) (send (enc k k1-0))))
  (label 1189)
  (parent 465)
  (unrealized (3 3) (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (0 0)) ((3 4) (0 0))
    ((4 1) (3 0)) ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation nonce-test (added-strand make 2) k (4 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk-0 (cat pt-4 k "init"))
    (ch-msg lk-0 (cat pt-5 k "decrypt")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt-5 k "decrypt")))
    ((load lk-1 (cat pt-6 old-1)) (stor lk-1 (cat pt-7 k "init"))))
  (label 1190)
  (parent 466)
  (unrealized (3 3) (4 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "decrypt") (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (0 0)) ((3 4) (0 0))
    ((4 1) (3 0)) ((5 1) (6 0)) ((6 1) (7 0)) ((7 1) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k1 "wrap"))
  (facts (neq "decrypt" "decrypt") (neq "decrypt" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule fact-set-decrypt-neq0 trRl_set-wrap-at-1 gen-st-set-wrap-0
    fact-set-wrap-neq0 trRl_make-at-0)
  (operation nonce-test (added-strand set-wrap 2) k (4 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk-0 (cat pt-4 k "init"))
    (ch-msg lk-0 (cat pt-5 k "decrypt")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt-5 k "decrypt")))
    ((load lk-0 (cat pt-5 k "decrypt"))
      (stor lk-0 (cat pt-6 k "wrap"))))
  (label 1191)
  (parent 466)
  (unrealized (3 3) (4 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "decrypt") (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (0 0)) ((3 4) (0 0))
    ((4 1) (3 0)) ((5 1) (6 0)) ((6 1) (7 0)) ((7 1) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k1 "wrap"))
  (facts (neq "decrypt" "decrypt") (neq "decrypt" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-decrypt-at-1 fact-set-decrypt-neq0 gen-st-set-wrap-0
    fact-set-wrap-neq0 trRl_make-at-0)
  (operation nonce-test (added-strand set-decrypt 2) k (4 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk-0 (cat pt-4 k "init"))
    (ch-msg lk-0 (cat pt-5 k "decrypt")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt-5 k "decrypt")))
    ((load lk-0 (cat pt-5 k "decrypt"))
      (stor lk-0 (cat pt-6 k "decrypt"))))
  (label 1192)
  (parent 466)
  (unrealized (3 3) (4 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1-0) (lk lk-0))
  (precedes ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (0 0)) ((3 4) (0 0))
    ((4 1) (3 0)) ((5 1) (6 0)) ((6 1) (7 2)) ((7 4) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k1 "wrap")
    (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0 gen-st-wrap-1)
  (operation nonce-test (added-strand wrap 5) k (4 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk-0 (cat pt-4 k "init"))
    (ch-msg lk-0 (cat pt-5 k "decrypt")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt-5 k "decrypt")))
    ((recv (hash k)) (recv (hash k1-0))
      (load lk-0 (cat pt-5 k "decrypt"))
      (load lk-0 (cat pt-6 k1-0 "wrap")) (send (enc k k1-0))))
  (label 1193)
  (parent 466)
  (unrealized (3 3) (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk-0))
  (precedes ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (0 0)) ((3 4) (0 0))
    ((4 1) (3 0)) ((5 1) (6 0)) ((5 1) (7 2)) ((6 1) (4 0))
    ((7 4) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (4 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk-0 (cat pt-4 k "init"))
    (ch-msg lk-0 (cat pt-5 k "decrypt")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt-5 k "decrypt")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-0 (cat pt-4 k "init"))
      (load lk-0 (cat pt-6 k1-0 "wrap")) (send (enc k k1-0))))
  (label 1194)
  (parent 466)
  (unrealized (3 3) (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (7 2)) ((2 1) (0 0)) ((3 4) (0 0))
    ((4 1) (3 0)) ((5 1) (6 0)) ((6 1) (4 0)) ((7 4) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (4 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk-0 (cat pt-4 k "init"))
    (ch-msg lk-0 (cat pt-5 k "decrypt")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt-5 k "decrypt")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-6 k1-0 "wrap")) (send (enc k k1-0))))
  (label 1195)
  (parent 466)
  (unrealized (3 3) (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (0 0)) ((3 4) (0 0))
    ((4 1) (3 0)) ((5 4) (4 0)) ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 5 7 make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-0 (cat pt-5 k "init"))
      (load lk-0 (cat pt-3 k1-0 "wrap")) (send (enc k k1-0)))
    ((load lk-0 (cat pt-4 old-0)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k))))
  (label 1196)
  (parent 467)
  (unrealized (3 3) (5 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 k1-0 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk-0))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (0 0)) ((3 4) (0 0))
    ((4 1) (3 0)) ((5 1) (6 2)) ((6 4) (4 0)) ((7 2) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation encryption-test (added-strand make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-0 (cat pt-4 k "init"))
      (load lk-0 (cat pt-5 k1-0 "wrap")) (send (enc k k1-0)))
    ((load lk-1 (cat pt-6 old-1)) (stor lk-1 (cat pt-7 k "init"))
      (send (hash k))))
  (label 1197)
  (parent 467)
  (unrealized (3 3) (6 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk-0))
  (deflistener k)
  (precedes ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (0 0)) ((3 4) (0 0))
    ((4 1) (3 0)) ((5 1) (6 2)) ((6 4) (4 0)) ((7 1) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (added-listener k) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-0 (cat pt-4 k "init"))
      (load lk-0 (cat pt-5 k1-0 "wrap")) (send (enc k k1-0)))
    ((recv k) (send k)))
  (label 1198)
  (parent 467)
  (unrealized (3 3) (6 3) (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (precedes ((1 1) (0 0)) ((2 4) (0 0)) ((3 1) (2 0)) ((4 1) (3 0))
    ((5 4) (3 0)) ((6 1) (1 0)) ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 1 7 make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-5 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-5 k "init"))
      (load lk-0 (cat pt-0 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk (cat pt-1 old)) (stor lk (cat pt-2 k "init")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-0 (cat pt-5 k "init"))
      (load lk-0 (cat pt-3 k1-0 "wrap")) (send (enc k k1-0)))
    ((load lk-0 (cat pt-4 old-0)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k))))
  (label 1199)
  (parent 468)
  (unrealized (2 3) (5 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 k1-0 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((1 1) (6 2)) ((2 1) (0 0)) ((3 4) (0 0))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 4) (4 0)) ((7 2) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation encryption-test (added-strand make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-5 k1-0 "wrap")) (send (enc k k1-0)))
    ((load lk-1 (cat pt-6 old-1)) (stor lk-1 (cat pt-7 k "init"))
      (send (hash k))))
  (label 1200)
  (parent 468)
  (unrealized (3 3) (6 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk))
  (deflistener k)
  (precedes ((1 1) (2 0)) ((1 1) (6 2)) ((2 1) (0 0)) ((3 4) (0 0))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 4) (4 0)) ((7 1) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (added-listener k) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-5 k1-0 "wrap")) (send (enc k k1-0)))
    ((recv k) (send k)))
  (label 1201)
  (parent 468)
  (unrealized (3 3) (6 3) (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old mesg) (pt pt-0 pt-1 pt-2 pt-3 pval) (k k1 k1-0 skey)
    (lk locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk))
  (deflistener k)
  (precedes ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (0 0)) ((3 4) (0 0))
    ((4 1) (3 0)) ((5 4) (4 0)) ((6 1) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation nonce-test (displaced 7 1 make 2) k (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((recv (hash k)) (recv (hash k1-0)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-3 k1-0 "wrap")) (send (enc k k1-0)))
    ((recv k) (send k)))
  (label 1202)
  (parent 471)
  (unrealized (3 3) (5 3) (6 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((1 1) (5 2)) ((2 1) (0 0)) ((3 4) (0 0))
    ((4 1) (3 0)) ((5 4) (4 0)) ((6 1) (5 0)) ((7 1) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation nonce-test (added-strand make 2) k (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((recv (hash k)) (recv (hash k1-0)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-3 k1-0 "wrap")) (send (enc k k1-0)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-4 old-0)) (stor lk-0 (cat pt-5 k "init"))))
  (label 1203)
  (parent 471)
  (unrealized (3 3) (5 3) (6 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 cur mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 k1-0 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur cur) (k k1-0) (lk lk-0))
  (precedes ((1 1) (2 0)) ((1 1) (3 2)) ((2 1) (0 0)) ((3 4) (0 0))
    ((4 1) (3 0)) ((5 4) (4 0)) ((6 1) (7 0)) ((6 2) (5 0))
    ((7 1) (5 3)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 cur)
    (cat k1-0 "wrap"))
  (facts (neq cur "decrypt") (neq cur "wrap") (neq "init" "decrypt")
    (neq "init" "wrap"))
  (rule invShearsRule fact-set-decrypt-neq0 trRl_set-wrap-at-1
    gen-st-set-wrap-0 fact-set-wrap-neq0 trRl_make-at-0)
  (operation channel-test (added-strand set-wrap 2)
    (ch-msg lk-0 (cat pt-3 k1-0 "wrap")) (5 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-0 (cat pt-5 k "init"))
      (load lk-0 (cat pt-3 k1-0 "wrap")) (send (enc k k1-0)))
    ((load lk-0 (cat pt-4 old-0)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-6 k1-0 cur))
      (stor lk-0 (cat pt-3 k1-0 "wrap"))))
  (label 1204)
  (parent 482)
  (unrealized (3 3) (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 cur mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 k1-0 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur cur) (k k1-0) (lk lk))
  (precedes ((1 1) (0 0)) ((2 4) (0 0)) ((3 1) (2 0)) ((4 1) (5 2))
    ((4 1) (7 0)) ((5 4) (3 0)) ((6 1) (1 0)) ((6 2) (5 0))
    ((7 1) (5 3)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 cur)
    (cat k1-0 "wrap"))
  (facts (neq cur "decrypt") (neq cur "wrap") (neq "init" "decrypt")
    (neq "init" "wrap"))
  (rule invShearsRule fact-set-decrypt-neq0 trRl_set-wrap-at-1
    gen-st-set-wrap-0 fact-set-wrap-neq0 trRl_make-at-0)
  (operation channel-test (added-strand set-wrap 2)
    (ch-msg lk (cat pt-3 k1-0 "wrap")) (5 3))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-5 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-5 k "init"))
      (load lk-0 (cat pt-0 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk (cat pt-1 old)) (stor lk (cat pt-2 k "init")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk (cat pt-2 k "init"))
      (load lk (cat pt-3 k1-0 "wrap")) (send (enc k k1-0)))
    ((load lk-0 (cat pt-4 old-0)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k)))
    ((load lk (cat pt-6 k1-0 cur)) (stor lk (cat pt-3 k1-0 "wrap"))))
  (label 1205)
  (parent 483)
  (unrealized (2 3) (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (deflistener k)
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (0 0)) ((3 1) (4 2))
    ((4 4) (0 0)) ((5 1) (4 0)) ((6 1) (7 0)) ((7 1) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-decrypt-at-1 trRl_make-at-0)
  (operation nonce-test (added-strand set-decrypt 2) k (5 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk-1 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init")))
    ((load lk-1 (cat pt-6 k "init"))
      (stor lk-1 (cat pt-7 k "decrypt"))))
  (label 1206)
  (parent 489)
  (unrealized (4 3) (5 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (deflistener k)
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (0 0)) ((3 1) (5 0))
    ((4 4) (0 0)) ((5 1) (4 0)) ((6 1) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation nonce-test (displaced 7 3 make 2) k (5 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk-1 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init"))))
  (label 1207)
  (parent 489)
  (unrealized (4 3) (5 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (deflistener k)
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (0 0)) ((3 1) (4 2))
    ((4 4) (0 0)) ((5 1) (4 0)) ((6 1) (7 0)) ((7 1) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-wrap-at-1 trRl_make-at-0)
  (operation nonce-test (added-strand set-wrap 2) k (5 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk-1 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init")))
    ((load lk-1 (cat pt-6 k "init")) (stor lk-1 (cat pt-7 k "wrap"))))
  (label 1208)
  (parent 489)
  (unrealized (4 3) (5 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 k1-0 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (deflistener k)
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk-1))
  (precedes ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (0 0)) ((3 1) (4 2))
    ((4 4) (0 0)) ((5 1) (4 0)) ((6 1) (7 2)) ((7 4) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (5 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk-1 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-1 (cat pt-6 k "init"))
      (load lk-1 (cat pt-7 k1-0 "wrap")) (send (enc k k1-0))))
  (label 1209)
  (parent 489)
  (unrealized (4 3) (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 k1-0 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (deflistener k)
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (7 2)) ((2 1) (0 0)) ((3 1) (4 2))
    ((4 4) (0 0)) ((5 1) (4 0)) ((6 1) (5 0)) ((7 4) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (5 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk-1 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-7 k1-0 "wrap")) (send (enc k k1-0))))
  (label 1210)
  (parent 489)
  (unrealized (4 3) (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (deflistener k)
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk-0))
  (precedes ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (0 0)) ((3 1) (6 2))
    ((4 4) (0 0)) ((5 1) (4 0)) ((6 4) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (5 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk-0 (cat pt-3 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-5 k1-0 "wrap")) (send (enc k k1-0))))
  (label 1211)
  (parent 490)
  (unrealized (4 3) (6 0) (6 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (deflistener k)
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (6 2)) ((2 1) (0 0)) ((3 1) (5 0))
    ((4 4) (0 0)) ((5 1) (4 0)) ((6 4) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (5 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk-0 (cat pt-3 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((recv (hash k)) (recv (hash k1-0)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-5 k1-0 "wrap")) (send (enc k k1-0))))
  (label 1212)
  (parent 490)
  (unrealized (4 3) (6 0) (6 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (deflistener k)
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((1 1) (5 2)) ((2 1) (0 0)) ((3 4) (0 0))
    ((4 1) (3 0)) ((5 4) (4 0)) ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 3 7 make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-5 k "init"))
      (load lk-0 (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((recv (hash k)) (recv (hash k1-0)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-3 k1-0 "wrap")) (send (enc k k1-0)))
    ((load lk-0 (cat pt-4 old-0)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k))))
  (label 1213)
  (parent 491)
  (unrealized (3 3) (5 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (precedes ((1 1) (0 0)) ((2 1) (3 2)) ((3 4) (0 0)) ((4 1) (3 0))
    ((5 4) (4 0)) ((6 1) (1 0)) ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 1 7 make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-5 k "init")) (stor lk-0 (cat pt k "decrypt")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-0 (cat pt-5 k "init"))
      (load lk-0 (cat pt-3 k1-0 "wrap")) (send (enc k k1-0)))
    ((load lk-0 (cat pt-4 old-0)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k))))
  (label 1214)
  (parent 491)
  (unrealized (3 3) (5 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 k1-0 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (deflistener k)
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((1 1) (6 2)) ((2 1) (0 0)) ((3 1) (4 2))
    ((4 4) (0 0)) ((5 1) (4 0)) ((6 4) (5 0)) ((7 2) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation encryption-test (added-strand make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((recv (hash k)) (recv (hash k1-0)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-5 k1-0 "wrap")) (send (enc k k1-0)))
    ((load lk-1 (cat pt-6 old-1)) (stor lk-1 (cat pt-7 k "init"))
      (send (hash k))))
  (label 1215)
  (parent 491)
  (unrealized (4 3) (6 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (deflistener k)
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk))
  (deflistener k)
  (precedes ((1 1) (2 0)) ((1 1) (6 2)) ((2 1) (0 0)) ((3 1) (4 2))
    ((4 4) (0 0)) ((5 1) (4 0)) ((6 4) (5 0)) ((7 1) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (added-listener k) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((recv (hash k)) (recv (hash k1-0)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-5 k1-0 "wrap")) (send (enc k k1-0)))
    ((recv k) (send k)))
  (label 1216)
  (parent 491)
  (unrealized (4 3) (6 3) (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (deflistener k)
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((2 1) (0 0)) ((3 1) (5 0)) ((4 4) (0 0))
    ((5 1) (4 0)) ((6 1) (7 0)) ((7 1) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-wrap-at-1 trRl_make-at-0)
  (operation nonce-test (added-strand set-wrap 2) k (5 0)
    (ch-msg lk-0 (cat pt-3 k "init")) (ch-msg lk-1 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init")))
    ((load lk-1 (cat pt-6 k "init")) (stor lk-1 (cat pt-7 k "wrap"))))
  (label 1217)
  (parent 492)
  (unrealized (4 3) (5 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (deflistener k)
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((2 1) (0 0)) ((3 1) (5 0)) ((4 4) (0 0))
    ((5 1) (4 0)) ((6 1) (7 0)) ((7 1) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-decrypt-at-1 trRl_make-at-0)
  (operation nonce-test (added-strand set-decrypt 2) k (5 0)
    (ch-msg lk-0 (cat pt-3 k "init")) (ch-msg lk-1 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init")))
    ((load lk-1 (cat pt-6 k "init"))
      (stor lk-1 (cat pt-7 k "decrypt"))))
  (label 1218)
  (parent 492)
  (unrealized (4 3) (5 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 k1-0 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (deflistener k)
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk-1))
  (precedes ((1 1) (2 0)) ((2 1) (0 0)) ((3 1) (5 0)) ((4 4) (0 0))
    ((5 1) (4 0)) ((6 1) (7 2)) ((7 4) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (5 0)
    (ch-msg lk-0 (cat pt-3 k "init")) (ch-msg lk-1 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-1 (cat pt-6 k "init"))
      (load lk-1 (cat pt-7 k1-0 "wrap")) (send (enc k k1-0))))
  (label 1219)
  (parent 492)
  (unrealized (4 3) (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 k1-0 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (deflistener k)
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (0 0)) ((3 1) (7 2)) ((4 4) (0 0))
    ((5 1) (4 0)) ((6 1) (5 0)) ((7 4) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (5 0)
    (ch-msg lk-0 (cat pt-3 k "init")) (ch-msg lk-1 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-7 k1-0 "wrap")) (send (enc k k1-0))))
  (label 1220)
  (parent 492)
  (unrealized (4 3) (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (deflistener k)
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (0 0)) ((3 4) (0 0)) ((4 1) (3 0))
    ((5 4) (4 0)) ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 3 7 make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-5 k "init"))
      (load lk-0 (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-0 (cat pt-5 k "init"))
      (load lk-0 (cat pt-3 k1-0 "wrap")) (send (enc k k1-0)))
    ((load lk-0 (cat pt-4 old-0)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k))))
  (label 1221)
  (parent 493)
  (unrealized (3 3) (5 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (precedes ((1 1) (0 0)) ((2 1) (5 2)) ((3 4) (0 0)) ((4 1) (3 0))
    ((5 4) (4 0)) ((6 1) (1 0)) ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 1 7 make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-5 k "init")) (stor lk-0 (cat pt k "decrypt")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((recv (hash k)) (recv (hash k1-0)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt-3 k1-0 "wrap")) (send (enc k k1-0)))
    ((load lk-0 (cat pt-4 old-0)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k))))
  (label 1222)
  (parent 493)
  (unrealized (3 3) (5 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 k1-0 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (deflistener k)
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk-0))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((2 1) (0 0)) ((3 1) (6 2)) ((4 4) (0 0))
    ((5 1) (4 0)) ((6 4) (5 0)) ((7 2) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation encryption-test (added-strand make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-5 k1-0 "wrap")) (send (enc k k1-0)))
    ((load lk-1 (cat pt-6 old-1)) (stor lk-1 (cat pt-7 k "init"))
      (send (hash k))))
  (label 1223)
  (parent 493)
  (unrealized (4 3) (6 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (deflistener k)
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk-0))
  (deflistener k)
  (precedes ((1 1) (2 0)) ((2 1) (0 0)) ((3 1) (6 2)) ((4 4) (0 0))
    ((5 1) (4 0)) ((6 4) (5 0)) ((7 1) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (added-listener k) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-5 k1-0 "wrap")) (send (enc k k1-0)))
    ((recv k) (send k)))
  (label 1224)
  (parent 493)
  (unrealized (4 3) (6 3) (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (4 2)) ((3 1) (0 0))
    ((4 4) (0 0)) ((5 1) (4 0)) ((6 1) (7 0)) ((7 1) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-decrypt-at-1 trRl_make-at-0)
  (operation nonce-test (added-strand set-decrypt 2) k (5 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk-1 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "decrypt"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init")))
    ((load lk-1 (cat pt-6 k "init"))
      (stor lk-1 (cat pt-7 k "decrypt"))))
  (label 1225)
  (parent 499)
  (unrealized (4 3) (5 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (4 2)) ((3 1) (0 0))
    ((4 4) (0 0)) ((5 1) (4 0)) ((6 1) (7 0)) ((7 1) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-wrap-at-1 trRl_make-at-0)
  (operation nonce-test (added-strand set-wrap 2) k (5 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk-1 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "decrypt"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init")))
    ((load lk-1 (cat pt-6 k "init")) (stor lk-1 (cat pt-7 k "wrap"))))
  (label 1226)
  (parent 499)
  (unrealized (4 3) (5 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((2 1) (5 0)) ((3 1) (0 0)) ((4 4) (0 0))
    ((5 1) (4 0)) ((6 1) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule scissorsRule trRl_set-decrypt-at-1 trRl_make-at-0)
  (operation nonce-test (added-strand set-decrypt 2) k (5 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk-1 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "decrypt"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init"))))
  (label 1227)
  (parent 499)
  (unrealized (4 3) (5 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 k1-0 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk-1))
  (precedes ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (4 2)) ((3 1) (0 0))
    ((4 4) (0 0)) ((5 1) (4 0)) ((6 1) (7 2)) ((7 4) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k1 "wrap")
    (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (5 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk-1 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "decrypt"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-1 (cat pt-6 k "init"))
      (load lk-1 (cat pt-7 k1-0 "wrap")) (send (enc k k1-0))))
  (label 1228)
  (parent 499)
  (unrealized (4 3) (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 k1-0 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (7 2)) ((2 1) (4 2)) ((3 1) (0 0))
    ((4 4) (0 0)) ((5 1) (4 0)) ((6 1) (5 0)) ((7 4) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k1 "wrap")
    (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (5 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk-1 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "decrypt"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-7 k1-0 "wrap")) (send (enc k k1-0))))
  (label 1229)
  (parent 499)
  (unrealized (4 3) (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1-0) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (6 2)) ((3 1) (0 0)) ((4 4) (0 0))
    ((5 1) (4 0)) ((6 4) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k1 "wrap")
    (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (5 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "decrypt")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "decrypt"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((recv (hash k)) (recv (hash k1-0)) (load lk (cat pt-1 k "decrypt"))
      (load lk (cat pt-5 k1-0 "wrap")) (send (enc k k1-0))))
  (label 1230)
  (parent 500)
  (unrealized (4 3) (6 0) (6 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (6 2)) ((2 1) (5 0)) ((3 1) (0 0))
    ((4 4) (0 0)) ((5 1) (4 0)) ((6 4) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k1 "wrap")
    (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (5 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "decrypt")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "decrypt"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((recv (hash k)) (recv (hash k1-0)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-5 k1-0 "wrap")) (send (enc k k1-0))))
  (label 1231)
  (parent 500)
  (unrealized (4 3) (6 0) (6 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1) (lk lk-0))
  (deflistener k)
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (precedes ((1 1) (3 2)) ((2 1) (0 0)) ((3 4) (0 0)) ((4 1) (3 0))
    ((5 4) (4 0)) ((6 1) (1 0)) ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k1 "wrap")
    (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 1 7 make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-5 k "init")) (stor lk-0 (cat pt k "decrypt")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt k "decrypt"))
      (load lk-0 (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-0 (cat pt-5 k "init"))
      (load lk-0 (cat pt-3 k1-0 "wrap")) (send (enc k k1-0)))
    ((load lk-0 (cat pt-4 old-0)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k))))
  (label 1232)
  (parent 501)
  (unrealized (3 3) (5 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 k1-0 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((1 1) (6 2)) ((2 1) (4 2)) ((3 1) (0 0))
    ((4 4) (0 0)) ((5 1) (4 0)) ((6 4) (5 0)) ((7 2) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k1 "wrap")
    (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation encryption-test (added-strand make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "decrypt"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((recv (hash k)) (recv (hash k1-0)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-5 k1-0 "wrap")) (send (enc k k1-0)))
    ((load lk-1 (cat pt-6 old-1)) (stor lk-1 (cat pt-7 k "init"))
      (send (hash k))))
  (label 1233)
  (parent 501)
  (unrealized (4 3) (6 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk))
  (deflistener k)
  (precedes ((1 1) (2 0)) ((1 1) (6 2)) ((2 1) (4 2)) ((3 1) (0 0))
    ((4 4) (0 0)) ((5 1) (4 0)) ((6 4) (5 0)) ((7 1) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k1 "wrap")
    (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (added-listener k) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "decrypt"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((recv (hash k)) (recv (hash k1-0)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-5 k1-0 "wrap")) (send (enc k k1-0)))
    ((recv k) (send k)))
  (label 1234)
  (parent 501)
  (unrealized (4 3) (6 3) (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (0 0)) ((3 1) (0 0))
    ((4 4) (0 0)) ((5 1) (4 0)) ((6 1) (7 0)) ((7 1) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-decrypt-at-1 trRl_make-at-0)
  (operation nonce-test (added-strand set-decrypt 2) k (5 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk-1 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init")))
    ((load lk-1 (cat pt-6 k "init"))
      (stor lk-1 (cat pt-7 k "decrypt"))))
  (label 1235)
  (parent 507)
  (unrealized (4 3) (5 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (0 0)) ((3 1) (0 0))
    ((4 4) (0 0)) ((5 1) (4 0)) ((6 1) (7 0)) ((7 1) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-wrap-at-1 trRl_make-at-0)
  (operation nonce-test (added-strand set-wrap 2) k (5 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk-1 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init")))
    ((load lk-1 (cat pt-6 k "init")) (stor lk-1 (cat pt-7 k "wrap"))))
  (label 1236)
  (parent 507)
  (unrealized (4 3) (5 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 k1-0 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk-1))
  (precedes ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (0 0)) ((3 1) (0 0))
    ((4 4) (0 0)) ((5 1) (4 0)) ((6 1) (7 2)) ((7 4) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (5 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk-1 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-1 (cat pt-6 k "init"))
      (load lk-1 (cat pt-7 k1-0 "wrap")) (send (enc k k1-0))))
  (label 1237)
  (parent 507)
  (unrealized (4 3) (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 k1-0 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (7 2)) ((2 1) (0 0)) ((3 1) (0 0))
    ((4 4) (0 0)) ((5 1) (4 0)) ((6 1) (5 0)) ((7 4) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (5 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk-1 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-7 k1-0 "wrap")) (send (enc k k1-0))))
  (label 1238)
  (parent 507)
  (unrealized (4 3) (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (deflistener k)
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (precedes ((1 1) (0 0)) ((2 1) (0 0)) ((3 4) (0 0)) ((4 1) (3 0))
    ((5 4) (4 0)) ((6 1) (1 0)) ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 1 7 make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-5 k "init")) (stor lk-0 (cat pt k "decrypt")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-5 k "init"))
      (load lk-0 (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-0 (cat pt-5 k "init"))
      (load lk-0 (cat pt-3 k1-0 "wrap")) (send (enc k k1-0)))
    ((load lk-0 (cat pt-4 old-0)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k))))
  (label 1239)
  (parent 508)
  (unrealized (3 3) (5 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 k1-0 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((1 1) (6 2)) ((2 1) (0 0)) ((3 1) (0 0))
    ((4 4) (0 0)) ((5 1) (4 0)) ((6 4) (5 0)) ((7 2) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation encryption-test (added-strand make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((recv (hash k)) (recv (hash k1-0)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-5 k1-0 "wrap")) (send (enc k k1-0)))
    ((load lk-1 (cat pt-6 old-1)) (stor lk-1 (cat pt-7 k "init"))
      (send (hash k))))
  (label 1240)
  (parent 508)
  (unrealized (4 3) (6 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk))
  (deflistener k)
  (precedes ((1 1) (2 0)) ((1 1) (6 2)) ((2 1) (0 0)) ((3 1) (0 0))
    ((4 4) (0 0)) ((5 1) (4 0)) ((6 4) (5 0)) ((7 1) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (added-listener k) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((recv (hash k)) (recv (hash k1-0)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-5 k1-0 "wrap")) (send (enc k k1-0)))
    ((recv k) (send k)))
  (label 1241)
  (parent 508)
  (unrealized (4 3) (6 3) (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "decrypt") (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (3 2)) ((3 4) (0 0))
    ((4 1) (3 0)) ((5 1) (6 0)) ((6 1) (7 0)) ((7 1) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k1 "wrap"))
  (facts (neq "decrypt" "decrypt") (neq "decrypt" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-decrypt-at-1 fact-set-decrypt-neq0 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation nonce-test (added-strand set-decrypt 2) k (4 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk-0 (cat pt-4 k "init"))
    (ch-msg lk-0 (cat pt-5 k "decrypt")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "decrypt"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt-5 k "decrypt")))
    ((load lk-0 (cat pt-5 k "decrypt"))
      (stor lk-0 (cat pt-6 k "decrypt"))))
  (label 1242)
  (parent 512)
  (unrealized (3 3) (4 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (3 2)) ((3 4) (0 0))
    ((4 1) (3 0)) ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation nonce-test (added-strand make 2) k (4 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk-0 (cat pt-4 k "init"))
    (ch-msg lk-0 (cat pt-5 k "decrypt")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "decrypt"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt-5 k "decrypt")))
    ((load lk-1 (cat pt-6 old-1)) (stor lk-1 (cat pt-7 k "init"))))
  (label 1243)
  (parent 512)
  (unrealized (3 3) (4 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "decrypt") (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (3 2)) ((3 4) (0 0))
    ((4 1) (3 0)) ((5 1) (6 0)) ((6 1) (7 0)) ((7 1) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k1 "wrap"))
  (facts (neq "decrypt" "decrypt") (neq "decrypt" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule fact-set-decrypt-neq0 trRl_set-wrap-at-1 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation nonce-test (added-strand set-wrap 2) k (4 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk-0 (cat pt-4 k "init"))
    (ch-msg lk-0 (cat pt-5 k "decrypt")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "decrypt"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt-5 k "decrypt")))
    ((load lk-0 (cat pt-5 k "decrypt"))
      (stor lk-0 (cat pt-6 k "wrap"))))
  (label 1244)
  (parent 512)
  (unrealized (3 3) (4 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (4 0)) ((3 4) (0 0)) ((4 1) (3 0))
    ((5 1) (6 0)) ((6 1) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule scissorsRule trRl_set-decrypt-at-1 trRl_make-at-0)
  (operation nonce-test (added-strand set-decrypt 2) k (4 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk-0 (cat pt-4 k "init"))
    (ch-msg lk-0 (cat pt-5 k "decrypt")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "decrypt"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((load lk-0 (cat pt-4 k "init"))
      (stor lk-0 (cat pt-5 k "decrypt"))))
  (label 1245)
  (parent 512)
  (unrealized (3 3) (4 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1-0) (lk lk-0))
  (precedes ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (3 2)) ((3 4) (0 0))
    ((4 1) (3 0)) ((5 1) (6 0)) ((6 1) (7 2)) ((7 4) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k1 "wrap")
    (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (4 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk-0 (cat pt-4 k "init"))
    (ch-msg lk-0 (cat pt-5 k "decrypt")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "decrypt"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt-5 k "decrypt")))
    ((recv (hash k)) (recv (hash k1-0))
      (load lk-0 (cat pt-5 k "decrypt"))
      (load lk-0 (cat pt-6 k1-0 "wrap")) (send (enc k k1-0))))
  (label 1246)
  (parent 512)
  (unrealized (3 3) (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk-0))
  (precedes ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (3 2)) ((3 4) (0 0))
    ((4 1) (3 0)) ((5 1) (6 0)) ((5 1) (7 2)) ((6 1) (4 0))
    ((7 4) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k1 "wrap")
    (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (4 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk-0 (cat pt-4 k "init"))
    (ch-msg lk-0 (cat pt-5 k "decrypt")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "decrypt"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt-5 k "decrypt")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-0 (cat pt-4 k "init"))
      (load lk-0 (cat pt-6 k1-0 "wrap")) (send (enc k k1-0))))
  (label 1247)
  (parent 512)
  (unrealized (3 3) (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (7 2)) ((2 1) (3 2)) ((3 4) (0 0))
    ((4 1) (3 0)) ((5 1) (6 0)) ((6 1) (4 0)) ((7 4) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k1 "wrap")
    (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (4 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk-0 (cat pt-4 k "init"))
    (ch-msg lk-0 (cat pt-5 k "decrypt")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "decrypt"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt-5 k "decrypt")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-6 k1-0 "wrap")) (send (enc k k1-0))))
  (label 1248)
  (parent 512)
  (unrealized (3 3) (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "wrap") (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (3 2)) ((3 4) (0 0))
    ((4 1) (3 0)) ((5 1) (6 0)) ((6 1) (7 0)) ((7 1) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k "wrap")
    (cat k1 "wrap"))
  (facts (neq "wrap" "decrypt") (neq "wrap" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-decrypt-at-1 fact-set-decrypt-neq0 gen-st-set-wrap-0
    fact-set-wrap-neq0 trRl_make-at-0)
  (operation nonce-test (added-strand set-decrypt 2) k (4 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk-0 (cat pt-4 k "init"))
    (ch-msg lk-0 (cat pt-5 k "wrap")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "decrypt"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt-5 k "wrap")))
    ((load lk-0 (cat pt-5 k "wrap"))
      (stor lk-0 (cat pt-6 k "decrypt"))))
  (label 1249)
  (parent 513)
  (unrealized (3 3) (4 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (3 2)) ((3 4) (0 0))
    ((4 1) (3 0)) ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation nonce-test (added-strand make 2) k (4 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk-0 (cat pt-4 k "init"))
    (ch-msg lk-0 (cat pt-5 k "wrap")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "decrypt"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt-5 k "wrap")))
    ((load lk-1 (cat pt-6 old-1)) (stor lk-1 (cat pt-7 k "init"))))
  (label 1250)
  (parent 513)
  (unrealized (3 3) (4 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "wrap") (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (3 2)) ((3 4) (0 0))
    ((4 1) (3 0)) ((5 1) (6 0)) ((6 1) (7 0)) ((7 1) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k "wrap")
    (cat k1 "wrap"))
  (facts (neq "wrap" "decrypt") (neq "wrap" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule fact-set-decrypt-neq0 trRl_set-wrap-at-1 gen-st-set-wrap-0
    fact-set-wrap-neq0 trRl_make-at-0)
  (operation nonce-test (added-strand set-wrap 2) k (4 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk-0 (cat pt-4 k "init"))
    (ch-msg lk-0 (cat pt-5 k "wrap")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "decrypt"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt-5 k "wrap")))
    ((load lk-0 (cat pt-5 k "wrap")) (stor lk-0 (cat pt-6 k "wrap"))))
  (label 1251)
  (parent 513)
  (unrealized (3 3) (4 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (4 0)) ((3 4) (0 0)) ((4 1) (3 0))
    ((5 1) (6 0)) ((6 1) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule scissorsRule trRl_set-decrypt-at-1 trRl_make-at-0)
  (operation nonce-test (added-strand set-decrypt 2) k (4 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk-0 (cat pt-4 k "init"))
    (ch-msg lk-0 (cat pt-5 k "wrap")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "decrypt"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt-5 k "wrap"))))
  (label 1252)
  (parent 513)
  (unrealized (3 3) (4 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1-0) (lk lk-0))
  (precedes ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (3 2)) ((3 4) (0 0))
    ((4 1) (3 0)) ((5 1) (6 0)) ((6 1) (7 2)) ((7 4) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k "wrap")
    (cat k1 "wrap") (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0 gen-st-wrap-1)
  (operation nonce-test (added-strand wrap 5) k (4 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk-0 (cat pt-4 k "init"))
    (ch-msg lk-0 (cat pt-5 k "wrap")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "decrypt"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt-5 k "wrap")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-0 (cat pt-5 k "wrap"))
      (load lk-0 (cat pt-6 k1-0 "wrap")) (send (enc k k1-0))))
  (label 1253)
  (parent 513)
  (unrealized (3 3) (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk-0))
  (precedes ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (3 2)) ((3 4) (0 0))
    ((4 1) (3 0)) ((5 1) (6 0)) ((5 1) (7 2)) ((6 1) (4 0))
    ((7 4) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k1 "wrap")
    (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (4 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk-0 (cat pt-4 k "init"))
    (ch-msg lk-0 (cat pt-5 k "wrap")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "decrypt"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt-5 k "wrap")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-0 (cat pt-4 k "init"))
      (load lk-0 (cat pt-6 k1-0 "wrap")) (send (enc k k1-0))))
  (label 1254)
  (parent 513)
  (unrealized (3 3) (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (7 2)) ((2 1) (3 2)) ((3 4) (0 0))
    ((4 1) (3 0)) ((5 1) (6 0)) ((6 1) (4 0)) ((7 4) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k1 "wrap")
    (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (4 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk-0 (cat pt-4 k "init"))
    (ch-msg lk-0 (cat pt-5 k "wrap")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "decrypt"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt-5 k "wrap")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-6 k1-0 "wrap")) (send (enc k k1-0))))
  (label 1255)
  (parent 513)
  (unrealized (3 3) (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (4 0)) ((3 4) (0 0)) ((4 1) (3 0))
    ((5 1) (6 2)) ((6 4) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k1 "wrap")
    (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (4 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "decrypt"))
    (ch-msg lk-0 (cat pt-4 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "decrypt"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-0 (cat pt-4 k "init"))
      (load lk-0 (cat pt-5 k1-0 "wrap")) (send (enc k k1-0))))
  (label 1256)
  (parent 514)
  (unrealized (3 3) (6 0) (6 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1-0) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (6 2)) ((3 4) (0 0)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 4) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k1 "wrap")
    (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (4 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "decrypt"))
    (ch-msg lk-0 (cat pt-4 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "decrypt"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk (cat pt-1 k "decrypt"))
      (load lk (cat pt-5 k1-0 "wrap")) (send (enc k k1-0))))
  (label 1257)
  (parent 514)
  (unrealized (3 3) (6 0) (6 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (6 2)) ((2 1) (4 0)) ((3 4) (0 0))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 4) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k1 "wrap")
    (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (4 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "decrypt"))
    (ch-msg lk-0 (cat pt-4 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "decrypt"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-5 k1-0 "wrap")) (send (enc k k1-0))))
  (label 1258)
  (parent 514)
  (unrealized (3 3) (6 0) (6 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (3 2)) ((3 4) (0 0))
    ((4 1) (3 0)) ((5 4) (4 0)) ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k1 "wrap")
    (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 5 7 make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "decrypt"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-0 (cat pt-5 k "init"))
      (load lk-0 (cat pt-3 k1-0 "wrap")) (send (enc k k1-0)))
    ((load lk-0 (cat pt-4 old-0)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k))))
  (label 1259)
  (parent 515)
  (unrealized (3 3) (5 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 k1-0 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk-0))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (3 2)) ((3 4) (0 0))
    ((4 1) (3 0)) ((5 1) (6 2)) ((6 4) (4 0)) ((7 2) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k1 "wrap")
    (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation encryption-test (added-strand make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "decrypt"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-0 (cat pt-4 k "init"))
      (load lk-0 (cat pt-5 k1-0 "wrap")) (send (enc k k1-0)))
    ((load lk-1 (cat pt-6 old-1)) (stor lk-1 (cat pt-7 k "init"))
      (send (hash k))))
  (label 1260)
  (parent 515)
  (unrealized (3 3) (6 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk-0))
  (deflistener k)
  (precedes ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (3 2)) ((3 4) (0 0))
    ((4 1) (3 0)) ((5 1) (6 2)) ((6 4) (4 0)) ((7 1) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k1 "wrap")
    (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (added-listener k) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "decrypt"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-0 (cat pt-4 k "init"))
      (load lk-0 (cat pt-5 k1-0 "wrap")) (send (enc k k1-0)))
    ((recv k) (send k)))
  (label 1261)
  (parent 515)
  (unrealized (3 3) (6 3) (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1) (lk lk-0))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (precedes ((1 1) (2 2)) ((2 4) (0 0)) ((3 1) (2 0)) ((4 1) (3 0))
    ((5 4) (3 0)) ((6 1) (1 0)) ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k1 "wrap")
    (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 1 7 make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-5 k "init")) (stor lk-0 (cat pt k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt k "decrypt"))
      (load lk-0 (cat pt-0 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk (cat pt-1 old)) (stor lk (cat pt-2 k "init")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-0 (cat pt-5 k "init"))
      (load lk-0 (cat pt-3 k1-0 "wrap")) (send (enc k k1-0)))
    ((load lk-0 (cat pt-4 old-0)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k))))
  (label 1262)
  (parent 516)
  (unrealized (2 3) (5 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 k1-0 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((1 1) (6 2)) ((2 1) (3 2)) ((3 4) (0 0))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 4) (4 0)) ((7 2) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k1 "wrap")
    (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation encryption-test (added-strand make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "decrypt"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-5 k1-0 "wrap")) (send (enc k k1-0)))
    ((load lk-1 (cat pt-6 old-1)) (stor lk-1 (cat pt-7 k "init"))
      (send (hash k))))
  (label 1263)
  (parent 516)
  (unrealized (3 3) (6 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk))
  (deflistener k)
  (precedes ((1 1) (2 0)) ((1 1) (6 2)) ((2 1) (3 2)) ((3 4) (0 0))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 4) (4 0)) ((7 1) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k1 "wrap")
    (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (added-listener k) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "decrypt"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-5 k1-0 "wrap")) (send (enc k k1-0)))
    ((recv k) (send k)))
  (label 1264)
  (parent 516)
  (unrealized (3 3) (6 3) (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old mesg) (pt pt-0 pt-1 pt-2 pt-3 pval) (k k1 k1-0 skey)
    (lk locn))
  (deflistener k)
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1-0) (lk lk))
  (defstrand make 3 (old old) (k k) (lk lk))
  (precedes ((1 1) (4 2)) ((2 4) (0 0)) ((3 1) (2 0)) ((4 4) (3 0))
    ((5 1) (1 0)) ((5 2) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k1 "wrap")
    (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 1 6 make 3) (hash k) (5 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt-3 k "init")) (stor lk (cat pt k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt k "decrypt"))
      (load lk (cat pt-0 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((recv (hash k)) (recv (hash k1-0)) (load lk (cat pt k "decrypt"))
      (load lk (cat pt-1 k1-0 "wrap")) (send (enc k k1-0)))
    ((load lk (cat pt-2 old)) (stor lk (cat pt-3 k "init"))
      (send (hash k))))
  (label 1265)
  (parent 517)
  (unrealized (2 3) (4 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1-0) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (5 2)) ((3 4) (0 0)) ((4 1) (3 0))
    ((5 4) (4 0)) ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k1 "wrap")
    (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation encryption-test (added-strand make 3) (hash k) (5 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "decrypt"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((recv (hash k)) (recv (hash k1-0)) (load lk (cat pt-1 k "decrypt"))
      (load lk (cat pt-3 k1-0 "wrap")) (send (enc k k1-0)))
    ((load lk-0 (cat pt-4 old-0)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k))))
  (label 1266)
  (parent 517)
  (unrealized (3 3) (5 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old mesg) (pt pt-0 pt-1 pt-2 pt-3 pval) (k k1 k1-0 skey)
    (lk locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1-0) (lk lk))
  (deflistener k)
  (precedes ((1 1) (2 0)) ((2 1) (5 2)) ((3 4) (0 0)) ((4 1) (3 0))
    ((5 4) (4 0)) ((6 1) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k1 "wrap")
    (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (added-listener k) (hash k) (5 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "decrypt"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((recv (hash k)) (recv (hash k1-0)) (load lk (cat pt-1 k "decrypt"))
      (load lk (cat pt-3 k1-0 "wrap")) (send (enc k k1-0)))
    ((recv k) (send k)))
  (label 1267)
  (parent 517)
  (unrealized (3 3) (5 3) (6 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old mesg) (pt pt-0 pt-1 pt-2 pt-3 pval) (k k1 k1-0 skey)
    (lk locn))
  (deflistener k)
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk))
  (defstrand make 3 (old old) (k k) (lk lk))
  (precedes ((1 1) (3 0)) ((2 4) (0 0)) ((3 1) (2 0)) ((4 4) (3 0))
    ((5 1) (1 0)) ((5 2) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k1 "wrap")
    (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 1 6 make 3) (hash k) (5 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt-3 k "init")) (stor lk (cat pt k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt k "decrypt"))
      (load lk (cat pt-0 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((recv (hash k)) (recv (hash k1-0)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k1-0 "wrap")) (send (enc k k1-0)))
    ((load lk (cat pt-2 old)) (stor lk (cat pt-3 k "init"))
      (send (hash k))))
  (label 1268)
  (parent 518)
  (unrealized (2 3) (4 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((1 1) (5 2)) ((2 1) (4 0)) ((3 4) (0 0))
    ((4 1) (3 0)) ((5 4) (4 0)) ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k1 "wrap")
    (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation encryption-test (added-strand make 3) (hash k) (5 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "decrypt"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((recv (hash k)) (recv (hash k1-0)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-3 k1-0 "wrap")) (send (enc k k1-0)))
    ((load lk-0 (cat pt-4 old-0)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k))))
  (label 1269)
  (parent 518)
  (unrealized (3 3) (5 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old mesg) (pt pt-0 pt-1 pt-2 pt-3 pval) (k k1 k1-0 skey)
    (lk locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk))
  (deflistener k)
  (precedes ((1 1) (2 0)) ((1 1) (5 2)) ((2 1) (4 0)) ((3 4) (0 0))
    ((4 1) (3 0)) ((5 4) (4 0)) ((6 1) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k1 "wrap")
    (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (added-listener k) (hash k) (5 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "decrypt"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((recv (hash k)) (recv (hash k1-0)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-3 k1-0 "wrap")) (send (enc k k1-0)))
    ((recv k) (send k)))
  (label 1270)
  (parent 518)
  (unrealized (3 3) (5 3) (6 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old mesg) (pt pt-0 pt-1 pt-2 pt-3 pval) (k k1 k1-0 skey)
    (lk locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk))
  (deflistener k)
  (precedes ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (3 2)) ((3 4) (0 0))
    ((4 1) (3 0)) ((5 4) (4 0)) ((6 1) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k1 "wrap")
    (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation nonce-test (displaced 7 1 make 2) k (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "decrypt"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((recv (hash k)) (recv (hash k1-0)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-3 k1-0 "wrap")) (send (enc k k1-0)))
    ((recv k) (send k)))
  (label 1271)
  (parent 521)
  (unrealized (3 3) (5 3) (6 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((1 1) (5 2)) ((2 1) (3 2)) ((3 4) (0 0))
    ((4 1) (3 0)) ((5 4) (4 0)) ((6 1) (5 0)) ((7 1) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k1 "wrap")
    (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation nonce-test (added-strand make 2) k (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "decrypt"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((recv (hash k)) (recv (hash k1-0)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-3 k1-0 "wrap")) (send (enc k k1-0)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-4 old-0)) (stor lk-0 (cat pt-5 k "init"))))
  (label 1272)
  (parent 521)
  (unrealized (3 3) (5 3) (6 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 cur mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 k1-0 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur cur) (k k1-0) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (3 2)) ((3 4) (0 0)) ((4 1) (3 0))
    ((5 4) (4 0)) ((6 1) (7 0)) ((6 2) (5 0)) ((7 1) (5 3)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k1 "wrap")
    (cat k1-0 cur) (cat k1-0 "wrap"))
  (facts (neq cur "decrypt") (neq cur "wrap") (neq "init" "decrypt")
    (neq "init" "wrap"))
  (rule invShearsRule fact-set-decrypt-neq0 trRl_set-wrap-at-1
    gen-st-set-wrap-0 fact-set-wrap-neq0 trRl_make-at-0)
  (operation channel-test (added-strand set-wrap 2)
    (ch-msg lk-0 (cat pt-3 k1-0 "wrap")) (5 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "decrypt"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-0 (cat pt-5 k "init"))
      (load lk-0 (cat pt-3 k1-0 "wrap")) (send (enc k k1-0)))
    ((load lk-0 (cat pt-4 old-0)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-6 k1-0 cur))
      (stor lk-0 (cat pt-3 k1-0 "wrap"))))
  (label 1273)
  (parent 532)
  (unrealized (3 3) (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 cur mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 k1-0 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1) (lk lk-0))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur cur) (k k1-0) (lk lk))
  (precedes ((1 1) (2 2)) ((2 4) (0 0)) ((3 1) (2 0)) ((4 1) (5 2))
    ((4 1) (7 0)) ((5 4) (3 0)) ((6 1) (1 0)) ((6 2) (5 0))
    ((7 1) (5 3)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k1 "wrap")
    (cat k1-0 cur) (cat k1-0 "wrap"))
  (facts (neq cur "decrypt") (neq cur "wrap") (neq "init" "decrypt")
    (neq "init" "wrap"))
  (rule invShearsRule fact-set-decrypt-neq0 trRl_set-wrap-at-1
    gen-st-set-wrap-0 fact-set-wrap-neq0 trRl_make-at-0)
  (operation channel-test (added-strand set-wrap 2)
    (ch-msg lk (cat pt-3 k1-0 "wrap")) (5 3))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-5 k "init")) (stor lk-0 (cat pt k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt k "decrypt"))
      (load lk-0 (cat pt-0 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk (cat pt-1 old)) (stor lk (cat pt-2 k "init")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk (cat pt-2 k "init"))
      (load lk (cat pt-3 k1-0 "wrap")) (send (enc k k1-0)))
    ((load lk-0 (cat pt-4 old-0)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k)))
    ((load lk (cat pt-6 k1-0 cur)) (stor lk (cat pt-3 k1-0 "wrap"))))
  (label 1274)
  (parent 533)
  (unrealized (2 3) (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "decrypt") (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (0 0)) ((3 4) (0 0))
    ((4 1) (3 0)) ((5 1) (6 0)) ((6 1) (7 0)) ((7 1) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k1 "wrap"))
  (facts (neq "decrypt" "decrypt") (neq "decrypt" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-decrypt-at-1 fact-set-decrypt-neq0 gen-st-set-wrap-0
    fact-set-wrap-neq0 trRl_make-at-0)
  (operation nonce-test (added-strand set-decrypt 2) k (4 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk-0 (cat pt-4 k "init"))
    (ch-msg lk-0 (cat pt-5 k "decrypt")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt-5 k "decrypt")))
    ((load lk-0 (cat pt-5 k "decrypt"))
      (stor lk-0 (cat pt-6 k "decrypt"))))
  (label 1275)
  (parent 536)
  (unrealized (3 3) (4 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (0 0)) ((3 4) (0 0))
    ((4 1) (3 0)) ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation nonce-test (added-strand make 2) k (4 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk-0 (cat pt-4 k "init"))
    (ch-msg lk-0 (cat pt-5 k "decrypt")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt-5 k "decrypt")))
    ((load lk-1 (cat pt-6 old-1)) (stor lk-1 (cat pt-7 k "init"))))
  (label 1276)
  (parent 536)
  (unrealized (3 3) (4 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "decrypt") (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (0 0)) ((3 4) (0 0))
    ((4 1) (3 0)) ((5 1) (6 0)) ((6 1) (7 0)) ((7 1) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k1 "wrap"))
  (facts (neq "decrypt" "decrypt") (neq "decrypt" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule fact-set-decrypt-neq0 trRl_set-wrap-at-1 gen-st-set-wrap-0
    fact-set-wrap-neq0 trRl_make-at-0)
  (operation nonce-test (added-strand set-wrap 2) k (4 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk-0 (cat pt-4 k "init"))
    (ch-msg lk-0 (cat pt-5 k "decrypt")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt-5 k "decrypt")))
    ((load lk-0 (cat pt-5 k "decrypt"))
      (stor lk-0 (cat pt-6 k "wrap"))))
  (label 1277)
  (parent 536)
  (unrealized (3 3) (4 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1-0) (lk lk-0))
  (precedes ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (0 0)) ((3 4) (0 0))
    ((4 1) (3 0)) ((5 1) (6 0)) ((6 1) (7 2)) ((7 4) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k1 "wrap")
    (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0 gen-st-wrap-1)
  (operation nonce-test (added-strand wrap 5) k (4 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk-0 (cat pt-4 k "init"))
    (ch-msg lk-0 (cat pt-5 k "decrypt")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt-5 k "decrypt")))
    ((recv (hash k)) (recv (hash k1-0))
      (load lk-0 (cat pt-5 k "decrypt"))
      (load lk-0 (cat pt-6 k1-0 "wrap")) (send (enc k k1-0))))
  (label 1278)
  (parent 536)
  (unrealized (3 3) (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk-0))
  (precedes ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (0 0)) ((3 4) (0 0))
    ((4 1) (3 0)) ((5 1) (6 0)) ((5 1) (7 2)) ((6 1) (4 0))
    ((7 4) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (4 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk-0 (cat pt-4 k "init"))
    (ch-msg lk-0 (cat pt-5 k "decrypt")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt-5 k "decrypt")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-0 (cat pt-4 k "init"))
      (load lk-0 (cat pt-6 k1-0 "wrap")) (send (enc k k1-0))))
  (label 1279)
  (parent 536)
  (unrealized (3 3) (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (7 2)) ((2 1) (0 0)) ((3 4) (0 0))
    ((4 1) (3 0)) ((5 1) (6 0)) ((6 1) (4 0)) ((7 4) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (4 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk-0 (cat pt-4 k "init"))
    (ch-msg lk-0 (cat pt-5 k "decrypt")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt-5 k "decrypt")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-6 k1-0 "wrap")) (send (enc k k1-0))))
  (label 1280)
  (parent 536)
  (unrealized (3 3) (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "wrap") (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (0 0)) ((3 4) (0 0))
    ((4 1) (3 0)) ((5 1) (6 0)) ((6 1) (7 0)) ((7 1) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "wrap" "decrypt") (neq "wrap" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-decrypt-at-1 fact-set-decrypt-neq0 gen-st-set-wrap-0
    fact-set-wrap-neq0 trRl_make-at-0)
  (operation nonce-test (added-strand set-decrypt 2) k (4 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk-0 (cat pt-4 k "init"))
    (ch-msg lk-0 (cat pt-5 k "wrap")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt-5 k "wrap")))
    ((load lk-0 (cat pt-5 k "wrap"))
      (stor lk-0 (cat pt-6 k "decrypt"))))
  (label 1281)
  (parent 537)
  (unrealized (3 3) (4 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (0 0)) ((3 4) (0 0))
    ((4 1) (3 0)) ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation nonce-test (added-strand make 2) k (4 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk-0 (cat pt-4 k "init"))
    (ch-msg lk-0 (cat pt-5 k "wrap")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt-5 k "wrap")))
    ((load lk-1 (cat pt-6 old-1)) (stor lk-1 (cat pt-7 k "init"))))
  (label 1282)
  (parent 537)
  (unrealized (3 3) (4 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "wrap") (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (0 0)) ((3 4) (0 0))
    ((4 1) (3 0)) ((5 1) (6 0)) ((6 1) (7 0)) ((7 1) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "wrap" "decrypt") (neq "wrap" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule fact-set-decrypt-neq0 trRl_set-wrap-at-1 gen-st-set-wrap-0
    fact-set-wrap-neq0 trRl_make-at-0)
  (operation nonce-test (added-strand set-wrap 2) k (4 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk-0 (cat pt-4 k "init"))
    (ch-msg lk-0 (cat pt-5 k "wrap")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt-5 k "wrap")))
    ((load lk-0 (cat pt-5 k "wrap")) (stor lk-0 (cat pt-6 k "wrap"))))
  (label 1283)
  (parent 537)
  (unrealized (3 3) (4 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1-0) (lk lk-0))
  (precedes ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (0 0)) ((3 4) (0 0))
    ((4 1) (3 0)) ((5 1) (6 0)) ((6 1) (7 2)) ((7 4) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap")
    (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0 gen-st-wrap-1)
  (operation nonce-test (added-strand wrap 5) k (4 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk-0 (cat pt-4 k "init"))
    (ch-msg lk-0 (cat pt-5 k "wrap")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt-5 k "wrap")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-0 (cat pt-5 k "wrap"))
      (load lk-0 (cat pt-6 k1-0 "wrap")) (send (enc k k1-0))))
  (label 1284)
  (parent 537)
  (unrealized (3 3) (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk-0))
  (precedes ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (0 0)) ((3 4) (0 0))
    ((4 1) (3 0)) ((5 1) (6 0)) ((5 1) (7 2)) ((6 1) (4 0))
    ((7 4) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (4 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk-0 (cat pt-4 k "init"))
    (ch-msg lk-0 (cat pt-5 k "wrap")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt-5 k "wrap")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-0 (cat pt-4 k "init"))
      (load lk-0 (cat pt-6 k1-0 "wrap")) (send (enc k k1-0))))
  (label 1285)
  (parent 537)
  (unrealized (3 3) (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (7 2)) ((2 1) (0 0)) ((3 4) (0 0))
    ((4 1) (3 0)) ((5 1) (6 0)) ((6 1) (4 0)) ((7 4) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (4 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk-0 (cat pt-4 k "init"))
    (ch-msg lk-0 (cat pt-5 k "wrap")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt-5 k "wrap")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-6 k1-0 "wrap")) (send (enc k k1-0))))
  (label 1286)
  (parent 537)
  (unrealized (3 3) (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (0 0)) ((3 4) (0 0))
    ((4 1) (3 0)) ((5 4) (4 0)) ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 5 7 make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-0 (cat pt-5 k "init"))
      (load lk-0 (cat pt-3 k1-0 "wrap")) (send (enc k k1-0)))
    ((load lk-0 (cat pt-4 old-0)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k))))
  (label 1287)
  (parent 538)
  (unrealized (3 3) (5 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 k1-0 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk-0))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (0 0)) ((3 4) (0 0))
    ((4 1) (3 0)) ((5 1) (6 2)) ((6 4) (4 0)) ((7 2) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation encryption-test (added-strand make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-0 (cat pt-4 k "init"))
      (load lk-0 (cat pt-5 k1-0 "wrap")) (send (enc k k1-0)))
    ((load lk-1 (cat pt-6 old-1)) (stor lk-1 (cat pt-7 k "init"))
      (send (hash k))))
  (label 1288)
  (parent 538)
  (unrealized (3 3) (6 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk-0))
  (deflistener k)
  (precedes ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (0 0)) ((3 4) (0 0))
    ((4 1) (3 0)) ((5 1) (6 2)) ((6 4) (4 0)) ((7 1) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (added-listener k) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-0 (cat pt-4 k "init"))
      (load lk-0 (cat pt-5 k1-0 "wrap")) (send (enc k k1-0)))
    ((recv k) (send k)))
  (label 1289)
  (parent 538)
  (unrealized (3 3) (6 3) (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (precedes ((1 1) (0 0)) ((2 4) (0 0)) ((3 1) (2 0)) ((4 1) (3 0))
    ((5 4) (3 0)) ((6 1) (1 0)) ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 1 7 make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-5 k "init")) (stor lk-0 (cat pt k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-5 k "init"))
      (load lk-0 (cat pt-0 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk (cat pt-1 old)) (stor lk (cat pt-2 k "init")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-0 (cat pt-5 k "init"))
      (load lk-0 (cat pt-3 k1-0 "wrap")) (send (enc k k1-0)))
    ((load lk-0 (cat pt-4 old-0)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k))))
  (label 1290)
  (parent 539)
  (unrealized (2 3) (5 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 k1-0 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((1 1) (6 2)) ((2 1) (0 0)) ((3 4) (0 0))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 4) (4 0)) ((7 2) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation encryption-test (added-strand make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-5 k1-0 "wrap")) (send (enc k k1-0)))
    ((load lk-1 (cat pt-6 old-1)) (stor lk-1 (cat pt-7 k "init"))
      (send (hash k))))
  (label 1291)
  (parent 539)
  (unrealized (3 3) (6 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk))
  (deflistener k)
  (precedes ((1 1) (2 0)) ((1 1) (6 2)) ((2 1) (0 0)) ((3 4) (0 0))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 4) (4 0)) ((7 1) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (added-listener k) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-5 k1-0 "wrap")) (send (enc k k1-0)))
    ((recv k) (send k)))
  (label 1292)
  (parent 539)
  (unrealized (3 3) (6 3) (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old mesg) (pt pt-0 pt-1 pt-2 pt-3 pval) (k k1 k1-0 skey)
    (lk locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk))
  (deflistener k)
  (precedes ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (0 0)) ((3 4) (0 0))
    ((4 1) (3 0)) ((5 4) (4 0)) ((6 1) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation nonce-test (displaced 7 1 make 2) k (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((recv (hash k)) (recv (hash k1-0)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-3 k1-0 "wrap")) (send (enc k k1-0)))
    ((recv k) (send k)))
  (label 1293)
  (parent 542)
  (unrealized (3 3) (5 3) (6 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((1 1) (5 2)) ((2 1) (0 0)) ((3 4) (0 0))
    ((4 1) (3 0)) ((5 4) (4 0)) ((6 1) (5 0)) ((7 1) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation nonce-test (added-strand make 2) k (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((recv (hash k)) (recv (hash k1-0)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-3 k1-0 "wrap")) (send (enc k k1-0)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-4 old-0)) (stor lk-0 (cat pt-5 k "init"))))
  (label 1294)
  (parent 542)
  (unrealized (3 3) (5 3) (6 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 cur mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 k1-0 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur cur) (k k1-0) (lk lk-0))
  (precedes ((1 1) (2 0)) ((1 1) (3 2)) ((2 1) (0 0)) ((3 4) (0 0))
    ((4 1) (3 0)) ((5 4) (4 0)) ((6 1) (7 0)) ((6 2) (5 0))
    ((7 1) (5 3)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 cur)
    (cat k1-0 "wrap"))
  (facts (neq cur "decrypt") (neq cur "wrap") (neq "init" "decrypt")
    (neq "init" "wrap"))
  (rule invShearsRule fact-set-decrypt-neq0 trRl_set-wrap-at-1
    gen-st-set-wrap-0 fact-set-wrap-neq0 trRl_make-at-0)
  (operation channel-test (added-strand set-wrap 2)
    (ch-msg lk-0 (cat pt-3 k1-0 "wrap")) (5 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-0 (cat pt-5 k "init"))
      (load lk-0 (cat pt-3 k1-0 "wrap")) (send (enc k k1-0)))
    ((load lk-0 (cat pt-4 old-0)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-6 k1-0 cur))
      (stor lk-0 (cat pt-3 k1-0 "wrap"))))
  (label 1295)
  (parent 553)
  (unrealized (3 3) (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 cur mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 k1-0 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur cur) (k k1-0) (lk lk))
  (precedes ((1 1) (0 0)) ((2 4) (0 0)) ((3 1) (2 0)) ((4 1) (5 2))
    ((4 1) (7 0)) ((5 4) (3 0)) ((6 1) (1 0)) ((6 2) (5 0))
    ((7 1) (5 3)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 cur)
    (cat k1-0 "wrap"))
  (facts (neq cur "decrypt") (neq cur "wrap") (neq "init" "decrypt")
    (neq "init" "wrap"))
  (rule invShearsRule fact-set-decrypt-neq0 trRl_set-wrap-at-1
    gen-st-set-wrap-0 fact-set-wrap-neq0 trRl_make-at-0)
  (operation channel-test (added-strand set-wrap 2)
    (ch-msg lk (cat pt-3 k1-0 "wrap")) (5 3))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-5 k "init")) (stor lk-0 (cat pt k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-5 k "init"))
      (load lk-0 (cat pt-0 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk (cat pt-1 old)) (stor lk (cat pt-2 k "init")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk (cat pt-2 k "init"))
      (load lk (cat pt-3 k1-0 "wrap")) (send (enc k k1-0)))
    ((load lk-0 (cat pt-4 old-0)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k)))
    ((load lk (cat pt-6 k1-0 cur)) (stor lk (cat pt-3 k1-0 "wrap"))))
  (label 1296)
  (parent 554)
  (unrealized (2 3) (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (defstrand set-decrypt 2 (cur "decrypt") (k k) (lk lk-0))
  (precedes ((1 1) (3 0)) ((2 4) (0 0)) ((3 1) (2 0)) ((4 1) (5 0))
    ((5 1) (7 0)) ((6 1) (3 0)) ((7 1) (3 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k1 "wrap"))
  (facts (neq "decrypt" "decrypt") (neq "decrypt" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-decrypt-at-1 fact-set-decrypt-neq0 gen-st-set-wrap-0
    fact-set-wrap-neq0 trRl_make-at-0)
  (operation nonce-test (added-strand set-decrypt 2) k (3 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk-0 (cat pt-3 k "init"))
    (ch-msg lk-0 (cat pt-4 k "decrypt"))
    (ch-msg lk-1 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init")))
    ((load lk-0 (cat pt-4 k "decrypt"))
      (stor lk-0 (cat pt-7 k "decrypt"))))
  (label 1297)
  (parent 557)
  (unrealized (2 3) (3 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-1))
  (precedes ((1 1) (3 0)) ((2 4) (0 0)) ((3 1) (2 0)) ((4 1) (5 0))
    ((5 1) (3 0)) ((6 1) (7 0)) ((7 1) (3 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-wrap-at-1 trRl_make-at-0)
  (operation nonce-test (added-strand set-wrap 2) k (3 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk-0 (cat pt-3 k "init"))
    (ch-msg lk-0 (cat pt-4 k "decrypt"))
    (ch-msg lk-1 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init")))
    ((load lk-1 (cat pt-6 k "init")) (stor lk-1 (cat pt-7 k "wrap"))))
  (label 1298)
  (parent 557)
  (unrealized (2 3) (3 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (defstrand set-wrap 2 (cur "decrypt") (k k) (lk lk-0))
  (precedes ((1 1) (3 0)) ((2 4) (0 0)) ((3 1) (2 0)) ((4 1) (5 0))
    ((5 1) (7 0)) ((6 1) (3 0)) ((7 1) (3 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k1 "wrap"))
  (facts (neq "decrypt" "decrypt") (neq "decrypt" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule fact-set-decrypt-neq0 trRl_set-wrap-at-1 gen-st-set-wrap-0
    fact-set-wrap-neq0 trRl_make-at-0)
  (operation nonce-test (added-strand set-wrap 2) k (3 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk-0 (cat pt-3 k "init"))
    (ch-msg lk-0 (cat pt-4 k "decrypt"))
    (ch-msg lk-1 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init")))
    ((load lk-0 (cat pt-4 k "decrypt"))
      (stor lk-0 (cat pt-7 k "wrap"))))
  (label 1299)
  (parent 557)
  (unrealized (2 3) (3 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 k1-0 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk-1))
  (precedes ((1 1) (3 0)) ((2 4) (0 0)) ((3 1) (2 0)) ((4 1) (5 0))
    ((5 1) (3 0)) ((6 1) (7 2)) ((7 4) (3 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (3 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk-0 (cat pt-3 k "init"))
    (ch-msg lk-0 (cat pt-4 k "decrypt"))
    (ch-msg lk-1 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-1 (cat pt-6 k "init"))
      (load lk-1 (cat pt-7 k1-0 "wrap")) (send (enc k k1-0))))
  (label 1300)
  (parent 557)
  (unrealized (2 3) (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 k1-0 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1-0) (lk lk-0))
  (precedes ((1 1) (3 0)) ((2 4) (0 0)) ((3 1) (2 0)) ((4 1) (5 0))
    ((5 1) (7 2)) ((6 1) (3 0)) ((7 4) (3 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k1 "wrap")
    (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0 gen-st-wrap-1)
  (operation nonce-test (added-strand wrap 5) k (3 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk-0 (cat pt-3 k "init"))
    (ch-msg lk-0 (cat pt-4 k "decrypt"))
    (ch-msg lk-1 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init")))
    ((recv (hash k)) (recv (hash k1-0))
      (load lk-0 (cat pt-4 k "decrypt"))
      (load lk-0 (cat pt-7 k1-0 "wrap")) (send (enc k k1-0))))
  (label 1301)
  (parent 557)
  (unrealized (2 3) (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 k1-0 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk-0))
  (precedes ((1 1) (3 0)) ((2 4) (0 0)) ((3 1) (2 0)) ((4 1) (5 0))
    ((4 1) (7 2)) ((5 1) (3 0)) ((6 1) (3 0)) ((7 4) (3 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (3 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk-0 (cat pt-3 k "init"))
    (ch-msg lk-0 (cat pt-4 k "decrypt"))
    (ch-msg lk-1 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-7 k1-0 "wrap")) (send (enc k k1-0))))
  (label 1302)
  (parent 557)
  (unrealized (2 3) (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 k1-0 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk))
  (precedes ((1 1) (7 2)) ((2 4) (0 0)) ((3 1) (2 0)) ((4 1) (5 0))
    ((5 1) (3 0)) ((6 1) (3 0)) ((7 4) (3 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (3 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk-0 (cat pt-3 k "init"))
    (ch-msg lk-0 (cat pt-4 k "decrypt"))
    (ch-msg lk-1 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-7 k1-0 "wrap")) (send (enc k k1-0))))
  (label 1303)
  (parent 557)
  (unrealized (2 3) (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1-0) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (precedes ((1 1) (3 0)) ((2 4) (0 0)) ((3 1) (2 0)) ((4 1) (5 2))
    ((5 4) (3 0)) ((6 1) (4 0)) ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k1 "wrap")
    (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 4 7 make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-5 k "init")) (stor lk-0 (cat pt-2 k "decrypt")))
    ((recv (hash k)) (recv (hash k1-0))
      (load lk-0 (cat pt-2 k "decrypt"))
      (load lk-0 (cat pt-3 k1-0 "wrap")) (send (enc k k1-0)))
    ((load lk-0 (cat pt-4 old-0)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k))))
  (label 1304)
  (parent 560)
  (unrealized (2 3) (5 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 k1-0 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1-0) (lk lk-0))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (3 0)) ((2 4) (0 0)) ((3 1) (2 0)) ((4 1) (5 0))
    ((5 1) (6 2)) ((6 4) (3 0)) ((7 2) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k1 "wrap")
    (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation encryption-test (added-strand make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k1-0))
      (load lk-0 (cat pt-4 k "decrypt"))
      (load lk-0 (cat pt-5 k1-0 "wrap")) (send (enc k k1-0)))
    ((load lk-1 (cat pt-6 old-1)) (stor lk-1 (cat pt-7 k "init"))
      (send (hash k))))
  (label 1305)
  (parent 560)
  (unrealized (2 3) (6 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1-0) (lk lk-0))
  (deflistener k)
  (precedes ((1 1) (3 0)) ((2 4) (0 0)) ((3 1) (2 0)) ((4 1) (5 0))
    ((5 1) (6 2)) ((6 4) (3 0)) ((7 1) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k1 "wrap")
    (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (added-listener k) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k1-0))
      (load lk-0 (cat pt-4 k "decrypt"))
      (load lk-0 (cat pt-5 k1-0 "wrap")) (send (enc k k1-0)))
    ((recv k) (send k)))
  (label 1306)
  (parent 560)
  (unrealized (2 3) (6 3) (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (precedes ((1 1) (3 0)) ((2 4) (0 0)) ((3 1) (2 0)) ((4 1) (3 0))
    ((5 4) (3 0)) ((6 1) (4 0)) ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 4 7 make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-5 k "init")) (stor lk-0 (cat pt-2 k "decrypt")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-0 (cat pt-5 k "init"))
      (load lk-0 (cat pt-3 k1-0 "wrap")) (send (enc k k1-0)))
    ((load lk-0 (cat pt-4 old-0)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k))))
  (label 1307)
  (parent 561)
  (unrealized (2 3) (5 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 k1-0 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk-0))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (3 0)) ((2 4) (0 0)) ((3 1) (2 0)) ((4 1) (5 0))
    ((4 1) (6 2)) ((5 1) (3 0)) ((6 4) (3 0)) ((7 2) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation encryption-test (added-strand make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-5 k1-0 "wrap")) (send (enc k k1-0)))
    ((load lk-1 (cat pt-6 old-1)) (stor lk-1 (cat pt-7 k "init"))
      (send (hash k))))
  (label 1308)
  (parent 561)
  (unrealized (2 3) (6 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk-0))
  (deflistener k)
  (precedes ((1 1) (3 0)) ((2 4) (0 0)) ((3 1) (2 0)) ((4 1) (5 0))
    ((4 1) (6 2)) ((5 1) (3 0)) ((6 4) (3 0)) ((7 1) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (added-listener k) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-5 k1-0 "wrap")) (send (enc k k1-0)))
    ((recv k) (send k)))
  (label 1309)
  (parent 561)
  (unrealized (2 3) (6 3) (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (precedes ((1 1) (5 2)) ((2 4) (0 0)) ((3 1) (2 0)) ((4 1) (3 0))
    ((5 4) (3 0)) ((6 1) (4 0)) ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 4 7 make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-5 k "init")) (stor lk-0 (cat pt-2 k "decrypt")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-3 k1-0 "wrap")) (send (enc k k1-0)))
    ((load lk-0 (cat pt-4 old-0)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k))))
  (label 1310)
  (parent 562)
  (unrealized (2 3) (5 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (precedes ((1 4) (0 0)) ((2 1) (1 0)) ((3 1) (4 0)) ((4 1) (2 0))
    ((5 4) (2 0)) ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 1 7 make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-5 k "init"))
      (load lk-0 (cat pt k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-0 (cat pt-5 k "init"))
      (load lk-0 (cat pt-3 k1-0 "wrap")) (send (enc k k1-0)))
    ((load lk-0 (cat pt-4 old-0)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k))))
  (label 1311)
  (parent 562)
  (unrealized (1 3) (5 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 k1-0 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (6 2)) ((2 4) (0 0)) ((3 1) (2 0)) ((4 1) (5 0))
    ((5 1) (3 0)) ((6 4) (3 0)) ((7 2) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation encryption-test (added-strand make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-5 k1-0 "wrap")) (send (enc k k1-0)))
    ((load lk-1 (cat pt-6 old-1)) (stor lk-1 (cat pt-7 k "init"))
      (send (hash k))))
  (label 1312)
  (parent 562)
  (unrealized (2 3) (6 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk))
  (deflistener k)
  (precedes ((1 1) (6 2)) ((2 4) (0 0)) ((3 1) (2 0)) ((4 1) (5 0))
    ((5 1) (3 0)) ((6 4) (3 0)) ((7 1) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (added-listener k) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-5 k1-0 "wrap")) (send (enc k k1-0)))
    ((recv k) (send k)))
  (label 1313)
  (parent 562)
  (unrealized (2 3) (6 3) (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (defstrand set-decrypt 2 (cur "wrap") (k k) (lk lk-0))
  (precedes ((1 1) (3 0)) ((2 4) (0 0)) ((3 1) (2 0)) ((4 1) (5 0))
    ((5 1) (7 0)) ((6 1) (3 0)) ((7 1) (3 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "wrap" "decrypt") (neq "wrap" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-decrypt-at-1 fact-set-decrypt-neq0 gen-st-set-wrap-0
    fact-set-wrap-neq0 trRl_make-at-0)
  (operation nonce-test (added-strand set-decrypt 2) k (3 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk-0 (cat pt-3 k "init"))
    (ch-msg lk-0 (cat pt-4 k "wrap")) (ch-msg lk-1 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "wrap")))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init")))
    ((load lk-0 (cat pt-4 k "wrap"))
      (stor lk-0 (cat pt-7 k "decrypt"))))
  (label 1314)
  (parent 564)
  (unrealized (2 3) (3 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (defstrand set-wrap 2 (cur "wrap") (k k) (lk lk-0))
  (precedes ((1 1) (3 0)) ((2 4) (0 0)) ((3 1) (2 0)) ((4 1) (5 0))
    ((5 1) (7 0)) ((6 1) (3 0)) ((7 1) (3 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "wrap" "decrypt") (neq "wrap" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule fact-set-decrypt-neq0 trRl_set-wrap-at-1 gen-st-set-wrap-0
    fact-set-wrap-neq0 trRl_make-at-0)
  (operation nonce-test (added-strand set-wrap 2) k (3 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk-0 (cat pt-3 k "init"))
    (ch-msg lk-0 (cat pt-4 k "wrap")) (ch-msg lk-1 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "wrap")))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init")))
    ((load lk-0 (cat pt-4 k "wrap")) (stor lk-0 (cat pt-7 k "wrap"))))
  (label 1315)
  (parent 564)
  (unrealized (2 3) (3 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 k1-0 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk-1))
  (precedes ((1 1) (3 0)) ((2 4) (0 0)) ((3 1) (2 0)) ((4 1) (5 0))
    ((5 1) (3 0)) ((6 1) (7 2)) ((7 4) (3 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (3 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk-0 (cat pt-3 k "init"))
    (ch-msg lk-0 (cat pt-4 k "wrap")) (ch-msg lk-1 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "wrap")))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-1 (cat pt-6 k "init"))
      (load lk-1 (cat pt-7 k1-0 "wrap")) (send (enc k k1-0))))
  (label 1316)
  (parent 564)
  (unrealized (2 3) (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 k1-0 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1-0) (lk lk-0))
  (precedes ((1 1) (3 0)) ((2 4) (0 0)) ((3 1) (2 0)) ((4 1) (5 0))
    ((5 1) (7 2)) ((6 1) (3 0)) ((7 4) (3 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap")
    (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0 gen-st-wrap-1)
  (operation nonce-test (added-strand wrap 5) k (3 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk-0 (cat pt-3 k "init"))
    (ch-msg lk-0 (cat pt-4 k "wrap")) (ch-msg lk-1 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "wrap")))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-0 (cat pt-4 k "wrap"))
      (load lk-0 (cat pt-7 k1-0 "wrap")) (send (enc k k1-0))))
  (label 1317)
  (parent 564)
  (unrealized (2 3) (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 k1-0 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk-0))
  (precedes ((1 1) (3 0)) ((2 4) (0 0)) ((3 1) (2 0)) ((4 1) (5 0))
    ((4 1) (7 2)) ((5 1) (3 0)) ((6 1) (3 0)) ((7 4) (3 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (3 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk-0 (cat pt-3 k "init"))
    (ch-msg lk-0 (cat pt-4 k "wrap")) (ch-msg lk-1 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "wrap")))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-7 k1-0 "wrap")) (send (enc k k1-0))))
  (label 1318)
  (parent 564)
  (unrealized (2 3) (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 k1-0 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk))
  (precedes ((1 1) (7 2)) ((2 4) (0 0)) ((3 1) (2 0)) ((4 1) (5 0))
    ((5 1) (3 0)) ((6 1) (3 0)) ((7 4) (3 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (3 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk-0 (cat pt-3 k "init"))
    (ch-msg lk-0 (cat pt-4 k "wrap")) (ch-msg lk-1 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "wrap")))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-7 k1-0 "wrap")) (send (enc k k1-0))))
  (label 1319)
  (parent 564)
  (unrealized (2 3) (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1-0) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (precedes ((1 1) (3 0)) ((2 4) (0 0)) ((3 1) (2 0)) ((4 1) (5 2))
    ((5 4) (3 0)) ((6 1) (4 0)) ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap")
    (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 4 7 make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-5 k "init")) (stor lk-0 (cat pt-2 k "wrap")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-0 (cat pt-2 k "wrap"))
      (load lk-0 (cat pt-3 k1-0 "wrap")) (send (enc k k1-0)))
    ((load lk-0 (cat pt-4 old-0)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k))))
  (label 1320)
  (parent 566)
  (unrealized (2 3) (5 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 k1-0 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1-0) (lk lk-0))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (3 0)) ((2 4) (0 0)) ((3 1) (2 0)) ((4 1) (5 0))
    ((5 1) (6 2)) ((6 4) (3 0)) ((7 2) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap")
    (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation encryption-test (added-strand make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "wrap")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-0 (cat pt-4 k "wrap"))
      (load lk-0 (cat pt-5 k1-0 "wrap")) (send (enc k k1-0)))
    ((load lk-1 (cat pt-6 old-1)) (stor lk-1 (cat pt-7 k "init"))
      (send (hash k))))
  (label 1321)
  (parent 566)
  (unrealized (2 3) (6 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1-0) (lk lk-0))
  (deflistener k)
  (precedes ((1 1) (3 0)) ((2 4) (0 0)) ((3 1) (2 0)) ((4 1) (5 0))
    ((5 1) (6 2)) ((6 4) (3 0)) ((7 1) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap")
    (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (added-listener k) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "wrap")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-0 (cat pt-4 k "wrap"))
      (load lk-0 (cat pt-5 k1-0 "wrap")) (send (enc k k1-0)))
    ((recv k) (send k)))
  (label 1322)
  (parent 566)
  (unrealized (2 3) (6 3) (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (precedes ((1 1) (3 0)) ((2 4) (0 0)) ((3 1) (2 0)) ((4 1) (3 0))
    ((5 4) (3 0)) ((6 1) (4 0)) ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 4 7 make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-5 k "init")) (stor lk-0 (cat pt-2 k "wrap")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-0 (cat pt-5 k "init"))
      (load lk-0 (cat pt-3 k1-0 "wrap")) (send (enc k k1-0)))
    ((load lk-0 (cat pt-4 old-0)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k))))
  (label 1323)
  (parent 567)
  (unrealized (2 3) (5 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 k1-0 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk-0))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (3 0)) ((2 4) (0 0)) ((3 1) (2 0)) ((4 1) (5 0))
    ((4 1) (6 2)) ((5 1) (3 0)) ((6 4) (3 0)) ((7 2) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation encryption-test (added-strand make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "wrap")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-5 k1-0 "wrap")) (send (enc k k1-0)))
    ((load lk-1 (cat pt-6 old-1)) (stor lk-1 (cat pt-7 k "init"))
      (send (hash k))))
  (label 1324)
  (parent 567)
  (unrealized (2 3) (6 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk-0))
  (deflistener k)
  (precedes ((1 1) (3 0)) ((2 4) (0 0)) ((3 1) (2 0)) ((4 1) (5 0))
    ((4 1) (6 2)) ((5 1) (3 0)) ((6 4) (3 0)) ((7 1) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (added-listener k) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "wrap")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-5 k1-0 "wrap")) (send (enc k k1-0)))
    ((recv k) (send k)))
  (label 1325)
  (parent 567)
  (unrealized (2 3) (6 3) (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (precedes ((1 1) (5 2)) ((2 4) (0 0)) ((3 1) (2 0)) ((4 1) (3 0))
    ((5 4) (3 0)) ((6 1) (4 0)) ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 4 7 make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-5 k "init")) (stor lk-0 (cat pt-2 k "wrap")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-3 k1-0 "wrap")) (send (enc k k1-0)))
    ((load lk-0 (cat pt-4 old-0)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k))))
  (label 1326)
  (parent 568)
  (unrealized (2 3) (5 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (precedes ((1 4) (0 0)) ((2 1) (1 0)) ((3 1) (4 0)) ((4 1) (2 0))
    ((5 4) (2 0)) ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 1 7 make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-5 k "init"))
      (load lk-0 (cat pt k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-2 k "wrap")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-0 (cat pt-5 k "init"))
      (load lk-0 (cat pt-3 k1-0 "wrap")) (send (enc k k1-0)))
    ((load lk-0 (cat pt-4 old-0)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k))))
  (label 1327)
  (parent 568)
  (unrealized (1 3) (5 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 k1-0 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (6 2)) ((2 4) (0 0)) ((3 1) (2 0)) ((4 1) (5 0))
    ((5 1) (3 0)) ((6 4) (3 0)) ((7 2) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation encryption-test (added-strand make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "wrap")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-5 k1-0 "wrap")) (send (enc k k1-0)))
    ((load lk-1 (cat pt-6 old-1)) (stor lk-1 (cat pt-7 k "init"))
      (send (hash k))))
  (label 1328)
  (parent 568)
  (unrealized (2 3) (6 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk))
  (deflistener k)
  (precedes ((1 1) (6 2)) ((2 4) (0 0)) ((3 1) (2 0)) ((4 1) (5 0))
    ((5 1) (3 0)) ((6 4) (3 0)) ((7 1) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (added-listener k) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "wrap")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-5 k1-0 "wrap")) (send (enc k k1-0)))
    ((recv k) (send k)))
  (label 1329)
  (parent 568)
  (unrealized (2 3) (6 3) (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 cur mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur cur) (k k1-0) (lk lk-0))
  (precedes ((1 1) (3 0)) ((2 4) (0 0)) ((3 1) (2 0)) ((4 4) (3 0))
    ((5 1) (6 0)) ((5 2) (4 0)) ((6 1) (4 3)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 cur)
    (cat k1-0 "wrap"))
  (facts (neq cur "decrypt") (neq cur "wrap"))
  (rule invShearsRule fact-set-decrypt-neq0 trRl_set-wrap-at-1
    gen-st-set-wrap-0 fact-set-wrap-neq0 trRl_make-at-0)
  (operation channel-test (added-strand set-wrap 2)
    (ch-msg lk-0 (cat pt-2 k1-0 "wrap")) (4 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-0 (cat pt-4 k "init"))
      (load lk-0 (cat pt-2 k1-0 "wrap")) (send (enc k k1-0)))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-5 k1-0 cur))
      (stor lk-0 (cat pt-2 k1-0 "wrap"))))
  (label 1330)
  (parent 569)
  (unrealized (2 3) (6 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 cur mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 k1-0 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk-0))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand set-wrap 2 (cur cur) (k k1-0) (lk lk-0))
  (precedes ((1 1) (3 0)) ((2 4) (0 0)) ((3 1) (2 0)) ((4 1) (5 2))
    ((4 1) (7 0)) ((5 4) (3 0)) ((6 2) (5 0)) ((7 1) (5 3)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 cur)
    (cat k1-0 "wrap"))
  (facts (neq cur "decrypt") (neq cur "wrap"))
  (rule invShearsRule fact-set-decrypt-neq0 trRl_set-wrap-at-1
    gen-st-set-wrap-0 fact-set-wrap-neq0 trRl_make-at-0)
  (operation channel-test (added-strand set-wrap 2)
    (ch-msg lk-0 (cat pt-4 k1-0 "wrap")) (5 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-4 k1-0 "wrap")) (send (enc k k1-0)))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-7 k1-0 cur))
      (stor lk-0 (cat pt-4 k1-0 "wrap"))))
  (label 1331)
  (parent 570)
  (unrealized (2 3) (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk-0))
  (deflistener k)
  (precedes ((1 1) (3 0)) ((2 4) (0 0)) ((3 1) (2 0)) ((4 1) (6 0))
    ((5 4) (3 0)) ((6 1) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 "wrap"))
  (operation nonce-test (displaced 7 4 make 2) k (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-4 k1-0 "wrap")) (send (enc k k1-0)))
    ((recv k) (send k)))
  (label 1332)
  (parent 571)
  (unrealized (2 3) (5 3) (6 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 k1-0 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk-0))
  (deflistener k)
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (3 0)) ((2 4) (0 0)) ((3 1) (2 0)) ((4 1) (5 2))
    ((5 4) (3 0)) ((6 1) (5 0)) ((7 1) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation nonce-test (added-strand make 2) k (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-4 k1-0 "wrap")) (send (enc k k1-0)))
    ((recv k) (send k))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init"))))
  (label 1333)
  (parent 571)
  (unrealized (2 3) (5 3) (6 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk))
  (deflistener k)
  (precedes ((1 1) (6 0)) ((2 4) (0 0)) ((3 1) (2 0)) ((4 1) (3 0))
    ((5 4) (3 0)) ((6 1) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 "wrap"))
  (operation nonce-test (displaced 7 1 make 2) k (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-4 k1-0 "wrap")) (send (enc k k1-0)))
    ((recv k) (send k)))
  (label 1334)
  (parent 574)
  (unrealized (2 3) (5 3) (6 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 k1-0 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (5 2)) ((2 4) (0 0)) ((3 1) (2 0)) ((4 1) (3 0))
    ((5 4) (3 0)) ((6 1) (5 0)) ((7 1) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation nonce-test (added-strand make 2) k (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-4 k1-0 "wrap")) (send (enc k k1-0)))
    ((recv k) (send k))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init"))))
  (label 1335)
  (parent 574)
  (unrealized (2 3) (5 3) (6 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (precedes ((1 1) (5 0)) ((2 4) (0 0)) ((3 1) (2 0)) ((4 4) (3 0))
    ((5 1) (4 0)) ((6 1) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation nonce-test (added-strand make 2) k (5 0)
    (ch-msg lk (cat pt-0 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((recv (hash k)) (recv (hash k1-0)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-2 k1-0 "wrap")) (send (enc k k1-0)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init"))))
  (label 1336)
  (parent 575)
  (unrealized (2 3) (4 3) (5 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old mesg) (pt pt-0 pt-1 pt-2 pt-3 pval) (k k1 k1-0 k1-1 skey)
    (lk locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk))
  (deflistener k)
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-1) (lk lk))
  (precedes ((1 1) (6 2)) ((2 4) (0 0)) ((3 1) (2 0)) ((4 4) (3 0))
    ((5 1) (4 0)) ((6 4) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 "wrap")
    (cat k1-1 "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (5 0)
    (ch-msg lk (cat pt-0 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((recv (hash k)) (recv (hash k1-0)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-2 k1-0 "wrap")) (send (enc k k1-0)))
    ((recv k) (send k))
    ((recv (hash k)) (recv (hash k1-1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-3 k1-1 "wrap")) (send (enc k k1-1))))
  (label 1337)
  (parent 575)
  (unrealized (2 3) (4 3) (6 0) (6 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (precedes ((1 1) (4 2)) ((2 4) (0 0)) ((3 1) (2 0)) ((4 4) (3 0))
    ((5 1) (4 0)) ((6 1) (7 0)) ((7 1) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule fact-set-decrypt-neq0 trRl_set-wrap-at-1 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation nonce-test (added-strand set-wrap 2) k (5 0)
    (ch-msg lk-0 (cat pt-4 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((recv (hash k)) (recv (hash k1-0)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-2 k1-0 "wrap")) (send (enc k k1-0)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt-5 k "wrap"))))
  (label 1338)
  (parent 576)
  (unrealized (2 3) (4 3) (5 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (precedes ((1 1) (4 2)) ((2 4) (0 0)) ((3 1) (2 0)) ((4 4) (3 0))
    ((5 1) (4 0)) ((6 1) (7 0)) ((7 1) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-decrypt-at-1 fact-set-decrypt-neq0 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation nonce-test (added-strand set-decrypt 2) k (5 0)
    (ch-msg lk-0 (cat pt-4 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((recv (hash k)) (recv (hash k1-0)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-2 k1-0 "wrap")) (send (enc k k1-0)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((load lk-0 (cat pt-4 k "init"))
      (stor lk-0 (cat pt-5 k "decrypt"))))
  (label 1339)
  (parent 576)
  (unrealized (2 3) (4 3) (5 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 k1-0 k1-1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-1) (lk lk-0))
  (precedes ((1 1) (4 2)) ((2 4) (0 0)) ((3 1) (2 0)) ((4 4) (3 0))
    ((5 1) (4 0)) ((6 1) (7 2)) ((7 4) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 "wrap")
    (cat k1-1 "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (5 0)
    (ch-msg lk-0 (cat pt-4 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((recv (hash k)) (recv (hash k1-0)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-2 k1-0 "wrap")) (send (enc k k1-0)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((recv (hash k)) (recv (hash k1-1)) (load lk-0 (cat pt-4 k "init"))
      (load lk-0 (cat pt-5 k1-1 "wrap")) (send (enc k k1-1))))
  (label 1340)
  (parent 576)
  (unrealized (2 3) (4 3) (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (precedes ((1 1) (2 2)) ((2 4) (0 0)) ((3 1) (2 0)) ((4 1) (5 2))
    ((5 4) (3 0)) ((6 1) (4 0)) ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation channel-test (displaced 7 4 set-wrap 2)
    (ch-msg lk-0 (cat pt-5 k1-0 "wrap")) (5 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt-2 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt-2 k "wrap"))
      (load lk-0 (cat pt-2 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init"))
      (send (hash k))))
  (label 1341)
  (parent 583)
  (unrealized (2 3) (3 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 cur mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 k1-0 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1-0) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur cur) (k k1-0) (lk lk-0))
  (precedes ((1 1) (2 2)) ((2 4) (0 0)) ((3 1) (2 0)) ((4 1) (5 2))
    ((4 1) (7 0)) ((5 4) (3 0)) ((6 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 3)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap") (cat k1-0 cur)
    (cat k1-0 "wrap"))
  (facts (neq cur "decrypt") (neq cur "wrap") (neq "init" "decrypt")
    (neq "init" "wrap"))
  (rule invShearsRule fact-set-decrypt-neq0 trRl_set-wrap-at-1
    gen-st-set-wrap-0 fact-set-wrap-neq0 trRl_make-at-0)
  (operation channel-test (added-strand set-wrap 2)
    (ch-msg lk-0 (cat pt-3 k1-0 "wrap")) (5 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-5 k "init")) (stor lk-0 (cat pt-2 k "wrap")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-0 (cat pt-2 k "wrap"))
      (load lk-0 (cat pt-3 k1-0 "wrap")) (send (enc k k1-0)))
    ((load lk-0 (cat pt-4 old-0)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-6 k1-0 cur))
      (stor lk-0 (cat pt-3 k1-0 "wrap"))))
  (label 1342)
  (parent 583)
  (unrealized (2 3) (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (precedes ((1 4) (0 0)) ((2 1) (1 0)) ((3 1) (4 0)) ((4 1) (5 2))
    ((5 4) (2 0)) ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation channel-test (displaced 7 4 set-wrap 2)
    (ch-msg lk (cat pt-5 k1-0 "wrap")) (5 3))
  (traces ((recv k) (send k))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-4 k "init"))
      (load lk-0 (cat pt k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-2 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-2 k "wrap"))
      (load lk (cat pt-2 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init"))
      (send (hash k))))
  (label 1343)
  (parent 584)
  (unrealized (1 3) (2 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 cur mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 k1-0 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1-0) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur cur) (k k1-0) (lk lk))
  (precedes ((1 4) (0 0)) ((2 1) (1 0)) ((3 1) (4 0)) ((4 1) (5 2))
    ((4 1) (7 0)) ((5 4) (2 0)) ((6 2) (5 0)) ((7 1) (5 3)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap") (cat k1-0 cur)
    (cat k1-0 "wrap"))
  (facts (neq cur "decrypt") (neq cur "wrap") (neq "init" "decrypt")
    (neq "init" "wrap"))
  (rule invShearsRule fact-set-decrypt-neq0 trRl_set-wrap-at-1
    gen-st-set-wrap-0 fact-set-wrap-neq0 trRl_make-at-0)
  (operation channel-test (added-strand set-wrap 2)
    (ch-msg lk (cat pt-3 k1-0 "wrap")) (5 3))
  (traces ((recv k) (send k))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-5 k "init"))
      (load lk-0 (cat pt k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-2 k "wrap")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk (cat pt-2 k "wrap"))
      (load lk (cat pt-3 k1-0 "wrap")) (send (enc k k1-0)))
    ((load lk-0 (cat pt-4 old-0)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k)))
    ((load lk (cat pt-6 k1-0 cur)) (stor lk (cat pt-3 k1-0 "wrap"))))
  (label 1344)
  (parent 584)
  (unrealized (1 3) (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (precedes ((1 1) (2 2)) ((2 4) (0 0)) ((3 1) (2 0)) ((4 1) (5 3))
    ((5 4) (3 0)) ((6 1) (4 0)) ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation channel-test (displaced 7 4 set-wrap 2)
    (ch-msg lk-0 (cat pt-5 k1-0 "wrap")) (5 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt-2 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt-4 k "init"))
      (load lk-0 (cat pt-2 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init"))
      (send (hash k))))
  (label 1345)
  (parent 587)
  (unrealized (2 3) (3 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 cur mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 k1-0 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur cur) (k k1-0) (lk lk-0))
  (precedes ((1 1) (2 2)) ((2 4) (0 0)) ((3 1) (2 0)) ((4 1) (7 0))
    ((5 4) (3 0)) ((6 1) (4 0)) ((6 2) (5 0)) ((7 1) (5 3)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 cur)
    (cat k1-0 "wrap"))
  (facts (neq cur "decrypt") (neq cur "wrap") (neq "init" "decrypt")
    (neq "init" "wrap"))
  (rule shearsRule invShearsRule fact-set-decrypt-neq0
    trRl_set-wrap-at-1 gen-st-set-wrap-0 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation channel-test (added-strand set-wrap 2)
    (ch-msg lk-0 (cat pt-3 k1-0 "wrap")) (5 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-5 k "init")) (stor lk-0 (cat pt-2 k "wrap")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-0 (cat pt-5 k "init"))
      (load lk-0 (cat pt-3 k1-0 "wrap")) (send (enc k k1-0)))
    ((load lk-0 (cat pt-4 old-0)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-6 k1-0 cur))
      (stor lk-0 (cat pt-3 k1-0 "wrap"))))
  (label 1346)
  (parent 587)
  (unrealized (2 3) (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (precedes ((1 4) (0 0)) ((2 1) (1 0)) ((3 1) (4 0)) ((3 1) (5 2))
    ((4 1) (5 3)) ((5 4) (2 0)) ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation channel-test (displaced 7 4 set-wrap 2)
    (ch-msg lk (cat pt-5 k1-0 "wrap")) (5 3))
  (traces ((recv k) (send k))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-4 k "init"))
      (load lk-0 (cat pt k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-2 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt-2 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init"))
      (send (hash k))))
  (label 1347)
  (parent 588)
  (unrealized (1 3) (2 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 cur mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 k1-0 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur cur) (k k1-0) (lk lk))
  (precedes ((1 4) (0 0)) ((2 1) (1 0)) ((3 1) (4 0)) ((3 1) (5 2))
    ((4 1) (7 0)) ((5 4) (2 0)) ((6 2) (5 0)) ((7 1) (5 3)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 cur)
    (cat k1-0 "wrap"))
  (facts (neq cur "decrypt") (neq cur "wrap") (neq "init" "decrypt")
    (neq "init" "wrap"))
  (rule shearsRule invShearsRule fact-set-decrypt-neq0
    trRl_set-wrap-at-1 gen-st-set-wrap-0 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation channel-test (added-strand set-wrap 2)
    (ch-msg lk (cat pt-3 k1-0 "wrap")) (5 3))
  (traces ((recv k) (send k))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-5 k "init"))
      (load lk-0 (cat pt k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-2 k "wrap")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt-3 k1-0 "wrap")) (send (enc k k1-0)))
    ((load lk-0 (cat pt-4 old-0)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k)))
    ((load lk (cat pt-6 k1-0 cur)) (stor lk (cat pt-3 k1-0 "wrap"))))
  (label 1348)
  (parent 588)
  (unrealized (1 3) (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 cur mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 k1-0 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1-0) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur cur) (k k1-0) (lk lk-0))
  (precedes ((1 1) (2 2)) ((2 4) (0 0)) ((3 1) (2 0)) ((4 1) (5 2))
    ((4 1) (7 0)) ((5 4) (3 0)) ((6 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 3)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k1 "wrap")
    (cat k1-0 cur) (cat k1-0 "wrap"))
  (facts (neq cur "decrypt") (neq cur "wrap") (neq "init" "decrypt")
    (neq "init" "wrap"))
  (rule invShearsRule fact-set-decrypt-neq0 trRl_set-wrap-at-1
    gen-st-set-wrap-0 fact-set-wrap-neq0 trRl_make-at-0)
  (operation channel-test (added-strand set-wrap 2)
    (ch-msg lk-0 (cat pt-3 k1-0 "wrap")) (5 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-5 k "init")) (stor lk-0 (cat pt-2 k "decrypt")))
    ((recv (hash k)) (recv (hash k1-0))
      (load lk-0 (cat pt-2 k "decrypt"))
      (load lk-0 (cat pt-3 k1-0 "wrap")) (send (enc k k1-0)))
    ((load lk-0 (cat pt-4 old-0)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-6 k1-0 cur))
      (stor lk-0 (cat pt-3 k1-0 "wrap"))))
  (label 1349)
  (parent 596)
  (unrealized (2 3) (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 cur mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 k1-0 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1-0) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur cur) (k k1-0) (lk lk))
  (precedes ((1 4) (0 0)) ((2 1) (1 0)) ((3 1) (4 0)) ((4 1) (5 2))
    ((4 1) (7 0)) ((5 4) (2 0)) ((6 2) (5 0)) ((7 1) (5 3)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k1 "wrap")
    (cat k1-0 cur) (cat k1-0 "wrap"))
  (facts (neq cur "decrypt") (neq cur "wrap") (neq "init" "decrypt")
    (neq "init" "wrap"))
  (rule invShearsRule fact-set-decrypt-neq0 trRl_set-wrap-at-1
    gen-st-set-wrap-0 fact-set-wrap-neq0 trRl_make-at-0)
  (operation channel-test (added-strand set-wrap 2)
    (ch-msg lk (cat pt-3 k1-0 "wrap")) (5 3))
  (traces ((recv k) (send k))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-5 k "init"))
      (load lk-0 (cat pt k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk (cat pt-2 k "decrypt"))
      (load lk (cat pt-3 k1-0 "wrap")) (send (enc k k1-0)))
    ((load lk-0 (cat pt-4 old-0)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k)))
    ((load lk (cat pt-6 k1-0 cur)) (stor lk (cat pt-3 k1-0 "wrap"))))
  (label 1350)
  (parent 597)
  (unrealized (1 3) (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 cur mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 k1-0 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur cur) (k k1-0) (lk lk-0))
  (precedes ((1 1) (2 2)) ((2 4) (0 0)) ((3 1) (2 0)) ((4 1) (7 0))
    ((5 4) (3 0)) ((6 1) (4 0)) ((6 2) (5 0)) ((7 1) (5 3)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 cur)
    (cat k1-0 "wrap"))
  (facts (neq cur "decrypt") (neq cur "wrap") (neq "init" "decrypt")
    (neq "init" "wrap"))
  (rule shearsRule invShearsRule fact-set-decrypt-neq0
    trRl_set-wrap-at-1 gen-st-set-wrap-0 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation channel-test (added-strand set-wrap 2)
    (ch-msg lk-0 (cat pt-3 k1-0 "wrap")) (5 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-5 k "init")) (stor lk-0 (cat pt-2 k "decrypt")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-0 (cat pt-5 k "init"))
      (load lk-0 (cat pt-3 k1-0 "wrap")) (send (enc k k1-0)))
    ((load lk-0 (cat pt-4 old-0)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-6 k1-0 cur))
      (stor lk-0 (cat pt-3 k1-0 "wrap"))))
  (label 1351)
  (parent 600)
  (unrealized (2 3) (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 cur mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 k1-0 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur cur) (k k1-0) (lk lk))
  (precedes ((1 4) (0 0)) ((2 1) (1 0)) ((3 1) (4 0)) ((3 1) (5 2))
    ((4 1) (7 0)) ((5 4) (2 0)) ((6 2) (5 0)) ((7 1) (5 3)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 cur)
    (cat k1-0 "wrap"))
  (facts (neq cur "decrypt") (neq cur "wrap") (neq "init" "decrypt")
    (neq "init" "wrap"))
  (rule shearsRule invShearsRule fact-set-decrypt-neq0
    trRl_set-wrap-at-1 gen-st-set-wrap-0 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation channel-test (added-strand set-wrap 2)
    (ch-msg lk (cat pt-3 k1-0 "wrap")) (5 3))
  (traces ((recv k) (send k))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-5 k "init"))
      (load lk-0 (cat pt k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt-3 k1-0 "wrap")) (send (enc k k1-0)))
    ((load lk-0 (cat pt-4 old-0)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k)))
    ((load lk (cat pt-6 k1-0 cur)) (stor lk (cat pt-3 k1-0 "wrap"))))
  (label 1352)
  (parent 601)
  (unrealized (1 3) (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 k1-0 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k1-0) (lk lk-0))
  (defstrand make 2 (old old-1) (k k1-0) (lk lk-0))
  (precedes ((1 1) (2 2)) ((2 4) (0 0)) ((3 1) (2 0)) ((4 4) (3 0))
    ((5 1) (7 0)) ((5 2) (4 0)) ((6 1) (4 3)) ((7 1) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 "init")
    (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule invShearsRule trRl_make-at-0 trRl_make-at-1)
  (operation channel-test (added-strand make 2)
    (ch-msg lk-0 (cat pt-5 k1-0 "init")) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-0 (cat pt-4 k "init"))
      (load lk-0 (cat pt-2 k1-0 "wrap")) (send (enc k k1-0)))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-5 k1-0 "init"))
      (stor lk-0 (cat pt-2 k1-0 "wrap")))
    ((load lk-0 (cat pt-6 old-1)) (stor lk-0 (cat pt-5 k1-0 "init"))))
  (label 1353)
  (parent 604)
  (unrealized (2 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 k1-0 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k1-0) (lk lk))
  (defstrand make 2 (old old-1) (k k1-0) (lk lk))
  (precedes ((1 4) (0 0)) ((2 1) (1 0)) ((3 1) (4 2)) ((3 1) (7 0))
    ((4 4) (2 0)) ((5 2) (4 0)) ((6 1) (4 3)) ((7 1) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 "init")
    (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule invShearsRule trRl_make-at-0 trRl_make-at-1)
  (operation channel-test (added-strand make 2)
    (ch-msg lk (cat pt-5 k1-0 "init")) (6 0))
  (traces ((recv k) (send k))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-4 k "init"))
      (load lk-0 (cat pt k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt-2 k1-0 "wrap")) (send (enc k k1-0)))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init"))
      (send (hash k)))
    ((load lk (cat pt-5 k1-0 "init")) (stor lk (cat pt-2 k1-0 "wrap")))
    ((load lk (cat pt-6 old-1)) (stor lk (cat pt-5 k1-0 "init"))))
  (label 1354)
  (parent 605)
  (unrealized (1 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 k1-0 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk-0))
  (deflistener k)
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (6 0)) ((2 4) (0 0)) ((3 1) (2 0)) ((4 1) (5 2))
    ((5 4) (3 0)) ((6 1) (5 0)) ((7 1) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation nonce-test (added-strand make 2) k (6 0)
    (ch-msg lk (cat pt-0 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-4 k1-0 "wrap")) (send (enc k k1-0)))
    ((recv k) (send k))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init"))))
  (label 1355)
  (parent 607)
  (unrealized (2 3) (5 3) (6 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk-0))
  (deflistener k)
  (precedes ((1 1) (6 0)) ((2 4) (0 0)) ((3 1) (2 0)) ((4 1) (6 0))
    ((5 4) (3 0)) ((6 1) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 "wrap"))
  (operation nonce-test (displaced 7 4 make 2) k (6 0)
    (ch-msg lk (cat pt-0 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-4 k1-0 "wrap")) (send (enc k k1-0)))
    ((recv k) (send k)))
  (label 1356)
  (parent 607)
  (unrealized (2 3) (5 3) (6 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 k1-0 k1-1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk-0))
  (deflistener k)
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-1) (lk lk))
  (precedes ((1 1) (7 2)) ((2 4) (0 0)) ((3 1) (2 0)) ((4 1) (5 2))
    ((5 4) (3 0)) ((6 1) (5 0)) ((7 4) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 "wrap")
    (cat k1-1 "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (6 0)
    (ch-msg lk (cat pt-0 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-4 k1-0 "wrap")) (send (enc k k1-0)))
    ((recv k) (send k))
    ((recv (hash k)) (recv (hash k1-1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-5 k1-1 "wrap")) (send (enc k k1-1))))
  (label 1357)
  (parent 607)
  (unrealized (2 3) (5 3) (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 k1-0 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk-0))
  (deflistener k)
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (2 2)) ((2 4) (0 0)) ((3 1) (2 0)) ((4 1) (6 0))
    ((5 4) (3 0)) ((6 1) (5 0)) ((7 1) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation nonce-test (added-strand make 2) k (6 0)
    (ch-msg lk-0 (cat pt-3 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-4 k1-0 "wrap")) (send (enc k k1-0)))
    ((recv k) (send k))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init"))))
  (label 1358)
  (parent 608)
  (unrealized (2 3) (5 3) (6 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 k1-0 k1-1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk-0))
  (deflistener k)
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-1) (lk lk-0))
  (precedes ((1 1) (2 2)) ((2 4) (0 0)) ((3 1) (2 0)) ((4 1) (7 2))
    ((5 4) (3 0)) ((6 1) (5 0)) ((7 4) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 "wrap")
    (cat k1-1 "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (6 0)
    (ch-msg lk-0 (cat pt-3 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-4 k1-0 "wrap")) (send (enc k k1-0)))
    ((recv k) (send k))
    ((recv (hash k)) (recv (hash k1-1)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-5 k1-1 "wrap")) (send (enc k k1-1))))
  (label 1359)
  (parent 608)
  (unrealized (2 3) (5 3) (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (5 3)) ((3 1) (1 0)) ((3 1) (5 2))
    ((4 1) (7 0)) ((5 4) (0 0)) ((6 1) (3 0)) ((6 2) (5 0))
    ((7 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-decrypt-at-1 trRl_make-at-0)
  (operation nonce-test (added-strand set-decrypt 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk (cat pt-2 k "decrypt")) (ch-msg lk (cat pt-6 k "init"))
    (ch-msg lk-0 (cat pt-4 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk (cat pt-6 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-2 k "decrypt"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-5 old-1)) (stor lk (cat pt-6 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-4 k "init"))
      (stor lk-0 (cat pt-7 k "decrypt"))))
  (label 1360)
  (parent 610)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (5 3)) ((3 1) (1 0)) ((3 1) (5 2))
    ((4 1) (7 0)) ((5 4) (0 0)) ((6 1) (3 0)) ((6 2) (5 0))
    ((7 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-wrap-at-1 trRl_make-at-0)
  (operation nonce-test (added-strand set-wrap 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk (cat pt-2 k "decrypt")) (ch-msg lk (cat pt-6 k "init"))
    (ch-msg lk-0 (cat pt-4 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk (cat pt-6 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-2 k "decrypt"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-5 old-1)) (stor lk (cat pt-6 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt-7 k "wrap"))))
  (label 1361)
  (parent 610)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "wrap") (k k) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (5 3)) ((2 1) (7 0)) ((3 1) (1 0))
    ((3 1) (5 2)) ((4 1) (0 0)) ((5 4) (0 0)) ((6 1) (3 0))
    ((6 2) (5 0)) ((7 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k "wrap"))
  (facts (neq "wrap" "decrypt") (neq "wrap" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule fact-set-decrypt-neq0 trRl_set-wrap-at-1 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation nonce-test (added-strand set-wrap 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk (cat pt-2 k "decrypt")) (ch-msg lk (cat pt-6 k "init"))
    (ch-msg lk-0 (cat pt-4 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk (cat pt-6 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-2 k "decrypt"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-5 old-1)) (stor lk (cat pt-6 k "init"))
      (send (hash k)))
    ((load lk (cat pt-1 k "wrap")) (stor lk (cat pt-7 k "wrap"))))
  (label 1362)
  (parent 610)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (5 3)) ((3 1) (1 0)) ((3 1) (5 2))
    ((4 1) (0 0)) ((5 4) (0 0)) ((6 1) (3 0)) ((6 1) (7 2))
    ((6 2) (5 0)) ((7 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k "wrap")
    (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk (cat pt-2 k "decrypt")) (ch-msg lk (cat pt-6 k "init"))
    (ch-msg lk-0 (cat pt-4 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk (cat pt-6 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-2 k "decrypt"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-5 old-1)) (stor lk (cat pt-6 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-6 k "init"))
      (load lk (cat pt-7 k1 "wrap")) (send (enc k k1))))
  (label 1363)
  (parent 610)
  (unrealized (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (5 3)) ((3 1) (1 0)) ((3 1) (5 2))
    ((4 1) (7 2)) ((5 4) (0 0)) ((6 1) (3 0)) ((6 2) (5 0))
    ((7 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k "wrap")
    (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk (cat pt-2 k "decrypt")) (ch-msg lk (cat pt-6 k "init"))
    (ch-msg lk-0 (cat pt-4 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk (cat pt-6 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-2 k "decrypt"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-5 old-1)) (stor lk (cat pt-6 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-4 k "init"))
      (load lk-0 (cat pt-7 k1 "wrap")) (send (enc k k1))))
  (label 1364)
  (parent 610)
  (unrealized (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "wrap") (k k) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (5 3)) ((2 1) (7 0)) ((3 1) (1 0))
    ((3 1) (5 2)) ((4 1) (0 0)) ((5 4) (0 0)) ((6 1) (3 0))
    ((6 2) (5 0)) ((7 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k "wrap"))
  (facts (neq "wrap" "decrypt") (neq "wrap" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-decrypt-at-1 fact-set-decrypt-neq0 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation nonce-test (added-strand set-decrypt 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk (cat pt-2 k "decrypt")) (ch-msg lk (cat pt-6 k "init"))
    (ch-msg lk-0 (cat pt-4 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk (cat pt-6 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-2 k "decrypt"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-5 old-1)) (stor lk (cat pt-6 k "init"))
      (send (hash k)))
    ((load lk (cat pt-1 k "wrap")) (stor lk (cat pt-7 k "decrypt"))))
  (label 1365)
  (parent 610)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (5 3)) ((2 1) (7 2)) ((3 1) (1 0))
    ((3 1) (5 2)) ((4 1) (0 0)) ((5 4) (0 0)) ((6 1) (3 0))
    ((6 2) (5 0)) ((7 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k "wrap")
    (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk (cat pt-2 k "decrypt")) (ch-msg lk (cat pt-6 k "init"))
    (ch-msg lk-0 (cat pt-4 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk (cat pt-6 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-2 k "decrypt"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-5 old-1)) (stor lk (cat pt-6 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-7 k1 "wrap")) (send (enc k k1))))
  (label 1366)
  (parent 610)
  (unrealized (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (5 3)) ((3 1) (1 0)) ((3 1) (5 2))
    ((3 1) (7 2)) ((4 1) (0 0)) ((5 4) (0 0)) ((6 1) (3 0))
    ((6 2) (5 0)) ((7 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k "wrap")
    (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk (cat pt-2 k "decrypt")) (ch-msg lk (cat pt-6 k "init"))
    (ch-msg lk-0 (cat pt-4 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk (cat pt-6 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-2 k "decrypt"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-5 old-1)) (stor lk (cat pt-6 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-2 k "decrypt"))
      (load lk (cat pt-7 k1 "wrap")) (send (enc k k1))))
  (label 1367)
  (parent 610)
  (unrealized (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (7 2)) ((2 1) (5 3)) ((3 1) (1 0))
    ((3 1) (5 2)) ((4 1) (0 0)) ((5 4) (0 0)) ((6 1) (3 0))
    ((6 2) (5 0)) ((7 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k "wrap")
    (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk (cat pt-2 k "decrypt")) (ch-msg lk (cat pt-6 k "init"))
    (ch-msg lk-0 (cat pt-4 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk (cat pt-6 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-2 k "decrypt"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-5 old-1)) (stor lk (cat pt-6 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-7 k1 "wrap")) (send (enc k k1))))
  (label 1368)
  (parent 610)
  (unrealized (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk))
  (defstrand decrypt 4 (x k) (k k) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (5 3)) ((3 1) (1 0)) ((3 1) (5 2))
    ((3 1) (7 2)) ((4 1) (0 0)) ((5 4) (0 0)) ((6 1) (3 0))
    ((6 2) (5 0)) ((7 3) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation nonce-test (added-strand decrypt 4) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk (cat pt-2 k "decrypt")) (ch-msg lk (cat pt-6 k "init"))
    (ch-msg lk-0 (cat pt-4 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk (cat pt-6 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-2 k "decrypt"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-5 old-1)) (stor lk (cat pt-6 k "init"))
      (send (hash k)))
    ((recv (enc k k)) (recv (hash k)) (load lk (cat pt-2 k "decrypt"))
      (send k)))
  (label 1369)
  (parent 610)
  (unrealized (7 0) (7 1))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (5 3)) ((3 1) (4 0)) ((4 1) (0 0))
    ((5 4) (0 0)) ((6 1) (7 0)) ((6 2) (5 0)) ((7 1) (1 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule invShearsRule shearsRule trRl_set-decrypt-at-1 trRl_make-at-0)
  (operation nonce-test (added-strand set-decrypt 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk (cat pt-6 k "init")) (ch-msg lk-0 (cat pt-3 k "init"))
    (ch-msg lk-0 (cat pt-4 k "decrypt")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-6 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-5 old-1)) (stor lk (cat pt-6 k "init"))
      (send (hash k)))
    ((load lk (cat pt-6 k "init")) (stor lk (cat pt-7 k "decrypt"))))
  (label 1370)
  (parent 619)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "decrypt") (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (5 3)) ((3 1) (4 0)) ((4 1) (7 0))
    ((5 4) (0 0)) ((6 1) (1 0)) ((6 2) (5 0)) ((7 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k "wrap"))
  (facts (neq "decrypt" "decrypt") (neq "decrypt" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-decrypt-at-1 fact-set-decrypt-neq0 gen-st-set-wrap-0
    fact-set-wrap-neq0 trRl_make-at-0)
  (operation nonce-test (added-strand set-decrypt 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk (cat pt-6 k "init")) (ch-msg lk-0 (cat pt-3 k "init"))
    (ch-msg lk-0 (cat pt-4 k "decrypt")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-6 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-5 old-1)) (stor lk (cat pt-6 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-4 k "decrypt"))
      (stor lk-0 (cat pt-7 k "decrypt"))))
  (label 1371)
  (parent 619)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "wrap") (k k) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (5 3)) ((2 1) (7 0)) ((3 1) (4 0))
    ((4 1) (0 0)) ((5 4) (0 0)) ((6 1) (1 0)) ((6 2) (5 0))
    ((7 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "wrap" "decrypt") (neq "wrap" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule fact-set-decrypt-neq0 trRl_set-wrap-at-1 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation nonce-test (added-strand set-wrap 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk (cat pt-6 k "init")) (ch-msg lk-0 (cat pt-3 k "init"))
    (ch-msg lk-0 (cat pt-4 k "decrypt")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-6 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-5 old-1)) (stor lk (cat pt-6 k "init"))
      (send (hash k)))
    ((load lk (cat pt-1 k "wrap")) (stor lk (cat pt-7 k "wrap"))))
  (label 1372)
  (parent 619)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "decrypt") (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (5 3)) ((3 1) (4 0)) ((4 1) (7 0))
    ((5 4) (0 0)) ((6 1) (1 0)) ((6 2) (5 0)) ((7 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k "wrap"))
  (facts (neq "decrypt" "decrypt") (neq "decrypt" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule fact-set-decrypt-neq0 trRl_set-wrap-at-1 gen-st-set-wrap-0
    fact-set-wrap-neq0 trRl_make-at-0)
  (operation nonce-test (added-strand set-wrap 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk (cat pt-6 k "init")) (ch-msg lk-0 (cat pt-3 k "init"))
    (ch-msg lk-0 (cat pt-4 k "decrypt")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-6 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-5 old-1)) (stor lk (cat pt-6 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-4 k "decrypt"))
      (stor lk-0 (cat pt-7 k "wrap"))))
  (label 1373)
  (parent 619)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 old-2 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval) (k skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk))
  (defstrand make 2 (old old-2) (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((2 1) (5 3)) ((3 1) (4 0)) ((4 1) (0 0))
    ((5 4) (0 0)) ((6 1) (1 0)) ((6 2) (5 0)) ((7 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation nonce-test (added-strand make 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk (cat pt-6 k "init")) (ch-msg lk-0 (cat pt-3 k "init"))
    (ch-msg lk-0 (cat pt-4 k "decrypt")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-6 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-5 old-1)) (stor lk (cat pt-6 k "init"))
      (send (hash k)))
    ((load lk-1 (cat pt-7 old-2)) (stor lk-1 (cat pt-8 k "init"))))
  (label 1374)
  (parent 619)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (5 3)) ((3 1) (4 0)) ((4 1) (0 0))
    ((5 4) (0 0)) ((6 1) (7 0)) ((6 2) (5 0)) ((7 1) (1 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule invShearsRule shearsRule trRl_set-wrap-at-1 trRl_make-at-0)
  (operation nonce-test (added-strand set-wrap 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk (cat pt-6 k "init")) (ch-msg lk-0 (cat pt-3 k "init"))
    (ch-msg lk-0 (cat pt-4 k "decrypt")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-6 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-5 old-1)) (stor lk (cat pt-6 k "init"))
      (send (hash k)))
    ((load lk (cat pt-6 k "init")) (stor lk (cat pt-7 k "wrap"))))
  (label 1375)
  (parent 619)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (5 3)) ((3 1) (4 0)) ((3 1) (7 2))
    ((4 1) (0 0)) ((5 4) (0 0)) ((6 1) (1 0)) ((6 2) (5 0))
    ((7 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk (cat pt-6 k "init")) (ch-msg lk-0 (cat pt-3 k "init"))
    (ch-msg lk-0 (cat pt-4 k "decrypt")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-6 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-5 old-1)) (stor lk (cat pt-6 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-7 k1 "wrap")) (send (enc k k1))))
  (label 1376)
  (parent 619)
  (unrealized (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (5 3)) ((3 1) (4 0)) ((4 1) (7 2))
    ((5 4) (0 0)) ((6 1) (1 0)) ((6 2) (5 0)) ((7 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k "wrap")
    (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0 gen-st-wrap-1)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk (cat pt-6 k "init")) (ch-msg lk-0 (cat pt-3 k "init"))
    (ch-msg lk-0 (cat pt-4 k "decrypt")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-6 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-5 old-1)) (stor lk (cat pt-6 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-4 k "decrypt"))
      (load lk-0 (cat pt-7 k1 "wrap")) (send (enc k k1))))
  (label 1377)
  (parent 619)
  (unrealized (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "wrap") (k k) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (5 3)) ((2 1) (7 0)) ((3 1) (4 0))
    ((4 1) (0 0)) ((5 4) (0 0)) ((6 1) (1 0)) ((6 2) (5 0))
    ((7 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "wrap" "decrypt") (neq "wrap" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-decrypt-at-1 fact-set-decrypt-neq0 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation nonce-test (added-strand set-decrypt 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk (cat pt-6 k "init")) (ch-msg lk-0 (cat pt-3 k "init"))
    (ch-msg lk-0 (cat pt-4 k "decrypt")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-6 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-5 old-1)) (stor lk (cat pt-6 k "init"))
      (send (hash k)))
    ((load lk (cat pt-1 k "wrap")) (stor lk (cat pt-7 k "decrypt"))))
  (label 1378)
  (parent 619)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (5 3)) ((2 1) (7 2)) ((3 1) (4 0))
    ((4 1) (0 0)) ((5 4) (0 0)) ((6 1) (1 0)) ((6 2) (5 0))
    ((7 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk (cat pt-6 k "init")) (ch-msg lk-0 (cat pt-3 k "init"))
    (ch-msg lk-0 (cat pt-4 k "decrypt")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-6 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-5 old-1)) (stor lk (cat pt-6 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-7 k1 "wrap")) (send (enc k k1))))
  (label 1379)
  (parent 619)
  (unrealized (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (5 3)) ((3 1) (4 0)) ((4 1) (0 0))
    ((5 4) (0 0)) ((6 1) (1 0)) ((6 1) (7 2)) ((6 2) (5 0))
    ((7 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk (cat pt-6 k "init")) (ch-msg lk-0 (cat pt-3 k "init"))
    (ch-msg lk-0 (cat pt-4 k "decrypt")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-6 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-5 old-1)) (stor lk (cat pt-6 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-6 k "init"))
      (load lk (cat pt-7 k1 "wrap")) (send (enc k k1))))
  (label 1380)
  (parent 619)
  (unrealized (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (7 2)) ((2 1) (5 3)) ((3 1) (4 0))
    ((4 1) (0 0)) ((5 4) (0 0)) ((6 1) (1 0)) ((6 2) (5 0))
    ((7 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk (cat pt-6 k "init")) (ch-msg lk-0 (cat pt-3 k "init"))
    (ch-msg lk-0 (cat pt-4 k "decrypt")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-6 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-5 old-1)) (stor lk (cat pt-6 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-7 k1 "wrap")) (send (enc k k1))))
  (label 1381)
  (parent 619)
  (unrealized (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk))
  (defstrand decrypt 4 (x k) (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (5 3)) ((3 1) (4 0)) ((4 1) (7 2))
    ((5 4) (0 0)) ((6 1) (1 0)) ((6 2) (5 0)) ((7 3) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-decrypt-0)
  (operation nonce-test (added-strand decrypt 4) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk (cat pt-6 k "init")) (ch-msg lk-0 (cat pt-3 k "init"))
    (ch-msg lk-0 (cat pt-4 k "decrypt")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-6 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-5 old-1)) (stor lk (cat pt-6 k "init"))
      (send (hash k)))
    ((recv (enc k k)) (recv (hash k)) (load lk-0 (cat pt-4 k "decrypt"))
      (send k)))
  (label 1382)
  (parent 619)
  (unrealized (7 0) (7 1))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (5 3)) ((3 1) (1 0)) ((4 1) (7 0))
    ((5 4) (0 0)) ((6 1) (3 0)) ((6 2) (5 0)) ((7 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-wrap-at-1 trRl_make-at-0)
  (operation nonce-test (added-strand set-wrap 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk (cat pt-2 k "decrypt")) (ch-msg lk (cat pt-6 k "init"))
    (ch-msg lk-0 (cat pt-4 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk (cat pt-6 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-6 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-5 old-1)) (stor lk (cat pt-6 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt-7 k "wrap"))))
  (label 1383)
  (parent 621)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "wrap") (k k) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (5 3)) ((2 1) (7 0)) ((3 1) (1 0))
    ((4 1) (0 0)) ((5 4) (0 0)) ((6 1) (3 0)) ((6 2) (5 0))
    ((7 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "wrap" "decrypt") (neq "wrap" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule fact-set-decrypt-neq0 trRl_set-wrap-at-1 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation nonce-test (added-strand set-wrap 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk (cat pt-2 k "decrypt")) (ch-msg lk (cat pt-6 k "init"))
    (ch-msg lk-0 (cat pt-4 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk (cat pt-6 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-6 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-5 old-1)) (stor lk (cat pt-6 k "init"))
      (send (hash k)))
    ((load lk (cat pt-1 k "wrap")) (stor lk (cat pt-7 k "wrap"))))
  (label 1384)
  (parent 621)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (5 3)) ((3 1) (1 0)) ((4 1) (0 0))
    ((5 4) (0 0)) ((6 1) (3 0)) ((6 1) (7 2)) ((6 2) (5 0))
    ((7 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk (cat pt-2 k "decrypt")) (ch-msg lk (cat pt-6 k "init"))
    (ch-msg lk-0 (cat pt-4 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk (cat pt-6 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-6 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-5 old-1)) (stor lk (cat pt-6 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-6 k "init"))
      (load lk (cat pt-7 k1 "wrap")) (send (enc k k1))))
  (label 1385)
  (parent 621)
  (unrealized (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (5 3)) ((3 1) (1 0)) ((4 1) (7 2))
    ((5 4) (0 0)) ((6 1) (3 0)) ((6 2) (5 0)) ((7 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk (cat pt-2 k "decrypt")) (ch-msg lk (cat pt-6 k "init"))
    (ch-msg lk-0 (cat pt-4 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk (cat pt-6 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-6 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-5 old-1)) (stor lk (cat pt-6 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-4 k "init"))
      (load lk-0 (cat pt-7 k1 "wrap")) (send (enc k k1))))
  (label 1386)
  (parent 621)
  (unrealized (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "wrap") (k k) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (5 3)) ((2 1) (7 0)) ((3 1) (1 0))
    ((4 1) (0 0)) ((5 4) (0 0)) ((6 1) (3 0)) ((6 2) (5 0))
    ((7 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "wrap" "decrypt") (neq "wrap" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-decrypt-at-1 fact-set-decrypt-neq0 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation nonce-test (added-strand set-decrypt 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk (cat pt-2 k "decrypt")) (ch-msg lk (cat pt-6 k "init"))
    (ch-msg lk-0 (cat pt-4 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk (cat pt-6 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-6 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-5 old-1)) (stor lk (cat pt-6 k "init"))
      (send (hash k)))
    ((load lk (cat pt-1 k "wrap")) (stor lk (cat pt-7 k "decrypt"))))
  (label 1387)
  (parent 621)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (5 3)) ((2 1) (7 2)) ((3 1) (1 0))
    ((4 1) (0 0)) ((5 4) (0 0)) ((6 1) (3 0)) ((6 2) (5 0))
    ((7 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk (cat pt-2 k "decrypt")) (ch-msg lk (cat pt-6 k "init"))
    (ch-msg lk-0 (cat pt-4 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk (cat pt-6 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-6 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-5 old-1)) (stor lk (cat pt-6 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-7 k1 "wrap")) (send (enc k k1))))
  (label 1388)
  (parent 621)
  (unrealized (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (5 3)) ((3 1) (1 0)) ((3 1) (7 2))
    ((4 1) (0 0)) ((5 4) (0 0)) ((6 1) (3 0)) ((6 2) (5 0))
    ((7 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k "wrap")
    (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0 gen-st-wrap-1)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk (cat pt-2 k "decrypt")) (ch-msg lk (cat pt-6 k "init"))
    (ch-msg lk-0 (cat pt-4 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk (cat pt-6 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-6 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-5 old-1)) (stor lk (cat pt-6 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-2 k "decrypt"))
      (load lk (cat pt-7 k1 "wrap")) (send (enc k k1))))
  (label 1389)
  (parent 621)
  (unrealized (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (7 2)) ((2 1) (5 3)) ((3 1) (1 0))
    ((4 1) (0 0)) ((5 4) (0 0)) ((6 1) (3 0)) ((6 2) (5 0))
    ((7 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk (cat pt-2 k "decrypt")) (ch-msg lk (cat pt-6 k "init"))
    (ch-msg lk-0 (cat pt-4 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk (cat pt-6 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-6 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-5 old-1)) (stor lk (cat pt-6 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-7 k1 "wrap")) (send (enc k k1))))
  (label 1390)
  (parent 621)
  (unrealized (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk))
  (defstrand decrypt 4 (x k) (k k) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (5 3)) ((3 1) (1 0)) ((3 1) (7 2))
    ((4 1) (0 0)) ((5 4) (0 0)) ((6 1) (3 0)) ((6 2) (5 0))
    ((7 3) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-decrypt-0)
  (operation nonce-test (added-strand decrypt 4) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk (cat pt-2 k "decrypt")) (ch-msg lk (cat pt-6 k "init"))
    (ch-msg lk-0 (cat pt-4 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk (cat pt-6 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-6 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-5 old-1)) (stor lk (cat pt-6 k "init"))
      (send (hash k)))
    ((recv (enc k k)) (recv (hash k)) (load lk (cat pt-2 k "decrypt"))
      (send k)))
  (label 1391)
  (parent 621)
  (unrealized (7 0) (7 1))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-1))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk-1))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand set-decrypt 2 (cur "wrap") (k k) (lk lk-1))
  (precedes ((1 1) (5 2)) ((1 1) (7 0)) ((2 1) (3 0)) ((3 1) (0 0))
    ((4 1) (0 0)) ((5 4) (0 0)) ((6 1) (1 0)) ((6 2) (5 0))
    ((7 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "wrap" "decrypt") (neq "wrap" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-decrypt-at-1 fact-set-decrypt-neq0 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation nonce-test (added-strand set-decrypt 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-1 k "init")) (ch-msg lk (cat pt-2 k "decrypt"))
    (ch-msg lk-0 (cat pt-4 k "init")) (ch-msg lk-1 (cat pt k "wrap"))
    (ch-msg lk-1 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk-1 (cat pt-6 k "init")) (stor lk-1 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk-1 (cat pt k "wrap"))
      (load lk-1 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init"))
      (send (hash k)))
    ((load lk-1 (cat pt k "wrap")) (stor lk-1 (cat pt-7 k "decrypt"))))
  (label 1392)
  (parent 626)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-1))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk-1))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand set-wrap 2 (cur "decrypt") (k k) (lk lk))
  (precedes ((1 1) (5 2)) ((2 1) (3 0)) ((3 1) (7 0)) ((4 1) (0 0))
    ((5 4) (0 0)) ((6 1) (1 0)) ((6 2) (5 0)) ((7 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k "wrap"))
  (facts (neq "decrypt" "decrypt") (neq "decrypt" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule fact-set-decrypt-neq0 trRl_set-wrap-at-1 gen-st-set-wrap-0
    fact-set-wrap-neq0 trRl_make-at-0)
  (operation nonce-test (added-strand set-wrap 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-1 k "init")) (ch-msg lk (cat pt-2 k "decrypt"))
    (ch-msg lk-0 (cat pt-4 k "init")) (ch-msg lk-1 (cat pt k "wrap"))
    (ch-msg lk-1 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk-1 (cat pt-6 k "init")) (stor lk-1 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk-1 (cat pt k "wrap"))
      (load lk-1 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init"))
      (send (hash k)))
    ((load lk (cat pt-2 k "decrypt")) (stor lk (cat pt-7 k "wrap"))))
  (label 1393)
  (parent 626)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-1))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk-1))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand set-wrap 2 (cur "wrap") (k k) (lk lk-1))
  (precedes ((1 1) (5 2)) ((1 1) (7 0)) ((2 1) (3 0)) ((3 1) (0 0))
    ((4 1) (0 0)) ((5 4) (0 0)) ((6 1) (1 0)) ((6 2) (5 0))
    ((7 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "wrap" "decrypt") (neq "wrap" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule fact-set-decrypt-neq0 trRl_set-wrap-at-1 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation nonce-test (added-strand set-wrap 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-1 k "init")) (ch-msg lk (cat pt-2 k "decrypt"))
    (ch-msg lk-0 (cat pt-4 k "init")) (ch-msg lk-1 (cat pt k "wrap"))
    (ch-msg lk-1 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk-1 (cat pt-6 k "init")) (stor lk-1 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk-1 (cat pt k "wrap"))
      (load lk-1 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init"))
      (send (hash k)))
    ((load lk-1 (cat pt k "wrap")) (stor lk-1 (cat pt-7 k "wrap"))))
  (label 1394)
  (parent 626)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-1))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk-1))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (precedes ((1 1) (5 2)) ((2 1) (3 0)) ((3 1) (0 0)) ((4 1) (7 0))
    ((5 4) (0 0)) ((6 1) (1 0)) ((6 2) (5 0)) ((7 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-wrap-at-1 trRl_make-at-0)
  (operation nonce-test (added-strand set-wrap 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-1 k "init")) (ch-msg lk (cat pt-2 k "decrypt"))
    (ch-msg lk-0 (cat pt-4 k "init")) (ch-msg lk-1 (cat pt k "wrap"))
    (ch-msg lk-1 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk-1 (cat pt-6 k "init")) (stor lk-1 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk-1 (cat pt k "wrap"))
      (load lk-1 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt-7 k "wrap"))))
  (label 1395)
  (parent 626)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-1))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk-1))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk-1))
  (precedes ((1 1) (5 2)) ((1 1) (7 2)) ((2 1) (3 0)) ((3 1) (0 0))
    ((4 1) (0 0)) ((5 4) (0 0)) ((6 1) (1 0)) ((6 2) (5 0))
    ((7 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-1 k "init")) (ch-msg lk (cat pt-2 k "decrypt"))
    (ch-msg lk-0 (cat pt-4 k "init")) (ch-msg lk-1 (cat pt k "wrap"))
    (ch-msg lk-1 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk-1 (cat pt-6 k "init")) (stor lk-1 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk-1 (cat pt k "wrap"))
      (load lk-1 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk-1 (cat pt k "wrap"))
      (load lk-1 (cat pt-7 k1 "wrap")) (send (enc k k1))))
  (label 1396)
  (parent 626)
  (unrealized (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-1))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk-1))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-1))
  (precedes ((1 1) (5 2)) ((2 1) (3 0)) ((3 1) (0 0)) ((4 1) (0 0))
    ((5 4) (0 0)) ((6 1) (1 0)) ((6 1) (7 2)) ((6 2) (5 0))
    ((7 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-1 k "init")) (ch-msg lk (cat pt-2 k "decrypt"))
    (ch-msg lk-0 (cat pt-4 k "init")) (ch-msg lk-1 (cat pt k "wrap"))
    (ch-msg lk-1 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk-1 (cat pt-6 k "init")) (stor lk-1 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk-1 (cat pt k "wrap"))
      (load lk-1 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk-1 (cat pt-6 k "init"))
      (load lk-1 (cat pt-7 k1 "wrap")) (send (enc k k1))))
  (label 1397)
  (parent 626)
  (unrealized (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-1))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk-1))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand set-decrypt 2 (cur "decrypt") (k k) (lk lk))
  (precedes ((1 1) (5 2)) ((2 1) (3 0)) ((3 1) (7 0)) ((4 1) (0 0))
    ((5 4) (0 0)) ((6 1) (1 0)) ((6 2) (5 0)) ((7 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k "wrap"))
  (facts (neq "decrypt" "decrypt") (neq "decrypt" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-decrypt-at-1 fact-set-decrypt-neq0 gen-st-set-wrap-0
    fact-set-wrap-neq0 trRl_make-at-0)
  (operation nonce-test (added-strand set-decrypt 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-1 k "init")) (ch-msg lk (cat pt-2 k "decrypt"))
    (ch-msg lk-0 (cat pt-4 k "init")) (ch-msg lk-1 (cat pt k "wrap"))
    (ch-msg lk-1 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk-1 (cat pt-6 k "init")) (stor lk-1 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk-1 (cat pt k "wrap"))
      (load lk-1 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init"))
      (send (hash k)))
    ((load lk (cat pt-2 k "decrypt")) (stor lk (cat pt-7 k "decrypt"))))
  (label 1398)
  (parent 626)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-1))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk-1))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (5 2)) ((2 1) (3 0)) ((3 1) (7 2)) ((4 1) (0 0))
    ((5 4) (0 0)) ((6 1) (1 0)) ((6 2) (5 0)) ((7 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k "wrap")
    (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0 gen-st-wrap-1)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-1 k "init")) (ch-msg lk (cat pt-2 k "decrypt"))
    (ch-msg lk-0 (cat pt-4 k "init")) (ch-msg lk-1 (cat pt k "wrap"))
    (ch-msg lk-1 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk-1 (cat pt-6 k "init")) (stor lk-1 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk-1 (cat pt k "wrap"))
      (load lk-1 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-2 k "decrypt"))
      (load lk (cat pt-7 k1 "wrap")) (send (enc k k1))))
  (label 1399)
  (parent 626)
  (unrealized (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-1))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk-1))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (precedes ((1 1) (5 2)) ((2 1) (3 0)) ((3 1) (0 0)) ((4 1) (7 2))
    ((5 4) (0 0)) ((6 1) (1 0)) ((6 2) (5 0)) ((7 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-1 k "init")) (ch-msg lk (cat pt-2 k "decrypt"))
    (ch-msg lk-0 (cat pt-4 k "init")) (ch-msg lk-1 (cat pt k "wrap"))
    (ch-msg lk-1 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk-1 (cat pt-6 k "init")) (stor lk-1 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk-1 (cat pt k "wrap"))
      (load lk-1 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-4 k "init"))
      (load lk-0 (cat pt-7 k1 "wrap")) (send (enc k k1))))
  (label 1400)
  (parent 626)
  (unrealized (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-1))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk-1))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (5 2)) ((2 1) (3 0)) ((2 1) (7 2)) ((3 1) (0 0))
    ((4 1) (0 0)) ((5 4) (0 0)) ((6 1) (1 0)) ((6 2) (5 0))
    ((7 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-1 k "init")) (ch-msg lk (cat pt-2 k "decrypt"))
    (ch-msg lk-0 (cat pt-4 k "init")) (ch-msg lk-1 (cat pt k "wrap"))
    (ch-msg lk-1 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk-1 (cat pt-6 k "init")) (stor lk-1 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk-1 (cat pt k "wrap"))
      (load lk-1 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt-7 k1 "wrap")) (send (enc k k1))))
  (label 1401)
  (parent 626)
  (unrealized (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-1))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk-1))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand decrypt 4 (x k) (k k) (lk lk))
  (precedes ((1 1) (5 2)) ((2 1) (3 0)) ((3 1) (7 2)) ((4 1) (0 0))
    ((5 4) (0 0)) ((6 1) (1 0)) ((6 2) (5 0)) ((7 3) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-decrypt-0)
  (operation nonce-test (added-strand decrypt 4) k (0 0) (enc k k)
    (ch-msg lk (cat pt-1 k "init")) (ch-msg lk (cat pt-2 k "decrypt"))
    (ch-msg lk-0 (cat pt-4 k "init")) (ch-msg lk-1 (cat pt k "wrap"))
    (ch-msg lk-1 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk-1 (cat pt-6 k "init")) (stor lk-1 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk-1 (cat pt k "wrap"))
      (load lk-1 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init"))
      (send (hash k)))
    ((recv (enc k k)) (recv (hash k)) (load lk (cat pt-2 k "decrypt"))
      (send k)))
  (label 1402)
  (parent 626)
  (unrealized (7 0) (7 1))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-1))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk-1))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand set-decrypt 2 (cur "wrap") (k k) (lk lk-1))
  (precedes ((1 1) (5 3)) ((1 1) (7 0)) ((2 1) (3 0)) ((3 1) (0 0))
    ((4 1) (0 0)) ((5 4) (0 0)) ((6 1) (1 0)) ((6 2) (5 0))
    ((7 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "wrap" "decrypt") (neq "wrap" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-decrypt-at-1 fact-set-decrypt-neq0 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation nonce-test (added-strand set-decrypt 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-1 k "init")) (ch-msg lk (cat pt-2 k "decrypt"))
    (ch-msg lk-0 (cat pt-4 k "init")) (ch-msg lk-1 (cat pt k "wrap"))
    (ch-msg lk-1 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk-1 (cat pt-6 k "init")) (stor lk-1 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk-1 (cat pt-6 k "init"))
      (load lk-1 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init"))
      (send (hash k)))
    ((load lk-1 (cat pt k "wrap")) (stor lk-1 (cat pt-7 k "decrypt"))))
  (label 1403)
  (parent 629)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-1))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk-1))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand set-wrap 2 (cur "decrypt") (k k) (lk lk))
  (precedes ((1 1) (5 3)) ((2 1) (3 0)) ((3 1) (7 0)) ((4 1) (0 0))
    ((5 4) (0 0)) ((6 1) (1 0)) ((6 2) (5 0)) ((7 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k "wrap"))
  (facts (neq "decrypt" "decrypt") (neq "decrypt" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule fact-set-decrypt-neq0 trRl_set-wrap-at-1 gen-st-set-wrap-0
    fact-set-wrap-neq0 trRl_make-at-0)
  (operation nonce-test (added-strand set-wrap 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-1 k "init")) (ch-msg lk (cat pt-2 k "decrypt"))
    (ch-msg lk-0 (cat pt-4 k "init")) (ch-msg lk-1 (cat pt k "wrap"))
    (ch-msg lk-1 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk-1 (cat pt-6 k "init")) (stor lk-1 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk-1 (cat pt-6 k "init"))
      (load lk-1 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init"))
      (send (hash k)))
    ((load lk (cat pt-2 k "decrypt")) (stor lk (cat pt-7 k "wrap"))))
  (label 1404)
  (parent 629)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-1))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk-1))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand set-wrap 2 (cur "wrap") (k k) (lk lk-1))
  (precedes ((1 1) (5 3)) ((1 1) (7 0)) ((2 1) (3 0)) ((3 1) (0 0))
    ((4 1) (0 0)) ((5 4) (0 0)) ((6 1) (1 0)) ((6 2) (5 0))
    ((7 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "wrap" "decrypt") (neq "wrap" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule fact-set-decrypt-neq0 trRl_set-wrap-at-1 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation nonce-test (added-strand set-wrap 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-1 k "init")) (ch-msg lk (cat pt-2 k "decrypt"))
    (ch-msg lk-0 (cat pt-4 k "init")) (ch-msg lk-1 (cat pt k "wrap"))
    (ch-msg lk-1 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk-1 (cat pt-6 k "init")) (stor lk-1 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk-1 (cat pt-6 k "init"))
      (load lk-1 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init"))
      (send (hash k)))
    ((load lk-1 (cat pt k "wrap")) (stor lk-1 (cat pt-7 k "wrap"))))
  (label 1405)
  (parent 629)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-1))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk-1))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (precedes ((1 1) (5 3)) ((2 1) (3 0)) ((3 1) (0 0)) ((4 1) (7 0))
    ((5 4) (0 0)) ((6 1) (1 0)) ((6 2) (5 0)) ((7 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-wrap-at-1 trRl_make-at-0)
  (operation nonce-test (added-strand set-wrap 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-1 k "init")) (ch-msg lk (cat pt-2 k "decrypt"))
    (ch-msg lk-0 (cat pt-4 k "init")) (ch-msg lk-1 (cat pt k "wrap"))
    (ch-msg lk-1 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk-1 (cat pt-6 k "init")) (stor lk-1 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk-1 (cat pt-6 k "init"))
      (load lk-1 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt-7 k "wrap"))))
  (label 1406)
  (parent 629)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-1))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk-1))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk-1))
  (precedes ((1 1) (5 3)) ((1 1) (7 2)) ((2 1) (3 0)) ((3 1) (0 0))
    ((4 1) (0 0)) ((5 4) (0 0)) ((6 1) (1 0)) ((6 2) (5 0))
    ((7 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-1 k "init")) (ch-msg lk (cat pt-2 k "decrypt"))
    (ch-msg lk-0 (cat pt-4 k "init")) (ch-msg lk-1 (cat pt k "wrap"))
    (ch-msg lk-1 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk-1 (cat pt-6 k "init")) (stor lk-1 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk-1 (cat pt-6 k "init"))
      (load lk-1 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk-1 (cat pt k "wrap"))
      (load lk-1 (cat pt-7 k1 "wrap")) (send (enc k k1))))
  (label 1407)
  (parent 629)
  (unrealized (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-1))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk-1))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-1))
  (precedes ((1 1) (5 3)) ((2 1) (3 0)) ((3 1) (0 0)) ((4 1) (0 0))
    ((5 4) (0 0)) ((6 1) (1 0)) ((6 1) (7 2)) ((6 2) (5 0))
    ((7 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-1 k "init")) (ch-msg lk (cat pt-2 k "decrypt"))
    (ch-msg lk-0 (cat pt-4 k "init")) (ch-msg lk-1 (cat pt k "wrap"))
    (ch-msg lk-1 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk-1 (cat pt-6 k "init")) (stor lk-1 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk-1 (cat pt-6 k "init"))
      (load lk-1 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk-1 (cat pt-6 k "init"))
      (load lk-1 (cat pt-7 k1 "wrap")) (send (enc k k1))))
  (label 1408)
  (parent 629)
  (unrealized (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-1))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk-1))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand set-decrypt 2 (cur "decrypt") (k k) (lk lk))
  (precedes ((1 1) (5 3)) ((2 1) (3 0)) ((3 1) (7 0)) ((4 1) (0 0))
    ((5 4) (0 0)) ((6 1) (1 0)) ((6 2) (5 0)) ((7 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k "wrap"))
  (facts (neq "decrypt" "decrypt") (neq "decrypt" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-decrypt-at-1 fact-set-decrypt-neq0 gen-st-set-wrap-0
    fact-set-wrap-neq0 trRl_make-at-0)
  (operation nonce-test (added-strand set-decrypt 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-1 k "init")) (ch-msg lk (cat pt-2 k "decrypt"))
    (ch-msg lk-0 (cat pt-4 k "init")) (ch-msg lk-1 (cat pt k "wrap"))
    (ch-msg lk-1 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk-1 (cat pt-6 k "init")) (stor lk-1 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk-1 (cat pt-6 k "init"))
      (load lk-1 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init"))
      (send (hash k)))
    ((load lk (cat pt-2 k "decrypt")) (stor lk (cat pt-7 k "decrypt"))))
  (label 1409)
  (parent 629)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-1))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk-1))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (5 3)) ((2 1) (3 0)) ((3 1) (7 2)) ((4 1) (0 0))
    ((5 4) (0 0)) ((6 1) (1 0)) ((6 2) (5 0)) ((7 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k "wrap")
    (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0 gen-st-wrap-1)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-1 k "init")) (ch-msg lk (cat pt-2 k "decrypt"))
    (ch-msg lk-0 (cat pt-4 k "init")) (ch-msg lk-1 (cat pt k "wrap"))
    (ch-msg lk-1 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk-1 (cat pt-6 k "init")) (stor lk-1 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk-1 (cat pt-6 k "init"))
      (load lk-1 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-2 k "decrypt"))
      (load lk (cat pt-7 k1 "wrap")) (send (enc k k1))))
  (label 1410)
  (parent 629)
  (unrealized (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-1))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk-1))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (precedes ((1 1) (5 3)) ((2 1) (3 0)) ((3 1) (0 0)) ((4 1) (7 2))
    ((5 4) (0 0)) ((6 1) (1 0)) ((6 2) (5 0)) ((7 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-1 k "init")) (ch-msg lk (cat pt-2 k "decrypt"))
    (ch-msg lk-0 (cat pt-4 k "init")) (ch-msg lk-1 (cat pt k "wrap"))
    (ch-msg lk-1 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk-1 (cat pt-6 k "init")) (stor lk-1 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk-1 (cat pt-6 k "init"))
      (load lk-1 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-4 k "init"))
      (load lk-0 (cat pt-7 k1 "wrap")) (send (enc k k1))))
  (label 1411)
  (parent 629)
  (unrealized (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-1))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk-1))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (5 3)) ((2 1) (3 0)) ((2 1) (7 2)) ((3 1) (0 0))
    ((4 1) (0 0)) ((5 4) (0 0)) ((6 1) (1 0)) ((6 2) (5 0))
    ((7 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-1 k "init")) (ch-msg lk (cat pt-2 k "decrypt"))
    (ch-msg lk-0 (cat pt-4 k "init")) (ch-msg lk-1 (cat pt k "wrap"))
    (ch-msg lk-1 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk-1 (cat pt-6 k "init")) (stor lk-1 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk-1 (cat pt-6 k "init"))
      (load lk-1 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt-7 k1 "wrap")) (send (enc k k1))))
  (label 1412)
  (parent 629)
  (unrealized (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-1))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk-1))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand decrypt 4 (x k) (k k) (lk lk))
  (precedes ((1 1) (5 3)) ((2 1) (3 0)) ((3 1) (7 2)) ((4 1) (0 0))
    ((5 4) (0 0)) ((6 1) (1 0)) ((6 2) (5 0)) ((7 3) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-decrypt-0)
  (operation nonce-test (added-strand decrypt 4) k (0 0) (enc k k)
    (ch-msg lk (cat pt-1 k "init")) (ch-msg lk (cat pt-2 k "decrypt"))
    (ch-msg lk-0 (cat pt-4 k "init")) (ch-msg lk-1 (cat pt k "wrap"))
    (ch-msg lk-1 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk-1 (cat pt-6 k "init")) (stor lk-1 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk-1 (cat pt-6 k "init"))
      (load lk-1 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init"))
      (send (hash k)))
    ((recv (enc k k)) (recv (hash k)) (load lk (cat pt-2 k "decrypt"))
      (send k)))
  (label 1413)
  (parent 629)
  (unrealized (7 0) (7 1))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (4 3)) ((3 1) (1 0)) ((3 1) (4 2))
    ((4 4) (0 0)) ((5 1) (3 0)) ((5 2) (4 0)) ((5 2) (6 0))
    ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k "wrap")
    (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 7 5 make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk (cat pt-4 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-2 k "decrypt"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-3 old-0)) (stor lk (cat pt-4 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-4 k "init"))
      (load lk (cat pt-5 k1 "wrap")) (send (enc k k1))))
  (label 1414)
  (parent 632)
  (unrealized (6 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (4 3)) ((3 1) (1 0)) ((3 1) (4 2))
    ((4 4) (0 0)) ((5 1) (3 0)) ((5 1) (6 2)) ((5 2) (4 0))
    ((6 4) (0 0)) ((7 2) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k "wrap")
    (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation encryption-test (added-strand make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk (cat pt-4 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-2 k "decrypt"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-3 old-0)) (stor lk (cat pt-4 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-4 k "init"))
      (load lk (cat pt-5 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-6 old-1)) (stor lk-0 (cat pt-7 k "init"))
      (send (hash k))))
  (label 1415)
  (parent 632)
  (unrealized (6 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (precedes ((1 1) (2 0)) ((2 1) (4 3)) ((3 1) (1 0)) ((3 1) (4 2))
    ((4 4) (0 0)) ((5 1) (3 0)) ((5 1) (6 2)) ((5 2) (4 0))
    ((6 4) (0 0)) ((7 1) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k "wrap")
    (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (added-listener k) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk (cat pt-4 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-2 k "decrypt"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-3 old-0)) (stor lk (cat pt-4 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-4 k "init"))
      (load lk (cat pt-5 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k)))
  (label 1416)
  (parent 632)
  (unrealized (6 3) (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (4 3)) ((2 1) (6 2)) ((3 1) (1 0))
    ((3 1) (4 2)) ((4 4) (0 0)) ((5 1) (3 0)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k "wrap")
    (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 7 5 make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk (cat pt-4 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-2 k "decrypt"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-3 old-0)) (stor lk (cat pt-4 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-5 k1 "wrap")) (send (enc k k1))))
  (label 1417)
  (parent 634)
  (unrealized (6 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (precedes ((1 1) (3 3)) ((1 1) (5 2)) ((2 1) (3 2)) ((2 1) (6 0))
    ((3 4) (0 0)) ((4 1) (2 0)) ((4 2) (3 0)) ((5 4) (0 0))
    ((6 1) (1 0)) ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k "wrap")
    (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 1 7 make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt-5 k "init")) (stor lk (cat pt k "wrap")))
    ((load lk (cat pt-2 k "init")) (stor lk (cat pt-0 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "decrypt"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-1 old)) (stor lk (cat pt-2 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt k "wrap"))
      (load lk (cat pt-3 k1 "wrap")) (send (enc k k1)))
    ((load lk (cat pt-4 old-0)) (stor lk (cat pt-5 k "init"))
      (send (hash k))))
  (label 1418)
  (parent 634)
  (unrealized (5 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (4 3)) ((2 1) (6 2)) ((3 1) (1 0))
    ((3 1) (4 2)) ((4 4) (0 0)) ((5 1) (3 0)) ((5 2) (4 0))
    ((6 4) (0 0)) ((7 2) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k "wrap")
    (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation encryption-test (added-strand make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk (cat pt-4 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-2 k "decrypt"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-3 old-0)) (stor lk (cat pt-4 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-5 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-6 old-1)) (stor lk-0 (cat pt-7 k "init"))
      (send (hash k))))
  (label 1419)
  (parent 634)
  (unrealized (6 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (precedes ((1 1) (2 0)) ((2 1) (4 3)) ((2 1) (6 2)) ((3 1) (1 0))
    ((3 1) (4 2)) ((4 4) (0 0)) ((5 1) (3 0)) ((5 2) (4 0))
    ((6 4) (0 0)) ((7 1) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k "wrap")
    (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (added-listener k) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk (cat pt-4 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-2 k "decrypt"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-3 old-0)) (stor lk (cat pt-4 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-5 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k)))
  (label 1420)
  (parent 634)
  (unrealized (6 3) (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (4 3)) ((3 1) (1 0)) ((3 1) (4 2))
    ((3 1) (6 2)) ((4 4) (0 0)) ((5 1) (3 0)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k "wrap")
    (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 7 5 make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk (cat pt-4 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-2 k "decrypt"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-3 old-0)) (stor lk (cat pt-4 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-2 k "decrypt"))
      (load lk (cat pt-5 k1 "wrap")) (send (enc k k1))))
  (label 1421)
  (parent 635)
  (unrealized (6 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (4 3)) ((3 1) (1 0)) ((3 1) (4 2))
    ((3 1) (6 2)) ((4 4) (0 0)) ((5 1) (3 0)) ((5 2) (4 0))
    ((6 4) (0 0)) ((7 2) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k "wrap")
    (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation encryption-test (added-strand make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk (cat pt-4 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-2 k "decrypt"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-3 old-0)) (stor lk (cat pt-4 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-2 k "decrypt"))
      (load lk (cat pt-5 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-6 old-1)) (stor lk-0 (cat pt-7 k "init"))
      (send (hash k))))
  (label 1422)
  (parent 635)
  (unrealized (6 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (precedes ((1 1) (2 0)) ((2 1) (4 3)) ((3 1) (1 0)) ((3 1) (4 2))
    ((3 1) (6 2)) ((4 4) (0 0)) ((5 1) (3 0)) ((5 2) (4 0))
    ((6 4) (0 0)) ((7 1) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k "wrap")
    (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (added-listener k) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk (cat pt-4 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-2 k "decrypt"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-3 old-0)) (stor lk (cat pt-4 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-2 k "decrypt"))
      (load lk (cat pt-5 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k)))
  (label 1423)
  (parent 635)
  (unrealized (6 3) (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (6 2)) ((2 1) (4 3)) ((3 1) (1 0))
    ((3 1) (4 2)) ((4 4) (0 0)) ((5 1) (3 0)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k "wrap")
    (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 7 5 make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk (cat pt-4 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-2 k "decrypt"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-3 old-0)) (stor lk (cat pt-4 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-5 k1 "wrap")) (send (enc k k1))))
  (label 1424)
  (parent 636)
  (unrealized (6 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (precedes ((1 1) (3 3)) ((2 1) (3 2)) ((2 1) (6 0)) ((3 4) (0 0))
    ((4 1) (2 0)) ((4 2) (3 0)) ((5 4) (0 0)) ((6 1) (1 0))
    ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k "wrap")
    (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 1 7 make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt-5 k "init")) (stor lk (cat pt k "wrap")))
    ((load lk (cat pt-2 k "init")) (stor lk (cat pt-0 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "decrypt"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-1 old)) (stor lk (cat pt-2 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-5 k "init"))
      (load lk (cat pt-3 k1 "wrap")) (send (enc k k1)))
    ((load lk (cat pt-4 old-0)) (stor lk (cat pt-5 k "init"))
      (send (hash k))))
  (label 1425)
  (parent 636)
  (unrealized (5 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((1 1) (6 2)) ((2 1) (4 3)) ((3 1) (1 0))
    ((3 1) (4 2)) ((4 4) (0 0)) ((5 1) (3 0)) ((5 2) (4 0))
    ((6 4) (0 0)) ((7 2) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k "wrap")
    (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation encryption-test (added-strand make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk (cat pt-4 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-2 k "decrypt"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-3 old-0)) (stor lk (cat pt-4 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-5 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-6 old-1)) (stor lk-0 (cat pt-7 k "init"))
      (send (hash k))))
  (label 1426)
  (parent 636)
  (unrealized (6 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (precedes ((1 1) (2 0)) ((1 1) (6 2)) ((2 1) (4 3)) ((3 1) (1 0))
    ((3 1) (4 2)) ((4 4) (0 0)) ((5 1) (3 0)) ((5 2) (4 0))
    ((6 4) (0 0)) ((7 1) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k "wrap")
    (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (added-listener k) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk (cat pt-4 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-2 k "decrypt"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-3 old-0)) (stor lk (cat pt-4 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-5 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k)))
  (label 1427)
  (parent 636)
  (unrealized (6 3) (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 cur mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand decrypt 4 (x k) (k k) (lk lk))
  (defstrand wrap 5 (cur cur) (k0 k) (k1 k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (4 3)) ((3 1) (1 0)) ((3 1) (4 2))
    ((3 1) (6 2)) ((4 4) (0 0)) ((5 1) (3 0)) ((5 2) (4 0))
    ((6 3) (0 0)) ((7 4) (6 0)))
  (pen-non-orig k)
  (genStV (cat k cur) (cat k "decrypt") (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-1)
  (operation encryption-test (added-strand wrap 5) (enc k k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk (cat pt-4 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-2 k "decrypt"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-3 old-0)) (stor lk (cat pt-4 k "init"))
      (send (hash k)))
    ((recv (enc k k)) (recv (hash k)) (load lk (cat pt-2 k "decrypt"))
      (send k))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt-5 k cur))
      (load lk-0 (cat pt-6 k "wrap")) (send (enc k k))))
  (label 1428)
  (parent 637)
  (unrealized (6 1) (7 0) (7 1) (7 2) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k skey)
    (lk locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand decrypt 4 (x k) (k k) (lk lk))
  (deflistener k)
  (precedes ((1 1) (2 0)) ((2 1) (4 3)) ((3 1) (1 0)) ((3 1) (4 2))
    ((3 1) (6 2)) ((4 4) (0 0)) ((5 1) (3 0)) ((5 2) (4 0))
    ((6 3) (0 0)) ((7 1) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (added-listener k) (enc k k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk (cat pt-4 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-2 k "decrypt"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-3 old-0)) (stor lk (cat pt-4 k "init"))
      (send (hash k)))
    ((recv (enc k k)) (recv (hash k)) (load lk (cat pt-2 k "decrypt"))
      (send k)) ((recv k) (send k)))
  (label 1429)
  (parent 637)
  (unrealized (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1) (lk lk-0))
  (deflistener k)
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (0 0)) ((3 1) (4 0))
    ((3 1) (6 0)) ((4 1) (5 2)) ((5 4) (0 0)) ((6 1) (5 0))
    ((7 1) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation nonce-test (added-strand make 2) k (6 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk-0 (cat pt-3 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-4 k "decrypt"))
      (load lk-0 (cat pt-5 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-1 (cat pt-6 old-1)) (stor lk-1 (cat pt-7 k "init"))))
  (label 1430)
  (parent 652)
  (unrealized (5 3) (6 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1) (lk lk-0))
  (deflistener k)
  (precedes ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (0 0)) ((3 1) (4 0))
    ((4 1) (6 0)) ((5 4) (0 0)) ((6 1) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule scissorsRule trRl_set-decrypt-at-1 trRl_make-at-0)
  (operation nonce-test (added-strand set-decrypt 2) k (6 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk-0 (cat pt-3 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-4 k "decrypt"))
      (load lk-0 (cat pt-5 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k)))
  (label 1431)
  (parent 652)
  (unrealized (5 3) (6 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1) (lk lk-0))
  (deflistener k)
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk-0))
  (precedes ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (0 0)) ((3 1) (4 0))
    ((3 1) (7 2)) ((4 1) (5 2)) ((5 4) (0 0)) ((6 1) (5 0))
    ((7 4) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k1 "wrap")
    (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (6 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk-0 (cat pt-3 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-4 k "decrypt"))
      (load lk-0 (cat pt-5 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-6 k1-0 "wrap")) (send (enc k k1-0))))
  (label 1432)
  (parent 652)
  (unrealized (5 3) (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1) (lk lk-0))
  (deflistener k)
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (7 2)) ((2 1) (0 0)) ((3 1) (4 0))
    ((3 1) (6 0)) ((4 1) (5 2)) ((5 4) (0 0)) ((6 1) (5 0))
    ((7 4) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k1 "wrap")
    (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (6 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk-0 (cat pt-3 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-4 k "decrypt"))
      (load lk-0 (cat pt-5 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((recv (hash k)) (recv (hash k1-0)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-6 k1-0 "wrap")) (send (enc k k1-0))))
  (label 1433)
  (parent 652)
  (unrealized (5 3) (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1) (lk lk-0))
  (deflistener k)
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((2 1) (0 0)) ((3 1) (4 0)) ((4 1) (6 0))
    ((5 4) (0 0)) ((6 1) (5 0)) ((7 1) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation nonce-test (added-strand make 2) k (6 0)
    (ch-msg lk-0 (cat pt-3 k "init"))
    (ch-msg lk-0 (cat pt-4 k "decrypt")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-4 k "decrypt"))
      (load lk-0 (cat pt-5 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-1 (cat pt-6 old-1)) (stor lk-1 (cat pt-7 k "init"))))
  (label 1434)
  (parent 655)
  (unrealized (5 3) (6 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1) (lk lk-0))
  (deflistener k)
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1-0) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (0 0)) ((3 1) (4 0)) ((4 1) (7 2))
    ((5 4) (0 0)) ((6 1) (5 0)) ((7 4) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k1 "wrap")
    (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (6 0)
    (ch-msg lk-0 (cat pt-3 k "init"))
    (ch-msg lk-0 (cat pt-4 k "decrypt")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-4 k "decrypt"))
      (load lk-0 (cat pt-5 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((recv (hash k)) (recv (hash k1-0))
      (load lk-0 (cat pt-4 k "decrypt"))
      (load lk-0 (cat pt-6 k1-0 "wrap")) (send (enc k k1-0))))
  (label 1435)
  (parent 655)
  (unrealized (5 3) (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1) (lk lk-0))
  (deflistener k)
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (0 0)) ((3 1) (4 0)) ((3 1) (7 2))
    ((4 1) (6 0)) ((5 4) (0 0)) ((6 1) (5 0)) ((7 4) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k1 "wrap")
    (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (6 0)
    (ch-msg lk-0 (cat pt-3 k "init"))
    (ch-msg lk-0 (cat pt-4 k "decrypt")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-4 k "decrypt"))
      (load lk-0 (cat pt-5 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-6 k1-0 "wrap")) (send (enc k k1-0))))
  (label 1436)
  (parent 655)
  (unrealized (5 3) (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (4 3)) ((3 1) (1 0)) ((4 4) (0 0))
    ((5 1) (3 0)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 7 5 make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk (cat pt-4 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-4 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-3 old-0)) (stor lk (cat pt-4 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-4 k "init"))
      (load lk (cat pt-5 k1 "wrap")) (send (enc k k1))))
  (label 1437)
  (parent 658)
  (unrealized (6 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (4 3)) ((3 1) (1 0)) ((4 4) (0 0))
    ((5 1) (3 0)) ((5 1) (6 2)) ((5 2) (4 0)) ((6 4) (0 0))
    ((7 2) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation encryption-test (added-strand make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk (cat pt-4 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-4 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-3 old-0)) (stor lk (cat pt-4 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-4 k "init"))
      (load lk (cat pt-5 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-6 old-1)) (stor lk-0 (cat pt-7 k "init"))
      (send (hash k))))
  (label 1438)
  (parent 658)
  (unrealized (6 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (precedes ((1 1) (2 0)) ((2 1) (4 3)) ((3 1) (1 0)) ((4 4) (0 0))
    ((5 1) (3 0)) ((5 1) (6 2)) ((5 2) (4 0)) ((6 4) (0 0))
    ((7 1) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (added-listener k) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk (cat pt-4 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-4 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-3 old-0)) (stor lk (cat pt-4 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-4 k "init"))
      (load lk (cat pt-5 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k)))
  (label 1439)
  (parent 658)
  (unrealized (6 3) (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (4 3)) ((2 1) (6 2)) ((3 1) (1 0))
    ((4 4) (0 0)) ((5 1) (3 0)) ((5 2) (4 0)) ((5 2) (6 0))
    ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 7 5 make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk (cat pt-4 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-4 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-3 old-0)) (stor lk (cat pt-4 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-5 k1 "wrap")) (send (enc k k1))))
  (label 1440)
  (parent 660)
  (unrealized (6 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (precedes ((1 1) (3 3)) ((1 1) (5 2)) ((2 1) (6 0)) ((3 4) (0 0))
    ((4 1) (2 0)) ((4 2) (3 0)) ((5 4) (0 0)) ((6 1) (1 0))
    ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 1 7 make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt-5 k "init")) (stor lk (cat pt k "wrap")))
    ((load lk (cat pt-2 k "init")) (stor lk (cat pt-0 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-2 k "init"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-1 old)) (stor lk (cat pt-2 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt k "wrap"))
      (load lk (cat pt-3 k1 "wrap")) (send (enc k k1)))
    ((load lk (cat pt-4 old-0)) (stor lk (cat pt-5 k "init"))
      (send (hash k))))
  (label 1441)
  (parent 660)
  (unrealized (5 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (4 3)) ((2 1) (6 2)) ((3 1) (1 0))
    ((4 4) (0 0)) ((5 1) (3 0)) ((5 2) (4 0)) ((6 4) (0 0))
    ((7 2) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation encryption-test (added-strand make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk (cat pt-4 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-4 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-3 old-0)) (stor lk (cat pt-4 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-5 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-6 old-1)) (stor lk-0 (cat pt-7 k "init"))
      (send (hash k))))
  (label 1442)
  (parent 660)
  (unrealized (6 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (precedes ((1 1) (2 0)) ((2 1) (4 3)) ((2 1) (6 2)) ((3 1) (1 0))
    ((4 4) (0 0)) ((5 1) (3 0)) ((5 2) (4 0)) ((6 4) (0 0))
    ((7 1) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (added-listener k) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk (cat pt-4 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-4 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-3 old-0)) (stor lk (cat pt-4 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-5 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k)))
  (label 1443)
  (parent 660)
  (unrealized (6 3) (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (4 3)) ((3 1) (1 0)) ((3 1) (6 2))
    ((4 4) (0 0)) ((5 1) (3 0)) ((5 2) (4 0)) ((5 2) (6 0))
    ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k "wrap")
    (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 7 5 make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk (cat pt-4 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-4 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-3 old-0)) (stor lk (cat pt-4 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-2 k "decrypt"))
      (load lk (cat pt-5 k1 "wrap")) (send (enc k k1))))
  (label 1444)
  (parent 661)
  (unrealized (6 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (4 3)) ((3 1) (1 0)) ((3 1) (6 2))
    ((4 4) (0 0)) ((5 1) (3 0)) ((5 2) (4 0)) ((6 4) (0 0))
    ((7 2) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k "wrap")
    (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation encryption-test (added-strand make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk (cat pt-4 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-4 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-3 old-0)) (stor lk (cat pt-4 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-2 k "decrypt"))
      (load lk (cat pt-5 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-6 old-1)) (stor lk-0 (cat pt-7 k "init"))
      (send (hash k))))
  (label 1445)
  (parent 661)
  (unrealized (6 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (precedes ((1 1) (2 0)) ((2 1) (4 3)) ((3 1) (1 0)) ((3 1) (6 2))
    ((4 4) (0 0)) ((5 1) (3 0)) ((5 2) (4 0)) ((6 4) (0 0))
    ((7 1) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k "wrap")
    (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (added-listener k) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk (cat pt-4 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-4 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-3 old-0)) (stor lk (cat pt-4 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-2 k "decrypt"))
      (load lk (cat pt-5 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k)))
  (label 1446)
  (parent 661)
  (unrealized (6 3) (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (6 2)) ((2 1) (4 3)) ((3 1) (1 0))
    ((4 4) (0 0)) ((5 1) (3 0)) ((5 2) (4 0)) ((5 2) (6 0))
    ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 7 5 make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk (cat pt-4 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-4 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-3 old-0)) (stor lk (cat pt-4 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-5 k1 "wrap")) (send (enc k k1))))
  (label 1447)
  (parent 662)
  (unrealized (6 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (precedes ((1 1) (3 3)) ((2 1) (6 0)) ((3 4) (0 0)) ((4 1) (2 0))
    ((4 2) (3 0)) ((5 4) (0 0)) ((6 1) (1 0)) ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 1 7 make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt-5 k "init")) (stor lk (cat pt k "wrap")))
    ((load lk (cat pt-2 k "init")) (stor lk (cat pt-0 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-2 k "init"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-1 old)) (stor lk (cat pt-2 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-5 k "init"))
      (load lk (cat pt-3 k1 "wrap")) (send (enc k k1)))
    ((load lk (cat pt-4 old-0)) (stor lk (cat pt-5 k "init"))
      (send (hash k))))
  (label 1448)
  (parent 662)
  (unrealized (5 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((1 1) (6 2)) ((2 1) (4 3)) ((3 1) (1 0))
    ((4 4) (0 0)) ((5 1) (3 0)) ((5 2) (4 0)) ((6 4) (0 0))
    ((7 2) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation encryption-test (added-strand make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk (cat pt-4 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-4 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-3 old-0)) (stor lk (cat pt-4 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-5 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-6 old-1)) (stor lk-0 (cat pt-7 k "init"))
      (send (hash k))))
  (label 1449)
  (parent 662)
  (unrealized (6 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (precedes ((1 1) (2 0)) ((1 1) (6 2)) ((2 1) (4 3)) ((3 1) (1 0))
    ((4 4) (0 0)) ((5 1) (3 0)) ((5 2) (4 0)) ((6 4) (0 0))
    ((7 1) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (added-listener k) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk (cat pt-4 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-4 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-3 old-0)) (stor lk (cat pt-4 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-5 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k)))
  (label 1450)
  (parent 662)
  (unrealized (6 3) (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 cur mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand decrypt 4 (x k) (k k) (lk lk))
  (defstrand wrap 5 (cur cur) (k0 k) (k1 k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (4 3)) ((3 1) (1 0)) ((3 1) (6 2))
    ((4 4) (0 0)) ((5 1) (3 0)) ((5 2) (4 0)) ((6 3) (0 0))
    ((7 4) (6 0)))
  (pen-non-orig k)
  (genStV (cat k cur) (cat k "decrypt") (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-1)
  (operation encryption-test (added-strand wrap 5) (enc k k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk (cat pt-4 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-4 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-3 old-0)) (stor lk (cat pt-4 k "init"))
      (send (hash k)))
    ((recv (enc k k)) (recv (hash k)) (load lk (cat pt-2 k "decrypt"))
      (send k))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt-5 k cur))
      (load lk-0 (cat pt-6 k "wrap")) (send (enc k k))))
  (label 1451)
  (parent 663)
  (unrealized (6 1) (7 0) (7 1) (7 2) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k skey)
    (lk locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand decrypt 4 (x k) (k k) (lk lk))
  (deflistener k)
  (precedes ((1 1) (2 0)) ((2 1) (4 3)) ((3 1) (1 0)) ((3 1) (6 2))
    ((4 4) (0 0)) ((5 1) (3 0)) ((5 2) (4 0)) ((6 3) (0 0))
    ((7 1) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (added-listener k) (enc k k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk (cat pt-4 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-4 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-3 old-0)) (stor lk (cat pt-4 k "init"))
      (send (hash k)))
    ((recv (enc k k)) (recv (hash k)) (load lk (cat pt-2 k "decrypt"))
      (send k)) ((recv k) (send k)))
  (label 1452)
  (parent 663)
  (unrealized (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (deflistener k)
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (0 0)) ((3 1) (4 0))
    ((3 1) (6 0)) ((4 1) (0 0)) ((5 4) (0 0)) ((6 1) (5 0))
    ((7 1) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation nonce-test (added-strand make 2) k (6 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk-0 (cat pt-3 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-5 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-1 (cat pt-6 old-1)) (stor lk-1 (cat pt-7 k "init"))))
  (label 1453)
  (parent 678)
  (unrealized (5 3) (6 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (deflistener k)
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk-0))
  (precedes ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (0 0)) ((3 1) (4 0))
    ((3 1) (7 2)) ((4 1) (0 0)) ((5 4) (0 0)) ((6 1) (5 0))
    ((7 4) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (6 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk-0 (cat pt-3 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-5 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-6 k1-0 "wrap")) (send (enc k k1-0))))
  (label 1454)
  (parent 678)
  (unrealized (5 3) (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (deflistener k)
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (7 2)) ((2 1) (0 0)) ((3 1) (4 0))
    ((3 1) (6 0)) ((4 1) (0 0)) ((5 4) (0 0)) ((6 1) (5 0))
    ((7 4) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (6 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk-0 (cat pt-3 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-5 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((recv (hash k)) (recv (hash k1-0)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-6 k1-0 "wrap")) (send (enc k k1-0))))
  (label 1455)
  (parent 678)
  (unrealized (5 3) (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (precedes ((1 1) (4 2)) ((2 1) (3 0)) ((3 1) (0 0)) ((4 4) (0 0))
    ((5 1) (1 0)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 7 5 make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt k "wrap"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-4 k "init"))
      (load lk-0 (cat pt-5 k1 "wrap")) (send (enc k k1))))
  (label 1456)
  (parent 685)
  (unrealized (6 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (precedes ((1 1) (3 2)) ((2 1) (0 0)) ((3 4) (0 0)) ((4 1) (1 0))
    ((4 1) (5 2)) ((4 2) (3 0)) ((5 4) (0 0)) ((6 1) (2 0))
    ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 2 7 make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt-2 k "init")) (stor lk (cat pt k "wrap")))
    ((load lk-0 (cat pt-5 k "init")) (stor lk-0 (cat pt-0 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt k "wrap"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-1 old)) (stor lk (cat pt-2 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-2 k "init"))
      (load lk (cat pt-3 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-4 old-0)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k))))
  (label 1457)
  (parent 685)
  (unrealized (5 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (4 2)) ((2 1) (3 0)) ((3 1) (0 0)) ((4 4) (0 0))
    ((5 1) (1 0)) ((5 1) (6 2)) ((5 2) (4 0)) ((6 4) (0 0))
    ((7 2) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation encryption-test (added-strand make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt k "wrap"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-4 k "init"))
      (load lk-0 (cat pt-5 k1 "wrap")) (send (enc k k1)))
    ((load lk-1 (cat pt-6 old-1)) (stor lk-1 (cat pt-7 k "init"))
      (send (hash k))))
  (label 1458)
  (parent 685)
  (unrealized (6 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (deflistener k)
  (precedes ((1 1) (4 2)) ((2 1) (3 0)) ((3 1) (0 0)) ((4 4) (0 0))
    ((5 1) (1 0)) ((5 1) (6 2)) ((5 2) (4 0)) ((6 4) (0 0))
    ((7 1) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (added-listener k) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt k "wrap"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-4 k "init"))
      (load lk-0 (cat pt-5 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k)))
  (label 1459)
  (parent 685)
  (unrealized (6 3) (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (4 2)) ((2 1) (3 0)) ((3 1) (6 2)) ((4 4) (0 0))
    ((5 1) (1 0)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k "wrap")
    (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 7 5 make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt k "wrap"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-2 k "decrypt"))
      (load lk (cat pt-5 k1 "wrap")) (send (enc k k1))))
  (label 1460)
  (parent 688)
  (unrealized (6 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (precedes ((1 1) (3 2)) ((2 1) (5 2)) ((3 4) (0 0)) ((4 1) (1 0))
    ((4 2) (3 0)) ((5 4) (0 0)) ((6 1) (2 0)) ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k "wrap")
    (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 2 7 make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt-2 k "init")) (stor lk (cat pt k "wrap")))
    ((load lk-0 (cat pt-5 k "init")) (stor lk-0 (cat pt-0 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt k "wrap"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-1 old)) (stor lk (cat pt-2 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-0 k "decrypt"))
      (load lk-0 (cat pt-3 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-4 old-0)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k))))
  (label 1461)
  (parent 688)
  (unrealized (5 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (4 2)) ((2 1) (3 0)) ((3 1) (6 2)) ((4 4) (0 0))
    ((5 1) (1 0)) ((5 2) (4 0)) ((6 4) (0 0)) ((7 2) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k "wrap")
    (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation encryption-test (added-strand make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt k "wrap"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-2 k "decrypt"))
      (load lk (cat pt-5 k1 "wrap")) (send (enc k k1)))
    ((load lk-1 (cat pt-6 old-1)) (stor lk-1 (cat pt-7 k "init"))
      (send (hash k))))
  (label 1462)
  (parent 688)
  (unrealized (6 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (precedes ((1 1) (4 2)) ((2 1) (3 0)) ((3 1) (6 2)) ((4 4) (0 0))
    ((5 1) (1 0)) ((5 2) (4 0)) ((6 4) (0 0)) ((7 1) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k "wrap")
    (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (added-listener k) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt k "wrap"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-2 k "decrypt"))
      (load lk (cat pt-5 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k)))
  (label 1463)
  (parent 688)
  (unrealized (6 3) (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk-0))
  (precedes ((1 1) (4 2)) ((1 1) (6 2)) ((2 1) (3 0)) ((3 1) (0 0))
    ((4 4) (0 0)) ((5 1) (1 0)) ((5 2) (4 0)) ((5 2) (6 0))
    ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 7 5 make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt k "wrap"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt k "wrap"))
      (load lk-0 (cat pt-5 k1 "wrap")) (send (enc k k1))))
  (label 1464)
  (parent 689)
  (unrealized (6 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (precedes ((1 1) (3 2)) ((1 1) (5 2)) ((2 1) (0 0)) ((3 4) (0 0))
    ((4 1) (1 0)) ((4 2) (3 0)) ((5 4) (0 0)) ((6 1) (2 0))
    ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 2 7 make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt-2 k "init")) (stor lk (cat pt k "wrap")))
    ((load lk-0 (cat pt-5 k "init")) (stor lk-0 (cat pt-0 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt k "wrap"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-1 old)) (stor lk (cat pt-2 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt k "wrap"))
      (load lk (cat pt-3 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-4 old-0)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k))))
  (label 1465)
  (parent 689)
  (unrealized (5 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk-0))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (4 2)) ((1 1) (6 2)) ((2 1) (3 0)) ((3 1) (0 0))
    ((4 4) (0 0)) ((5 1) (1 0)) ((5 2) (4 0)) ((6 4) (0 0))
    ((7 2) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation encryption-test (added-strand make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt k "wrap"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt k "wrap"))
      (load lk-0 (cat pt-5 k1 "wrap")) (send (enc k k1)))
    ((load lk-1 (cat pt-6 old-1)) (stor lk-1 (cat pt-7 k "init"))
      (send (hash k))))
  (label 1466)
  (parent 689)
  (unrealized (6 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk-0))
  (deflistener k)
  (precedes ((1 1) (4 2)) ((1 1) (6 2)) ((2 1) (3 0)) ((3 1) (0 0))
    ((4 4) (0 0)) ((5 1) (1 0)) ((5 2) (4 0)) ((6 4) (0 0))
    ((7 1) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (added-listener k) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt k "wrap"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt k "wrap"))
      (load lk-0 (cat pt-5 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k)))
  (label 1467)
  (parent 689)
  (unrealized (6 3) (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (4 2)) ((2 1) (3 0)) ((2 1) (6 2)) ((3 1) (0 0))
    ((4 4) (0 0)) ((5 1) (1 0)) ((5 2) (4 0)) ((5 2) (6 0))
    ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 7 5 make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt k "wrap"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt-5 k1 "wrap")) (send (enc k k1))))
  (label 1468)
  (parent 690)
  (unrealized (6 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (precedes ((1 1) (3 2)) ((2 1) (0 0)) ((3 4) (0 0)) ((4 1) (1 0))
    ((4 2) (3 0)) ((5 4) (0 0)) ((6 1) (2 0)) ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 2 7 make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt-2 k "init")) (stor lk (cat pt k "wrap")))
    ((load lk-0 (cat pt-5 k "init")) (stor lk-0 (cat pt-0 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt k "wrap"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-1 old)) (stor lk (cat pt-2 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-5 k "init"))
      (load lk-0 (cat pt-3 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-4 old-0)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k))))
  (label 1469)
  (parent 690)
  (unrealized (5 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (4 2)) ((2 1) (3 0)) ((2 1) (6 2)) ((3 1) (0 0))
    ((4 4) (0 0)) ((5 1) (1 0)) ((5 2) (4 0)) ((6 4) (0 0))
    ((7 2) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation encryption-test (added-strand make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt k "wrap"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt-5 k1 "wrap")) (send (enc k k1)))
    ((load lk-1 (cat pt-6 old-1)) (stor lk-1 (cat pt-7 k "init"))
      (send (hash k))))
  (label 1470)
  (parent 690)
  (unrealized (6 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (precedes ((1 1) (4 2)) ((2 1) (3 0)) ((2 1) (6 2)) ((3 1) (0 0))
    ((4 4) (0 0)) ((5 1) (1 0)) ((5 2) (4 0)) ((6 4) (0 0))
    ((7 1) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (added-listener k) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt k "wrap"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt-5 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k)))
  (label 1471)
  (parent 690)
  (unrealized (6 3) (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand decrypt 4 (x k) (k k) (lk lk))
  (precedes ((1 1) (4 2)) ((2 1) (3 0)) ((3 1) (6 2)) ((4 4) (6 0))
    ((5 1) (1 0)) ((5 2) (4 0)) ((6 3) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 7 4 wrap 5) (enc k k) (6 0))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt k "wrap"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init"))
      (send (hash k)))
    ((recv (enc k k)) (recv (hash k)) (load lk (cat pt-2 k "decrypt"))
      (send k)))
  (label 1472)
  (parent 691)
  (realized)
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 cur mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand decrypt 4 (x k) (k k) (lk lk))
  (defstrand wrap 5 (cur cur) (k0 k) (k1 k) (lk lk-1))
  (precedes ((1 1) (4 2)) ((2 1) (3 0)) ((3 1) (6 2)) ((4 4) (0 0))
    ((5 1) (1 0)) ((5 2) (4 0)) ((6 3) (0 0)) ((7 4) (6 0)))
  (pen-non-orig k)
  (genStV (cat k cur) (cat k "decrypt") (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-1)
  (operation encryption-test (added-strand wrap 5) (enc k k) (6 0))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt k "wrap"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init"))
      (send (hash k)))
    ((recv (enc k k)) (recv (hash k)) (load lk (cat pt-2 k "decrypt"))
      (send k))
    ((recv (hash k)) (recv (hash k)) (load lk-1 (cat pt-5 k cur))
      (load lk-1 (cat pt-6 k "wrap")) (send (enc k k))))
  (label 1473)
  (parent 691)
  (unrealized (6 1) (7 0) (7 1) (7 2) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand decrypt 4 (x k) (k k) (lk lk))
  (deflistener k)
  (precedes ((1 1) (4 2)) ((2 1) (3 0)) ((3 1) (6 2)) ((4 4) (0 0))
    ((5 1) (1 0)) ((5 2) (4 0)) ((6 3) (0 0)) ((7 1) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (added-listener k) (enc k k) (6 0))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt k "wrap"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init"))
      (send (hash k)))
    ((recv (enc k k)) (recv (hash k)) (load lk (cat pt-2 k "decrypt"))
      (send k)) ((recv k) (send k)))
  (label 1474)
  (parent 691)
  (unrealized (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (precedes ((1 1) (2 0)) ((2 1) (6 0)) ((3 1) (4 0)) ((3 1) (6 0))
    ((4 1) (0 0)) ((5 4) (0 0)) ((6 1) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation nonce-test (displaced 7 3 make 2) k (6 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-5 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k)))
  (label 1475)
  (parent 706)
  (unrealized (5 3) (6 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((2 1) (6 0)) ((3 1) (4 0)) ((4 1) (0 0))
    ((5 4) (0 0)) ((6 1) (5 0)) ((7 1) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation nonce-test (added-strand make 2) k (6 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-5 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-1 (cat pt-6 old-1)) (stor lk-1 (cat pt-7 k "init"))))
  (label 1476)
  (parent 706)
  (unrealized (5 3) (6 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1-0) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (7 2)) ((3 1) (4 0)) ((4 1) (0 0))
    ((5 4) (0 0)) ((6 1) (5 0)) ((7 4) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap")
    (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (6 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-5 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((recv (hash k)) (recv (hash k1-0)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-6 k1-0 "wrap")) (send (enc k k1-0))))
  (label 1477)
  (parent 706)
  (unrealized (5 3) (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (7 2)) ((2 1) (6 0)) ((3 1) (4 0))
    ((4 1) (0 0)) ((5 4) (0 0)) ((6 1) (5 0)) ((7 4) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap")
    (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (6 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-5 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((recv (hash k)) (recv (hash k1-0)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-6 k1-0 "wrap")) (send (enc k k1-0))))
  (label 1478)
  (parent 706)
  (unrealized (5 3) (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (5 2)) ((3 1) (4 0))
    ((3 1) (6 0)) ((4 1) (0 0)) ((5 4) (0 0)) ((6 1) (5 0))
    ((7 1) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation nonce-test (added-strand make 2) k (6 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk-0 (cat pt-3 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-5 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-1 (cat pt-6 old-1)) (stor lk-1 (cat pt-7 k "init"))))
  (label 1479)
  (parent 708)
  (unrealized (5 3) (6 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk-0))
  (precedes ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (5 2)) ((3 1) (4 0))
    ((3 1) (7 2)) ((4 1) (0 0)) ((5 4) (0 0)) ((6 1) (5 0))
    ((7 4) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap")
    (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (6 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk-0 (cat pt-3 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-5 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-6 k1-0 "wrap")) (send (enc k k1-0))))
  (label 1480)
  (parent 708)
  (unrealized (5 3) (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (7 2)) ((2 1) (5 2)) ((3 1) (4 0))
    ((3 1) (6 0)) ((4 1) (0 0)) ((5 4) (0 0)) ((6 1) (5 0))
    ((7 4) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap")
    (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (6 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk-0 (cat pt-3 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-5 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((recv (hash k)) (recv (hash k1-0)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-6 k1-0 "wrap")) (send (enc k k1-0))))
  (label 1481)
  (parent 708)
  (unrealized (5 3) (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (precedes ((1 1) (4 3)) ((2 1) (3 0)) ((3 1) (0 0)) ((4 4) (0 0))
    ((5 1) (1 0)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 7 5 make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt-4 k "init"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-4 k "init"))
      (load lk-0 (cat pt-5 k1 "wrap")) (send (enc k k1))))
  (label 1482)
  (parent 715)
  (unrealized (6 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (precedes ((1 1) (3 3)) ((2 1) (0 0)) ((3 4) (0 0)) ((4 1) (1 0))
    ((4 1) (5 2)) ((4 2) (3 0)) ((5 4) (0 0)) ((6 1) (2 0))
    ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 2 7 make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt-2 k "init")) (stor lk (cat pt k "wrap")))
    ((load lk-0 (cat pt-5 k "init")) (stor lk-0 (cat pt-0 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-2 k "init"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-1 old)) (stor lk (cat pt-2 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-2 k "init"))
      (load lk (cat pt-3 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-4 old-0)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k))))
  (label 1483)
  (parent 715)
  (unrealized (5 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (4 3)) ((2 1) (3 0)) ((3 1) (0 0)) ((4 4) (0 0))
    ((5 1) (1 0)) ((5 1) (6 2)) ((5 2) (4 0)) ((6 4) (0 0))
    ((7 2) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation encryption-test (added-strand make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt-4 k "init"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-4 k "init"))
      (load lk-0 (cat pt-5 k1 "wrap")) (send (enc k k1)))
    ((load lk-1 (cat pt-6 old-1)) (stor lk-1 (cat pt-7 k "init"))
      (send (hash k))))
  (label 1484)
  (parent 715)
  (unrealized (6 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (deflistener k)
  (precedes ((1 1) (4 3)) ((2 1) (3 0)) ((3 1) (0 0)) ((4 4) (0 0))
    ((5 1) (1 0)) ((5 1) (6 2)) ((5 2) (4 0)) ((6 4) (0 0))
    ((7 1) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (added-listener k) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt-4 k "init"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-4 k "init"))
      (load lk-0 (cat pt-5 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k)))
  (label 1485)
  (parent 715)
  (unrealized (6 3) (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (4 3)) ((2 1) (3 0)) ((3 1) (6 2)) ((4 4) (0 0))
    ((5 1) (1 0)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k "wrap")
    (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 7 5 make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt-4 k "init"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-2 k "decrypt"))
      (load lk (cat pt-5 k1 "wrap")) (send (enc k k1))))
  (label 1486)
  (parent 718)
  (unrealized (6 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (precedes ((1 1) (3 3)) ((2 1) (5 2)) ((3 4) (0 0)) ((4 1) (1 0))
    ((4 2) (3 0)) ((5 4) (0 0)) ((6 1) (2 0)) ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k "wrap")
    (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 2 7 make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt-2 k "init")) (stor lk (cat pt k "wrap")))
    ((load lk-0 (cat pt-5 k "init")) (stor lk-0 (cat pt-0 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-2 k "init"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-1 old)) (stor lk (cat pt-2 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-0 k "decrypt"))
      (load lk-0 (cat pt-3 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-4 old-0)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k))))
  (label 1487)
  (parent 718)
  (unrealized (5 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (4 3)) ((2 1) (3 0)) ((3 1) (6 2)) ((4 4) (0 0))
    ((5 1) (1 0)) ((5 2) (4 0)) ((6 4) (0 0)) ((7 2) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k "wrap")
    (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation encryption-test (added-strand make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt-4 k "init"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-2 k "decrypt"))
      (load lk (cat pt-5 k1 "wrap")) (send (enc k k1)))
    ((load lk-1 (cat pt-6 old-1)) (stor lk-1 (cat pt-7 k "init"))
      (send (hash k))))
  (label 1488)
  (parent 718)
  (unrealized (6 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (precedes ((1 1) (4 3)) ((2 1) (3 0)) ((3 1) (6 2)) ((4 4) (0 0))
    ((5 1) (1 0)) ((5 2) (4 0)) ((6 4) (0 0)) ((7 1) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k "wrap")
    (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (added-listener k) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt-4 k "init"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-2 k "decrypt"))
      (load lk (cat pt-5 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k)))
  (label 1489)
  (parent 718)
  (unrealized (6 3) (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk-0))
  (precedes ((1 1) (4 3)) ((1 1) (6 2)) ((2 1) (3 0)) ((3 1) (0 0))
    ((4 4) (0 0)) ((5 1) (1 0)) ((5 2) (4 0)) ((5 2) (6 0))
    ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 7 5 make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt-4 k "init"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt k "wrap"))
      (load lk-0 (cat pt-5 k1 "wrap")) (send (enc k k1))))
  (label 1490)
  (parent 719)
  (unrealized (6 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (precedes ((1 1) (3 3)) ((1 1) (5 2)) ((2 1) (0 0)) ((3 4) (0 0))
    ((4 1) (1 0)) ((4 2) (3 0)) ((5 4) (0 0)) ((6 1) (2 0))
    ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 2 7 make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt-2 k "init")) (stor lk (cat pt k "wrap")))
    ((load lk-0 (cat pt-5 k "init")) (stor lk-0 (cat pt-0 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-2 k "init"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-1 old)) (stor lk (cat pt-2 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt k "wrap"))
      (load lk (cat pt-3 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-4 old-0)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k))))
  (label 1491)
  (parent 719)
  (unrealized (5 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk-0))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (4 3)) ((1 1) (6 2)) ((2 1) (3 0)) ((3 1) (0 0))
    ((4 4) (0 0)) ((5 1) (1 0)) ((5 2) (4 0)) ((6 4) (0 0))
    ((7 2) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation encryption-test (added-strand make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt-4 k "init"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt k "wrap"))
      (load lk-0 (cat pt-5 k1 "wrap")) (send (enc k k1)))
    ((load lk-1 (cat pt-6 old-1)) (stor lk-1 (cat pt-7 k "init"))
      (send (hash k))))
  (label 1492)
  (parent 719)
  (unrealized (6 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk-0))
  (deflistener k)
  (precedes ((1 1) (4 3)) ((1 1) (6 2)) ((2 1) (3 0)) ((3 1) (0 0))
    ((4 4) (0 0)) ((5 1) (1 0)) ((5 2) (4 0)) ((6 4) (0 0))
    ((7 1) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (added-listener k) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt-4 k "init"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt k "wrap"))
      (load lk-0 (cat pt-5 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k)))
  (label 1493)
  (parent 719)
  (unrealized (6 3) (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (4 3)) ((2 1) (3 0)) ((2 1) (6 2)) ((3 1) (0 0))
    ((4 4) (0 0)) ((5 1) (1 0)) ((5 2) (4 0)) ((5 2) (6 0))
    ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 7 5 make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt-4 k "init"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt-5 k1 "wrap")) (send (enc k k1))))
  (label 1494)
  (parent 720)
  (unrealized (6 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (precedes ((1 1) (3 3)) ((2 1) (0 0)) ((3 4) (0 0)) ((4 1) (1 0))
    ((4 2) (3 0)) ((5 4) (0 0)) ((6 1) (2 0)) ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 2 7 make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt-2 k "init")) (stor lk (cat pt k "wrap")))
    ((load lk-0 (cat pt-5 k "init")) (stor lk-0 (cat pt-0 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-2 k "init"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-1 old)) (stor lk (cat pt-2 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-5 k "init"))
      (load lk-0 (cat pt-3 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-4 old-0)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k))))
  (label 1495)
  (parent 720)
  (unrealized (5 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (4 3)) ((2 1) (3 0)) ((2 1) (6 2)) ((3 1) (0 0))
    ((4 4) (0 0)) ((5 1) (1 0)) ((5 2) (4 0)) ((6 4) (0 0))
    ((7 2) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation encryption-test (added-strand make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt-4 k "init"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt-5 k1 "wrap")) (send (enc k k1)))
    ((load lk-1 (cat pt-6 old-1)) (stor lk-1 (cat pt-7 k "init"))
      (send (hash k))))
  (label 1496)
  (parent 720)
  (unrealized (6 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (precedes ((1 1) (4 3)) ((2 1) (3 0)) ((2 1) (6 2)) ((3 1) (0 0))
    ((4 4) (0 0)) ((5 1) (1 0)) ((5 2) (4 0)) ((6 4) (0 0))
    ((7 1) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (added-listener k) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt-4 k "init"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt-5 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k)))
  (label 1497)
  (parent 720)
  (unrealized (6 3) (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand decrypt 4 (x k) (k k) (lk lk))
  (precedes ((1 1) (4 3)) ((2 1) (3 0)) ((3 1) (6 2)) ((4 4) (6 0))
    ((5 1) (1 0)) ((5 2) (4 0)) ((6 3) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 7 4 wrap 5) (enc k k) (6 0))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt-4 k "init"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init"))
      (send (hash k)))
    ((recv (enc k k)) (recv (hash k)) (load lk (cat pt-2 k "decrypt"))
      (send k)))
  (label 1498)
  (parent 721)
  (realized)
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 cur mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand decrypt 4 (x k) (k k) (lk lk))
  (defstrand wrap 5 (cur cur) (k0 k) (k1 k) (lk lk-1))
  (precedes ((1 1) (4 3)) ((2 1) (3 0)) ((3 1) (6 2)) ((4 4) (0 0))
    ((5 1) (1 0)) ((5 2) (4 0)) ((6 3) (0 0)) ((7 4) (6 0)))
  (pen-non-orig k)
  (genStV (cat k cur) (cat k "decrypt") (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-1)
  (operation encryption-test (added-strand wrap 5) (enc k k) (6 0))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt-4 k "init"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init"))
      (send (hash k)))
    ((recv (enc k k)) (recv (hash k)) (load lk (cat pt-2 k "decrypt"))
      (send k))
    ((recv (hash k)) (recv (hash k)) (load lk-1 (cat pt-5 k cur))
      (load lk-1 (cat pt-6 k "wrap")) (send (enc k k))))
  (label 1499)
  (parent 721)
  (unrealized (6 1) (7 0) (7 1) (7 2) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand decrypt 4 (x k) (k k) (lk lk))
  (deflistener k)
  (precedes ((1 1) (4 3)) ((2 1) (3 0)) ((3 1) (6 2)) ((4 4) (0 0))
    ((5 1) (1 0)) ((5 2) (4 0)) ((6 3) (0 0)) ((7 1) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (added-listener k) (enc k k) (6 0))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt-4 k "init"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init"))
      (send (hash k)))
    ((recv (enc k k)) (recv (hash k)) (load lk (cat pt-2 k "decrypt"))
      (send k)) ((recv k) (send k)))
  (label 1500)
  (parent 721)
  (unrealized (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (0 0)) ((3 1) (4 0))
    ((3 1) (6 0)) ((4 1) (0 0)) ((5 4) (0 0)) ((6 1) (5 0))
    ((7 1) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation nonce-test (added-strand make 2) k (6 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk-0 (cat pt-3 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-5 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-1 (cat pt-6 old-1)) (stor lk-1 (cat pt-7 k "init"))))
  (label 1501)
  (parent 737)
  (unrealized (5 3) (6 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk-0))
  (precedes ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (0 0)) ((3 1) (4 0))
    ((3 1) (7 2)) ((4 1) (0 0)) ((5 4) (0 0)) ((6 1) (5 0))
    ((7 4) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (6 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk-0 (cat pt-3 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-5 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-6 k1-0 "wrap")) (send (enc k k1-0))))
  (label 1502)
  (parent 737)
  (unrealized (5 3) (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (7 2)) ((2 1) (0 0)) ((3 1) (4 0))
    ((3 1) (6 0)) ((4 1) (0 0)) ((5 4) (0 0)) ((6 1) (5 0))
    ((7 4) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (6 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk-0 (cat pt-3 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-5 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((recv (hash k)) (recv (hash k1-0)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-6 k1-0 "wrap")) (send (enc k k1-0))))
  (label 1503)
  (parent 737)
  (unrealized (5 3) (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand make 2 (old old-1) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "wrap") (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (3 3)) ((3 4) (0 0)) ((4 1) (1 0))
    ((4 2) (3 0)) ((5 1) (6 0)) ((6 1) (7 0)) ((7 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "wrap" "decrypt") (neq "wrap" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-decrypt-at-1 fact-set-decrypt-neq0 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation nonce-test (added-strand set-decrypt 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk (cat pt-3 k "init")) (ch-msg lk-0 (cat pt-5 k "init"))
    (ch-msg lk-0 (cat pt-6 k "wrap")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-4 old-1)) (stor lk-0 (cat pt-5 k "init")))
    ((load lk-0 (cat pt-5 k "init")) (stor lk-0 (cat pt-6 k "wrap")))
    ((load lk-0 (cat pt-6 k "wrap"))
      (stor lk-0 (cat pt-7 k "decrypt"))))
  (label 1504)
  (parent 741)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand make 2 (old old-1) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "wrap") (k k) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (3 3)) ((2 1) (7 0)) ((3 4) (0 0))
    ((4 1) (1 0)) ((4 2) (3 0)) ((5 1) (6 0)) ((6 1) (0 0))
    ((7 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "wrap" "decrypt") (neq "wrap" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule fact-set-decrypt-neq0 trRl_set-wrap-at-1 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation nonce-test (added-strand set-wrap 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk (cat pt-3 k "init")) (ch-msg lk-0 (cat pt-5 k "init"))
    (ch-msg lk-0 (cat pt-6 k "wrap")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-4 old-1)) (stor lk-0 (cat pt-5 k "init")))
    ((load lk-0 (cat pt-5 k "init")) (stor lk-0 (cat pt-6 k "wrap")))
    ((load lk (cat pt-1 k "wrap")) (stor lk (cat pt-7 k "wrap"))))
  (label 1505)
  (parent 741)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 old-2 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval) (k skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand make 2 (old old-1) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old-2) (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((2 1) (3 3)) ((3 4) (0 0)) ((4 1) (1 0))
    ((4 2) (3 0)) ((5 1) (6 0)) ((6 1) (0 0)) ((7 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation nonce-test (added-strand make 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk (cat pt-3 k "init")) (ch-msg lk-0 (cat pt-5 k "init"))
    (ch-msg lk-0 (cat pt-6 k "wrap")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-4 old-1)) (stor lk-0 (cat pt-5 k "init")))
    ((load lk-0 (cat pt-5 k "init")) (stor lk-0 (cat pt-6 k "wrap")))
    ((load lk-1 (cat pt-7 old-2)) (stor lk-1 (cat pt-8 k "init"))))
  (label 1506)
  (parent 741)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand make 2 (old old-1) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "wrap") (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (3 3)) ((3 4) (0 0)) ((4 1) (1 0))
    ((4 2) (3 0)) ((5 1) (6 0)) ((6 1) (7 0)) ((7 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "wrap" "decrypt") (neq "wrap" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule fact-set-decrypt-neq0 trRl_set-wrap-at-1 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation nonce-test (added-strand set-wrap 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk (cat pt-3 k "init")) (ch-msg lk-0 (cat pt-5 k "init"))
    (ch-msg lk-0 (cat pt-6 k "wrap")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-4 old-1)) (stor lk-0 (cat pt-5 k "init")))
    ((load lk-0 (cat pt-5 k "init")) (stor lk-0 (cat pt-6 k "wrap")))
    ((load lk-0 (cat pt-6 k "wrap")) (stor lk-0 (cat pt-7 k "wrap"))))
  (label 1507)
  (parent 741)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand make 2 (old old-1) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (3 3)) ((3 4) (0 0)) ((4 1) (7 0))
    ((4 2) (3 0)) ((5 1) (6 0)) ((6 1) (0 0)) ((7 1) (1 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule invShearsRule shearsRule trRl_set-wrap-at-1 trRl_make-at-0)
  (operation nonce-test (added-strand set-wrap 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk (cat pt-3 k "init")) (ch-msg lk-0 (cat pt-5 k "init"))
    (ch-msg lk-0 (cat pt-6 k "wrap")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-4 old-1)) (stor lk-0 (cat pt-5 k "init")))
    ((load lk-0 (cat pt-5 k "init")) (stor lk-0 (cat pt-6 k "wrap")))
    ((load lk (cat pt-3 k "init")) (stor lk (cat pt-7 k "wrap"))))
  (label 1508)
  (parent 741)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand make 2 (old old-1) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (3 3)) ((3 4) (0 0)) ((4 1) (1 0))
    ((4 2) (3 0)) ((5 1) (6 0)) ((6 1) (7 2)) ((7 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk (cat pt-3 k "init")) (ch-msg lk-0 (cat pt-5 k "init"))
    (ch-msg lk-0 (cat pt-6 k "wrap")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-4 old-1)) (stor lk-0 (cat pt-5 k "init")))
    ((load lk-0 (cat pt-5 k "init")) (stor lk-0 (cat pt-6 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-6 k "wrap"))
      (load lk-0 (cat pt-7 k1 "wrap")) (send (enc k k1))))
  (label 1509)
  (parent 741)
  (unrealized (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand make 2 (old old-1) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "wrap") (k k) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (3 3)) ((2 1) (7 0)) ((3 4) (0 0))
    ((4 1) (1 0)) ((4 2) (3 0)) ((5 1) (6 0)) ((6 1) (0 0))
    ((7 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "wrap" "decrypt") (neq "wrap" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-decrypt-at-1 fact-set-decrypt-neq0 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation nonce-test (added-strand set-decrypt 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk (cat pt-3 k "init")) (ch-msg lk-0 (cat pt-5 k "init"))
    (ch-msg lk-0 (cat pt-6 k "wrap")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-4 old-1)) (stor lk-0 (cat pt-5 k "init")))
    ((load lk-0 (cat pt-5 k "init")) (stor lk-0 (cat pt-6 k "wrap")))
    ((load lk (cat pt-1 k "wrap")) (stor lk (cat pt-7 k "decrypt"))))
  (label 1510)
  (parent 741)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand make 2 (old old-1) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (3 3)) ((3 4) (0 0)) ((4 1) (1 0))
    ((4 1) (7 2)) ((4 2) (3 0)) ((5 1) (6 0)) ((6 1) (0 0))
    ((7 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk (cat pt-3 k "init")) (ch-msg lk-0 (cat pt-5 k "init"))
    (ch-msg lk-0 (cat pt-6 k "wrap")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-4 old-1)) (stor lk-0 (cat pt-5 k "init")))
    ((load lk-0 (cat pt-5 k "init")) (stor lk-0 (cat pt-6 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-7 k1 "wrap")) (send (enc k k1))))
  (label 1511)
  (parent 741)
  (unrealized (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand make 2 (old old-1) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (3 3)) ((3 4) (0 0)) ((4 1) (1 0))
    ((4 2) (3 0)) ((5 1) (6 0)) ((5 1) (7 2)) ((6 1) (0 0))
    ((7 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk (cat pt-3 k "init")) (ch-msg lk-0 (cat pt-5 k "init"))
    (ch-msg lk-0 (cat pt-6 k "wrap")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-4 old-1)) (stor lk-0 (cat pt-5 k "init")))
    ((load lk-0 (cat pt-5 k "init")) (stor lk-0 (cat pt-6 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-5 k "init"))
      (load lk-0 (cat pt-7 k1 "wrap")) (send (enc k k1))))
  (label 1512)
  (parent 741)
  (unrealized (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand make 2 (old old-1) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (3 3)) ((2 1) (7 2)) ((3 4) (0 0))
    ((4 1) (1 0)) ((4 2) (3 0)) ((5 1) (6 0)) ((6 1) (0 0))
    ((7 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk (cat pt-3 k "init")) (ch-msg lk-0 (cat pt-5 k "init"))
    (ch-msg lk-0 (cat pt-6 k "wrap")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-4 old-1)) (stor lk-0 (cat pt-5 k "init")))
    ((load lk-0 (cat pt-5 k "init")) (stor lk-0 (cat pt-6 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-7 k1 "wrap")) (send (enc k k1))))
  (label 1513)
  (parent 741)
  (unrealized (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand make 2 (old old-1) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (7 2)) ((2 1) (3 3)) ((3 4) (0 0))
    ((4 1) (1 0)) ((4 2) (3 0)) ((5 1) (6 0)) ((6 1) (0 0))
    ((7 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk (cat pt-3 k "init")) (ch-msg lk-0 (cat pt-5 k "init"))
    (ch-msg lk-0 (cat pt-6 k "wrap")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-4 old-1)) (stor lk-0 (cat pt-5 k "init")))
    ((load lk-0 (cat pt-5 k "init")) (stor lk-0 (cat pt-6 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-7 k1 "wrap")) (send (enc k k1))))
  (label 1514)
  (parent 741)
  (unrealized (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand make 2 (old old-1) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand set-wrap 2 (cur "wrap") (k k) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (3 3)) ((2 1) (7 0)) ((3 4) (0 0))
    ((4 1) (6 0)) ((4 2) (3 0)) ((5 1) (0 0)) ((6 1) (1 0))
    ((7 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "wrap" "decrypt") (neq "wrap" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule fact-set-decrypt-neq0 trRl_set-wrap-at-1 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation nonce-test (added-strand set-wrap 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk (cat pt-3 k "init")) (ch-msg lk (cat pt-6 k "wrap"))
    (ch-msg lk-0 (cat pt-5 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-4 old-1)) (stor lk-0 (cat pt-5 k "init")))
    ((load lk (cat pt-3 k "init")) (stor lk (cat pt-6 k "wrap")))
    ((load lk (cat pt-1 k "wrap")) (stor lk (cat pt-7 k "wrap"))))
  (label 1515)
  (parent 742)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand make 2 (old old-1) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (3 3)) ((3 4) (0 0)) ((4 1) (6 0))
    ((4 2) (3 0)) ((5 1) (7 2)) ((6 1) (1 0)) ((7 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk (cat pt-3 k "init")) (ch-msg lk (cat pt-6 k "wrap"))
    (ch-msg lk-0 (cat pt-5 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-4 old-1)) (stor lk-0 (cat pt-5 k "init")))
    ((load lk (cat pt-3 k "init")) (stor lk (cat pt-6 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-5 k "init"))
      (load lk-0 (cat pt-7 k1 "wrap")) (send (enc k k1))))
  (label 1516)
  (parent 742)
  (unrealized (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand make 2 (old old-1) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "wrap") (k k) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (3 3)) ((2 1) (7 0)) ((3 4) (0 0))
    ((4 1) (6 0)) ((4 2) (3 0)) ((5 1) (0 0)) ((6 1) (1 0))
    ((7 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "wrap" "decrypt") (neq "wrap" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-decrypt-at-1 fact-set-decrypt-neq0 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation nonce-test (added-strand set-decrypt 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk (cat pt-3 k "init")) (ch-msg lk (cat pt-6 k "wrap"))
    (ch-msg lk-0 (cat pt-5 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-4 old-1)) (stor lk-0 (cat pt-5 k "init")))
    ((load lk (cat pt-3 k "init")) (stor lk (cat pt-6 k "wrap")))
    ((load lk (cat pt-1 k "wrap")) (stor lk (cat pt-7 k "decrypt"))))
  (label 1517)
  (parent 742)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand make 2 (old old-1) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (3 3)) ((3 4) (0 0)) ((4 1) (6 0))
    ((4 1) (7 2)) ((4 2) (3 0)) ((5 1) (0 0)) ((6 1) (1 0))
    ((7 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk (cat pt-3 k "init")) (ch-msg lk (cat pt-6 k "wrap"))
    (ch-msg lk-0 (cat pt-5 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-4 old-1)) (stor lk-0 (cat pt-5 k "init")))
    ((load lk (cat pt-3 k "init")) (stor lk (cat pt-6 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-7 k1 "wrap")) (send (enc k k1))))
  (label 1518)
  (parent 742)
  (unrealized (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand make 2 (old old-1) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (3 3)) ((3 4) (0 0)) ((4 1) (6 0))
    ((4 2) (3 0)) ((5 1) (0 0)) ((6 1) (1 0)) ((6 1) (7 2))
    ((7 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk (cat pt-3 k "init")) (ch-msg lk (cat pt-6 k "wrap"))
    (ch-msg lk-0 (cat pt-5 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-4 old-1)) (stor lk-0 (cat pt-5 k "init")))
    ((load lk (cat pt-3 k "init")) (stor lk (cat pt-6 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-6 k "wrap"))
      (load lk (cat pt-7 k1 "wrap")) (send (enc k k1))))
  (label 1519)
  (parent 742)
  (unrealized (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand make 2 (old old-1) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (3 3)) ((2 1) (7 2)) ((3 4) (0 0))
    ((4 1) (6 0)) ((4 2) (3 0)) ((5 1) (0 0)) ((6 1) (1 0))
    ((7 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk (cat pt-3 k "init")) (ch-msg lk (cat pt-6 k "wrap"))
    (ch-msg lk-0 (cat pt-5 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-4 old-1)) (stor lk-0 (cat pt-5 k "init")))
    ((load lk (cat pt-3 k "init")) (stor lk (cat pt-6 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-7 k1 "wrap")) (send (enc k k1))))
  (label 1520)
  (parent 742)
  (unrealized (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand make 2 (old old-1) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (7 2)) ((2 1) (3 3)) ((3 4) (0 0))
    ((4 1) (6 0)) ((4 2) (3 0)) ((5 1) (0 0)) ((6 1) (1 0))
    ((7 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk (cat pt-3 k "init")) (ch-msg lk (cat pt-6 k "wrap"))
    (ch-msg lk-0 (cat pt-5 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-4 old-1)) (stor lk-0 (cat pt-5 k "init")))
    ((load lk (cat pt-3 k "init")) (stor lk (cat pt-6 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-7 k1 "wrap")) (send (enc k k1))))
  (label 1521)
  (parent 742)
  (unrealized (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand make 2 (old old-1) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (3 3)) ((3 4) (0 0)) ((4 1) (1 0))
    ((4 2) (3 0)) ((4 2) (6 0)) ((5 1) (6 2)) ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 7 4 make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-4 old-1)) (stor lk-0 (cat pt-5 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-5 k "init"))
      (load lk-0 (cat pt-6 k1 "wrap")) (send (enc k k1))))
  (label 1522)
  (parent 744)
  (unrealized (6 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old) (k k) (lk lk-0))
  (defstrand make 2 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-0))
  (precedes ((1 1) (2 3)) ((2 4) (0 0)) ((3 1) (6 0)) ((3 2) (2 0))
    ((4 1) (5 2)) ((5 4) (0 0)) ((6 1) (1 0)) ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 1 7 make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-6 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt-1 k "init"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-0 old)) (stor lk-0 (cat pt-1 k "init"))
      (send (hash k)))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-5 old-1)) (stor lk-0 (cat pt-6 k "init"))
      (send (hash k))))
  (label 1523)
  (parent 744)
  (unrealized (5 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (defstrand make 3 (old old-1) (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (3 3)) ((3 4) (0 0)) ((4 1) (1 0))
    ((4 2) (3 0)) ((5 4) (0 0)) ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 5 7 make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-6 k "init"))
      (load lk-0 (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-5 old-1)) (stor lk-0 (cat pt-6 k "init"))
      (send (hash k))))
  (label 1524)
  (parent 744)
  (unrealized (5 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 old-2 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand make 2 (old old-1) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (defstrand make 3 (old old-2) (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((2 1) (3 3)) ((3 4) (0 0)) ((4 1) (1 0))
    ((4 2) (3 0)) ((5 1) (6 2)) ((6 4) (0 0)) ((7 2) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation encryption-test (added-strand make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-4 old-1)) (stor lk-0 (cat pt-5 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-5 k "init"))
      (load lk-0 (cat pt-6 k1 "wrap")) (send (enc k k1)))
    ((load lk-1 (cat pt-7 old-2)) (stor lk-1 (cat pt-8 k "init"))
      (send (hash k))))
  (label 1525)
  (parent 744)
  (unrealized (6 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand make 2 (old old-1) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (deflistener k)
  (precedes ((1 1) (2 0)) ((2 1) (3 3)) ((3 4) (0 0)) ((4 1) (1 0))
    ((4 2) (3 0)) ((5 1) (6 2)) ((6 4) (0 0)) ((7 1) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (added-listener k) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-4 old-1)) (stor lk-0 (cat pt-5 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-5 k "init"))
      (load lk-0 (cat pt-6 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k)))
  (label 1526)
  (parent 744)
  (unrealized (6 3) (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand make 2 (old old-1) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (3 3)) ((3 4) (0 0)) ((4 1) (1 0))
    ((4 2) (3 0)) ((4 2) (6 0)) ((5 1) (0 0)) ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 7 4 make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-4 old-1)) (stor lk-0 (cat pt-5 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-6 k1 "wrap")) (send (enc k k1))))
  (label 1527)
  (parent 746)
  (unrealized (6 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 old-2 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand make 2 (old old-1) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old-2) (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((2 1) (3 3)) ((3 4) (0 0)) ((4 1) (1 0))
    ((4 1) (6 2)) ((4 2) (3 0)) ((5 1) (0 0)) ((6 4) (0 0))
    ((7 2) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation encryption-test (added-strand make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-4 old-1)) (stor lk-0 (cat pt-5 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-6 k1 "wrap")) (send (enc k k1)))
    ((load lk-1 (cat pt-7 old-2)) (stor lk-1 (cat pt-8 k "init"))
      (send (hash k))))
  (label 1528)
  (parent 746)
  (unrealized (6 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand make 2 (old old-1) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (precedes ((1 1) (2 0)) ((2 1) (3 3)) ((3 4) (0 0)) ((4 1) (1 0))
    ((4 1) (6 2)) ((4 2) (3 0)) ((5 1) (0 0)) ((6 4) (0 0))
    ((7 1) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (added-listener k) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-4 old-1)) (stor lk-0 (cat pt-5 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-6 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k)))
  (label 1529)
  (parent 746)
  (unrealized (6 3) (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand make 2 (old old-1) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (3 3)) ((2 1) (6 2)) ((3 4) (0 0))
    ((4 1) (1 0)) ((4 2) (3 0)) ((4 2) (6 0)) ((5 1) (0 0))
    ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 7 4 make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-4 old-1)) (stor lk-0 (cat pt-5 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-6 k1 "wrap")) (send (enc k k1))))
  (label 1530)
  (parent 747)
  (unrealized (6 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old) (k k) (lk lk-0))
  (defstrand make 2 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk-0))
  (defstrand make 3 (old old-1) (k k) (lk lk-0))
  (precedes ((1 1) (2 3)) ((1 1) (5 2)) ((2 4) (0 0)) ((3 1) (6 0))
    ((3 2) (2 0)) ((4 1) (0 0)) ((5 4) (0 0)) ((6 1) (1 0))
    ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 1 7 make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-6 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt-1 k "init"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-0 old)) (stor lk-0 (cat pt-1 k "init"))
      (send (hash k)))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt k "wrap"))
      (load lk-0 (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-5 old-1)) (stor lk-0 (cat pt-6 k "init"))
      (send (hash k))))
  (label 1531)
  (parent 747)
  (unrealized (5 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 old-2 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand make 2 (old old-1) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old-2) (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((2 1) (3 3)) ((2 1) (6 2)) ((3 4) (0 0))
    ((4 1) (1 0)) ((4 2) (3 0)) ((5 1) (0 0)) ((6 4) (0 0))
    ((7 2) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation encryption-test (added-strand make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-4 old-1)) (stor lk-0 (cat pt-5 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-6 k1 "wrap")) (send (enc k k1)))
    ((load lk-1 (cat pt-7 old-2)) (stor lk-1 (cat pt-8 k "init"))
      (send (hash k))))
  (label 1532)
  (parent 747)
  (unrealized (6 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand make 2 (old old-1) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (precedes ((1 1) (2 0)) ((2 1) (3 3)) ((2 1) (6 2)) ((3 4) (0 0))
    ((4 1) (1 0)) ((4 2) (3 0)) ((5 1) (0 0)) ((6 4) (0 0))
    ((7 1) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (added-listener k) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-4 old-1)) (stor lk-0 (cat pt-5 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-6 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k)))
  (label 1533)
  (parent 747)
  (unrealized (6 3) (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand make 2 (old old-1) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (6 2)) ((2 1) (3 3)) ((3 4) (0 0))
    ((4 1) (1 0)) ((4 2) (3 0)) ((4 2) (6 0)) ((5 1) (0 0))
    ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 7 4 make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-4 old-1)) (stor lk-0 (cat pt-5 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-6 k1 "wrap")) (send (enc k k1))))
  (label 1534)
  (parent 748)
  (unrealized (6 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old) (k k) (lk lk-0))
  (defstrand make 2 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (defstrand make 3 (old old-1) (k k) (lk lk-0))
  (precedes ((1 1) (2 3)) ((2 4) (0 0)) ((3 1) (6 0)) ((3 2) (2 0))
    ((4 1) (0 0)) ((5 4) (0 0)) ((6 1) (1 0)) ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 1 7 make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-6 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt-1 k "init"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-0 old)) (stor lk-0 (cat pt-1 k "init"))
      (send (hash k)))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-6 k "init"))
      (load lk-0 (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-5 old-1)) (stor lk-0 (cat pt-6 k "init"))
      (send (hash k))))
  (label 1535)
  (parent 748)
  (unrealized (5 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 old-2 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand make 2 (old old-1) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old-2) (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((1 1) (6 2)) ((2 1) (3 3)) ((3 4) (0 0))
    ((4 1) (1 0)) ((4 2) (3 0)) ((5 1) (0 0)) ((6 4) (0 0))
    ((7 2) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation encryption-test (added-strand make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-4 old-1)) (stor lk-0 (cat pt-5 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-6 k1 "wrap")) (send (enc k k1)))
    ((load lk-1 (cat pt-7 old-2)) (stor lk-1 (cat pt-8 k "init"))
      (send (hash k))))
  (label 1536)
  (parent 748)
  (unrealized (6 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand make 2 (old old-1) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (precedes ((1 1) (2 0)) ((1 1) (6 2)) ((2 1) (3 3)) ((3 4) (0 0))
    ((4 1) (1 0)) ((4 2) (3 0)) ((5 1) (0 0)) ((6 4) (0 0))
    ((7 1) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (added-listener k) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-4 old-1)) (stor lk-0 (cat pt-5 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-6 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k)))
  (label 1537)
  (parent 748)
  (unrealized (6 3) (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (3 3)) ((3 4) (0 0)) ((4 1) (5 0))
    ((4 2) (3 0)) ((4 2) (6 0)) ((5 1) (1 0)) ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 7 4 make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk (cat pt-3 k "init")) (stor lk (cat pt-4 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-5 k1 "wrap")) (send (enc k k1))))
  (label 1538)
  (parent 751)
  (unrealized (6 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (3 3)) ((3 4) (0 0)) ((4 1) (5 0))
    ((4 1) (6 2)) ((4 2) (3 0)) ((5 1) (1 0)) ((6 4) (0 0))
    ((7 2) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation encryption-test (added-strand make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk (cat pt-3 k "init")) (stor lk (cat pt-4 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-5 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-6 old-1)) (stor lk-0 (cat pt-7 k "init"))
      (send (hash k))))
  (label 1539)
  (parent 751)
  (unrealized (6 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (precedes ((1 1) (2 0)) ((2 1) (3 3)) ((3 4) (0 0)) ((4 1) (5 0))
    ((4 1) (6 2)) ((4 2) (3 0)) ((5 1) (1 0)) ((6 4) (0 0))
    ((7 1) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (added-listener k) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk (cat pt-3 k "init")) (stor lk (cat pt-4 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-5 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k)))
  (label 1540)
  (parent 751)
  (unrealized (6 3) (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (3 3)) ((3 4) (0 0)) ((4 1) (5 0))
    ((4 2) (3 0)) ((4 2) (6 0)) ((5 1) (1 0)) ((5 1) (6 2))
    ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 7 4 make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk (cat pt-3 k "init")) (stor lk (cat pt-4 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-4 k "wrap"))
      (load lk (cat pt-5 k1 "wrap")) (send (enc k k1))))
  (label 1541)
  (parent 752)
  (unrealized (6 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (3 3)) ((3 4) (0 0)) ((4 1) (5 0))
    ((4 2) (3 0)) ((5 1) (1 0)) ((5 1) (6 2)) ((6 4) (0 0))
    ((7 2) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation encryption-test (added-strand make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk (cat pt-3 k "init")) (stor lk (cat pt-4 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-4 k "wrap"))
      (load lk (cat pt-5 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-6 old-1)) (stor lk-0 (cat pt-7 k "init"))
      (send (hash k))))
  (label 1542)
  (parent 752)
  (unrealized (6 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (precedes ((1 1) (2 0)) ((2 1) (3 3)) ((3 4) (0 0)) ((4 1) (5 0))
    ((4 2) (3 0)) ((5 1) (1 0)) ((5 1) (6 2)) ((6 4) (0 0))
    ((7 1) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (added-listener k) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk (cat pt-3 k "init")) (stor lk (cat pt-4 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-4 k "wrap"))
      (load lk (cat pt-5 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k)))
  (label 1543)
  (parent 752)
  (unrealized (6 3) (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (3 3)) ((2 1) (6 2)) ((3 4) (0 0))
    ((4 1) (5 0)) ((4 2) (3 0)) ((4 2) (6 0)) ((5 1) (1 0))
    ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 7 4 make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk (cat pt-3 k "init")) (stor lk (cat pt-4 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-5 k1 "wrap")) (send (enc k k1))))
  (label 1544)
  (parent 753)
  (unrealized (6 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (precedes ((1 1) (2 3)) ((1 1) (5 2)) ((2 4) (0 0)) ((3 1) (4 0))
    ((3 2) (2 0)) ((4 1) (6 0)) ((5 4) (0 0)) ((6 1) (1 0))
    ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 1 7 make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt-5 k "init")) (stor lk (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init"))
      (send (hash k)))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-2 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt k "wrap"))
      (load lk (cat pt-3 k1 "wrap")) (send (enc k k1)))
    ((load lk (cat pt-4 old-0)) (stor lk (cat pt-5 k "init"))
      (send (hash k))))
  (label 1545)
  (parent 753)
  (unrealized (5 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (3 3)) ((2 1) (6 2)) ((3 4) (0 0))
    ((4 1) (5 0)) ((4 2) (3 0)) ((5 1) (1 0)) ((6 4) (0 0))
    ((7 2) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation encryption-test (added-strand make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk (cat pt-3 k "init")) (stor lk (cat pt-4 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-5 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-6 old-1)) (stor lk-0 (cat pt-7 k "init"))
      (send (hash k))))
  (label 1546)
  (parent 753)
  (unrealized (6 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (precedes ((1 1) (2 0)) ((2 1) (3 3)) ((2 1) (6 2)) ((3 4) (0 0))
    ((4 1) (5 0)) ((4 2) (3 0)) ((5 1) (1 0)) ((6 4) (0 0))
    ((7 1) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (added-listener k) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk (cat pt-3 k "init")) (stor lk (cat pt-4 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-5 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k)))
  (label 1547)
  (parent 753)
  (unrealized (6 3) (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (6 2)) ((2 1) (3 3)) ((3 4) (0 0))
    ((4 1) (5 0)) ((4 2) (3 0)) ((4 2) (6 0)) ((5 1) (1 0))
    ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 7 4 make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk (cat pt-3 k "init")) (stor lk (cat pt-4 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-5 k1 "wrap")) (send (enc k k1))))
  (label 1548)
  (parent 754)
  (unrealized (6 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (precedes ((1 1) (2 3)) ((2 4) (0 0)) ((3 1) (4 0)) ((3 2) (2 0))
    ((4 1) (6 0)) ((5 4) (0 0)) ((6 1) (1 0)) ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 1 7 make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt-5 k "init")) (stor lk (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init"))
      (send (hash k)))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-2 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-5 k "init"))
      (load lk (cat pt-3 k1 "wrap")) (send (enc k k1)))
    ((load lk (cat pt-4 old-0)) (stor lk (cat pt-5 k "init"))
      (send (hash k))))
  (label 1549)
  (parent 754)
  (unrealized (5 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((1 1) (6 2)) ((2 1) (3 3)) ((3 4) (0 0))
    ((4 1) (5 0)) ((4 2) (3 0)) ((5 1) (1 0)) ((6 4) (0 0))
    ((7 2) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation encryption-test (added-strand make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk (cat pt-3 k "init")) (stor lk (cat pt-4 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-5 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-6 old-1)) (stor lk-0 (cat pt-7 k "init"))
      (send (hash k))))
  (label 1550)
  (parent 754)
  (unrealized (6 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (precedes ((1 1) (2 0)) ((1 1) (6 2)) ((2 1) (3 3)) ((3 4) (0 0))
    ((4 1) (5 0)) ((4 2) (3 0)) ((5 1) (1 0)) ((6 4) (0 0))
    ((7 1) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (added-listener k) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk (cat pt-3 k "init")) (stor lk (cat pt-4 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-5 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k)))
  (label 1551)
  (parent 754)
  (unrealized (6 3) (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (3 3)) ((2 1) (5 3)) ((3 4) (0 0))
    ((4 1) (1 0)) ((4 1) (5 2)) ((4 2) (3 0)) ((5 4) (0 0))
    ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation channel-test (displaced 7 2 set-wrap 2)
    (ch-msg lk (cat pt-6 k1 "wrap")) (5 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-4 old-1)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k))))
  (label 1552)
  (parent 756)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 cur mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur cur) (k k1) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (3 3)) ((3 4) (0 0)) ((4 1) (1 0))
    ((4 1) (5 2)) ((4 1) (7 0)) ((4 2) (3 0)) ((5 4) (0 0))
    ((6 2) (5 0)) ((7 1) (5 3)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 cur) (cat k1 "wrap"))
  (facts (neq cur "decrypt") (neq cur "wrap") (neq "init" "decrypt")
    (neq "init" "wrap"))
  (rule invShearsRule fact-set-decrypt-neq0 trRl_set-wrap-at-1
    gen-st-set-wrap-0 fact-set-wrap-neq0 trRl_make-at-0)
  (operation channel-test (added-strand set-wrap 2)
    (ch-msg lk (cat pt-4 k1 "wrap")) (5 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-5 old-1)) (stor lk-0 (cat pt-6 k "init"))
      (send (hash k)))
    ((load lk (cat pt-7 k1 cur)) (stor lk (cat pt-4 k1 "wrap"))))
  (label 1553)
  (parent 756)
  (unrealized (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k k1 skey)
    (lk locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (precedes ((1 1) (2 0)) ((2 1) (3 3)) ((3 4) (0 0)) ((4 1) (1 0))
    ((4 1) (6 0)) ((4 2) (3 0)) ((5 4) (0 0)) ((6 1) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation nonce-test (displaced 7 4 make 2) k (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k)))
  (label 1554)
  (parent 757)
  (unrealized (5 3) (6 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-1) (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (3 3)) ((3 4) (0 0)) ((4 1) (1 0))
    ((4 1) (5 2)) ((4 2) (3 0)) ((5 4) (0 0)) ((6 1) (5 0))
    ((7 1) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation nonce-test (added-strand make 2) k (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-5 old-1)) (stor lk-0 (cat pt-6 k "init"))))
  (label 1555)
  (parent 757)
  (unrealized (5 3) (6 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 cur mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (defstrand set-wrap 2 (cur cur) (k k1) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (3 3)) ((2 1) (5 2))
    ((3 4) (0 0)) ((4 1) (1 0)) ((4 2) (3 0)) ((4 2) (5 0))
    ((5 4) (0 0)) ((6 1) (5 3)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 cur) (cat k1 "wrap"))
  (facts (neq cur "decrypt") (neq cur "wrap") (neq "init" "decrypt")
    (neq "init" "wrap"))
  (rule invShearsRule fact-set-decrypt-neq0 trRl_set-wrap-at-1
    gen-st-set-wrap-0 fact-set-wrap-neq0 trRl_make-at-0)
  (operation channel-test (added-strand set-wrap 2)
    (ch-msg lk (cat pt-4 k1 "wrap")) (5 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((load lk (cat pt-5 k1 cur)) (stor lk (cat pt-4 k1 "wrap"))))
  (label 1556)
  (parent 758)
  (unrealized (6 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 cur mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (defstrand set-wrap 2 (cur cur) (k k1) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (3 3)) ((2 1) (5 2)) ((2 1) (6 0))
    ((3 4) (0 0)) ((4 1) (1 0)) ((4 2) (3 0)) ((4 2) (5 0))
    ((5 4) (0 0)) ((6 1) (5 3)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 cur) (cat k1 "wrap"))
  (facts (neq cur "decrypt") (neq cur "wrap") (neq "init" "decrypt")
    (neq "init" "wrap"))
  (rule invShearsRule fact-set-decrypt-neq0 trRl_set-wrap-at-1
    gen-st-set-wrap-0 fact-set-wrap-neq0 trRl_make-at-0)
  (operation channel-test (added-strand set-wrap 2)
    (ch-msg lk (cat pt-4 k1 "wrap")) (5 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((load lk (cat pt-5 k1 cur)) (stor lk (cat pt-4 k1 "wrap"))))
  (label 1557)
  (parent 758)
  (unrealized (6 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pval) (k skey)
    (lk locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (precedes ((1 1) (2 3)) ((1 1) (4 2)) ((2 4) (0 0)) ((3 1) (5 0))
    ((3 2) (2 0)) ((4 4) (0 0)) ((5 1) (1 0)) ((5 2) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation channel-test (displaced 6 1 set-wrap 2)
    (ch-msg lk (cat pt-4 k1 "wrap")) (4 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt-3 k "init")) (stor lk (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt k "wrap"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init"))
      (send (hash k))))
  (label 1558)
  (parent 759)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 cur mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand set-wrap 2 (cur cur) (k k1) (lk lk))
  (precedes ((1 1) (2 3)) ((1 1) (4 2)) ((2 4) (0 0)) ((3 1) (5 0))
    ((3 2) (2 0)) ((4 4) (0 0)) ((5 1) (1 0)) ((5 1) (6 0))
    ((5 2) (4 0)) ((6 1) (4 3)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 cur) (cat k1 "wrap"))
  (facts (neq cur "decrypt") (neq cur "wrap") (neq "init" "decrypt")
    (neq "init" "wrap"))
  (rule invShearsRule fact-set-decrypt-neq0 trRl_set-wrap-at-1
    gen-st-set-wrap-0 fact-set-wrap-neq0 trRl_make-at-0)
  (operation channel-test (added-strand set-wrap 2)
    (ch-msg lk (cat pt-2 k1 "wrap")) (4 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt-4 k "init")) (stor lk (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt k "wrap"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((load lk (cat pt-3 old-0)) (stor lk (cat pt-4 k "init"))
      (send (hash k)))
    ((load lk (cat pt-5 k1 cur)) (stor lk (cat pt-2 k1 "wrap"))))
  (label 1559)
  (parent 759)
  (unrealized (6 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 cur mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand set-wrap 2 (cur cur) (k k1) (lk lk))
  (precedes ((1 1) (2 3)) ((1 1) (4 2)) ((1 1) (6 0)) ((2 4) (0 0))
    ((3 1) (5 0)) ((3 2) (2 0)) ((4 4) (0 0)) ((5 1) (1 0))
    ((5 2) (4 0)) ((6 1) (4 3)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 cur) (cat k1 "wrap"))
  (facts (neq cur "decrypt") (neq cur "wrap") (neq "init" "decrypt")
    (neq "init" "wrap"))
  (rule invShearsRule fact-set-decrypt-neq0 trRl_set-wrap-at-1
    gen-st-set-wrap-0 fact-set-wrap-neq0 trRl_make-at-0)
  (operation channel-test (added-strand set-wrap 2)
    (ch-msg lk (cat pt-2 k1 "wrap")) (4 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt-4 k "init")) (stor lk (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt k "wrap"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((load lk (cat pt-3 old-0)) (stor lk (cat pt-4 k "init"))
      (send (hash k)))
    ((load lk (cat pt-5 k1 cur)) (stor lk (cat pt-2 k1 "wrap"))))
  (label 1560)
  (parent 759)
  (unrealized (6 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (3 3)) ((2 1) (5 2)) ((3 4) (0 0))
    ((4 1) (1 0)) ((4 2) (3 0)) ((5 4) (0 0)) ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation channel-test (displaced 7 2 set-wrap 2)
    (ch-msg lk (cat pt-6 k1 "wrap")) (5 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-4 old-1)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k))))
  (label 1561)
  (parent 760)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 cur mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur cur) (k k1) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (7 0)) ((2 1) (3 3)) ((2 1) (5 2))
    ((3 4) (0 0)) ((4 1) (1 0)) ((4 2) (3 0)) ((5 4) (0 0))
    ((6 2) (5 0)) ((7 1) (5 3)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 cur) (cat k1 "wrap"))
  (facts (neq cur "decrypt") (neq cur "wrap") (neq "init" "decrypt")
    (neq "init" "wrap"))
  (rule invShearsRule fact-set-decrypt-neq0 trRl_set-wrap-at-1
    gen-st-set-wrap-0 fact-set-wrap-neq0 trRl_make-at-0)
  (operation channel-test (added-strand set-wrap 2)
    (ch-msg lk (cat pt-4 k1 "wrap")) (5 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-5 old-1)) (stor lk-0 (cat pt-6 k "init"))
      (send (hash k)))
    ((load lk (cat pt-7 k1 cur)) (stor lk (cat pt-4 k1 "wrap"))))
  (label 1562)
  (parent 760)
  (unrealized (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 cur mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur cur) (k k1) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (3 3)) ((2 1) (5 2)) ((2 1) (7 0))
    ((3 4) (0 0)) ((4 1) (1 0)) ((4 2) (3 0)) ((5 4) (0 0))
    ((6 2) (5 0)) ((7 1) (5 3)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 cur) (cat k1 "wrap"))
  (facts (neq cur "decrypt") (neq cur "wrap") (neq "init" "decrypt")
    (neq "init" "wrap"))
  (rule invShearsRule fact-set-decrypt-neq0 trRl_set-wrap-at-1
    gen-st-set-wrap-0 fact-set-wrap-neq0 trRl_make-at-0)
  (operation channel-test (added-strand set-wrap 2)
    (ch-msg lk (cat pt-4 k1 "wrap")) (5 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-5 old-1)) (stor lk-0 (cat pt-6 k "init"))
      (send (hash k)))
    ((load lk (cat pt-7 k1 cur)) (stor lk (cat pt-4 k1 "wrap"))))
  (label 1563)
  (parent 760)
  (unrealized (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k k1 skey)
    (lk locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (precedes ((1 1) (2 0)) ((2 1) (3 3)) ((2 1) (5 2)) ((3 4) (0 0))
    ((4 1) (1 0)) ((4 1) (6 0)) ((4 2) (3 0)) ((5 4) (0 0))
    ((6 1) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation nonce-test (displaced 7 4 make 2) k (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k)))
  (label 1564)
  (parent 761)
  (unrealized (5 3) (6 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-1) (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (3 3)) ((2 1) (5 2)) ((3 4) (0 0))
    ((4 1) (1 0)) ((4 2) (3 0)) ((5 4) (0 0)) ((6 1) (5 0))
    ((7 1) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation nonce-test (added-strand make 2) k (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-5 old-1)) (stor lk-0 (cat pt-6 k "init"))))
  (label 1565)
  (parent 761)
  (unrealized (5 3) (6 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 cur mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (defstrand set-wrap 2 (cur cur) (k k1) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (5 2)) ((1 1) (6 0)) ((2 1) (3 3))
    ((3 4) (0 0)) ((4 1) (1 0)) ((4 2) (3 0)) ((4 2) (5 0))
    ((5 4) (0 0)) ((6 1) (5 3)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 cur) (cat k1 "wrap"))
  (facts (neq cur "decrypt") (neq cur "wrap") (neq "init" "decrypt")
    (neq "init" "wrap"))
  (rule invShearsRule fact-set-decrypt-neq0 trRl_set-wrap-at-1
    gen-st-set-wrap-0 fact-set-wrap-neq0 trRl_make-at-0)
  (operation channel-test (added-strand set-wrap 2)
    (ch-msg lk (cat pt-4 k1 "wrap")) (5 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((load lk (cat pt-5 k1 cur)) (stor lk (cat pt-4 k1 "wrap"))))
  (label 1566)
  (parent 762)
  (unrealized (6 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 cur mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (defstrand set-wrap 2 (cur cur) (k k1) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (5 2)) ((2 1) (3 3)) ((3 4) (0 0))
    ((4 1) (1 0)) ((4 1) (6 0)) ((4 2) (3 0)) ((4 2) (5 0))
    ((5 4) (0 0)) ((6 1) (5 3)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 cur) (cat k1 "wrap"))
  (facts (neq cur "decrypt") (neq cur "wrap") (neq "init" "decrypt")
    (neq "init" "wrap"))
  (rule invShearsRule fact-set-decrypt-neq0 trRl_set-wrap-at-1
    gen-st-set-wrap-0 fact-set-wrap-neq0 trRl_make-at-0)
  (operation channel-test (added-strand set-wrap 2)
    (ch-msg lk (cat pt-4 k1 "wrap")) (5 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((load lk (cat pt-5 k1 cur)) (stor lk (cat pt-4 k1 "wrap"))))
  (label 1567)
  (parent 762)
  (unrealized (6 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 cur mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (defstrand set-wrap 2 (cur cur) (k k1) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (5 2)) ((2 1) (3 3)) ((2 1) (6 0))
    ((3 4) (0 0)) ((4 1) (1 0)) ((4 2) (3 0)) ((4 2) (5 0))
    ((5 4) (0 0)) ((6 1) (5 3)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 cur) (cat k1 "wrap"))
  (facts (neq cur "decrypt") (neq cur "wrap") (neq "init" "decrypt")
    (neq "init" "wrap"))
  (rule shearsRule invShearsRule fact-set-decrypt-neq0
    trRl_set-wrap-at-1 gen-st-set-wrap-0 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation channel-test (added-strand set-wrap 2)
    (ch-msg lk (cat pt-4 k1 "wrap")) (5 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((load lk (cat pt-5 k1 cur)) (stor lk (cat pt-4 k1 "wrap"))))
  (label 1568)
  (parent 762)
  (unrealized (6 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pval) (k skey)
    (lk locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (precedes ((1 1) (2 3)) ((1 1) (4 3)) ((2 4) (0 0)) ((3 1) (5 0))
    ((3 2) (2 0)) ((4 4) (0 0)) ((5 1) (1 0)) ((5 2) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation channel-test (displaced 6 1 set-wrap 2)
    (ch-msg lk (cat pt-4 k1 "wrap")) (4 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt-3 k "init")) (stor lk (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init"))
      (send (hash k))))
  (label 1569)
  (parent 763)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 cur mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand set-wrap 2 (cur cur) (k k1) (lk lk))
  (precedes ((1 1) (2 3)) ((2 4) (0 0)) ((3 1) (5 0)) ((3 2) (2 0))
    ((4 4) (0 0)) ((5 1) (1 0)) ((5 1) (6 0)) ((5 2) (4 0))
    ((6 1) (4 3)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 cur) (cat k1 "wrap"))
  (facts (neq cur "decrypt") (neq cur "wrap") (neq "init" "decrypt")
    (neq "init" "wrap"))
  (rule invShearsRule fact-set-decrypt-neq0 trRl_set-wrap-at-1
    gen-st-set-wrap-0 fact-set-wrap-neq0 trRl_make-at-0)
  (operation channel-test (added-strand set-wrap 2)
    (ch-msg lk (cat pt-2 k1 "wrap")) (4 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt-4 k "init")) (stor lk (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-4 k "init"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((load lk (cat pt-3 old-0)) (stor lk (cat pt-4 k "init"))
      (send (hash k)))
    ((load lk (cat pt-5 k1 cur)) (stor lk (cat pt-2 k1 "wrap"))))
  (label 1570)
  (parent 763)
  (unrealized (6 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 cur mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand set-wrap 2 (cur cur) (k k1) (lk lk))
  (precedes ((1 1) (2 3)) ((2 4) (0 0)) ((3 1) (5 0)) ((3 1) (6 0))
    ((3 2) (2 0)) ((4 4) (0 0)) ((5 1) (1 0)) ((5 2) (4 0))
    ((6 1) (4 3)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 cur) (cat k1 "wrap"))
  (facts (neq cur "decrypt") (neq cur "wrap") (neq "init" "decrypt")
    (neq "init" "wrap"))
  (rule invShearsRule fact-set-decrypt-neq0 trRl_set-wrap-at-1
    gen-st-set-wrap-0 fact-set-wrap-neq0 trRl_make-at-0)
  (operation channel-test (added-strand set-wrap 2)
    (ch-msg lk (cat pt-2 k1 "wrap")) (4 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt-4 k "init")) (stor lk (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-4 k "init"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((load lk (cat pt-3 old-0)) (stor lk (cat pt-4 k "init"))
      (send (hash k)))
    ((load lk (cat pt-5 k1 cur)) (stor lk (cat pt-2 k1 "wrap"))))
  (label 1571)
  (parent 763)
  (unrealized (6 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 cur mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand set-wrap 2 (cur cur) (k k1) (lk lk))
  (precedes ((1 1) (2 3)) ((1 1) (6 0)) ((2 4) (0 0)) ((3 1) (5 0))
    ((3 2) (2 0)) ((4 4) (0 0)) ((5 1) (1 0)) ((5 2) (4 0))
    ((6 1) (4 3)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 cur) (cat k1 "wrap"))
  (facts (neq cur "decrypt") (neq cur "wrap") (neq "init" "decrypt")
    (neq "init" "wrap"))
  (rule shearsRule invShearsRule fact-set-decrypt-neq0
    trRl_set-wrap-at-1 gen-st-set-wrap-0 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation channel-test (added-strand set-wrap 2)
    (ch-msg lk (cat pt-2 k1 "wrap")) (4 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt-4 k "init")) (stor lk (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-4 k "init"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((load lk (cat pt-3 old-0)) (stor lk (cat pt-4 k "init"))
      (send (hash k)))
    ((load lk (cat pt-5 k1 cur)) (stor lk (cat pt-2 k1 "wrap"))))
  (label 1572)
  (parent 763)
  (unrealized (6 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((1 1) (5 2)) ((2 1) (3 3)) ((2 1) (5 3))
    ((3 4) (0 0)) ((4 1) (1 0)) ((4 2) (3 0)) ((5 4) (0 0))
    ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation channel-test (displaced 7 2 set-wrap 2)
    (ch-msg lk (cat pt-6 k1 "wrap")) (5 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-4 old-1)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k))))
  (label 1573)
  (parent 764)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 cur mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur cur) (k k1) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (5 2)) ((1 1) (7 0)) ((2 1) (3 3))
    ((3 4) (0 0)) ((4 1) (1 0)) ((4 2) (3 0)) ((5 4) (0 0))
    ((6 2) (5 0)) ((7 1) (5 3)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 cur) (cat k1 "wrap"))
  (facts (neq cur "decrypt") (neq cur "wrap") (neq "init" "decrypt")
    (neq "init" "wrap"))
  (rule invShearsRule fact-set-decrypt-neq0 trRl_set-wrap-at-1
    gen-st-set-wrap-0 fact-set-wrap-neq0 trRl_make-at-0)
  (operation channel-test (added-strand set-wrap 2)
    (ch-msg lk (cat pt-4 k1 "wrap")) (5 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-5 old-1)) (stor lk-0 (cat pt-6 k "init"))
      (send (hash k)))
    ((load lk (cat pt-7 k1 cur)) (stor lk (cat pt-4 k1 "wrap"))))
  (label 1574)
  (parent 764)
  (unrealized (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 cur mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur cur) (k k1) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (5 2)) ((2 1) (3 3)) ((3 4) (0 0))
    ((4 1) (1 0)) ((4 1) (7 0)) ((4 2) (3 0)) ((5 4) (0 0))
    ((6 2) (5 0)) ((7 1) (5 3)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 cur) (cat k1 "wrap"))
  (facts (neq cur "decrypt") (neq cur "wrap") (neq "init" "decrypt")
    (neq "init" "wrap"))
  (rule invShearsRule fact-set-decrypt-neq0 trRl_set-wrap-at-1
    gen-st-set-wrap-0 fact-set-wrap-neq0 trRl_make-at-0)
  (operation channel-test (added-strand set-wrap 2)
    (ch-msg lk (cat pt-4 k1 "wrap")) (5 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-5 old-1)) (stor lk-0 (cat pt-6 k "init"))
      (send (hash k)))
    ((load lk (cat pt-7 k1 cur)) (stor lk (cat pt-4 k1 "wrap"))))
  (label 1575)
  (parent 764)
  (unrealized (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 cur mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur cur) (k k1) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (5 2)) ((2 1) (3 3)) ((2 1) (7 0))
    ((3 4) (0 0)) ((4 1) (1 0)) ((4 2) (3 0)) ((5 4) (0 0))
    ((6 2) (5 0)) ((7 1) (5 3)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 cur) (cat k1 "wrap"))
  (facts (neq cur "decrypt") (neq cur "wrap") (neq "init" "decrypt")
    (neq "init" "wrap"))
  (rule shearsRule invShearsRule fact-set-decrypt-neq0
    trRl_set-wrap-at-1 gen-st-set-wrap-0 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation channel-test (added-strand set-wrap 2)
    (ch-msg lk (cat pt-4 k1 "wrap")) (5 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-5 old-1)) (stor lk-0 (cat pt-6 k "init"))
      (send (hash k)))
    ((load lk (cat pt-7 k1 cur)) (stor lk (cat pt-4 k1 "wrap"))))
  (label 1576)
  (parent 764)
  (unrealized (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k k1 skey)
    (lk locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (precedes ((1 1) (2 0)) ((1 1) (5 2)) ((2 1) (3 3)) ((3 4) (0 0))
    ((4 1) (1 0)) ((4 1) (6 0)) ((4 2) (3 0)) ((5 4) (0 0))
    ((6 1) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation nonce-test (displaced 7 4 make 2) k (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k)))
  (label 1577)
  (parent 765)
  (unrealized (5 3) (6 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-1) (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((1 1) (5 2)) ((2 1) (3 3)) ((3 4) (0 0))
    ((4 1) (1 0)) ((4 2) (3 0)) ((5 4) (0 0)) ((6 1) (5 0))
    ((7 1) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation nonce-test (added-strand make 2) k (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-5 old-1)) (stor lk-0 (cat pt-6 k "init"))))
  (label 1578)
  (parent 765)
  (unrealized (5 3) (6 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (defstrand set-decrypt 2 (cur "wrap") (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (0 0)) ((3 4) (0 0)) ((4 1) (5 0))
    ((4 2) (3 0)) ((5 1) (3 3)) ((5 1) (7 0)) ((6 1) (0 0))
    ((7 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "wrap" "decrypt") (neq "wrap" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-decrypt-at-1 fact-set-decrypt-neq0 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation nonce-test (added-strand set-decrypt 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-2 k "wrap")) (ch-msg lk-0 (cat pt-4 k "init"))
    (ch-msg lk-1 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt-4 k "init"))
      (load lk-0 (cat pt-2 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt-2 k "wrap")))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init")))
    ((load lk-0 (cat pt-2 k "wrap"))
      (stor lk-0 (cat pt-7 k "decrypt"))))
  (label 1579)
  (parent 769)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (defstrand set-wrap 2 (cur "wrap") (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (0 0)) ((3 4) (0 0)) ((4 1) (5 0))
    ((4 2) (3 0)) ((5 1) (3 3)) ((5 1) (7 0)) ((6 1) (0 0))
    ((7 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "wrap" "decrypt") (neq "wrap" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule fact-set-decrypt-neq0 trRl_set-wrap-at-1 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation nonce-test (added-strand set-wrap 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-2 k "wrap")) (ch-msg lk-0 (cat pt-4 k "init"))
    (ch-msg lk-1 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt-4 k "init"))
      (load lk-0 (cat pt-2 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt-2 k "wrap")))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init")))
    ((load lk-0 (cat pt-2 k "wrap")) (stor lk-0 (cat pt-7 k "wrap"))))
  (label 1580)
  (parent 769)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (defstrand set-wrap 2 (cur "wrap") (k k) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (7 0)) ((3 4) (0 0)) ((4 1) (5 0))
    ((4 2) (3 0)) ((5 1) (3 3)) ((6 1) (0 0)) ((7 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "wrap" "decrypt") (neq "wrap" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule fact-set-decrypt-neq0 trRl_set-wrap-at-1 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation nonce-test (added-strand set-wrap 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-2 k "wrap")) (ch-msg lk-0 (cat pt-4 k "init"))
    (ch-msg lk-1 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt-4 k "init"))
      (load lk-0 (cat pt-2 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt-2 k "wrap")))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init")))
    ((load lk (cat pt-1 k "wrap")) (stor lk (cat pt-7 k "wrap"))))
  (label 1581)
  (parent 769)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-1))
  (precedes ((1 1) (2 0)) ((2 1) (0 0)) ((3 4) (0 0)) ((4 1) (5 0))
    ((4 2) (3 0)) ((5 1) (3 3)) ((6 1) (7 2)) ((7 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-2 k "wrap")) (ch-msg lk-0 (cat pt-4 k "init"))
    (ch-msg lk-1 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt-4 k "init"))
      (load lk-0 (cat pt-2 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt-2 k "wrap")))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk-1 (cat pt-6 k "init"))
      (load lk-1 (cat pt-7 k1 "wrap")) (send (enc k k1))))
  (label 1582)
  (parent 769)
  (unrealized (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (defstrand set-decrypt 2 (cur "wrap") (k k) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (7 0)) ((3 4) (0 0)) ((4 1) (5 0))
    ((4 2) (3 0)) ((5 1) (3 3)) ((6 1) (0 0)) ((7 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "wrap" "decrypt") (neq "wrap" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-decrypt-at-1 fact-set-decrypt-neq0 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation nonce-test (added-strand set-decrypt 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-2 k "wrap")) (ch-msg lk-0 (cat pt-4 k "init"))
    (ch-msg lk-1 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt-4 k "init"))
      (load lk-0 (cat pt-2 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt-2 k "wrap")))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init")))
    ((load lk (cat pt-1 k "wrap")) (stor lk (cat pt-7 k "decrypt"))))
  (label 1583)
  (parent 769)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (0 0)) ((3 4) (0 0)) ((4 1) (5 0))
    ((4 2) (3 0)) ((5 1) (3 3)) ((5 1) (7 2)) ((6 1) (0 0))
    ((7 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-2 k "wrap")) (ch-msg lk-0 (cat pt-4 k "init"))
    (ch-msg lk-1 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt-4 k "init"))
      (load lk-0 (cat pt-2 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt-2 k "wrap")))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-2 k "wrap"))
      (load lk-0 (cat pt-7 k1 "wrap")) (send (enc k k1))))
  (label 1584)
  (parent 769)
  (unrealized (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (0 0)) ((3 4) (0 0)) ((4 1) (5 0))
    ((4 1) (7 2)) ((4 2) (3 0)) ((5 1) (3 3)) ((6 1) (0 0))
    ((7 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-2 k "wrap")) (ch-msg lk-0 (cat pt-4 k "init"))
    (ch-msg lk-1 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt-4 k "init"))
      (load lk-0 (cat pt-2 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt-2 k "wrap")))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-4 k "init"))
      (load lk-0 (cat pt-7 k1 "wrap")) (send (enc k k1))))
  (label 1585)
  (parent 769)
  (unrealized (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (7 2)) ((3 4) (0 0)) ((4 1) (5 0))
    ((4 2) (3 0)) ((5 1) (3 3)) ((6 1) (0 0)) ((7 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-2 k "wrap")) (ch-msg lk-0 (cat pt-4 k "init"))
    (ch-msg lk-1 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt-4 k "init"))
      (load lk-0 (cat pt-2 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt-2 k "wrap")))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-7 k1 "wrap")) (send (enc k k1))))
  (label 1586)
  (parent 769)
  (unrealized (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (7 2)) ((2 1) (0 0)) ((3 4) (0 0))
    ((4 1) (5 0)) ((4 2) (3 0)) ((5 1) (3 3)) ((6 1) (0 0))
    ((7 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-2 k "wrap")) (ch-msg lk-0 (cat pt-4 k "init"))
    (ch-msg lk-1 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt-4 k "init"))
      (load lk-0 (cat pt-2 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt-2 k "wrap")))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-7 k1 "wrap")) (send (enc k k1))))
  (label 1587)
  (parent 769)
  (unrealized (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (0 0)) ((3 4) (0 0)) ((4 1) (5 0))
    ((4 2) (3 0)) ((4 2) (6 0)) ((5 1) (3 3)) ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 7 4 make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt-4 k "init"))
      (load lk-0 (cat pt-2 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt-2 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-4 k "init"))
      (load lk-0 (cat pt-5 k1 "wrap")) (send (enc k k1))))
  (label 1588)
  (parent 771)
  (unrealized (6 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (precedes ((1 1) (0 0)) ((2 4) (0 0)) ((3 1) (4 0)) ((3 1) (5 2))
    ((3 2) (2 0)) ((4 1) (2 3)) ((5 4) (0 0)) ((6 1) (1 0))
    ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 1 7 make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-5 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-2 k "init"))
      (load lk (cat pt-0 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-1 old)) (stor lk (cat pt-2 k "init"))
      (send (hash k)))
    ((load lk (cat pt-2 k "init")) (stor lk (cat pt-0 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-2 k "init"))
      (load lk (cat pt-3 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-4 old-0)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k))))
  (label 1589)
  (parent 771)
  (unrealized (5 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((2 1) (0 0)) ((3 4) (0 0)) ((4 1) (5 0))
    ((4 1) (6 2)) ((4 2) (3 0)) ((5 1) (3 3)) ((6 4) (0 0))
    ((7 2) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation encryption-test (added-strand make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt-4 k "init"))
      (load lk-0 (cat pt-2 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt-2 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-4 k "init"))
      (load lk-0 (cat pt-5 k1 "wrap")) (send (enc k k1)))
    ((load lk-1 (cat pt-6 old-1)) (stor lk-1 (cat pt-7 k "init"))
      (send (hash k))))
  (label 1590)
  (parent 771)
  (unrealized (6 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (deflistener k)
  (precedes ((1 1) (2 0)) ((2 1) (0 0)) ((3 4) (0 0)) ((4 1) (5 0))
    ((4 1) (6 2)) ((4 2) (3 0)) ((5 1) (3 3)) ((6 4) (0 0))
    ((7 1) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (added-listener k) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt-4 k "init"))
      (load lk-0 (cat pt-2 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt-2 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-4 k "init"))
      (load lk-0 (cat pt-5 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k)))
  (label 1591)
  (parent 771)
  (unrealized (6 3) (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (0 0)) ((3 4) (0 0)) ((4 1) (5 0))
    ((4 2) (3 0)) ((4 2) (6 0)) ((5 1) (3 3)) ((5 1) (6 2))
    ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 7 4 make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt-4 k "init"))
      (load lk-0 (cat pt-2 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt-2 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-2 k "wrap"))
      (load lk-0 (cat pt-5 k1 "wrap")) (send (enc k k1))))
  (label 1592)
  (parent 773)
  (unrealized (6 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (precedes ((1 1) (0 0)) ((2 4) (0 0)) ((3 1) (4 0)) ((3 2) (2 0))
    ((4 1) (2 3)) ((4 1) (5 2)) ((5 4) (0 0)) ((6 1) (1 0))
    ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 1 7 make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-5 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-2 k "init"))
      (load lk (cat pt-0 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-1 old)) (stor lk (cat pt-2 k "init"))
      (send (hash k)))
    ((load lk (cat pt-2 k "init")) (stor lk (cat pt-0 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "wrap"))
      (load lk (cat pt-3 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-4 old-0)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k))))
  (label 1593)
  (parent 773)
  (unrealized (5 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk-0))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((2 1) (0 0)) ((3 4) (0 0)) ((4 1) (5 0))
    ((4 2) (3 0)) ((5 1) (3 3)) ((5 1) (6 2)) ((6 4) (0 0))
    ((7 2) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation encryption-test (added-strand make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt-4 k "init"))
      (load lk-0 (cat pt-2 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt-2 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-2 k "wrap"))
      (load lk-0 (cat pt-5 k1 "wrap")) (send (enc k k1)))
    ((load lk-1 (cat pt-6 old-1)) (stor lk-1 (cat pt-7 k "init"))
      (send (hash k))))
  (label 1594)
  (parent 773)
  (unrealized (6 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk-0))
  (deflistener k)
  (precedes ((1 1) (2 0)) ((2 1) (0 0)) ((3 4) (0 0)) ((4 1) (5 0))
    ((4 2) (3 0)) ((5 1) (3 3)) ((5 1) (6 2)) ((6 4) (0 0))
    ((7 1) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (added-listener k) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt-4 k "init"))
      (load lk-0 (cat pt-2 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt-2 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-2 k "wrap"))
      (load lk-0 (cat pt-5 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k)))
  (label 1595)
  (parent 773)
  (unrealized (6 3) (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (6 2)) ((3 4) (0 0)) ((4 1) (5 0))
    ((4 2) (3 0)) ((4 2) (6 0)) ((5 1) (3 3)) ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 7 4 make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt-4 k "init"))
      (load lk-0 (cat pt-2 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt-2 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-5 k1 "wrap")) (send (enc k k1))))
  (label 1596)
  (parent 774)
  (unrealized (6 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (precedes ((1 1) (5 2)) ((2 4) (0 0)) ((3 1) (4 0)) ((3 2) (2 0))
    ((4 1) (2 3)) ((5 4) (0 0)) ((6 1) (1 0)) ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 1 7 make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-5 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-2 k "init"))
      (load lk (cat pt-0 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-1 old)) (stor lk (cat pt-2 k "init"))
      (send (hash k)))
    ((load lk (cat pt-2 k "init")) (stor lk (cat pt-0 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt k "wrap"))
      (load lk-0 (cat pt-3 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-4 old-0)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k))))
  (label 1597)
  (parent 774)
  (unrealized (5 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((2 1) (6 2)) ((3 4) (0 0)) ((4 1) (5 0))
    ((4 2) (3 0)) ((5 1) (3 3)) ((6 4) (0 0)) ((7 2) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation encryption-test (added-strand make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt-4 k "init"))
      (load lk-0 (cat pt-2 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt-2 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-5 k1 "wrap")) (send (enc k k1)))
    ((load lk-1 (cat pt-6 old-1)) (stor lk-1 (cat pt-7 k "init"))
      (send (hash k))))
  (label 1598)
  (parent 774)
  (unrealized (6 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (precedes ((1 1) (2 0)) ((2 1) (6 2)) ((3 4) (0 0)) ((4 1) (5 0))
    ((4 2) (3 0)) ((5 1) (3 3)) ((6 4) (0 0)) ((7 1) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (added-listener k) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt-4 k "init"))
      (load lk-0 (cat pt-2 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt-2 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-5 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k)))
  (label 1599)
  (parent 774)
  (unrealized (6 3) (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (6 2)) ((2 1) (0 0)) ((3 4) (0 0))
    ((4 1) (5 0)) ((4 2) (3 0)) ((4 2) (6 0)) ((5 1) (3 3))
    ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 7 4 make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt-4 k "init"))
      (load lk-0 (cat pt-2 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt-2 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-5 k1 "wrap")) (send (enc k k1))))
  (label 1600)
  (parent 775)
  (unrealized (6 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (precedes ((1 1) (0 0)) ((2 4) (0 0)) ((3 1) (4 0)) ((3 2) (2 0))
    ((4 1) (2 3)) ((5 4) (0 0)) ((6 1) (1 0)) ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 1 7 make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-5 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-2 k "init"))
      (load lk (cat pt-0 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-1 old)) (stor lk (cat pt-2 k "init"))
      (send (hash k)))
    ((load lk (cat pt-2 k "init")) (stor lk (cat pt-0 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-5 k "init"))
      (load lk-0 (cat pt-3 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-4 old-0)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k))))
  (label 1601)
  (parent 775)
  (unrealized (5 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((1 1) (6 2)) ((2 1) (0 0)) ((3 4) (0 0))
    ((4 1) (5 0)) ((4 2) (3 0)) ((5 1) (3 3)) ((6 4) (0 0))
    ((7 2) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation encryption-test (added-strand make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt-4 k "init"))
      (load lk-0 (cat pt-2 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt-2 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-5 k1 "wrap")) (send (enc k k1)))
    ((load lk-1 (cat pt-6 old-1)) (stor lk-1 (cat pt-7 k "init"))
      (send (hash k))))
  (label 1602)
  (parent 775)
  (unrealized (6 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (precedes ((1 1) (2 0)) ((1 1) (6 2)) ((2 1) (0 0)) ((3 4) (0 0))
    ((4 1) (5 0)) ((4 2) (3 0)) ((5 1) (3 3)) ((6 4) (0 0))
    ((7 1) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (added-listener k) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt-4 k "init"))
      (load lk-0 (cat pt-2 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt-2 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-5 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k)))
  (label 1603)
  (parent 775)
  (unrealized (6 3) (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 cur mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (defstrand make 3 (old old-1) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur cur) (k k1) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (3 3)) ((3 4) (0 0)) ((4 2) (3 0))
    ((5 4) (0 0)) ((6 1) (1 0)) ((6 1) (3 2)) ((6 1) (7 0))
    ((6 2) (5 0)) ((7 1) (5 3)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 cur) (cat k1 "wrap"))
  (facts (neq cur "decrypt") (neq cur "wrap") (neq "init" "decrypt")
    (neq "init" "wrap"))
  (rule invShearsRule fact-set-decrypt-neq0 trRl_set-wrap-at-1
    gen-st-set-wrap-0 fact-set-wrap-neq0 trRl_make-at-0)
  (operation channel-test (added-strand set-wrap 2)
    (ch-msg lk-0 (cat pt-4 k1 "wrap")) (5 3))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt old)) (stor lk-0 (cat pt-0 k "init")))
    ((load lk-0 (cat pt-0 k "init")) (stor lk-0 (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt-6 k "init"))
      (load lk-0 (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-6 k "init"))
      (load lk-0 (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-5 old-1)) (stor lk-0 (cat pt-6 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-7 k1 cur)) (stor lk-0 (cat pt-4 k1 "wrap"))))
  (label 1604)
  (parent 816)
  (unrealized (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 cur mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur cur) (k k1) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (3 3)) ((3 4) (0 0)) ((4 1) (5 2))
    ((4 1) (7 0)) ((4 2) (3 0)) ((5 4) (0 0)) ((6 1) (1 0))
    ((6 1) (3 2)) ((6 2) (5 0)) ((7 1) (5 3)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 cur) (cat k1 "wrap"))
  (facts (neq cur "decrypt") (neq cur "wrap") (neq "init" "decrypt")
    (neq "init" "wrap"))
  (rule invShearsRule fact-set-decrypt-neq0 trRl_set-wrap-at-1
    gen-st-set-wrap-0 fact-set-wrap-neq0 trRl_make-at-0)
  (operation channel-test (added-strand set-wrap 2)
    (ch-msg lk (cat pt-4 k1 "wrap")) (5 3))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt old)) (stor lk-0 (cat pt-0 k "init")))
    ((load lk-0 (cat pt-0 k "init")) (stor lk-0 (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt-6 k "init"))
      (load lk-0 (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-5 old-1)) (stor lk-0 (cat pt-6 k "init"))
      (send (hash k)))
    ((load lk (cat pt-7 k1 cur)) (stor lk (cat pt-4 k1 "wrap"))))
  (label 1605)
  (parent 820)
  (unrealized (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 cur mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur cur) (k k1) (lk lk))
  (precedes ((1 1) (3 3)) ((2 1) (3 2)) ((2 1) (6 0)) ((3 4) (0 0))
    ((4 1) (5 2)) ((4 1) (7 0)) ((4 2) (3 0)) ((5 4) (0 0))
    ((6 1) (1 0)) ((6 2) (5 0)) ((7 1) (5 3)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 cur) (cat k1 "wrap"))
  (facts (neq cur "decrypt") (neq cur "wrap") (neq "init" "decrypt")
    (neq "init" "wrap"))
  (rule invShearsRule fact-set-decrypt-neq0 trRl_set-wrap-at-1
    gen-st-set-wrap-0 fact-set-wrap-neq0 trRl_make-at-0)
  (operation channel-test (added-strand set-wrap 2)
    (ch-msg lk (cat pt-4 k1 "wrap")) (5 3))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-6 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((load lk-0 (cat pt-0 old)) (stor lk-0 (cat pt-1 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt-1 k "init"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-5 old-1)) (stor lk-0 (cat pt-6 k "init"))
      (send (hash k)))
    ((load lk (cat pt-7 k1 cur)) (stor lk (cat pt-4 k1 "wrap"))))
  (label 1606)
  (parent 821)
  (unrealized (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k skey) (lk locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (4 3)) ((2 1) (6 3)) ((3 1) (1 0))
    ((3 1) (4 2)) ((4 4) (0 0)) ((5 1) (2 0)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule invShearsRule)
  (operation channel-test (displaced 7 2 set-wrap 2)
    (ch-msg lk-0 (cat pt-6 k1 "wrap")) (6 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-4 old-1)) (stor lk (cat pt-5 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-5 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k))))
  (label 1607)
  (parent 822)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k skey) (lk locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (4 3)) ((2 1) (6 3)) ((3 1) (1 0))
    ((3 1) (4 2)) ((4 4) (0 0)) ((5 1) (3 0)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule invShearsRule)
  (operation channel-test (displaced 7 2 set-wrap 2)
    (ch-msg lk-0 (cat pt-6 k1 "wrap")) (6 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-4 old-1)) (stor lk (cat pt-5 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-5 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k))))
  (label 1608)
  (parent 822)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 cur mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (defstrand set-wrap 2 (cur cur) (k k1) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (4 3)) ((3 1) (1 0)) ((3 1) (4 2))
    ((4 4) (0 0)) ((5 1) (7 0)) ((5 2) (4 0)) ((5 2) (6 0))
    ((6 4) (0 0)) ((7 1) (6 3)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 cur) (cat k1 "wrap"))
  (facts (neq cur "decrypt") (neq cur "wrap") (neq "init" "decrypt")
    (neq "init" "wrap"))
  (rule invShearsRule fact-set-decrypt-neq0 trRl_set-wrap-at-1
    gen-st-set-wrap-0 fact-set-wrap-neq0 trRl_make-at-0)
  (operation channel-test (added-strand set-wrap 2)
    (ch-msg lk-0 (cat pt-6 k1 "wrap")) (6 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-4 old-1)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-5 k "init"))
      (load lk-0 (cat pt-6 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-7 k1 cur)) (stor lk-0 (cat pt-6 k1 "wrap"))))
  (label 1609)
  (parent 822)
  (unrealized (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 cur mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk-0))
  (defstrand make 3 (old old-1) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur cur) (k k1) (lk lk-0))
  (precedes ((1 1) (2 0)) ((1 1) (7 0)) ((2 1) (3 3)) ((2 1) (5 2))
    ((3 4) (0 0)) ((4 2) (3 0)) ((5 4) (0 0)) ((6 1) (1 0))
    ((6 1) (3 2)) ((6 2) (5 0)) ((7 1) (5 3)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 cur) (cat k1 "wrap"))
  (facts (neq cur "decrypt") (neq cur "wrap") (neq "init" "decrypt")
    (neq "init" "wrap"))
  (rule invShearsRule fact-set-decrypt-neq0 trRl_set-wrap-at-1
    gen-st-set-wrap-0 fact-set-wrap-neq0 trRl_make-at-0)
  (operation channel-test (added-strand set-wrap 2)
    (ch-msg lk-0 (cat pt-4 k1 "wrap")) (5 3))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt old)) (stor lk-0 (cat pt-0 k "init")))
    ((load lk-0 (cat pt-0 k "init")) (stor lk-0 (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt-6 k "init"))
      (load lk-0 (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-1 k "wrap"))
      (load lk-0 (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-5 old-1)) (stor lk-0 (cat pt-6 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-7 k1 cur)) (stor lk-0 (cat pt-4 k1 "wrap"))))
  (label 1610)
  (parent 825)
  (unrealized (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 cur mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk-0))
  (defstrand make 3 (old old-1) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur cur) (k k1) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (3 3)) ((2 1) (5 2)) ((2 1) (7 0))
    ((3 4) (0 0)) ((4 2) (3 0)) ((5 4) (0 0)) ((6 1) (1 0))
    ((6 1) (3 2)) ((6 2) (5 0)) ((7 1) (5 3)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 cur) (cat k1 "wrap"))
  (facts (neq cur "decrypt") (neq cur "wrap") (neq "init" "decrypt")
    (neq "init" "wrap"))
  (rule invShearsRule fact-set-decrypt-neq0 trRl_set-wrap-at-1
    gen-st-set-wrap-0 fact-set-wrap-neq0 trRl_make-at-0)
  (operation channel-test (added-strand set-wrap 2)
    (ch-msg lk-0 (cat pt-4 k1 "wrap")) (5 3))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt old)) (stor lk-0 (cat pt-0 k "init")))
    ((load lk-0 (cat pt-0 k "init")) (stor lk-0 (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt-6 k "init"))
      (load lk-0 (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-1 k "wrap"))
      (load lk-0 (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-5 old-1)) (stor lk-0 (cat pt-6 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-7 k1 cur)) (stor lk-0 (cat pt-4 k1 "wrap"))))
  (label 1611)
  (parent 825)
  (unrealized (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-1) (k k) (lk lk-0))
  (precedes ((1 1) (3 3)) ((1 1) (5 2)) ((2 1) (3 2)) ((2 1) (6 0))
    ((3 4) (0 0)) ((4 2) (3 0)) ((5 4) (0 0)) ((6 1) (1 0))
    ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation channel-test (displaced 7 1 set-wrap 2)
    (ch-msg lk-0 (cat pt-6 k1 "wrap")) (5 3))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-5 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((load lk-0 (cat pt-0 old)) (stor lk-0 (cat pt-1 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt-1 k "init"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt k "wrap"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-4 old-1)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k))))
  (label 1612)
  (parent 826)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 cur mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk-0))
  (defstrand make 3 (old old-1) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur cur) (k k1) (lk lk-0))
  (precedes ((1 1) (3 3)) ((1 1) (5 2)) ((2 1) (3 2)) ((2 1) (6 0))
    ((3 4) (0 0)) ((4 2) (3 0)) ((5 4) (0 0)) ((6 1) (1 0))
    ((6 1) (7 0)) ((6 2) (5 0)) ((7 1) (5 3)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 cur) (cat k1 "wrap"))
  (facts (neq cur "decrypt") (neq cur "wrap") (neq "init" "decrypt")
    (neq "init" "wrap"))
  (rule invShearsRule fact-set-decrypt-neq0 trRl_set-wrap-at-1
    gen-st-set-wrap-0 fact-set-wrap-neq0 trRl_make-at-0)
  (operation channel-test (added-strand set-wrap 2)
    (ch-msg lk-0 (cat pt-4 k1 "wrap")) (5 3))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-6 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((load lk-0 (cat pt-0 old)) (stor lk-0 (cat pt-1 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt-1 k "init"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt k "wrap"))
      (load lk-0 (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-5 old-1)) (stor lk-0 (cat pt-6 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-7 k1 cur)) (stor lk-0 (cat pt-4 k1 "wrap"))))
  (label 1613)
  (parent 826)
  (unrealized (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 cur mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk-0))
  (defstrand make 3 (old old-1) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur cur) (k k1) (lk lk-0))
  (precedes ((1 1) (3 3)) ((1 1) (5 2)) ((1 1) (7 0)) ((2 1) (3 2))
    ((2 1) (6 0)) ((3 4) (0 0)) ((4 2) (3 0)) ((5 4) (0 0))
    ((6 1) (1 0)) ((6 2) (5 0)) ((7 1) (5 3)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 cur) (cat k1 "wrap"))
  (facts (neq cur "decrypt") (neq cur "wrap") (neq "init" "decrypt")
    (neq "init" "wrap"))
  (rule invShearsRule fact-set-decrypt-neq0 trRl_set-wrap-at-1
    gen-st-set-wrap-0 fact-set-wrap-neq0 trRl_make-at-0)
  (operation channel-test (added-strand set-wrap 2)
    (ch-msg lk-0 (cat pt-4 k1 "wrap")) (5 3))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-6 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((load lk-0 (cat pt-0 old)) (stor lk-0 (cat pt-1 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt-1 k "init"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt k "wrap"))
      (load lk-0 (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-5 old-1)) (stor lk-0 (cat pt-6 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-7 k1 cur)) (stor lk-0 (cat pt-4 k1 "wrap"))))
  (label 1614)
  (parent 826)
  (unrealized (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (4 3)) ((2 1) (6 2)) ((3 1) (1 0))
    ((3 1) (4 2)) ((4 4) (0 0)) ((5 2) (4 0)) ((5 2) (6 0))
    ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation channel-test (displaced 7 2 set-wrap 2)
    (ch-msg lk (cat pt-6 k1 "wrap")) (6 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-4 old-1)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k))))
  (label 1615)
  (parent 827)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 cur mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (defstrand set-wrap 2 (cur cur) (k k1) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (7 0)) ((2 1) (4 3)) ((2 1) (6 2))
    ((3 1) (1 0)) ((3 1) (4 2)) ((4 4) (0 0)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 4) (0 0)) ((7 1) (6 3)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 cur) (cat k1 "wrap"))
  (facts (neq cur "decrypt") (neq cur "wrap") (neq "init" "decrypt")
    (neq "init" "wrap"))
  (rule invShearsRule fact-set-decrypt-neq0 trRl_set-wrap-at-1
    gen-st-set-wrap-0 fact-set-wrap-neq0 trRl_make-at-0)
  (operation channel-test (added-strand set-wrap 2)
    (ch-msg lk (cat pt-6 k1 "wrap")) (6 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-4 old-1)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-6 k1 "wrap")) (send (enc k k1)))
    ((load lk (cat pt-7 k1 cur)) (stor lk (cat pt-6 k1 "wrap"))))
  (label 1616)
  (parent 827)
  (unrealized (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 cur mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (defstrand set-wrap 2 (cur cur) (k k1) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (4 3)) ((2 1) (6 2)) ((2 1) (7 0))
    ((3 1) (1 0)) ((3 1) (4 2)) ((4 4) (0 0)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 4) (0 0)) ((7 1) (6 3)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 cur) (cat k1 "wrap"))
  (facts (neq cur "decrypt") (neq cur "wrap") (neq "init" "decrypt")
    (neq "init" "wrap"))
  (rule invShearsRule fact-set-decrypt-neq0 trRl_set-wrap-at-1
    gen-st-set-wrap-0 fact-set-wrap-neq0 trRl_make-at-0)
  (operation channel-test (added-strand set-wrap 2)
    (ch-msg lk (cat pt-6 k1 "wrap")) (6 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-4 old-1)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-6 k1 "wrap")) (send (enc k k1)))
    ((load lk (cat pt-7 k1 cur)) (stor lk (cat pt-6 k1 "wrap"))))
  (label 1617)
  (parent 827)
  (unrealized (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 cur mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (defstrand make 3 (old old-1) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur cur) (k k1) (lk lk-0))
  (precedes ((1 1) (2 0)) ((1 1) (5 2)) ((1 1) (7 0)) ((2 1) (3 3))
    ((3 4) (0 0)) ((4 2) (3 0)) ((5 4) (0 0)) ((6 1) (1 0))
    ((6 1) (3 2)) ((6 2) (5 0)) ((7 1) (5 3)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 cur) (cat k1 "wrap"))
  (facts (neq cur "decrypt") (neq cur "wrap") (neq "init" "decrypt")
    (neq "init" "wrap"))
  (rule invShearsRule fact-set-decrypt-neq0 trRl_set-wrap-at-1
    gen-st-set-wrap-0 fact-set-wrap-neq0 trRl_make-at-0)
  (operation channel-test (added-strand set-wrap 2)
    (ch-msg lk-0 (cat pt-4 k1 "wrap")) (5 3))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt old)) (stor lk-0 (cat pt-0 k "init")))
    ((load lk-0 (cat pt-0 k "init")) (stor lk-0 (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt-6 k "init"))
      (load lk-0 (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-0 k "init"))
      (load lk-0 (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-5 old-1)) (stor lk-0 (cat pt-6 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-7 k1 cur)) (stor lk-0 (cat pt-4 k1 "wrap"))))
  (label 1618)
  (parent 830)
  (unrealized (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 cur mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (defstrand make 3 (old old-1) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur cur) (k k1) (lk lk-0))
  (precedes ((1 1) (2 0)) ((1 1) (5 2)) ((2 1) (3 3)) ((3 4) (0 0))
    ((4 2) (3 0)) ((5 4) (0 0)) ((6 1) (1 0)) ((6 1) (3 2))
    ((6 1) (7 0)) ((6 2) (5 0)) ((7 1) (5 3)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 cur) (cat k1 "wrap"))
  (facts (neq cur "decrypt") (neq cur "wrap") (neq "init" "decrypt")
    (neq "init" "wrap"))
  (rule invShearsRule fact-set-decrypt-neq0 trRl_set-wrap-at-1
    gen-st-set-wrap-0 fact-set-wrap-neq0 trRl_make-at-0)
  (operation channel-test (added-strand set-wrap 2)
    (ch-msg lk-0 (cat pt-4 k1 "wrap")) (5 3))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt old)) (stor lk-0 (cat pt-0 k "init")))
    ((load lk-0 (cat pt-0 k "init")) (stor lk-0 (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt-6 k "init"))
      (load lk-0 (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-0 k "init"))
      (load lk-0 (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-5 old-1)) (stor lk-0 (cat pt-6 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-7 k1 cur)) (stor lk-0 (cat pt-4 k1 "wrap"))))
  (label 1619)
  (parent 830)
  (unrealized (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 cur mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (defstrand make 3 (old old-1) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur cur) (k k1) (lk lk-0))
  (precedes ((1 1) (2 0)) ((1 1) (5 2)) ((2 1) (3 3)) ((2 1) (7 0))
    ((3 4) (0 0)) ((4 2) (3 0)) ((5 4) (0 0)) ((6 1) (1 0))
    ((6 1) (3 2)) ((6 2) (5 0)) ((7 1) (5 3)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 cur) (cat k1 "wrap"))
  (facts (neq cur "decrypt") (neq cur "wrap") (neq "init" "decrypt")
    (neq "init" "wrap"))
  (rule shearsRule invShearsRule fact-set-decrypt-neq0
    trRl_set-wrap-at-1 gen-st-set-wrap-0 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation channel-test (added-strand set-wrap 2)
    (ch-msg lk-0 (cat pt-4 k1 "wrap")) (5 3))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt old)) (stor lk-0 (cat pt-0 k "init")))
    ((load lk-0 (cat pt-0 k "init")) (stor lk-0 (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt-6 k "init"))
      (load lk-0 (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-0 k "init"))
      (load lk-0 (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-5 old-1)) (stor lk-0 (cat pt-6 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-7 k1 cur)) (stor lk-0 (cat pt-4 k1 "wrap"))))
  (label 1620)
  (parent 830)
  (unrealized (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-1) (k k) (lk lk-0))
  (precedes ((1 1) (3 3)) ((1 1) (5 3)) ((2 1) (3 2)) ((2 1) (6 0))
    ((3 4) (0 0)) ((4 2) (3 0)) ((5 4) (0 0)) ((6 1) (1 0))
    ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation channel-test (displaced 7 1 set-wrap 2)
    (ch-msg lk-0 (cat pt-6 k1 "wrap")) (5 3))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-5 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((load lk-0 (cat pt-0 old)) (stor lk-0 (cat pt-1 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt-1 k "init"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt-5 k "init"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-4 old-1)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k))))
  (label 1621)
  (parent 831)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 cur mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (defstrand make 3 (old old-1) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur cur) (k k1) (lk lk-0))
  (precedes ((1 1) (3 3)) ((2 1) (3 2)) ((2 1) (6 0)) ((3 4) (0 0))
    ((4 2) (3 0)) ((5 4) (0 0)) ((6 1) (1 0)) ((6 1) (7 0))
    ((6 2) (5 0)) ((7 1) (5 3)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 cur) (cat k1 "wrap"))
  (facts (neq cur "decrypt") (neq cur "wrap") (neq "init" "decrypt")
    (neq "init" "wrap"))
  (rule invShearsRule fact-set-decrypt-neq0 trRl_set-wrap-at-1
    gen-st-set-wrap-0 fact-set-wrap-neq0 trRl_make-at-0)
  (operation channel-test (added-strand set-wrap 2)
    (ch-msg lk-0 (cat pt-4 k1 "wrap")) (5 3))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-6 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((load lk-0 (cat pt-0 old)) (stor lk-0 (cat pt-1 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt-1 k "init"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-6 k "init"))
      (load lk-0 (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-5 old-1)) (stor lk-0 (cat pt-6 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-7 k1 cur)) (stor lk-0 (cat pt-4 k1 "wrap"))))
  (label 1622)
  (parent 831)
  (unrealized (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 cur mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (defstrand make 3 (old old-1) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur cur) (k k1) (lk lk-0))
  (precedes ((1 1) (3 3)) ((2 1) (3 2)) ((2 1) (6 0)) ((2 1) (7 0))
    ((3 4) (0 0)) ((4 2) (3 0)) ((5 4) (0 0)) ((6 1) (1 0))
    ((6 2) (5 0)) ((7 1) (5 3)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 cur) (cat k1 "wrap"))
  (facts (neq cur "decrypt") (neq cur "wrap") (neq "init" "decrypt")
    (neq "init" "wrap"))
  (rule invShearsRule fact-set-decrypt-neq0 trRl_set-wrap-at-1
    gen-st-set-wrap-0 fact-set-wrap-neq0 trRl_make-at-0)
  (operation channel-test (added-strand set-wrap 2)
    (ch-msg lk-0 (cat pt-4 k1 "wrap")) (5 3))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-6 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((load lk-0 (cat pt-0 old)) (stor lk-0 (cat pt-1 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt-1 k "init"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-6 k "init"))
      (load lk-0 (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-5 old-1)) (stor lk-0 (cat pt-6 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-7 k1 cur)) (stor lk-0 (cat pt-4 k1 "wrap"))))
  (label 1623)
  (parent 831)
  (unrealized (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 cur mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (defstrand make 3 (old old-1) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur cur) (k k1) (lk lk-0))
  (precedes ((1 1) (3 3)) ((1 1) (7 0)) ((2 1) (3 2)) ((2 1) (6 0))
    ((3 4) (0 0)) ((4 2) (3 0)) ((5 4) (0 0)) ((6 1) (1 0))
    ((6 2) (5 0)) ((7 1) (5 3)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 cur) (cat k1 "wrap"))
  (facts (neq cur "decrypt") (neq cur "wrap") (neq "init" "decrypt")
    (neq "init" "wrap"))
  (rule shearsRule invShearsRule fact-set-decrypt-neq0
    trRl_set-wrap-at-1 gen-st-set-wrap-0 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation channel-test (added-strand set-wrap 2)
    (ch-msg lk-0 (cat pt-4 k1 "wrap")) (5 3))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-6 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((load lk-0 (cat pt-0 old)) (stor lk-0 (cat pt-1 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt-1 k "init"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-6 k "init"))
      (load lk-0 (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-5 old-1)) (stor lk-0 (cat pt-6 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-7 k1 cur)) (stor lk-0 (cat pt-4 k1 "wrap"))))
  (label 1624)
  (parent 831)
  (unrealized (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (6 2)) ((2 1) (4 3)) ((2 1) (6 3))
    ((3 1) (1 0)) ((3 1) (4 2)) ((4 4) (0 0)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation channel-test (displaced 7 2 set-wrap 2)
    (ch-msg lk (cat pt-6 k1 "wrap")) (6 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-4 old-1)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k))))
  (label 1625)
  (parent 832)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 cur mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (defstrand set-wrap 2 (cur cur) (k k1) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (6 2)) ((1 1) (7 0)) ((2 1) (4 3))
    ((3 1) (1 0)) ((3 1) (4 2)) ((4 4) (0 0)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 4) (0 0)) ((7 1) (6 3)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 cur) (cat k1 "wrap"))
  (facts (neq cur "decrypt") (neq cur "wrap") (neq "init" "decrypt")
    (neq "init" "wrap"))
  (rule invShearsRule fact-set-decrypt-neq0 trRl_set-wrap-at-1
    gen-st-set-wrap-0 fact-set-wrap-neq0 trRl_make-at-0)
  (operation channel-test (added-strand set-wrap 2)
    (ch-msg lk (cat pt-6 k1 "wrap")) (6 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-4 old-1)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-6 k1 "wrap")) (send (enc k k1)))
    ((load lk (cat pt-7 k1 cur)) (stor lk (cat pt-6 k1 "wrap"))))
  (label 1626)
  (parent 832)
  (unrealized (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 cur mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (defstrand set-wrap 2 (cur cur) (k k1) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (6 2)) ((2 1) (4 3)) ((3 1) (1 0))
    ((3 1) (4 2)) ((3 1) (7 0)) ((4 4) (0 0)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 4) (0 0)) ((7 1) (6 3)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 cur) (cat k1 "wrap"))
  (facts (neq cur "decrypt") (neq cur "wrap") (neq "init" "decrypt")
    (neq "init" "wrap"))
  (rule invShearsRule fact-set-decrypt-neq0 trRl_set-wrap-at-1
    gen-st-set-wrap-0 fact-set-wrap-neq0 trRl_make-at-0)
  (operation channel-test (added-strand set-wrap 2)
    (ch-msg lk (cat pt-6 k1 "wrap")) (6 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-4 old-1)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-6 k1 "wrap")) (send (enc k k1)))
    ((load lk (cat pt-7 k1 cur)) (stor lk (cat pt-6 k1 "wrap"))))
  (label 1627)
  (parent 832)
  (unrealized (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 cur mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (defstrand set-wrap 2 (cur cur) (k k1) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (6 2)) ((2 1) (4 3)) ((2 1) (7 0))
    ((3 1) (1 0)) ((3 1) (4 2)) ((4 4) (0 0)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 4) (0 0)) ((7 1) (6 3)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 cur) (cat k1 "wrap"))
  (facts (neq cur "decrypt") (neq cur "wrap") (neq "init" "decrypt")
    (neq "init" "wrap"))
  (rule shearsRule invShearsRule fact-set-decrypt-neq0
    trRl_set-wrap-at-1 gen-st-set-wrap-0 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation channel-test (added-strand set-wrap 2)
    (ch-msg lk (cat pt-6 k1 "wrap")) (6 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-4 old-1)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-6 k1 "wrap")) (send (enc k k1)))
    ((load lk (cat pt-7 k1 cur)) (stor lk (cat pt-6 k1 "wrap"))))
  (label 1628)
  (parent 832)
  (unrealized (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (deflistener k)
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (0 0)) ((3 1) (5 0))
    ((4 4) (0 0)) ((5 1) (4 0)) ((6 1) (7 0)) ((7 1) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-decrypt-at-1 trRl_make-at-0)
  (operation nonce-test (added-strand set-decrypt 2) k (5 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk-0 (cat pt-3 k "init"))
    (ch-msg lk-1 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init")))
    ((load lk-1 (cat pt-6 k "init"))
      (stor lk-1 (cat pt-7 k "decrypt"))))
  (label 1629)
  (parent 847)
  (unrealized (4 3) (5 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (deflistener k)
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (0 0)) ((3 1) (5 0))
    ((4 4) (0 0)) ((5 1) (4 0)) ((6 1) (7 0)) ((7 1) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-wrap-at-1 trRl_make-at-0)
  (operation nonce-test (added-strand set-wrap 2) k (5 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk-0 (cat pt-3 k "init"))
    (ch-msg lk-1 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init")))
    ((load lk-1 (cat pt-6 k "init")) (stor lk-1 (cat pt-7 k "wrap"))))
  (label 1630)
  (parent 847)
  (unrealized (4 3) (5 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 k1-0 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (deflistener k)
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk-1))
  (precedes ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (0 0)) ((3 1) (5 0))
    ((4 4) (0 0)) ((5 1) (4 0)) ((6 1) (7 2)) ((7 4) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (5 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk-0 (cat pt-3 k "init"))
    (ch-msg lk-1 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-1 (cat pt-6 k "init"))
      (load lk-1 (cat pt-7 k1-0 "wrap")) (send (enc k k1-0))))
  (label 1631)
  (parent 847)
  (unrealized (4 3) (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 k1-0 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (deflistener k)
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk-0))
  (precedes ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (0 0)) ((3 1) (7 2))
    ((4 4) (0 0)) ((5 1) (4 0)) ((6 1) (5 0)) ((7 4) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (5 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk-0 (cat pt-3 k "init"))
    (ch-msg lk-1 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-7 k1-0 "wrap")) (send (enc k k1-0))))
  (label 1632)
  (parent 847)
  (unrealized (4 3) (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 k1-0 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (deflistener k)
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (7 2)) ((2 1) (0 0)) ((3 1) (5 0))
    ((4 4) (0 0)) ((5 1) (4 0)) ((6 1) (5 0)) ((7 4) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (5 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk-0 (cat pt-3 k "init"))
    (ch-msg lk-1 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-7 k1-0 "wrap")) (send (enc k k1-0))))
  (label 1633)
  (parent 847)
  (unrealized (4 3) (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (deflistener k)
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (0 0)) ((3 4) (0 0))
    ((4 1) (3 0)) ((5 4) (4 0)) ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 3 7 make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-5 k "init"))
      (load lk-0 (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-0 (cat pt-5 k "init"))
      (load lk-0 (cat pt-3 k1-0 "wrap")) (send (enc k k1-0)))
    ((load lk-0 (cat pt-4 old-0)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k))))
  (label 1634)
  (parent 852)
  (unrealized (3 3) (5 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 k1-0 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (deflistener k)
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk-0))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (0 0)) ((3 1) (6 2))
    ((4 4) (0 0)) ((5 1) (4 0)) ((6 4) (5 0)) ((7 2) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation encryption-test (added-strand make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-5 k1-0 "wrap")) (send (enc k k1-0)))
    ((load lk-1 (cat pt-6 old-1)) (stor lk-1 (cat pt-7 k "init"))
      (send (hash k))))
  (label 1635)
  (parent 852)
  (unrealized (4 3) (6 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (deflistener k)
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk-0))
  (deflistener k)
  (precedes ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (0 0)) ((3 1) (6 2))
    ((4 4) (0 0)) ((5 1) (4 0)) ((6 4) (5 0)) ((7 1) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (added-listener k) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-5 k1-0 "wrap")) (send (enc k k1-0)))
    ((recv k) (send k)))
  (label 1636)
  (parent 852)
  (unrealized (4 3) (6 3) (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (precedes ((1 1) (0 0)) ((2 1) (4 0)) ((3 4) (0 0)) ((4 1) (3 0))
    ((5 4) (4 0)) ((6 1) (1 0)) ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 1 7 make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-5 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-0 (cat pt-5 k "init"))
      (load lk-0 (cat pt-3 k1-0 "wrap")) (send (enc k k1-0)))
    ((load lk-0 (cat pt-4 old-0)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k))))
  (label 1637)
  (parent 853)
  (unrealized (3 3) (5 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 k1-0 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (deflistener k)
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((1 1) (6 2)) ((2 1) (0 0)) ((3 1) (5 0))
    ((4 4) (0 0)) ((5 1) (4 0)) ((6 4) (5 0)) ((7 2) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation encryption-test (added-strand make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((recv (hash k)) (recv (hash k1-0)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-5 k1-0 "wrap")) (send (enc k k1-0)))
    ((load lk-1 (cat pt-6 old-1)) (stor lk-1 (cat pt-7 k "init"))
      (send (hash k))))
  (label 1638)
  (parent 853)
  (unrealized (4 3) (6 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (deflistener k)
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk))
  (deflistener k)
  (precedes ((1 1) (2 0)) ((1 1) (6 2)) ((2 1) (0 0)) ((3 1) (5 0))
    ((4 4) (0 0)) ((5 1) (4 0)) ((6 4) (5 0)) ((7 1) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (added-listener k) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((recv (hash k)) (recv (hash k1-0)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-5 k1-0 "wrap")) (send (enc k k1-0)))
    ((recv k) (send k)))
  (label 1639)
  (parent 853)
  (unrealized (4 3) (6 3) (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (defstrand set-decrypt 2 (cur "wrap") (k k) (lk lk-0))
  (precedes ((1 1) (3 2)) ((1 1) (7 0)) ((2 1) (5 0)) ((3 4) (0 0))
    ((4 1) (1 0)) ((4 2) (3 0)) ((5 1) (0 0)) ((6 1) (0 0))
    ((7 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "wrap" "decrypt") (neq "wrap" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-decrypt-at-1 fact-set-decrypt-neq0 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation nonce-test (added-strand set-decrypt 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-1 k "init")) (ch-msg lk (cat pt-4 k "wrap"))
    (ch-msg lk-0 (cat pt k "wrap")) (ch-msg lk-0 (cat pt-3 k "init"))
    (ch-msg lk-1 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt k "wrap"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-4 k "wrap")))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init")))
    ((load lk-0 (cat pt k "wrap")) (stor lk-0 (cat pt-7 k "decrypt"))))
  (label 1640)
  (parent 868)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (defstrand set-wrap 2 (cur "wrap") (k k) (lk lk-0))
  (precedes ((1 1) (3 2)) ((1 1) (7 0)) ((2 1) (5 0)) ((3 4) (0 0))
    ((4 1) (1 0)) ((4 2) (3 0)) ((5 1) (0 0)) ((6 1) (0 0))
    ((7 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "wrap" "decrypt") (neq "wrap" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule fact-set-decrypt-neq0 trRl_set-wrap-at-1 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation nonce-test (added-strand set-wrap 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-1 k "init")) (ch-msg lk (cat pt-4 k "wrap"))
    (ch-msg lk-0 (cat pt k "wrap")) (ch-msg lk-0 (cat pt-3 k "init"))
    (ch-msg lk-1 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt k "wrap"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-4 k "wrap")))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init")))
    ((load lk-0 (cat pt k "wrap")) (stor lk-0 (cat pt-7 k "wrap"))))
  (label 1641)
  (parent 868)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (defstrand set-wrap 2 (cur "wrap") (k k) (lk lk))
  (precedes ((1 1) (3 2)) ((2 1) (5 0)) ((3 4) (0 0)) ((4 1) (1 0))
    ((4 2) (3 0)) ((5 1) (7 0)) ((6 1) (0 0)) ((7 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "wrap" "decrypt") (neq "wrap" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule fact-set-decrypt-neq0 trRl_set-wrap-at-1 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation nonce-test (added-strand set-wrap 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-1 k "init")) (ch-msg lk (cat pt-4 k "wrap"))
    (ch-msg lk-0 (cat pt k "wrap")) (ch-msg lk-0 (cat pt-3 k "init"))
    (ch-msg lk-1 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt k "wrap"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-4 k "wrap")))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init")))
    ((load lk (cat pt-4 k "wrap")) (stor lk (cat pt-7 k "wrap"))))
  (label 1642)
  (parent 868)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-1))
  (precedes ((1 1) (3 2)) ((2 1) (5 0)) ((3 4) (0 0)) ((4 1) (1 0))
    ((4 2) (3 0)) ((5 1) (0 0)) ((6 1) (7 2)) ((7 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-1 k "init")) (ch-msg lk (cat pt-4 k "wrap"))
    (ch-msg lk-0 (cat pt k "wrap")) (ch-msg lk-0 (cat pt-3 k "init"))
    (ch-msg lk-1 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt k "wrap"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-4 k "wrap")))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk-1 (cat pt-6 k "init"))
      (load lk-1 (cat pt-7 k1 "wrap")) (send (enc k k1))))
  (label 1643)
  (parent 868)
  (unrealized (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (defstrand set-decrypt 2 (cur "wrap") (k k) (lk lk))
  (precedes ((1 1) (3 2)) ((2 1) (5 0)) ((3 4) (0 0)) ((4 1) (1 0))
    ((4 2) (3 0)) ((5 1) (7 0)) ((6 1) (0 0)) ((7 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "wrap" "decrypt") (neq "wrap" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-decrypt-at-1 fact-set-decrypt-neq0 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation nonce-test (added-strand set-decrypt 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-1 k "init")) (ch-msg lk (cat pt-4 k "wrap"))
    (ch-msg lk-0 (cat pt k "wrap")) (ch-msg lk-0 (cat pt-3 k "init"))
    (ch-msg lk-1 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt k "wrap"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-4 k "wrap")))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init")))
    ((load lk (cat pt-4 k "wrap")) (stor lk (cat pt-7 k "decrypt"))))
  (label 1644)
  (parent 868)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk-0))
  (precedes ((1 1) (3 2)) ((1 1) (7 2)) ((2 1) (5 0)) ((3 4) (0 0))
    ((4 1) (1 0)) ((4 2) (3 0)) ((5 1) (0 0)) ((6 1) (0 0))
    ((7 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-1 k "init")) (ch-msg lk (cat pt-4 k "wrap"))
    (ch-msg lk-0 (cat pt k "wrap")) (ch-msg lk-0 (cat pt-3 k "init"))
    (ch-msg lk-1 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt k "wrap"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-4 k "wrap")))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt k "wrap"))
      (load lk-0 (cat pt-7 k1 "wrap")) (send (enc k k1))))
  (label 1645)
  (parent 868)
  (unrealized (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (precedes ((1 1) (3 2)) ((2 1) (5 0)) ((3 4) (0 0)) ((4 1) (1 0))
    ((4 1) (7 2)) ((4 2) (3 0)) ((5 1) (0 0)) ((6 1) (0 0))
    ((7 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-1 k "init")) (ch-msg lk (cat pt-4 k "wrap"))
    (ch-msg lk-0 (cat pt k "wrap")) (ch-msg lk-0 (cat pt-3 k "init"))
    (ch-msg lk-1 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt k "wrap"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-4 k "wrap")))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-7 k1 "wrap")) (send (enc k k1))))
  (label 1646)
  (parent 868)
  (unrealized (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (3 2)) ((2 1) (5 0)) ((3 4) (0 0)) ((4 1) (1 0))
    ((4 2) (3 0)) ((5 1) (7 2)) ((6 1) (0 0)) ((7 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-1 k "init")) (ch-msg lk (cat pt-4 k "wrap"))
    (ch-msg lk-0 (cat pt k "wrap")) (ch-msg lk-0 (cat pt-3 k "init"))
    (ch-msg lk-1 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt k "wrap"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-4 k "wrap")))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-4 k "wrap"))
      (load lk (cat pt-7 k1 "wrap")) (send (enc k k1))))
  (label 1647)
  (parent 868)
  (unrealized (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (3 2)) ((2 1) (5 0)) ((2 1) (7 2)) ((3 4) (0 0))
    ((4 1) (1 0)) ((4 2) (3 0)) ((5 1) (0 0)) ((6 1) (0 0))
    ((7 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-1 k "init")) (ch-msg lk (cat pt-4 k "wrap"))
    (ch-msg lk-0 (cat pt k "wrap")) (ch-msg lk-0 (cat pt-3 k "init"))
    (ch-msg lk-1 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt k "wrap"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-4 k "wrap")))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt-7 k1 "wrap")) (send (enc k k1))))
  (label 1648)
  (parent 868)
  (unrealized (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (precedes ((1 1) (3 2)) ((2 1) (5 0)) ((3 4) (0 0)) ((4 1) (1 0))
    ((4 2) (3 0)) ((4 2) (6 0)) ((5 1) (0 0)) ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 7 4 make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt k "wrap"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-4 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-5 k1 "wrap")) (send (enc k k1))))
  (label 1649)
  (parent 870)
  (unrealized (6 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (precedes ((1 1) (2 2)) ((2 4) (0 0)) ((3 1) (1 0)) ((3 1) (5 2))
    ((3 2) (2 0)) ((4 1) (0 0)) ((5 4) (0 0)) ((6 1) (4 0))
    ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 2 7 make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt k "wrap"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-5 k "init")) (stor lk-0 (cat pt-2 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt-3 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-4 old-0)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k))))
  (label 1650)
  (parent 870)
  (unrealized (5 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (3 2)) ((2 1) (5 0)) ((3 4) (0 0)) ((4 1) (1 0))
    ((4 1) (6 2)) ((4 2) (3 0)) ((5 1) (0 0)) ((6 4) (0 0))
    ((7 2) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation encryption-test (added-strand make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt k "wrap"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-4 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-5 k1 "wrap")) (send (enc k k1)))
    ((load lk-1 (cat pt-6 old-1)) (stor lk-1 (cat pt-7 k "init"))
      (send (hash k))))
  (label 1651)
  (parent 870)
  (unrealized (6 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (deflistener k)
  (precedes ((1 1) (3 2)) ((2 1) (5 0)) ((3 4) (0 0)) ((4 1) (1 0))
    ((4 1) (6 2)) ((4 2) (3 0)) ((5 1) (0 0)) ((6 4) (0 0))
    ((7 1) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (added-listener k) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt k "wrap"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-4 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-5 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k)))
  (label 1652)
  (parent 870)
  (unrealized (6 3) (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk-0))
  (precedes ((1 1) (3 2)) ((1 1) (6 2)) ((2 1) (5 0)) ((3 4) (0 0))
    ((4 1) (1 0)) ((4 2) (3 0)) ((4 2) (6 0)) ((5 1) (0 0))
    ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 7 4 make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt k "wrap"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-4 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt k "wrap"))
      (load lk-0 (cat pt-5 k1 "wrap")) (send (enc k k1))))
  (label 1653)
  (parent 872)
  (unrealized (6 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (precedes ((1 1) (2 2)) ((1 1) (5 2)) ((2 4) (0 0)) ((3 1) (1 0))
    ((3 2) (2 0)) ((4 1) (0 0)) ((5 4) (0 0)) ((6 1) (4 0))
    ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 2 7 make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt k "wrap"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-5 k "init")) (stor lk-0 (cat pt-2 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt k "wrap"))
      (load lk (cat pt-3 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-4 old-0)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k))))
  (label 1654)
  (parent 872)
  (unrealized (5 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk-0))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (3 2)) ((1 1) (6 2)) ((2 1) (5 0)) ((3 4) (0 0))
    ((4 1) (1 0)) ((4 2) (3 0)) ((5 1) (0 0)) ((6 4) (0 0))
    ((7 2) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation encryption-test (added-strand make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt k "wrap"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-4 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt k "wrap"))
      (load lk-0 (cat pt-5 k1 "wrap")) (send (enc k k1)))
    ((load lk-1 (cat pt-6 old-1)) (stor lk-1 (cat pt-7 k "init"))
      (send (hash k))))
  (label 1655)
  (parent 872)
  (unrealized (6 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk-0))
  (deflistener k)
  (precedes ((1 1) (3 2)) ((1 1) (6 2)) ((2 1) (5 0)) ((3 4) (0 0))
    ((4 1) (1 0)) ((4 2) (3 0)) ((5 1) (0 0)) ((6 4) (0 0))
    ((7 1) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (added-listener k) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt k "wrap"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-4 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt k "wrap"))
      (load lk-0 (cat pt-5 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k)))
  (label 1656)
  (parent 872)
  (unrealized (6 3) (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (3 2)) ((2 1) (5 0)) ((3 4) (0 0)) ((4 1) (1 0))
    ((4 2) (3 0)) ((4 2) (6 0)) ((5 1) (6 2)) ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 7 4 make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt k "wrap"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-4 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-4 k "wrap"))
      (load lk (cat pt-5 k1 "wrap")) (send (enc k k1))))
  (label 1657)
  (parent 873)
  (unrealized (6 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (precedes ((1 1) (2 2)) ((2 4) (0 0)) ((3 1) (1 0)) ((3 2) (2 0))
    ((4 1) (5 2)) ((5 4) (0 0)) ((6 1) (4 0)) ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 2 7 make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt k "wrap"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-5 k "init")) (stor lk-0 (cat pt-2 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-2 k "wrap"))
      (load lk-0 (cat pt-3 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-4 old-0)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k))))
  (label 1658)
  (parent 873)
  (unrealized (5 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (3 2)) ((2 1) (5 0)) ((3 4) (0 0)) ((4 1) (1 0))
    ((4 2) (3 0)) ((5 1) (6 2)) ((6 4) (0 0)) ((7 2) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation encryption-test (added-strand make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt k "wrap"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-4 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-4 k "wrap"))
      (load lk (cat pt-5 k1 "wrap")) (send (enc k k1)))
    ((load lk-1 (cat pt-6 old-1)) (stor lk-1 (cat pt-7 k "init"))
      (send (hash k))))
  (label 1659)
  (parent 873)
  (unrealized (6 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (precedes ((1 1) (3 2)) ((2 1) (5 0)) ((3 4) (0 0)) ((4 1) (1 0))
    ((4 2) (3 0)) ((5 1) (6 2)) ((6 4) (0 0)) ((7 1) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (added-listener k) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt k "wrap"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-4 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-4 k "wrap"))
      (load lk (cat pt-5 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k)))
  (label 1660)
  (parent 873)
  (unrealized (6 3) (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (3 2)) ((2 1) (5 0)) ((2 1) (6 2)) ((3 4) (0 0))
    ((4 1) (1 0)) ((4 2) (3 0)) ((4 2) (6 0)) ((5 1) (0 0))
    ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 7 4 make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt k "wrap"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-4 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt-5 k1 "wrap")) (send (enc k k1))))
  (label 1661)
  (parent 874)
  (unrealized (6 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (precedes ((1 1) (2 2)) ((2 4) (0 0)) ((3 1) (1 0)) ((3 2) (2 0))
    ((4 1) (0 0)) ((5 4) (0 0)) ((6 1) (4 0)) ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 2 7 make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt k "wrap"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-5 k "init")) (stor lk-0 (cat pt-2 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-5 k "init"))
      (load lk-0 (cat pt-3 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-4 old-0)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k))))
  (label 1662)
  (parent 874)
  (unrealized (5 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (3 2)) ((2 1) (5 0)) ((2 1) (6 2)) ((3 4) (0 0))
    ((4 1) (1 0)) ((4 2) (3 0)) ((5 1) (0 0)) ((6 4) (0 0))
    ((7 2) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation encryption-test (added-strand make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt k "wrap"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-4 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt-5 k1 "wrap")) (send (enc k k1)))
    ((load lk-1 (cat pt-6 old-1)) (stor lk-1 (cat pt-7 k "init"))
      (send (hash k))))
  (label 1663)
  (parent 874)
  (unrealized (6 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (precedes ((1 1) (3 2)) ((2 1) (5 0)) ((2 1) (6 2)) ((3 4) (0 0))
    ((4 1) (1 0)) ((4 2) (3 0)) ((5 1) (0 0)) ((6 4) (0 0))
    ((7 1) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (added-listener k) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt k "wrap"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-4 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt-5 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k)))
  (label 1664)
  (parent 874)
  (unrealized (6 3) (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pval) (k skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk-0))
  (precedes ((1 1) (3 2)) ((1 1) (5 3)) ((2 1) (0 0)) ((3 4) (0 0))
    ((4 1) (1 0)) ((4 2) (3 0)) ((4 2) (5 0)) ((5 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation channel-test (displaced 6 1 set-wrap 2)
    (ch-msg lk-0 (cat pt-4 k1 "wrap")) (5 3))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt k "wrap"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k))))
  (label 1665)
  (parent 875)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 cur mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (defstrand set-wrap 2 (cur cur) (k k1) (lk lk-0))
  (precedes ((1 1) (3 2)) ((1 1) (6 0)) ((2 1) (0 0)) ((3 4) (0 0))
    ((4 1) (1 0)) ((4 2) (3 0)) ((4 2) (5 0)) ((5 4) (0 0))
    ((6 1) (5 3)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 cur) (cat k1 "wrap"))
  (facts (neq cur "decrypt") (neq cur "wrap") (neq "init" "decrypt")
    (neq "init" "wrap"))
  (rule shearsRule invShearsRule fact-set-decrypt-neq0
    trRl_set-wrap-at-1 gen-st-set-wrap-0 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation channel-test (added-strand set-wrap 2)
    (ch-msg lk-0 (cat pt-4 k1 "wrap")) (5 3))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt k "wrap"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-5 k1 cur)) (stor lk-0 (cat pt-4 k1 "wrap"))))
  (label 1666)
  (parent 875)
  (unrealized (6 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k skey) (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (3 2)) ((1 1) (5 3)) ((2 1) (0 0)) ((3 4) (0 0))
    ((4 1) (1 0)) ((4 1) (5 2)) ((4 2) (3 0)) ((5 4) (0 0))
    ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation channel-test (displaced 7 1 set-wrap 2)
    (ch-msg lk-0 (cat pt-6 k1 "wrap")) (5 3))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt k "wrap"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-1 (cat pt-4 old-1)) (stor lk-1 (cat pt-5 k "init"))
      (send (hash k))))
  (label 1667)
  (parent 876)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 cur mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand set-wrap 2 (cur cur) (k k1) (lk lk-0))
  (precedes ((1 1) (3 2)) ((1 1) (7 0)) ((2 1) (0 0)) ((3 4) (0 0))
    ((4 1) (1 0)) ((4 1) (5 2)) ((4 2) (3 0)) ((5 4) (0 0))
    ((6 2) (5 0)) ((7 1) (5 3)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 cur) (cat k1 "wrap"))
  (facts (neq cur "decrypt") (neq cur "wrap") (neq "init" "decrypt")
    (neq "init" "wrap"))
  (rule shearsRule invShearsRule fact-set-decrypt-neq0
    trRl_set-wrap-at-1 gen-st-set-wrap-0 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation channel-test (added-strand set-wrap 2)
    (ch-msg lk-0 (cat pt-4 k1 "wrap")) (5 3))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt k "wrap"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-7 k1 cur)) (stor lk-0 (cat pt-4 k1 "wrap"))))
  (label 1668)
  (parent 876)
  (unrealized (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (deflistener k)
  (precedes ((1 1) (3 2)) ((2 1) (0 0)) ((3 4) (0 0)) ((4 1) (1 0))
    ((4 1) (6 0)) ((4 2) (3 0)) ((5 4) (0 0)) ((6 1) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation nonce-test (displaced 7 4 make 2) k (6 0))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt k "wrap"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k)))
  (label 1669)
  (parent 877)
  (unrealized (5 3) (6 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (deflistener k)
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (3 2)) ((2 1) (0 0)) ((3 4) (0 0)) ((4 1) (1 0))
    ((4 1) (5 2)) ((4 2) (3 0)) ((5 4) (0 0)) ((6 1) (5 0))
    ((7 1) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation nonce-test (added-strand make 2) k (6 0))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt k "wrap"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init"))))
  (label 1670)
  (parent 877)
  (unrealized (5 3) (6 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k skey) (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (3 2)) ((1 1) (5 2)) ((2 1) (0 0)) ((3 4) (0 0))
    ((4 1) (1 0)) ((4 2) (3 0)) ((5 4) (0 0)) ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation channel-test (displaced 7 1 set-wrap 2)
    (ch-msg lk-0 (cat pt-6 k1 "wrap")) (5 3))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt k "wrap"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt k "wrap"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-1 (cat pt-4 old-1)) (stor lk-1 (cat pt-5 k "init"))
      (send (hash k))))
  (label 1671)
  (parent 879)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 cur mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk-0))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand set-wrap 2 (cur cur) (k k1) (lk lk-0))
  (precedes ((1 1) (3 2)) ((1 1) (5 2)) ((1 1) (7 0)) ((2 1) (0 0))
    ((3 4) (0 0)) ((4 1) (1 0)) ((4 2) (3 0)) ((5 4) (0 0))
    ((6 2) (5 0)) ((7 1) (5 3)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 cur) (cat k1 "wrap"))
  (facts (neq cur "decrypt") (neq cur "wrap") (neq "init" "decrypt")
    (neq "init" "wrap"))
  (rule invShearsRule fact-set-decrypt-neq0 trRl_set-wrap-at-1
    gen-st-set-wrap-0 fact-set-wrap-neq0 trRl_make-at-0)
  (operation channel-test (added-strand set-wrap 2)
    (ch-msg lk-0 (cat pt-4 k1 "wrap")) (5 3))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt k "wrap"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt k "wrap"))
      (load lk-0 (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-7 k1 cur)) (stor lk-0 (cat pt-4 k1 "wrap"))))
  (label 1672)
  (parent 879)
  (unrealized (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk-0))
  (deflistener k)
  (precedes ((1 1) (3 2)) ((1 1) (5 2)) ((2 1) (0 0)) ((3 4) (0 0))
    ((4 1) (1 0)) ((4 1) (6 0)) ((4 2) (3 0)) ((5 4) (0 0))
    ((6 1) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation nonce-test (displaced 7 4 make 2) k (6 0))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt k "wrap"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt k "wrap"))
      (load lk-0 (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k)))
  (label 1673)
  (parent 880)
  (unrealized (5 3) (6 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk-0))
  (deflistener k)
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (3 2)) ((1 1) (5 2)) ((2 1) (0 0)) ((3 4) (0 0))
    ((4 1) (1 0)) ((4 2) (3 0)) ((5 4) (0 0)) ((6 1) (5 0))
    ((7 1) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation nonce-test (added-strand make 2) k (6 0))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt k "wrap"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt k "wrap"))
      (load lk-0 (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init"))))
  (label 1674)
  (parent 880)
  (unrealized (5 3) (6 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 cur mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (defstrand set-wrap 2 (cur cur) (k k1) (lk lk))
  (precedes ((1 1) (3 2)) ((2 1) (5 2)) ((2 1) (6 0)) ((3 4) (0 0))
    ((4 1) (1 0)) ((4 2) (3 0)) ((4 2) (5 0)) ((5 4) (0 0))
    ((6 1) (5 3)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 cur) (cat k1 "wrap"))
  (facts (neq cur "decrypt") (neq cur "wrap") (neq "init" "decrypt")
    (neq "init" "wrap"))
  (rule invShearsRule fact-set-decrypt-neq0 trRl_set-wrap-at-1
    gen-st-set-wrap-0 fact-set-wrap-neq0 trRl_make-at-0)
  (operation channel-test (added-strand set-wrap 2)
    (ch-msg lk (cat pt-4 k1 "wrap")) (5 3))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt k "wrap"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((load lk (cat pt-5 k1 cur)) (stor lk (cat pt-4 k1 "wrap"))))
  (label 1675)
  (parent 881)
  (unrealized (6 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 cur mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur cur) (k k1) (lk lk-0))
  (precedes ((1 1) (2 2)) ((2 4) (0 0)) ((3 1) (1 0)) ((3 2) (2 0))
    ((4 4) (0 0)) ((5 1) (6 0)) ((5 2) (4 0)) ((6 1) (4 3)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 cur) (cat k1 "wrap"))
  (facts (neq cur "decrypt") (neq cur "wrap") (neq "init" "decrypt")
    (neq "init" "wrap"))
  (rule invShearsRule fact-set-decrypt-neq0 trRl_set-wrap-at-1
    gen-st-set-wrap-0 fact-set-wrap-neq0 trRl_make-at-0)
  (operation channel-test (added-strand set-wrap 2)
    (ch-msg lk-0 (cat pt-2 k1 "wrap")) (4 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt k "wrap"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-4 k "init"))
      (load lk-0 (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-5 k1 cur)) (stor lk-0 (cat pt-2 k1 "wrap"))))
  (label 1676)
  (parent 882)
  (unrealized (6 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 cur mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand set-wrap 2 (cur cur) (k k1) (lk lk))
  (precedes ((1 1) (3 2)) ((2 1) (5 2)) ((2 1) (7 0)) ((3 4) (0 0))
    ((4 1) (1 0)) ((4 2) (3 0)) ((5 4) (0 0)) ((6 2) (5 0))
    ((7 1) (5 3)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 cur) (cat k1 "wrap"))
  (facts (neq cur "decrypt") (neq cur "wrap") (neq "init" "decrypt")
    (neq "init" "wrap"))
  (rule invShearsRule fact-set-decrypt-neq0 trRl_set-wrap-at-1
    gen-st-set-wrap-0 fact-set-wrap-neq0 trRl_make-at-0)
  (operation channel-test (added-strand set-wrap 2)
    (ch-msg lk (cat pt-4 k1 "wrap")) (5 3))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt k "wrap"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init"))
      (send (hash k)))
    ((load lk (cat pt-7 k1 cur)) (stor lk (cat pt-4 k1 "wrap"))))
  (label 1677)
  (parent 883)
  (unrealized (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (precedes ((1 1) (3 2)) ((2 1) (6 0)) ((3 4) (0 0)) ((4 1) (1 0))
    ((4 2) (3 0)) ((5 4) (0 0)) ((6 1) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation nonce-test (displaced 7 2 make 2) k (6 0))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt k "wrap"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k)))
  (label 1678)
  (parent 884)
  (unrealized (5 3) (6 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (precedes ((1 1) (3 2)) ((2 1) (5 2)) ((3 4) (0 0)) ((4 1) (1 0))
    ((4 1) (6 0)) ((4 2) (3 0)) ((5 4) (0 0)) ((6 1) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation nonce-test (displaced 7 4 make 2) k (6 0))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt k "wrap"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k)))
  (label 1679)
  (parent 884)
  (unrealized (5 3) (6 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (3 2)) ((2 1) (5 2)) ((3 4) (0 0)) ((4 1) (1 0))
    ((4 2) (3 0)) ((5 4) (0 0)) ((6 1) (5 0)) ((7 1) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation nonce-test (added-strand make 2) k (6 0))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt k "wrap"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init"))))
  (label 1680)
  (parent 884)
  (unrealized (5 3) (6 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-1) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((1 1) (3 2)) ((2 1) (5 0)) ((3 4) (0 0))
    ((4 1) (1 0)) ((4 2) (3 0)) ((5 1) (3 3)) ((6 1) (7 0))
    ((7 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-decrypt-at-1 trRl_make-at-0)
  (operation nonce-test (added-strand set-decrypt 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt k "wrap")) (ch-msg lk (cat pt-1 k "init"))
    (ch-msg lk (cat pt-2 k "wrap")) (ch-msg lk (cat pt-4 k "init"))
    (ch-msg lk-0 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt-4 k "init")) (stor lk (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt k "wrap"))
      (load lk (cat pt-2 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-3 old-0)) (stor lk (cat pt-4 k "init"))
      (send (hash k)))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-2 k "wrap")))
    ((load lk-0 (cat pt-5 old-1)) (stor lk-0 (cat pt-6 k "init")))
    ((load lk-0 (cat pt-6 k "init"))
      (stor lk-0 (cat pt-7 k "decrypt"))))
  (label 1681)
  (parent 885)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-1) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((1 1) (3 2)) ((2 1) (5 0)) ((3 4) (0 0))
    ((4 1) (1 0)) ((4 2) (3 0)) ((5 1) (3 3)) ((6 1) (7 0))
    ((7 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-wrap-at-1 trRl_make-at-0)
  (operation nonce-test (added-strand set-wrap 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt k "wrap")) (ch-msg lk (cat pt-1 k "init"))
    (ch-msg lk (cat pt-2 k "wrap")) (ch-msg lk (cat pt-4 k "init"))
    (ch-msg lk-0 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt-4 k "init")) (stor lk (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt k "wrap"))
      (load lk (cat pt-2 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-3 old-0)) (stor lk (cat pt-4 k "init"))
      (send (hash k)))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-2 k "wrap")))
    ((load lk-0 (cat pt-5 old-1)) (stor lk-0 (cat pt-6 k "init")))
    ((load lk-0 (cat pt-6 k "init")) (stor lk-0 (cat pt-7 k "wrap"))))
  (label 1682)
  (parent 885)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-1) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "wrap") (k k) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (3 2)) ((2 1) (5 0)) ((3 4) (0 0))
    ((4 1) (1 0)) ((4 2) (3 0)) ((5 1) (3 3)) ((5 1) (7 0))
    ((6 1) (0 0)) ((7 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "wrap" "decrypt") (neq "wrap" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule fact-set-decrypt-neq0 trRl_set-wrap-at-1 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation nonce-test (added-strand set-wrap 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt k "wrap")) (ch-msg lk (cat pt-1 k "init"))
    (ch-msg lk (cat pt-2 k "wrap")) (ch-msg lk (cat pt-4 k "init"))
    (ch-msg lk-0 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt-4 k "init")) (stor lk (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt k "wrap"))
      (load lk (cat pt-2 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-3 old-0)) (stor lk (cat pt-4 k "init"))
      (send (hash k)))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-2 k "wrap")))
    ((load lk-0 (cat pt-5 old-1)) (stor lk-0 (cat pt-6 k "init")))
    ((load lk (cat pt-2 k "wrap")) (stor lk (cat pt-7 k "wrap"))))
  (label 1683)
  (parent 885)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-1) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (precedes ((1 1) (2 0)) ((1 1) (3 2)) ((2 1) (5 0)) ((3 4) (0 0))
    ((4 1) (1 0)) ((4 2) (3 0)) ((5 1) (3 3)) ((6 1) (7 2))
    ((7 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt k "wrap")) (ch-msg lk (cat pt-1 k "init"))
    (ch-msg lk (cat pt-2 k "wrap")) (ch-msg lk (cat pt-4 k "init"))
    (ch-msg lk-0 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt-4 k "init")) (stor lk (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt k "wrap"))
      (load lk (cat pt-2 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-3 old-0)) (stor lk (cat pt-4 k "init"))
      (send (hash k)))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-2 k "wrap")))
    ((load lk-0 (cat pt-5 old-1)) (stor lk-0 (cat pt-6 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-6 k "init"))
      (load lk-0 (cat pt-7 k1 "wrap")) (send (enc k k1))))
  (label 1684)
  (parent 885)
  (unrealized (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-1) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "wrap") (k k) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (3 2)) ((2 1) (5 0)) ((3 4) (0 0))
    ((4 1) (1 0)) ((4 2) (3 0)) ((5 1) (3 3)) ((5 1) (7 0))
    ((6 1) (0 0)) ((7 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "wrap" "decrypt") (neq "wrap" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-decrypt-at-1 fact-set-decrypt-neq0 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation nonce-test (added-strand set-decrypt 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt k "wrap")) (ch-msg lk (cat pt-1 k "init"))
    (ch-msg lk (cat pt-2 k "wrap")) (ch-msg lk (cat pt-4 k "init"))
    (ch-msg lk-0 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt-4 k "init")) (stor lk (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt k "wrap"))
      (load lk (cat pt-2 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-3 old-0)) (stor lk (cat pt-4 k "init"))
      (send (hash k)))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-2 k "wrap")))
    ((load lk-0 (cat pt-5 old-1)) (stor lk-0 (cat pt-6 k "init")))
    ((load lk (cat pt-2 k "wrap")) (stor lk (cat pt-7 k "decrypt"))))
  (label 1685)
  (parent 885)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-1) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (3 2)) ((2 1) (5 0)) ((3 4) (0 0))
    ((4 1) (1 0)) ((4 2) (3 0)) ((5 1) (3 3)) ((5 1) (7 2))
    ((6 1) (0 0)) ((7 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt k "wrap")) (ch-msg lk (cat pt-1 k "init"))
    (ch-msg lk (cat pt-2 k "wrap")) (ch-msg lk (cat pt-4 k "init"))
    (ch-msg lk-0 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt-4 k "init")) (stor lk (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt k "wrap"))
      (load lk (cat pt-2 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-3 old-0)) (stor lk (cat pt-4 k "init"))
      (send (hash k)))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-2 k "wrap")))
    ((load lk-0 (cat pt-5 old-1)) (stor lk-0 (cat pt-6 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-2 k "wrap"))
      (load lk (cat pt-7 k1 "wrap")) (send (enc k k1))))
  (label 1686)
  (parent 885)
  (unrealized (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-1) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (3 2)) ((2 1) (5 0)) ((3 4) (0 0))
    ((4 1) (1 0)) ((4 1) (7 2)) ((4 2) (3 0)) ((5 1) (3 3))
    ((6 1) (0 0)) ((7 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt k "wrap")) (ch-msg lk (cat pt-1 k "init"))
    (ch-msg lk (cat pt-2 k "wrap")) (ch-msg lk (cat pt-4 k "init"))
    (ch-msg lk-0 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt-4 k "init")) (stor lk (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt k "wrap"))
      (load lk (cat pt-2 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-3 old-0)) (stor lk (cat pt-4 k "init"))
      (send (hash k)))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-2 k "wrap")))
    ((load lk-0 (cat pt-5 old-1)) (stor lk-0 (cat pt-6 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-4 k "init"))
      (load lk (cat pt-7 k1 "wrap")) (send (enc k k1))))
  (label 1687)
  (parent 885)
  (unrealized (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-1) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (3 2)) ((2 1) (5 0)) ((2 1) (7 2))
    ((3 4) (0 0)) ((4 1) (1 0)) ((4 2) (3 0)) ((5 1) (3 3))
    ((6 1) (0 0)) ((7 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt k "wrap")) (ch-msg lk (cat pt-1 k "init"))
    (ch-msg lk (cat pt-2 k "wrap")) (ch-msg lk (cat pt-4 k "init"))
    (ch-msg lk-0 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt-4 k "init")) (stor lk (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt k "wrap"))
      (load lk (cat pt-2 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-3 old-0)) (stor lk (cat pt-4 k "init"))
      (send (hash k)))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-2 k "wrap")))
    ((load lk-0 (cat pt-5 old-1)) (stor lk-0 (cat pt-6 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt-7 k1 "wrap")) (send (enc k k1))))
  (label 1688)
  (parent 885)
  (unrealized (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-1) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (3 2)) ((1 1) (7 2)) ((2 1) (5 0))
    ((3 4) (0 0)) ((4 1) (1 0)) ((4 2) (3 0)) ((5 1) (3 3))
    ((6 1) (0 0)) ((7 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt k "wrap")) (ch-msg lk (cat pt-1 k "init"))
    (ch-msg lk (cat pt-2 k "wrap")) (ch-msg lk (cat pt-4 k "init"))
    (ch-msg lk-0 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt-4 k "init")) (stor lk (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt k "wrap"))
      (load lk (cat pt-2 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-3 old-0)) (stor lk (cat pt-4 k "init"))
      (send (hash k)))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-2 k "wrap")))
    ((load lk-0 (cat pt-5 old-1)) (stor lk-0 (cat pt-6 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt k "wrap"))
      (load lk (cat pt-7 k1 "wrap")) (send (enc k k1))))
  (label 1689)
  (parent 885)
  (unrealized (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (3 2)) ((2 1) (5 0)) ((3 4) (0 0))
    ((4 1) (1 0)) ((4 2) (3 0)) ((4 2) (6 0)) ((5 1) (3 3))
    ((5 1) (6 2)) ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 7 4 make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt-4 k "init")) (stor lk (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt k "wrap"))
      (load lk (cat pt-2 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-3 old-0)) (stor lk (cat pt-4 k "init"))
      (send (hash k)))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-2 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-2 k "wrap"))
      (load lk (cat pt-5 k1 "wrap")) (send (enc k k1))))
  (label 1690)
  (parent 888)
  (unrealized (6 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (precedes ((1 1) (2 2)) ((1 1) (6 0)) ((2 4) (0 0)) ((3 1) (1 0))
    ((3 2) (2 0)) ((4 1) (2 3)) ((4 1) (5 2)) ((5 4) (0 0))
    ((6 1) (4 0)) ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 2 7 make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt-2 k "init")) (stor lk (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt k "wrap"))
      (load lk (cat pt-0 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-1 old)) (stor lk (cat pt-2 k "init"))
      (send (hash k)))
    ((load lk (cat pt-5 k "init")) (stor lk (cat pt-0 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "wrap"))
      (load lk (cat pt-3 k1 "wrap")) (send (enc k k1)))
    ((load lk (cat pt-4 old-0)) (stor lk (cat pt-5 k "init"))
      (send (hash k))))
  (label 1691)
  (parent 888)
  (unrealized (5 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((1 1) (3 2)) ((2 1) (5 0)) ((3 4) (0 0))
    ((4 1) (1 0)) ((4 2) (3 0)) ((5 1) (3 3)) ((5 1) (6 2))
    ((6 4) (0 0)) ((7 2) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation encryption-test (added-strand make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt-4 k "init")) (stor lk (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt k "wrap"))
      (load lk (cat pt-2 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-3 old-0)) (stor lk (cat pt-4 k "init"))
      (send (hash k)))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-2 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-2 k "wrap"))
      (load lk (cat pt-5 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-6 old-1)) (stor lk-0 (cat pt-7 k "init"))
      (send (hash k))))
  (label 1692)
  (parent 888)
  (unrealized (6 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (precedes ((1 1) (2 0)) ((1 1) (3 2)) ((2 1) (5 0)) ((3 4) (0 0))
    ((4 1) (1 0)) ((4 2) (3 0)) ((5 1) (3 3)) ((5 1) (6 2))
    ((6 4) (0 0)) ((7 1) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (added-listener k) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt-4 k "init")) (stor lk (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt k "wrap"))
      (load lk (cat pt-2 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-3 old-0)) (stor lk (cat pt-4 k "init"))
      (send (hash k)))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-2 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-2 k "wrap"))
      (load lk (cat pt-5 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k)))
  (label 1693)
  (parent 888)
  (unrealized (6 3) (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (3 2)) ((2 1) (5 0)) ((3 4) (0 0))
    ((4 1) (1 0)) ((4 2) (3 0)) ((4 2) (6 0)) ((5 1) (3 3))
    ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 7 4 make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt-4 k "init")) (stor lk (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt k "wrap"))
      (load lk (cat pt-2 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-3 old-0)) (stor lk (cat pt-4 k "init"))
      (send (hash k)))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-2 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-4 k "init"))
      (load lk (cat pt-5 k1 "wrap")) (send (enc k k1))))
  (label 1694)
  (parent 889)
  (unrealized (6 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((1 1) (3 2)) ((2 1) (5 0)) ((3 4) (0 0))
    ((4 1) (1 0)) ((4 1) (6 2)) ((4 2) (3 0)) ((5 1) (3 3))
    ((6 4) (0 0)) ((7 2) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation encryption-test (added-strand make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt-4 k "init")) (stor lk (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt k "wrap"))
      (load lk (cat pt-2 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-3 old-0)) (stor lk (cat pt-4 k "init"))
      (send (hash k)))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-2 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-4 k "init"))
      (load lk (cat pt-5 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-6 old-1)) (stor lk-0 (cat pt-7 k "init"))
      (send (hash k))))
  (label 1695)
  (parent 889)
  (unrealized (6 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (precedes ((1 1) (2 0)) ((1 1) (3 2)) ((2 1) (5 0)) ((3 4) (0 0))
    ((4 1) (1 0)) ((4 1) (6 2)) ((4 2) (3 0)) ((5 1) (3 3))
    ((6 4) (0 0)) ((7 1) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (added-listener k) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt-4 k "init")) (stor lk (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt k "wrap"))
      (load lk (cat pt-2 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-3 old-0)) (stor lk (cat pt-4 k "init"))
      (send (hash k)))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-2 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-4 k "init"))
      (load lk (cat pt-5 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k)))
  (label 1696)
  (parent 889)
  (unrealized (6 3) (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (3 2)) ((2 1) (5 0)) ((2 1) (6 2))
    ((3 4) (0 0)) ((4 1) (1 0)) ((4 2) (3 0)) ((4 2) (6 0))
    ((5 1) (3 3)) ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 7 4 make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt-4 k "init")) (stor lk (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt k "wrap"))
      (load lk (cat pt-2 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-3 old-0)) (stor lk (cat pt-4 k "init"))
      (send (hash k)))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-2 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt-5 k1 "wrap")) (send (enc k k1))))
  (label 1697)
  (parent 890)
  (unrealized (6 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (precedes ((1 1) (2 2)) ((1 1) (6 0)) ((2 4) (0 0)) ((3 1) (1 0))
    ((3 2) (2 0)) ((4 1) (2 3)) ((5 4) (0 0)) ((6 1) (4 0))
    ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 2 7 make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt-2 k "init")) (stor lk (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt k "wrap"))
      (load lk (cat pt-0 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-1 old)) (stor lk (cat pt-2 k "init"))
      (send (hash k)))
    ((load lk (cat pt-5 k "init")) (stor lk (cat pt-0 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-5 k "init"))
      (load lk (cat pt-3 k1 "wrap")) (send (enc k k1)))
    ((load lk (cat pt-4 old-0)) (stor lk (cat pt-5 k "init"))
      (send (hash k))))
  (label 1698)
  (parent 890)
  (unrealized (5 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((1 1) (3 2)) ((2 1) (5 0)) ((2 1) (6 2))
    ((3 4) (0 0)) ((4 1) (1 0)) ((4 2) (3 0)) ((5 1) (3 3))
    ((6 4) (0 0)) ((7 2) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation encryption-test (added-strand make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt-4 k "init")) (stor lk (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt k "wrap"))
      (load lk (cat pt-2 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-3 old-0)) (stor lk (cat pt-4 k "init"))
      (send (hash k)))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-2 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt-5 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-6 old-1)) (stor lk-0 (cat pt-7 k "init"))
      (send (hash k))))
  (label 1699)
  (parent 890)
  (unrealized (6 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (precedes ((1 1) (2 0)) ((1 1) (3 2)) ((2 1) (5 0)) ((2 1) (6 2))
    ((3 4) (0 0)) ((4 1) (1 0)) ((4 2) (3 0)) ((5 1) (3 3))
    ((6 4) (0 0)) ((7 1) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (added-listener k) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt-4 k "init")) (stor lk (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt k "wrap"))
      (load lk (cat pt-2 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-3 old-0)) (stor lk (cat pt-4 k "init"))
      (send (hash k)))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-2 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt-5 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k)))
  (label 1700)
  (parent 890)
  (unrealized (6 3) (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (3 2)) ((1 1) (6 2)) ((2 1) (5 0))
    ((3 4) (0 0)) ((4 1) (1 0)) ((4 2) (3 0)) ((4 2) (6 0))
    ((5 1) (3 3)) ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 7 4 make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt-4 k "init")) (stor lk (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt k "wrap"))
      (load lk (cat pt-2 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-3 old-0)) (stor lk (cat pt-4 k "init"))
      (send (hash k)))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-2 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt k "wrap"))
      (load lk (cat pt-5 k1 "wrap")) (send (enc k k1))))
  (label 1701)
  (parent 891)
  (unrealized (6 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((1 1) (3 2)) ((1 1) (6 2)) ((2 1) (5 0))
    ((3 4) (0 0)) ((4 1) (1 0)) ((4 2) (3 0)) ((5 1) (3 3))
    ((6 4) (0 0)) ((7 2) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation encryption-test (added-strand make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt-4 k "init")) (stor lk (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt k "wrap"))
      (load lk (cat pt-2 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-3 old-0)) (stor lk (cat pt-4 k "init"))
      (send (hash k)))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-2 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt k "wrap"))
      (load lk (cat pt-5 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-6 old-1)) (stor lk-0 (cat pt-7 k "init"))
      (send (hash k))))
  (label 1702)
  (parent 891)
  (unrealized (6 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (precedes ((1 1) (2 0)) ((1 1) (3 2)) ((1 1) (6 2)) ((2 1) (5 0))
    ((3 4) (0 0)) ((4 1) (1 0)) ((4 2) (3 0)) ((5 1) (3 3))
    ((6 4) (0 0)) ((7 1) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (added-listener k) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt-4 k "init")) (stor lk (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt k "wrap"))
      (load lk (cat pt-2 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-3 old-0)) (stor lk (cat pt-4 k "init"))
      (send (hash k)))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-2 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt k "wrap"))
      (load lk (cat pt-5 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k)))
  (label 1703)
  (parent 891)
  (unrealized (6 3) (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-1) (k k) (lk lk-0))
  (precedes ((1 1) (3 2)) ((1 1) (5 3)) ((2 1) (0 0)) ((3 4) (0 0))
    ((4 1) (1 0)) ((4 1) (5 2)) ((4 2) (3 0)) ((5 4) (0 0))
    ((6 1) (1 0)) ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule invShearsRule)
  (operation channel-test (displaced 7 1 set-wrap 2)
    (ch-msg lk-1 (cat pt-6 k1 "wrap")) (5 3))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-5 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt k "wrap"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-4 old-1)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k))))
  (label 1704)
  (parent 923)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 cur mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-1))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk-1))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand set-wrap 2 (cur cur) (k k1) (lk lk-0))
  (precedes ((1 1) (3 2)) ((2 1) (0 0)) ((3 4) (0 0)) ((4 1) (5 2))
    ((4 1) (7 0)) ((4 2) (3 0)) ((5 4) (0 0)) ((6 1) (1 0))
    ((6 2) (5 0)) ((7 1) (5 3)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 cur) (cat k1 "wrap"))
  (facts (neq cur "decrypt") (neq cur "wrap") (neq "init" "decrypt")
    (neq "init" "wrap"))
  (rule invShearsRule fact-set-decrypt-neq0 trRl_set-wrap-at-1
    gen-st-set-wrap-0 fact-set-wrap-neq0 trRl_make-at-0)
  (operation channel-test (added-strand set-wrap 2)
    (ch-msg lk-0 (cat pt-4 k1 "wrap")) (5 3))
  (traces ((recv k) (send k))
    ((load lk-1 (cat pt-6 k "init")) (stor lk-1 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk-1 (cat pt k "wrap"))
      (load lk-1 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-7 k1 cur)) (stor lk-0 (cat pt-4 k1 "wrap"))))
  (label 1705)
  (parent 923)
  (unrealized (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (4 2)) ((2 1) (6 3)) ((3 1) (0 0))
    ((4 4) (0 0)) ((5 1) (1 0)) ((5 2) (4 0)) ((5 2) (6 0))
    ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule invShearsRule)
  (operation channel-test (displaced 7 2 set-wrap 2)
    (ch-msg lk-1 (cat pt-6 k1 "wrap")) (6 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-4 old-1)) (stor lk (cat pt-5 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-5 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k))))
  (label 1706)
  (parent 924)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (4 2)) ((2 1) (6 3)) ((3 1) (0 0))
    ((4 4) (0 0)) ((5 1) (2 0)) ((5 2) (4 0)) ((5 2) (6 0))
    ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule invShearsRule)
  (operation channel-test (displaced 7 2 set-wrap 2)
    (ch-msg lk-1 (cat pt-6 k1 "wrap")) (6 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-4 old-1)) (stor lk (cat pt-5 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-5 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k))))
  (label 1707)
  (parent 924)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 cur mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-1))
  (defstrand set-wrap 2 (cur cur) (k k1) (lk lk-1))
  (precedes ((1 1) (2 0)) ((2 1) (4 2)) ((3 1) (0 0)) ((4 4) (0 0))
    ((5 1) (7 0)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 4) (0 0))
    ((7 1) (6 3)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 cur) (cat k1 "wrap"))
  (facts (neq cur "decrypt") (neq cur "wrap") (neq "init" "decrypt")
    (neq "init" "wrap"))
  (rule invShearsRule fact-set-decrypt-neq0 trRl_set-wrap-at-1
    gen-st-set-wrap-0 fact-set-wrap-neq0 trRl_make-at-0)
  (operation channel-test (added-strand set-wrap 2)
    (ch-msg lk-1 (cat pt-6 k1 "wrap")) (6 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-1 (cat pt-4 old-1)) (stor lk-1 (cat pt-5 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk-1 (cat pt-5 k "init"))
      (load lk-1 (cat pt-6 k1 "wrap")) (send (enc k k1)))
    ((load lk-1 (cat pt-7 k1 cur)) (stor lk-1 (cat pt-6 k1 "wrap"))))
  (label 1708)
  (parent 924)
  (unrealized (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 cur mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-1))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand set-wrap 2 (cur cur) (k k1) (lk lk-1))
  (precedes ((1 1) (2 0)) ((2 1) (3 2)) ((3 4) (0 0)) ((4 2) (3 0))
    ((5 4) (0 0)) ((6 1) (7 0)) ((6 2) (5 0)) ((7 1) (5 3)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 cur) (cat k1 "wrap"))
  (facts (neq cur "decrypt") (neq cur "wrap") (neq "init" "decrypt")
    (neq "init" "wrap"))
  (rule invShearsRule fact-set-decrypt-neq0 trRl_set-wrap-at-1
    gen-st-set-wrap-0 fact-set-wrap-neq0 trRl_make-at-0)
  (operation channel-test (added-strand set-wrap 2)
    (ch-msg lk-1 (cat pt-4 k1 "wrap")) (5 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk-1 (cat pt-6 k "init"))
      (load lk-1 (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init"))
      (send (hash k)))
    ((load lk-1 (cat pt-7 k1 cur)) (stor lk-1 (cat pt-4 k1 "wrap"))))
  (label 1709)
  (parent 927)
  (unrealized (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 cur mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-1))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk-1))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand set-wrap 2 (cur cur) (k k1) (lk lk))
  (precedes ((1 1) (3 2)) ((2 1) (5 2)) ((2 1) (7 0)) ((3 4) (0 0))
    ((4 2) (3 0)) ((5 4) (0 0)) ((6 1) (1 0)) ((6 2) (5 0))
    ((7 1) (5 3)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 cur) (cat k1 "wrap"))
  (facts (neq cur "decrypt") (neq cur "wrap") (neq "init" "decrypt")
    (neq "init" "wrap"))
  (rule invShearsRule fact-set-decrypt-neq0 trRl_set-wrap-at-1
    gen-st-set-wrap-0 fact-set-wrap-neq0 trRl_make-at-0)
  (operation channel-test (added-strand set-wrap 2)
    (ch-msg lk (cat pt-4 k1 "wrap")) (5 3))
  (traces ((recv k) (send k))
    ((load lk-1 (cat pt-6 k "init")) (stor lk-1 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk-1 (cat pt k "wrap"))
      (load lk-1 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init"))
      (send (hash k)))
    ((load lk (cat pt-7 k1 cur)) (stor lk (cat pt-4 k1 "wrap"))))
  (label 1710)
  (parent 928)
  (unrealized (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 cur mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (defstrand set-wrap 2 (cur cur) (k k1) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (4 2)) ((3 1) (6 2)) ((3 1) (7 0))
    ((4 4) (0 0)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 4) (0 0))
    ((7 1) (6 3)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 cur) (cat k1 "wrap"))
  (facts (neq cur "decrypt") (neq cur "wrap") (neq "init" "decrypt")
    (neq "init" "wrap"))
  (rule invShearsRule fact-set-decrypt-neq0 trRl_set-wrap-at-1
    gen-st-set-wrap-0 fact-set-wrap-neq0 trRl_make-at-0)
  (operation channel-test (added-strand set-wrap 2)
    (ch-msg lk-0 (cat pt-6 k1 "wrap")) (6 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-1 (cat pt-4 old-1)) (stor lk-1 (cat pt-5 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-6 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-7 k1 cur)) (stor lk-0 (cat pt-6 k1 "wrap"))))
  (label 1711)
  (parent 929)
  (unrealized (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 cur mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-1))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk-1))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk-1))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand set-wrap 2 (cur cur) (k k1) (lk lk-1))
  (precedes ((1 1) (3 2)) ((1 1) (5 2)) ((1 1) (7 0)) ((2 1) (0 0))
    ((3 4) (0 0)) ((4 2) (3 0)) ((5 4) (0 0)) ((6 1) (1 0))
    ((6 2) (5 0)) ((7 1) (5 3)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 cur) (cat k1 "wrap"))
  (facts (neq cur "decrypt") (neq cur "wrap") (neq "init" "decrypt")
    (neq "init" "wrap"))
  (rule invShearsRule fact-set-decrypt-neq0 trRl_set-wrap-at-1
    gen-st-set-wrap-0 fact-set-wrap-neq0 trRl_make-at-0)
  (operation channel-test (added-strand set-wrap 2)
    (ch-msg lk-1 (cat pt-4 k1 "wrap")) (5 3))
  (traces ((recv k) (send k))
    ((load lk-1 (cat pt-6 k "init")) (stor lk-1 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk-1 (cat pt k "wrap"))
      (load lk-1 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk-1 (cat pt k "wrap"))
      (load lk-1 (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init"))
      (send (hash k)))
    ((load lk-1 (cat pt-7 k1 cur)) (stor lk-1 (cat pt-4 k1 "wrap"))))
  (label 1712)
  (parent 932)
  (unrealized (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k skey) (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-1))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk-1))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk-1))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (3 2)) ((1 1) (5 3)) ((2 1) (0 0)) ((3 4) (0 0))
    ((4 2) (3 0)) ((5 4) (0 0)) ((6 1) (1 0)) ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation channel-test (displaced 7 1 set-wrap 2)
    (ch-msg lk-1 (cat pt-6 k1 "wrap")) (5 3))
  (traces ((recv k) (send k))
    ((load lk-1 (cat pt-5 k "init")) (stor lk-1 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk-1 (cat pt k "wrap"))
      (load lk-1 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k)) (load lk-1 (cat pt-5 k "init"))
      (load lk-1 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-1 (cat pt-4 old-1)) (stor lk-1 (cat pt-5 k "init"))
      (send (hash k))))
  (label 1713)
  (parent 936)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 cur mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-1))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk-1))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-1))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand set-wrap 2 (cur cur) (k k1) (lk lk-1))
  (precedes ((1 1) (3 2)) ((1 1) (7 0)) ((2 1) (0 0)) ((3 4) (0 0))
    ((4 2) (3 0)) ((5 4) (0 0)) ((6 1) (1 0)) ((6 2) (5 0))
    ((7 1) (5 3)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 cur) (cat k1 "wrap"))
  (facts (neq cur "decrypt") (neq cur "wrap") (neq "init" "decrypt")
    (neq "init" "wrap"))
  (rule shearsRule invShearsRule fact-set-decrypt-neq0
    trRl_set-wrap-at-1 gen-st-set-wrap-0 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation channel-test (added-strand set-wrap 2)
    (ch-msg lk-1 (cat pt-4 k1 "wrap")) (5 3))
  (traces ((recv k) (send k))
    ((load lk-1 (cat pt-6 k "init")) (stor lk-1 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk-1 (cat pt k "wrap"))
      (load lk-1 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk-1 (cat pt-6 k "init"))
      (load lk-1 (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init"))
      (send (hash k)))
    ((load lk-1 (cat pt-7 k1 cur)) (stor lk-1 (cat pt-4 k1 "wrap"))))
  (label 1714)
  (parent 936)
  (unrealized (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k skey) (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (6 2)) ((2 1) (4 2)) ((2 1) (6 3))
    ((3 1) (0 0)) ((4 4) (0 0)) ((5 2) (4 0)) ((5 2) (6 0))
    ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation channel-test (displaced 7 2 set-wrap 2)
    (ch-msg lk (cat pt-6 k1 "wrap")) (6 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-1 (cat pt-4 old-1)) (stor lk-1 (cat pt-5 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k))))
  (label 1715)
  (parent 937)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 cur mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (defstrand set-wrap 2 (cur cur) (k k1) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (6 2)) ((2 1) (4 2)) ((2 1) (7 0))
    ((3 1) (0 0)) ((4 4) (0 0)) ((5 2) (4 0)) ((5 2) (6 0))
    ((6 4) (0 0)) ((7 1) (6 3)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 cur) (cat k1 "wrap"))
  (facts (neq cur "decrypt") (neq cur "wrap") (neq "init" "decrypt")
    (neq "init" "wrap"))
  (rule shearsRule invShearsRule fact-set-decrypt-neq0
    trRl_set-wrap-at-1 gen-st-set-wrap-0 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation channel-test (added-strand set-wrap 2)
    (ch-msg lk (cat pt-6 k1 "wrap")) (6 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-1 (cat pt-4 old-1)) (stor lk-1 (cat pt-5 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-6 k1 "wrap")) (send (enc k k1)))
    ((load lk (cat pt-7 k1 cur)) (stor lk (cat pt-6 k1 "wrap"))))
  (label 1716)
  (parent 937)
  (unrealized (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((2 1) (5 0)) ((3 1) (0 0)) ((4 4) (0 0))
    ((5 1) (4 0)) ((6 1) (7 0)) ((7 1) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-decrypt-at-1 trRl_make-at-0)
  (operation nonce-test (added-strand set-decrypt 2) k (5 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-1 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init")))
    ((load lk-1 (cat pt-6 k "init"))
      (stor lk-1 (cat pt-7 k "decrypt"))))
  (label 1717)
  (parent 950)
  (unrealized (4 3) (5 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((2 1) (5 0)) ((3 1) (0 0)) ((4 4) (0 0))
    ((5 1) (4 0)) ((6 1) (7 0)) ((7 1) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-wrap-at-1 trRl_make-at-0)
  (operation nonce-test (added-strand set-wrap 2) k (5 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-1 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init")))
    ((load lk-1 (cat pt-6 k "init")) (stor lk-1 (cat pt-7 k "wrap"))))
  (label 1718)
  (parent 950)
  (unrealized (4 3) (5 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 k1-0 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk-1))
  (precedes ((1 1) (2 0)) ((2 1) (5 0)) ((3 1) (0 0)) ((4 4) (0 0))
    ((5 1) (4 0)) ((6 1) (7 2)) ((7 4) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap")
    (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (5 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-1 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-1 (cat pt-6 k "init"))
      (load lk-1 (cat pt-7 k1-0 "wrap")) (send (enc k k1-0))))
  (label 1719)
  (parent 950)
  (unrealized (4 3) (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 k1-0 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1-0) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (7 2)) ((3 1) (0 0)) ((4 4) (0 0))
    ((5 1) (4 0)) ((6 1) (5 0)) ((7 4) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap")
    (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (5 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-1 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-7 k1-0 "wrap")) (send (enc k k1-0))))
  (label 1720)
  (parent 950)
  (unrealized (4 3) (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 k1-0 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (7 2)) ((2 1) (5 0)) ((3 1) (0 0))
    ((4 4) (0 0)) ((5 1) (4 0)) ((6 1) (5 0)) ((7 4) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap")
    (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (5 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-1 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-7 k1-0 "wrap")) (send (enc k k1-0))))
  (label 1721)
  (parent 950)
  (unrealized (4 3) (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk-0))
  (deflistener k)
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1-0) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (precedes ((1 1) (5 2)) ((2 1) (0 0)) ((3 4) (0 0)) ((4 1) (3 0))
    ((5 4) (4 0)) ((6 1) (1 0)) ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap")
    (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 1 7 make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-5 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt k "wrap"))
      (load lk-0 (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-0 (cat pt k "wrap"))
      (load lk-0 (cat pt-3 k1-0 "wrap")) (send (enc k k1-0)))
    ((load lk-0 (cat pt-4 old-0)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k))))
  (label 1722)
  (parent 951)
  (unrealized (3 3) (5 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 k1-0 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1-0) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((2 1) (6 2)) ((3 1) (0 0)) ((4 4) (0 0))
    ((5 1) (4 0)) ((6 4) (5 0)) ((7 2) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap")
    (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation encryption-test (added-strand make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((recv (hash k)) (recv (hash k1-0)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-5 k1-0 "wrap")) (send (enc k k1-0)))
    ((load lk-1 (cat pt-6 old-1)) (stor lk-1 (cat pt-7 k "init"))
      (send (hash k))))
  (label 1723)
  (parent 951)
  (unrealized (4 3) (6 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1-0) (lk lk))
  (deflistener k)
  (precedes ((1 1) (2 0)) ((2 1) (6 2)) ((3 1) (0 0)) ((4 4) (0 0))
    ((5 1) (4 0)) ((6 4) (5 0)) ((7 1) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap")
    (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (added-listener k) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((recv (hash k)) (recv (hash k1-0)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-5 k1-0 "wrap")) (send (enc k k1-0)))
    ((recv k) (send k)))
  (label 1724)
  (parent 951)
  (unrealized (4 3) (6 3) (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk-0))
  (deflistener k)
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (precedes ((1 1) (4 0)) ((2 1) (0 0)) ((3 4) (0 0)) ((4 1) (3 0))
    ((5 4) (4 0)) ((6 1) (1 0)) ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap")
    (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 1 7 make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-5 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt k "wrap"))
      (load lk-0 (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-0 (cat pt-5 k "init"))
      (load lk-0 (cat pt-3 k1-0 "wrap")) (send (enc k k1-0)))
    ((load lk-0 (cat pt-4 old-0)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k))))
  (label 1725)
  (parent 952)
  (unrealized (3 3) (5 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 k1-0 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((1 1) (6 2)) ((2 1) (5 0)) ((3 1) (0 0))
    ((4 4) (0 0)) ((5 1) (4 0)) ((6 4) (5 0)) ((7 2) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap")
    (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation encryption-test (added-strand make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((recv (hash k)) (recv (hash k1-0)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-5 k1-0 "wrap")) (send (enc k k1-0)))
    ((load lk-1 (cat pt-6 old-1)) (stor lk-1 (cat pt-7 k "init"))
      (send (hash k))))
  (label 1726)
  (parent 952)
  (unrealized (4 3) (6 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk))
  (deflistener k)
  (precedes ((1 1) (2 0)) ((1 1) (6 2)) ((2 1) (5 0)) ((3 1) (0 0))
    ((4 4) (0 0)) ((5 1) (4 0)) ((6 4) (5 0)) ((7 1) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap")
    (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (added-listener k) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((recv (hash k)) (recv (hash k1-0)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-5 k1-0 "wrap")) (send (enc k k1-0)))
    ((recv k) (send k)))
  (label 1727)
  (parent 952)
  (unrealized (4 3) (6 3) (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk-0))
  (deflistener k)
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (precedes ((1 1) (5 3)) ((2 1) (0 0)) ((3 4) (0 0)) ((4 1) (3 0))
    ((5 4) (4 0)) ((6 1) (1 0)) ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation channel-test (displaced 7 1 set-wrap 2)
    (ch-msg lk-0 (cat pt-5 k1-0 "wrap")) (5 3))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt k "wrap"))
      (load lk-0 (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt-4 k "init"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init"))
      (send (hash k))))
  (label 1728)
  (parent 957)
  (unrealized (3 3) (4 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k skey) (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (3 3)) ((1 1) (5 3)) ((2 1) (0 0)) ((3 4) (0 0))
    ((4 1) (1 0)) ((4 1) (5 2)) ((4 2) (3 0)) ((5 4) (0 0))
    ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation channel-test (displaced 7 1 set-wrap 2)
    (ch-msg lk-0 (cat pt-6 k1 "wrap")) (5 3))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-1 (cat pt-4 old-1)) (stor lk-1 (cat pt-5 k "init"))
      (send (hash k))))
  (label 1729)
  (parent 961)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 cur mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand set-wrap 2 (cur cur) (k k1) (lk lk-0))
  (precedes ((1 1) (3 3)) ((1 1) (7 0)) ((2 1) (0 0)) ((3 4) (0 0))
    ((4 1) (1 0)) ((4 1) (5 2)) ((4 2) (3 0)) ((5 4) (0 0))
    ((6 2) (5 0)) ((7 1) (5 3)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 cur) (cat k1 "wrap"))
  (facts (neq cur "decrypt") (neq cur "wrap") (neq "init" "decrypt")
    (neq "init" "wrap"))
  (rule shearsRule invShearsRule fact-set-decrypt-neq0
    trRl_set-wrap-at-1 gen-st-set-wrap-0 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation channel-test (added-strand set-wrap 2)
    (ch-msg lk-0 (cat pt-4 k1 "wrap")) (5 3))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-7 k1 cur)) (stor lk-0 (cat pt-4 k1 "wrap"))))
  (label 1730)
  (parent 961)
  (unrealized (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (deflistener k)
  (precedes ((1 1) (3 3)) ((2 1) (0 0)) ((3 4) (0 0)) ((4 1) (1 0))
    ((4 1) (6 0)) ((4 2) (3 0)) ((5 4) (0 0)) ((6 1) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation nonce-test (displaced 7 4 make 2) k (6 0))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k)))
  (label 1731)
  (parent 962)
  (unrealized (5 3) (6 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (deflistener k)
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (3 3)) ((2 1) (0 0)) ((3 4) (0 0)) ((4 1) (1 0))
    ((4 1) (5 2)) ((4 2) (3 0)) ((5 4) (0 0)) ((6 1) (5 0))
    ((7 1) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation nonce-test (added-strand make 2) k (6 0))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init"))))
  (label 1732)
  (parent 962)
  (unrealized (5 3) (6 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 cur mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk-0))
  (defstrand set-wrap 2 (cur cur) (k k1) (lk lk-0))
  (precedes ((1 1) (3 3)) ((1 1) (5 2)) ((1 1) (6 0)) ((2 1) (0 0))
    ((3 4) (0 0)) ((4 1) (1 0)) ((4 2) (3 0)) ((4 2) (5 0))
    ((5 4) (0 0)) ((6 1) (5 3)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 cur) (cat k1 "wrap"))
  (facts (neq cur "decrypt") (neq cur "wrap") (neq "init" "decrypt")
    (neq "init" "wrap"))
  (rule invShearsRule fact-set-decrypt-neq0 trRl_set-wrap-at-1
    gen-st-set-wrap-0 fact-set-wrap-neq0 trRl_make-at-0)
  (operation channel-test (added-strand set-wrap 2)
    (ch-msg lk-0 (cat pt-4 k1 "wrap")) (5 3))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt k "wrap"))
      (load lk-0 (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-5 k1 cur)) (stor lk-0 (cat pt-4 k1 "wrap"))))
  (label 1733)
  (parent 963)
  (unrealized (6 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 cur mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk-0))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand set-wrap 2 (cur cur) (k k1) (lk lk-0))
  (precedes ((1 1) (3 3)) ((1 1) (5 2)) ((1 1) (7 0)) ((2 1) (0 0))
    ((3 4) (0 0)) ((4 1) (1 0)) ((4 2) (3 0)) ((5 4) (0 0))
    ((6 2) (5 0)) ((7 1) (5 3)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 cur) (cat k1 "wrap"))
  (facts (neq cur "decrypt") (neq cur "wrap") (neq "init" "decrypt")
    (neq "init" "wrap"))
  (rule invShearsRule fact-set-decrypt-neq0 trRl_set-wrap-at-1
    gen-st-set-wrap-0 fact-set-wrap-neq0 trRl_make-at-0)
  (operation channel-test (added-strand set-wrap 2)
    (ch-msg lk-0 (cat pt-4 k1 "wrap")) (5 3))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt k "wrap"))
      (load lk-0 (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-7 k1 cur)) (stor lk-0 (cat pt-4 k1 "wrap"))))
  (label 1734)
  (parent 964)
  (unrealized (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk-0))
  (deflistener k)
  (precedes ((1 1) (3 3)) ((1 1) (5 2)) ((2 1) (0 0)) ((3 4) (0 0))
    ((4 1) (1 0)) ((4 1) (6 0)) ((4 2) (3 0)) ((5 4) (0 0))
    ((6 1) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation nonce-test (displaced 7 4 make 2) k (6 0))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt k "wrap"))
      (load lk-0 (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k)))
  (label 1735)
  (parent 965)
  (unrealized (5 3) (6 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk-0))
  (deflistener k)
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (3 3)) ((1 1) (5 2)) ((2 1) (0 0)) ((3 4) (0 0))
    ((4 1) (1 0)) ((4 2) (3 0)) ((5 4) (0 0)) ((6 1) (5 0))
    ((7 1) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation nonce-test (added-strand make 2) k (6 0))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt k "wrap"))
      (load lk-0 (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init"))))
  (label 1736)
  (parent 965)
  (unrealized (5 3) (6 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 cur mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (defstrand set-wrap 2 (cur cur) (k k1) (lk lk))
  (precedes ((1 1) (3 3)) ((2 1) (5 2)) ((2 1) (6 0)) ((3 4) (0 0))
    ((4 1) (1 0)) ((4 2) (3 0)) ((4 2) (5 0)) ((5 4) (0 0))
    ((6 1) (5 3)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 cur) (cat k1 "wrap"))
  (facts (neq cur "decrypt") (neq cur "wrap") (neq "init" "decrypt")
    (neq "init" "wrap"))
  (rule invShearsRule fact-set-decrypt-neq0 trRl_set-wrap-at-1
    gen-st-set-wrap-0 fact-set-wrap-neq0 trRl_make-at-0)
  (operation channel-test (added-strand set-wrap 2)
    (ch-msg lk (cat pt-4 k1 "wrap")) (5 3))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((load lk (cat pt-5 k1 cur)) (stor lk (cat pt-4 k1 "wrap"))))
  (label 1737)
  (parent 966)
  (unrealized (6 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 cur mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur cur) (k k1) (lk lk-0))
  (precedes ((1 1) (2 3)) ((2 4) (0 0)) ((3 1) (1 0)) ((3 2) (2 0))
    ((4 4) (0 0)) ((5 1) (6 0)) ((5 2) (4 0)) ((6 1) (4 3)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 cur) (cat k1 "wrap"))
  (facts (neq cur "decrypt") (neq cur "wrap") (neq "init" "decrypt")
    (neq "init" "wrap"))
  (rule invShearsRule fact-set-decrypt-neq0 trRl_set-wrap-at-1
    gen-st-set-wrap-0 fact-set-wrap-neq0 trRl_make-at-0)
  (operation channel-test (added-strand set-wrap 2)
    (ch-msg lk-0 (cat pt-2 k1 "wrap")) (4 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-4 k "init"))
      (load lk-0 (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-5 k1 cur)) (stor lk-0 (cat pt-2 k1 "wrap"))))
  (label 1738)
  (parent 967)
  (unrealized (6 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 cur mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand set-wrap 2 (cur cur) (k k1) (lk lk))
  (precedes ((1 1) (3 3)) ((2 1) (5 2)) ((2 1) (7 0)) ((3 4) (0 0))
    ((4 1) (1 0)) ((4 2) (3 0)) ((5 4) (0 0)) ((6 2) (5 0))
    ((7 1) (5 3)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 cur) (cat k1 "wrap"))
  (facts (neq cur "decrypt") (neq cur "wrap") (neq "init" "decrypt")
    (neq "init" "wrap"))
  (rule invShearsRule fact-set-decrypt-neq0 trRl_set-wrap-at-1
    gen-st-set-wrap-0 fact-set-wrap-neq0 trRl_make-at-0)
  (operation channel-test (added-strand set-wrap 2)
    (ch-msg lk (cat pt-4 k1 "wrap")) (5 3))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init"))
      (send (hash k)))
    ((load lk (cat pt-7 k1 cur)) (stor lk (cat pt-4 k1 "wrap"))))
  (label 1739)
  (parent 968)
  (unrealized (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (precedes ((1 1) (3 3)) ((2 1) (6 0)) ((3 4) (0 0)) ((4 1) (1 0))
    ((4 2) (3 0)) ((5 4) (0 0)) ((6 1) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation nonce-test (displaced 7 2 make 2) k (6 0))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k)))
  (label 1740)
  (parent 969)
  (unrealized (5 3) (6 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (precedes ((1 1) (3 3)) ((2 1) (5 2)) ((3 4) (0 0)) ((4 1) (1 0))
    ((4 1) (6 0)) ((4 2) (3 0)) ((5 4) (0 0)) ((6 1) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation nonce-test (displaced 7 4 make 2) k (6 0))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k)))
  (label 1741)
  (parent 969)
  (unrealized (5 3) (6 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (3 3)) ((2 1) (5 2)) ((3 4) (0 0)) ((4 1) (1 0))
    ((4 2) (3 0)) ((5 4) (0 0)) ((6 1) (5 0)) ((7 1) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation nonce-test (added-strand make 2) k (6 0))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init"))))
  (label 1742)
  (parent 969)
  (unrealized (5 3) (6 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-1) (k k) (lk lk-0))
  (precedes ((1 1) (3 3)) ((1 1) (5 3)) ((2 1) (0 0)) ((3 4) (0 0))
    ((4 1) (1 0)) ((4 1) (5 2)) ((4 2) (3 0)) ((5 4) (0 0))
    ((6 1) (1 0)) ((6 1) (3 2)) ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule invShearsRule)
  (operation channel-test (displaced 7 1 set-wrap 2)
    (ch-msg lk-1 (cat pt-6 k1 "wrap")) (5 3))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-5 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt-5 k "init"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-4 old-1)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k))))
  (label 1743)
  (parent 989)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 cur mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-1))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk-1))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand set-wrap 2 (cur cur) (k k1) (lk lk-0))
  (precedes ((1 1) (3 3)) ((2 1) (0 0)) ((3 4) (0 0)) ((4 1) (5 2))
    ((4 1) (7 0)) ((4 2) (3 0)) ((5 4) (0 0)) ((6 1) (1 0))
    ((6 1) (3 2)) ((6 2) (5 0)) ((7 1) (5 3)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 cur) (cat k1 "wrap"))
  (facts (neq cur "decrypt") (neq cur "wrap") (neq "init" "decrypt")
    (neq "init" "wrap"))
  (rule invShearsRule fact-set-decrypt-neq0 trRl_set-wrap-at-1
    gen-st-set-wrap-0 fact-set-wrap-neq0 trRl_make-at-0)
  (operation channel-test (added-strand set-wrap 2)
    (ch-msg lk-0 (cat pt-4 k1 "wrap")) (5 3))
  (traces ((recv k) (send k))
    ((load lk-1 (cat pt-6 k "init")) (stor lk-1 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk-1 (cat pt-6 k "init"))
      (load lk-1 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-7 k1 cur)) (stor lk-0 (cat pt-4 k1 "wrap"))))
  (label 1744)
  (parent 989)
  (unrealized (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (4 2)) ((2 1) (4 3)) ((2 1) (6 3))
    ((3 1) (0 0)) ((4 4) (0 0)) ((5 1) (1 0)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule invShearsRule)
  (operation channel-test (displaced 7 2 set-wrap 2)
    (ch-msg lk-1 (cat pt-6 k1 "wrap")) (6 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-4 old-1)) (stor lk (cat pt-5 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-5 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k))))
  (label 1745)
  (parent 990)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (4 2)) ((2 1) (4 3)) ((2 1) (6 3))
    ((3 1) (0 0)) ((4 4) (0 0)) ((5 1) (2 0)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule invShearsRule)
  (operation channel-test (displaced 7 2 set-wrap 2)
    (ch-msg lk-1 (cat pt-6 k1 "wrap")) (6 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-4 old-1)) (stor lk (cat pt-5 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-5 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k))))
  (label 1746)
  (parent 990)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 cur mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-1))
  (defstrand set-wrap 2 (cur cur) (k k1) (lk lk-1))
  (precedes ((1 1) (2 0)) ((1 1) (4 2)) ((2 1) (4 3)) ((3 1) (0 0))
    ((4 4) (0 0)) ((5 1) (7 0)) ((5 2) (4 0)) ((5 2) (6 0))
    ((6 4) (0 0)) ((7 1) (6 3)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 cur) (cat k1 "wrap"))
  (facts (neq cur "decrypt") (neq cur "wrap") (neq "init" "decrypt")
    (neq "init" "wrap"))
  (rule invShearsRule fact-set-decrypt-neq0 trRl_set-wrap-at-1
    gen-st-set-wrap-0 fact-set-wrap-neq0 trRl_make-at-0)
  (operation channel-test (added-strand set-wrap 2)
    (ch-msg lk-1 (cat pt-6 k1 "wrap")) (6 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-1 (cat pt-4 old-1)) (stor lk-1 (cat pt-5 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk-1 (cat pt-5 k "init"))
      (load lk-1 (cat pt-6 k1 "wrap")) (send (enc k k1)))
    ((load lk-1 (cat pt-7 k1 cur)) (stor lk-1 (cat pt-6 k1 "wrap"))))
  (label 1747)
  (parent 990)
  (unrealized (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 cur mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-1))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand set-wrap 2 (cur cur) (k k1) (lk lk-1))
  (precedes ((1 1) (2 0)) ((1 1) (3 2)) ((2 1) (3 3)) ((3 4) (0 0))
    ((4 2) (3 0)) ((5 4) (0 0)) ((6 1) (7 0)) ((6 2) (5 0))
    ((7 1) (5 3)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 cur) (cat k1 "wrap"))
  (facts (neq cur "decrypt") (neq cur "wrap") (neq "init" "decrypt")
    (neq "init" "wrap"))
  (rule invShearsRule fact-set-decrypt-neq0 trRl_set-wrap-at-1
    gen-st-set-wrap-0 fact-set-wrap-neq0 trRl_make-at-0)
  (operation channel-test (added-strand set-wrap 2)
    (ch-msg lk-1 (cat pt-4 k1 "wrap")) (5 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk-1 (cat pt-6 k "init"))
      (load lk-1 (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init"))
      (send (hash k)))
    ((load lk-1 (cat pt-7 k1 cur)) (stor lk-1 (cat pt-4 k1 "wrap"))))
  (label 1748)
  (parent 993)
  (unrealized (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 cur mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-1))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk-1))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand set-wrap 2 (cur cur) (k k1) (lk lk))
  (precedes ((1 1) (3 3)) ((2 1) (5 2)) ((2 1) (7 0)) ((3 4) (0 0))
    ((4 2) (3 0)) ((5 4) (0 0)) ((6 1) (1 0)) ((6 1) (3 2))
    ((6 2) (5 0)) ((7 1) (5 3)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 cur) (cat k1 "wrap"))
  (facts (neq cur "decrypt") (neq cur "wrap") (neq "init" "decrypt")
    (neq "init" "wrap"))
  (rule invShearsRule fact-set-decrypt-neq0 trRl_set-wrap-at-1
    gen-st-set-wrap-0 fact-set-wrap-neq0 trRl_make-at-0)
  (operation channel-test (added-strand set-wrap 2)
    (ch-msg lk (cat pt-4 k1 "wrap")) (5 3))
  (traces ((recv k) (send k))
    ((load lk-1 (cat pt-6 k "init")) (stor lk-1 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk-1 (cat pt-6 k "init"))
      (load lk-1 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init"))
      (send (hash k)))
    ((load lk (cat pt-7 k1 cur)) (stor lk (cat pt-4 k1 "wrap"))))
  (label 1749)
  (parent 994)
  (unrealized (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 cur mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (defstrand set-wrap 2 (cur cur) (k k1) (lk lk-0))
  (precedes ((1 1) (2 0)) ((1 1) (4 2)) ((2 1) (4 3)) ((3 1) (6 2))
    ((3 1) (7 0)) ((4 4) (0 0)) ((5 2) (4 0)) ((5 2) (6 0))
    ((6 4) (0 0)) ((7 1) (6 3)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 cur) (cat k1 "wrap"))
  (facts (neq cur "decrypt") (neq cur "wrap") (neq "init" "decrypt")
    (neq "init" "wrap"))
  (rule invShearsRule fact-set-decrypt-neq0 trRl_set-wrap-at-1
    gen-st-set-wrap-0 fact-set-wrap-neq0 trRl_make-at-0)
  (operation channel-test (added-strand set-wrap 2)
    (ch-msg lk-0 (cat pt-6 k1 "wrap")) (6 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-1 (cat pt-4 old-1)) (stor lk-1 (cat pt-5 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-6 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-7 k1 cur)) (stor lk-0 (cat pt-6 k1 "wrap"))))
  (label 1750)
  (parent 995)
  (unrealized (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 cur mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-1))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk-1))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk-1))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand set-wrap 2 (cur cur) (k k1) (lk lk-1))
  (precedes ((1 1) (3 3)) ((1 1) (5 2)) ((1 1) (7 0)) ((2 1) (0 0))
    ((3 4) (0 0)) ((4 2) (3 0)) ((5 4) (0 0)) ((6 1) (1 0))
    ((6 1) (3 2)) ((6 2) (5 0)) ((7 1) (5 3)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 cur) (cat k1 "wrap"))
  (facts (neq cur "decrypt") (neq cur "wrap") (neq "init" "decrypt")
    (neq "init" "wrap"))
  (rule invShearsRule fact-set-decrypt-neq0 trRl_set-wrap-at-1
    gen-st-set-wrap-0 fact-set-wrap-neq0 trRl_make-at-0)
  (operation channel-test (added-strand set-wrap 2)
    (ch-msg lk-1 (cat pt-4 k1 "wrap")) (5 3))
  (traces ((recv k) (send k))
    ((load lk-1 (cat pt-6 k "init")) (stor lk-1 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk-1 (cat pt-6 k "init"))
      (load lk-1 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk-1 (cat pt k "wrap"))
      (load lk-1 (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init"))
      (send (hash k)))
    ((load lk-1 (cat pt-7 k1 cur)) (stor lk-1 (cat pt-4 k1 "wrap"))))
  (label 1751)
  (parent 998)
  (unrealized (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 cur mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (defstrand set-wrap 2 (cur cur) (k k1) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (4 2)) ((2 1) (4 3)) ((2 1) (6 2))
    ((2 1) (7 0)) ((3 1) (0 0)) ((4 4) (0 0)) ((5 2) (4 0))
    ((5 2) (6 0)) ((6 4) (0 0)) ((7 1) (6 3)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 cur) (cat k1 "wrap"))
  (facts (neq cur "decrypt") (neq cur "wrap") (neq "init" "decrypt")
    (neq "init" "wrap"))
  (rule invShearsRule fact-set-decrypt-neq0 trRl_set-wrap-at-1
    gen-st-set-wrap-0 fact-set-wrap-neq0 trRl_make-at-0)
  (operation channel-test (added-strand set-wrap 2)
    (ch-msg lk (cat pt-6 k1 "wrap")) (6 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-1 (cat pt-4 old-1)) (stor lk-1 (cat pt-5 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-6 k1 "wrap")) (send (enc k k1)))
    ((load lk (cat pt-7 k1 cur)) (stor lk (cat pt-6 k1 "wrap"))))
  (label 1752)
  (parent 999)
  (unrealized (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 cur mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-1))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk-1))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-1))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand set-wrap 2 (cur cur) (k k1) (lk lk-1))
  (precedes ((1 1) (3 3)) ((1 1) (7 0)) ((2 1) (0 0)) ((3 4) (0 0))
    ((4 2) (3 0)) ((5 4) (0 0)) ((6 1) (1 0)) ((6 1) (3 2))
    ((6 2) (5 0)) ((7 1) (5 3)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 cur) (cat k1 "wrap"))
  (facts (neq cur "decrypt") (neq cur "wrap") (neq "init" "decrypt")
    (neq "init" "wrap"))
  (rule shearsRule invShearsRule fact-set-decrypt-neq0
    trRl_set-wrap-at-1 gen-st-set-wrap-0 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation channel-test (added-strand set-wrap 2)
    (ch-msg lk-1 (cat pt-4 k1 "wrap")) (5 3))
  (traces ((recv k) (send k))
    ((load lk-1 (cat pt-6 k "init")) (stor lk-1 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk-1 (cat pt-6 k "init"))
      (load lk-1 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk-1 (cat pt-6 k "init"))
      (load lk-1 (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init"))
      (send (hash k)))
    ((load lk-1 (cat pt-7 k1 cur)) (stor lk-1 (cat pt-4 k1 "wrap"))))
  (label 1753)
  (parent 1002)
  (unrealized (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old mesg) (pt pt-0 pt-1 pt-2 pval) (k skey) (lk locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand set-wrap 2 (cur "wrap") (k k) (lk lk))
  (precedes ((1 1) (2 2)) ((1 1) (4 3)) ((1 1) (5 0)) ((2 4) (0 0))
    ((3 1) (1 0)) ((3 2) (2 0)) ((3 2) (4 0)) ((4 4) (0 0))
    ((5 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "wrap" "decrypt") (neq "wrap" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule fact-set-decrypt-neq0 trRl_set-wrap-at-1 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation nonce-test (added-strand set-wrap 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt k "wrap")) (ch-msg lk (cat pt-1 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt k "wrap"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt k "wrap")) (stor lk (cat pt-2 k "wrap"))))
  (label 1754)
  (parent 1013)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old mesg) (pt pt-0 pt-1 pt-2 pval) (k skey) (lk locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand set-decrypt 2 (cur "wrap") (k k) (lk lk))
  (precedes ((1 1) (2 2)) ((1 1) (4 3)) ((1 1) (5 0)) ((2 4) (0 0))
    ((3 1) (1 0)) ((3 2) (2 0)) ((3 2) (4 0)) ((4 4) (0 0))
    ((5 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "wrap" "decrypt") (neq "wrap" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-decrypt-at-1 fact-set-decrypt-neq0 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation nonce-test (added-strand set-decrypt 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt k "wrap")) (ch-msg lk (cat pt-1 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt k "wrap"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt k "wrap")) (stor lk (cat pt-2 k "decrypt"))))
  (label 1755)
  (parent 1013)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old mesg) (pt pt-0 pt-1 pt-2 pval) (k k1 skey) (lk locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (2 2)) ((1 1) (4 3)) ((2 4) (0 0)) ((3 1) (1 0))
    ((3 1) (5 2)) ((3 2) (2 0)) ((3 2) (4 0)) ((4 4) (0 0))
    ((5 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt k "wrap")) (ch-msg lk (cat pt-1 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt k "wrap"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1))))
  (label 1756)
  (parent 1013)
  (unrealized (5 0) (5 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old mesg) (pt pt-0 pt-1 pt-2 pval) (k k1 skey) (lk locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (2 2)) ((1 1) (4 3)) ((1 1) (5 2)) ((2 4) (0 0))
    ((3 1) (1 0)) ((3 2) (2 0)) ((3 2) (4 0)) ((4 4) (0 0))
    ((5 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt k "wrap")) (ch-msg lk (cat pt-1 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt k "wrap"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt k "wrap"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1))))
  (label 1757)
  (parent 1013)
  (unrealized (5 0) (5 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k k1 skey)
    (lk locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k1) (lk lk))
  (defstrand make 2 (old old-0) (k k1) (lk lk))
  (precedes ((1 1) (2 2)) ((1 1) (6 0)) ((2 4) (0 0)) ((3 1) (1 0))
    ((3 2) (2 0)) ((3 2) (4 0)) ((4 4) (0 0)) ((5 1) (4 3))
    ((6 1) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule invShearsRule trRl_make-at-0 trRl_make-at-1)
  (operation channel-test (added-strand make 2)
    (ch-msg lk (cat pt-3 k1 "init")) (5 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt k "wrap"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((load lk (cat pt-3 k1 "init")) (stor lk (cat pt-2 k1 "wrap")))
    ((load lk (cat pt-4 old-0)) (stor lk (cat pt-3 k1 "init"))))
  (label 1758)
  (parent 1014)
  (realized)
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (precedes ((1 1) (2 2)) ((1 1) (4 3)) ((2 4) (0 0)) ((3 1) (1 0))
    ((3 1) (4 2)) ((3 2) (2 0)) ((4 4) (0 0)) ((5 1) (6 0))
    ((5 2) (4 0)) ((6 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-decrypt-at-1 trRl_make-at-0)
  (operation nonce-test (added-strand set-decrypt 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt k "wrap")) (ch-msg lk (cat pt-1 k "init"))
    (ch-msg lk-0 (cat pt-3 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt k "wrap"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-3 k "init"))
      (stor lk-0 (cat pt-4 k "decrypt"))))
  (label 1759)
  (parent 1015)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "wrap") (k k) (lk lk))
  (precedes ((1 1) (2 2)) ((1 1) (4 3)) ((1 1) (6 0)) ((2 4) (0 0))
    ((3 1) (1 0)) ((3 1) (4 2)) ((3 2) (2 0)) ((4 4) (0 0))
    ((5 2) (4 0)) ((6 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "wrap" "decrypt") (neq "wrap" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule fact-set-decrypt-neq0 trRl_set-wrap-at-1 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation nonce-test (added-strand set-wrap 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt k "wrap")) (ch-msg lk (cat pt-1 k "init"))
    (ch-msg lk-0 (cat pt-3 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt k "wrap"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk (cat pt k "wrap")) (stor lk (cat pt-4 k "wrap"))))
  (label 1760)
  (parent 1015)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (precedes ((1 1) (2 2)) ((1 1) (4 3)) ((2 4) (0 0)) ((3 1) (1 0))
    ((3 1) (4 2)) ((3 2) (2 0)) ((4 4) (0 0)) ((5 1) (6 0))
    ((5 2) (4 0)) ((6 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-wrap-at-1 trRl_make-at-0)
  (operation nonce-test (added-strand set-wrap 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt k "wrap")) (ch-msg lk (cat pt-1 k "init"))
    (ch-msg lk-0 (cat pt-3 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt k "wrap"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "wrap"))))
  (label 1761)
  (parent 1015)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "wrap") (k k) (lk lk))
  (precedes ((1 1) (2 2)) ((1 1) (4 3)) ((1 1) (6 0)) ((2 4) (0 0))
    ((3 1) (1 0)) ((3 1) (4 2)) ((3 2) (2 0)) ((4 4) (0 0))
    ((5 2) (4 0)) ((6 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "wrap" "decrypt") (neq "wrap" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-decrypt-at-1 fact-set-decrypt-neq0 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation nonce-test (added-strand set-decrypt 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt k "wrap")) (ch-msg lk (cat pt-1 k "init"))
    (ch-msg lk-0 (cat pt-3 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt k "wrap"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk (cat pt k "wrap")) (stor lk (cat pt-4 k "decrypt"))))
  (label 1762)
  (parent 1015)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (precedes ((1 1) (2 2)) ((1 1) (4 3)) ((2 4) (0 0)) ((3 1) (1 0))
    ((3 1) (4 2)) ((3 2) (2 0)) ((4 4) (0 0)) ((5 1) (6 2))
    ((5 2) (4 0)) ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt k "wrap")) (ch-msg lk (cat pt-1 k "init"))
    (ch-msg lk-0 (cat pt-3 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt k "wrap"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-4 k1 "wrap")) (send (enc k k1))))
  (label 1763)
  (parent 1015)
  (unrealized (6 0) (6 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (2 2)) ((1 1) (4 3)) ((2 4) (0 0)) ((3 1) (1 0))
    ((3 1) (4 2)) ((3 1) (6 2)) ((3 2) (2 0)) ((4 4) (0 0))
    ((5 2) (4 0)) ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt k "wrap")) (ch-msg lk (cat pt-1 k "init"))
    (ch-msg lk-0 (cat pt-3 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt k "wrap"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1))))
  (label 1764)
  (parent 1015)
  (unrealized (6 0) (6 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (2 2)) ((1 1) (4 3)) ((1 1) (6 2)) ((2 4) (0 0))
    ((3 1) (1 0)) ((3 1) (4 2)) ((3 2) (2 0)) ((4 4) (0 0))
    ((5 2) (4 0)) ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt k "wrap")) (ch-msg lk (cat pt-1 k "init"))
    (ch-msg lk-0 (cat pt-3 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt k "wrap"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt k "wrap"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1))))
  (label 1765)
  (parent 1015)
  (unrealized (6 0) (6 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k1) (lk lk))
  (defstrand make 2 (old old-1) (k k1) (lk lk))
  (precedes ((1 1) (2 2)) ((1 1) (7 0)) ((2 4) (0 0)) ((3 1) (1 0))
    ((3 1) (4 2)) ((3 2) (2 0)) ((4 4) (0 0)) ((5 2) (4 0))
    ((6 1) (4 3)) ((7 1) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule invShearsRule trRl_make-at-0 trRl_make-at-1)
  (operation channel-test (added-strand make 2)
    (ch-msg lk (cat pt-5 k1 "init")) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt k "wrap"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init"))
      (send (hash k)))
    ((load lk (cat pt-5 k1 "init")) (stor lk (cat pt-2 k1 "wrap")))
    ((load lk (cat pt-6 old-1)) (stor lk (cat pt-5 k1 "init"))))
  (label 1766)
  (parent 1016)
  (realized)
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (precedes ((1 1) (2 2)) ((2 4) (0 0)) ((3 1) (1 0)) ((3 1) (5 0))
    ((3 2) (2 0)) ((4 4) (0 0)) ((5 1) (4 0)) ((6 1) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation nonce-test (added-strand make 2) k (5 0)
    (ch-msg lk (cat pt-1 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt k "wrap"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init"))))
  (label 1767)
  (parent 1017)
  (unrealized (4 3) (5 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old mesg) (pt pt-0 pt-1 pt-2 pt-3 pval) (k k1 k1-0 skey)
    (lk locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk))
  (precedes ((1 1) (2 2)) ((2 4) (0 0)) ((3 1) (1 0)) ((3 1) (6 2))
    ((3 2) (2 0)) ((4 4) (0 0)) ((5 1) (4 0)) ((6 4) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap")
    (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (5 0)
    (ch-msg lk (cat pt-1 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt k "wrap"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((recv (hash k)) (recv (hash k1-0)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt-3 k1-0 "wrap")) (send (enc k k1-0))))
  (label 1768)
  (parent 1017)
  (unrealized (4 3) (6 0) (6 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (precedes ((1 1) (2 2)) ((2 4) (0 0)) ((3 1) (1 0)) ((3 1) (4 2))
    ((3 2) (2 0)) ((4 4) (0 0)) ((5 1) (4 0)) ((6 1) (7 0))
    ((7 1) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-wrap-at-1 trRl_make-at-0)
  (operation nonce-test (added-strand set-wrap 2) k (5 0)
    (ch-msg lk-0 (cat pt-4 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt k "wrap"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt-5 k "wrap"))))
  (label 1769)
  (parent 1018)
  (unrealized (4 3) (5 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (precedes ((1 1) (2 2)) ((2 4) (0 0)) ((3 1) (1 0)) ((3 1) (4 2))
    ((3 2) (2 0)) ((4 4) (0 0)) ((5 1) (4 0)) ((6 1) (7 0))
    ((7 1) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-decrypt-at-1 trRl_make-at-0)
  (operation nonce-test (added-strand set-decrypt 2) k (5 0)
    (ch-msg lk-0 (cat pt-4 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt k "wrap"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((load lk-0 (cat pt-4 k "init"))
      (stor lk-0 (cat pt-5 k "decrypt"))))
  (label 1770)
  (parent 1018)
  (unrealized (4 3) (5 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk-0))
  (precedes ((1 1) (2 2)) ((2 4) (0 0)) ((3 1) (1 0)) ((3 1) (4 2))
    ((3 2) (2 0)) ((4 4) (0 0)) ((5 1) (4 0)) ((6 1) (7 2))
    ((7 4) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap")
    (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (5 0)
    (ch-msg lk-0 (cat pt-4 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt k "wrap"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-0 (cat pt-4 k "init"))
      (load lk-0 (cat pt-5 k1-0 "wrap")) (send (enc k k1-0))))
  (label 1771)
  (parent 1018)
  (unrealized (4 3) (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (precedes ((1 1) (2 2)) ((1 1) (4 2)) ((2 4) (0 0)) ((3 1) (1 0))
    ((3 2) (2 0)) ((4 4) (0 0)) ((5 1) (6 0)) ((5 2) (4 0))
    ((6 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-decrypt-at-1 trRl_make-at-0)
  (operation nonce-test (added-strand set-decrypt 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt k "wrap")) (ch-msg lk (cat pt-1 k "init"))
    (ch-msg lk-0 (cat pt-3 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt k "wrap"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt k "wrap"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-3 k "init"))
      (stor lk-0 (cat pt-4 k "decrypt"))))
  (label 1772)
  (parent 1019)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "wrap") (k k) (lk lk))
  (precedes ((1 1) (2 2)) ((1 1) (4 2)) ((1 1) (6 0)) ((2 4) (0 0))
    ((3 1) (1 0)) ((3 2) (2 0)) ((4 4) (0 0)) ((5 2) (4 0))
    ((6 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "wrap" "decrypt") (neq "wrap" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule fact-set-decrypt-neq0 trRl_set-wrap-at-1 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation nonce-test (added-strand set-wrap 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt k "wrap")) (ch-msg lk (cat pt-1 k "init"))
    (ch-msg lk-0 (cat pt-3 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt k "wrap"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt k "wrap"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk (cat pt k "wrap")) (stor lk (cat pt-4 k "wrap"))))
  (label 1773)
  (parent 1019)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (precedes ((1 1) (2 2)) ((1 1) (4 2)) ((2 4) (0 0)) ((3 1) (1 0))
    ((3 2) (2 0)) ((4 4) (0 0)) ((5 1) (6 0)) ((5 2) (4 0))
    ((6 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-wrap-at-1 trRl_make-at-0)
  (operation nonce-test (added-strand set-wrap 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt k "wrap")) (ch-msg lk (cat pt-1 k "init"))
    (ch-msg lk-0 (cat pt-3 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt k "wrap"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt k "wrap"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "wrap"))))
  (label 1774)
  (parent 1019)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "wrap") (k k) (lk lk))
  (precedes ((1 1) (2 2)) ((1 1) (4 2)) ((1 1) (6 0)) ((2 4) (0 0))
    ((3 1) (1 0)) ((3 2) (2 0)) ((4 4) (0 0)) ((5 2) (4 0))
    ((6 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "wrap" "decrypt") (neq "wrap" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-decrypt-at-1 fact-set-decrypt-neq0 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation nonce-test (added-strand set-decrypt 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt k "wrap")) (ch-msg lk (cat pt-1 k "init"))
    (ch-msg lk-0 (cat pt-3 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt k "wrap"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt k "wrap"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk (cat pt k "wrap")) (stor lk (cat pt-4 k "decrypt"))))
  (label 1775)
  (parent 1019)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (precedes ((1 1) (2 2)) ((1 1) (4 2)) ((2 4) (0 0)) ((3 1) (1 0))
    ((3 2) (2 0)) ((4 4) (0 0)) ((5 1) (6 2)) ((5 2) (4 0))
    ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt k "wrap")) (ch-msg lk (cat pt-1 k "init"))
    (ch-msg lk-0 (cat pt-3 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt k "wrap"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt k "wrap"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-4 k1 "wrap")) (send (enc k k1))))
  (label 1776)
  (parent 1019)
  (unrealized (6 0) (6 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (2 2)) ((1 1) (4 2)) ((2 4) (0 0)) ((3 1) (1 0))
    ((3 1) (6 2)) ((3 2) (2 0)) ((4 4) (0 0)) ((5 2) (4 0))
    ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt k "wrap")) (ch-msg lk (cat pt-1 k "init"))
    (ch-msg lk-0 (cat pt-3 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt k "wrap"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt k "wrap"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1))))
  (label 1777)
  (parent 1019)
  (unrealized (6 0) (6 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (2 2)) ((1 1) (4 2)) ((1 1) (6 2)) ((2 4) (0 0))
    ((3 1) (1 0)) ((3 2) (2 0)) ((4 4) (0 0)) ((5 2) (4 0))
    ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt k "wrap")) (ch-msg lk (cat pt-1 k "init"))
    (ch-msg lk-0 (cat pt-3 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt k "wrap"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt k "wrap"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt k "wrap"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1))))
  (label 1778)
  (parent 1019)
  (unrealized (6 0) (6 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k1) (lk lk))
  (defstrand make 2 (old old-1) (k k1) (lk lk))
  (precedes ((1 1) (2 2)) ((1 1) (4 2)) ((1 1) (7 0)) ((2 4) (0 0))
    ((3 1) (1 0)) ((3 2) (2 0)) ((4 4) (0 0)) ((5 2) (4 0))
    ((6 1) (4 3)) ((7 1) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule invShearsRule trRl_make-at-0 trRl_make-at-1)
  (operation channel-test (added-strand make 2)
    (ch-msg lk (cat pt-5 k1 "init")) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt k "wrap"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt k "wrap"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init"))
      (send (hash k)))
    ((load lk (cat pt-5 k1 "init")) (stor lk (cat pt-2 k1 "wrap")))
    ((load lk (cat pt-6 old-1)) (stor lk (cat pt-5 k1 "init"))))
  (label 1779)
  (parent 1020)
  (realized)
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old mesg) (pt pt-0 pt-1 pt-2 pval) (k k1 skey) (lk locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (precedes ((1 1) (2 2)) ((1 1) (5 0)) ((2 4) (0 0)) ((3 1) (1 0))
    ((3 2) (2 0)) ((4 4) (0 0)) ((5 1) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation nonce-test (displaced 6 1 set-wrap 2) k (5 0)
    (ch-msg lk (cat pt-1 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt k "wrap"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt k "wrap"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k)))
  (label 1780)
  (parent 1021)
  (unrealized (4 3) (5 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (precedes ((1 1) (2 2)) ((1 1) (4 2)) ((2 4) (0 0)) ((3 1) (1 0))
    ((3 1) (5 0)) ((3 2) (2 0)) ((4 4) (0 0)) ((5 1) (4 0))
    ((6 1) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation nonce-test (added-strand make 2) k (5 0)
    (ch-msg lk (cat pt-1 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt k "wrap"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt k "wrap"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init"))))
  (label 1781)
  (parent 1021)
  (unrealized (4 3) (5 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old mesg) (pt pt-0 pt-1 pt-2 pt-3 pval) (k k1 k1-0 skey)
    (lk locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk))
  (precedes ((1 1) (2 2)) ((1 1) (4 2)) ((2 4) (0 0)) ((3 1) (1 0))
    ((3 1) (6 2)) ((3 2) (2 0)) ((4 4) (0 0)) ((5 1) (4 0))
    ((6 4) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap")
    (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (5 0)
    (ch-msg lk (cat pt-1 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt k "wrap"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt k "wrap"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((recv (hash k)) (recv (hash k1-0)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt-3 k1-0 "wrap")) (send (enc k k1-0))))
  (label 1782)
  (parent 1021)
  (unrealized (4 3) (6 0) (6 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (precedes ((1 1) (2 2)) ((1 1) (4 2)) ((2 4) (0 0)) ((3 1) (1 0))
    ((3 2) (2 0)) ((4 4) (0 0)) ((5 1) (4 0)) ((6 1) (7 0))
    ((7 1) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-wrap-at-1 trRl_make-at-0)
  (operation nonce-test (added-strand set-wrap 2) k (5 0)
    (ch-msg lk-0 (cat pt-4 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt k "wrap"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt k "wrap"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt-5 k "wrap"))))
  (label 1783)
  (parent 1022)
  (unrealized (4 3) (5 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (precedes ((1 1) (2 2)) ((1 1) (4 2)) ((2 4) (0 0)) ((3 1) (1 0))
    ((3 2) (2 0)) ((4 4) (0 0)) ((5 1) (4 0)) ((6 1) (7 0))
    ((7 1) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-decrypt-at-1 trRl_make-at-0)
  (operation nonce-test (added-strand set-decrypt 2) k (5 0)
    (ch-msg lk-0 (cat pt-4 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt k "wrap"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt k "wrap"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((load lk-0 (cat pt-4 k "init"))
      (stor lk-0 (cat pt-5 k "decrypt"))))
  (label 1784)
  (parent 1022)
  (unrealized (4 3) (5 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk-0))
  (precedes ((1 1) (2 2)) ((1 1) (4 2)) ((2 4) (0 0)) ((3 1) (1 0))
    ((3 2) (2 0)) ((4 4) (0 0)) ((5 1) (4 0)) ((6 1) (7 2))
    ((7 4) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap")
    (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (5 0)
    (ch-msg lk-0 (cat pt-4 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt k "wrap"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt k "wrap"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-0 (cat pt-4 k "init"))
      (load lk-0 (cat pt-5 k1-0 "wrap")) (send (enc k k1-0))))
  (label 1785)
  (parent 1022)
  (unrealized (4 3) (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k skey)
    (lk locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (3 2)) ((2 1) (6 3)) ((3 4) (0 0))
    ((4 1) (5 0)) ((4 2) (3 0)) ((4 2) (6 0)) ((5 1) (1 0))
    ((5 1) (6 2)) ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule invShearsRule shearsRule)
  (operation channel-test (displaced 7 2 set-wrap 2)
    (ch-msg lk-0 (cat pt-5 k1 "wrap")) (6 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk (cat pt-3 k "init")) (stor lk (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-4 k "decrypt"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k))))
  (label 1786)
  (parent 1023)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k skey)
    (lk locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (3 2)) ((2 1) (6 3)) ((3 4) (0 0))
    ((4 1) (1 0)) ((4 1) (5 0)) ((4 2) (3 0)) ((4 2) (6 0))
    ((5 1) (2 0)) ((5 1) (6 2)) ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule invShearsRule shearsRule)
  (operation channel-test (displaced 7 2 set-wrap 2)
    (ch-msg lk-0 (cat pt-5 k1 "wrap")) (6 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk (cat pt-3 k "init")) (stor lk (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-4 k "decrypt"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k))))
  (label 1787)
  (parent 1023)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k skey)
    (lk locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (3 2)) ((2 1) (6 3)) ((3 4) (0 0))
    ((4 1) (2 0)) ((4 1) (5 0)) ((4 2) (3 0)) ((4 2) (6 0))
    ((5 1) (6 2)) ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule invShearsRule)
  (operation channel-test (displaced 7 2 set-wrap 2)
    (ch-msg lk-0 (cat pt-5 k1 "wrap")) (6 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk (cat pt-3 k "init")) (stor lk (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-4 k "decrypt"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k))))
  (label 1788)
  (parent 1023)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k skey)
    (lk locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (3 2)) ((2 1) (6 3)) ((3 4) (0 0))
    ((4 1) (5 0)) ((4 2) (3 0)) ((4 2) (6 0)) ((5 1) (2 0))
    ((5 1) (6 2)) ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule invShearsRule)
  (operation channel-test (displaced 7 2 set-wrap 2)
    (ch-msg lk-0 (cat pt-5 k1 "wrap")) (6 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk (cat pt-3 k "init")) (stor lk (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-4 k "decrypt"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k))))
  (label 1789)
  (parent 1023)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 cur mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1) (lk lk-0))
  (defstrand set-wrap 2 (cur cur) (k k1) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (3 2)) ((3 4) (0 0)) ((4 1) (5 0))
    ((4 2) (3 0)) ((4 2) (6 0)) ((5 1) (6 2)) ((5 1) (7 0))
    ((6 4) (0 0)) ((7 1) (6 3)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k "wrap") (cat k1 cur)
    (cat k1 "wrap"))
  (facts (neq cur "decrypt") (neq cur "wrap") (neq "init" "decrypt")
    (neq "init" "wrap"))
  (rule invShearsRule fact-set-decrypt-neq0 trRl_set-wrap-at-1
    gen-st-set-wrap-0 fact-set-wrap-neq0 trRl_make-at-0)
  (operation channel-test (added-strand set-wrap 2)
    (ch-msg lk-0 (cat pt-5 k1 "wrap")) (6 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-4 k "decrypt"))
      (load lk-0 (cat pt-5 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-6 k1 cur)) (stor lk-0 (cat pt-5 k1 "wrap"))))
  (label 1790)
  (parent 1023)
  (unrealized (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k skey)
    (lk locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (precedes ((1 1) (2 2)) ((1 1) (5 3)) ((2 4) (0 0)) ((3 1) (4 0))
    ((3 1) (6 0)) ((3 2) (2 0)) ((4 1) (1 0)) ((4 1) (5 2))
    ((5 4) (0 0)) ((6 1) (1 0)) ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule invShearsRule shearsRule)
  (operation channel-test (displaced 7 1 set-wrap 2)
    (ch-msg lk-0 (cat pt-5 k1 "wrap")) (5 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt-4 k "init")) (stor lk (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt k "wrap"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init"))
      (send (hash k)))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-2 k "decrypt"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-3 old-0)) (stor lk (cat pt-4 k "init"))
      (send (hash k))))
  (label 1791)
  (parent 1024)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k skey)
    (lk locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (precedes ((1 1) (2 2)) ((1 1) (5 3)) ((2 4) (0 0)) ((3 1) (1 0))
    ((3 1) (4 0)) ((3 2) (2 0)) ((4 1) (5 2)) ((5 4) (0 0))
    ((6 1) (1 0)) ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule invShearsRule)
  (operation channel-test (displaced 7 1 set-wrap 2)
    (ch-msg lk-0 (cat pt-5 k1 "wrap")) (5 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt-4 k "init")) (stor lk (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt k "wrap"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init"))
      (send (hash k)))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-2 k "decrypt"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-3 old-0)) (stor lk (cat pt-4 k "init"))
      (send (hash k))))
  (label 1792)
  (parent 1024)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k skey)
    (lk locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (precedes ((1 1) (2 2)) ((1 1) (5 3)) ((2 4) (0 0)) ((3 1) (4 0))
    ((3 2) (2 0)) ((4 1) (1 0)) ((4 1) (5 2)) ((5 4) (0 0))
    ((6 1) (1 0)) ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule invShearsRule)
  (operation channel-test (displaced 7 1 set-wrap 2)
    (ch-msg lk-0 (cat pt-5 k1 "wrap")) (5 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt-4 k "init")) (stor lk (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt k "wrap"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init"))
      (send (hash k)))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-2 k "decrypt"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-3 old-0)) (stor lk (cat pt-4 k "init"))
      (send (hash k))))
  (label 1793)
  (parent 1024)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 cur mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur cur) (k k1) (lk lk))
  (precedes ((1 1) (2 2)) ((2 4) (0 0)) ((3 1) (4 0)) ((3 2) (2 0))
    ((4 1) (5 2)) ((4 1) (7 0)) ((5 4) (0 0)) ((6 1) (1 0))
    ((6 2) (5 0)) ((7 1) (5 3)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k "wrap") (cat k1 cur)
    (cat k1 "wrap"))
  (facts (neq cur "decrypt") (neq cur "wrap") (neq "init" "decrypt")
    (neq "init" "wrap"))
  (rule invShearsRule fact-set-decrypt-neq0 trRl_set-wrap-at-1
    gen-st-set-wrap-0 fact-set-wrap-neq0 trRl_make-at-0)
  (operation channel-test (added-strand set-wrap 2)
    (ch-msg lk (cat pt-3 k1 "wrap")) (5 3))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-5 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt k "wrap"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init"))
      (send (hash k)))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-2 k "decrypt"))
      (load lk (cat pt-3 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-4 old-0)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k)))
    ((load lk (cat pt-6 k1 cur)) (stor lk (cat pt-3 k1 "wrap"))))
  (label 1794)
  (parent 1024)
  (unrealized (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 cur mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur cur) (k k1) (lk lk-0))
  (precedes ((1 1) (2 2)) ((1 1) (5 2)) ((1 1) (7 0)) ((2 4) (0 0))
    ((3 1) (4 0)) ((3 2) (2 0)) ((4 1) (0 0)) ((5 4) (0 0))
    ((6 1) (1 0)) ((6 2) (5 0)) ((7 1) (5 3)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 cur) (cat k1 "wrap"))
  (facts (neq cur "decrypt") (neq cur "wrap") (neq "init" "decrypt")
    (neq "init" "wrap"))
  (rule invShearsRule fact-set-decrypt-neq0 trRl_set-wrap-at-1
    gen-st-set-wrap-0 fact-set-wrap-neq0 trRl_make-at-0)
  (operation channel-test (added-strand set-wrap 2)
    (ch-msg lk-0 (cat pt-3 k1 "wrap")) (5 3))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-5 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt k "wrap"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init"))
      (send (hash k)))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt k "wrap"))
      (load lk-0 (cat pt-3 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-4 old-0)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-6 k1 cur)) (stor lk-0 (cat pt-3 k1 "wrap"))))
  (label 1795)
  (parent 1028)
  (unrealized (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k skey)
    (lk locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (3 2)) ((2 1) (6 3)) ((3 4) (0 0))
    ((4 1) (5 0)) ((4 2) (3 0)) ((4 2) (6 0)) ((5 1) (1 0))
    ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule invShearsRule shearsRule)
  (operation channel-test (displaced 7 2 set-wrap 2)
    (ch-msg lk-0 (cat pt-5 k1 "wrap")) (6 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk (cat pt-3 k "init")) (stor lk (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k))))
  (label 1796)
  (parent 1031)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k skey)
    (lk locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (3 2)) ((2 1) (6 3)) ((3 4) (0 0))
    ((4 1) (1 0)) ((4 1) (5 0)) ((4 2) (3 0)) ((4 2) (6 0))
    ((5 1) (2 0)) ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule invShearsRule shearsRule)
  (operation channel-test (displaced 7 2 set-wrap 2)
    (ch-msg lk-0 (cat pt-5 k1 "wrap")) (6 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk (cat pt-3 k "init")) (stor lk (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k))))
  (label 1797)
  (parent 1031)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k skey)
    (lk locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (3 2)) ((2 1) (6 3)) ((3 4) (0 0))
    ((4 1) (2 0)) ((4 1) (5 0)) ((4 2) (3 0)) ((4 2) (6 0))
    ((5 1) (0 0)) ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule invShearsRule)
  (operation channel-test (displaced 7 2 set-wrap 2)
    (ch-msg lk-0 (cat pt-5 k1 "wrap")) (6 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk (cat pt-3 k "init")) (stor lk (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k))))
  (label 1798)
  (parent 1031)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 cur mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (defstrand set-wrap 2 (cur cur) (k k1) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (3 2)) ((3 4) (0 0)) ((4 1) (5 0))
    ((4 2) (3 0)) ((4 2) (6 0)) ((5 1) (7 0)) ((6 4) (0 0))
    ((7 1) (6 3)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 cur) (cat k1 "wrap"))
  (facts (neq cur "decrypt") (neq cur "wrap") (neq "init" "decrypt")
    (neq "init" "wrap"))
  (rule shearsRule invShearsRule fact-set-decrypt-neq0
    trRl_set-wrap-at-1 gen-st-set-wrap-0 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation channel-test (added-strand set-wrap 2)
    (ch-msg lk-0 (cat pt-5 k1 "wrap")) (6 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-5 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-6 k1 cur)) (stor lk-0 (cat pt-5 k1 "wrap"))))
  (label 1799)
  (parent 1031)
  (unrealized (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 cur mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur cur) (k k1) (lk lk))
  (precedes ((1 1) (2 2)) ((2 4) (0 0)) ((3 1) (4 0)) ((3 1) (5 2))
    ((3 2) (2 0)) ((4 1) (7 0)) ((5 4) (0 0)) ((6 1) (1 0))
    ((6 2) (5 0)) ((7 1) (5 3)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 cur) (cat k1 "wrap"))
  (facts (neq cur "decrypt") (neq cur "wrap") (neq "init" "decrypt")
    (neq "init" "wrap"))
  (rule shearsRule invShearsRule fact-set-decrypt-neq0
    trRl_set-wrap-at-1 gen-st-set-wrap-0 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation channel-test (added-strand set-wrap 2)
    (ch-msg lk (cat pt-3 k1 "wrap")) (5 3))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-5 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt k "wrap"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init"))
      (send (hash k)))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt-3 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-4 old-0)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k)))
    ((load lk (cat pt-6 k1 cur)) (stor lk (cat pt-3 k1 "wrap"))))
  (label 1800)
  (parent 1032)
  (unrealized (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (6 2)) ((2 1) (3 2)) ((2 1) (6 3))
    ((3 4) (0 0)) ((4 1) (5 0)) ((4 2) (3 0)) ((4 2) (6 0))
    ((5 1) (0 0)) ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation channel-test (displaced 7 2 set-wrap 2)
    (ch-msg lk (cat pt-5 k1 "wrap")) (6 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k))))
  (label 1801)
  (parent 1035)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 cur mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (defstrand set-wrap 2 (cur cur) (k k1) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (6 2)) ((2 1) (3 2)) ((2 1) (7 0))
    ((3 4) (0 0)) ((4 1) (5 0)) ((4 2) (3 0)) ((4 2) (6 0))
    ((5 1) (0 0)) ((6 4) (0 0)) ((7 1) (6 3)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 cur) (cat k1 "wrap"))
  (facts (neq cur "decrypt") (neq cur "wrap") (neq "init" "decrypt")
    (neq "init" "wrap"))
  (rule shearsRule invShearsRule fact-set-decrypt-neq0
    trRl_set-wrap-at-1 gen-st-set-wrap-0 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation channel-test (added-strand set-wrap 2)
    (ch-msg lk (cat pt-5 k1 "wrap")) (6 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-5 k1 "wrap")) (send (enc k k1)))
    ((load lk (cat pt-6 k1 cur)) (stor lk (cat pt-5 k1 "wrap"))))
  (label 1802)
  (parent 1035)
  (unrealized (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (precedes ((1 1) (2 2)) ((1 1) (5 3)) ((2 4) (0 0)) ((3 1) (4 0))
    ((3 2) (2 0)) ((4 1) (0 0)) ((5 4) (0 0)) ((6 1) (1 0))
    ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation channel-test (displaced 7 1 set-wrap 2)
    (ch-msg lk-0 (cat pt-5 k1 "wrap")) (5 3))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt k "wrap"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init"))
      (send (hash k)))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt-4 k "init"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init"))
      (send (hash k))))
  (label 1803)
  (parent 1036)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 cur mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur cur) (k k1) (lk lk-0))
  (precedes ((1 1) (2 2)) ((1 1) (7 0)) ((2 4) (0 0)) ((3 1) (4 0))
    ((3 2) (2 0)) ((4 1) (0 0)) ((5 4) (0 0)) ((6 1) (1 0))
    ((6 2) (5 0)) ((7 1) (5 3)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 cur) (cat k1 "wrap"))
  (facts (neq cur "decrypt") (neq cur "wrap") (neq "init" "decrypt")
    (neq "init" "wrap"))
  (rule shearsRule invShearsRule fact-set-decrypt-neq0
    trRl_set-wrap-at-1 gen-st-set-wrap-0 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation channel-test (added-strand set-wrap 2)
    (ch-msg lk-0 (cat pt-3 k1 "wrap")) (5 3))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-5 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt k "wrap"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init"))
      (send (hash k)))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-5 k "init"))
      (load lk-0 (cat pt-3 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-4 old-0)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-6 k1 cur)) (stor lk-0 (cat pt-3 k1 "wrap"))))
  (label 1804)
  (parent 1036)
  (unrealized (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k skey)
    (lk locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (3 2)) ((2 1) (6 3)) ((3 4) (0 0))
    ((4 1) (5 0)) ((4 2) (3 0)) ((4 2) (6 0)) ((5 1) (1 0))
    ((5 1) (6 2)) ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule invShearsRule shearsRule)
  (operation channel-test (displaced 7 2 set-wrap 2)
    (ch-msg lk-0 (cat pt-5 k1 "wrap")) (6 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk (cat pt-3 k "init")) (stor lk (cat pt-4 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-4 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k))))
  (label 1805)
  (parent 1042)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k skey)
    (lk locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (3 2)) ((2 1) (6 3)) ((3 4) (0 0))
    ((4 1) (1 0)) ((4 1) (5 0)) ((4 2) (3 0)) ((4 2) (6 0))
    ((5 1) (2 0)) ((5 1) (6 2)) ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule invShearsRule shearsRule)
  (operation channel-test (displaced 7 2 set-wrap 2)
    (ch-msg lk-0 (cat pt-5 k1 "wrap")) (6 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk (cat pt-3 k "init")) (stor lk (cat pt-4 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-4 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k))))
  (label 1806)
  (parent 1042)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k skey)
    (lk locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (3 2)) ((2 1) (6 3)) ((3 4) (0 0))
    ((4 1) (2 0)) ((4 1) (5 0)) ((4 2) (3 0)) ((4 2) (6 0))
    ((5 1) (6 2)) ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule invShearsRule)
  (operation channel-test (displaced 7 2 set-wrap 2)
    (ch-msg lk-0 (cat pt-5 k1 "wrap")) (6 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk (cat pt-3 k "init")) (stor lk (cat pt-4 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-4 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k))))
  (label 1807)
  (parent 1042)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k skey)
    (lk locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (3 2)) ((2 1) (6 3)) ((3 4) (0 0))
    ((4 1) (5 0)) ((4 2) (3 0)) ((4 2) (6 0)) ((5 1) (2 0))
    ((5 1) (6 2)) ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule invShearsRule)
  (operation channel-test (displaced 7 2 set-wrap 2)
    (ch-msg lk-0 (cat pt-5 k1 "wrap")) (6 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk (cat pt-3 k "init")) (stor lk (cat pt-4 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-4 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k))))
  (label 1808)
  (parent 1042)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (3 2)) ((3 4) (0 0)) ((4 1) (5 0))
    ((4 2) (3 0)) ((4 2) (6 0)) ((5 1) (6 2)) ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation channel-test (displaced 7 5 set-wrap 2)
    (ch-msg lk-0 (cat pt-5 k1 "wrap")) (6 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt-4 k "wrap"))
      (load lk-0 (cat pt-4 k "wrap")) (send (enc k k))))
  (label 1809)
  (parent 1042)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 cur mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk-0))
  (defstrand set-wrap 2 (cur cur) (k k1) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (3 2)) ((3 4) (0 0)) ((4 1) (5 0))
    ((4 2) (3 0)) ((4 2) (6 0)) ((5 1) (6 2)) ((5 1) (7 0))
    ((6 4) (0 0)) ((7 1) (6 3)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 cur) (cat k1 "wrap"))
  (facts (neq cur "decrypt") (neq cur "wrap") (neq "init" "decrypt")
    (neq "init" "wrap"))
  (rule invShearsRule fact-set-decrypt-neq0 trRl_set-wrap-at-1
    gen-st-set-wrap-0 fact-set-wrap-neq0 trRl_make-at-0)
  (operation channel-test (added-strand set-wrap 2)
    (ch-msg lk-0 (cat pt-5 k1 "wrap")) (6 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-4 k "wrap"))
      (load lk-0 (cat pt-5 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-6 k1 cur)) (stor lk-0 (cat pt-5 k1 "wrap"))))
  (label 1810)
  (parent 1042)
  (unrealized (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k skey)
    (lk locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (precedes ((1 1) (2 2)) ((1 1) (5 3)) ((2 4) (0 0)) ((3 1) (4 0))
    ((3 1) (6 0)) ((3 2) (2 0)) ((4 1) (1 0)) ((4 1) (5 2))
    ((5 4) (0 0)) ((6 1) (1 0)) ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule invShearsRule shearsRule)
  (operation channel-test (displaced 7 1 set-wrap 2)
    (ch-msg lk-0 (cat pt-5 k1 "wrap")) (5 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt-4 k "init")) (stor lk (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt k "wrap"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init"))
      (send (hash k)))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-2 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-2 k "wrap"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-3 old-0)) (stor lk (cat pt-4 k "init"))
      (send (hash k))))
  (label 1811)
  (parent 1043)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k skey)
    (lk locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (precedes ((1 1) (2 2)) ((1 1) (5 3)) ((2 4) (0 0)) ((3 1) (1 0))
    ((3 1) (4 0)) ((3 2) (2 0)) ((4 1) (5 2)) ((5 4) (0 0))
    ((6 1) (1 0)) ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule invShearsRule)
  (operation channel-test (displaced 7 1 set-wrap 2)
    (ch-msg lk-0 (cat pt-5 k1 "wrap")) (5 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt-4 k "init")) (stor lk (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt k "wrap"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init"))
      (send (hash k)))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-2 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-2 k "wrap"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-3 old-0)) (stor lk (cat pt-4 k "init"))
      (send (hash k))))
  (label 1812)
  (parent 1043)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k skey)
    (lk locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (precedes ((1 1) (2 2)) ((1 1) (5 3)) ((2 4) (0 0)) ((3 1) (4 0))
    ((3 2) (2 0)) ((4 1) (1 0)) ((4 1) (5 2)) ((5 4) (0 0))
    ((6 1) (1 0)) ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule invShearsRule)
  (operation channel-test (displaced 7 1 set-wrap 2)
    (ch-msg lk-0 (cat pt-5 k1 "wrap")) (5 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt-4 k "init")) (stor lk (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt k "wrap"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init"))
      (send (hash k)))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-2 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-2 k "wrap"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-3 old-0)) (stor lk (cat pt-4 k "init"))
      (send (hash k))))
  (label 1813)
  (parent 1043)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 cur mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur cur) (k k1) (lk lk))
  (precedes ((1 1) (2 2)) ((2 4) (0 0)) ((3 1) (4 0)) ((3 2) (2 0))
    ((4 1) (5 2)) ((4 1) (7 0)) ((5 4) (0 0)) ((6 1) (1 0))
    ((6 2) (5 0)) ((7 1) (5 3)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 cur) (cat k1 "wrap"))
  (facts (neq cur "decrypt") (neq cur "wrap") (neq "init" "decrypt")
    (neq "init" "wrap"))
  (rule invShearsRule fact-set-decrypt-neq0 trRl_set-wrap-at-1
    gen-st-set-wrap-0 fact-set-wrap-neq0 trRl_make-at-0)
  (operation channel-test (added-strand set-wrap 2)
    (ch-msg lk (cat pt-3 k1 "wrap")) (5 3))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-5 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt k "wrap"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init"))
      (send (hash k)))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-2 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-2 k "wrap"))
      (load lk (cat pt-3 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-4 old-0)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k)))
    ((load lk (cat pt-6 k1 cur)) (stor lk (cat pt-3 k1 "wrap"))))
  (label 1814)
  (parent 1043)
  (unrealized (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k skey)
    (lk locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (3 2)) ((2 1) (6 3)) ((3 4) (0 0))
    ((4 1) (5 0)) ((4 2) (3 0)) ((4 2) (6 0)) ((5 1) (1 0))
    ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule invShearsRule shearsRule)
  (operation channel-test (displaced 7 2 set-wrap 2)
    (ch-msg lk-0 (cat pt-5 k1 "wrap")) (6 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk (cat pt-3 k "init")) (stor lk (cat pt-4 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k))))
  (label 1815)
  (parent 1046)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k skey)
    (lk locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (3 2)) ((2 1) (6 3)) ((3 4) (0 0))
    ((4 1) (1 0)) ((4 1) (5 0)) ((4 2) (3 0)) ((4 2) (6 0))
    ((5 1) (2 0)) ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule invShearsRule shearsRule)
  (operation channel-test (displaced 7 2 set-wrap 2)
    (ch-msg lk-0 (cat pt-5 k1 "wrap")) (6 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk (cat pt-3 k "init")) (stor lk (cat pt-4 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k))))
  (label 1816)
  (parent 1046)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k skey)
    (lk locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (3 2)) ((2 1) (6 3)) ((3 4) (0 0))
    ((4 1) (2 0)) ((4 1) (5 0)) ((4 2) (3 0)) ((4 2) (6 0))
    ((5 1) (0 0)) ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule invShearsRule)
  (operation channel-test (displaced 7 2 set-wrap 2)
    (ch-msg lk-0 (cat pt-5 k1 "wrap")) (6 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk (cat pt-3 k "init")) (stor lk (cat pt-4 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k))))
  (label 1817)
  (parent 1046)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (3 2)) ((3 4) (0 0)) ((4 1) (5 0))
    ((4 2) (3 0)) ((4 2) (6 0)) ((5 1) (6 3)) ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation channel-test (displaced 7 5 set-wrap 2)
    (ch-msg lk-0 (cat pt-5 k1 "wrap")) (6 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-4 k "wrap")) (send (enc k k))))
  (label 1818)
  (parent 1046)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 cur mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (defstrand set-wrap 2 (cur cur) (k k1) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (3 2)) ((3 4) (0 0)) ((4 1) (5 0))
    ((4 2) (3 0)) ((4 2) (6 0)) ((5 1) (7 0)) ((6 4) (0 0))
    ((7 1) (6 3)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 cur) (cat k1 "wrap"))
  (facts (neq cur "decrypt") (neq cur "wrap") (neq "init" "decrypt")
    (neq "init" "wrap"))
  (rule shearsRule invShearsRule fact-set-decrypt-neq0
    trRl_set-wrap-at-1 gen-st-set-wrap-0 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation channel-test (added-strand set-wrap 2)
    (ch-msg lk-0 (cat pt-5 k1 "wrap")) (6 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-5 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-6 k1 cur)) (stor lk-0 (cat pt-5 k1 "wrap"))))
  (label 1819)
  (parent 1046)
  (unrealized (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k skey)
    (lk locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (precedes ((1 1) (2 2)) ((1 1) (5 3)) ((2 4) (0 0)) ((3 1) (1 0))
    ((3 1) (4 0)) ((3 1) (5 2)) ((3 2) (2 0)) ((4 1) (0 0))
    ((5 4) (0 0)) ((6 1) (1 0)) ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule invShearsRule)
  (operation channel-test (displaced 7 1 set-wrap 2)
    (ch-msg lk-0 (cat pt-5 k1 "wrap")) (5 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt-4 k "init")) (stor lk (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt k "wrap"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init"))
      (send (hash k)))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-2 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-3 old-0)) (stor lk (cat pt-4 k "init"))
      (send (hash k))))
  (label 1820)
  (parent 1047)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (precedes ((1 1) (2 2)) ((2 4) (0 0)) ((3 1) (4 0)) ((3 1) (5 2))
    ((3 2) (2 0)) ((4 1) (5 3)) ((5 4) (0 0)) ((6 1) (1 0))
    ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation channel-test (displaced 7 4 set-wrap 2)
    (ch-msg lk (cat pt-5 k1 "wrap")) (5 3))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt k "wrap"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init"))
      (send (hash k)))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-2 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt-2 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init"))
      (send (hash k))))
  (label 1821)
  (parent 1047)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 cur mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur cur) (k k1) (lk lk))
  (precedes ((1 1) (2 2)) ((2 4) (0 0)) ((3 1) (4 0)) ((3 1) (5 2))
    ((3 2) (2 0)) ((4 1) (7 0)) ((5 4) (0 0)) ((6 1) (1 0))
    ((6 2) (5 0)) ((7 1) (5 3)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 cur) (cat k1 "wrap"))
  (facts (neq cur "decrypt") (neq cur "wrap") (neq "init" "decrypt")
    (neq "init" "wrap"))
  (rule shearsRule invShearsRule fact-set-decrypt-neq0
    trRl_set-wrap-at-1 gen-st-set-wrap-0 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation channel-test (added-strand set-wrap 2)
    (ch-msg lk (cat pt-3 k1 "wrap")) (5 3))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-5 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt k "wrap"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init"))
      (send (hash k)))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-2 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt-3 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-4 old-0)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k)))
    ((load lk (cat pt-6 k1 cur)) (stor lk (cat pt-3 k1 "wrap"))))
  (label 1822)
  (parent 1047)
  (unrealized (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k skey)
    (lk locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (precedes ((1 1) (2 2)) ((1 1) (4 0)) ((1 1) (5 2)) ((2 4) (0 0))
    ((3 1) (4 0)) ((3 2) (2 0)) ((4 1) (5 3)) ((5 4) (0 0))
    ((6 1) (1 0)) ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule invShearsRule)
  (operation channel-test (displaced 7 4 set-wrap 2)
    (ch-msg lk-0 (cat pt-5 k1 "wrap")) (5 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt-4 k "init")) (stor lk (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt k "wrap"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init"))
      (send (hash k)))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-2 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt k "wrap"))
      (load lk (cat pt-2 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-3 old-0)) (stor lk (cat pt-4 k "init"))
      (send (hash k))))
  (label 1823)
  (parent 1051)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k skey)
    (lk locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (precedes ((1 1) (2 2)) ((1 1) (5 2)) ((2 4) (0 0)) ((3 1) (1 0))
    ((3 1) (4 0)) ((3 2) (2 0)) ((4 1) (5 3)) ((5 4) (0 0))
    ((6 1) (1 0)) ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule invShearsRule)
  (operation channel-test (displaced 7 4 set-wrap 2)
    (ch-msg lk-0 (cat pt-5 k1 "wrap")) (5 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt-4 k "init")) (stor lk (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt k "wrap"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init"))
      (send (hash k)))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-2 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt k "wrap"))
      (load lk (cat pt-2 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-3 old-0)) (stor lk (cat pt-4 k "init"))
      (send (hash k))))
  (label 1824)
  (parent 1051)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 cur mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur cur) (k k1) (lk lk-0))
  (precedes ((1 1) (2 2)) ((1 1) (5 2)) ((1 1) (7 0)) ((2 4) (0 0))
    ((3 1) (4 0)) ((3 2) (2 0)) ((4 1) (0 0)) ((5 4) (0 0))
    ((6 1) (1 0)) ((6 2) (5 0)) ((7 1) (5 3)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 cur) (cat k1 "wrap"))
  (facts (neq cur "decrypt") (neq cur "wrap") (neq "init" "decrypt")
    (neq "init" "wrap"))
  (rule invShearsRule fact-set-decrypt-neq0 trRl_set-wrap-at-1
    gen-st-set-wrap-0 fact-set-wrap-neq0 trRl_make-at-0)
  (operation channel-test (added-strand set-wrap 2)
    (ch-msg lk-0 (cat pt-3 k1 "wrap")) (5 3))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-5 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt k "wrap"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init"))
      (send (hash k)))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-2 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt k "wrap"))
      (load lk-0 (cat pt-3 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-4 old-0)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-6 k1 cur)) (stor lk-0 (cat pt-3 k1 "wrap"))))
  (label 1825)
  (parent 1051)
  (unrealized (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (6 2)) ((2 1) (3 2)) ((2 1) (6 3))
    ((3 4) (0 0)) ((4 1) (5 0)) ((4 2) (3 0)) ((4 2) (6 0))
    ((5 1) (0 0)) ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation channel-test (displaced 7 2 set-wrap 2)
    (ch-msg lk (cat pt-5 k1 "wrap")) (6 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k))))
  (label 1826)
  (parent 1054)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k skey)
    (lk locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (6 2)) ((2 1) (3 2)) ((3 4) (0 0))
    ((4 1) (1 0)) ((4 1) (5 0)) ((4 2) (3 0)) ((4 2) (6 0))
    ((5 1) (2 0)) ((5 1) (6 3)) ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule invShearsRule shearsRule)
  (operation channel-test (displaced 7 5 set-wrap 2)
    (ch-msg lk-0 (cat pt-5 k1 "wrap")) (6 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk (cat pt-3 k "init")) (stor lk (cat pt-4 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-4 k "wrap")) (send (enc k k))))
  (label 1827)
  (parent 1054)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k skey)
    (lk locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (6 2)) ((2 1) (3 2)) ((3 4) (0 0))
    ((4 1) (2 0)) ((4 1) (5 0)) ((4 2) (3 0)) ((4 2) (6 0))
    ((5 1) (6 3)) ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule invShearsRule)
  (operation channel-test (displaced 7 5 set-wrap 2)
    (ch-msg lk-0 (cat pt-5 k1 "wrap")) (6 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk (cat pt-3 k "init")) (stor lk (cat pt-4 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-4 k "wrap")) (send (enc k k))))
  (label 1828)
  (parent 1054)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k skey)
    (lk locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (6 2)) ((2 1) (3 2)) ((3 4) (0 0))
    ((4 1) (1 0)) ((4 1) (5 0)) ((4 2) (3 0)) ((4 2) (6 0))
    ((5 1) (6 3)) ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule invShearsRule)
  (operation channel-test (displaced 7 5 set-wrap 2)
    (ch-msg lk-0 (cat pt-5 k1 "wrap")) (6 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk (cat pt-3 k "init")) (stor lk (cat pt-4 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-4 k "wrap")) (send (enc k k))))
  (label 1829)
  (parent 1054)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 cur mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (defstrand set-wrap 2 (cur cur) (k k1) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (6 2)) ((2 1) (3 2)) ((2 1) (7 0))
    ((3 4) (0 0)) ((4 1) (5 0)) ((4 2) (3 0)) ((4 2) (6 0))
    ((5 1) (0 0)) ((6 4) (0 0)) ((7 1) (6 3)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 cur) (cat k1 "wrap"))
  (facts (neq cur "decrypt") (neq cur "wrap") (neq "init" "decrypt")
    (neq "init" "wrap"))
  (rule shearsRule invShearsRule fact-set-decrypt-neq0
    trRl_set-wrap-at-1 gen-st-set-wrap-0 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation channel-test (added-strand set-wrap 2)
    (ch-msg lk (cat pt-5 k1 "wrap")) (6 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-5 k1 "wrap")) (send (enc k k1)))
    ((load lk (cat pt-6 k1 cur)) (stor lk (cat pt-5 k1 "wrap"))))
  (label 1830)
  (parent 1054)
  (unrealized (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (precedes ((1 1) (2 2)) ((1 1) (5 3)) ((2 4) (0 0)) ((3 1) (4 0))
    ((3 2) (2 0)) ((4 1) (0 0)) ((5 4) (0 0)) ((6 1) (1 0))
    ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation channel-test (displaced 7 1 set-wrap 2)
    (ch-msg lk-0 (cat pt-5 k1 "wrap")) (5 3))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt k "wrap"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init"))
      (send (hash k)))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-2 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt-4 k "init"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init"))
      (send (hash k))))
  (label 1831)
  (parent 1055)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k skey)
    (lk locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (precedes ((1 1) (2 2)) ((2 4) (0 0)) ((3 1) (4 0)) ((3 1) (6 0))
    ((3 2) (2 0)) ((4 1) (1 0)) ((4 1) (5 3)) ((5 4) (0 0))
    ((6 1) (1 0)) ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule invShearsRule shearsRule)
  (operation channel-test (displaced 7 4 set-wrap 2)
    (ch-msg lk-0 (cat pt-5 k1 "wrap")) (5 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt-4 k "init")) (stor lk (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt k "wrap"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init"))
      (send (hash k)))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-2 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-4 k "init"))
      (load lk (cat pt-2 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-3 old-0)) (stor lk (cat pt-4 k "init"))
      (send (hash k))))
  (label 1832)
  (parent 1055)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k skey)
    (lk locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (precedes ((1 1) (2 2)) ((2 4) (0 0)) ((3 1) (1 0)) ((3 1) (4 0))
    ((3 2) (2 0)) ((4 1) (5 3)) ((5 4) (0 0)) ((6 1) (1 0))
    ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule invShearsRule)
  (operation channel-test (displaced 7 4 set-wrap 2)
    (ch-msg lk-0 (cat pt-5 k1 "wrap")) (5 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt-4 k "init")) (stor lk (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt k "wrap"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init"))
      (send (hash k)))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-2 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-4 k "init"))
      (load lk (cat pt-2 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-3 old-0)) (stor lk (cat pt-4 k "init"))
      (send (hash k))))
  (label 1833)
  (parent 1055)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 cur mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur cur) (k k1) (lk lk-0))
  (precedes ((1 1) (2 2)) ((1 1) (7 0)) ((2 4) (0 0)) ((3 1) (4 0))
    ((3 2) (2 0)) ((4 1) (0 0)) ((5 4) (0 0)) ((6 1) (1 0))
    ((6 2) (5 0)) ((7 1) (5 3)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 cur) (cat k1 "wrap"))
  (facts (neq cur "decrypt") (neq cur "wrap") (neq "init" "decrypt")
    (neq "init" "wrap"))
  (rule shearsRule invShearsRule fact-set-decrypt-neq0
    trRl_set-wrap-at-1 gen-st-set-wrap-0 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation channel-test (added-strand set-wrap 2)
    (ch-msg lk-0 (cat pt-3 k1 "wrap")) (5 3))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-5 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt k "wrap"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init"))
      (send (hash k)))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-2 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-5 k "init"))
      (load lk-0 (cat pt-3 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-4 old-0)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-6 k1 cur)) (stor lk-0 (cat pt-3 k1 "wrap"))))
  (label 1834)
  (parent 1055)
  (unrealized (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k skey)
    (lk locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand set-wrap 2 (cur "wrap") (k k) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (3 2)) ((2 1) (5 3)) ((2 1) (6 0))
    ((3 4) (0 0)) ((4 1) (1 0)) ((4 2) (3 0)) ((4 2) (5 0))
    ((5 4) (0 0)) ((6 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "wrap" "decrypt") (neq "wrap" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule fact-set-decrypt-neq0 trRl_set-wrap-at-1 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation nonce-test (added-strand set-wrap 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk (cat pt-3 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-1 k "wrap")) (stor lk (cat pt-4 k "wrap"))))
  (label 1835)
  (parent 1058)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k skey)
    (lk locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand set-decrypt 2 (cur "wrap") (k k) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (3 2)) ((2 1) (5 3)) ((2 1) (6 0))
    ((3 4) (0 0)) ((4 1) (1 0)) ((4 2) (3 0)) ((4 2) (5 0))
    ((5 4) (0 0)) ((6 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "wrap" "decrypt") (neq "wrap" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-decrypt-at-1 fact-set-decrypt-neq0 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation nonce-test (added-strand set-decrypt 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk (cat pt-3 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-1 k "wrap")) (stor lk (cat pt-4 k "decrypt"))))
  (label 1836)
  (parent 1058)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k k1 skey)
    (lk locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (3 2)) ((2 1) (5 3)) ((3 4) (0 0))
    ((4 1) (1 0)) ((4 1) (6 2)) ((4 2) (3 0)) ((4 2) (5 0))
    ((5 4) (0 0)) ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk (cat pt-3 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1))))
  (label 1837)
  (parent 1058)
  (unrealized (6 0) (6 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k k1 skey)
    (lk locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (3 2)) ((2 1) (5 3)) ((2 1) (6 2))
    ((3 4) (0 0)) ((4 1) (1 0)) ((4 2) (3 0)) ((4 2) (5 0))
    ((5 4) (0 0)) ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk (cat pt-3 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1))))
  (label 1838)
  (parent 1058)
  (unrealized (6 0) (6 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k k1 skey)
    (lk locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (6 2)) ((2 1) (3 2)) ((2 1) (5 3))
    ((3 4) (0 0)) ((4 1) (1 0)) ((4 2) (3 0)) ((4 2) (5 0))
    ((5 4) (0 0)) ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk (cat pt-3 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1))))
  (label 1839)
  (parent 1058)
  (unrealized (6 0) (6 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k1) (lk lk-0))
  (defstrand make 2 (old old-1) (k k1) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (3 2)) ((3 4) (0 0)) ((4 1) (7 0))
    ((4 2) (3 0)) ((4 2) (5 0)) ((5 4) (0 0)) ((6 1) (5 3))
    ((7 1) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule invShearsRule trRl_make-at-0 trRl_make-at-1)
  (operation channel-test (added-strand make 2)
    (ch-msg lk-0 (cat pt-5 k1 "init")) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-5 k1 "init")) (stor lk-0 (cat pt-4 k1 "wrap")))
    ((load lk-0 (cat pt-6 old-1)) (stor lk-0 (cat pt-5 k1 "init"))))
  (label 1840)
  (parent 1060)
  (realized)
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k1) (lk lk))
  (defstrand make 2 (old old-1) (k k1) (lk lk))
  (precedes ((1 1) (2 2)) ((2 4) (0 0)) ((3 1) (4 2)) ((3 1) (7 0))
    ((3 2) (2 0)) ((4 4) (0 0)) ((5 1) (1 0)) ((5 2) (4 0))
    ((6 1) (4 3)) ((7 1) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule invShearsRule trRl_make-at-0 trRl_make-at-1)
  (operation channel-test (added-strand make 2)
    (ch-msg lk (cat pt-5 k1 "init")) (6 0))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt k "wrap"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init"))
      (send (hash k)))
    ((load lk (cat pt-5 k1 "init")) (stor lk (cat pt-2 k1 "wrap")))
    ((load lk (cat pt-6 old-1)) (stor lk (cat pt-5 k1 "init"))))
  (label 1841)
  (parent 1062)
  (realized)
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (3 2)) ((2 1) (5 3)) ((3 4) (0 0))
    ((4 1) (1 0)) ((4 1) (5 2)) ((4 2) (3 0)) ((5 4) (0 0))
    ((6 1) (7 0)) ((6 2) (5 0)) ((7 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-decrypt-at-1 trRl_make-at-0)
  (operation nonce-test (added-strand set-decrypt 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk (cat pt-3 k "init")) (ch-msg lk-0 (cat pt-5 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-4 old-1)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-5 k "init"))
      (stor lk-0 (cat pt-6 k "decrypt"))))
  (label 1842)
  (parent 1063)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "wrap") (k k) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (3 2)) ((2 1) (5 3)) ((2 1) (7 0))
    ((3 4) (0 0)) ((4 1) (1 0)) ((4 1) (5 2)) ((4 2) (3 0))
    ((5 4) (0 0)) ((6 2) (5 0)) ((7 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "wrap" "decrypt") (neq "wrap" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule fact-set-decrypt-neq0 trRl_set-wrap-at-1 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation nonce-test (added-strand set-wrap 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk (cat pt-3 k "init")) (ch-msg lk-0 (cat pt-5 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-4 old-1)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k)))
    ((load lk (cat pt-1 k "wrap")) (stor lk (cat pt-6 k "wrap"))))
  (label 1843)
  (parent 1063)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 old-2 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-0))
  (defstrand make 2 (old old-2) (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((2 1) (3 2)) ((2 1) (5 3)) ((3 4) (0 0))
    ((4 1) (1 0)) ((4 1) (5 2)) ((4 2) (3 0)) ((5 4) (0 0))
    ((6 2) (5 0)) ((7 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation nonce-test (added-strand make 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk (cat pt-3 k "init")) (ch-msg lk-0 (cat pt-5 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-4 old-1)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k)))
    ((load lk-1 (cat pt-6 old-2)) (stor lk-1 (cat pt-7 k "init"))))
  (label 1844)
  (parent 1063)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (3 2)) ((2 1) (5 3)) ((3 4) (0 0))
    ((4 1) (1 0)) ((4 1) (5 2)) ((4 2) (3 0)) ((5 4) (0 0))
    ((6 1) (7 0)) ((6 2) (5 0)) ((7 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-wrap-at-1 trRl_make-at-0)
  (operation nonce-test (added-strand set-wrap 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk (cat pt-3 k "init")) (ch-msg lk-0 (cat pt-5 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-4 old-1)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-5 k "init")) (stor lk-0 (cat pt-6 k "wrap"))))
  (label 1845)
  (parent 1063)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (3 2)) ((2 1) (5 3)) ((3 4) (0 0))
    ((4 1) (5 2)) ((4 1) (7 0)) ((4 2) (3 0)) ((5 4) (0 0))
    ((6 2) (5 0)) ((7 1) (1 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule invShearsRule shearsRule trRl_set-wrap-at-1 trRl_make-at-0)
  (operation nonce-test (added-strand set-wrap 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk (cat pt-3 k "init")) (ch-msg lk-0 (cat pt-5 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-4 old-1)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k)))
    ((load lk (cat pt-3 k "init")) (stor lk (cat pt-6 k "wrap"))))
  (label 1846)
  (parent 1063)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "wrap") (k k) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (3 2)) ((2 1) (5 3)) ((2 1) (7 0))
    ((3 4) (0 0)) ((4 1) (1 0)) ((4 1) (5 2)) ((4 2) (3 0))
    ((5 4) (0 0)) ((6 2) (5 0)) ((7 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "wrap" "decrypt") (neq "wrap" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-decrypt-at-1 fact-set-decrypt-neq0 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation nonce-test (added-strand set-decrypt 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk (cat pt-3 k "init")) (ch-msg lk-0 (cat pt-5 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-4 old-1)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k)))
    ((load lk (cat pt-1 k "wrap")) (stor lk (cat pt-6 k "decrypt"))))
  (label 1847)
  (parent 1063)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (3 2)) ((2 1) (5 3)) ((3 4) (0 0))
    ((4 1) (5 2)) ((4 1) (7 0)) ((4 2) (3 0)) ((5 4) (0 0))
    ((6 2) (5 0)) ((7 1) (1 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule invShearsRule shearsRule trRl_set-decrypt-at-1 trRl_make-at-0)
  (operation nonce-test (added-strand set-decrypt 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk (cat pt-3 k "init")) (ch-msg lk-0 (cat pt-5 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-4 old-1)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k)))
    ((load lk (cat pt-3 k "init")) (stor lk (cat pt-6 k "decrypt"))))
  (label 1848)
  (parent 1063)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (3 2)) ((2 1) (5 3)) ((3 4) (0 0))
    ((4 1) (1 0)) ((4 1) (5 2)) ((4 1) (7 2)) ((4 2) (3 0))
    ((5 4) (0 0)) ((6 2) (5 0)) ((7 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk (cat pt-3 k "init")) (ch-msg lk-0 (cat pt-5 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-4 old-1)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-6 k1 "wrap")) (send (enc k k1))))
  (label 1849)
  (parent 1063)
  (unrealized (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (3 2)) ((2 1) (5 3)) ((3 4) (0 0))
    ((4 1) (1 0)) ((4 1) (5 2)) ((4 2) (3 0)) ((5 4) (0 0))
    ((6 1) (7 2)) ((6 2) (5 0)) ((7 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk (cat pt-3 k "init")) (ch-msg lk-0 (cat pt-5 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-4 old-1)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-5 k "init"))
      (load lk-0 (cat pt-6 k1 "wrap")) (send (enc k k1))))
  (label 1850)
  (parent 1063)
  (unrealized (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (3 2)) ((2 1) (5 3)) ((2 1) (7 2))
    ((3 4) (0 0)) ((4 1) (1 0)) ((4 1) (5 2)) ((4 2) (3 0))
    ((5 4) (0 0)) ((6 2) (5 0)) ((7 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk (cat pt-3 k "init")) (ch-msg lk-0 (cat pt-5 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-4 old-1)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-6 k1 "wrap")) (send (enc k k1))))
  (label 1851)
  (parent 1063)
  (unrealized (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (7 2)) ((2 1) (3 2)) ((2 1) (5 3))
    ((3 4) (0 0)) ((4 1) (1 0)) ((4 1) (5 2)) ((4 2) (3 0))
    ((5 4) (0 0)) ((6 2) (5 0)) ((7 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk (cat pt-3 k "init")) (ch-msg lk-0 (cat pt-5 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-4 old-1)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-6 k1 "wrap")) (send (enc k k1))))
  (label 1852)
  (parent 1063)
  (unrealized (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (deflistener k)
  (precedes ((1 1) (2 0)) ((2 1) (3 2)) ((2 1) (6 0)) ((3 4) (0 0))
    ((4 1) (5 2)) ((4 2) (3 0)) ((5 4) (0 0)) ((6 1) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation nonce-test (displaced 7 2 set-wrap 2) k (6 0)
    (ch-msg lk (cat pt-0 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k)))
  (label 1853)
  (parent 1066)
  (unrealized (5 3) (6 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (deflistener k)
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (3 2)) ((3 4) (0 0))
    ((4 1) (5 2)) ((4 2) (3 0)) ((5 4) (0 0)) ((6 1) (5 0))
    ((7 1) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation nonce-test (added-strand make 2) k (6 0)
    (ch-msg lk (cat pt-0 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init"))))
  (label 1854)
  (parent 1066)
  (unrealized (5 3) (6 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (deflistener k)
  (precedes ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (3 2)) ((3 4) (0 0))
    ((4 1) (6 0)) ((4 2) (3 0)) ((5 4) (0 0)) ((6 1) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation nonce-test (displaced 7 4 make 2) k (6 0)
    (ch-msg lk (cat pt-0 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k)))
  (label 1855)
  (parent 1066)
  (unrealized (5 3) (6 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (deflistener k)
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (7 2)) ((2 1) (3 2)) ((3 4) (0 0))
    ((4 1) (5 2)) ((4 2) (3 0)) ((5 4) (0 0)) ((6 1) (5 0))
    ((7 4) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap")
    (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (6 0)
    (ch-msg lk (cat pt-0 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((recv (hash k)) (recv (hash k1-0)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-5 k1-0 "wrap")) (send (enc k k1-0))))
  (label 1856)
  (parent 1066)
  (unrealized (5 3) (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (deflistener k)
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((2 1) (3 2)) ((3 4) (0 0)) ((4 1) (6 0))
    ((4 2) (3 0)) ((5 4) (0 0)) ((6 1) (5 0)) ((7 1) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation nonce-test (added-strand make 2) k (6 0)
    (ch-msg lk-0 (cat pt-3 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init"))))
  (label 1857)
  (parent 1067)
  (unrealized (5 3) (6 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (deflistener k)
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (3 2)) ((3 4) (0 0)) ((4 1) (7 2))
    ((4 2) (3 0)) ((5 4) (0 0)) ((6 1) (5 0)) ((7 4) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap")
    (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (6 0)
    (ch-msg lk-0 (cat pt-3 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-5 k1-0 "wrap")) (send (enc k k1-0))))
  (label 1858)
  (parent 1067)
  (unrealized (5 3) (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k1) (lk lk-0))
  (defstrand make 2 (old old-1) (k k1) (lk lk-0))
  (precedes ((1 1) (2 2)) ((1 1) (4 2)) ((1 1) (7 0)) ((2 4) (0 0))
    ((3 2) (2 0)) ((4 4) (0 0)) ((5 1) (1 0)) ((5 2) (4 0))
    ((6 1) (4 3)) ((7 1) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule invShearsRule trRl_make-at-0 trRl_make-at-1)
  (operation channel-test (added-strand make 2)
    (ch-msg lk-0 (cat pt-5 k1 "init")) (6 0))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt k "wrap"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt k "wrap"))
      (load lk-0 (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-5 k1 "init")) (stor lk-0 (cat pt-2 k1 "wrap")))
    ((load lk-0 (cat pt-6 old-1)) (stor lk-0 (cat pt-5 k1 "init"))))
  (label 1859)
  (parent 1069)
  (realized)
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (precedes ((1 1) (2 0)) ((2 1) (3 2)) ((2 1) (6 0)) ((3 4) (0 0))
    ((4 2) (3 0)) ((5 4) (0 0)) ((6 1) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation nonce-test (displaced 7 2 set-wrap 2) k (6 0)
    (ch-msg lk (cat pt-0 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k)))
  (label 1860)
  (parent 1071)
  (unrealized (5 3) (6 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (3 2)) ((2 1) (5 2))
    ((3 4) (0 0)) ((4 2) (3 0)) ((5 4) (0 0)) ((6 1) (5 0))
    ((7 1) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation nonce-test (added-strand make 2) k (6 0)
    (ch-msg lk (cat pt-0 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init"))))
  (label 1861)
  (parent 1071)
  (unrealized (5 3) (6 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (precedes ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (3 2)) ((2 1) (5 2))
    ((3 4) (0 0)) ((4 1) (6 0)) ((4 2) (3 0)) ((5 4) (0 0))
    ((6 1) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation nonce-test (displaced 7 4 make 2) k (6 0)
    (ch-msg lk (cat pt-0 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k)))
  (label 1862)
  (parent 1071)
  (unrealized (5 3) (6 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (7 2)) ((2 1) (3 2)) ((2 1) (5 2))
    ((3 4) (0 0)) ((4 2) (3 0)) ((5 4) (0 0)) ((6 1) (5 0))
    ((7 4) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap")
    (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (6 0)
    (ch-msg lk (cat pt-0 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((recv (hash k)) (recv (hash k1-0)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-5 k1-0 "wrap")) (send (enc k k1-0))))
  (label 1863)
  (parent 1071)
  (unrealized (5 3) (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((2 1) (3 2)) ((2 1) (5 2)) ((3 4) (0 0))
    ((4 1) (6 0)) ((4 2) (3 0)) ((5 4) (0 0)) ((6 1) (5 0))
    ((7 1) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation nonce-test (added-strand make 2) k (6 0)
    (ch-msg lk-0 (cat pt-3 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init"))))
  (label 1864)
  (parent 1072)
  (unrealized (5 3) (6 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (3 2)) ((2 1) (5 2)) ((3 4) (0 0))
    ((4 1) (7 0)) ((4 2) (3 0)) ((5 4) (0 0)) ((6 1) (5 0))
    ((7 1) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-wrap-at-1 trRl_make-at-0)
  (operation nonce-test (added-strand set-wrap 2) k (6 0)
    (ch-msg lk-0 (cat pt-3 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-5 k "wrap"))))
  (label 1865)
  (parent 1072)
  (unrealized (5 3) (6 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (3 2)) ((2 1) (5 2)) ((3 4) (0 0))
    ((4 1) (7 0)) ((4 2) (3 0)) ((5 4) (0 0)) ((6 1) (5 0))
    ((7 1) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-decrypt-at-1 trRl_make-at-0)
  (operation nonce-test (added-strand set-decrypt 2) k (6 0)
    (ch-msg lk-0 (cat pt-3 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 k "init"))
      (stor lk-0 (cat pt-5 k "decrypt"))))
  (label 1866)
  (parent 1072)
  (unrealized (5 3) (6 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (3 2)) ((2 1) (5 2)) ((3 4) (0 0))
    ((4 1) (7 2)) ((4 2) (3 0)) ((5 4) (0 0)) ((6 1) (5 0))
    ((7 4) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap")
    (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (6 0)
    (ch-msg lk-0 (cat pt-3 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-5 k1-0 "wrap")) (send (enc k k1-0))))
  (label 1867)
  (parent 1072)
  (unrealized (5 3) (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand set-wrap 2 (cur "wrap") (k k) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (5 2)) ((2 1) (3 2)) ((2 1) (5 3))
    ((2 1) (6 0)) ((3 4) (0 0)) ((4 2) (3 0)) ((4 2) (5 0))
    ((5 4) (0 0)) ((6 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "wrap" "decrypt") (neq "wrap" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule fact-set-decrypt-neq0 trRl_set-wrap-at-1 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation nonce-test (added-strand set-wrap 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-1 k "wrap")) (stor lk (cat pt-4 k "wrap"))))
  (label 1868)
  (parent 1074)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand set-decrypt 2 (cur "wrap") (k k) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (5 2)) ((2 1) (3 2)) ((2 1) (5 3))
    ((2 1) (6 0)) ((3 4) (0 0)) ((4 2) (3 0)) ((4 2) (5 0))
    ((5 4) (0 0)) ((6 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "wrap" "decrypt") (neq "wrap" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-decrypt-at-1 fact-set-decrypt-neq0 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation nonce-test (added-strand set-decrypt 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-1 k "wrap")) (stor lk (cat pt-4 k "decrypt"))))
  (label 1869)
  (parent 1074)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (precedes ((1 1) (2 0)) ((1 1) (5 2)) ((2 1) (3 2)) ((2 1) (5 3))
    ((3 4) (0 0)) ((4 1) (6 2)) ((4 2) (3 0)) ((4 2) (5 0))
    ((5 4) (0 0)) ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-4 k1 "wrap")) (send (enc k k1))))
  (label 1870)
  (parent 1074)
  (unrealized (6 0) (6 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (5 2)) ((2 1) (3 2)) ((2 1) (5 3))
    ((2 1) (6 2)) ((3 4) (0 0)) ((4 2) (3 0)) ((4 2) (5 0))
    ((5 4) (0 0)) ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1))))
  (label 1871)
  (parent 1074)
  (unrealized (6 0) (6 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (5 2)) ((1 1) (6 2)) ((2 1) (3 2))
    ((2 1) (5 3)) ((3 4) (0 0)) ((4 2) (3 0)) ((4 2) (5 0))
    ((5 4) (0 0)) ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1))))
  (label 1872)
  (parent 1074)
  (unrealized (6 0) (6 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k1) (lk lk))
  (defstrand make 2 (old old-1) (k k1) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (5 2)) ((2 1) (3 2)) ((2 1) (7 0))
    ((3 4) (0 0)) ((4 2) (3 0)) ((4 2) (5 0)) ((5 4) (0 0))
    ((6 1) (5 3)) ((7 1) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule invShearsRule trRl_make-at-0 trRl_make-at-1)
  (operation channel-test (added-strand make 2)
    (ch-msg lk (cat pt-5 k1 "init")) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((load lk (cat pt-5 k1 "init")) (stor lk (cat pt-4 k1 "wrap")))
    ((load lk (cat pt-6 old-1)) (stor lk (cat pt-5 k1 "init"))))
  (label 1873)
  (parent 1075)
  (realized)
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "wrap") (k k) (lk lk-0))
  (precedes ((1 1) (2 2)) ((1 1) (4 3)) ((1 1) (6 0)) ((2 4) (0 0))
    ((3 2) (2 0)) ((4 4) (0 0)) ((5 1) (1 0)) ((5 2) (4 0))
    ((6 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "wrap" "decrypt") (neq "wrap" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule fact-set-decrypt-neq0 trRl_set-wrap-at-1 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation nonce-test (added-strand set-wrap 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-1 k "init")) (ch-msg lk-0 (cat pt k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init")))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt k "wrap"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt k "wrap")) (stor lk-0 (cat pt-4 k "wrap"))))
  (label 1874)
  (parent 1076)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "wrap") (k k) (lk lk-0))
  (precedes ((1 1) (2 2)) ((1 1) (4 3)) ((1 1) (6 0)) ((2 4) (0 0))
    ((3 2) (2 0)) ((4 4) (0 0)) ((5 1) (1 0)) ((5 2) (4 0))
    ((6 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "wrap" "decrypt") (neq "wrap" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-decrypt-at-1 fact-set-decrypt-neq0 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation nonce-test (added-strand set-decrypt 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-1 k "init")) (ch-msg lk-0 (cat pt k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init")))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt k "wrap"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt k "wrap")) (stor lk-0 (cat pt-4 k "decrypt"))))
  (label 1875)
  (parent 1076)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (precedes ((1 1) (2 2)) ((1 1) (4 3)) ((2 4) (0 0)) ((3 2) (2 0))
    ((4 4) (0 0)) ((5 1) (1 0)) ((5 1) (6 2)) ((5 2) (4 0))
    ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-1 k "init")) (ch-msg lk-0 (cat pt k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init")))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt k "wrap"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-4 k1 "wrap")) (send (enc k k1))))
  (label 1876)
  (parent 1076)
  (unrealized (6 0) (6 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk-0))
  (precedes ((1 1) (2 2)) ((1 1) (4 3)) ((1 1) (6 2)) ((2 4) (0 0))
    ((3 2) (2 0)) ((4 4) (0 0)) ((5 1) (1 0)) ((5 2) (4 0))
    ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-1 k "init")) (ch-msg lk-0 (cat pt k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init")))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt k "wrap"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt k "wrap"))
      (load lk-0 (cat pt-4 k1 "wrap")) (send (enc k k1))))
  (label 1877)
  (parent 1076)
  (unrealized (6 0) (6 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (2 2)) ((1 1) (4 3)) ((2 4) (0 0)) ((3 1) (6 2))
    ((3 2) (2 0)) ((4 4) (0 0)) ((5 1) (1 0)) ((5 2) (4 0))
    ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-1 k "init")) (ch-msg lk-0 (cat pt k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init")))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt k "wrap"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1))))
  (label 1878)
  (parent 1076)
  (unrealized (6 0) (6 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k1) (lk lk-0))
  (defstrand make 2 (old old-1) (k k1) (lk lk-0))
  (precedes ((1 1) (2 2)) ((1 1) (7 0)) ((2 4) (0 0)) ((3 2) (2 0))
    ((4 4) (0 0)) ((5 1) (1 0)) ((5 2) (4 0)) ((6 1) (4 3))
    ((7 1) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule invShearsRule trRl_make-at-0 trRl_make-at-1)
  (operation channel-test (added-strand make 2)
    (ch-msg lk-0 (cat pt-5 k1 "init")) (6 0))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt k "wrap"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-4 k "init"))
      (load lk-0 (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-5 k1 "init")) (stor lk-0 (cat pt-2 k1 "wrap")))
    ((load lk-0 (cat pt-6 old-1)) (stor lk-0 (cat pt-5 k1 "init"))))
  (label 1879)
  (parent 1077)
  (realized)
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((1 1) (5 2)) ((2 1) (3 2)) ((2 1) (5 3))
    ((3 4) (0 0)) ((4 2) (3 0)) ((5 4) (0 0)) ((6 1) (7 0))
    ((6 2) (5 0)) ((7 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-decrypt-at-1 trRl_make-at-0)
  (operation nonce-test (added-strand set-decrypt 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init")) (ch-msg lk-1 (cat pt-5 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-1 (cat pt-4 old-1)) (stor lk-1 (cat pt-5 k "init"))
      (send (hash k)))
    ((load lk-1 (cat pt-5 k "init"))
      (stor lk-1 (cat pt-6 k "decrypt"))))
  (label 1880)
  (parent 1078)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand set-wrap 2 (cur "wrap") (k k) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (5 2)) ((2 1) (3 2)) ((2 1) (5 3))
    ((2 1) (7 0)) ((3 4) (0 0)) ((4 2) (3 0)) ((5 4) (0 0))
    ((6 2) (5 0)) ((7 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "wrap" "decrypt") (neq "wrap" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule fact-set-decrypt-neq0 trRl_set-wrap-at-1 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation nonce-test (added-strand set-wrap 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init")) (ch-msg lk-1 (cat pt-5 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-1 (cat pt-4 old-1)) (stor lk-1 (cat pt-5 k "init"))
      (send (hash k)))
    ((load lk (cat pt-1 k "wrap")) (stor lk (cat pt-6 k "wrap"))))
  (label 1881)
  (parent 1078)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 old-2 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k skey)
    (lk lk-0 lk-1 lk-2 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand make 2 (old old-2) (k k) (lk lk-2))
  (precedes ((1 1) (2 0)) ((1 1) (5 2)) ((2 1) (3 2)) ((2 1) (5 3))
    ((3 4) (0 0)) ((4 2) (3 0)) ((5 4) (0 0)) ((6 2) (5 0))
    ((7 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation nonce-test (added-strand make 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init")) (ch-msg lk-1 (cat pt-5 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-1 (cat pt-4 old-1)) (stor lk-1 (cat pt-5 k "init"))
      (send (hash k)))
    ((load lk-2 (cat pt-6 old-2)) (stor lk-2 (cat pt-7 k "init"))))
  (label 1882)
  (parent 1078)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((1 1) (5 2)) ((2 1) (3 2)) ((2 1) (5 3))
    ((3 4) (0 0)) ((4 2) (3 0)) ((5 4) (0 0)) ((6 1) (7 0))
    ((6 2) (5 0)) ((7 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-wrap-at-1 trRl_make-at-0)
  (operation nonce-test (added-strand set-wrap 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init")) (ch-msg lk-1 (cat pt-5 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-1 (cat pt-4 old-1)) (stor lk-1 (cat pt-5 k "init"))
      (send (hash k)))
    ((load lk-1 (cat pt-5 k "init")) (stor lk-1 (cat pt-6 k "wrap"))))
  (label 1883)
  (parent 1078)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((1 1) (5 2)) ((2 1) (3 2)) ((2 1) (5 3))
    ((3 4) (0 0)) ((4 1) (7 0)) ((4 2) (3 0)) ((5 4) (0 0))
    ((6 2) (5 0)) ((7 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-wrap-at-1 trRl_make-at-0)
  (operation nonce-test (added-strand set-wrap 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init")) (ch-msg lk-1 (cat pt-5 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-1 (cat pt-4 old-1)) (stor lk-1 (cat pt-5 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-6 k "wrap"))))
  (label 1884)
  (parent 1078)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand set-decrypt 2 (cur "wrap") (k k) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (5 2)) ((2 1) (3 2)) ((2 1) (5 3))
    ((2 1) (7 0)) ((3 4) (0 0)) ((4 2) (3 0)) ((5 4) (0 0))
    ((6 2) (5 0)) ((7 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "wrap" "decrypt") (neq "wrap" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-decrypt-at-1 fact-set-decrypt-neq0 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation nonce-test (added-strand set-decrypt 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init")) (ch-msg lk-1 (cat pt-5 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-1 (cat pt-4 old-1)) (stor lk-1 (cat pt-5 k "init"))
      (send (hash k)))
    ((load lk (cat pt-1 k "wrap")) (stor lk (cat pt-6 k "decrypt"))))
  (label 1885)
  (parent 1078)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((1 1) (5 2)) ((2 1) (3 2)) ((2 1) (5 3))
    ((3 4) (0 0)) ((4 1) (7 0)) ((4 2) (3 0)) ((5 4) (0 0))
    ((6 2) (5 0)) ((7 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-decrypt-at-1 trRl_make-at-0)
  (operation nonce-test (added-strand set-decrypt 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init")) (ch-msg lk-1 (cat pt-5 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-1 (cat pt-4 old-1)) (stor lk-1 (cat pt-5 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-3 k "init"))
      (stor lk-0 (cat pt-6 k "decrypt"))))
  (label 1886)
  (parent 1078)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (precedes ((1 1) (2 0)) ((1 1) (5 2)) ((2 1) (3 2)) ((2 1) (5 3))
    ((3 4) (0 0)) ((4 1) (7 2)) ((4 2) (3 0)) ((5 4) (0 0))
    ((6 2) (5 0)) ((7 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init")) (ch-msg lk-1 (cat pt-5 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-1 (cat pt-4 old-1)) (stor lk-1 (cat pt-5 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-6 k1 "wrap")) (send (enc k k1))))
  (label 1887)
  (parent 1078)
  (unrealized (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-1))
  (precedes ((1 1) (2 0)) ((1 1) (5 2)) ((2 1) (3 2)) ((2 1) (5 3))
    ((3 4) (0 0)) ((4 2) (3 0)) ((5 4) (0 0)) ((6 1) (7 2))
    ((6 2) (5 0)) ((7 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init")) (ch-msg lk-1 (cat pt-5 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-1 (cat pt-4 old-1)) (stor lk-1 (cat pt-5 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk-1 (cat pt-5 k "init"))
      (load lk-1 (cat pt-6 k1 "wrap")) (send (enc k k1))))
  (label 1888)
  (parent 1078)
  (unrealized (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (5 2)) ((2 1) (3 2)) ((2 1) (5 3))
    ((2 1) (7 2)) ((3 4) (0 0)) ((4 2) (3 0)) ((5 4) (0 0))
    ((6 2) (5 0)) ((7 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init")) (ch-msg lk-1 (cat pt-5 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-1 (cat pt-4 old-1)) (stor lk-1 (cat pt-5 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-6 k1 "wrap")) (send (enc k k1))))
  (label 1889)
  (parent 1078)
  (unrealized (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (5 2)) ((1 1) (7 2)) ((2 1) (3 2))
    ((2 1) (5 3)) ((3 4) (0 0)) ((4 2) (3 0)) ((5 4) (0 0))
    ((6 2) (5 0)) ((7 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init")) (ch-msg lk-1 (cat pt-5 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-1 (cat pt-4 old-1)) (stor lk-1 (cat pt-5 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-6 k1 "wrap")) (send (enc k k1))))
  (label 1890)
  (parent 1078)
  (unrealized (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (3 2)) ((3 4) (0 0))
    ((4 2) (3 0)) ((5 4) (0 0)) ((6 1) (5 0)) ((7 1) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation nonce-test (added-strand make 2) k (6 0)
    (ch-msg lk (cat pt-0 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init"))))
  (label 1891)
  (parent 1080)
  (unrealized (5 3) (6 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (precedes ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (3 2)) ((3 4) (0 0))
    ((4 1) (6 0)) ((4 2) (3 0)) ((5 4) (0 0)) ((6 1) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation nonce-test (displaced 7 4 make 2) k (6 0)
    (ch-msg lk (cat pt-0 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k)))
  (label 1892)
  (parent 1080)
  (unrealized (5 3) (6 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (7 2)) ((2 1) (3 2)) ((3 4) (0 0))
    ((4 2) (3 0)) ((5 4) (0 0)) ((6 1) (5 0)) ((7 4) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap")
    (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (6 0)
    (ch-msg lk (cat pt-0 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((recv (hash k)) (recv (hash k1-0)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-5 k1-0 "wrap")) (send (enc k k1-0))))
  (label 1893)
  (parent 1080)
  (unrealized (5 3) (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((1 1) (5 2)) ((2 1) (3 2)) ((3 4) (0 0))
    ((4 1) (6 0)) ((4 2) (3 0)) ((5 4) (0 0)) ((6 1) (5 0))
    ((7 1) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation nonce-test (added-strand make 2) k (6 0)
    (ch-msg lk-0 (cat pt-3 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init"))))
  (label 1894)
  (parent 1081)
  (unrealized (5 3) (6 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((1 1) (5 2)) ((2 1) (3 2)) ((3 4) (0 0))
    ((4 1) (7 0)) ((4 2) (3 0)) ((5 4) (0 0)) ((6 1) (5 0))
    ((7 1) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-wrap-at-1 trRl_make-at-0)
  (operation nonce-test (added-strand set-wrap 2) k (6 0)
    (ch-msg lk-0 (cat pt-3 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-5 k "wrap"))))
  (label 1895)
  (parent 1081)
  (unrealized (5 3) (6 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((1 1) (5 2)) ((2 1) (3 2)) ((3 4) (0 0))
    ((4 1) (7 0)) ((4 2) (3 0)) ((5 4) (0 0)) ((6 1) (5 0))
    ((7 1) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-decrypt-at-1 trRl_make-at-0)
  (operation nonce-test (added-strand set-decrypt 2) k (6 0)
    (ch-msg lk-0 (cat pt-3 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 k "init"))
      (stor lk-0 (cat pt-5 k "decrypt"))))
  (label 1896)
  (parent 1081)
  (unrealized (5 3) (6 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk-0))
  (precedes ((1 1) (2 0)) ((1 1) (5 2)) ((2 1) (3 2)) ((3 4) (0 0))
    ((4 1) (7 2)) ((4 2) (3 0)) ((5 4) (0 0)) ((6 1) (5 0))
    ((7 4) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap")
    (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (6 0)
    (ch-msg lk-0 (cat pt-3 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-5 k1-0 "wrap")) (send (enc k k1-0))))
  (label 1897)
  (parent 1081)
  (unrealized (5 3) (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "decrypt") (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (4 0)) ((3 4) (0 0)) ((4 1) (3 0))
    ((5 1) (6 0)) ((6 1) (7 0)) ((7 1) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k "wrap")
    (cat k1 "wrap"))
  (facts (neq "decrypt" "decrypt") (neq "decrypt" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-decrypt-at-1 fact-set-decrypt-neq0 gen-st-set-wrap-0
    fact-set-wrap-neq0 trRl_make-at-0)
  (operation nonce-test (added-strand set-decrypt 2) k (4 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-4 k "init"))
    (ch-msg lk-0 (cat pt-5 k "decrypt")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt-5 k "decrypt")))
    ((load lk-0 (cat pt-5 k "decrypt"))
      (stor lk-0 (cat pt-6 k "decrypt"))))
  (label 1898)
  (parent 1083)
  (unrealized (3 3) (4 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((2 1) (4 0)) ((3 4) (0 0)) ((4 1) (3 0))
    ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation nonce-test (added-strand make 2) k (4 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-4 k "init"))
    (ch-msg lk-0 (cat pt-5 k "decrypt")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt-5 k "decrypt")))
    ((load lk-1 (cat pt-6 old-1)) (stor lk-1 (cat pt-7 k "init"))))
  (label 1899)
  (parent 1083)
  (unrealized (3 3) (4 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "decrypt") (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (4 0)) ((3 4) (0 0)) ((4 1) (3 0))
    ((5 1) (6 0)) ((6 1) (7 0)) ((7 1) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k "wrap")
    (cat k1 "wrap"))
  (facts (neq "decrypt" "decrypt") (neq "decrypt" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule fact-set-decrypt-neq0 trRl_set-wrap-at-1 gen-st-set-wrap-0
    fact-set-wrap-neq0 trRl_make-at-0)
  (operation nonce-test (added-strand set-wrap 2) k (4 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-4 k "init"))
    (ch-msg lk-0 (cat pt-5 k "decrypt")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt-5 k "decrypt")))
    ((load lk-0 (cat pt-5 k "decrypt"))
      (stor lk-0 (cat pt-6 k "wrap"))))
  (label 1900)
  (parent 1083)
  (unrealized (3 3) (4 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1-0) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (4 0)) ((3 4) (0 0)) ((4 1) (3 0))
    ((5 1) (6 0)) ((6 1) (7 2)) ((7 4) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k "wrap")
    (cat k1 "wrap") (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0 gen-st-wrap-1)
  (operation nonce-test (added-strand wrap 5) k (4 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-4 k "init"))
    (ch-msg lk-0 (cat pt-5 k "decrypt")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt-5 k "decrypt")))
    ((recv (hash k)) (recv (hash k1-0))
      (load lk-0 (cat pt-5 k "decrypt"))
      (load lk-0 (cat pt-6 k1-0 "wrap")) (send (enc k k1-0))))
  (label 1901)
  (parent 1083)
  (unrealized (3 3) (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (4 0)) ((3 4) (0 0)) ((4 1) (3 0))
    ((5 1) (6 0)) ((5 1) (7 2)) ((6 1) (4 0)) ((7 4) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap")
    (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (4 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-4 k "init"))
    (ch-msg lk-0 (cat pt-5 k "decrypt")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt-5 k "decrypt")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-0 (cat pt-4 k "init"))
      (load lk-0 (cat pt-6 k1-0 "wrap")) (send (enc k k1-0))))
  (label 1902)
  (parent 1083)
  (unrealized (3 3) (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1-0) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (7 2)) ((3 4) (0 0)) ((4 1) (3 0))
    ((5 1) (6 0)) ((6 1) (4 0)) ((7 4) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap")
    (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (4 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-4 k "init"))
    (ch-msg lk-0 (cat pt-5 k "decrypt")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt-5 k "decrypt")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-6 k1-0 "wrap")) (send (enc k k1-0))))
  (label 1903)
  (parent 1083)
  (unrealized (3 3) (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (7 2)) ((2 1) (4 0)) ((3 4) (0 0))
    ((4 1) (3 0)) ((5 1) (6 0)) ((6 1) (4 0)) ((7 4) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap")
    (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (4 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-4 k "init"))
    (ch-msg lk-0 (cat pt-5 k "decrypt")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt-5 k "decrypt")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-6 k1-0 "wrap")) (send (enc k k1-0))))
  (label 1904)
  (parent 1083)
  (unrealized (3 3) (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "wrap") (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (4 0)) ((3 4) (0 0)) ((4 1) (3 0))
    ((5 1) (6 0)) ((6 1) (7 0)) ((7 1) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "wrap" "decrypt") (neq "wrap" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-decrypt-at-1 fact-set-decrypt-neq0 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation nonce-test (added-strand set-decrypt 2) k (4 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-4 k "init")) (ch-msg lk-0 (cat pt-5 k "wrap")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt-5 k "wrap")))
    ((load lk-0 (cat pt-5 k "wrap"))
      (stor lk-0 (cat pt-6 k "decrypt"))))
  (label 1905)
  (parent 1084)
  (unrealized (3 3) (4 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((2 1) (4 0)) ((3 4) (0 0)) ((4 1) (3 0))
    ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation nonce-test (added-strand make 2) k (4 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-4 k "init")) (ch-msg lk-0 (cat pt-5 k "wrap")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt-5 k "wrap")))
    ((load lk-1 (cat pt-6 old-1)) (stor lk-1 (cat pt-7 k "init"))))
  (label 1906)
  (parent 1084)
  (unrealized (3 3) (4 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "wrap") (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (4 0)) ((3 4) (0 0)) ((4 1) (3 0))
    ((5 1) (6 0)) ((6 1) (7 0)) ((7 1) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "wrap" "decrypt") (neq "wrap" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule fact-set-decrypt-neq0 trRl_set-wrap-at-1 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation nonce-test (added-strand set-wrap 2) k (4 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-4 k "init")) (ch-msg lk-0 (cat pt-5 k "wrap")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt-5 k "wrap")))
    ((load lk-0 (cat pt-5 k "wrap")) (stor lk-0 (cat pt-6 k "wrap"))))
  (label 1907)
  (parent 1084)
  (unrealized (3 3) (4 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1-0) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (4 0)) ((3 4) (0 0)) ((4 1) (3 0))
    ((5 1) (6 0)) ((6 1) (7 2)) ((7 4) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap")
    (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (4 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-4 k "init")) (ch-msg lk-0 (cat pt-5 k "wrap")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt-5 k "wrap")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-0 (cat pt-5 k "wrap"))
      (load lk-0 (cat pt-6 k1-0 "wrap")) (send (enc k k1-0))))
  (label 1908)
  (parent 1084)
  (unrealized (3 3) (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (4 0)) ((3 4) (0 0)) ((4 1) (3 0))
    ((5 1) (6 0)) ((5 1) (7 2)) ((6 1) (4 0)) ((7 4) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap")
    (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (4 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-4 k "init")) (ch-msg lk-0 (cat pt-5 k "wrap")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt-5 k "wrap")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-0 (cat pt-4 k "init"))
      (load lk-0 (cat pt-6 k1-0 "wrap")) (send (enc k k1-0))))
  (label 1909)
  (parent 1084)
  (unrealized (3 3) (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1-0) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (7 2)) ((3 4) (0 0)) ((4 1) (3 0))
    ((5 1) (6 0)) ((6 1) (4 0)) ((7 4) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap")
    (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (4 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-4 k "init")) (ch-msg lk-0 (cat pt-5 k "wrap")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt-5 k "wrap")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-6 k1-0 "wrap")) (send (enc k k1-0))))
  (label 1910)
  (parent 1084)
  (unrealized (3 3) (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (7 2)) ((2 1) (4 0)) ((3 4) (0 0))
    ((4 1) (3 0)) ((5 1) (6 0)) ((6 1) (4 0)) ((7 4) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap")
    (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (4 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-4 k "init")) (ch-msg lk-0 (cat pt-5 k "wrap")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt-5 k "wrap")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-6 k1-0 "wrap")) (send (enc k k1-0))))
  (label 1911)
  (parent 1084)
  (unrealized (3 3) (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (4 0)) ((3 4) (0 0)) ((4 1) (3 0))
    ((5 4) (4 0)) ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap")
    (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 5 7 make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-0 (cat pt-5 k "init"))
      (load lk-0 (cat pt-3 k1-0 "wrap")) (send (enc k k1-0)))
    ((load lk-0 (cat pt-4 old-0)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k))))
  (label 1912)
  (parent 1085)
  (unrealized (3 3) (5 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk-0))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (precedes ((1 1) (3 0)) ((2 4) (0 0)) ((3 1) (2 0)) ((4 1) (5 2))
    ((5 4) (3 0)) ((6 1) (1 0)) ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap")
    (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 1 7 make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-5 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt k "wrap"))
      (load lk-0 (cat pt-0 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk (cat pt-1 old)) (stor lk (cat pt-2 k "init")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk (cat pt-2 k "init"))
      (load lk (cat pt-3 k1-0 "wrap")) (send (enc k k1-0)))
    ((load lk-0 (cat pt-4 old-0)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k))))
  (label 1913)
  (parent 1085)
  (unrealized (2 3) (5 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 k1-0 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk-0))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((2 1) (4 0)) ((3 4) (0 0)) ((4 1) (3 0))
    ((5 1) (6 2)) ((6 4) (4 0)) ((7 2) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap")
    (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation encryption-test (added-strand make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-0 (cat pt-4 k "init"))
      (load lk-0 (cat pt-5 k1-0 "wrap")) (send (enc k k1-0)))
    ((load lk-1 (cat pt-6 old-1)) (stor lk-1 (cat pt-7 k "init"))
      (send (hash k))))
  (label 1914)
  (parent 1085)
  (unrealized (3 3) (6 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk-0))
  (deflistener k)
  (precedes ((1 1) (2 0)) ((2 1) (4 0)) ((3 4) (0 0)) ((4 1) (3 0))
    ((5 1) (6 2)) ((6 4) (4 0)) ((7 1) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap")
    (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (added-listener k) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-0 (cat pt-4 k "init"))
      (load lk-0 (cat pt-5 k1-0 "wrap")) (send (enc k k1-0)))
    ((recv k) (send k)))
  (label 1915)
  (parent 1085)
  (unrealized (3 3) (6 3) (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk-0))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1-0) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (precedes ((1 1) (5 2)) ((2 4) (0 0)) ((3 1) (2 0)) ((4 1) (3 0))
    ((5 4) (3 0)) ((6 1) (1 0)) ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap")
    (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 1 7 make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-5 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt k "wrap"))
      (load lk-0 (cat pt-0 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk (cat pt-1 old)) (stor lk (cat pt-2 k "init")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-0 (cat pt k "wrap"))
      (load lk-0 (cat pt-3 k1-0 "wrap")) (send (enc k k1-0)))
    ((load lk-0 (cat pt-4 old-0)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k))))
  (label 1916)
  (parent 1086)
  (unrealized (2 3) (5 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 k1-0 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1-0) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((2 1) (6 2)) ((3 4) (0 0)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 4) (4 0)) ((7 2) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap")
    (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation encryption-test (added-strand make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-5 k1-0 "wrap")) (send (enc k k1-0)))
    ((load lk-1 (cat pt-6 old-1)) (stor lk-1 (cat pt-7 k "init"))
      (send (hash k))))
  (label 1917)
  (parent 1086)
  (unrealized (3 3) (6 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1-0) (lk lk))
  (deflistener k)
  (precedes ((1 1) (2 0)) ((2 1) (6 2)) ((3 4) (0 0)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 4) (4 0)) ((7 1) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap")
    (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (added-listener k) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-5 k1-0 "wrap")) (send (enc k k1-0)))
    ((recv k) (send k)))
  (label 1918)
  (parent 1086)
  (unrealized (3 3) (6 3) (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk-0))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (precedes ((1 1) (3 0)) ((2 4) (0 0)) ((3 1) (2 0)) ((4 1) (3 0))
    ((5 4) (3 0)) ((6 1) (1 0)) ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap")
    (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (displaced 1 7 make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-5 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt k "wrap"))
      (load lk-0 (cat pt-0 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk (cat pt-1 old)) (stor lk (cat pt-2 k "init")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-0 (cat pt-5 k "init"))
      (load lk-0 (cat pt-3 k1-0 "wrap")) (send (enc k k1-0)))
    ((load lk-0 (cat pt-4 old-0)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k))))
  (label 1919)
  (parent 1087)
  (unrealized (2 3) (5 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 k1-0 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((1 1) (6 2)) ((2 1) (4 0)) ((3 4) (0 0))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 4) (4 0)) ((7 2) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap")
    (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation encryption-test (added-strand make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-5 k1-0 "wrap")) (send (enc k k1-0)))
    ((load lk-1 (cat pt-6 old-1)) (stor lk-1 (cat pt-7 k "init"))
      (send (hash k))))
  (label 1920)
  (parent 1087)
  (unrealized (3 3) (6 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk))
  (deflistener k)
  (precedes ((1 1) (2 0)) ((1 1) (6 2)) ((2 1) (4 0)) ((3 4) (0 0))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 4) (4 0)) ((7 1) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap")
    (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (added-listener k) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-5 k1-0 "wrap")) (send (enc k k1-0)))
    ((recv k) (send k)))
  (label 1921)
  (parent 1087)
  (unrealized (3 3) (6 3) (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old mesg) (pt pt-0 pt-1 pt-2 pval) (k k1 skey) (lk locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old) (k k) (lk lk))
  (precedes ((1 1) (4 2)) ((2 4) (0 0)) ((3 1) (2 0)) ((4 4) (3 0))
    ((5 1) (1 0)) ((5 2) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation channel-test (displaced 6 1 set-wrap 2)
    (ch-msg lk (cat pt-3 k1-0 "wrap")) (4 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt-2 k "init")) (stor lk (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt k "wrap"))
      (load lk (cat pt-0 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt k "wrap"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-1 old)) (stor lk (cat pt-2 k "init"))
      (send (hash k))))
  (label 1922)
  (parent 1088)
  (unrealized (2 3) (3 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (5 2)) ((3 4) (0 0)) ((4 1) (3 0))
    ((5 4) (4 0)) ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation channel-test (displaced 7 2 set-wrap 2)
    (ch-msg lk (cat pt-5 k1-0 "wrap")) (5 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init"))
      (send (hash k))))
  (label 1923)
  (parent 1089)
  (unrealized (3 3) (4 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old mesg) (pt pt-0 pt-1 pt-2 pt-3 pval) (k k1 k1-0 skey)
    (lk locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1-0) (lk lk))
  (deflistener k)
  (precedes ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (5 2)) ((3 4) (0 0))
    ((4 1) (3 0)) ((5 4) (4 0)) ((6 1) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap")
    (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation nonce-test (displaced 7 1 make 2) k (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((recv (hash k)) (recv (hash k1-0)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-3 k1-0 "wrap")) (send (enc k k1-0)))
    ((recv k) (send k)))
  (label 1924)
  (parent 1090)
  (unrealized (3 3) (5 3) (6 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1-0) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (5 2)) ((3 4) (0 0)) ((4 1) (3 0))
    ((5 4) (4 0)) ((6 1) (5 0)) ((7 1) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap")
    (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation nonce-test (added-strand make 2) k (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((recv (hash k)) (recv (hash k1-0)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-3 k1-0 "wrap")) (send (enc k k1-0)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-4 old-0)) (stor lk-0 (cat pt-5 k "init"))))
  (label 1925)
  (parent 1090)
  (unrealized (3 3) (5 3) (6 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old mesg) (pt pt-0 pt-1 pt-2 pt-3 pval) (k k1 k1-0 skey)
    (lk locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk))
  (deflistener k)
  (precedes ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (4 0)) ((3 4) (0 0))
    ((4 1) (3 0)) ((5 4) (4 0)) ((6 1) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap")
    (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation nonce-test (displaced 7 1 make 2) k (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((recv (hash k)) (recv (hash k1-0)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-3 k1-0 "wrap")) (send (enc k k1-0)))
    ((recv k) (send k)))
  (label 1926)
  (parent 1093)
  (unrealized (3 3) (5 3) (6 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((1 1) (5 2)) ((2 1) (4 0)) ((3 4) (0 0))
    ((4 1) (3 0)) ((5 4) (4 0)) ((6 1) (5 0)) ((7 1) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap")
    (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation nonce-test (added-strand make 2) k (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((recv (hash k)) (recv (hash k1-0)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-3 k1-0 "wrap")) (send (enc k k1-0)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-4 old-0)) (stor lk-0 (cat pt-5 k "init"))))
  (label 1927)
  (parent 1093)
  (unrealized (3 3) (5 3) (6 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 cur mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 k1-0 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur cur) (k k1-0) (lk lk-0))
  (precedes ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (3 2)) ((3 4) (0 0))
    ((4 1) (3 0)) ((5 4) (4 0)) ((6 1) (7 0)) ((6 2) (5 0))
    ((7 1) (5 3)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap") (cat k1-0 cur)
    (cat k1-0 "wrap"))
  (facts (neq cur "decrypt") (neq cur "wrap") (neq "init" "decrypt")
    (neq "init" "wrap"))
  (rule invShearsRule fact-set-decrypt-neq0 trRl_set-wrap-at-1
    gen-st-set-wrap-0 fact-set-wrap-neq0 trRl_make-at-0)
  (operation channel-test (added-strand set-wrap 2)
    (ch-msg lk-0 (cat pt-3 k1-0 "wrap")) (5 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-0 (cat pt-5 k "init"))
      (load lk-0 (cat pt-3 k1-0 "wrap")) (send (enc k k1-0)))
    ((load lk-0 (cat pt-4 old-0)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-6 k1-0 cur))
      (stor lk-0 (cat pt-3 k1-0 "wrap"))))
  (label 1928)
  (parent 1106)
  (unrealized (3 3) (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk-0))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (precedes ((1 1) (5 3)) ((2 4) (0 0)) ((3 1) (2 0)) ((4 1) (3 0))
    ((5 4) (3 0)) ((6 1) (1 0)) ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation channel-test (displaced 7 1 set-wrap 2)
    (ch-msg lk-0 (cat pt-5 k1-0 "wrap")) (5 3))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt k "wrap"))
      (load lk-0 (cat pt-0 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk (cat pt-1 old)) (stor lk (cat pt-2 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt-4 k "init"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init"))
      (send (hash k))))
  (label 1929)
  (parent 1109)
  (unrealized (2 3) (3 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old mesg) (pt pt-0 pt-1 pt-2 pt-3 pval) (k k1 k1-0 skey)
    (lk locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk))
  (precedes ((1 1) (4 3)) ((2 4) (0 0)) ((3 1) (2 0)) ((4 4) (3 0))
    ((5 1) (1 0)) ((5 1) (6 2)) ((5 2) (4 0)) ((6 4) (3 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap")
    (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (3 0) (enc k k)
    (ch-msg lk (cat pt k "wrap")) (ch-msg lk (cat pt-2 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt-2 k "init")) (stor lk (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt k "wrap"))
      (load lk (cat pt-0 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-2 k "init"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-1 old)) (stor lk (cat pt-2 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1-0)) (load lk (cat pt-2 k "init"))
      (load lk (cat pt-3 k1-0 "wrap")) (send (enc k k1-0))))
  (label 1930)
  (parent 1112)
  (unrealized (2 3) (6 0) (6 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old mesg) (pt pt-0 pt-1 pt-2 pt-3 pval) (k k1 k1-0 skey)
    (lk locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1-0) (lk lk))
  (precedes ((1 1) (4 3)) ((1 1) (6 2)) ((2 4) (0 0)) ((3 1) (2 0))
    ((4 4) (3 0)) ((5 1) (1 0)) ((5 2) (4 0)) ((6 4) (3 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap")
    (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (3 0) (enc k k)
    (ch-msg lk (cat pt k "wrap")) (ch-msg lk (cat pt-2 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt-2 k "init")) (stor lk (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt k "wrap"))
      (load lk (cat pt-0 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-2 k "init"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-1 old)) (stor lk (cat pt-2 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1-0)) (load lk (cat pt k "wrap"))
      (load lk (cat pt-3 k1-0 "wrap")) (send (enc k k1-0))))
  (label 1931)
  (parent 1112)
  (unrealized (2 3) (6 0) (6 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((1 1) (5 2)) ((2 1) (5 3)) ((3 4) (0 0))
    ((4 1) (3 0)) ((5 4) (4 0)) ((6 1) (7 0)) ((6 2) (5 0))
    ((7 1) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-decrypt-at-1 trRl_make-at-0)
  (operation nonce-test (added-strand set-decrypt 2) k (4 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-4 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-4 k "init"))
      (stor lk-0 (cat pt-5 k "decrypt"))))
  (label 1932)
  (parent 1113)
  (unrealized (3 3) (4 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((1 1) (5 2)) ((2 1) (5 3)) ((3 4) (0 0))
    ((4 1) (3 0)) ((5 4) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation nonce-test (added-strand make 2) k (4 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-4 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init"))
      (send (hash k)))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init"))))
  (label 1933)
  (parent 1113)
  (unrealized (3 3) (4 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((1 1) (5 2)) ((2 1) (5 3)) ((3 4) (0 0))
    ((4 1) (3 0)) ((5 4) (4 0)) ((6 1) (7 0)) ((6 2) (5 0))
    ((7 1) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-wrap-at-1 trRl_make-at-0)
  (operation nonce-test (added-strand set-wrap 2) k (4 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-4 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt-5 k "wrap"))))
  (label 1934)
  (parent 1113)
  (unrealized (3 3) (4 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk-0))
  (precedes ((1 1) (2 0)) ((1 1) (5 2)) ((2 1) (5 3)) ((3 4) (0 0))
    ((4 1) (3 0)) ((5 4) (4 0)) ((6 1) (7 2)) ((6 2) (5 0))
    ((7 4) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap")
    (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (4 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-4 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-0 (cat pt-4 k "init"))
      (load lk-0 (cat pt-5 k1-0 "wrap")) (send (enc k k1-0))))
  (label 1935)
  (parent 1113)
  (unrealized (3 3) (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1-0) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (5 2)) ((2 1) (5 3)) ((2 1) (7 2))
    ((3 4) (0 0)) ((4 1) (3 0)) ((5 4) (4 0)) ((6 2) (5 0))
    ((7 4) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap")
    (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (4 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-4 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1-0)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-5 k1-0 "wrap")) (send (enc k k1-0))))
  (label 1936)
  (parent 1113)
  (unrealized (3 3) (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (5 2)) ((1 1) (7 2)) ((2 1) (5 3))
    ((3 4) (0 0)) ((4 1) (3 0)) ((5 4) (4 0)) ((6 2) (5 0))
    ((7 4) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap")
    (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (4 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-4 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1-0)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-5 k1-0 "wrap")) (send (enc k k1-0))))
  (label 1937)
  (parent 1113)
  (unrealized (3 3) (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (3 2)) ((3 4) (0 0))
    ((4 1) (3 0)) ((5 4) (4 0)) ((6 1) (5 0)) ((7 1) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap")
    (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation nonce-test (added-strand make 2) k (6 0)
    (ch-msg lk (cat pt-0 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((recv (hash k)) (recv (hash k1-0)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-3 k1-0 "wrap")) (send (enc k k1-0)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-4 old-0)) (stor lk-0 (cat pt-5 k "init"))))
  (label 1938)
  (parent 1114)
  (unrealized (3 3) (5 3) (6 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (k k1 k1-0 k1-1 skey) (lk locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk))
  (deflistener k)
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-1) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (7 2)) ((2 1) (3 2)) ((3 4) (0 0))
    ((4 1) (3 0)) ((5 4) (4 0)) ((6 1) (5 0)) ((7 4) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap")
    (cat k1-0 "wrap") (cat k1-1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (6 0)
    (ch-msg lk (cat pt-0 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((recv (hash k)) (recv (hash k1-0)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-3 k1-0 "wrap")) (send (enc k k1-0)))
    ((recv k) (send k))
    ((recv (hash k)) (recv (hash k1-1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-4 k1-1 "wrap")) (send (enc k k1-1))))
  (label 1939)
  (parent 1114)
  (unrealized (3 3) (5 3) (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (5 3)) ((3 4) (0 0)) ((4 1) (3 0))
    ((5 4) (4 0)) ((6 1) (7 0)) ((6 2) (5 0)) ((7 1) (1 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule invShearsRule shearsRule trRl_set-decrypt-at-1 trRl_make-at-0)
  (operation nonce-test (added-strand set-decrypt 2) k (4 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk (cat pt-4 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-4 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-3 old-0)) (stor lk (cat pt-4 k "init"))
      (send (hash k)))
    ((load lk (cat pt-4 k "init")) (stor lk (cat pt-5 k "decrypt"))))
  (label 1940)
  (parent 1116)
  (unrealized (3 3) (4 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand make 2 (old old-1) (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (5 3)) ((3 4) (0 0)) ((4 1) (3 0))
    ((5 4) (4 0)) ((6 1) (1 0)) ((6 2) (5 0)) ((7 1) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation nonce-test (added-strand make 2) k (4 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk (cat pt-4 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-4 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-3 old-0)) (stor lk (cat pt-4 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-5 old-1)) (stor lk-0 (cat pt-6 k "init"))))
  (label 1941)
  (parent 1116)
  (unrealized (3 3) (4 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (5 3)) ((3 4) (0 0)) ((4 1) (3 0))
    ((5 4) (4 0)) ((6 1) (7 0)) ((6 2) (5 0)) ((7 1) (1 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule invShearsRule shearsRule trRl_set-wrap-at-1 trRl_make-at-0)
  (operation nonce-test (added-strand set-wrap 2) k (4 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk (cat pt-4 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-4 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-3 old-0)) (stor lk (cat pt-4 k "init"))
      (send (hash k)))
    ((load lk (cat pt-4 k "init")) (stor lk (cat pt-5 k "wrap"))))
  (label 1942)
  (parent 1116)
  (unrealized (3 3) (4 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 k1-0 skey) (lk locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (5 3)) ((3 4) (0 0)) ((4 1) (3 0))
    ((5 4) (4 0)) ((6 1) (1 0)) ((6 1) (7 2)) ((6 2) (5 0))
    ((7 4) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap")
    (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (4 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk (cat pt-4 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-4 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-3 old-0)) (stor lk (cat pt-4 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1-0)) (load lk (cat pt-4 k "init"))
      (load lk (cat pt-5 k1-0 "wrap")) (send (enc k k1-0))))
  (label 1943)
  (parent 1116)
  (unrealized (3 3) (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 k1-0 skey) (lk locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1-0) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (5 3)) ((2 1) (7 2)) ((3 4) (0 0))
    ((4 1) (3 0)) ((5 4) (4 0)) ((6 1) (1 0)) ((6 2) (5 0))
    ((7 4) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap")
    (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (4 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk (cat pt-4 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-4 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-3 old-0)) (stor lk (cat pt-4 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1-0)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-5 k1-0 "wrap")) (send (enc k k1-0))))
  (label 1944)
  (parent 1116)
  (unrealized (3 3) (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 k1-0 skey) (lk locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (7 2)) ((2 1) (5 3)) ((3 4) (0 0))
    ((4 1) (3 0)) ((5 4) (4 0)) ((6 1) (1 0)) ((6 2) (5 0))
    ((7 4) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap")
    (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (4 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk (cat pt-4 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-4 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-3 old-0)) (stor lk (cat pt-4 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1-0)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-5 k1-0 "wrap")) (send (enc k k1-0))))
  (label 1945)
  (parent 1116)
  (unrealized (3 3) (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (precedes ((1 1) (2 3)) ((1 1) (4 3)) ((2 4) (0 0)) ((3 1) (1 0))
    ((3 1) (4 2)) ((3 2) (2 0)) ((4 4) (0 0)) ((5 1) (6 0))
    ((5 2) (4 0)) ((6 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-decrypt-at-1 trRl_make-at-0)
  (operation nonce-test (added-strand set-decrypt 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt k "wrap")) (ch-msg lk (cat pt-1 k "init"))
    (ch-msg lk-0 (cat pt-3 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-3 k "init"))
      (stor lk-0 (cat pt-4 k "decrypt"))))
  (label 1946)
  (parent 1119)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "wrap") (k k) (lk lk))
  (precedes ((1 1) (2 3)) ((1 1) (4 3)) ((1 1) (6 0)) ((2 4) (0 0))
    ((3 1) (1 0)) ((3 1) (4 2)) ((3 2) (2 0)) ((4 4) (0 0))
    ((5 2) (4 0)) ((6 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "wrap" "decrypt") (neq "wrap" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule fact-set-decrypt-neq0 trRl_set-wrap-at-1 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation nonce-test (added-strand set-wrap 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt k "wrap")) (ch-msg lk (cat pt-1 k "init"))
    (ch-msg lk-0 (cat pt-3 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk (cat pt k "wrap")) (stor lk (cat pt-4 k "wrap"))))
  (label 1947)
  (parent 1119)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (precedes ((1 1) (2 3)) ((1 1) (4 3)) ((2 4) (0 0)) ((3 1) (1 0))
    ((3 1) (4 2)) ((3 2) (2 0)) ((4 4) (0 0)) ((5 1) (6 0))
    ((5 2) (4 0)) ((6 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-wrap-at-1 trRl_make-at-0)
  (operation nonce-test (added-strand set-wrap 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt k "wrap")) (ch-msg lk (cat pt-1 k "init"))
    (ch-msg lk-0 (cat pt-3 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "wrap"))))
  (label 1948)
  (parent 1119)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "wrap") (k k) (lk lk))
  (precedes ((1 1) (2 3)) ((1 1) (4 3)) ((1 1) (6 0)) ((2 4) (0 0))
    ((3 1) (1 0)) ((3 1) (4 2)) ((3 2) (2 0)) ((4 4) (0 0))
    ((5 2) (4 0)) ((6 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "wrap" "decrypt") (neq "wrap" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-decrypt-at-1 fact-set-decrypt-neq0 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation nonce-test (added-strand set-decrypt 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt k "wrap")) (ch-msg lk (cat pt-1 k "init"))
    (ch-msg lk-0 (cat pt-3 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk (cat pt k "wrap")) (stor lk (cat pt-4 k "decrypt"))))
  (label 1949)
  (parent 1119)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (precedes ((1 1) (2 3)) ((1 1) (4 3)) ((2 4) (0 0)) ((3 1) (1 0))
    ((3 1) (4 2)) ((3 2) (2 0)) ((4 4) (0 0)) ((5 1) (6 2))
    ((5 2) (4 0)) ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt k "wrap")) (ch-msg lk (cat pt-1 k "init"))
    (ch-msg lk-0 (cat pt-3 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-4 k1 "wrap")) (send (enc k k1))))
  (label 1950)
  (parent 1119)
  (unrealized (6 0) (6 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (2 3)) ((1 1) (4 3)) ((2 4) (0 0)) ((3 1) (1 0))
    ((3 1) (4 2)) ((3 1) (6 2)) ((3 2) (2 0)) ((4 4) (0 0))
    ((5 2) (4 0)) ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt k "wrap")) (ch-msg lk (cat pt-1 k "init"))
    (ch-msg lk-0 (cat pt-3 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1))))
  (label 1951)
  (parent 1119)
  (unrealized (6 0) (6 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (2 3)) ((1 1) (4 3)) ((1 1) (6 2)) ((2 4) (0 0))
    ((3 1) (1 0)) ((3 1) (4 2)) ((3 2) (2 0)) ((4 4) (0 0))
    ((5 2) (4 0)) ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt k "wrap")) (ch-msg lk (cat pt-1 k "init"))
    (ch-msg lk-0 (cat pt-3 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt k "wrap"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1))))
  (label 1952)
  (parent 1119)
  (unrealized (6 0) (6 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k1) (lk lk))
  (defstrand make 2 (old old-1) (k k1) (lk lk))
  (precedes ((1 1) (2 3)) ((1 1) (7 0)) ((2 4) (0 0)) ((3 1) (1 0))
    ((3 1) (4 2)) ((3 2) (2 0)) ((4 4) (0 0)) ((5 2) (4 0))
    ((6 1) (4 3)) ((7 1) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule invShearsRule trRl_make-at-0 trRl_make-at-1)
  (operation channel-test (added-strand make 2)
    (ch-msg lk (cat pt-5 k1 "init")) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init"))
      (send (hash k)))
    ((load lk (cat pt-5 k1 "init")) (stor lk (cat pt-2 k1 "wrap")))
    ((load lk (cat pt-6 old-1)) (stor lk (cat pt-5 k1 "init"))))
  (label 1953)
  (parent 1120)
  (realized)
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (precedes ((1 1) (2 3)) ((2 4) (0 0)) ((3 1) (1 0)) ((3 1) (5 0))
    ((3 2) (2 0)) ((4 4) (0 0)) ((5 1) (4 0)) ((6 1) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation nonce-test (added-strand make 2) k (5 0)
    (ch-msg lk (cat pt-1 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init"))))
  (label 1954)
  (parent 1121)
  (unrealized (4 3) (5 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old mesg) (pt pt-0 pt-1 pt-2 pt-3 pval) (k k1 k1-0 skey)
    (lk locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk))
  (precedes ((1 1) (2 3)) ((2 4) (0 0)) ((3 1) (1 0)) ((3 1) (6 2))
    ((3 2) (2 0)) ((4 4) (0 0)) ((5 1) (4 0)) ((6 4) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap")
    (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (5 0)
    (ch-msg lk (cat pt-1 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((recv (hash k)) (recv (hash k1-0)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt-3 k1-0 "wrap")) (send (enc k k1-0))))
  (label 1955)
  (parent 1121)
  (unrealized (4 3) (6 0) (6 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (precedes ((1 1) (2 3)) ((2 4) (0 0)) ((3 1) (1 0)) ((3 1) (4 2))
    ((3 2) (2 0)) ((4 4) (0 0)) ((5 1) (4 0)) ((6 1) (7 0))
    ((7 1) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-wrap-at-1 trRl_make-at-0)
  (operation nonce-test (added-strand set-wrap 2) k (5 0)
    (ch-msg lk-0 (cat pt-4 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt-5 k "wrap"))))
  (label 1956)
  (parent 1122)
  (unrealized (4 3) (5 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (precedes ((1 1) (2 3)) ((2 4) (0 0)) ((3 1) (1 0)) ((3 1) (4 2))
    ((3 2) (2 0)) ((4 4) (0 0)) ((5 1) (4 0)) ((6 1) (7 0))
    ((7 1) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-decrypt-at-1 trRl_make-at-0)
  (operation nonce-test (added-strand set-decrypt 2) k (5 0)
    (ch-msg lk-0 (cat pt-4 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((load lk-0 (cat pt-4 k "init"))
      (stor lk-0 (cat pt-5 k "decrypt"))))
  (label 1957)
  (parent 1122)
  (unrealized (4 3) (5 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk-0))
  (precedes ((1 1) (2 3)) ((2 4) (0 0)) ((3 1) (1 0)) ((3 1) (4 2))
    ((3 2) (2 0)) ((4 4) (0 0)) ((5 1) (4 0)) ((6 1) (7 2))
    ((7 4) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap")
    (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (5 0)
    (ch-msg lk-0 (cat pt-4 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-0 (cat pt-4 k "init"))
      (load lk-0 (cat pt-5 k1-0 "wrap")) (send (enc k k1-0))))
  (label 1958)
  (parent 1122)
  (unrealized (4 3) (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k k1 skey)
    (lk locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k1) (lk lk))
  (defstrand make 2 (old old-0) (k k1) (lk lk))
  (precedes ((1 1) (2 3)) ((1 1) (4 2)) ((1 1) (6 0)) ((2 4) (0 0))
    ((3 1) (1 0)) ((3 2) (2 0)) ((3 2) (4 0)) ((4 4) (0 0))
    ((5 1) (4 3)) ((6 1) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule invShearsRule trRl_make-at-0 trRl_make-at-1)
  (operation channel-test (added-strand make 2)
    (ch-msg lk (cat pt-3 k1 "init")) (5 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt k "wrap"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((load lk (cat pt-3 k1 "init")) (stor lk (cat pt-2 k1 "wrap")))
    ((load lk (cat pt-4 old-0)) (stor lk (cat pt-3 k1 "init"))))
  (label 1959)
  (parent 1123)
  (realized)
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k1) (lk lk))
  (defstrand make 2 (old old-1) (k k1) (lk lk))
  (precedes ((1 1) (2 3)) ((1 1) (4 2)) ((1 1) (7 0)) ((2 4) (0 0))
    ((3 1) (1 0)) ((3 2) (2 0)) ((4 4) (0 0)) ((5 2) (4 0))
    ((6 1) (4 3)) ((7 1) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule invShearsRule trRl_make-at-0 trRl_make-at-1)
  (operation channel-test (added-strand make 2)
    (ch-msg lk (cat pt-5 k1 "init")) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt k "wrap"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init"))
      (send (hash k)))
    ((load lk (cat pt-5 k1 "init")) (stor lk (cat pt-2 k1 "wrap")))
    ((load lk (cat pt-6 old-1)) (stor lk (cat pt-5 k1 "init"))))
  (label 1960)
  (parent 1124)
  (realized)
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old mesg) (pt pt-0 pt-1 pt-2 pval) (k k1 skey) (lk locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (precedes ((1 1) (2 3)) ((1 1) (5 0)) ((2 4) (0 0)) ((3 1) (1 0))
    ((3 2) (2 0)) ((4 4) (0 0)) ((5 1) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation nonce-test (displaced 6 1 set-wrap 2) k (5 0)
    (ch-msg lk (cat pt-1 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt k "wrap"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k)))
  (label 1961)
  (parent 1125)
  (unrealized (4 3) (5 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (precedes ((1 1) (2 3)) ((1 1) (4 2)) ((2 4) (0 0)) ((3 1) (1 0))
    ((3 1) (5 0)) ((3 2) (2 0)) ((4 4) (0 0)) ((5 1) (4 0))
    ((6 1) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation nonce-test (added-strand make 2) k (5 0)
    (ch-msg lk (cat pt-1 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt k "wrap"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init"))))
  (label 1962)
  (parent 1125)
  (unrealized (4 3) (5 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old mesg) (pt pt-0 pt-1 pt-2 pt-3 pval) (k k1 k1-0 skey)
    (lk locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk))
  (precedes ((1 1) (2 3)) ((1 1) (4 2)) ((2 4) (0 0)) ((3 1) (1 0))
    ((3 1) (6 2)) ((3 2) (2 0)) ((4 4) (0 0)) ((5 1) (4 0))
    ((6 4) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap")
    (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (5 0)
    (ch-msg lk (cat pt-1 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt k "wrap"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((recv (hash k)) (recv (hash k1-0)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt-3 k1-0 "wrap")) (send (enc k k1-0))))
  (label 1963)
  (parent 1125)
  (unrealized (4 3) (6 0) (6 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (precedes ((1 1) (2 3)) ((1 1) (4 2)) ((2 4) (0 0)) ((3 1) (1 0))
    ((3 2) (2 0)) ((4 4) (0 0)) ((5 1) (4 0)) ((6 1) (7 0))
    ((7 1) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-wrap-at-1 trRl_make-at-0)
  (operation nonce-test (added-strand set-wrap 2) k (5 0)
    (ch-msg lk-0 (cat pt-4 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt k "wrap"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt-5 k "wrap"))))
  (label 1964)
  (parent 1126)
  (unrealized (4 3) (5 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (precedes ((1 1) (2 3)) ((1 1) (4 2)) ((2 4) (0 0)) ((3 1) (1 0))
    ((3 2) (2 0)) ((4 4) (0 0)) ((5 1) (4 0)) ((6 1) (7 0))
    ((7 1) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-decrypt-at-1 trRl_make-at-0)
  (operation nonce-test (added-strand set-decrypt 2) k (5 0)
    (ch-msg lk-0 (cat pt-4 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt k "wrap"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((load lk-0 (cat pt-4 k "init"))
      (stor lk-0 (cat pt-5 k "decrypt"))))
  (label 1965)
  (parent 1126)
  (unrealized (4 3) (5 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk-0))
  (precedes ((1 1) (2 3)) ((1 1) (4 2)) ((2 4) (0 0)) ((3 1) (1 0))
    ((3 2) (2 0)) ((4 4) (0 0)) ((5 1) (4 0)) ((6 1) (7 2))
    ((7 4) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap")
    (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (5 0)
    (ch-msg lk-0 (cat pt-4 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt k "wrap"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-0 (cat pt-4 k "init"))
      (load lk-0 (cat pt-5 k1-0 "wrap")) (send (enc k k1-0))))
  (label 1966)
  (parent 1126)
  (unrealized (4 3) (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k skey)
    (lk locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (3 2)) ((2 1) (3 3)) ((2 1) (6 3))
    ((3 4) (0 0)) ((4 1) (5 0)) ((4 2) (3 0)) ((4 2) (6 0))
    ((5 1) (1 0)) ((5 1) (6 2)) ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule invShearsRule shearsRule)
  (operation channel-test (displaced 7 2 set-wrap 2)
    (ch-msg lk-0 (cat pt-5 k1 "wrap")) (6 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk (cat pt-3 k "init")) (stor lk (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-4 k "decrypt"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k))))
  (label 1967)
  (parent 1127)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k skey)
    (lk locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (3 2)) ((2 1) (3 3)) ((2 1) (6 3))
    ((3 4) (0 0)) ((4 1) (1 0)) ((4 1) (5 0)) ((4 2) (3 0))
    ((4 2) (6 0)) ((5 1) (2 0)) ((5 1) (6 2)) ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule invShearsRule shearsRule)
  (operation channel-test (displaced 7 2 set-wrap 2)
    (ch-msg lk-0 (cat pt-5 k1 "wrap")) (6 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk (cat pt-3 k "init")) (stor lk (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-4 k "decrypt"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k))))
  (label 1968)
  (parent 1127)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k skey)
    (lk locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (3 2)) ((2 1) (3 3)) ((2 1) (6 3))
    ((3 4) (0 0)) ((4 1) (2 0)) ((4 1) (5 0)) ((4 2) (3 0))
    ((4 2) (6 0)) ((5 1) (6 2)) ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule invShearsRule)
  (operation channel-test (displaced 7 2 set-wrap 2)
    (ch-msg lk-0 (cat pt-5 k1 "wrap")) (6 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk (cat pt-3 k "init")) (stor lk (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-4 k "decrypt"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k))))
  (label 1969)
  (parent 1127)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k skey)
    (lk locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (3 2)) ((2 1) (3 3)) ((2 1) (6 3))
    ((3 4) (0 0)) ((4 1) (1 0)) ((4 1) (5 0)) ((4 2) (3 0))
    ((4 2) (6 0)) ((5 1) (6 2)) ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule invShearsRule)
  (operation channel-test (displaced 7 2 set-wrap 2)
    (ch-msg lk-0 (cat pt-5 k1 "wrap")) (6 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk (cat pt-3 k "init")) (stor lk (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-4 k "decrypt"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k))))
  (label 1970)
  (parent 1127)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k skey)
    (lk locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (3 2)) ((2 1) (3 3)) ((2 1) (6 3))
    ((3 4) (0 0)) ((4 1) (5 0)) ((4 2) (3 0)) ((4 2) (6 0))
    ((5 1) (2 0)) ((5 1) (6 2)) ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule invShearsRule)
  (operation channel-test (displaced 7 2 set-wrap 2)
    (ch-msg lk-0 (cat pt-5 k1 "wrap")) (6 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk (cat pt-3 k "init")) (stor lk (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-4 k "decrypt"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k))))
  (label 1971)
  (parent 1127)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 cur mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1) (lk lk-0))
  (defstrand set-wrap 2 (cur cur) (k k1) (lk lk-0))
  (precedes ((1 1) (2 0)) ((1 1) (3 2)) ((2 1) (3 3)) ((3 4) (0 0))
    ((4 1) (5 0)) ((4 2) (3 0)) ((4 2) (6 0)) ((5 1) (6 2))
    ((5 1) (7 0)) ((6 4) (0 0)) ((7 1) (6 3)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k "wrap") (cat k1 cur)
    (cat k1 "wrap"))
  (facts (neq cur "decrypt") (neq cur "wrap") (neq "init" "decrypt")
    (neq "init" "wrap"))
  (rule invShearsRule fact-set-decrypt-neq0 trRl_set-wrap-at-1
    gen-st-set-wrap-0 fact-set-wrap-neq0 trRl_make-at-0)
  (operation channel-test (added-strand set-wrap 2)
    (ch-msg lk-0 (cat pt-5 k1 "wrap")) (6 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-4 k "decrypt"))
      (load lk-0 (cat pt-5 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-6 k1 cur)) (stor lk-0 (cat pt-5 k1 "wrap"))))
  (label 1972)
  (parent 1127)
  (unrealized (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k skey)
    (lk locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (precedes ((1 1) (2 3)) ((1 1) (5 3)) ((2 4) (0 0)) ((3 1) (4 0))
    ((3 1) (6 0)) ((3 2) (2 0)) ((4 1) (1 0)) ((4 1) (5 2))
    ((5 4) (0 0)) ((6 1) (1 0)) ((6 1) (2 2)) ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule invShearsRule shearsRule)
  (operation channel-test (displaced 7 1 set-wrap 2)
    (ch-msg lk-0 (cat pt-5 k1 "wrap")) (5 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt-4 k "init")) (stor lk (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-4 k "init"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init"))
      (send (hash k)))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-2 k "decrypt"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-3 old-0)) (stor lk (cat pt-4 k "init"))
      (send (hash k))))
  (label 1973)
  (parent 1128)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k skey)
    (lk locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (precedes ((1 1) (2 3)) ((1 1) (5 3)) ((2 4) (0 0)) ((3 1) (4 0))
    ((3 1) (6 0)) ((3 2) (2 0)) ((4 1) (5 2)) ((5 4) (0 0))
    ((6 1) (1 0)) ((6 1) (2 2)) ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule invShearsRule)
  (operation channel-test (displaced 7 1 set-wrap 2)
    (ch-msg lk-0 (cat pt-5 k1 "wrap")) (5 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt-4 k "init")) (stor lk (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-4 k "init"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init"))
      (send (hash k)))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-2 k "decrypt"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-3 old-0)) (stor lk (cat pt-4 k "init"))
      (send (hash k))))
  (label 1974)
  (parent 1128)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k skey)
    (lk locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (precedes ((1 1) (2 3)) ((1 1) (5 3)) ((2 4) (0 0)) ((3 1) (1 0))
    ((3 1) (4 0)) ((3 2) (2 0)) ((4 1) (5 2)) ((5 4) (0 0))
    ((6 1) (1 0)) ((6 1) (2 2)) ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule invShearsRule)
  (operation channel-test (displaced 7 1 set-wrap 2)
    (ch-msg lk-0 (cat pt-5 k1 "wrap")) (5 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt-4 k "init")) (stor lk (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-4 k "init"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init"))
      (send (hash k)))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-2 k "decrypt"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-3 old-0)) (stor lk (cat pt-4 k "init"))
      (send (hash k))))
  (label 1975)
  (parent 1128)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k skey)
    (lk locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (precedes ((1 1) (2 3)) ((1 1) (5 3)) ((2 4) (0 0)) ((3 1) (4 0))
    ((3 2) (2 0)) ((4 1) (1 0)) ((4 1) (5 2)) ((5 4) (0 0))
    ((6 1) (1 0)) ((6 1) (2 2)) ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule invShearsRule)
  (operation channel-test (displaced 7 1 set-wrap 2)
    (ch-msg lk-0 (cat pt-5 k1 "wrap")) (5 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt-4 k "init")) (stor lk (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-4 k "init"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init"))
      (send (hash k)))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-2 k "decrypt"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-3 old-0)) (stor lk (cat pt-4 k "init"))
      (send (hash k))))
  (label 1976)
  (parent 1128)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 cur mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur cur) (k k1) (lk lk))
  (precedes ((1 1) (2 3)) ((2 4) (0 0)) ((3 1) (4 0)) ((3 2) (2 0))
    ((4 1) (5 2)) ((4 1) (7 0)) ((5 4) (0 0)) ((6 1) (1 0))
    ((6 1) (2 2)) ((6 2) (5 0)) ((7 1) (5 3)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k "wrap") (cat k1 cur)
    (cat k1 "wrap"))
  (facts (neq cur "decrypt") (neq cur "wrap") (neq "init" "decrypt")
    (neq "init" "wrap"))
  (rule invShearsRule fact-set-decrypt-neq0 trRl_set-wrap-at-1
    gen-st-set-wrap-0 fact-set-wrap-neq0 trRl_make-at-0)
  (operation channel-test (added-strand set-wrap 2)
    (ch-msg lk (cat pt-3 k1 "wrap")) (5 3))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-5 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt-5 k "init"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init"))
      (send (hash k)))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-2 k "decrypt"))
      (load lk (cat pt-3 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-4 old-0)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k)))
    ((load lk (cat pt-6 k1 cur)) (stor lk (cat pt-3 k1 "wrap"))))
  (label 1977)
  (parent 1128)
  (unrealized (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 cur mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (defstrand set-wrap 2 (cur cur) (k k1) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (3 2)) ((2 1) (3 3)) ((2 1) (6 2))
    ((2 1) (7 0)) ((3 4) (0 0)) ((4 1) (5 0)) ((4 2) (3 0))
    ((4 2) (6 0)) ((5 1) (0 0)) ((6 4) (0 0)) ((7 1) (6 3)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 cur) (cat k1 "wrap"))
  (facts (neq cur "decrypt") (neq cur "wrap") (neq "init" "decrypt")
    (neq "init" "wrap"))
  (rule invShearsRule fact-set-decrypt-neq0 trRl_set-wrap-at-1
    gen-st-set-wrap-0 fact-set-wrap-neq0 trRl_make-at-0)
  (operation channel-test (added-strand set-wrap 2)
    (ch-msg lk (cat pt-5 k1 "wrap")) (6 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-5 k1 "wrap")) (send (enc k k1)))
    ((load lk (cat pt-6 k1 cur)) (stor lk (cat pt-5 k1 "wrap"))))
  (label 1978)
  (parent 1131)
  (unrealized (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 cur mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur cur) (k k1) (lk lk-0))
  (precedes ((1 1) (2 3)) ((1 1) (5 2)) ((1 1) (7 0)) ((2 4) (0 0))
    ((3 1) (4 0)) ((3 2) (2 0)) ((4 1) (0 0)) ((5 4) (0 0))
    ((6 1) (1 0)) ((6 1) (2 2)) ((6 2) (5 0)) ((7 1) (5 3)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 cur) (cat k1 "wrap"))
  (facts (neq cur "decrypt") (neq cur "wrap") (neq "init" "decrypt")
    (neq "init" "wrap"))
  (rule invShearsRule fact-set-decrypt-neq0 trRl_set-wrap-at-1
    gen-st-set-wrap-0 fact-set-wrap-neq0 trRl_make-at-0)
  (operation channel-test (added-strand set-wrap 2)
    (ch-msg lk-0 (cat pt-3 k1 "wrap")) (5 3))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-5 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt-5 k "init"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init"))
      (send (hash k)))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt k "wrap"))
      (load lk-0 (cat pt-3 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-4 old-0)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-6 k1 cur)) (stor lk-0 (cat pt-3 k1 "wrap"))))
  (label 1979)
  (parent 1132)
  (unrealized (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k skey)
    (lk locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (3 2)) ((2 1) (3 3)) ((2 1) (6 3))
    ((3 4) (0 0)) ((4 1) (5 0)) ((4 2) (3 0)) ((4 2) (6 0))
    ((5 1) (1 0)) ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule invShearsRule shearsRule)
  (operation channel-test (displaced 7 2 set-wrap 2)
    (ch-msg lk-0 (cat pt-5 k1 "wrap")) (6 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk (cat pt-3 k "init")) (stor lk (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k))))
  (label 1980)
  (parent 1135)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k skey)
    (lk locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (3 2)) ((2 1) (3 3)) ((2 1) (6 3))
    ((3 4) (0 0)) ((4 1) (1 0)) ((4 1) (5 0)) ((4 2) (3 0))
    ((4 2) (6 0)) ((5 1) (2 0)) ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule invShearsRule shearsRule)
  (operation channel-test (displaced 7 2 set-wrap 2)
    (ch-msg lk-0 (cat pt-5 k1 "wrap")) (6 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk (cat pt-3 k "init")) (stor lk (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k))))
  (label 1981)
  (parent 1135)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k skey)
    (lk locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (3 2)) ((2 1) (3 3)) ((2 1) (6 3))
    ((3 4) (0 0)) ((4 1) (2 0)) ((4 1) (5 0)) ((4 2) (3 0))
    ((4 2) (6 0)) ((5 1) (0 0)) ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule invShearsRule)
  (operation channel-test (displaced 7 2 set-wrap 2)
    (ch-msg lk-0 (cat pt-5 k1 "wrap")) (6 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk (cat pt-3 k "init")) (stor lk (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k))))
  (label 1982)
  (parent 1135)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k skey)
    (lk locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (3 2)) ((2 1) (3 3)) ((2 1) (6 3))
    ((3 4) (0 0)) ((4 1) (1 0)) ((4 1) (5 0)) ((4 2) (3 0))
    ((4 2) (6 0)) ((5 1) (0 0)) ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule invShearsRule)
  (operation channel-test (displaced 7 2 set-wrap 2)
    (ch-msg lk-0 (cat pt-5 k1 "wrap")) (6 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk (cat pt-3 k "init")) (stor lk (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k))))
  (label 1983)
  (parent 1135)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 cur mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (defstrand set-wrap 2 (cur cur) (k k1) (lk lk-0))
  (precedes ((1 1) (2 0)) ((1 1) (3 2)) ((2 1) (3 3)) ((3 4) (0 0))
    ((4 1) (5 0)) ((4 2) (3 0)) ((4 2) (6 0)) ((5 1) (7 0))
    ((6 4) (0 0)) ((7 1) (6 3)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 cur) (cat k1 "wrap"))
  (facts (neq cur "decrypt") (neq cur "wrap") (neq "init" "decrypt")
    (neq "init" "wrap"))
  (rule shearsRule invShearsRule fact-set-decrypt-neq0
    trRl_set-wrap-at-1 gen-st-set-wrap-0 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation channel-test (added-strand set-wrap 2)
    (ch-msg lk-0 (cat pt-5 k1 "wrap")) (6 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-5 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-6 k1 cur)) (stor lk-0 (cat pt-5 k1 "wrap"))))
  (label 1984)
  (parent 1135)
  (unrealized (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 cur mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur cur) (k k1) (lk lk))
  (precedes ((1 1) (2 3)) ((2 4) (0 0)) ((3 1) (4 0)) ((3 1) (5 2))
    ((3 2) (2 0)) ((4 1) (7 0)) ((5 4) (0 0)) ((6 1) (1 0))
    ((6 1) (2 2)) ((6 2) (5 0)) ((7 1) (5 3)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 cur) (cat k1 "wrap"))
  (facts (neq cur "decrypt") (neq cur "wrap") (neq "init" "decrypt")
    (neq "init" "wrap"))
  (rule shearsRule invShearsRule fact-set-decrypt-neq0
    trRl_set-wrap-at-1 gen-st-set-wrap-0 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation channel-test (added-strand set-wrap 2)
    (ch-msg lk (cat pt-3 k1 "wrap")) (5 3))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-5 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt-5 k "init"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init"))
      (send (hash k)))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt-3 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-4 old-0)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k)))
    ((load lk (cat pt-6 k1 cur)) (stor lk (cat pt-3 k1 "wrap"))))
  (label 1985)
  (parent 1136)
  (unrealized (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 cur mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur cur) (k k1) (lk lk-0))
  (precedes ((1 1) (2 3)) ((1 1) (7 0)) ((2 4) (0 0)) ((3 1) (4 0))
    ((3 2) (2 0)) ((4 1) (0 0)) ((5 4) (0 0)) ((6 1) (1 0))
    ((6 1) (2 2)) ((6 2) (5 0)) ((7 1) (5 3)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 cur) (cat k1 "wrap"))
  (facts (neq cur "decrypt") (neq cur "wrap") (neq "init" "decrypt")
    (neq "init" "wrap"))
  (rule shearsRule invShearsRule fact-set-decrypt-neq0
    trRl_set-wrap-at-1 gen-st-set-wrap-0 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation channel-test (added-strand set-wrap 2)
    (ch-msg lk-0 (cat pt-3 k1 "wrap")) (5 3))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-5 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt-5 k "init"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init"))
      (send (hash k)))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-5 k "init"))
      (load lk-0 (cat pt-3 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-4 old-0)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-6 k1 cur)) (stor lk-0 (cat pt-3 k1 "wrap"))))
  (label 1986)
  (parent 1140)
  (unrealized (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k skey)
    (lk locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (3 2)) ((2 1) (3 3)) ((2 1) (6 3))
    ((3 4) (0 0)) ((4 1) (5 0)) ((4 2) (3 0)) ((4 2) (6 0))
    ((5 1) (1 0)) ((5 1) (6 2)) ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule invShearsRule shearsRule)
  (operation channel-test (displaced 7 2 set-wrap 2)
    (ch-msg lk-0 (cat pt-5 k1 "wrap")) (6 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk (cat pt-3 k "init")) (stor lk (cat pt-4 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-4 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k))))
  (label 1987)
  (parent 1146)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k skey)
    (lk locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (3 2)) ((2 1) (3 3)) ((2 1) (6 3))
    ((3 4) (0 0)) ((4 1) (1 0)) ((4 1) (5 0)) ((4 2) (3 0))
    ((4 2) (6 0)) ((5 1) (2 0)) ((5 1) (6 2)) ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule invShearsRule shearsRule)
  (operation channel-test (displaced 7 2 set-wrap 2)
    (ch-msg lk-0 (cat pt-5 k1 "wrap")) (6 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk (cat pt-3 k "init")) (stor lk (cat pt-4 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-4 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k))))
  (label 1988)
  (parent 1146)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k skey)
    (lk locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (3 2)) ((2 1) (3 3)) ((2 1) (6 3))
    ((3 4) (0 0)) ((4 1) (2 0)) ((4 1) (5 0)) ((4 2) (3 0))
    ((4 2) (6 0)) ((5 1) (6 2)) ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule invShearsRule)
  (operation channel-test (displaced 7 2 set-wrap 2)
    (ch-msg lk-0 (cat pt-5 k1 "wrap")) (6 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk (cat pt-3 k "init")) (stor lk (cat pt-4 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-4 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k))))
  (label 1989)
  (parent 1146)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k skey)
    (lk locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (3 2)) ((2 1) (3 3)) ((2 1) (6 3))
    ((3 4) (0 0)) ((4 1) (1 0)) ((4 1) (5 0)) ((4 2) (3 0))
    ((4 2) (6 0)) ((5 1) (6 2)) ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule invShearsRule)
  (operation channel-test (displaced 7 2 set-wrap 2)
    (ch-msg lk-0 (cat pt-5 k1 "wrap")) (6 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk (cat pt-3 k "init")) (stor lk (cat pt-4 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-4 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k))))
  (label 1990)
  (parent 1146)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k skey)
    (lk locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (3 2)) ((2 1) (3 3)) ((2 1) (6 3))
    ((3 4) (0 0)) ((4 1) (5 0)) ((4 2) (3 0)) ((4 2) (6 0))
    ((5 1) (2 0)) ((5 1) (6 2)) ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule invShearsRule)
  (operation channel-test (displaced 7 2 set-wrap 2)
    (ch-msg lk-0 (cat pt-5 k1 "wrap")) (6 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk (cat pt-3 k "init")) (stor lk (cat pt-4 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-4 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k))))
  (label 1991)
  (parent 1146)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((1 1) (3 2)) ((2 1) (3 3)) ((3 4) (0 0))
    ((4 1) (5 0)) ((4 2) (3 0)) ((4 2) (6 0)) ((5 1) (6 2))
    ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation channel-test (displaced 7 5 set-wrap 2)
    (ch-msg lk-0 (cat pt-5 k1 "wrap")) (6 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt-4 k "wrap"))
      (load lk-0 (cat pt-4 k "wrap")) (send (enc k k))))
  (label 1992)
  (parent 1146)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 cur mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk-0))
  (defstrand set-wrap 2 (cur cur) (k k1) (lk lk-0))
  (precedes ((1 1) (2 0)) ((1 1) (3 2)) ((2 1) (3 3)) ((3 4) (0 0))
    ((4 1) (5 0)) ((4 2) (3 0)) ((4 2) (6 0)) ((5 1) (6 2))
    ((5 1) (7 0)) ((6 4) (0 0)) ((7 1) (6 3)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 cur) (cat k1 "wrap"))
  (facts (neq cur "decrypt") (neq cur "wrap") (neq "init" "decrypt")
    (neq "init" "wrap"))
  (rule invShearsRule fact-set-decrypt-neq0 trRl_set-wrap-at-1
    gen-st-set-wrap-0 fact-set-wrap-neq0 trRl_make-at-0)
  (operation channel-test (added-strand set-wrap 2)
    (ch-msg lk-0 (cat pt-5 k1 "wrap")) (6 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-4 k "wrap"))
      (load lk-0 (cat pt-5 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-6 k1 cur)) (stor lk-0 (cat pt-5 k1 "wrap"))))
  (label 1993)
  (parent 1146)
  (unrealized (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k skey)
    (lk locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (precedes ((1 1) (2 3)) ((1 1) (5 3)) ((2 4) (0 0)) ((3 1) (1 0))
    ((3 1) (4 0)) ((3 2) (2 0)) ((4 1) (5 2)) ((5 4) (0 0))
    ((6 1) (1 0)) ((6 1) (2 2)) ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule invShearsRule)
  (operation channel-test (displaced 7 1 set-wrap 2)
    (ch-msg lk-0 (cat pt-5 k1 "wrap")) (5 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt-4 k "init")) (stor lk (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-4 k "init"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init"))
      (send (hash k)))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-2 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-2 k "wrap"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-3 old-0)) (stor lk (cat pt-4 k "init"))
      (send (hash k))))
  (label 1994)
  (parent 1147)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k skey)
    (lk locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (precedes ((1 1) (2 3)) ((1 1) (5 3)) ((2 4) (0 0)) ((3 1) (4 0))
    ((3 1) (6 0)) ((3 2) (2 0)) ((4 1) (5 2)) ((5 4) (0 0))
    ((6 1) (1 0)) ((6 1) (2 2)) ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule invShearsRule)
  (operation channel-test (displaced 7 1 set-wrap 2)
    (ch-msg lk-0 (cat pt-5 k1 "wrap")) (5 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt-4 k "init")) (stor lk (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-4 k "init"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init"))
      (send (hash k)))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-2 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-2 k "wrap"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-3 old-0)) (stor lk (cat pt-4 k "init"))
      (send (hash k))))
  (label 1995)
  (parent 1147)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k skey)
    (lk locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (precedes ((1 1) (2 3)) ((1 1) (5 3)) ((2 4) (0 0)) ((3 1) (4 0))
    ((3 2) (2 0)) ((4 1) (1 0)) ((4 1) (5 2)) ((5 4) (0 0))
    ((6 1) (1 0)) ((6 1) (2 2)) ((6 2) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule invShearsRule)
  (operation channel-test (displaced 7 1 set-wrap 2)
    (ch-msg lk-0 (cat pt-5 k1 "wrap")) (5 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt-4 k "init")) (stor lk (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-4 k "init"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init"))
      (send (hash k)))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-2 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-2 k "wrap"))
      (load lk (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-3 old-0)) (stor lk (cat pt-4 k "init"))
      (send (hash k))))
  (label 1996)
  (parent 1147)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 cur mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur cur) (k k1) (lk lk))
  (precedes ((1 1) (2 3)) ((2 4) (0 0)) ((3 1) (4 0)) ((3 2) (2 0))
    ((4 1) (5 2)) ((4 1) (7 0)) ((5 4) (0 0)) ((6 1) (1 0))
    ((6 1) (2 2)) ((6 2) (5 0)) ((7 1) (5 3)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 cur) (cat k1 "wrap"))
  (facts (neq cur "decrypt") (neq cur "wrap") (neq "init" "decrypt")
    (neq "init" "wrap"))
  (rule invShearsRule fact-set-decrypt-neq0 trRl_set-wrap-at-1
    gen-st-set-wrap-0 fact-set-wrap-neq0 trRl_make-at-0)
  (operation channel-test (added-strand set-wrap 2)
    (ch-msg lk (cat pt-3 k1 "wrap")) (5 3))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-5 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt-5 k "init"))
      (load lk-0 (cat pt k "wrap")) (send (enc k k)))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init"))
      (send (hash k)))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-2 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-2 k "wrap"))
      (load lk (cat pt-3 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-4 old-0)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k)))
    ((load lk (cat pt-6 k1 cur)) (stor lk (cat pt-3 k1 "wrap"))))
  (label 1997)
  (parent 1147)
  (unrealized (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k skey)
    (lk locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (3 2)) ((2 1) (3 3)) ((2 1) (6 3))
    ((3 4) (0 0)) ((4 1) (5 0)) ((4 2) (3 0)) ((4 2) (6 0))
    ((5 1) (1 0)) ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule invShearsRule shearsRule)
  (operation channel-test (displaced 7 2 set-wrap 2)
    (ch-msg lk-0 (cat pt-5 k1 "wrap")) (6 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk (cat pt-3 k "init")) (stor lk (cat pt-4 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k))))
  (label 1998)
  (parent 1150)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k skey)
    (lk locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (3 2)) ((2 1) (3 3)) ((2 1) (6 3))
    ((3 4) (0 0)) ((4 1) (1 0)) ((4 1) (5 0)) ((4 2) (3 0))
    ((4 2) (6 0)) ((5 1) (2 0)) ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule invShearsRule shearsRule)
  (operation channel-test (displaced 7 2 set-wrap 2)
    (ch-msg lk-0 (cat pt-5 k1 "wrap")) (6 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk (cat pt-3 k "init")) (stor lk (cat pt-4 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k))))
  (label 1999)
  (parent 1150)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k skey)
    (lk locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (3 2)) ((2 1) (3 3)) ((2 1) (6 3))
    ((3 4) (0 0)) ((4 1) (2 0)) ((4 1) (5 0)) ((4 2) (3 0))
    ((4 2) (6 0)) ((5 1) (0 0)) ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule invShearsRule)
  (operation channel-test (displaced 7 2 set-wrap 2)
    (ch-msg lk-0 (cat pt-5 k1 "wrap")) (6 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk (cat pt-3 k "init")) (stor lk (cat pt-4 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k))))
  (label 2000)
  (parent 1150)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k skey)
    (lk locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (3 2)) ((2 1) (3 3)) ((2 1) (6 3))
    ((3 4) (0 0)) ((4 1) (1 0)) ((4 1) (5 0)) ((4 2) (3 0))
    ((4 2) (6 0)) ((5 1) (0 0)) ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule invShearsRule)
  (operation channel-test (displaced 7 2 set-wrap 2)
    (ch-msg lk-0 (cat pt-5 k1 "wrap")) (6 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk (cat pt-3 k "init")) (stor lk (cat pt-4 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k))))
  (label 2001)
  (parent 1150)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((1 1) (3 2)) ((2 1) (3 3)) ((3 4) (0 0))
    ((4 1) (5 0)) ((4 2) (3 0)) ((4 2) (6 0)) ((5 1) (6 3))
    ((6 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation channel-test (displaced 7 5 set-wrap 2)
    (ch-msg lk-0 (cat pt-5 k1 "wrap")) (6 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-4 k "wrap")) (send (enc k k))))
  (label 2002)
  (parent 1150)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 cur mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (defstrand set-wrap 2 (cur cur) (k k1) (lk lk-0))
  (precedes ((1 1) (2 0)) ((1 1) (3 2)) ((2 1) (3 3)) ((3 4) (0 0))
    ((4 1) (5 0)) ((4 2) (3 0)) ((4 2) (6 0)) ((5 1) (7 0))
    ((6 4) (0 0)) ((7 1) (6 3)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 cur) (cat k1 "wrap"))
  (facts (neq cur "decrypt") (neq cur "wrap") (neq "init" "decrypt")
    (neq "init" "wrap"))
  (rule shearsRule invShearsRule fact-set-decrypt-neq0
    trRl_set-wrap-at-1 gen-st-set-wrap-0 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation channel-test (added-strand set-wrap 2)
    (ch-msg lk-0 (cat pt-5 k1 "wrap")) (6 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-5 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-6 k1 cur)) (stor lk-0 (cat pt-5 k1 "wrap"))))
  (label 2003)
  (parent 1150)
  (unrealized (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 old-2 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pt-9 pval)
    (k k1 skey) (lk lk-0 lk-1 lk-2 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1) (lk lk-0))
  (defstrand make 3 (old old-2) (k k) (lk lk-2))
  (precedes ((1 1) (2 0)) ((2 1) (0 0)) ((3 1) (4 0)) ((4 1) (6 2))
    ((5 1) (0 0)) ((6 4) (0 0)) ((7 2) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation encryption-test (added-strand make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-4 k "decrypt"))
      (load lk-0 (cat pt-7 k1 "wrap")) (send (enc k k1)))
    ((load lk-2 (cat pt-8 old-2)) (stor lk-2 (cat pt-9 k "init"))
      (send (hash k))))
  (label 237)
  (parent 107)
  (unrealized (6 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1) (lk lk-0))
  (deflistener k)
  (precedes ((1 1) (2 0)) ((2 1) (0 0)) ((3 1) (4 0)) ((4 1) (6 2))
    ((5 1) (0 0)) ((6 4) (0 0)) ((7 1) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (added-listener k) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-4 k "decrypt"))
      (load lk-0 (cat pt-7 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k)))
  (label 238)
  (parent 107)
  (unrealized (6 3) (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 old-2 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pt-9 pval)
    (k k1 skey) (lk lk-0 lk-1 lk-2 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-1))
  (defstrand make 3 (old old-2) (k k) (lk lk-2))
  (precedes ((1 1) (2 0)) ((2 1) (0 0)) ((3 1) (4 0)) ((4 1) (0 0))
    ((5 1) (6 2)) ((6 4) (0 0)) ((7 2) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation encryption-test (added-strand make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk-1 (cat pt-6 k "init"))
      (load lk-1 (cat pt-7 k1 "wrap")) (send (enc k k1)))
    ((load lk-2 (cat pt-8 old-2)) (stor lk-2 (cat pt-9 k "init"))
      (send (hash k))))
  (label 242)
  (parent 108)
  (unrealized (6 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-1))
  (deflistener k)
  (precedes ((1 1) (2 0)) ((2 1) (0 0)) ((3 1) (4 0)) ((4 1) (0 0))
    ((5 1) (6 2)) ((6 4) (0 0)) ((7 1) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (added-listener k) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk-1 (cat pt-6 k "init"))
      (load lk-1 (cat pt-7 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k)))
  (label 243)
  (parent 108)
  (unrealized (6 3) (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 old-2 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pt-9 pval)
    (k k1 skey) (lk lk-0 lk-1 lk-2 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (defstrand make 3 (old old-2) (k k) (lk lk-2))
  (precedes ((1 1) (2 0)) ((2 1) (0 0)) ((3 1) (4 0)) ((3 1) (6 2))
    ((4 1) (0 0)) ((5 1) (0 0)) ((6 4) (0 0)) ((7 2) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation encryption-test (added-strand make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-7 k1 "wrap")) (send (enc k k1)))
    ((load lk-2 (cat pt-8 old-2)) (stor lk-2 (cat pt-9 k "init"))
      (send (hash k))))
  (label 246)
  (parent 109)
  (unrealized (6 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (deflistener k)
  (precedes ((1 1) (2 0)) ((2 1) (0 0)) ((3 1) (4 0)) ((3 1) (6 2))
    ((4 1) (0 0)) ((5 1) (0 0)) ((6 4) (0 0)) ((7 1) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (added-listener k) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-7 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k)))
  (label 247)
  (parent 109)
  (unrealized (6 3) (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 old-2 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pt-9 pval)
    (k k1 skey) (lk lk-0 lk-1 lk-2 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old-2) (k k) (lk lk-2))
  (precedes ((1 1) (2 0)) ((2 1) (6 2)) ((3 1) (4 0)) ((4 1) (0 0))
    ((5 1) (0 0)) ((6 4) (0 0)) ((7 2) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation encryption-test (added-strand make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-7 k1 "wrap")) (send (enc k k1)))
    ((load lk-2 (cat pt-8 old-2)) (stor lk-2 (cat pt-9 k "init"))
      (send (hash k))))
  (label 250)
  (parent 110)
  (unrealized (6 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (precedes ((1 1) (2 0)) ((2 1) (6 2)) ((3 1) (4 0)) ((4 1) (0 0))
    ((5 1) (0 0)) ((6 4) (0 0)) ((7 1) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (added-listener k) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-7 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k)))
  (label 251)
  (parent 110)
  (unrealized (6 3) (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 old-2 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pt-9 pval)
    (k k1 skey) (lk lk-0 lk-1 lk-2 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old-2) (k k) (lk lk-2))
  (precedes ((1 1) (2 0)) ((1 1) (6 2)) ((2 1) (0 0)) ((3 1) (4 0))
    ((4 1) (0 0)) ((5 1) (0 0)) ((6 4) (0 0)) ((7 2) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation encryption-test (added-strand make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-7 k1 "wrap")) (send (enc k k1)))
    ((load lk-2 (cat pt-8 old-2)) (stor lk-2 (cat pt-9 k "init"))
      (send (hash k))))
  (label 254)
  (parent 111)
  (unrealized (6 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (precedes ((1 1) (2 0)) ((1 1) (6 2)) ((2 1) (0 0)) ((3 1) (4 0))
    ((4 1) (0 0)) ((5 1) (0 0)) ((6 4) (0 0)) ((7 1) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (added-listener k) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-7 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k)))
  (label 255)
  (parent 111)
  (unrealized (6 3) (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 cur mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1) (lk lk-0))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand set-wrap 2 (cur cur) (k k1) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (0 0)) ((3 1) (4 0)) ((4 1) (5 2))
    ((4 1) (7 0)) ((5 4) (0 0)) ((6 2) (5 0)) ((7 1) (5 3)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k1 cur) (cat k1 "wrap"))
  (facts (neq cur "decrypt") (neq cur "wrap") (neq "init" "decrypt")
    (neq "init" "wrap"))
  (rule invShearsRule fact-set-decrypt-neq0 trRl_set-wrap-at-1
    gen-st-set-wrap-0 fact-set-wrap-neq0 trRl_make-at-0)
  (operation channel-test (added-strand set-wrap 2)
    (ch-msg lk-0 (cat pt-5 k1 "wrap")) (5 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-4 k "decrypt"))
      (load lk-0 (cat pt-5 k1 "wrap")) (send (enc k k1)))
    ((load lk-1 (cat pt-6 old-1)) (stor lk-1 (cat pt-7 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-8 k1 cur)) (stor lk-0 (cat pt-5 k1 "wrap"))))
  (label 266)
  (parent 114)
  (unrealized (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1) (lk lk-0))
  (deflistener k)
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((2 1) (0 0)) ((3 1) (4 0)) ((4 1) (5 2))
    ((5 4) (0 0)) ((6 1) (5 0)) ((7 1) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation nonce-test (added-strand make 2) k (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-4 k "decrypt"))
      (load lk-0 (cat pt-5 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-1 (cat pt-6 old-1)) (stor lk-1 (cat pt-7 k "init"))))
  (label 269)
  (parent 115)
  (unrealized (5 3) (6 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 cur mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand set-wrap 2 (cur cur) (k k1) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (0 0)) ((3 1) (4 0)) ((3 1) (5 2))
    ((4 1) (7 0)) ((5 4) (0 0)) ((6 2) (5 0)) ((7 1) (5 3)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 cur) (cat k1 "wrap"))
  (facts (neq cur "decrypt") (neq cur "wrap") (neq "init" "decrypt")
    (neq "init" "wrap"))
  (rule shearsRule invShearsRule fact-set-decrypt-neq0
    trRl_set-wrap-at-1 gen-st-set-wrap-0 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation channel-test (added-strand set-wrap 2)
    (ch-msg lk-0 (cat pt-5 k1 "wrap")) (5 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-5 k1 "wrap")) (send (enc k k1)))
    ((load lk-1 (cat pt-6 old-1)) (stor lk-1 (cat pt-7 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-8 k1 cur)) (stor lk-0 (cat pt-5 k1 "wrap"))))
  (label 276)
  (parent 118)
  (unrealized (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (deflistener k)
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((2 1) (0 0)) ((3 1) (4 0)) ((3 1) (5 2))
    ((4 1) (0 0)) ((5 4) (0 0)) ((6 1) (5 0)) ((7 1) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation nonce-test (added-strand make 2) k (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-5 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-1 (cat pt-6 old-1)) (stor lk-1 (cat pt-7 k "init"))))
  (label 279)
  (parent 119)
  (unrealized (5 3) (6 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 cur mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand set-wrap 2 (cur cur) (k k1) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (5 2)) ((2 1) (7 0)) ((3 1) (4 0))
    ((4 1) (0 0)) ((5 4) (0 0)) ((6 2) (5 0)) ((7 1) (5 3)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 cur) (cat k1 "wrap"))
  (facts (neq cur "decrypt") (neq cur "wrap") (neq "init" "decrypt")
    (neq "init" "wrap"))
  (rule invShearsRule fact-set-decrypt-neq0 trRl_set-wrap-at-1
    gen-st-set-wrap-0 fact-set-wrap-neq0 trRl_make-at-0)
  (operation channel-test (added-strand set-wrap 2)
    (ch-msg lk (cat pt-5 k1 "wrap")) (5 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-5 k1 "wrap")) (send (enc k k1)))
    ((load lk-1 (cat pt-6 old-1)) (stor lk-1 (cat pt-7 k "init"))
      (send (hash k)))
    ((load lk (cat pt-8 k1 cur)) (stor lk (cat pt-5 k1 "wrap"))))
  (label 284)
  (parent 122)
  (unrealized (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((2 1) (5 2)) ((3 1) (4 0)) ((4 1) (0 0))
    ((5 4) (0 0)) ((6 1) (5 0)) ((7 1) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation nonce-test (added-strand make 2) k (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-5 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-1 (cat pt-6 old-1)) (stor lk-1 (cat pt-7 k "init"))))
  (label 287)
  (parent 123)
  (unrealized (5 3) (6 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 cur mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand set-wrap 2 (cur cur) (k k1) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (5 2)) ((2 1) (7 0)) ((3 1) (4 0))
    ((4 1) (0 0)) ((5 4) (0 0)) ((6 2) (5 0)) ((7 1) (5 3)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 cur) (cat k1 "wrap"))
  (facts (neq cur "decrypt") (neq cur "wrap") (neq "init" "decrypt")
    (neq "init" "wrap"))
  (rule shearsRule invShearsRule fact-set-decrypt-neq0
    trRl_set-wrap-at-1 gen-st-set-wrap-0 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation channel-test (added-strand set-wrap 2)
    (ch-msg lk (cat pt-5 k1 "wrap")) (5 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-5 k1 "wrap")) (send (enc k k1)))
    ((load lk-1 (cat pt-6 old-1)) (stor lk-1 (cat pt-7 k "init"))
      (send (hash k)))
    ((load lk (cat pt-8 k1 cur)) (stor lk (cat pt-5 k1 "wrap"))))
  (label 292)
  (parent 126)
  (unrealized (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((1 1) (5 2)) ((2 1) (0 0)) ((3 1) (4 0))
    ((4 1) (0 0)) ((5 4) (0 0)) ((6 1) (5 0)) ((7 1) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation nonce-test (added-strand make 2) k (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-5 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-1 (cat pt-6 old-1)) (stor lk-1 (cat pt-7 k "init"))))
  (label 295)
  (parent 127)
  (unrealized (5 3) (6 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 old-2 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand set-wrap 2 (cur "init") (k k1) (lk lk-0))
  (defstrand make 2 (old old-2) (k k1) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (0 0)) ((3 1) (4 2)) ((3 1) (7 0))
    ((4 4) (0 0)) ((5 2) (4 0)) ((6 1) (4 3)) ((7 1) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule invShearsRule trRl_make-at-0 trRl_make-at-1)
  (operation channel-test (added-strand make 2)
    (ch-msg lk-0 (cat pt-7 k1 "init")) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-7 k1 "init")) (stor lk-0 (cat pt-4 k1 "wrap")))
    ((load lk-0 (cat pt-8 old-2)) (stor lk-0 (cat pt-7 k1 "init"))))
  (label 317)
  (parent 132)
  (realized)
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (deflistener k)
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((2 1) (0 0)) ((3 1) (4 2)) ((4 4) (0 0))
    ((5 1) (4 0)) ((6 1) (7 0)) ((7 1) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-wrap-at-1 trRl_make-at-0)
  (operation nonce-test (added-strand set-wrap 2) k (5 0)
    (ch-msg lk-1 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init")))
    ((load lk-1 (cat pt-6 k "init")) (stor lk-1 (cat pt-7 k "wrap"))))
  (label 323)
  (parent 135)
  (unrealized (4 3) (5 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (deflistener k)
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((2 1) (0 0)) ((3 1) (4 2)) ((4 4) (0 0))
    ((5 1) (4 0)) ((6 1) (7 0)) ((7 1) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-decrypt-at-1 trRl_make-at-0)
  (operation nonce-test (added-strand set-decrypt 2) k (5 0)
    (ch-msg lk-1 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init")))
    ((load lk-1 (cat pt-6 k "init"))
      (stor lk-1 (cat pt-7 k "decrypt"))))
  (label 324)
  (parent 135)
  (unrealized (4 3) (5 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 k1-0 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (deflistener k)
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk-1))
  (precedes ((1 1) (2 0)) ((2 1) (0 0)) ((3 1) (4 2)) ((4 4) (0 0))
    ((5 1) (4 0)) ((6 1) (7 2)) ((7 4) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (5 0)
    (ch-msg lk-1 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-1 (cat pt-6 k "init"))
      (load lk-1 (cat pt-7 k1-0 "wrap")) (send (enc k k1-0))))
  (label 325)
  (parent 135)
  (unrealized (4 3) (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 old-2 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand set-wrap 2 (cur "init") (k k1) (lk lk))
  (defstrand make 2 (old old-2) (k k1) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (4 2)) ((2 1) (7 0)) ((3 1) (0 0))
    ((4 4) (0 0)) ((5 2) (4 0)) ((6 1) (4 3)) ((7 1) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule invShearsRule trRl_make-at-0 trRl_make-at-1)
  (operation channel-test (added-strand make 2)
    (ch-msg lk (cat pt-7 k1 "init")) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init"))
      (send (hash k)))
    ((load lk (cat pt-7 k1 "init")) (stor lk (cat pt-4 k1 "wrap")))
    ((load lk (cat pt-8 old-2)) (stor lk (cat pt-7 k1 "init"))))
  (label 344)
  (parent 139)
  (realized)
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((2 1) (4 2)) ((3 1) (0 0)) ((4 4) (0 0))
    ((5 1) (4 0)) ((6 1) (7 0)) ((7 1) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-wrap-at-1 trRl_make-at-0)
  (operation nonce-test (added-strand set-wrap 2) k (5 0)
    (ch-msg lk-1 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init")))
    ((load lk-1 (cat pt-6 k "init")) (stor lk-1 (cat pt-7 k "wrap"))))
  (label 348)
  (parent 141)
  (unrealized (4 3) (5 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((2 1) (4 2)) ((3 1) (0 0)) ((4 4) (0 0))
    ((5 1) (4 0)) ((6 1) (7 0)) ((7 1) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-decrypt-at-1 trRl_make-at-0)
  (operation nonce-test (added-strand set-decrypt 2) k (5 0)
    (ch-msg lk-1 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init")))
    ((load lk-1 (cat pt-6 k "init"))
      (stor lk-1 (cat pt-7 k "decrypt"))))
  (label 349)
  (parent 141)
  (unrealized (4 3) (5 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 k1-0 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk-1))
  (precedes ((1 1) (2 0)) ((2 1) (4 2)) ((3 1) (0 0)) ((4 4) (0 0))
    ((5 1) (4 0)) ((6 1) (7 2)) ((7 4) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap")
    (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (5 0)
    (ch-msg lk-1 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-1 (cat pt-6 k "init"))
      (load lk-1 (cat pt-7 k1-0 "wrap")) (send (enc k k1-0))))
  (label 350)
  (parent 141)
  (unrealized (4 3) (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 old-2 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand set-wrap 2 (cur "init") (k k1) (lk lk))
  (defstrand make 2 (old old-2) (k k1) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (4 2)) ((2 1) (7 0)) ((3 1) (0 0))
    ((4 4) (0 0)) ((5 2) (4 0)) ((6 1) (4 3)) ((7 1) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule invShearsRule trRl_make-at-0 trRl_make-at-1)
  (operation channel-test (added-strand make 2)
    (ch-msg lk (cat pt-7 k1 "init")) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init"))
      (send (hash k)))
    ((load lk (cat pt-7 k1 "init")) (stor lk (cat pt-4 k1 "wrap")))
    ((load lk (cat pt-8 old-2)) (stor lk (cat pt-7 k1 "init"))))
  (label 365)
  (parent 145)
  (realized)
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((1 1) (4 2)) ((2 1) (0 0)) ((3 1) (0 0))
    ((4 4) (0 0)) ((5 1) (4 0)) ((6 1) (7 0)) ((7 1) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-wrap-at-1 trRl_make-at-0)
  (operation nonce-test (added-strand set-wrap 2) k (5 0)
    (ch-msg lk-1 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init")))
    ((load lk-1 (cat pt-6 k "init")) (stor lk-1 (cat pt-7 k "wrap"))))
  (label 368)
  (parent 147)
  (unrealized (4 3) (5 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((1 1) (4 2)) ((2 1) (0 0)) ((3 1) (0 0))
    ((4 4) (0 0)) ((5 1) (4 0)) ((6 1) (7 0)) ((7 1) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-decrypt-at-1 trRl_make-at-0)
  (operation nonce-test (added-strand set-decrypt 2) k (5 0)
    (ch-msg lk-1 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init")))
    ((load lk-1 (cat pt-6 k "init"))
      (stor lk-1 (cat pt-7 k "decrypt"))))
  (label 369)
  (parent 147)
  (unrealized (4 3) (5 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 k1-0 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk-1))
  (precedes ((1 1) (2 0)) ((1 1) (4 2)) ((2 1) (0 0)) ((3 1) (0 0))
    ((4 4) (0 0)) ((5 1) (4 0)) ((6 1) (7 2)) ((7 4) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (5 0)
    (ch-msg lk-1 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-1 (cat pt-6 k "init"))
      (load lk-1 (cat pt-7 k1-0 "wrap")) (send (enc k k1-0))))
  (label 370)
  (parent 147)
  (unrealized (4 3) (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "wrap") (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (3 2)) ((3 4) (0 0)) ((4 1) (3 0))
    ((5 1) (6 0)) ((6 1) (7 0)) ((7 1) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "wrap" "decrypt") (neq "wrap" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule fact-set-decrypt-neq0 trRl_set-wrap-at-1 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation nonce-test (added-strand set-wrap 2) k (4 0)
    (ch-msg lk-0 (cat pt-4 k "init")) (ch-msg lk-0 (cat pt-5 k "wrap")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt-5 k "wrap")))
    ((load lk-0 (cat pt-5 k "wrap")) (stor lk-0 (cat pt-6 k "wrap"))))
  (label 416)
  (parent 164)
  (unrealized (3 3) (4 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((2 1) (3 2)) ((3 4) (0 0)) ((4 1) (3 0))
    ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation nonce-test (added-strand make 2) k (4 0)
    (ch-msg lk-0 (cat pt-4 k "init")) (ch-msg lk-0 (cat pt-5 k "wrap")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt-5 k "wrap")))
    ((load lk-1 (cat pt-6 old-1)) (stor lk-1 (cat pt-7 k "init"))))
  (label 417)
  (parent 164)
  (unrealized (3 3) (4 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "wrap") (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (3 2)) ((3 4) (0 0)) ((4 1) (3 0))
    ((5 1) (6 0)) ((6 1) (7 0)) ((7 1) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "wrap" "decrypt") (neq "wrap" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-decrypt-at-1 fact-set-decrypt-neq0 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation nonce-test (added-strand set-decrypt 2) k (4 0)
    (ch-msg lk-0 (cat pt-4 k "init")) (ch-msg lk-0 (cat pt-5 k "wrap")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt-5 k "wrap")))
    ((load lk-0 (cat pt-5 k "wrap"))
      (stor lk-0 (cat pt-6 k "decrypt"))))
  (label 418)
  (parent 164)
  (unrealized (3 3) (4 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1-0) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (3 2)) ((3 4) (0 0)) ((4 1) (3 0))
    ((5 1) (6 0)) ((6 1) (7 2)) ((7 4) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap")
    (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (4 0)
    (ch-msg lk-0 (cat pt-4 k "init")) (ch-msg lk-0 (cat pt-5 k "wrap")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt-5 k "wrap")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-0 (cat pt-5 k "wrap"))
      (load lk-0 (cat pt-6 k1-0 "wrap")) (send (enc k k1-0))))
  (label 419)
  (parent 164)
  (unrealized (3 3) (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (3 2)) ((3 4) (0 0)) ((4 1) (3 0))
    ((5 1) (6 0)) ((5 1) (7 2)) ((6 1) (4 0)) ((7 4) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap")
    (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (4 0)
    (ch-msg lk-0 (cat pt-4 k "init")) (ch-msg lk-0 (cat pt-5 k "wrap")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt-5 k "wrap")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-0 (cat pt-4 k "init"))
      (load lk-0 (cat pt-6 k1-0 "wrap")) (send (enc k k1-0))))
  (label 420)
  (parent 164)
  (unrealized (3 3) (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "decrypt") (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (3 2)) ((3 4) (0 0)) ((4 1) (3 0))
    ((5 1) (6 0)) ((6 1) (7 0)) ((7 1) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k "wrap")
    (cat k1 "wrap"))
  (facts (neq "decrypt" "decrypt") (neq "decrypt" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule fact-set-decrypt-neq0 trRl_set-wrap-at-1 gen-st-set-wrap-0
    fact-set-wrap-neq0 trRl_make-at-0)
  (operation nonce-test (added-strand set-wrap 2) k (4 0)
    (ch-msg lk-0 (cat pt-4 k "init"))
    (ch-msg lk-0 (cat pt-5 k "decrypt")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt-5 k "decrypt")))
    ((load lk-0 (cat pt-5 k "decrypt"))
      (stor lk-0 (cat pt-6 k "wrap"))))
  (label 421)
  (parent 165)
  (unrealized (3 3) (4 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((2 1) (3 2)) ((3 4) (0 0)) ((4 1) (3 0))
    ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation nonce-test (added-strand make 2) k (4 0)
    (ch-msg lk-0 (cat pt-4 k "init"))
    (ch-msg lk-0 (cat pt-5 k "decrypt")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt-5 k "decrypt")))
    ((load lk-1 (cat pt-6 old-1)) (stor lk-1 (cat pt-7 k "init"))))
  (label 422)
  (parent 165)
  (unrealized (3 3) (4 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "decrypt") (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (3 2)) ((3 4) (0 0)) ((4 1) (3 0))
    ((5 1) (6 0)) ((6 1) (7 0)) ((7 1) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k "wrap")
    (cat k1 "wrap"))
  (facts (neq "decrypt" "decrypt") (neq "decrypt" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-decrypt-at-1 fact-set-decrypt-neq0 gen-st-set-wrap-0
    fact-set-wrap-neq0 trRl_make-at-0)
  (operation nonce-test (added-strand set-decrypt 2) k (4 0)
    (ch-msg lk-0 (cat pt-4 k "init"))
    (ch-msg lk-0 (cat pt-5 k "decrypt")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt-5 k "decrypt")))
    ((load lk-0 (cat pt-5 k "decrypt"))
      (stor lk-0 (cat pt-6 k "decrypt"))))
  (label 423)
  (parent 165)
  (unrealized (3 3) (4 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1-0) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (3 2)) ((3 4) (0 0)) ((4 1) (3 0))
    ((5 1) (6 0)) ((6 1) (7 2)) ((7 4) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k "wrap")
    (cat k1 "wrap") (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0 gen-st-wrap-1)
  (operation nonce-test (added-strand wrap 5) k (4 0)
    (ch-msg lk-0 (cat pt-4 k "init"))
    (ch-msg lk-0 (cat pt-5 k "decrypt")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt-5 k "decrypt")))
    ((recv (hash k)) (recv (hash k1-0))
      (load lk-0 (cat pt-5 k "decrypt"))
      (load lk-0 (cat pt-6 k1-0 "wrap")) (send (enc k k1-0))))
  (label 424)
  (parent 165)
  (unrealized (3 3) (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (3 2)) ((3 4) (0 0)) ((4 1) (3 0))
    ((5 1) (6 0)) ((5 1) (7 2)) ((6 1) (4 0)) ((7 4) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap")
    (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (4 0)
    (ch-msg lk-0 (cat pt-4 k "init"))
    (ch-msg lk-0 (cat pt-5 k "decrypt")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt-5 k "decrypt")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-0 (cat pt-4 k "init"))
      (load lk-0 (cat pt-6 k1-0 "wrap")) (send (enc k k1-0))))
  (label 425)
  (parent 165)
  (unrealized (3 3) (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 k1-0 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk-0))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((2 1) (3 2)) ((3 4) (0 0)) ((4 1) (3 0))
    ((5 1) (6 2)) ((6 4) (4 0)) ((7 2) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap")
    (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation encryption-test (added-strand make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-0 (cat pt-4 k "init"))
      (load lk-0 (cat pt-5 k1-0 "wrap")) (send (enc k k1-0)))
    ((load lk-1 (cat pt-6 old-1)) (stor lk-1 (cat pt-7 k "init"))
      (send (hash k))))
  (label 428)
  (parent 166)
  (unrealized (3 3) (6 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk-0))
  (deflistener k)
  (precedes ((1 1) (2 0)) ((2 1) (3 2)) ((3 4) (0 0)) ((4 1) (3 0))
    ((5 1) (6 2)) ((6 4) (4 0)) ((7 1) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap")
    (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (added-listener k) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-0 (cat pt-4 k "init"))
      (load lk-0 (cat pt-5 k1-0 "wrap")) (send (enc k k1-0)))
    ((recv k) (send k)))
  (label 429)
  (parent 166)
  (unrealized (3 3) (6 3) (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "wrap") (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((1 1) (3 2)) ((2 1) (0 0)) ((3 4) (0 0))
    ((4 1) (3 0)) ((5 1) (6 0)) ((6 1) (7 0)) ((7 1) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "wrap" "decrypt") (neq "wrap" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule fact-set-decrypt-neq0 trRl_set-wrap-at-1 gen-st-set-wrap-0
    fact-set-wrap-neq0 trRl_make-at-0)
  (operation nonce-test (added-strand set-wrap 2) k (4 0)
    (ch-msg lk-0 (cat pt-4 k "init")) (ch-msg lk-0 (cat pt-5 k "wrap")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt-5 k "wrap")))
    ((load lk-0 (cat pt-5 k "wrap")) (stor lk-0 (cat pt-6 k "wrap"))))
  (label 472)
  (parent 182)
  (unrealized (3 3) (4 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((1 1) (3 2)) ((2 1) (0 0)) ((3 4) (0 0))
    ((4 1) (3 0)) ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation nonce-test (added-strand make 2) k (4 0)
    (ch-msg lk-0 (cat pt-4 k "init")) (ch-msg lk-0 (cat pt-5 k "wrap")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt-5 k "wrap")))
    ((load lk-1 (cat pt-6 old-1)) (stor lk-1 (cat pt-7 k "init"))))
  (label 473)
  (parent 182)
  (unrealized (3 3) (4 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "wrap") (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((1 1) (3 2)) ((2 1) (0 0)) ((3 4) (0 0))
    ((4 1) (3 0)) ((5 1) (6 0)) ((6 1) (7 0)) ((7 1) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "wrap" "decrypt") (neq "wrap" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-decrypt-at-1 fact-set-decrypt-neq0 gen-st-set-wrap-0
    fact-set-wrap-neq0 trRl_make-at-0)
  (operation nonce-test (added-strand set-decrypt 2) k (4 0)
    (ch-msg lk-0 (cat pt-4 k "init")) (ch-msg lk-0 (cat pt-5 k "wrap")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt-5 k "wrap")))
    ((load lk-0 (cat pt-5 k "wrap"))
      (stor lk-0 (cat pt-6 k "decrypt"))))
  (label 474)
  (parent 182)
  (unrealized (3 3) (4 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1-0) (lk lk-0))
  (precedes ((1 1) (2 0)) ((1 1) (3 2)) ((2 1) (0 0)) ((3 4) (0 0))
    ((4 1) (3 0)) ((5 1) (6 0)) ((6 1) (7 2)) ((7 4) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap")
    (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0 gen-st-wrap-1)
  (operation nonce-test (added-strand wrap 5) k (4 0)
    (ch-msg lk-0 (cat pt-4 k "init")) (ch-msg lk-0 (cat pt-5 k "wrap")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt-5 k "wrap")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-0 (cat pt-5 k "wrap"))
      (load lk-0 (cat pt-6 k1-0 "wrap")) (send (enc k k1-0))))
  (label 475)
  (parent 182)
  (unrealized (3 3) (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk-0))
  (precedes ((1 1) (2 0)) ((1 1) (3 2)) ((2 1) (0 0)) ((3 4) (0 0))
    ((4 1) (3 0)) ((5 1) (6 0)) ((5 1) (7 2)) ((6 1) (4 0))
    ((7 4) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (4 0)
    (ch-msg lk-0 (cat pt-4 k "init")) (ch-msg lk-0 (cat pt-5 k "wrap")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt-5 k "wrap")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-0 (cat pt-4 k "init"))
      (load lk-0 (cat pt-6 k1-0 "wrap")) (send (enc k k1-0))))
  (label 476)
  (parent 182)
  (unrealized (3 3) (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "decrypt") (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((1 1) (3 2)) ((2 1) (0 0)) ((3 4) (0 0))
    ((4 1) (3 0)) ((5 1) (6 0)) ((6 1) (7 0)) ((7 1) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k1 "wrap"))
  (facts (neq "decrypt" "decrypt") (neq "decrypt" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule fact-set-decrypt-neq0 trRl_set-wrap-at-1 gen-st-set-wrap-0
    fact-set-wrap-neq0 trRl_make-at-0)
  (operation nonce-test (added-strand set-wrap 2) k (4 0)
    (ch-msg lk-0 (cat pt-4 k "init"))
    (ch-msg lk-0 (cat pt-5 k "decrypt")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt-5 k "decrypt")))
    ((load lk-0 (cat pt-5 k "decrypt"))
      (stor lk-0 (cat pt-6 k "wrap"))))
  (label 477)
  (parent 183)
  (unrealized (3 3) (4 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((1 1) (3 2)) ((2 1) (0 0)) ((3 4) (0 0))
    ((4 1) (3 0)) ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation nonce-test (added-strand make 2) k (4 0)
    (ch-msg lk-0 (cat pt-4 k "init"))
    (ch-msg lk-0 (cat pt-5 k "decrypt")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt-5 k "decrypt")))
    ((load lk-1 (cat pt-6 old-1)) (stor lk-1 (cat pt-7 k "init"))))
  (label 478)
  (parent 183)
  (unrealized (3 3) (4 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "decrypt") (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((1 1) (3 2)) ((2 1) (0 0)) ((3 4) (0 0))
    ((4 1) (3 0)) ((5 1) (6 0)) ((6 1) (7 0)) ((7 1) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k1 "wrap"))
  (facts (neq "decrypt" "decrypt") (neq "decrypt" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-decrypt-at-1 fact-set-decrypt-neq0 gen-st-set-wrap-0
    fact-set-wrap-neq0 trRl_make-at-0)
  (operation nonce-test (added-strand set-decrypt 2) k (4 0)
    (ch-msg lk-0 (cat pt-4 k "init"))
    (ch-msg lk-0 (cat pt-5 k "decrypt")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt-5 k "decrypt")))
    ((load lk-0 (cat pt-5 k "decrypt"))
      (stor lk-0 (cat pt-6 k "decrypt"))))
  (label 479)
  (parent 183)
  (unrealized (3 3) (4 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1-0) (lk lk-0))
  (precedes ((1 1) (2 0)) ((1 1) (3 2)) ((2 1) (0 0)) ((3 4) (0 0))
    ((4 1) (3 0)) ((5 1) (6 0)) ((6 1) (7 2)) ((7 4) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k1 "wrap")
    (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0 gen-st-wrap-1)
  (operation nonce-test (added-strand wrap 5) k (4 0)
    (ch-msg lk-0 (cat pt-4 k "init"))
    (ch-msg lk-0 (cat pt-5 k "decrypt")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt-5 k "decrypt")))
    ((recv (hash k)) (recv (hash k1-0))
      (load lk-0 (cat pt-5 k "decrypt"))
      (load lk-0 (cat pt-6 k1-0 "wrap")) (send (enc k k1-0))))
  (label 480)
  (parent 183)
  (unrealized (3 3) (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk-0))
  (precedes ((1 1) (2 0)) ((1 1) (3 2)) ((2 1) (0 0)) ((3 4) (0 0))
    ((4 1) (3 0)) ((5 1) (6 0)) ((5 1) (7 2)) ((6 1) (4 0))
    ((7 4) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (4 0)
    (ch-msg lk-0 (cat pt-4 k "init"))
    (ch-msg lk-0 (cat pt-5 k "decrypt")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt-5 k "decrypt")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-0 (cat pt-4 k "init"))
      (load lk-0 (cat pt-6 k1-0 "wrap")) (send (enc k k1-0))))
  (label 481)
  (parent 183)
  (unrealized (3 3) (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 k1-0 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk-0))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((1 1) (3 2)) ((2 1) (0 0)) ((3 4) (0 0))
    ((4 1) (3 0)) ((5 1) (6 2)) ((6 4) (4 0)) ((7 2) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation encryption-test (added-strand make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-0 (cat pt-4 k "init"))
      (load lk-0 (cat pt-5 k1-0 "wrap")) (send (enc k k1-0)))
    ((load lk-1 (cat pt-6 old-1)) (stor lk-1 (cat pt-7 k "init"))
      (send (hash k))))
  (label 484)
  (parent 184)
  (unrealized (3 3) (6 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk-0))
  (deflistener k)
  (precedes ((1 1) (2 0)) ((1 1) (3 2)) ((2 1) (0 0)) ((3 4) (0 0))
    ((4 1) (3 0)) ((5 1) (6 2)) ((6 4) (4 0)) ((7 1) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (added-listener k) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-0 (cat pt-4 k "init"))
      (load lk-0 (cat pt-5 k1-0 "wrap")) (send (enc k k1-0)))
    ((recv k) (send k)))
  (label 485)
  (parent 184)
  (unrealized (3 3) (6 3) (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 old-2 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand set-wrap 2 (cur "init") (k k1) (lk lk-0))
  (defstrand make 2 (old old-2) (k k1) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (0 0)) ((3 1) (4 2)) ((3 1) (7 0))
    ((4 4) (0 0)) ((5 2) (4 0)) ((6 1) (4 3)) ((7 1) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule invShearsRule trRl_make-at-0 trRl_make-at-1)
  (operation channel-test (added-strand make 2)
    (ch-msg lk-0 (cat pt-7 k1 "init")) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-7 k1 "init")) (stor lk-0 (cat pt-4 k1 "wrap")))
    ((load lk-0 (cat pt-8 old-2)) (stor lk-0 (cat pt-7 k1 "init"))))
  (label 488)
  (parent 187)
  (realized)
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (deflistener k)
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((2 1) (0 0)) ((3 1) (4 2)) ((4 4) (0 0))
    ((5 1) (4 0)) ((6 1) (7 0)) ((7 1) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-wrap-at-1 trRl_make-at-0)
  (operation nonce-test (added-strand set-wrap 2) k (5 0)
    (ch-msg lk-1 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init")))
    ((load lk-1 (cat pt-6 k "init")) (stor lk-1 (cat pt-7 k "wrap"))))
  (label 494)
  (parent 190)
  (unrealized (4 3) (5 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (deflistener k)
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((2 1) (0 0)) ((3 1) (4 2)) ((4 4) (0 0))
    ((5 1) (4 0)) ((6 1) (7 0)) ((7 1) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-decrypt-at-1 trRl_make-at-0)
  (operation nonce-test (added-strand set-decrypt 2) k (5 0)
    (ch-msg lk-1 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init")))
    ((load lk-1 (cat pt-6 k "init"))
      (stor lk-1 (cat pt-7 k "decrypt"))))
  (label 495)
  (parent 190)
  (unrealized (4 3) (5 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 k1-0 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (deflistener k)
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk-1))
  (precedes ((1 1) (2 0)) ((2 1) (0 0)) ((3 1) (4 2)) ((4 4) (0 0))
    ((5 1) (4 0)) ((6 1) (7 2)) ((7 4) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (5 0)
    (ch-msg lk-1 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-1 (cat pt-6 k "init"))
      (load lk-1 (cat pt-7 k1-0 "wrap")) (send (enc k k1-0))))
  (label 496)
  (parent 190)
  (unrealized (4 3) (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 old-2 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand set-wrap 2 (cur "init") (k k1) (lk lk))
  (defstrand make 2 (old old-2) (k k1) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (4 2)) ((2 1) (7 0)) ((3 1) (0 0))
    ((4 4) (0 0)) ((5 2) (4 0)) ((6 1) (4 3)) ((7 1) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k1 "init")
    (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule invShearsRule trRl_make-at-0 trRl_make-at-1)
  (operation channel-test (added-strand make 2)
    (ch-msg lk (cat pt-7 k1 "init")) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "decrypt"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init"))
      (send (hash k)))
    ((load lk (cat pt-7 k1 "init")) (stor lk (cat pt-4 k1 "wrap")))
    ((load lk (cat pt-8 old-2)) (stor lk (cat pt-7 k1 "init"))))
  (label 498)
  (parent 192)
  (realized)
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((2 1) (4 2)) ((3 1) (0 0)) ((4 4) (0 0))
    ((5 1) (4 0)) ((6 1) (7 0)) ((7 1) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-wrap-at-1 trRl_make-at-0)
  (operation nonce-test (added-strand set-wrap 2) k (5 0)
    (ch-msg lk-1 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "decrypt"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init")))
    ((load lk-1 (cat pt-6 k "init")) (stor lk-1 (cat pt-7 k "wrap"))))
  (label 502)
  (parent 194)
  (unrealized (4 3) (5 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((2 1) (4 2)) ((3 1) (0 0)) ((4 4) (0 0))
    ((5 1) (4 0)) ((6 1) (7 0)) ((7 1) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-decrypt-at-1 trRl_make-at-0)
  (operation nonce-test (added-strand set-decrypt 2) k (5 0)
    (ch-msg lk-1 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "decrypt"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init")))
    ((load lk-1 (cat pt-6 k "init"))
      (stor lk-1 (cat pt-7 k "decrypt"))))
  (label 503)
  (parent 194)
  (unrealized (4 3) (5 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 k1-0 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk-1))
  (precedes ((1 1) (2 0)) ((2 1) (4 2)) ((3 1) (0 0)) ((4 4) (0 0))
    ((5 1) (4 0)) ((6 1) (7 2)) ((7 4) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k1 "wrap")
    (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (5 0)
    (ch-msg lk-1 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "decrypt"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-1 (cat pt-6 k "init"))
      (load lk-1 (cat pt-7 k1-0 "wrap")) (send (enc k k1-0))))
  (label 504)
  (parent 194)
  (unrealized (4 3) (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 old-2 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand set-wrap 2 (cur "init") (k k1) (lk lk))
  (defstrand make 2 (old old-2) (k k1) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (4 2)) ((2 1) (7 0)) ((3 1) (0 0))
    ((4 4) (0 0)) ((5 2) (4 0)) ((6 1) (4 3)) ((7 1) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule invShearsRule trRl_make-at-0 trRl_make-at-1)
  (operation channel-test (added-strand make 2)
    (ch-msg lk (cat pt-7 k1 "init")) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init"))
      (send (hash k)))
    ((load lk (cat pt-7 k1 "init")) (stor lk (cat pt-4 k1 "wrap")))
    ((load lk (cat pt-8 old-2)) (stor lk (cat pt-7 k1 "init"))))
  (label 506)
  (parent 196)
  (realized)
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((1 1) (4 2)) ((2 1) (0 0)) ((3 1) (0 0))
    ((4 4) (0 0)) ((5 1) (4 0)) ((6 1) (7 0)) ((7 1) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-wrap-at-1 trRl_make-at-0)
  (operation nonce-test (added-strand set-wrap 2) k (5 0)
    (ch-msg lk-1 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init")))
    ((load lk-1 (cat pt-6 k "init")) (stor lk-1 (cat pt-7 k "wrap"))))
  (label 509)
  (parent 198)
  (unrealized (4 3) (5 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((1 1) (4 2)) ((2 1) (0 0)) ((3 1) (0 0))
    ((4 4) (0 0)) ((5 1) (4 0)) ((6 1) (7 0)) ((7 1) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-decrypt-at-1 trRl_make-at-0)
  (operation nonce-test (added-strand set-decrypt 2) k (5 0)
    (ch-msg lk-1 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init")))
    ((load lk-1 (cat pt-6 k "init"))
      (stor lk-1 (cat pt-7 k "decrypt"))))
  (label 510)
  (parent 198)
  (unrealized (4 3) (5 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 k1-0 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk-1))
  (precedes ((1 1) (2 0)) ((1 1) (4 2)) ((2 1) (0 0)) ((3 1) (0 0))
    ((4 4) (0 0)) ((5 1) (4 0)) ((6 1) (7 2)) ((7 4) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (5 0)
    (ch-msg lk-1 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-1 (cat pt-6 k "init"))
      (load lk-1 (cat pt-7 k1-0 "wrap")) (send (enc k k1-0))))
  (label 511)
  (parent 198)
  (unrealized (4 3) (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "wrap") (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (3 2)) ((3 4) (0 0)) ((4 1) (3 0))
    ((5 1) (6 0)) ((6 1) (7 0)) ((7 1) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k "wrap")
    (cat k1 "wrap"))
  (facts (neq "wrap" "decrypt") (neq "wrap" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule fact-set-decrypt-neq0 trRl_set-wrap-at-1 gen-st-set-wrap-0
    fact-set-wrap-neq0 trRl_make-at-0)
  (operation nonce-test (added-strand set-wrap 2) k (4 0)
    (ch-msg lk-0 (cat pt-4 k "init")) (ch-msg lk-0 (cat pt-5 k "wrap")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "decrypt"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt-5 k "wrap")))
    ((load lk-0 (cat pt-5 k "wrap")) (stor lk-0 (cat pt-6 k "wrap"))))
  (label 522)
  (parent 204)
  (unrealized (3 3) (4 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((2 1) (3 2)) ((3 4) (0 0)) ((4 1) (3 0))
    ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation nonce-test (added-strand make 2) k (4 0)
    (ch-msg lk-0 (cat pt-4 k "init")) (ch-msg lk-0 (cat pt-5 k "wrap")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "decrypt"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt-5 k "wrap")))
    ((load lk-1 (cat pt-6 old-1)) (stor lk-1 (cat pt-7 k "init"))))
  (label 523)
  (parent 204)
  (unrealized (3 3) (4 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "wrap") (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (3 2)) ((3 4) (0 0)) ((4 1) (3 0))
    ((5 1) (6 0)) ((6 1) (7 0)) ((7 1) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k "wrap")
    (cat k1 "wrap"))
  (facts (neq "wrap" "decrypt") (neq "wrap" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-decrypt-at-1 fact-set-decrypt-neq0 gen-st-set-wrap-0
    fact-set-wrap-neq0 trRl_make-at-0)
  (operation nonce-test (added-strand set-decrypt 2) k (4 0)
    (ch-msg lk-0 (cat pt-4 k "init")) (ch-msg lk-0 (cat pt-5 k "wrap")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "decrypt"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt-5 k "wrap")))
    ((load lk-0 (cat pt-5 k "wrap"))
      (stor lk-0 (cat pt-6 k "decrypt"))))
  (label 524)
  (parent 204)
  (unrealized (3 3) (4 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1-0) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (3 2)) ((3 4) (0 0)) ((4 1) (3 0))
    ((5 1) (6 0)) ((6 1) (7 2)) ((7 4) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k "wrap")
    (cat k1 "wrap") (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0 gen-st-wrap-1)
  (operation nonce-test (added-strand wrap 5) k (4 0)
    (ch-msg lk-0 (cat pt-4 k "init")) (ch-msg lk-0 (cat pt-5 k "wrap")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "decrypt"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt-5 k "wrap")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-0 (cat pt-5 k "wrap"))
      (load lk-0 (cat pt-6 k1-0 "wrap")) (send (enc k k1-0))))
  (label 525)
  (parent 204)
  (unrealized (3 3) (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (3 2)) ((3 4) (0 0)) ((4 1) (3 0))
    ((5 1) (6 0)) ((5 1) (7 2)) ((6 1) (4 0)) ((7 4) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k1 "wrap")
    (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (4 0)
    (ch-msg lk-0 (cat pt-4 k "init")) (ch-msg lk-0 (cat pt-5 k "wrap")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "decrypt"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt-5 k "wrap")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-0 (cat pt-4 k "init"))
      (load lk-0 (cat pt-6 k1-0 "wrap")) (send (enc k k1-0))))
  (label 526)
  (parent 204)
  (unrealized (3 3) (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "decrypt") (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (3 2)) ((3 4) (0 0)) ((4 1) (3 0))
    ((5 1) (6 0)) ((6 1) (7 0)) ((7 1) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k1 "wrap"))
  (facts (neq "decrypt" "decrypt") (neq "decrypt" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule fact-set-decrypt-neq0 trRl_set-wrap-at-1 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation nonce-test (added-strand set-wrap 2) k (4 0)
    (ch-msg lk-0 (cat pt-4 k "init"))
    (ch-msg lk-0 (cat pt-5 k "decrypt")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "decrypt"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt-5 k "decrypt")))
    ((load lk-0 (cat pt-5 k "decrypt"))
      (stor lk-0 (cat pt-6 k "wrap"))))
  (label 527)
  (parent 205)
  (unrealized (3 3) (4 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((2 1) (3 2)) ((3 4) (0 0)) ((4 1) (3 0))
    ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation nonce-test (added-strand make 2) k (4 0)
    (ch-msg lk-0 (cat pt-4 k "init"))
    (ch-msg lk-0 (cat pt-5 k "decrypt")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "decrypt"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt-5 k "decrypt")))
    ((load lk-1 (cat pt-6 old-1)) (stor lk-1 (cat pt-7 k "init"))))
  (label 528)
  (parent 205)
  (unrealized (3 3) (4 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "decrypt") (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (3 2)) ((3 4) (0 0)) ((4 1) (3 0))
    ((5 1) (6 0)) ((6 1) (7 0)) ((7 1) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k1 "wrap"))
  (facts (neq "decrypt" "decrypt") (neq "decrypt" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-decrypt-at-1 fact-set-decrypt-neq0 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation nonce-test (added-strand set-decrypt 2) k (4 0)
    (ch-msg lk-0 (cat pt-4 k "init"))
    (ch-msg lk-0 (cat pt-5 k "decrypt")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "decrypt"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt-5 k "decrypt")))
    ((load lk-0 (cat pt-5 k "decrypt"))
      (stor lk-0 (cat pt-6 k "decrypt"))))
  (label 529)
  (parent 205)
  (unrealized (3 3) (4 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1-0) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (3 2)) ((3 4) (0 0)) ((4 1) (3 0))
    ((5 1) (6 0)) ((6 1) (7 2)) ((7 4) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k1 "wrap")
    (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (4 0)
    (ch-msg lk-0 (cat pt-4 k "init"))
    (ch-msg lk-0 (cat pt-5 k "decrypt")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "decrypt"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt-5 k "decrypt")))
    ((recv (hash k)) (recv (hash k1-0))
      (load lk-0 (cat pt-5 k "decrypt"))
      (load lk-0 (cat pt-6 k1-0 "wrap")) (send (enc k k1-0))))
  (label 530)
  (parent 205)
  (unrealized (3 3) (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (3 2)) ((3 4) (0 0)) ((4 1) (3 0))
    ((5 1) (6 0)) ((5 1) (7 2)) ((6 1) (4 0)) ((7 4) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k1 "wrap")
    (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (4 0)
    (ch-msg lk-0 (cat pt-4 k "init"))
    (ch-msg lk-0 (cat pt-5 k "decrypt")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "decrypt"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt-5 k "decrypt")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-0 (cat pt-4 k "init"))
      (load lk-0 (cat pt-6 k1-0 "wrap")) (send (enc k k1-0))))
  (label 531)
  (parent 205)
  (unrealized (3 3) (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 k1-0 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk-0))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((2 1) (3 2)) ((3 4) (0 0)) ((4 1) (3 0))
    ((5 1) (6 2)) ((6 4) (4 0)) ((7 2) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k1 "wrap")
    (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation encryption-test (added-strand make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "decrypt"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-0 (cat pt-4 k "init"))
      (load lk-0 (cat pt-5 k1-0 "wrap")) (send (enc k k1-0)))
    ((load lk-1 (cat pt-6 old-1)) (stor lk-1 (cat pt-7 k "init"))
      (send (hash k))))
  (label 534)
  (parent 206)
  (unrealized (3 3) (6 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk-0))
  (deflistener k)
  (precedes ((1 1) (2 0)) ((2 1) (3 2)) ((3 4) (0 0)) ((4 1) (3 0))
    ((5 1) (6 2)) ((6 4) (4 0)) ((7 1) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k1 "wrap")
    (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (added-listener k) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "decrypt"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-0 (cat pt-4 k "init"))
      (load lk-0 (cat pt-5 k1-0 "wrap")) (send (enc k k1-0)))
    ((recv k) (send k)))
  (label 535)
  (parent 206)
  (unrealized (3 3) (6 3) (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "wrap") (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((1 1) (3 2)) ((2 1) (0 0)) ((3 4) (0 0))
    ((4 1) (3 0)) ((5 1) (6 0)) ((6 1) (7 0)) ((7 1) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "wrap" "decrypt") (neq "wrap" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule fact-set-decrypt-neq0 trRl_set-wrap-at-1 gen-st-set-wrap-0
    fact-set-wrap-neq0 trRl_make-at-0)
  (operation nonce-test (added-strand set-wrap 2) k (4 0)
    (ch-msg lk-0 (cat pt-4 k "init")) (ch-msg lk-0 (cat pt-5 k "wrap")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt-5 k "wrap")))
    ((load lk-0 (cat pt-5 k "wrap")) (stor lk-0 (cat pt-6 k "wrap"))))
  (label 543)
  (parent 211)
  (unrealized (3 3) (4 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((1 1) (3 2)) ((2 1) (0 0)) ((3 4) (0 0))
    ((4 1) (3 0)) ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation nonce-test (added-strand make 2) k (4 0)
    (ch-msg lk-0 (cat pt-4 k "init")) (ch-msg lk-0 (cat pt-5 k "wrap")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt-5 k "wrap")))
    ((load lk-1 (cat pt-6 old-1)) (stor lk-1 (cat pt-7 k "init"))))
  (label 544)
  (parent 211)
  (unrealized (3 3) (4 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "wrap") (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((1 1) (3 2)) ((2 1) (0 0)) ((3 4) (0 0))
    ((4 1) (3 0)) ((5 1) (6 0)) ((6 1) (7 0)) ((7 1) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "wrap" "decrypt") (neq "wrap" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-decrypt-at-1 fact-set-decrypt-neq0 gen-st-set-wrap-0
    fact-set-wrap-neq0 trRl_make-at-0)
  (operation nonce-test (added-strand set-decrypt 2) k (4 0)
    (ch-msg lk-0 (cat pt-4 k "init")) (ch-msg lk-0 (cat pt-5 k "wrap")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt-5 k "wrap")))
    ((load lk-0 (cat pt-5 k "wrap"))
      (stor lk-0 (cat pt-6 k "decrypt"))))
  (label 545)
  (parent 211)
  (unrealized (3 3) (4 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1-0) (lk lk-0))
  (precedes ((1 1) (2 0)) ((1 1) (3 2)) ((2 1) (0 0)) ((3 4) (0 0))
    ((4 1) (3 0)) ((5 1) (6 0)) ((6 1) (7 2)) ((7 4) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap")
    (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0 gen-st-wrap-1)
  (operation nonce-test (added-strand wrap 5) k (4 0)
    (ch-msg lk-0 (cat pt-4 k "init")) (ch-msg lk-0 (cat pt-5 k "wrap")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt-5 k "wrap")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-0 (cat pt-5 k "wrap"))
      (load lk-0 (cat pt-6 k1-0 "wrap")) (send (enc k k1-0))))
  (label 546)
  (parent 211)
  (unrealized (3 3) (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk-0))
  (precedes ((1 1) (2 0)) ((1 1) (3 2)) ((2 1) (0 0)) ((3 4) (0 0))
    ((4 1) (3 0)) ((5 1) (6 0)) ((5 1) (7 2)) ((6 1) (4 0))
    ((7 4) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (4 0)
    (ch-msg lk-0 (cat pt-4 k "init")) (ch-msg lk-0 (cat pt-5 k "wrap")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt-5 k "wrap")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-0 (cat pt-4 k "init"))
      (load lk-0 (cat pt-6 k1-0 "wrap")) (send (enc k k1-0))))
  (label 547)
  (parent 211)
  (unrealized (3 3) (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "decrypt") (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((1 1) (3 2)) ((2 1) (0 0)) ((3 4) (0 0))
    ((4 1) (3 0)) ((5 1) (6 0)) ((6 1) (7 0)) ((7 1) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k1 "wrap"))
  (facts (neq "decrypt" "decrypt") (neq "decrypt" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule fact-set-decrypt-neq0 trRl_set-wrap-at-1 gen-st-set-wrap-0
    fact-set-wrap-neq0 trRl_make-at-0)
  (operation nonce-test (added-strand set-wrap 2) k (4 0)
    (ch-msg lk-0 (cat pt-4 k "init"))
    (ch-msg lk-0 (cat pt-5 k "decrypt")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt-5 k "decrypt")))
    ((load lk-0 (cat pt-5 k "decrypt"))
      (stor lk-0 (cat pt-6 k "wrap"))))
  (label 548)
  (parent 212)
  (unrealized (3 3) (4 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((1 1) (3 2)) ((2 1) (0 0)) ((3 4) (0 0))
    ((4 1) (3 0)) ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation nonce-test (added-strand make 2) k (4 0)
    (ch-msg lk-0 (cat pt-4 k "init"))
    (ch-msg lk-0 (cat pt-5 k "decrypt")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt-5 k "decrypt")))
    ((load lk-1 (cat pt-6 old-1)) (stor lk-1 (cat pt-7 k "init"))))
  (label 549)
  (parent 212)
  (unrealized (3 3) (4 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "decrypt") (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((1 1) (3 2)) ((2 1) (0 0)) ((3 4) (0 0))
    ((4 1) (3 0)) ((5 1) (6 0)) ((6 1) (7 0)) ((7 1) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k1 "wrap"))
  (facts (neq "decrypt" "decrypt") (neq "decrypt" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-decrypt-at-1 fact-set-decrypt-neq0 gen-st-set-wrap-0
    fact-set-wrap-neq0 trRl_make-at-0)
  (operation nonce-test (added-strand set-decrypt 2) k (4 0)
    (ch-msg lk-0 (cat pt-4 k "init"))
    (ch-msg lk-0 (cat pt-5 k "decrypt")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt-5 k "decrypt")))
    ((load lk-0 (cat pt-5 k "decrypt"))
      (stor lk-0 (cat pt-6 k "decrypt"))))
  (label 550)
  (parent 212)
  (unrealized (3 3) (4 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1-0) (lk lk-0))
  (precedes ((1 1) (2 0)) ((1 1) (3 2)) ((2 1) (0 0)) ((3 4) (0 0))
    ((4 1) (3 0)) ((5 1) (6 0)) ((6 1) (7 2)) ((7 4) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k1 "wrap")
    (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0 gen-st-wrap-1)
  (operation nonce-test (added-strand wrap 5) k (4 0)
    (ch-msg lk-0 (cat pt-4 k "init"))
    (ch-msg lk-0 (cat pt-5 k "decrypt")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt-5 k "decrypt")))
    ((recv (hash k)) (recv (hash k1-0))
      (load lk-0 (cat pt-5 k "decrypt"))
      (load lk-0 (cat pt-6 k1-0 "wrap")) (send (enc k k1-0))))
  (label 551)
  (parent 212)
  (unrealized (3 3) (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk-0))
  (precedes ((1 1) (2 0)) ((1 1) (3 2)) ((2 1) (0 0)) ((3 4) (0 0))
    ((4 1) (3 0)) ((5 1) (6 0)) ((5 1) (7 2)) ((6 1) (4 0))
    ((7 4) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (4 0)
    (ch-msg lk-0 (cat pt-4 k "init"))
    (ch-msg lk-0 (cat pt-5 k "decrypt")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt-5 k "decrypt")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-0 (cat pt-4 k "init"))
      (load lk-0 (cat pt-6 k1-0 "wrap")) (send (enc k k1-0))))
  (label 552)
  (parent 212)
  (unrealized (3 3) (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 k1-0 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk-0))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((1 1) (3 2)) ((2 1) (0 0)) ((3 4) (0 0))
    ((4 1) (3 0)) ((5 1) (6 2)) ((6 4) (4 0)) ((7 2) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation encryption-test (added-strand make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-0 (cat pt-4 k "init"))
      (load lk-0 (cat pt-5 k1-0 "wrap")) (send (enc k k1-0)))
    ((load lk-1 (cat pt-6 old-1)) (stor lk-1 (cat pt-7 k "init"))
      (send (hash k))))
  (label 555)
  (parent 213)
  (unrealized (3 3) (6 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk-0))
  (deflistener k)
  (precedes ((1 1) (2 0)) ((1 1) (3 2)) ((2 1) (0 0)) ((3 4) (0 0))
    ((4 1) (3 0)) ((5 1) (6 2)) ((6 4) (4 0)) ((7 1) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (added-listener k) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-0 (cat pt-4 k "init"))
      (load lk-0 (cat pt-5 k1-0 "wrap")) (send (enc k k1-0)))
    ((recv k) (send k)))
  (label 556)
  (parent 213)
  (unrealized (3 3) (6 3) (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (defstrand set-decrypt 2 (cur "wrap") (k k) (lk lk-0))
  (precedes ((1 1) (2 2)) ((2 4) (0 0)) ((3 1) (2 0)) ((4 1) (5 0))
    ((5 1) (7 0)) ((6 1) (3 0)) ((7 1) (3 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "wrap" "decrypt") (neq "wrap" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-decrypt-at-1 fact-set-decrypt-neq0 gen-st-set-wrap-0
    fact-set-wrap-neq0 trRl_make-at-0)
  (operation nonce-test (added-strand set-decrypt 2) k (3 0)
    (ch-msg lk-0 (cat pt-3 k "init")) (ch-msg lk-0 (cat pt-4 k "wrap"))
    (ch-msg lk-1 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "wrap")))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init")))
    ((load lk-0 (cat pt-4 k "wrap"))
      (stor lk-0 (cat pt-7 k "decrypt"))))
  (label 577)
  (parent 222)
  (unrealized (2 3) (3 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-1))
  (precedes ((1 1) (2 2)) ((2 4) (0 0)) ((3 1) (2 0)) ((4 1) (5 0))
    ((5 1) (3 0)) ((6 1) (7 0)) ((7 1) (3 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-decrypt-at-1 trRl_make-at-0)
  (operation nonce-test (added-strand set-decrypt 2) k (3 0)
    (ch-msg lk-0 (cat pt-3 k "init")) (ch-msg lk-0 (cat pt-4 k "wrap"))
    (ch-msg lk-1 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "wrap")))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init")))
    ((load lk-1 (cat pt-6 k "init"))
      (stor lk-1 (cat pt-7 k "decrypt"))))
  (label 578)
  (parent 222)
  (unrealized (2 3) (3 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (defstrand set-wrap 2 (cur "wrap") (k k) (lk lk-0))
  (precedes ((1 1) (2 2)) ((2 4) (0 0)) ((3 1) (2 0)) ((4 1) (5 0))
    ((5 1) (7 0)) ((6 1) (3 0)) ((7 1) (3 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "wrap" "decrypt") (neq "wrap" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule fact-set-decrypt-neq0 trRl_set-wrap-at-1 gen-st-set-wrap-0
    fact-set-wrap-neq0 trRl_make-at-0)
  (operation nonce-test (added-strand set-wrap 2) k (3 0)
    (ch-msg lk-0 (cat pt-3 k "init")) (ch-msg lk-0 (cat pt-4 k "wrap"))
    (ch-msg lk-1 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "wrap")))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init")))
    ((load lk-0 (cat pt-4 k "wrap")) (stor lk-0 (cat pt-7 k "wrap"))))
  (label 579)
  (parent 222)
  (unrealized (2 3) (3 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 k1-0 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk-1))
  (precedes ((1 1) (2 2)) ((2 4) (0 0)) ((3 1) (2 0)) ((4 1) (5 0))
    ((5 1) (3 0)) ((6 1) (7 2)) ((7 4) (3 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (3 0)
    (ch-msg lk-0 (cat pt-3 k "init")) (ch-msg lk-0 (cat pt-4 k "wrap"))
    (ch-msg lk-1 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "wrap")))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-1 (cat pt-6 k "init"))
      (load lk-1 (cat pt-7 k1-0 "wrap")) (send (enc k k1-0))))
  (label 580)
  (parent 222)
  (unrealized (2 3) (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 k1-0 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1-0) (lk lk-0))
  (precedes ((1 1) (2 2)) ((2 4) (0 0)) ((3 1) (2 0)) ((4 1) (5 0))
    ((5 1) (7 2)) ((6 1) (3 0)) ((7 4) (3 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap")
    (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0 gen-st-wrap-1)
  (operation nonce-test (added-strand wrap 5) k (3 0)
    (ch-msg lk-0 (cat pt-3 k "init")) (ch-msg lk-0 (cat pt-4 k "wrap"))
    (ch-msg lk-1 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "wrap")))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-0 (cat pt-4 k "wrap"))
      (load lk-0 (cat pt-7 k1-0 "wrap")) (send (enc k k1-0))))
  (label 581)
  (parent 222)
  (unrealized (2 3) (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 k1-0 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk-0))
  (precedes ((1 1) (2 2)) ((2 4) (0 0)) ((3 1) (2 0)) ((4 1) (5 0))
    ((4 1) (7 2)) ((5 1) (3 0)) ((6 1) (3 0)) ((7 4) (3 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (3 0)
    (ch-msg lk-0 (cat pt-3 k "init")) (ch-msg lk-0 (cat pt-4 k "wrap"))
    (ch-msg lk-1 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "wrap")))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-7 k1-0 "wrap")) (send (enc k k1-0))))
  (label 582)
  (parent 222)
  (unrealized (2 3) (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 k1-0 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1-0) (lk lk-0))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (2 2)) ((2 4) (0 0)) ((3 1) (2 0)) ((4 1) (5 0))
    ((5 1) (6 2)) ((6 4) (3 0)) ((7 2) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap")
    (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation encryption-test (added-strand make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "wrap")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-0 (cat pt-4 k "wrap"))
      (load lk-0 (cat pt-5 k1-0 "wrap")) (send (enc k k1-0)))
    ((load lk-1 (cat pt-6 old-1)) (stor lk-1 (cat pt-7 k "init"))
      (send (hash k))))
  (label 585)
  (parent 224)
  (unrealized (2 3) (6 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1-0) (lk lk-0))
  (deflistener k)
  (precedes ((1 1) (2 2)) ((2 4) (0 0)) ((3 1) (2 0)) ((4 1) (5 0))
    ((5 1) (6 2)) ((6 4) (3 0)) ((7 1) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap")
    (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (added-listener k) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "wrap")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-0 (cat pt-4 k "wrap"))
      (load lk-0 (cat pt-5 k1-0 "wrap")) (send (enc k k1-0)))
    ((recv k) (send k)))
  (label 586)
  (parent 224)
  (unrealized (2 3) (6 3) (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 k1-0 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk-0))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (2 2)) ((2 4) (0 0)) ((3 1) (2 0)) ((4 1) (5 0))
    ((4 1) (6 2)) ((5 1) (3 0)) ((6 4) (3 0)) ((7 2) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation encryption-test (added-strand make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "wrap")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-5 k1-0 "wrap")) (send (enc k k1-0)))
    ((load lk-1 (cat pt-6 old-1)) (stor lk-1 (cat pt-7 k "init"))
      (send (hash k))))
  (label 589)
  (parent 225)
  (unrealized (2 3) (6 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk-0))
  (deflistener k)
  (precedes ((1 1) (2 2)) ((2 4) (0 0)) ((3 1) (2 0)) ((4 1) (5 0))
    ((4 1) (6 2)) ((5 1) (3 0)) ((6 4) (3 0)) ((7 1) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (added-listener k) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "wrap")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-5 k1-0 "wrap")) (send (enc k k1-0)))
    ((recv k) (send k)))
  (label 590)
  (parent 225)
  (unrealized (2 3) (6 3) (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (defstrand set-wrap 2 (cur "decrypt") (k k) (lk lk-0))
  (precedes ((1 1) (2 2)) ((2 4) (0 0)) ((3 1) (2 0)) ((4 1) (5 0))
    ((5 1) (7 0)) ((6 1) (3 0)) ((7 1) (3 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k1 "wrap"))
  (facts (neq "decrypt" "decrypt") (neq "decrypt" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule fact-set-decrypt-neq0 trRl_set-wrap-at-1 gen-st-set-wrap-0
    fact-set-wrap-neq0 trRl_make-at-0)
  (operation nonce-test (added-strand set-wrap 2) k (3 0)
    (ch-msg lk-0 (cat pt-3 k "init"))
    (ch-msg lk-0 (cat pt-4 k "decrypt"))
    (ch-msg lk-1 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init")))
    ((load lk-0 (cat pt-4 k "decrypt"))
      (stor lk-0 (cat pt-7 k "wrap"))))
  (label 591)
  (parent 227)
  (unrealized (2 3) (3 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 k1-0 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk-1))
  (precedes ((1 1) (2 2)) ((2 4) (0 0)) ((3 1) (2 0)) ((4 1) (5 0))
    ((5 1) (3 0)) ((6 1) (7 2)) ((7 4) (3 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (3 0)
    (ch-msg lk-0 (cat pt-3 k "init"))
    (ch-msg lk-0 (cat pt-4 k "decrypt"))
    (ch-msg lk-1 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-1 (cat pt-6 k "init"))
      (load lk-1 (cat pt-7 k1-0 "wrap")) (send (enc k k1-0))))
  (label 592)
  (parent 227)
  (unrealized (2 3) (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (defstrand set-decrypt 2 (cur "decrypt") (k k) (lk lk-0))
  (precedes ((1 1) (2 2)) ((2 4) (0 0)) ((3 1) (2 0)) ((4 1) (5 0))
    ((5 1) (7 0)) ((6 1) (3 0)) ((7 1) (3 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k1 "wrap"))
  (facts (neq "decrypt" "decrypt") (neq "decrypt" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-decrypt-at-1 fact-set-decrypt-neq0 gen-st-set-wrap-0
    fact-set-wrap-neq0 trRl_make-at-0)
  (operation nonce-test (added-strand set-decrypt 2) k (3 0)
    (ch-msg lk-0 (cat pt-3 k "init"))
    (ch-msg lk-0 (cat pt-4 k "decrypt"))
    (ch-msg lk-1 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init")))
    ((load lk-0 (cat pt-4 k "decrypt"))
      (stor lk-0 (cat pt-7 k "decrypt"))))
  (label 593)
  (parent 227)
  (unrealized (2 3) (3 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 k1-0 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1-0) (lk lk-0))
  (precedes ((1 1) (2 2)) ((2 4) (0 0)) ((3 1) (2 0)) ((4 1) (5 0))
    ((5 1) (7 2)) ((6 1) (3 0)) ((7 4) (3 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k1 "wrap")
    (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0 gen-st-wrap-1)
  (operation nonce-test (added-strand wrap 5) k (3 0)
    (ch-msg lk-0 (cat pt-3 k "init"))
    (ch-msg lk-0 (cat pt-4 k "decrypt"))
    (ch-msg lk-1 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init")))
    ((recv (hash k)) (recv (hash k1-0))
      (load lk-0 (cat pt-4 k "decrypt"))
      (load lk-0 (cat pt-7 k1-0 "wrap")) (send (enc k k1-0))))
  (label 594)
  (parent 227)
  (unrealized (2 3) (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 k1-0 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk-0))
  (precedes ((1 1) (2 2)) ((2 4) (0 0)) ((3 1) (2 0)) ((4 1) (5 0))
    ((4 1) (7 2)) ((5 1) (3 0)) ((6 1) (3 0)) ((7 4) (3 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (3 0)
    (ch-msg lk-0 (cat pt-3 k "init"))
    (ch-msg lk-0 (cat pt-4 k "decrypt"))
    (ch-msg lk-1 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-7 k1-0 "wrap")) (send (enc k k1-0))))
  (label 595)
  (parent 227)
  (unrealized (2 3) (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 k1-0 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1-0) (lk lk-0))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (2 2)) ((2 4) (0 0)) ((3 1) (2 0)) ((4 1) (5 0))
    ((5 1) (6 2)) ((6 4) (3 0)) ((7 2) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k1 "wrap")
    (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation encryption-test (added-strand make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k1-0))
      (load lk-0 (cat pt-4 k "decrypt"))
      (load lk-0 (cat pt-5 k1-0 "wrap")) (send (enc k k1-0)))
    ((load lk-1 (cat pt-6 old-1)) (stor lk-1 (cat pt-7 k "init"))
      (send (hash k))))
  (label 598)
  (parent 229)
  (unrealized (2 3) (6 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1-0) (lk lk-0))
  (deflistener k)
  (precedes ((1 1) (2 2)) ((2 4) (0 0)) ((3 1) (2 0)) ((4 1) (5 0))
    ((5 1) (6 2)) ((6 4) (3 0)) ((7 1) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k1 "wrap")
    (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (added-listener k) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k1-0))
      (load lk-0 (cat pt-4 k "decrypt"))
      (load lk-0 (cat pt-5 k1-0 "wrap")) (send (enc k k1-0)))
    ((recv k) (send k)))
  (label 599)
  (parent 229)
  (unrealized (2 3) (6 3) (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 k1-0 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk-0))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (2 2)) ((2 4) (0 0)) ((3 1) (2 0)) ((4 1) (5 0))
    ((4 1) (6 2)) ((5 1) (3 0)) ((6 4) (3 0)) ((7 2) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation encryption-test (added-strand make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-5 k1-0 "wrap")) (send (enc k k1-0)))
    ((load lk-1 (cat pt-6 old-1)) (stor lk-1 (cat pt-7 k "init"))
      (send (hash k))))
  (label 602)
  (parent 230)
  (unrealized (2 3) (6 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk-0))
  (deflistener k)
  (precedes ((1 1) (2 2)) ((2 4) (0 0)) ((3 1) (2 0)) ((4 1) (5 0))
    ((4 1) (6 2)) ((5 1) (3 0)) ((6 4) (3 0)) ((7 1) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (added-listener k) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-5 k1-0 "wrap")) (send (enc k k1-0)))
    ((recv k) (send k)))
  (label 603)
  (parent 230)
  (unrealized (2 3) (6 3) (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 cur mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 k1-0 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk-0))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand set-wrap 2 (cur cur) (k k1-0) (lk lk-0))
  (precedes ((1 1) (2 2)) ((2 4) (0 0)) ((3 1) (2 0)) ((4 1) (5 2))
    ((4 1) (7 0)) ((5 4) (3 0)) ((6 2) (5 0)) ((7 1) (5 3)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 cur)
    (cat k1-0 "wrap"))
  (facts (neq cur "decrypt") (neq cur "wrap"))
  (rule invShearsRule fact-set-decrypt-neq0 trRl_set-wrap-at-1
    gen-st-set-wrap-0 fact-set-wrap-neq0 trRl_make-at-0)
  (operation channel-test (added-strand set-wrap 2)
    (ch-msg lk-0 (cat pt-4 k1-0 "wrap")) (5 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-4 k1-0 "wrap")) (send (enc k k1-0)))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-7 k1-0 cur))
      (stor lk-0 (cat pt-4 k1-0 "wrap"))))
  (label 606)
  (parent 233)
  (unrealized (2 3) (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 k1-0 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk-0))
  (deflistener k)
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (2 2)) ((2 4) (0 0)) ((3 1) (2 0)) ((4 1) (5 2))
    ((5 4) (3 0)) ((6 1) (5 0)) ((7 1) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation nonce-test (added-strand make 2) k (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-4 k1-0 "wrap")) (send (enc k k1-0)))
    ((recv k) (send k))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init"))))
  (label 609)
  (parent 234)
  (unrealized (2 3) (5 3) (6 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 cur mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-1))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1) (lk lk-1))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand set-wrap 2 (cur cur) (k k1) (lk lk-1))
  (precedes ((1 1) (2 0)) ((2 1) (0 0)) ((3 1) (5 2)) ((3 1) (7 0))
    ((4 1) (0 0)) ((5 4) (0 0)) ((6 1) (3 0)) ((6 2) (5 0))
    ((7 1) (5 3)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k1 cur) (cat k1 "wrap"))
  (facts (neq cur "decrypt") (neq cur "wrap") (neq "init" "decrypt")
    (neq "init" "wrap"))
  (rule invShearsRule fact-set-decrypt-neq0 trRl_set-wrap-at-1
    gen-st-set-wrap-0 fact-set-wrap-neq0 trRl_make-at-0)
  (operation channel-test (added-strand set-wrap 2)
    (ch-msg lk-1 (cat pt-5 k1 "wrap")) (5 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-1 (cat pt-7 k "init")) (stor lk-1 (cat pt-2 k "decrypt")))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk-1 (cat pt-2 k "decrypt"))
      (load lk-1 (cat pt-5 k1 "wrap")) (send (enc k k1)))
    ((load lk-1 (cat pt-6 old-1)) (stor lk-1 (cat pt-7 k "init"))
      (send (hash k)))
    ((load lk-1 (cat pt-8 k1 cur)) (stor lk-1 (cat pt-5 k1 "wrap"))))
  (label 613)
  (parent 235)
  (unrealized (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 cur mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-1))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand set-wrap 2 (cur cur) (k k1) (lk lk))
  (precedes ((1 1) (0 0)) ((2 1) (3 0)) ((3 1) (5 2)) ((3 1) (7 0))
    ((4 1) (0 0)) ((5 4) (0 0)) ((6 1) (1 0)) ((6 2) (5 0))
    ((7 1) (5 3)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k1 cur) (cat k1 "wrap"))
  (facts (neq cur "decrypt") (neq cur "wrap") (neq "init" "decrypt")
    (neq "init" "wrap"))
  (rule invShearsRule fact-set-decrypt-neq0 trRl_set-wrap-at-1
    gen-st-set-wrap-0 fact-set-wrap-neq0 trRl_make-at-0)
  (operation channel-test (added-strand set-wrap 2)
    (ch-msg lk (cat pt-5 k1 "wrap")) (5 3))
  (traces ((recv k) (send k))
    ((load lk-1 (cat pt-7 k "init")) (stor lk-1 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-2 k "decrypt"))
      (load lk (cat pt-5 k1 "wrap")) (send (enc k k1)))
    ((load lk-1 (cat pt-6 old-1)) (stor lk-1 (cat pt-7 k "init"))
      (send (hash k)))
    ((load lk (cat pt-8 k1 cur)) (stor lk (cat pt-5 k1 "wrap"))))
  (label 616)
  (parent 236)
  (unrealized (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 cur mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-1))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand set-wrap 2 (cur cur) (k k1) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (0 0)) ((3 1) (0 0)) ((4 1) (5 2))
    ((4 1) (7 0)) ((5 4) (0 0)) ((6 1) (3 0)) ((6 2) (5 0))
    ((7 1) (5 3)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 cur) (cat k1 "wrap"))
  (facts (neq cur "decrypt") (neq cur "wrap") (neq "init" "decrypt")
    (neq "init" "wrap"))
  (rule invShearsRule fact-set-decrypt-neq0 trRl_set-wrap-at-1
    gen-st-set-wrap-0 fact-set-wrap-neq0 trRl_make-at-0)
  (operation channel-test (added-strand set-wrap 2)
    (ch-msg lk-0 (cat pt-5 k1 "wrap")) (5 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-1 (cat pt-7 k "init")) (stor lk-1 (cat pt-2 k "decrypt")))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-4 k "init"))
      (load lk-0 (cat pt-5 k1 "wrap")) (send (enc k k1)))
    ((load lk-1 (cat pt-6 old-1)) (stor lk-1 (cat pt-7 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-8 k1 cur)) (stor lk-0 (cat pt-5 k1 "wrap"))))
  (label 617)
  (parent 239)
  (unrealized (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 cur mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-1))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand set-wrap 2 (cur cur) (k k1) (lk lk-0))
  (precedes ((1 1) (0 0)) ((2 1) (3 0)) ((3 1) (0 0)) ((4 1) (5 2))
    ((4 1) (7 0)) ((5 4) (0 0)) ((6 1) (1 0)) ((6 2) (5 0))
    ((7 1) (5 3)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 cur) (cat k1 "wrap"))
  (facts (neq cur "decrypt") (neq cur "wrap") (neq "init" "decrypt")
    (neq "init" "wrap"))
  (rule invShearsRule fact-set-decrypt-neq0 trRl_set-wrap-at-1
    gen-st-set-wrap-0 fact-set-wrap-neq0 trRl_make-at-0)
  (operation channel-test (added-strand set-wrap 2)
    (ch-msg lk-0 (cat pt-5 k1 "wrap")) (5 3))
  (traces ((recv k) (send k))
    ((load lk-1 (cat pt-7 k "init")) (stor lk-1 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-4 k "init"))
      (load lk-0 (cat pt-5 k1 "wrap")) (send (enc k k1)))
    ((load lk-1 (cat pt-6 old-1)) (stor lk-1 (cat pt-7 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-8 k1 cur)) (stor lk-0 (cat pt-5 k1 "wrap"))))
  (label 618)
  (parent 240)
  (unrealized (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 cur mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-1))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand set-wrap 2 (cur cur) (k k1) (lk lk-1))
  (precedes ((1 1) (2 0)) ((2 1) (0 0)) ((3 1) (4 0)) ((4 1) (0 0))
    ((5 4) (0 0)) ((6 1) (7 0)) ((6 2) (5 0)) ((7 1) (5 3)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 cur) (cat k1 "wrap"))
  (facts (neq cur "decrypt") (neq cur "wrap") (neq "init" "decrypt")
    (neq "init" "wrap"))
  (rule invShearsRule fact-set-decrypt-neq0 trRl_set-wrap-at-1
    gen-st-set-wrap-0 fact-set-wrap-neq0 trRl_make-at-0)
  (operation channel-test (added-strand set-wrap 2)
    (ch-msg lk-1 (cat pt-5 k1 "wrap")) (5 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk-1 (cat pt-7 k "init"))
      (load lk-1 (cat pt-5 k1 "wrap")) (send (enc k k1)))
    ((load lk-1 (cat pt-6 old-1)) (stor lk-1 (cat pt-7 k "init"))
      (send (hash k)))
    ((load lk-1 (cat pt-8 k1 cur)) (stor lk-1 (cat pt-5 k1 "wrap"))))
  (label 620)
  (parent 241)
  (unrealized (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 cur mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-1))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-1))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand set-wrap 2 (cur cur) (k k1) (lk lk-1))
  (precedes ((1 1) (2 0)) ((2 1) (0 0)) ((3 1) (7 0)) ((4 1) (0 0))
    ((5 4) (0 0)) ((6 1) (3 0)) ((6 2) (5 0)) ((7 1) (5 3)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 cur) (cat k1 "wrap"))
  (facts (neq cur "decrypt") (neq cur "wrap") (neq "init" "decrypt")
    (neq "init" "wrap"))
  (rule shearsRule invShearsRule fact-set-decrypt-neq0
    trRl_set-wrap-at-1 gen-st-set-wrap-0 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation channel-test (added-strand set-wrap 2)
    (ch-msg lk-1 (cat pt-5 k1 "wrap")) (5 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-1 (cat pt-7 k "init")) (stor lk-1 (cat pt-2 k "decrypt")))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk-1 (cat pt-7 k "init"))
      (load lk-1 (cat pt-5 k1 "wrap")) (send (enc k k1)))
    ((load lk-1 (cat pt-6 old-1)) (stor lk-1 (cat pt-7 k "init"))
      (send (hash k)))
    ((load lk-1 (cat pt-8 k1 cur)) (stor lk-1 (cat pt-5 k1 "wrap"))))
  (label 623)
  (parent 244)
  (unrealized (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 cur mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-1))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand set-wrap 2 (cur cur) (k k1) (lk lk))
  (precedes ((1 1) (0 0)) ((2 1) (3 0)) ((2 1) (5 2)) ((3 1) (7 0))
    ((4 1) (0 0)) ((5 4) (0 0)) ((6 1) (1 0)) ((6 2) (5 0))
    ((7 1) (5 3)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 cur) (cat k1 "wrap"))
  (facts (neq cur "decrypt") (neq cur "wrap") (neq "init" "decrypt")
    (neq "init" "wrap"))
  (rule shearsRule invShearsRule fact-set-decrypt-neq0
    trRl_set-wrap-at-1 gen-st-set-wrap-0 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation channel-test (added-strand set-wrap 2)
    (ch-msg lk (cat pt-5 k1 "wrap")) (5 3))
  (traces ((recv k) (send k))
    ((load lk-1 (cat pt-7 k "init")) (stor lk-1 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt-5 k1 "wrap")) (send (enc k k1)))
    ((load lk-1 (cat pt-6 old-1)) (stor lk-1 (cat pt-7 k "init"))
      (send (hash k)))
    ((load lk (cat pt-8 k1 cur)) (stor lk (cat pt-5 k1 "wrap"))))
  (label 624)
  (parent 245)
  (unrealized (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 cur mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-1))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand set-wrap 2 (cur cur) (k k1) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (5 2)) ((2 1) (7 0)) ((3 1) (0 0))
    ((4 1) (0 0)) ((5 4) (0 0)) ((6 1) (3 0)) ((6 2) (5 0))
    ((7 1) (5 3)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 cur) (cat k1 "wrap"))
  (facts (neq cur "decrypt") (neq cur "wrap") (neq "init" "decrypt")
    (neq "init" "wrap"))
  (rule invShearsRule fact-set-decrypt-neq0 trRl_set-wrap-at-1
    gen-st-set-wrap-0 fact-set-wrap-neq0 trRl_make-at-0)
  (operation channel-test (added-strand set-wrap 2)
    (ch-msg lk (cat pt-5 k1 "wrap")) (5 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-1 (cat pt-7 k "init")) (stor lk-1 (cat pt-2 k "decrypt")))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-5 k1 "wrap")) (send (enc k k1)))
    ((load lk-1 (cat pt-6 old-1)) (stor lk-1 (cat pt-7 k "init"))
      (send (hash k)))
    ((load lk (cat pt-8 k1 cur)) (stor lk (cat pt-5 k1 "wrap"))))
  (label 625)
  (parent 248)
  (unrealized (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 cur mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-1))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk-1))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand set-wrap 2 (cur cur) (k k1) (lk lk-1))
  (precedes ((1 1) (5 2)) ((1 1) (7 0)) ((2 1) (3 0)) ((3 1) (0 0))
    ((4 1) (0 0)) ((5 4) (0 0)) ((6 1) (1 0)) ((6 2) (5 0))
    ((7 1) (5 3)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 cur) (cat k1 "wrap"))
  (facts (neq cur "decrypt") (neq cur "wrap") (neq "init" "decrypt")
    (neq "init" "wrap"))
  (rule invShearsRule fact-set-decrypt-neq0 trRl_set-wrap-at-1
    gen-st-set-wrap-0 fact-set-wrap-neq0 trRl_make-at-0)
  (operation channel-test (added-strand set-wrap 2)
    (ch-msg lk-1 (cat pt-5 k1 "wrap")) (5 3))
  (traces ((recv k) (send k))
    ((load lk-1 (cat pt-7 k "init")) (stor lk-1 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk-1 (cat pt k "wrap"))
      (load lk-1 (cat pt-5 k1 "wrap")) (send (enc k k1)))
    ((load lk-1 (cat pt-6 old-1)) (stor lk-1 (cat pt-7 k "init"))
      (send (hash k)))
    ((load lk-1 (cat pt-8 k1 cur)) (stor lk-1 (cat pt-5 k1 "wrap"))))
  (label 627)
  (parent 249)
  (unrealized (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 cur mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-1))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand set-wrap 2 (cur cur) (k k1) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (5 2)) ((2 1) (7 0)) ((3 1) (0 0))
    ((4 1) (0 0)) ((5 4) (0 0)) ((6 1) (3 0)) ((6 2) (5 0))
    ((7 1) (5 3)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 cur) (cat k1 "wrap"))
  (facts (neq cur "decrypt") (neq cur "wrap") (neq "init" "decrypt")
    (neq "init" "wrap"))
  (rule shearsRule invShearsRule fact-set-decrypt-neq0
    trRl_set-wrap-at-1 gen-st-set-wrap-0 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation channel-test (added-strand set-wrap 2)
    (ch-msg lk (cat pt-5 k1 "wrap")) (5 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-1 (cat pt-7 k "init")) (stor lk-1 (cat pt-2 k "decrypt")))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-5 k1 "wrap")) (send (enc k k1)))
    ((load lk-1 (cat pt-6 old-1)) (stor lk-1 (cat pt-7 k "init"))
      (send (hash k)))
    ((load lk (cat pt-8 k1 cur)) (stor lk (cat pt-5 k1 "wrap"))))
  (label 628)
  (parent 252)
  (unrealized (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 cur mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-1))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-1))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand set-wrap 2 (cur cur) (k k1) (lk lk-1))
  (precedes ((1 1) (7 0)) ((2 1) (3 0)) ((3 1) (0 0)) ((4 1) (0 0))
    ((5 4) (0 0)) ((6 1) (1 0)) ((6 2) (5 0)) ((7 1) (5 3)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 cur) (cat k1 "wrap"))
  (facts (neq cur "decrypt") (neq cur "wrap") (neq "init" "decrypt")
    (neq "init" "wrap"))
  (rule shearsRule invShearsRule fact-set-decrypt-neq0
    trRl_set-wrap-at-1 gen-st-set-wrap-0 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation channel-test (added-strand set-wrap 2)
    (ch-msg lk-1 (cat pt-5 k1 "wrap")) (5 3))
  (traces ((recv k) (send k))
    ((load lk-1 (cat pt-7 k "init")) (stor lk-1 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk-1 (cat pt-7 k "init"))
      (load lk-1 (cat pt-5 k1 "wrap")) (send (enc k k1)))
    ((load lk-1 (cat pt-6 old-1)) (stor lk-1 (cat pt-7 k "init"))
      (send (hash k)))
    ((load lk-1 (cat pt-8 k1 cur)) (stor lk-1 (cat pt-5 k1 "wrap"))))
  (label 630)
  (parent 253)
  (unrealized (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k1) (lk lk-0))
  (defstrand make 2 (old old-1) (k k1) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (0 0)) ((3 1) (4 2)) ((3 1) (7 0))
    ((4 4) (0 0)) ((5 1) (3 0)) ((5 2) (4 0)) ((6 1) (4 3))
    ((7 1) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k1 "init")
    (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule invShearsRule trRl_make-at-0 trRl_make-at-1)
  (operation channel-test (added-strand make 2)
    (ch-msg lk-0 (cat pt-6 k1 "init")) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-5 k "init")) (stor lk-0 (cat pt-2 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-2 k "decrypt"))
      (load lk-0 (cat pt-3 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-4 old-0)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-6 k1 "init")) (stor lk-0 (cat pt-3 k1 "wrap")))
    ((load lk-0 (cat pt-7 old-1)) (stor lk-0 (cat pt-6 k1 "init"))))
  (label 638)
  (parent 259)
  (realized)
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k1) (lk lk))
  (defstrand make 2 (old old-1) (k k1) (lk lk))
  (precedes ((1 1) (0 0)) ((2 1) (3 0)) ((3 1) (4 2)) ((3 1) (7 0))
    ((4 4) (0 0)) ((5 1) (1 0)) ((5 2) (4 0)) ((6 1) (4 3))
    ((7 1) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k1 "init")
    (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule invShearsRule trRl_make-at-0 trRl_make-at-1)
  (operation channel-test (added-strand make 2)
    (ch-msg lk (cat pt-6 k1 "init")) (6 0))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-5 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-2 k "decrypt"))
      (load lk (cat pt-3 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-4 old-0)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k)))
    ((load lk (cat pt-6 k1 "init")) (stor lk (cat pt-3 k1 "wrap")))
    ((load lk (cat pt-7 old-1)) (stor lk (cat pt-6 k1 "init"))))
  (label 639)
  (parent 262)
  (realized)
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (5 3)) ((3 1) (4 0)) ((4 1) (1 0))
    ((4 1) (5 2)) ((5 4) (0 0)) ((6 1) (7 0)) ((6 2) (5 0))
    ((7 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-decrypt-at-1 trRl_make-at-0)
  (operation nonce-test (added-strand set-decrypt 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk (cat pt-3 k "init")) (ch-msg lk (cat pt-4 k "decrypt"))
    (ch-msg lk-0 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init")))
    ((load lk (cat pt-3 k "init")) (stor lk (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-4 k "decrypt"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-5 old-1)) (stor lk-0 (cat pt-6 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-6 k "init"))
      (stor lk-0 (cat pt-7 k "decrypt"))))
  (label 640)
  (parent 263)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "wrap") (k k) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (5 3)) ((2 1) (7 0)) ((3 1) (4 0))
    ((4 1) (1 0)) ((4 1) (5 2)) ((5 4) (0 0)) ((6 2) (5 0))
    ((7 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k "wrap"))
  (facts (neq "wrap" "decrypt") (neq "wrap" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule fact-set-decrypt-neq0 trRl_set-wrap-at-1 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation nonce-test (added-strand set-wrap 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk (cat pt-3 k "init")) (ch-msg lk (cat pt-4 k "decrypt"))
    (ch-msg lk-0 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init")))
    ((load lk (cat pt-3 k "init")) (stor lk (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-4 k "decrypt"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-5 old-1)) (stor lk-0 (cat pt-6 k "init"))
      (send (hash k)))
    ((load lk (cat pt-1 k "wrap")) (stor lk (cat pt-7 k "wrap"))))
  (label 641)
  (parent 263)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (5 3)) ((3 1) (4 0)) ((4 1) (1 0))
    ((4 1) (5 2)) ((5 4) (0 0)) ((6 1) (7 0)) ((6 2) (5 0))
    ((7 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-wrap-at-1 trRl_make-at-0)
  (operation nonce-test (added-strand set-wrap 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk (cat pt-3 k "init")) (ch-msg lk (cat pt-4 k "decrypt"))
    (ch-msg lk-0 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init")))
    ((load lk (cat pt-3 k "init")) (stor lk (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-4 k "decrypt"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-5 old-1)) (stor lk-0 (cat pt-6 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-6 k "init")) (stor lk-0 (cat pt-7 k "wrap"))))
  (label 642)
  (parent 263)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 old-2 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval) (k skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-0))
  (defstrand make 2 (old old-2) (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((2 1) (5 3)) ((3 1) (4 0)) ((4 1) (1 0))
    ((4 1) (5 2)) ((5 4) (0 0)) ((6 2) (5 0)) ((7 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation nonce-test (added-strand make 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk (cat pt-3 k "init")) (ch-msg lk (cat pt-4 k "decrypt"))
    (ch-msg lk-0 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init")))
    ((load lk (cat pt-3 k "init")) (stor lk (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-4 k "decrypt"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-5 old-1)) (stor lk-0 (cat pt-6 k "init"))
      (send (hash k)))
    ((load lk-1 (cat pt-7 old-2)) (stor lk-1 (cat pt-8 k "init"))))
  (label 643)
  (parent 263)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (5 3)) ((3 1) (4 0)) ((4 1) (1 0))
    ((4 1) (5 2)) ((5 4) (0 0)) ((6 1) (7 2)) ((6 2) (5 0))
    ((7 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k "wrap")
    (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk (cat pt-3 k "init")) (ch-msg lk (cat pt-4 k "decrypt"))
    (ch-msg lk-0 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init")))
    ((load lk (cat pt-3 k "init")) (stor lk (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-4 k "decrypt"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-5 old-1)) (stor lk-0 (cat pt-6 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-6 k "init"))
      (load lk-0 (cat pt-7 k1 "wrap")) (send (enc k k1))))
  (label 644)
  (parent 263)
  (unrealized (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "wrap") (k k) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (5 3)) ((2 1) (7 0)) ((3 1) (4 0))
    ((4 1) (1 0)) ((4 1) (5 2)) ((5 4) (0 0)) ((6 2) (5 0))
    ((7 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k "wrap"))
  (facts (neq "wrap" "decrypt") (neq "wrap" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-decrypt-at-1 fact-set-decrypt-neq0 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation nonce-test (added-strand set-decrypt 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk (cat pt-3 k "init")) (ch-msg lk (cat pt-4 k "decrypt"))
    (ch-msg lk-0 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init")))
    ((load lk (cat pt-3 k "init")) (stor lk (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-4 k "decrypt"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-5 old-1)) (stor lk-0 (cat pt-6 k "init"))
      (send (hash k)))
    ((load lk (cat pt-1 k "wrap")) (stor lk (cat pt-7 k "decrypt"))))
  (label 645)
  (parent 263)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (5 3)) ((3 1) (4 0)) ((3 1) (7 2))
    ((4 1) (1 0)) ((4 1) (5 2)) ((5 4) (0 0)) ((6 2) (5 0))
    ((7 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k "wrap")
    (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk (cat pt-3 k "init")) (ch-msg lk (cat pt-4 k "decrypt"))
    (ch-msg lk-0 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init")))
    ((load lk (cat pt-3 k "init")) (stor lk (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-4 k "decrypt"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-5 old-1)) (stor lk-0 (cat pt-6 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-7 k1 "wrap")) (send (enc k k1))))
  (label 646)
  (parent 263)
  (unrealized (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (5 3)) ((3 1) (4 0)) ((4 1) (1 0))
    ((4 1) (5 2)) ((4 1) (7 2)) ((5 4) (0 0)) ((6 2) (5 0))
    ((7 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k "wrap")
    (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk (cat pt-3 k "init")) (ch-msg lk (cat pt-4 k "decrypt"))
    (ch-msg lk-0 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init")))
    ((load lk (cat pt-3 k "init")) (stor lk (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-4 k "decrypt"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-5 old-1)) (stor lk-0 (cat pt-6 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-4 k "decrypt"))
      (load lk (cat pt-7 k1 "wrap")) (send (enc k k1))))
  (label 647)
  (parent 263)
  (unrealized (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (5 3)) ((2 1) (7 2)) ((3 1) (4 0))
    ((4 1) (1 0)) ((4 1) (5 2)) ((5 4) (0 0)) ((6 2) (5 0))
    ((7 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k "wrap")
    (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk (cat pt-3 k "init")) (ch-msg lk (cat pt-4 k "decrypt"))
    (ch-msg lk-0 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init")))
    ((load lk (cat pt-3 k "init")) (stor lk (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-4 k "decrypt"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-5 old-1)) (stor lk-0 (cat pt-6 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-7 k1 "wrap")) (send (enc k k1))))
  (label 648)
  (parent 263)
  (unrealized (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (7 2)) ((2 1) (5 3)) ((3 1) (4 0))
    ((4 1) (1 0)) ((4 1) (5 2)) ((5 4) (0 0)) ((6 2) (5 0))
    ((7 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k "wrap")
    (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk (cat pt-3 k "init")) (ch-msg lk (cat pt-4 k "decrypt"))
    (ch-msg lk-0 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init")))
    ((load lk (cat pt-3 k "init")) (stor lk (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-4 k "decrypt"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-5 old-1)) (stor lk-0 (cat pt-6 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-7 k1 "wrap")) (send (enc k k1))))
  (label 649)
  (parent 263)
  (unrealized (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-0))
  (defstrand decrypt 4 (x k) (k k) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (5 3)) ((3 1) (4 0)) ((4 1) (1 0))
    ((4 1) (5 2)) ((4 1) (7 2)) ((5 4) (0 0)) ((6 2) (5 0))
    ((7 3) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation nonce-test (added-strand decrypt 4) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk (cat pt-3 k "init")) (ch-msg lk (cat pt-4 k "decrypt"))
    (ch-msg lk-0 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init")))
    ((load lk (cat pt-3 k "init")) (stor lk (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-4 k "decrypt"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-5 old-1)) (stor lk-0 (cat pt-6 k "init"))
      (send (hash k)))
    ((recv (enc k k)) (recv (hash k)) (load lk (cat pt-4 k "decrypt"))
      (send k)))
  (label 650)
  (parent 263)
  (unrealized (7 0) (7 1))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1) (lk lk-0))
  (deflistener k)
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (0 0)) ((3 1) (4 0))
    ((4 1) (5 2)) ((5 4) (0 0)) ((6 1) (5 0)) ((7 1) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation nonce-test (added-strand make 2) k (6 0)
    (ch-msg lk (cat pt-0 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-4 k "decrypt"))
      (load lk-0 (cat pt-5 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-1 (cat pt-6 old-1)) (stor lk-1 (cat pt-7 k "init"))))
  (label 651)
  (parent 267)
  (unrealized (5 3) (6 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1) (lk lk-0))
  (deflistener k)
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (7 2)) ((2 1) (0 0)) ((3 1) (4 0))
    ((4 1) (5 2)) ((5 4) (0 0)) ((6 1) (5 0)) ((7 4) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k1 "wrap")
    (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (6 0)
    (ch-msg lk (cat pt-0 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-4 k "decrypt"))
      (load lk-0 (cat pt-5 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((recv (hash k)) (recv (hash k1-0)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-6 k1-0 "wrap")) (send (enc k k1-0))))
  (label 653)
  (parent 267)
  (unrealized (5 3) (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1) (lk lk-0))
  (deflistener k)
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((2 1) (0 0)) ((3 1) (4 0)) ((3 1) (6 0))
    ((4 1) (5 2)) ((5 4) (0 0)) ((6 1) (5 0)) ((7 1) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation nonce-test (added-strand make 2) k (6 0)
    (ch-msg lk-0 (cat pt-3 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-4 k "decrypt"))
      (load lk-0 (cat pt-5 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-1 (cat pt-6 old-1)) (stor lk-1 (cat pt-7 k "init"))))
  (label 654)
  (parent 268)
  (unrealized (5 3) (6 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1) (lk lk-0))
  (deflistener k)
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (0 0)) ((3 1) (4 0)) ((3 1) (7 2))
    ((4 1) (5 2)) ((5 4) (0 0)) ((6 1) (5 0)) ((7 4) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k1 "wrap")
    (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (6 0)
    (ch-msg lk-0 (cat pt-3 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-4 k "decrypt"))
      (load lk-0 (cat pt-5 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-6 k1-0 "wrap")) (send (enc k k1-0))))
  (label 656)
  (parent 268)
  (unrealized (5 3) (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k1) (lk lk-0))
  (defstrand make 2 (old old-1) (k k1) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (0 0)) ((3 1) (7 0)) ((4 4) (0 0))
    ((5 1) (3 0)) ((5 2) (4 0)) ((6 1) (4 3)) ((7 1) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule invShearsRule trRl_make-at-0 trRl_make-at-1)
  (operation channel-test (added-strand make 2)
    (ch-msg lk-0 (cat pt-6 k1 "init")) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-5 k "init")) (stor lk-0 (cat pt-2 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-5 k "init"))
      (load lk-0 (cat pt-3 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-4 old-0)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-6 k1 "init")) (stor lk-0 (cat pt-3 k1 "wrap")))
    ((load lk-0 (cat pt-7 old-1)) (stor lk-0 (cat pt-6 k1 "init"))))
  (label 664)
  (parent 272)
  (realized)
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k1) (lk lk))
  (defstrand make 2 (old old-1) (k k1) (lk lk))
  (precedes ((1 1) (0 0)) ((2 1) (3 0)) ((2 1) (4 2)) ((3 1) (7 0))
    ((4 4) (0 0)) ((5 1) (1 0)) ((5 2) (4 0)) ((6 1) (4 3))
    ((7 1) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule invShearsRule trRl_make-at-0 trRl_make-at-1)
  (operation channel-test (added-strand make 2)
    (ch-msg lk (cat pt-6 k1 "init")) (6 0))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-5 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "init"))
      (load lk (cat pt-3 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-4 old-0)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k)))
    ((load lk (cat pt-6 k1 "init")) (stor lk (cat pt-3 k1 "wrap")))
    ((load lk (cat pt-7 old-1)) (stor lk (cat pt-6 k1 "init"))))
  (label 665)
  (parent 273)
  (realized)
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (5 3)) ((3 1) (4 0)) ((3 1) (5 2))
    ((4 1) (1 0)) ((5 4) (0 0)) ((6 1) (7 0)) ((6 2) (5 0))
    ((7 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-decrypt-at-1 trRl_make-at-0)
  (operation nonce-test (added-strand set-decrypt 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk (cat pt-3 k "init")) (ch-msg lk (cat pt-4 k "decrypt"))
    (ch-msg lk-0 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init")))
    ((load lk (cat pt-3 k "init")) (stor lk (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-5 old-1)) (stor lk-0 (cat pt-6 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-6 k "init"))
      (stor lk-0 (cat pt-7 k "decrypt"))))
  (label 666)
  (parent 274)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "wrap") (k k) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (5 3)) ((2 1) (7 0)) ((3 1) (4 0))
    ((3 1) (5 2)) ((4 1) (1 0)) ((5 4) (0 0)) ((6 2) (5 0))
    ((7 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "wrap" "decrypt") (neq "wrap" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule fact-set-decrypt-neq0 trRl_set-wrap-at-1 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation nonce-test (added-strand set-wrap 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk (cat pt-3 k "init")) (ch-msg lk (cat pt-4 k "decrypt"))
    (ch-msg lk-0 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init")))
    ((load lk (cat pt-3 k "init")) (stor lk (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-5 old-1)) (stor lk-0 (cat pt-6 k "init"))
      (send (hash k)))
    ((load lk (cat pt-1 k "wrap")) (stor lk (cat pt-7 k "wrap"))))
  (label 667)
  (parent 274)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (5 3)) ((3 1) (4 0)) ((3 1) (5 2))
    ((4 1) (1 0)) ((5 4) (0 0)) ((6 1) (7 0)) ((6 2) (5 0))
    ((7 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-wrap-at-1 trRl_make-at-0)
  (operation nonce-test (added-strand set-wrap 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk (cat pt-3 k "init")) (ch-msg lk (cat pt-4 k "decrypt"))
    (ch-msg lk-0 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init")))
    ((load lk (cat pt-3 k "init")) (stor lk (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-5 old-1)) (stor lk-0 (cat pt-6 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-6 k "init")) (stor lk-0 (cat pt-7 k "wrap"))))
  (label 668)
  (parent 274)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 old-2 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval) (k skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-0))
  (defstrand make 2 (old old-2) (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((2 1) (5 3)) ((3 1) (4 0)) ((3 1) (5 2))
    ((4 1) (1 0)) ((5 4) (0 0)) ((6 2) (5 0)) ((7 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation nonce-test (added-strand make 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk (cat pt-3 k "init")) (ch-msg lk (cat pt-4 k "decrypt"))
    (ch-msg lk-0 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init")))
    ((load lk (cat pt-3 k "init")) (stor lk (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-5 old-1)) (stor lk-0 (cat pt-6 k "init"))
      (send (hash k)))
    ((load lk-1 (cat pt-7 old-2)) (stor lk-1 (cat pt-8 k "init"))))
  (label 669)
  (parent 274)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (5 3)) ((3 1) (4 0)) ((3 1) (5 2))
    ((4 1) (1 0)) ((5 4) (0 0)) ((6 1) (7 2)) ((6 2) (5 0))
    ((7 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk (cat pt-3 k "init")) (ch-msg lk (cat pt-4 k "decrypt"))
    (ch-msg lk-0 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init")))
    ((load lk (cat pt-3 k "init")) (stor lk (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-5 old-1)) (stor lk-0 (cat pt-6 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-6 k "init"))
      (load lk-0 (cat pt-7 k1 "wrap")) (send (enc k k1))))
  (label 670)
  (parent 274)
  (unrealized (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "wrap") (k k) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (5 3)) ((2 1) (7 0)) ((3 1) (4 0))
    ((3 1) (5 2)) ((4 1) (1 0)) ((5 4) (0 0)) ((6 2) (5 0))
    ((7 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "wrap" "decrypt") (neq "wrap" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-decrypt-at-1 fact-set-decrypt-neq0 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation nonce-test (added-strand set-decrypt 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk (cat pt-3 k "init")) (ch-msg lk (cat pt-4 k "decrypt"))
    (ch-msg lk-0 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init")))
    ((load lk (cat pt-3 k "init")) (stor lk (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-5 old-1)) (stor lk-0 (cat pt-6 k "init"))
      (send (hash k)))
    ((load lk (cat pt-1 k "wrap")) (stor lk (cat pt-7 k "decrypt"))))
  (label 671)
  (parent 274)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (5 3)) ((3 1) (4 0)) ((3 1) (5 2))
    ((3 1) (7 2)) ((4 1) (1 0)) ((5 4) (0 0)) ((6 2) (5 0))
    ((7 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk (cat pt-3 k "init")) (ch-msg lk (cat pt-4 k "decrypt"))
    (ch-msg lk-0 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init")))
    ((load lk (cat pt-3 k "init")) (stor lk (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-5 old-1)) (stor lk-0 (cat pt-6 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-7 k1 "wrap")) (send (enc k k1))))
  (label 672)
  (parent 274)
  (unrealized (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (5 3)) ((3 1) (4 0)) ((3 1) (5 2))
    ((4 1) (1 0)) ((4 1) (7 2)) ((5 4) (0 0)) ((6 2) (5 0))
    ((7 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k "wrap")
    (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0 gen-st-wrap-1)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk (cat pt-3 k "init")) (ch-msg lk (cat pt-4 k "decrypt"))
    (ch-msg lk-0 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init")))
    ((load lk (cat pt-3 k "init")) (stor lk (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-5 old-1)) (stor lk-0 (cat pt-6 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-4 k "decrypt"))
      (load lk (cat pt-7 k1 "wrap")) (send (enc k k1))))
  (label 673)
  (parent 274)
  (unrealized (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (5 3)) ((2 1) (7 2)) ((3 1) (4 0))
    ((3 1) (5 2)) ((4 1) (1 0)) ((5 4) (0 0)) ((6 2) (5 0))
    ((7 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk (cat pt-3 k "init")) (ch-msg lk (cat pt-4 k "decrypt"))
    (ch-msg lk-0 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init")))
    ((load lk (cat pt-3 k "init")) (stor lk (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-5 old-1)) (stor lk-0 (cat pt-6 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-7 k1 "wrap")) (send (enc k k1))))
  (label 674)
  (parent 274)
  (unrealized (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (7 2)) ((2 1) (5 3)) ((3 1) (4 0))
    ((3 1) (5 2)) ((4 1) (1 0)) ((5 4) (0 0)) ((6 2) (5 0))
    ((7 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk (cat pt-3 k "init")) (ch-msg lk (cat pt-4 k "decrypt"))
    (ch-msg lk-0 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init")))
    ((load lk (cat pt-3 k "init")) (stor lk (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-5 old-1)) (stor lk-0 (cat pt-6 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-7 k1 "wrap")) (send (enc k k1))))
  (label 675)
  (parent 274)
  (unrealized (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-0))
  (defstrand decrypt 4 (x k) (k k) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (5 3)) ((3 1) (4 0)) ((3 1) (5 2))
    ((4 1) (1 0)) ((4 1) (7 2)) ((5 4) (0 0)) ((6 2) (5 0))
    ((7 3) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-decrypt-0)
  (operation nonce-test (added-strand decrypt 4) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk (cat pt-3 k "init")) (ch-msg lk (cat pt-4 k "decrypt"))
    (ch-msg lk-0 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init")))
    ((load lk (cat pt-3 k "init")) (stor lk (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-5 old-1)) (stor lk-0 (cat pt-6 k "init"))
      (send (hash k)))
    ((recv (enc k k)) (recv (hash k)) (load lk (cat pt-4 k "decrypt"))
      (send k)))
  (label 676)
  (parent 274)
  (unrealized (7 0) (7 1))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (deflistener k)
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (0 0)) ((3 1) (4 0))
    ((3 1) (5 2)) ((4 1) (0 0)) ((5 4) (0 0)) ((6 1) (5 0))
    ((7 1) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation nonce-test (added-strand make 2) k (6 0)
    (ch-msg lk (cat pt-0 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-5 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-1 (cat pt-6 old-1)) (stor lk-1 (cat pt-7 k "init"))))
  (label 677)
  (parent 277)
  (unrealized (5 3) (6 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (deflistener k)
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (7 2)) ((2 1) (0 0)) ((3 1) (4 0))
    ((3 1) (5 2)) ((4 1) (0 0)) ((5 4) (0 0)) ((6 1) (5 0))
    ((7 4) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (6 0)
    (ch-msg lk (cat pt-0 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-5 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((recv (hash k)) (recv (hash k1-0)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-6 k1-0 "wrap")) (send (enc k k1-0))))
  (label 679)
  (parent 277)
  (unrealized (5 3) (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (deflistener k)
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((2 1) (0 0)) ((3 1) (4 0)) ((3 1) (6 0))
    ((4 1) (0 0)) ((5 4) (0 0)) ((6 1) (5 0)) ((7 1) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation nonce-test (added-strand make 2) k (6 0)
    (ch-msg lk-0 (cat pt-3 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-5 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-1 (cat pt-6 old-1)) (stor lk-1 (cat pt-7 k "init"))))
  (label 680)
  (parent 278)
  (unrealized (5 3) (6 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (deflistener k)
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (0 0)) ((3 1) (4 0)) ((3 1) (7 2))
    ((4 1) (0 0)) ((5 4) (0 0)) ((6 1) (5 0)) ((7 4) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (6 0)
    (ch-msg lk-0 (cat pt-3 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-5 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-6 k1-0 "wrap")) (send (enc k k1-0))))
  (label 681)
  (parent 278)
  (unrealized (5 3) (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k1) (lk lk))
  (defstrand make 2 (old old-1) (k k1) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (4 2)) ((2 1) (7 0)) ((3 1) (0 0))
    ((4 4) (0 0)) ((5 1) (3 0)) ((5 2) (4 0)) ((6 1) (4 3))
    ((7 1) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule invShearsRule trRl_make-at-0 trRl_make-at-1)
  (operation channel-test (added-strand make 2)
    (ch-msg lk (cat pt-6 k1 "init")) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-5 k "init")) (stor lk-0 (cat pt-2 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-3 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-4 old-0)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k)))
    ((load lk (cat pt-6 k1 "init")) (stor lk (cat pt-3 k1 "wrap")))
    ((load lk (cat pt-7 old-1)) (stor lk (cat pt-6 k1 "init"))))
  (label 682)
  (parent 280)
  (realized)
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k1) (lk lk-0))
  (defstrand make 2 (old old-1) (k k1) (lk lk-0))
  (precedes ((1 1) (4 2)) ((1 1) (7 0)) ((2 1) (3 0)) ((3 1) (0 0))
    ((4 4) (0 0)) ((5 1) (1 0)) ((5 2) (4 0)) ((6 1) (4 3))
    ((7 1) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule invShearsRule trRl_make-at-0 trRl_make-at-1)
  (operation channel-test (added-strand make 2)
    (ch-msg lk-0 (cat pt-6 k1 "init")) (6 0))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-5 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt k "wrap"))
      (load lk-0 (cat pt-3 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-4 old-0)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-6 k1 "init")) (stor lk-0 (cat pt-3 k1 "wrap")))
    ((load lk-0 (cat pt-7 old-1)) (stor lk-0 (cat pt-6 k1 "init"))))
  (label 692)
  (parent 282)
  (realized)
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand set-decrypt 2 (cur "decrypt") (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (5 2)) ((3 1) (4 0)) ((4 1) (7 0))
    ((5 4) (0 0)) ((6 2) (5 0)) ((7 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k "wrap"))
  (facts (neq "decrypt" "decrypt") (neq "decrypt" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-decrypt-at-1 fact-set-decrypt-neq0 gen-st-set-wrap-0
    fact-set-wrap-neq0 trRl_make-at-0)
  (operation nonce-test (added-strand set-decrypt 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init"))
    (ch-msg lk-0 (cat pt-4 k "decrypt"))
    (ch-msg lk-1 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-4 k "decrypt"))
      (stor lk-0 (cat pt-7 k "decrypt"))))
  (label 693)
  (parent 283)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((2 1) (5 2)) ((3 1) (4 0)) ((4 1) (0 0))
    ((5 4) (0 0)) ((6 1) (7 0)) ((6 2) (5 0)) ((7 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-decrypt-at-1 trRl_make-at-0)
  (operation nonce-test (added-strand set-decrypt 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init"))
    (ch-msg lk-0 (cat pt-4 k "decrypt"))
    (ch-msg lk-1 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init"))
      (send (hash k)))
    ((load lk-1 (cat pt-6 k "init"))
      (stor lk-1 (cat pt-7 k "decrypt"))))
  (label 694)
  (parent 283)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand set-wrap 2 (cur "wrap") (k k) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (5 2)) ((2 1) (7 0)) ((3 1) (4 0))
    ((4 1) (0 0)) ((5 4) (0 0)) ((6 2) (5 0)) ((7 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "wrap" "decrypt") (neq "wrap" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule fact-set-decrypt-neq0 trRl_set-wrap-at-1 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation nonce-test (added-strand set-wrap 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init"))
    (ch-msg lk-0 (cat pt-4 k "decrypt"))
    (ch-msg lk-1 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init"))
      (send (hash k)))
    ((load lk (cat pt-1 k "wrap")) (stor lk (cat pt-7 k "wrap"))))
  (label 695)
  (parent 283)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((2 1) (5 2)) ((3 1) (4 0)) ((4 1) (0 0))
    ((5 4) (0 0)) ((6 1) (7 0)) ((6 2) (5 0)) ((7 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-wrap-at-1 trRl_make-at-0)
  (operation nonce-test (added-strand set-wrap 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init"))
    (ch-msg lk-0 (cat pt-4 k "decrypt"))
    (ch-msg lk-1 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init"))
      (send (hash k)))
    ((load lk-1 (cat pt-6 k "init")) (stor lk-1 (cat pt-7 k "wrap"))))
  (label 696)
  (parent 283)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 old-2 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval) (k skey)
    (lk lk-0 lk-1 lk-2 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand make 2 (old old-2) (k k) (lk lk-2))
  (precedes ((1 1) (2 0)) ((2 1) (5 2)) ((3 1) (4 0)) ((4 1) (0 0))
    ((5 4) (0 0)) ((6 2) (5 0)) ((7 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation nonce-test (added-strand make 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init"))
    (ch-msg lk-0 (cat pt-4 k "decrypt"))
    (ch-msg lk-1 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init"))
      (send (hash k)))
    ((load lk-2 (cat pt-7 old-2)) (stor lk-2 (cat pt-8 k "init"))))
  (label 697)
  (parent 283)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand set-wrap 2 (cur "decrypt") (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (5 2)) ((3 1) (4 0)) ((4 1) (7 0))
    ((5 4) (0 0)) ((6 2) (5 0)) ((7 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k "wrap"))
  (facts (neq "decrypt" "decrypt") (neq "decrypt" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule fact-set-decrypt-neq0 trRl_set-wrap-at-1 gen-st-set-wrap-0
    fact-set-wrap-neq0 trRl_make-at-0)
  (operation nonce-test (added-strand set-wrap 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init"))
    (ch-msg lk-0 (cat pt-4 k "decrypt"))
    (ch-msg lk-1 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-4 k "decrypt"))
      (stor lk-0 (cat pt-7 k "wrap"))))
  (label 698)
  (parent 283)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (5 2)) ((3 1) (4 0)) ((4 1) (7 2))
    ((5 4) (0 0)) ((6 2) (5 0)) ((7 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k "wrap")
    (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0 gen-st-wrap-1)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init"))
    (ch-msg lk-0 (cat pt-4 k "decrypt"))
    (ch-msg lk-1 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-4 k "decrypt"))
      (load lk-0 (cat pt-7 k1 "wrap")) (send (enc k k1))))
  (label 699)
  (parent 283)
  (unrealized (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-1))
  (precedes ((1 1) (2 0)) ((2 1) (5 2)) ((3 1) (4 0)) ((4 1) (0 0))
    ((5 4) (0 0)) ((6 1) (7 2)) ((6 2) (5 0)) ((7 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init"))
    (ch-msg lk-0 (cat pt-4 k "decrypt"))
    (ch-msg lk-1 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk-1 (cat pt-6 k "init"))
      (load lk-1 (cat pt-7 k1 "wrap")) (send (enc k k1))))
  (label 700)
  (parent 283)
  (unrealized (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand set-decrypt 2 (cur "wrap") (k k) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (5 2)) ((2 1) (7 0)) ((3 1) (4 0))
    ((4 1) (0 0)) ((5 4) (0 0)) ((6 2) (5 0)) ((7 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "wrap" "decrypt") (neq "wrap" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-decrypt-at-1 fact-set-decrypt-neq0 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation nonce-test (added-strand set-decrypt 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init"))
    (ch-msg lk-0 (cat pt-4 k "decrypt"))
    (ch-msg lk-1 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init"))
      (send (hash k)))
    ((load lk (cat pt-1 k "wrap")) (stor lk (cat pt-7 k "decrypt"))))
  (label 701)
  (parent 283)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (5 2)) ((2 1) (7 2)) ((3 1) (4 0))
    ((4 1) (0 0)) ((5 4) (0 0)) ((6 2) (5 0)) ((7 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init"))
    (ch-msg lk-0 (cat pt-4 k "decrypt"))
    (ch-msg lk-1 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-7 k1 "wrap")) (send (enc k k1))))
  (label 702)
  (parent 283)
  (unrealized (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (5 2)) ((3 1) (4 0)) ((3 1) (7 2))
    ((4 1) (0 0)) ((5 4) (0 0)) ((6 2) (5 0)) ((7 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init"))
    (ch-msg lk-0 (cat pt-4 k "decrypt"))
    (ch-msg lk-1 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-7 k1 "wrap")) (send (enc k k1))))
  (label 703)
  (parent 283)
  (unrealized (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (7 2)) ((2 1) (5 2)) ((3 1) (4 0))
    ((4 1) (0 0)) ((5 4) (0 0)) ((6 2) (5 0)) ((7 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init"))
    (ch-msg lk-0 (cat pt-4 k "decrypt"))
    (ch-msg lk-1 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-7 k1 "wrap")) (send (enc k k1))))
  (label 704)
  (parent 283)
  (unrealized (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand decrypt 4 (x k) (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (5 2)) ((3 1) (4 0)) ((4 1) (7 2))
    ((5 4) (0 0)) ((6 2) (5 0)) ((7 3) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-decrypt-0)
  (operation nonce-test (added-strand decrypt 4) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init"))
    (ch-msg lk-0 (cat pt-4 k "decrypt"))
    (ch-msg lk-1 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init"))
      (send (hash k)))
    ((recv (enc k k)) (recv (hash k)) (load lk-0 (cat pt-4 k "decrypt"))
      (send k)))
  (label 705)
  (parent 283)
  (unrealized (7 0) (7 1))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (5 2)) ((3 1) (4 0))
    ((4 1) (0 0)) ((5 4) (0 0)) ((6 1) (5 0)) ((7 1) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation nonce-test (added-strand make 2) k (6 0)
    (ch-msg lk (cat pt-0 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-5 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-1 (cat pt-6 old-1)) (stor lk-1 (cat pt-7 k "init"))))
  (label 707)
  (parent 285)
  (unrealized (5 3) (6 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (7 2)) ((2 1) (5 2)) ((3 1) (4 0))
    ((4 1) (0 0)) ((5 4) (0 0)) ((6 1) (5 0)) ((7 4) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap")
    (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (6 0)
    (ch-msg lk (cat pt-0 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-5 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((recv (hash k)) (recv (hash k1-0)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-6 k1-0 "wrap")) (send (enc k k1-0))))
  (label 709)
  (parent 285)
  (unrealized (5 3) (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((2 1) (5 2)) ((3 1) (4 0)) ((3 1) (6 0))
    ((4 1) (0 0)) ((5 4) (0 0)) ((6 1) (5 0)) ((7 1) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation nonce-test (added-strand make 2) k (6 0)
    (ch-msg lk-0 (cat pt-3 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-5 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-1 (cat pt-6 old-1)) (stor lk-1 (cat pt-7 k "init"))))
  (label 710)
  (parent 286)
  (unrealized (5 3) (6 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (5 2)) ((3 1) (4 0)) ((3 1) (7 2))
    ((4 1) (0 0)) ((5 4) (0 0)) ((6 1) (5 0)) ((7 4) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap")
    (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (6 0)
    (ch-msg lk-0 (cat pt-3 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-5 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-6 k1-0 "wrap")) (send (enc k k1-0))))
  (label 711)
  (parent 286)
  (unrealized (5 3) (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k1) (lk lk))
  (defstrand make 2 (old old-1) (k k1) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (4 2)) ((2 1) (7 0)) ((3 1) (0 0))
    ((4 4) (0 0)) ((5 1) (3 0)) ((5 2) (4 0)) ((6 1) (4 3))
    ((7 1) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule invShearsRule trRl_make-at-0 trRl_make-at-1)
  (operation channel-test (added-strand make 2)
    (ch-msg lk (cat pt-6 k1 "init")) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-5 k "init")) (stor lk-0 (cat pt-2 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-3 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-4 old-0)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k)))
    ((load lk (cat pt-6 k1 "init")) (stor lk (cat pt-3 k1 "wrap")))
    ((load lk (cat pt-7 old-1)) (stor lk (cat pt-6 k1 "init"))))
  (label 712)
  (parent 288)
  (realized)
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k1) (lk lk-0))
  (defstrand make 2 (old old-1) (k k1) (lk lk-0))
  (precedes ((1 1) (7 0)) ((2 1) (3 0)) ((3 1) (0 0)) ((4 4) (0 0))
    ((5 1) (1 0)) ((5 2) (4 0)) ((6 1) (4 3)) ((7 1) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule invShearsRule trRl_make-at-0 trRl_make-at-1)
  (operation channel-test (added-strand make 2)
    (ch-msg lk-0 (cat pt-6 k1 "init")) (6 0))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-5 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((load lk (cat pt-0 old)) (stor lk (cat pt-1 k "init")))
    ((load lk (cat pt-1 k "init")) (stor lk (cat pt-2 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-5 k "init"))
      (load lk-0 (cat pt-3 k1 "wrap")) (send (enc k k1)))
    ((load lk-0 (cat pt-4 old-0)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-6 k1 "init")) (stor lk-0 (cat pt-3 k1 "wrap")))
    ((load lk-0 (cat pt-7 old-1)) (stor lk-0 (cat pt-6 k1 "init"))))
  (label 722)
  (parent 290)
  (realized)
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand set-decrypt 2 (cur "decrypt") (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((1 1) (5 2)) ((2 1) (5 3)) ((3 1) (4 0))
    ((4 1) (7 0)) ((5 4) (0 0)) ((6 2) (5 0)) ((7 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k "wrap"))
  (facts (neq "decrypt" "decrypt") (neq "decrypt" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-decrypt-at-1 fact-set-decrypt-neq0 gen-st-set-wrap-0
    fact-set-wrap-neq0 trRl_make-at-0)
  (operation nonce-test (added-strand set-decrypt 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init"))
    (ch-msg lk-0 (cat pt-4 k "decrypt"))
    (ch-msg lk-1 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-4 k "decrypt"))
      (stor lk-0 (cat pt-7 k "decrypt"))))
  (label 723)
  (parent 291)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((1 1) (5 2)) ((2 1) (5 3)) ((3 1) (4 0))
    ((4 1) (0 0)) ((5 4) (0 0)) ((6 1) (7 0)) ((6 2) (5 0))
    ((7 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-decrypt-at-1 trRl_make-at-0)
  (operation nonce-test (added-strand set-decrypt 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init"))
    (ch-msg lk-0 (cat pt-4 k "decrypt"))
    (ch-msg lk-1 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init"))
      (send (hash k)))
    ((load lk-1 (cat pt-6 k "init"))
      (stor lk-1 (cat pt-7 k "decrypt"))))
  (label 724)
  (parent 291)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand set-wrap 2 (cur "wrap") (k k) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (5 2)) ((2 1) (5 3)) ((2 1) (7 0))
    ((3 1) (4 0)) ((4 1) (0 0)) ((5 4) (0 0)) ((6 2) (5 0))
    ((7 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "wrap" "decrypt") (neq "wrap" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule fact-set-decrypt-neq0 trRl_set-wrap-at-1 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation nonce-test (added-strand set-wrap 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init"))
    (ch-msg lk-0 (cat pt-4 k "decrypt"))
    (ch-msg lk-1 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init"))
      (send (hash k)))
    ((load lk (cat pt-1 k "wrap")) (stor lk (cat pt-7 k "wrap"))))
  (label 725)
  (parent 291)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((1 1) (5 2)) ((2 1) (5 3)) ((3 1) (4 0))
    ((4 1) (0 0)) ((5 4) (0 0)) ((6 1) (7 0)) ((6 2) (5 0))
    ((7 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-wrap-at-1 trRl_make-at-0)
  (operation nonce-test (added-strand set-wrap 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init"))
    (ch-msg lk-0 (cat pt-4 k "decrypt"))
    (ch-msg lk-1 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init"))
      (send (hash k)))
    ((load lk-1 (cat pt-6 k "init")) (stor lk-1 (cat pt-7 k "wrap"))))
  (label 726)
  (parent 291)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 old-2 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval) (k skey)
    (lk lk-0 lk-1 lk-2 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand make 2 (old old-2) (k k) (lk lk-2))
  (precedes ((1 1) (2 0)) ((1 1) (5 2)) ((2 1) (5 3)) ((3 1) (4 0))
    ((4 1) (0 0)) ((5 4) (0 0)) ((6 2) (5 0)) ((7 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation nonce-test (added-strand make 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init"))
    (ch-msg lk-0 (cat pt-4 k "decrypt"))
    (ch-msg lk-1 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init"))
      (send (hash k)))
    ((load lk-2 (cat pt-7 old-2)) (stor lk-2 (cat pt-8 k "init"))))
  (label 727)
  (parent 291)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand set-wrap 2 (cur "decrypt") (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((1 1) (5 2)) ((2 1) (5 3)) ((3 1) (4 0))
    ((4 1) (7 0)) ((5 4) (0 0)) ((6 2) (5 0)) ((7 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k "wrap"))
  (facts (neq "decrypt" "decrypt") (neq "decrypt" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule fact-set-decrypt-neq0 trRl_set-wrap-at-1 gen-st-set-wrap-0
    fact-set-wrap-neq0 trRl_make-at-0)
  (operation nonce-test (added-strand set-wrap 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init"))
    (ch-msg lk-0 (cat pt-4 k "decrypt"))
    (ch-msg lk-1 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-4 k "decrypt"))
      (stor lk-0 (cat pt-7 k "wrap"))))
  (label 728)
  (parent 291)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1) (lk lk-0))
  (precedes ((1 1) (2 0)) ((1 1) (5 2)) ((2 1) (5 3)) ((3 1) (4 0))
    ((4 1) (7 2)) ((5 4) (0 0)) ((6 2) (5 0)) ((7 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k "wrap")
    (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0 gen-st-wrap-1)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init"))
    (ch-msg lk-0 (cat pt-4 k "decrypt"))
    (ch-msg lk-1 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-4 k "decrypt"))
      (load lk-0 (cat pt-7 k1 "wrap")) (send (enc k k1))))
  (label 729)
  (parent 291)
  (unrealized (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-1))
  (precedes ((1 1) (2 0)) ((1 1) (5 2)) ((2 1) (5 3)) ((3 1) (4 0))
    ((4 1) (0 0)) ((5 4) (0 0)) ((6 1) (7 2)) ((6 2) (5 0))
    ((7 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init"))
    (ch-msg lk-0 (cat pt-4 k "decrypt"))
    (ch-msg lk-1 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk-1 (cat pt-6 k "init"))
      (load lk-1 (cat pt-7 k1 "wrap")) (send (enc k k1))))
  (label 730)
  (parent 291)
  (unrealized (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand set-decrypt 2 (cur "wrap") (k k) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (5 2)) ((2 1) (5 3)) ((2 1) (7 0))
    ((3 1) (4 0)) ((4 1) (0 0)) ((5 4) (0 0)) ((6 2) (5 0))
    ((7 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "wrap" "decrypt") (neq "wrap" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-decrypt-at-1 fact-set-decrypt-neq0 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation nonce-test (added-strand set-decrypt 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init"))
    (ch-msg lk-0 (cat pt-4 k "decrypt"))
    (ch-msg lk-1 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init"))
      (send (hash k)))
    ((load lk (cat pt-1 k "wrap")) (stor lk (cat pt-7 k "decrypt"))))
  (label 731)
  (parent 291)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (5 2)) ((2 1) (5 3)) ((2 1) (7 2))
    ((3 1) (4 0)) ((4 1) (0 0)) ((5 4) (0 0)) ((6 2) (5 0))
    ((7 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init"))
    (ch-msg lk-0 (cat pt-4 k "decrypt"))
    (ch-msg lk-1 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-7 k1 "wrap")) (send (enc k k1))))
  (label 732)
  (parent 291)
  (unrealized (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (precedes ((1 1) (2 0)) ((1 1) (5 2)) ((2 1) (5 3)) ((3 1) (4 0))
    ((3 1) (7 2)) ((4 1) (0 0)) ((5 4) (0 0)) ((6 2) (5 0))
    ((7 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init"))
    (ch-msg lk-0 (cat pt-4 k "decrypt"))
    (ch-msg lk-1 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-7 k1 "wrap")) (send (enc k k1))))
  (label 733)
  (parent 291)
  (unrealized (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (5 2)) ((1 1) (7 2)) ((2 1) (5 3))
    ((3 1) (4 0)) ((4 1) (0 0)) ((5 4) (0 0)) ((6 2) (5 0))
    ((7 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init"))
    (ch-msg lk-0 (cat pt-4 k "decrypt"))
    (ch-msg lk-1 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-7 k1 "wrap")) (send (enc k k1))))
  (label 734)
  (parent 291)
  (unrealized (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand decrypt 4 (x k) (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((1 1) (5 2)) ((2 1) (5 3)) ((3 1) (4 0))
    ((4 1) (7 2)) ((5 4) (0 0)) ((6 2) (5 0)) ((7 3) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-decrypt-0)
  (operation nonce-test (added-strand decrypt 4) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init"))
    (ch-msg lk-0 (cat pt-4 k "decrypt"))
    (ch-msg lk-1 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init"))
      (send (hash k)))
    ((recv (enc k k)) (recv (hash k)) (load lk-0 (cat pt-4 k "decrypt"))
      (send k)))
  (label 735)
  (parent 291)
  (unrealized (7 0) (7 1))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (0 0)) ((3 1) (4 0))
    ((4 1) (0 0)) ((5 4) (0 0)) ((6 1) (5 0)) ((7 1) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation nonce-test (added-strand make 2) k (6 0)
    (ch-msg lk (cat pt-0 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-5 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-1 (cat pt-6 old-1)) (stor lk-1 (cat pt-7 k "init"))))
  (label 736)
  (parent 293)
  (unrealized (5 3) (6 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (7 2)) ((2 1) (0 0)) ((3 1) (4 0))
    ((4 1) (0 0)) ((5 4) (0 0)) ((6 1) (5 0)) ((7 4) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (6 0)
    (ch-msg lk (cat pt-0 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-5 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((recv (hash k)) (recv (hash k1-0)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-6 k1-0 "wrap")) (send (enc k k1-0))))
  (label 738)
  (parent 293)
  (unrealized (5 3) (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((1 1) (5 2)) ((2 1) (0 0)) ((3 1) (4 0))
    ((3 1) (6 0)) ((4 1) (0 0)) ((5 4) (0 0)) ((6 1) (5 0))
    ((7 1) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation nonce-test (added-strand make 2) k (6 0)
    (ch-msg lk-0 (cat pt-3 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-5 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-1 (cat pt-6 old-1)) (stor lk-1 (cat pt-7 k "init"))))
  (label 739)
  (parent 294)
  (unrealized (5 3) (6 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk-0))
  (precedes ((1 1) (2 0)) ((1 1) (5 2)) ((2 1) (0 0)) ((3 1) (4 0))
    ((3 1) (7 2)) ((4 1) (0 0)) ((5 4) (0 0)) ((6 1) (5 0))
    ((7 4) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (6 0)
    (ch-msg lk-0 (cat pt-3 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-5 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-6 k1-0 "wrap")) (send (enc k k1-0))))
  (label 740)
  (parent 294)
  (unrealized (5 3) (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "decrypt") (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (4 3)) ((3 1) (1 0)) ((3 1) (4 2))
    ((4 4) (0 0)) ((5 1) (6 0)) ((5 2) (4 0)) ((6 1) (7 0))
    ((7 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k "wrap"))
  (facts (neq "decrypt" "decrypt") (neq "decrypt" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-decrypt-at-1 fact-set-decrypt-neq0 gen-st-set-wrap-0
    fact-set-wrap-neq0 trRl_make-at-0)
  (operation nonce-test (added-strand set-decrypt 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk (cat pt-3 k "init")) (ch-msg lk-0 (cat pt-5 k "init"))
    (ch-msg lk-0 (cat pt-6 k "decrypt")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-4 old-1)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-5 k "init")) (stor lk-0 (cat pt-6 k "decrypt")))
    ((load lk-0 (cat pt-6 k "decrypt"))
      (stor lk-0 (cat pt-7 k "decrypt"))))
  (label 776)
  (parent 306)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "wrap") (k k) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (4 3)) ((2 1) (7 0)) ((3 1) (1 0))
    ((3 1) (4 2)) ((4 4) (0 0)) ((5 1) (6 0)) ((5 2) (4 0))
    ((6 1) (0 0)) ((7 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "wrap" "decrypt") (neq "wrap" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule fact-set-decrypt-neq0 trRl_set-wrap-at-1 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation nonce-test (added-strand set-wrap 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk (cat pt-3 k "init")) (ch-msg lk-0 (cat pt-5 k "init"))
    (ch-msg lk-0 (cat pt-6 k "decrypt")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-4 old-1)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-5 k "init")) (stor lk-0 (cat pt-6 k "decrypt")))
    ((load lk (cat pt-1 k "wrap")) (stor lk (cat pt-7 k "wrap"))))
  (label 777)
  (parent 306)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "decrypt") (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (4 3)) ((3 1) (1 0)) ((3 1) (4 2))
    ((4 4) (0 0)) ((5 1) (6 0)) ((5 2) (4 0)) ((6 1) (7 0))
    ((7 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k "wrap"))
  (facts (neq "decrypt" "decrypt") (neq "decrypt" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule fact-set-decrypt-neq0 trRl_set-wrap-at-1 gen-st-set-wrap-0
    fact-set-wrap-neq0 trRl_make-at-0)
  (operation nonce-test (added-strand set-wrap 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk (cat pt-3 k "init")) (ch-msg lk-0 (cat pt-5 k "init"))
    (ch-msg lk-0 (cat pt-6 k "decrypt")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-4 old-1)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-5 k "init")) (stor lk-0 (cat pt-6 k "decrypt")))
    ((load lk-0 (cat pt-6 k "decrypt"))
      (stor lk-0 (cat pt-7 k "wrap"))))
  (label 778)
  (parent 306)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 old-2 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval) (k skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old-2) (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((2 1) (4 3)) ((3 1) (1 0)) ((3 1) (4 2))
    ((4 4) (0 0)) ((5 1) (6 0)) ((5 2) (4 0)) ((6 1) (0 0))
    ((7 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation nonce-test (added-strand make 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk (cat pt-3 k "init")) (ch-msg lk-0 (cat pt-5 k "init"))
    (ch-msg lk-0 (cat pt-6 k "decrypt")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-4 old-1)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-5 k "init")) (stor lk-0 (cat pt-6 k "decrypt")))
    ((load lk-1 (cat pt-7 old-2)) (stor lk-1 (cat pt-8 k "init"))))
  (label 779)
  (parent 306)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (4 3)) ((3 1) (4 2)) ((3 1) (7 0))
    ((4 4) (0 0)) ((5 1) (6 0)) ((5 2) (4 0)) ((6 1) (0 0))
    ((7 1) (1 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule invShearsRule shearsRule trRl_set-wrap-at-1 trRl_make-at-0)
  (operation nonce-test (added-strand set-wrap 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk (cat pt-3 k "init")) (ch-msg lk-0 (cat pt-5 k "init"))
    (ch-msg lk-0 (cat pt-6 k "decrypt")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-4 old-1)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-5 k "init")) (stor lk-0 (cat pt-6 k "decrypt")))
    ((load lk (cat pt-3 k "init")) (stor lk (cat pt-7 k "wrap"))))
  (label 780)
  (parent 306)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (4 3)) ((3 1) (1 0)) ((3 1) (4 2))
    ((4 4) (0 0)) ((5 1) (6 0)) ((5 1) (7 2)) ((5 2) (4 0))
    ((6 1) (0 0)) ((7 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk (cat pt-3 k "init")) (ch-msg lk-0 (cat pt-5 k "init"))
    (ch-msg lk-0 (cat pt-6 k "decrypt")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-4 old-1)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-5 k "init")) (stor lk-0 (cat pt-6 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-5 k "init"))
      (load lk-0 (cat pt-7 k1 "wrap")) (send (enc k k1))))
  (label 781)
  (parent 306)
  (unrealized (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (4 3)) ((3 1) (1 0)) ((3 1) (4 2))
    ((4 4) (0 0)) ((5 1) (6 0)) ((5 2) (4 0)) ((6 1) (7 2))
    ((7 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k "wrap")
    (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0 gen-st-wrap-1)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk (cat pt-3 k "init")) (ch-msg lk-0 (cat pt-5 k "init"))
    (ch-msg lk-0 (cat pt-6 k "decrypt")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-4 old-1)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-5 k "init")) (stor lk-0 (cat pt-6 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-6 k "decrypt"))
      (load lk-0 (cat pt-7 k1 "wrap")) (send (enc k k1))))
  (label 782)
  (parent 306)
  (unrealized (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "wrap") (k k) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (4 3)) ((2 1) (7 0)) ((3 1) (1 0))
    ((3 1) (4 2)) ((4 4) (0 0)) ((5 1) (6 0)) ((5 2) (4 0))
    ((6 1) (0 0)) ((7 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "wrap" "decrypt") (neq "wrap" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-decrypt-at-1 fact-set-decrypt-neq0 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation nonce-test (added-strand set-decrypt 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk (cat pt-3 k "init")) (ch-msg lk-0 (cat pt-5 k "init"))
    (ch-msg lk-0 (cat pt-6 k "decrypt")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-4 old-1)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-5 k "init")) (stor lk-0 (cat pt-6 k "decrypt")))
    ((load lk (cat pt-1 k "wrap")) (stor lk (cat pt-7 k "decrypt"))))
  (label 783)
  (parent 306)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (4 3)) ((2 1) (7 2)) ((3 1) (1 0))
    ((3 1) (4 2)) ((4 4) (0 0)) ((5 1) (6 0)) ((5 2) (4 0))
    ((6 1) (0 0)) ((7 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk (cat pt-3 k "init")) (ch-msg lk-0 (cat pt-5 k "init"))
    (ch-msg lk-0 (cat pt-6 k "decrypt")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-4 old-1)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-5 k "init")) (stor lk-0 (cat pt-6 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-7 k1 "wrap")) (send (enc k k1))))
  (label 784)
  (parent 306)
  (unrealized (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (4 3)) ((3 1) (1 0)) ((3 1) (4 2))
    ((3 1) (7 2)) ((4 4) (0 0)) ((5 1) (6 0)) ((5 2) (4 0))
    ((6 1) (0 0)) ((7 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk (cat pt-3 k "init")) (ch-msg lk-0 (cat pt-5 k "init"))
    (ch-msg lk-0 (cat pt-6 k "decrypt")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-4 old-1)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-5 k "init")) (stor lk-0 (cat pt-6 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-7 k1 "wrap")) (send (enc k k1))))
  (label 785)
  (parent 306)
  (unrealized (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (7 2)) ((2 1) (4 3)) ((3 1) (1 0))
    ((3 1) (4 2)) ((4 4) (0 0)) ((5 1) (6 0)) ((5 2) (4 0))
    ((6 1) (0 0)) ((7 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk (cat pt-3 k "init")) (ch-msg lk-0 (cat pt-5 k "init"))
    (ch-msg lk-0 (cat pt-6 k "decrypt")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-4 old-1)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-5 k "init")) (stor lk-0 (cat pt-6 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-7 k1 "wrap")) (send (enc k k1))))
  (label 786)
  (parent 306)
  (unrealized (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand decrypt 4 (x k) (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (4 3)) ((3 1) (1 0)) ((3 1) (4 2))
    ((4 4) (0 0)) ((5 1) (6 0)) ((5 2) (4 0)) ((6 1) (7 2))
    ((7 3) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-decrypt-0)
  (operation nonce-test (added-strand decrypt 4) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk (cat pt-3 k "init")) (ch-msg lk-0 (cat pt-5 k "init"))
    (ch-msg lk-0 (cat pt-6 k "decrypt")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-4 old-1)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-5 k "init")) (stor lk-0 (cat pt-6 k "decrypt")))
    ((recv (enc k k)) (recv (hash k)) (load lk-0 (cat pt-6 k "decrypt"))
      (send k)))
  (label 787)
  (parent 306)
  (unrealized (7 0) (7 1))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 old-2 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval) (k skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-0))
  (defstrand make 2 (old old-2) (k k) (lk lk-1))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((2 1) (4 3)) ((3 1) (1 0)) ((3 1) (4 2))
    ((4 4) (0 0)) ((5 2) (4 0)) ((6 1) (7 0)) ((7 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-decrypt-at-1 trRl_make-at-0)
  (operation nonce-test (added-strand set-decrypt 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk (cat pt-3 k "init")) (ch-msg lk-0 (cat pt-5 k "init"))
    (ch-msg lk-1 (cat pt-7 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-4 old-1)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k)))
    ((load lk-1 (cat pt-6 old-2)) (stor lk-1 (cat pt-7 k "init")))
    ((load lk-1 (cat pt-7 k "init"))
      (stor lk-1 (cat pt-8 k "decrypt"))))
  (label 788)
  (parent 308)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 old-2 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval) (k skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-0))
  (defstrand make 2 (old old-2) (k k) (lk lk-1))
  (defstrand set-wrap 2 (cur "wrap") (k k) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (4 3)) ((2 1) (7 0)) ((3 1) (1 0))
    ((3 1) (4 2)) ((4 4) (0 0)) ((5 2) (4 0)) ((6 1) (0 0))
    ((7 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "wrap" "decrypt") (neq "wrap" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule fact-set-decrypt-neq0 trRl_set-wrap-at-1 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation nonce-test (added-strand set-wrap 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk (cat pt-3 k "init")) (ch-msg lk-0 (cat pt-5 k "init"))
    (ch-msg lk-1 (cat pt-7 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-4 old-1)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k)))
    ((load lk-1 (cat pt-6 old-2)) (stor lk-1 (cat pt-7 k "init")))
    ((load lk (cat pt-1 k "wrap")) (stor lk (cat pt-8 k "wrap"))))
  (label 789)
  (parent 308)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 old-2 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval) (k skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-0))
  (defstrand make 2 (old old-2) (k k) (lk lk-1))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((2 1) (4 3)) ((3 1) (1 0)) ((3 1) (4 2))
    ((4 4) (0 0)) ((5 2) (4 0)) ((6 1) (7 0)) ((7 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-wrap-at-1 trRl_make-at-0)
  (operation nonce-test (added-strand set-wrap 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk (cat pt-3 k "init")) (ch-msg lk-0 (cat pt-5 k "init"))
    (ch-msg lk-1 (cat pt-7 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-4 old-1)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k)))
    ((load lk-1 (cat pt-6 old-2)) (stor lk-1 (cat pt-7 k "init")))
    ((load lk-1 (cat pt-7 k "init")) (stor lk-1 (cat pt-8 k "wrap"))))
  (label 790)
  (parent 308)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 old-2 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval) (k skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-0))
  (defstrand make 2 (old old-2) (k k) (lk lk-1))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (4 3)) ((3 1) (1 0)) ((3 1) (4 2))
    ((4 4) (0 0)) ((5 1) (7 0)) ((5 2) (4 0)) ((6 1) (0 0))
    ((7 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-wrap-at-1 trRl_make-at-0)
  (operation nonce-test (added-strand set-wrap 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk (cat pt-3 k "init")) (ch-msg lk-0 (cat pt-5 k "init"))
    (ch-msg lk-1 (cat pt-7 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-4 old-1)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k)))
    ((load lk-1 (cat pt-6 old-2)) (stor lk-1 (cat pt-7 k "init")))
    ((load lk-0 (cat pt-5 k "init")) (stor lk-0 (cat pt-8 k "wrap"))))
  (label 791)
  (parent 308)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 old-2 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval) (k skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-0))
  (defstrand make 2 (old old-2) (k k) (lk lk-1))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (4 3)) ((3 1) (4 2)) ((3 1) (7 0))
    ((4 4) (0 0)) ((5 2) (4 0)) ((6 1) (0 0)) ((7 1) (1 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule invShearsRule shearsRule trRl_set-wrap-at-1 trRl_make-at-0)
  (operation nonce-test (added-strand set-wrap 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk (cat pt-3 k "init")) (ch-msg lk-0 (cat pt-5 k "init"))
    (ch-msg lk-1 (cat pt-7 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-4 old-1)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k)))
    ((load lk-1 (cat pt-6 old-2)) (stor lk-1 (cat pt-7 k "init")))
    ((load lk (cat pt-3 k "init")) (stor lk (cat pt-8 k "wrap"))))
  (label 792)
  (parent 308)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 old-2 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-0))
  (defstrand make 2 (old old-2) (k k) (lk lk-1))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-1))
  (precedes ((1 1) (2 0)) ((2 1) (4 3)) ((3 1) (1 0)) ((3 1) (4 2))
    ((4 4) (0 0)) ((5 2) (4 0)) ((6 1) (7 2)) ((7 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk (cat pt-3 k "init")) (ch-msg lk-0 (cat pt-5 k "init"))
    (ch-msg lk-1 (cat pt-7 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-4 old-1)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k)))
    ((load lk-1 (cat pt-6 old-2)) (stor lk-1 (cat pt-7 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk-1 (cat pt-7 k "init"))
      (load lk-1 (cat pt-8 k1 "wrap")) (send (enc k k1))))
  (label 793)
  (parent 308)
  (unrealized (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 old-2 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval) (k skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-0))
  (defstrand make 2 (old old-2) (k k) (lk lk-1))
  (defstrand set-decrypt 2 (cur "wrap") (k k) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (4 3)) ((2 1) (7 0)) ((3 1) (1 0))
    ((3 1) (4 2)) ((4 4) (0 0)) ((5 2) (4 0)) ((6 1) (0 0))
    ((7 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "wrap" "decrypt") (neq "wrap" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-decrypt-at-1 fact-set-decrypt-neq0 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation nonce-test (added-strand set-decrypt 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk (cat pt-3 k "init")) (ch-msg lk-0 (cat pt-5 k "init"))
    (ch-msg lk-1 (cat pt-7 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-4 old-1)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k)))
    ((load lk-1 (cat pt-6 old-2)) (stor lk-1 (cat pt-7 k "init")))
    ((load lk (cat pt-1 k "wrap")) (stor lk (cat pt-8 k "decrypt"))))
  (label 794)
  (parent 308)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 old-2 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-0))
  (defstrand make 2 (old old-2) (k k) (lk lk-1))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (4 3)) ((3 1) (1 0)) ((3 1) (4 2))
    ((3 1) (7 2)) ((4 4) (0 0)) ((5 2) (4 0)) ((6 1) (0 0))
    ((7 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk (cat pt-3 k "init")) (ch-msg lk-0 (cat pt-5 k "init"))
    (ch-msg lk-1 (cat pt-7 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-4 old-1)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k)))
    ((load lk-1 (cat pt-6 old-2)) (stor lk-1 (cat pt-7 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-8 k1 "wrap")) (send (enc k k1))))
  (label 795)
  (parent 308)
  (unrealized (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 old-2 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-0))
  (defstrand make 2 (old old-2) (k k) (lk lk-1))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (4 3)) ((3 1) (1 0)) ((3 1) (4 2))
    ((4 4) (0 0)) ((5 1) (7 2)) ((5 2) (4 0)) ((6 1) (0 0))
    ((7 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk (cat pt-3 k "init")) (ch-msg lk-0 (cat pt-5 k "init"))
    (ch-msg lk-1 (cat pt-7 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-4 old-1)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k)))
    ((load lk-1 (cat pt-6 old-2)) (stor lk-1 (cat pt-7 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-5 k "init"))
      (load lk-0 (cat pt-8 k1 "wrap")) (send (enc k k1))))
  (label 796)
  (parent 308)
  (unrealized (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 old-2 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-0))
  (defstrand make 2 (old old-2) (k k) (lk lk-1))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (4 3)) ((2 1) (7 2)) ((3 1) (1 0))
    ((3 1) (4 2)) ((4 4) (0 0)) ((5 2) (4 0)) ((6 1) (0 0))
    ((7 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk (cat pt-3 k "init")) (ch-msg lk-0 (cat pt-5 k "init"))
    (ch-msg lk-1 (cat pt-7 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-4 old-1)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k)))
    ((load lk-1 (cat pt-6 old-2)) (stor lk-1 (cat pt-7 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-8 k1 "wrap")) (send (enc k k1))))
  (label 797)
  (parent 308)
  (unrealized (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 old-2 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-0))
  (defstrand make 2 (old old-2) (k k) (lk lk-1))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (7 2)) ((2 1) (4 3)) ((3 1) (1 0))
    ((3 1) (4 2)) ((4 4) (0 0)) ((5 2) (4 0)) ((6 1) (0 0))
    ((7 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk (cat pt-3 k "init")) (ch-msg lk-0 (cat pt-5 k "init"))
    (ch-msg lk-1 (cat pt-7 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-4 old-1)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k)))
    ((load lk-1 (cat pt-6 old-2)) (stor lk-1 (cat pt-7 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-8 k1 "wrap")) (send (enc k k1))))
  (label 798)
  (parent 308)
  (unrealized (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "wrap") (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (4 3)) ((3 1) (1 0)) ((3 1) (4 2))
    ((4 4) (0 0)) ((5 1) (6 0)) ((5 2) (4 0)) ((6 1) (7 0))
    ((7 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "wrap" "decrypt") (neq "wrap" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-decrypt-at-1 fact-set-decrypt-neq0 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation nonce-test (added-strand set-decrypt 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk (cat pt-3 k "init")) (ch-msg lk-0 (cat pt-5 k "init"))
    (ch-msg lk-0 (cat pt-6 k "wrap")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-4 old-1)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-5 k "init")) (stor lk-0 (cat pt-6 k "wrap")))
    ((load lk-0 (cat pt-6 k "wrap"))
      (stor lk-0 (cat pt-7 k "decrypt"))))
  (label 799)
  (parent 309)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "wrap") (k k) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (4 3)) ((2 1) (7 0)) ((3 1) (1 0))
    ((3 1) (4 2)) ((4 4) (0 0)) ((5 1) (6 0)) ((5 2) (4 0))
    ((6 1) (0 0)) ((7 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "wrap" "decrypt") (neq "wrap" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule fact-set-decrypt-neq0 trRl_set-wrap-at-1 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation nonce-test (added-strand set-wrap 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk (cat pt-3 k "init")) (ch-msg lk-0 (cat pt-5 k "init"))
    (ch-msg lk-0 (cat pt-6 k "wrap")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-4 old-1)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-5 k "init")) (stor lk-0 (cat pt-6 k "wrap")))
    ((load lk (cat pt-1 k "wrap")) (stor lk (cat pt-7 k "wrap"))))
  (label 800)
  (parent 309)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "wrap") (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (4 3)) ((3 1) (1 0)) ((3 1) (4 2))
    ((4 4) (0 0)) ((5 1) (6 0)) ((5 2) (4 0)) ((6 1) (7 0))
    ((7 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "wrap" "decrypt") (neq "wrap" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule fact-set-decrypt-neq0 trRl_set-wrap-at-1 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation nonce-test (added-strand set-wrap 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk (cat pt-3 k "init")) (ch-msg lk-0 (cat pt-5 k "init"))
    (ch-msg lk-0 (cat pt-6 k "wrap")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-4 old-1)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-5 k "init")) (stor lk-0 (cat pt-6 k "wrap")))
    ((load lk-0 (cat pt-6 k "wrap")) (stor lk-0 (cat pt-7 k "wrap"))))
  (label 801)
  (parent 309)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (4 3)) ((3 1) (4 2)) ((3 1) (7 0))
    ((4 4) (0 0)) ((5 1) (6 0)) ((5 2) (4 0)) ((6 1) (0 0))
    ((7 1) (1 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule invShearsRule shearsRule trRl_set-wrap-at-1 trRl_make-at-0)
  (operation nonce-test (added-strand set-wrap 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk (cat pt-3 k "init")) (ch-msg lk-0 (cat pt-5 k "init"))
    (ch-msg lk-0 (cat pt-6 k "wrap")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-4 old-1)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-5 k "init")) (stor lk-0 (cat pt-6 k "wrap")))
    ((load lk (cat pt-3 k "init")) (stor lk (cat pt-7 k "wrap"))))
  (label 802)
  (parent 309)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (4 3)) ((3 1) (1 0)) ((3 1) (4 2))
    ((4 4) (0 0)) ((5 1) (6 0)) ((5 2) (4 0)) ((6 1) (7 2))
    ((7 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk (cat pt-3 k "init")) (ch-msg lk-0 (cat pt-5 k "init"))
    (ch-msg lk-0 (cat pt-6 k "wrap")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-4 old-1)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-5 k "init")) (stor lk-0 (cat pt-6 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-6 k "wrap"))
      (load lk-0 (cat pt-7 k1 "wrap")) (send (enc k k1))))
  (label 803)
  (parent 309)
  (unrealized (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "wrap") (k k) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (4 3)) ((2 1) (7 0)) ((3 1) (1 0))
    ((3 1) (4 2)) ((4 4) (0 0)) ((5 1) (6 0)) ((5 2) (4 0))
    ((6 1) (0 0)) ((7 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "wrap" "decrypt") (neq "wrap" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-decrypt-at-1 fact-set-decrypt-neq0 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation nonce-test (added-strand set-decrypt 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk (cat pt-3 k "init")) (ch-msg lk-0 (cat pt-5 k "init"))
    (ch-msg lk-0 (cat pt-6 k "wrap")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-4 old-1)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-5 k "init")) (stor lk-0 (cat pt-6 k "wrap")))
    ((load lk (cat pt-1 k "wrap")) (stor lk (cat pt-7 k "decrypt"))))
  (label 804)
  (parent 309)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (4 3)) ((3 1) (1 0)) ((3 1) (4 2))
    ((3 1) (7 2)) ((4 4) (0 0)) ((5 1) (6 0)) ((5 2) (4 0))
    ((6 1) (0 0)) ((7 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk (cat pt-3 k "init")) (ch-msg lk-0 (cat pt-5 k "init"))
    (ch-msg lk-0 (cat pt-6 k "wrap")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-4 old-1)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-5 k "init")) (stor lk-0 (cat pt-6 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-7 k1 "wrap")) (send (enc k k1))))
  (label 805)
  (parent 309)
  (unrealized (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (4 3)) ((3 1) (1 0)) ((3 1) (4 2))
    ((4 4) (0 0)) ((5 1) (6 0)) ((5 1) (7 2)) ((5 2) (4 0))
    ((6 1) (0 0)) ((7 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk (cat pt-3 k "init")) (ch-msg lk-0 (cat pt-5 k "init"))
    (ch-msg lk-0 (cat pt-6 k "wrap")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-4 old-1)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-5 k "init")) (stor lk-0 (cat pt-6 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-5 k "init"))
      (load lk-0 (cat pt-7 k1 "wrap")) (send (enc k k1))))
  (label 806)
  (parent 309)
  (unrealized (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (4 3)) ((2 1) (7 2)) ((3 1) (1 0))
    ((3 1) (4 2)) ((4 4) (0 0)) ((5 1) (6 0)) ((5 2) (4 0))
    ((6 1) (0 0)) ((7 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk (cat pt-3 k "init")) (ch-msg lk-0 (cat pt-5 k "init"))
    (ch-msg lk-0 (cat pt-6 k "wrap")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-4 old-1)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-5 k "init")) (stor lk-0 (cat pt-6 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-7 k1 "wrap")) (send (enc k k1))))
  (label 807)
  (parent 309)
  (unrealized (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (7 2)) ((2 1) (4 3)) ((3 1) (1 0))
    ((3 1) (4 2)) ((4 4) (0 0)) ((5 1) (6 0)) ((5 2) (4 0))
    ((6 1) (0 0)) ((7 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk (cat pt-3 k "init")) (ch-msg lk-0 (cat pt-5 k "init"))
    (ch-msg lk-0 (cat pt-6 k "wrap")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-4 old-1)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-5 k "init")) (stor lk-0 (cat pt-6 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-7 k1 "wrap")) (send (enc k k1))))
  (label 808)
  (parent 309)
  (unrealized (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand set-wrap 2 (cur "wrap") (k k) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (4 3)) ((2 1) (7 0)) ((3 1) (4 2))
    ((3 1) (6 0)) ((4 4) (0 0)) ((5 2) (4 0)) ((6 1) (1 0))
    ((7 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "wrap" "decrypt") (neq "wrap" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule fact-set-decrypt-neq0 trRl_set-wrap-at-1 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation nonce-test (added-strand set-wrap 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk (cat pt-3 k "init")) (ch-msg lk (cat pt-6 k "wrap"))
    (ch-msg lk-0 (cat pt-5 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-4 old-1)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k)))
    ((load lk (cat pt-3 k "init")) (stor lk (cat pt-6 k "wrap")))
    ((load lk (cat pt-1 k "wrap")) (stor lk (cat pt-7 k "wrap"))))
  (label 809)
  (parent 310)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (4 3)) ((3 1) (4 2)) ((3 1) (6 0))
    ((4 4) (0 0)) ((5 1) (7 2)) ((5 2) (4 0)) ((6 1) (1 0))
    ((7 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk (cat pt-3 k "init")) (ch-msg lk (cat pt-6 k "wrap"))
    (ch-msg lk-0 (cat pt-5 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-4 old-1)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k)))
    ((load lk (cat pt-3 k "init")) (stor lk (cat pt-6 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-5 k "init"))
      (load lk-0 (cat pt-7 k1 "wrap")) (send (enc k k1))))
  (label 810)
  (parent 310)
  (unrealized (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "wrap") (k k) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (4 3)) ((2 1) (7 0)) ((3 1) (4 2))
    ((3 1) (6 0)) ((4 4) (0 0)) ((5 2) (4 0)) ((6 1) (1 0))
    ((7 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "wrap" "decrypt") (neq "wrap" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-decrypt-at-1 fact-set-decrypt-neq0 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation nonce-test (added-strand set-decrypt 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk (cat pt-3 k "init")) (ch-msg lk (cat pt-6 k "wrap"))
    (ch-msg lk-0 (cat pt-5 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-4 old-1)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k)))
    ((load lk (cat pt-3 k "init")) (stor lk (cat pt-6 k "wrap")))
    ((load lk (cat pt-1 k "wrap")) (stor lk (cat pt-7 k "decrypt"))))
  (label 811)
  (parent 310)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (4 3)) ((3 1) (4 2)) ((3 1) (6 0))
    ((3 1) (7 2)) ((4 4) (0 0)) ((5 2) (4 0)) ((6 1) (1 0))
    ((7 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk (cat pt-3 k "init")) (ch-msg lk (cat pt-6 k "wrap"))
    (ch-msg lk-0 (cat pt-5 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-4 old-1)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k)))
    ((load lk (cat pt-3 k "init")) (stor lk (cat pt-6 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-7 k1 "wrap")) (send (enc k k1))))
  (label 812)
  (parent 310)
  (unrealized (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (4 3)) ((3 1) (4 2)) ((3 1) (6 0))
    ((4 4) (0 0)) ((5 2) (4 0)) ((6 1) (1 0)) ((6 1) (7 2))
    ((7 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk (cat pt-3 k "init")) (ch-msg lk (cat pt-6 k "wrap"))
    (ch-msg lk-0 (cat pt-5 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-4 old-1)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k)))
    ((load lk (cat pt-3 k "init")) (stor lk (cat pt-6 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-6 k "wrap"))
      (load lk (cat pt-7 k1 "wrap")) (send (enc k k1))))
  (label 813)
  (parent 310)
  (unrealized (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (4 3)) ((2 1) (7 2)) ((3 1) (4 2))
    ((3 1) (6 0)) ((4 4) (0 0)) ((5 2) (4 0)) ((6 1) (1 0))
    ((7 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk (cat pt-3 k "init")) (ch-msg lk (cat pt-6 k "wrap"))
    (ch-msg lk-0 (cat pt-5 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-4 old-1)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k)))
    ((load lk (cat pt-3 k "init")) (stor lk (cat pt-6 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-7 k1 "wrap")) (send (enc k k1))))
  (label 814)
  (parent 310)
  (unrealized (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (7 2)) ((2 1) (4 3)) ((3 1) (4 2))
    ((3 1) (6 0)) ((4 4) (0 0)) ((5 2) (4 0)) ((6 1) (1 0))
    ((7 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk (cat pt-3 k "init")) (ch-msg lk (cat pt-6 k "wrap"))
    (ch-msg lk-0 (cat pt-5 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-4 old-1)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k)))
    ((load lk (cat pt-3 k "init")) (stor lk (cat pt-6 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-7 k1 "wrap")) (send (enc k k1))))
  (label 815)
  (parent 310)
  (unrealized (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 old-2 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old-2) (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((2 1) (4 3)) ((3 1) (1 0)) ((3 1) (4 2))
    ((3 1) (6 2)) ((4 4) (0 0)) ((5 2) (4 0)) ((6 4) (0 0))
    ((7 2) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation encryption-test (added-strand make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-4 old-1)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-6 k1 "wrap")) (send (enc k k1)))
    ((load lk-1 (cat pt-7 old-2)) (stor lk-1 (cat pt-8 k "init"))
      (send (hash k))))
  (label 818)
  (parent 312)
  (unrealized (6 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (precedes ((1 1) (2 0)) ((2 1) (4 3)) ((3 1) (1 0)) ((3 1) (4 2))
    ((3 1) (6 2)) ((4 4) (0 0)) ((5 2) (4 0)) ((6 4) (0 0))
    ((7 1) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (added-listener k) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-4 old-1)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-6 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k)))
  (label 819)
  (parent 312)
  (unrealized (6 3) (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 old-2 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (defstrand make 3 (old old-2) (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((2 1) (4 3)) ((3 1) (1 0)) ((3 1) (4 2))
    ((4 4) (0 0)) ((5 1) (6 2)) ((5 2) (4 0)) ((6 4) (0 0))
    ((7 2) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation encryption-test (added-strand make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-4 old-1)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-5 k "init"))
      (load lk-0 (cat pt-6 k1 "wrap")) (send (enc k k1)))
    ((load lk-1 (cat pt-7 old-2)) (stor lk-1 (cat pt-8 k "init"))
      (send (hash k))))
  (label 823)
  (parent 313)
  (unrealized (6 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (deflistener k)
  (precedes ((1 1) (2 0)) ((2 1) (4 3)) ((3 1) (1 0)) ((3 1) (4 2))
    ((4 4) (0 0)) ((5 1) (6 2)) ((5 2) (4 0)) ((6 4) (0 0))
    ((7 1) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (added-listener k) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-4 old-1)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-5 k "init"))
      (load lk-0 (cat pt-6 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k)))
  (label 824)
  (parent 313)
  (unrealized (6 3) (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 old-2 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old-2) (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((2 1) (4 3)) ((2 1) (6 2)) ((3 1) (1 0))
    ((3 1) (4 2)) ((4 4) (0 0)) ((5 2) (4 0)) ((6 4) (0 0))
    ((7 2) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation encryption-test (added-strand make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-4 old-1)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-6 k1 "wrap")) (send (enc k k1)))
    ((load lk-1 (cat pt-7 old-2)) (stor lk-1 (cat pt-8 k "init"))
      (send (hash k))))
  (label 828)
  (parent 314)
  (unrealized (6 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (precedes ((1 1) (2 0)) ((2 1) (4 3)) ((2 1) (6 2)) ((3 1) (1 0))
    ((3 1) (4 2)) ((4 4) (0 0)) ((5 2) (4 0)) ((6 4) (0 0))
    ((7 1) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (added-listener k) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-4 old-1)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-6 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k)))
  (label 829)
  (parent 314)
  (unrealized (6 3) (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 old-2 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old-2) (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((1 1) (6 2)) ((2 1) (4 3)) ((3 1) (1 0))
    ((3 1) (4 2)) ((4 4) (0 0)) ((5 2) (4 0)) ((6 4) (0 0))
    ((7 2) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation encryption-test (added-strand make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-4 old-1)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-6 k1 "wrap")) (send (enc k k1)))
    ((load lk-1 (cat pt-7 old-2)) (stor lk-1 (cat pt-8 k "init"))
      (send (hash k))))
  (label 833)
  (parent 315)
  (unrealized (6 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (precedes ((1 1) (2 0)) ((1 1) (6 2)) ((2 1) (4 3)) ((3 1) (1 0))
    ((3 1) (4 2)) ((4 4) (0 0)) ((5 2) (4 0)) ((6 4) (0 0))
    ((7 1) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (added-listener k) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-4 old-1)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-6 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k)))
  (label 834)
  (parent 315)
  (unrealized (6 3) (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "wrap") (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (0 0)) ((3 1) (4 2)) ((3 1) (6 0))
    ((4 4) (0 0)) ((5 2) (4 0)) ((6 1) (4 3)) ((6 1) (7 0))
    ((7 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "wrap" "decrypt") (neq "wrap" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-decrypt-at-1 fact-set-decrypt-neq0 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation nonce-test (added-strand set-decrypt 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init")) (ch-msg lk-0 (cat pt-4 k "wrap"))
    (ch-msg lk-1 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-4 k "wrap")) (send (enc k k)))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "wrap")))
    ((load lk-0 (cat pt-4 k "wrap"))
      (stor lk-0 (cat pt-7 k "decrypt"))))
  (label 835)
  (parent 316)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((2 1) (0 0)) ((3 1) (4 2)) ((3 1) (6 0))
    ((4 4) (0 0)) ((5 1) (7 0)) ((5 2) (4 0)) ((6 1) (4 3))
    ((7 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-decrypt-at-1 trRl_make-at-0)
  (operation nonce-test (added-strand set-decrypt 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init")) (ch-msg lk-0 (cat pt-4 k "wrap"))
    (ch-msg lk-1 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-4 k "wrap")) (send (enc k k)))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "wrap")))
    ((load lk-1 (cat pt-6 k "init"))
      (stor lk-1 (cat pt-7 k "decrypt"))))
  (label 836)
  (parent 316)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "wrap") (k k) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (7 0)) ((3 1) (4 2)) ((3 1) (6 0))
    ((4 4) (0 0)) ((5 2) (4 0)) ((6 1) (4 3)) ((7 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "wrap" "decrypt") (neq "wrap" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule fact-set-decrypt-neq0 trRl_set-wrap-at-1 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation nonce-test (added-strand set-wrap 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init")) (ch-msg lk-0 (cat pt-4 k "wrap"))
    (ch-msg lk-1 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-4 k "wrap")) (send (enc k k)))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "wrap")))
    ((load lk (cat pt-1 k "wrap")) (stor lk (cat pt-7 k "wrap"))))
  (label 837)
  (parent 316)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 old-2 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval) (k skey)
    (lk lk-0 lk-1 lk-2 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old-2) (k k) (lk lk-2))
  (precedes ((1 1) (2 0)) ((2 1) (0 0)) ((3 1) (4 2)) ((3 1) (6 0))
    ((4 4) (0 0)) ((5 2) (4 0)) ((6 1) (4 3)) ((7 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation nonce-test (added-strand make 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init")) (ch-msg lk-0 (cat pt-4 k "wrap"))
    (ch-msg lk-1 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-4 k "wrap")) (send (enc k k)))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "wrap")))
    ((load lk-2 (cat pt-7 old-2)) (stor lk-2 (cat pt-8 k "init"))))
  (label 838)
  (parent 316)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((2 1) (0 0)) ((3 1) (4 2)) ((3 1) (6 0))
    ((4 4) (0 0)) ((5 1) (7 0)) ((5 2) (4 0)) ((6 1) (4 3))
    ((7 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-wrap-at-1 trRl_make-at-0)
  (operation nonce-test (added-strand set-wrap 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init")) (ch-msg lk-0 (cat pt-4 k "wrap"))
    (ch-msg lk-1 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-4 k "wrap")) (send (enc k k)))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "wrap")))
    ((load lk-1 (cat pt-6 k "init")) (stor lk-1 (cat pt-7 k "wrap"))))
  (label 839)
  (parent 316)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "wrap") (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (0 0)) ((3 1) (4 2)) ((3 1) (6 0))
    ((4 4) (0 0)) ((5 2) (4 0)) ((6 1) (4 3)) ((6 1) (7 0))
    ((7 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "wrap" "decrypt") (neq "wrap" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule fact-set-decrypt-neq0 trRl_set-wrap-at-1 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation nonce-test (added-strand set-wrap 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init")) (ch-msg lk-0 (cat pt-4 k "wrap"))
    (ch-msg lk-1 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-4 k "wrap")) (send (enc k k)))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "wrap")))
    ((load lk-0 (cat pt-4 k "wrap")) (stor lk-0 (cat pt-7 k "wrap"))))
  (label 840)
  (parent 316)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-1))
  (precedes ((1 1) (2 0)) ((2 1) (0 0)) ((3 1) (4 2)) ((3 1) (6 0))
    ((4 4) (0 0)) ((5 1) (7 2)) ((5 2) (4 0)) ((6 1) (4 3))
    ((7 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init")) (ch-msg lk-0 (cat pt-4 k "wrap"))
    (ch-msg lk-1 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-4 k "wrap")) (send (enc k k)))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk-1 (cat pt-6 k "init"))
      (load lk-1 (cat pt-7 k1 "wrap")) (send (enc k k1))))
  (label 841)
  (parent 316)
  (unrealized (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "wrap") (k k) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (7 0)) ((3 1) (4 2)) ((3 1) (6 0))
    ((4 4) (0 0)) ((5 2) (4 0)) ((6 1) (4 3)) ((7 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "wrap" "decrypt") (neq "wrap" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-decrypt-at-1 fact-set-decrypt-neq0 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation nonce-test (added-strand set-decrypt 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init")) (ch-msg lk-0 (cat pt-4 k "wrap"))
    (ch-msg lk-1 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-4 k "wrap")) (send (enc k k)))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "wrap")))
    ((load lk (cat pt-1 k "wrap")) (stor lk (cat pt-7 k "decrypt"))))
  (label 842)
  (parent 316)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (0 0)) ((3 1) (4 2)) ((3 1) (6 0))
    ((3 1) (7 2)) ((4 4) (0 0)) ((5 2) (4 0)) ((6 1) (4 3))
    ((7 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init")) (ch-msg lk-0 (cat pt-4 k "wrap"))
    (ch-msg lk-1 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-4 k "wrap")) (send (enc k k)))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-7 k1 "wrap")) (send (enc k k1))))
  (label 843)
  (parent 316)
  (unrealized (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (0 0)) ((3 1) (4 2)) ((3 1) (6 0))
    ((4 4) (0 0)) ((5 2) (4 0)) ((6 1) (4 3)) ((6 1) (7 2))
    ((7 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init")) (ch-msg lk-0 (cat pt-4 k "wrap"))
    (ch-msg lk-1 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-4 k "wrap")) (send (enc k k)))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-4 k "wrap"))
      (load lk-0 (cat pt-7 k1 "wrap")) (send (enc k k1))))
  (label 844)
  (parent 316)
  (unrealized (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (7 2)) ((3 1) (4 2)) ((3 1) (6 0))
    ((4 4) (0 0)) ((5 2) (4 0)) ((6 1) (4 3)) ((7 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init")) (ch-msg lk-0 (cat pt-4 k "wrap"))
    (ch-msg lk-1 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-4 k "wrap")) (send (enc k k)))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-7 k1 "wrap")) (send (enc k k1))))
  (label 845)
  (parent 316)
  (unrealized (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk-0))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (7 2)) ((2 1) (0 0)) ((3 1) (4 2))
    ((3 1) (6 0)) ((4 4) (0 0)) ((5 2) (4 0)) ((6 1) (4 3))
    ((7 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init")) (ch-msg lk-0 (cat pt-4 k "wrap"))
    (ch-msg lk-1 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-4 k "wrap")) (send (enc k k)))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-7 k1 "wrap")) (send (enc k k1))))
  (label 846)
  (parent 316)
  (unrealized (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (deflistener k)
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (0 0)) ((3 1) (4 2))
    ((4 4) (0 0)) ((5 1) (4 0)) ((6 1) (7 0)) ((7 1) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-wrap-at-1 trRl_make-at-0)
  (operation nonce-test (added-strand set-wrap 2) k (5 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk-1 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init")))
    ((load lk-1 (cat pt-6 k "init")) (stor lk-1 (cat pt-7 k "wrap"))))
  (label 848)
  (parent 318)
  (unrealized (4 3) (5 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (deflistener k)
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (0 0)) ((3 1) (4 2))
    ((4 4) (0 0)) ((5 1) (4 0)) ((6 1) (7 0)) ((7 1) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-decrypt-at-1 trRl_make-at-0)
  (operation nonce-test (added-strand set-decrypt 2) k (5 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk-1 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init")))
    ((load lk-1 (cat pt-6 k "init"))
      (stor lk-1 (cat pt-7 k "decrypt"))))
  (label 849)
  (parent 318)
  (unrealized (4 3) (5 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 k1-0 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (deflistener k)
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk-1))
  (precedes ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (0 0)) ((3 1) (4 2))
    ((4 4) (0 0)) ((5 1) (4 0)) ((6 1) (7 2)) ((7 4) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (5 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk-1 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-1 (cat pt-6 k "init"))
      (load lk-1 (cat pt-7 k1-0 "wrap")) (send (enc k k1-0))))
  (label 850)
  (parent 318)
  (unrealized (4 3) (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 k1-0 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (deflistener k)
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (7 2)) ((2 1) (0 0)) ((3 1) (4 2))
    ((4 4) (0 0)) ((5 1) (4 0)) ((6 1) (5 0)) ((7 4) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (5 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk-1 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-7 k1-0 "wrap")) (send (enc k k1-0))))
  (label 851)
  (parent 318)
  (unrealized (4 3) (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 k1-0 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (deflistener k)
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((1 1) (6 2)) ((2 1) (0 0)) ((3 1) (4 2))
    ((4 4) (0 0)) ((5 1) (4 0)) ((6 4) (5 0)) ((7 2) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation encryption-test (added-strand make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((recv (hash k)) (recv (hash k1-0)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-5 k1-0 "wrap")) (send (enc k k1-0)))
    ((load lk-1 (cat pt-6 old-1)) (stor lk-1 (cat pt-7 k "init"))
      (send (hash k))))
  (label 856)
  (parent 320)
  (unrealized (4 3) (6 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (deflistener k)
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk))
  (deflistener k)
  (precedes ((1 1) (2 0)) ((1 1) (6 2)) ((2 1) (0 0)) ((3 1) (4 2))
    ((4 4) (0 0)) ((5 1) (4 0)) ((6 4) (5 0)) ((7 1) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (added-listener k) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((recv (hash k)) (recv (hash k1-0)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-5 k1-0 "wrap")) (send (enc k k1-0)))
    ((recv k) (send k)))
  (label 857)
  (parent 320)
  (unrealized (4 3) (6 3) (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (deflistener k)
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((2 1) (0 0)) ((3 1) (5 0)) ((4 4) (0 0))
    ((5 1) (4 0)) ((6 1) (7 0)) ((7 1) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-wrap-at-1 trRl_make-at-0)
  (operation nonce-test (added-strand set-wrap 2) k (5 0)
    (ch-msg lk-0 (cat pt-3 k "init")) (ch-msg lk-1 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init")))
    ((load lk-1 (cat pt-6 k "init")) (stor lk-1 (cat pt-7 k "wrap"))))
  (label 858)
  (parent 321)
  (unrealized (4 3) (5 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (deflistener k)
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((2 1) (0 0)) ((3 1) (5 0)) ((4 4) (0 0))
    ((5 1) (4 0)) ((6 1) (7 0)) ((7 1) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-decrypt-at-1 trRl_make-at-0)
  (operation nonce-test (added-strand set-decrypt 2) k (5 0)
    (ch-msg lk-0 (cat pt-3 k "init")) (ch-msg lk-1 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init")))
    ((load lk-1 (cat pt-6 k "init"))
      (stor lk-1 (cat pt-7 k "decrypt"))))
  (label 859)
  (parent 321)
  (unrealized (4 3) (5 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 k1-0 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (deflistener k)
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk-1))
  (precedes ((1 1) (2 0)) ((2 1) (0 0)) ((3 1) (5 0)) ((4 4) (0 0))
    ((5 1) (4 0)) ((6 1) (7 2)) ((7 4) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (5 0)
    (ch-msg lk-0 (cat pt-3 k "init")) (ch-msg lk-1 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-1 (cat pt-6 k "init"))
      (load lk-1 (cat pt-7 k1-0 "wrap")) (send (enc k k1-0))))
  (label 860)
  (parent 321)
  (unrealized (4 3) (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 k1-0 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (deflistener k)
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (0 0)) ((3 1) (7 2)) ((4 4) (0 0))
    ((5 1) (4 0)) ((6 1) (5 0)) ((7 4) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (5 0)
    (ch-msg lk-0 (cat pt-3 k "init")) (ch-msg lk-1 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-7 k1-0 "wrap")) (send (enc k k1-0))))
  (label 861)
  (parent 321)
  (unrealized (4 3) (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 k1-0 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (deflistener k)
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk-0))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((2 1) (0 0)) ((3 1) (6 2)) ((4 4) (0 0))
    ((5 1) (4 0)) ((6 4) (5 0)) ((7 2) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation encryption-test (added-strand make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-5 k1-0 "wrap")) (send (enc k k1-0)))
    ((load lk-1 (cat pt-6 old-1)) (stor lk-1 (cat pt-7 k "init"))
      (send (hash k))))
  (label 864)
  (parent 322)
  (unrealized (4 3) (6 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (deflistener k)
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk-0))
  (deflistener k)
  (precedes ((1 1) (2 0)) ((2 1) (0 0)) ((3 1) (6 2)) ((4 4) (0 0))
    ((5 1) (4 0)) ((6 4) (5 0)) ((7 1) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (added-listener k) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-5 k1-0 "wrap")) (send (enc k k1-0)))
    ((recv k) (send k)))
  (label 865)
  (parent 322)
  (unrealized (4 3) (6 3) (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-1))
  (defstrand set-decrypt 2 (cur "decrypt") (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((2 1) (4 2)) ((3 1) (0 0)) ((4 4) (0 0))
    ((5 1) (6 0)) ((5 2) (4 0)) ((6 1) (7 0)) ((7 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k "wrap"))
  (facts (neq "decrypt" "decrypt") (neq "decrypt" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-decrypt-at-1 fact-set-decrypt-neq0 gen-st-set-wrap-0
    fact-set-wrap-neq0 trRl_make-at-0)
  (operation nonce-test (added-strand set-decrypt 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init")) (ch-msg lk-1 (cat pt-5 k "init"))
    (ch-msg lk-1 (cat pt-6 k "decrypt")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-1 (cat pt-4 old-1)) (stor lk-1 (cat pt-5 k "init"))
      (send (hash k)))
    ((load lk-1 (cat pt-5 k "init")) (stor lk-1 (cat pt-6 k "decrypt")))
    ((load lk-1 (cat pt-6 k "decrypt"))
      (stor lk-1 (cat pt-7 k "decrypt"))))
  (label 892)
  (parent 334)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-1))
  (defstrand set-wrap 2 (cur "wrap") (k k) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (4 2)) ((2 1) (7 0)) ((3 1) (0 0))
    ((4 4) (0 0)) ((5 1) (6 0)) ((5 2) (4 0)) ((6 1) (0 0))
    ((7 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "wrap" "decrypt") (neq "wrap" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule fact-set-decrypt-neq0 trRl_set-wrap-at-1 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation nonce-test (added-strand set-wrap 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init")) (ch-msg lk-1 (cat pt-5 k "init"))
    (ch-msg lk-1 (cat pt-6 k "decrypt")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-1 (cat pt-4 old-1)) (stor lk-1 (cat pt-5 k "init"))
      (send (hash k)))
    ((load lk-1 (cat pt-5 k "init")) (stor lk-1 (cat pt-6 k "decrypt")))
    ((load lk (cat pt-1 k "wrap")) (stor lk (cat pt-7 k "wrap"))))
  (label 893)
  (parent 334)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-1))
  (defstrand set-wrap 2 (cur "decrypt") (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((2 1) (4 2)) ((3 1) (0 0)) ((4 4) (0 0))
    ((5 1) (6 0)) ((5 2) (4 0)) ((6 1) (7 0)) ((7 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k "wrap"))
  (facts (neq "decrypt" "decrypt") (neq "decrypt" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule fact-set-decrypt-neq0 trRl_set-wrap-at-1 gen-st-set-wrap-0
    fact-set-wrap-neq0 trRl_make-at-0)
  (operation nonce-test (added-strand set-wrap 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init")) (ch-msg lk-1 (cat pt-5 k "init"))
    (ch-msg lk-1 (cat pt-6 k "decrypt")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-1 (cat pt-4 old-1)) (stor lk-1 (cat pt-5 k "init"))
      (send (hash k)))
    ((load lk-1 (cat pt-5 k "init")) (stor lk-1 (cat pt-6 k "decrypt")))
    ((load lk-1 (cat pt-6 k "decrypt"))
      (stor lk-1 (cat pt-7 k "wrap"))))
  (label 894)
  (parent 334)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-1))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (4 2)) ((3 1) (7 0)) ((4 4) (0 0))
    ((5 1) (6 0)) ((5 2) (4 0)) ((6 1) (0 0)) ((7 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-wrap-at-1 trRl_make-at-0)
  (operation nonce-test (added-strand set-wrap 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init")) (ch-msg lk-1 (cat pt-5 k "init"))
    (ch-msg lk-1 (cat pt-6 k "decrypt")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-1 (cat pt-4 old-1)) (stor lk-1 (cat pt-5 k "init"))
      (send (hash k)))
    ((load lk-1 (cat pt-5 k "init")) (stor lk-1 (cat pt-6 k "decrypt")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-7 k "wrap"))))
  (label 895)
  (parent 334)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-1))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-1))
  (precedes ((1 1) (2 0)) ((2 1) (4 2)) ((3 1) (0 0)) ((4 4) (0 0))
    ((5 1) (6 0)) ((5 1) (7 2)) ((5 2) (4 0)) ((6 1) (0 0))
    ((7 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init")) (ch-msg lk-1 (cat pt-5 k "init"))
    (ch-msg lk-1 (cat pt-6 k "decrypt")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-1 (cat pt-4 old-1)) (stor lk-1 (cat pt-5 k "init"))
      (send (hash k)))
    ((load lk-1 (cat pt-5 k "init")) (stor lk-1 (cat pt-6 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk-1 (cat pt-5 k "init"))
      (load lk-1 (cat pt-7 k1 "wrap")) (send (enc k k1))))
  (label 896)
  (parent 334)
  (unrealized (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-1))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1) (lk lk-1))
  (precedes ((1 1) (2 0)) ((2 1) (4 2)) ((3 1) (0 0)) ((4 4) (0 0))
    ((5 1) (6 0)) ((5 2) (4 0)) ((6 1) (7 2)) ((7 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k "wrap")
    (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0 gen-st-wrap-1)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init")) (ch-msg lk-1 (cat pt-5 k "init"))
    (ch-msg lk-1 (cat pt-6 k "decrypt")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-1 (cat pt-4 old-1)) (stor lk-1 (cat pt-5 k "init"))
      (send (hash k)))
    ((load lk-1 (cat pt-5 k "init")) (stor lk-1 (cat pt-6 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk-1 (cat pt-6 k "decrypt"))
      (load lk-1 (cat pt-7 k1 "wrap")) (send (enc k k1))))
  (label 897)
  (parent 334)
  (unrealized (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-1))
  (defstrand set-decrypt 2 (cur "wrap") (k k) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (4 2)) ((2 1) (7 0)) ((3 1) (0 0))
    ((4 4) (0 0)) ((5 1) (6 0)) ((5 2) (4 0)) ((6 1) (0 0))
    ((7 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "wrap" "decrypt") (neq "wrap" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-decrypt-at-1 fact-set-decrypt-neq0 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation nonce-test (added-strand set-decrypt 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init")) (ch-msg lk-1 (cat pt-5 k "init"))
    (ch-msg lk-1 (cat pt-6 k "decrypt")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-1 (cat pt-4 old-1)) (stor lk-1 (cat pt-5 k "init"))
      (send (hash k)))
    ((load lk-1 (cat pt-5 k "init")) (stor lk-1 (cat pt-6 k "decrypt")))
    ((load lk (cat pt-1 k "wrap")) (stor lk (cat pt-7 k "decrypt"))))
  (label 898)
  (parent 334)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-1))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (4 2)) ((2 1) (7 2)) ((3 1) (0 0))
    ((4 4) (0 0)) ((5 1) (6 0)) ((5 2) (4 0)) ((6 1) (0 0))
    ((7 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init")) (ch-msg lk-1 (cat pt-5 k "init"))
    (ch-msg lk-1 (cat pt-6 k "decrypt")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-1 (cat pt-4 old-1)) (stor lk-1 (cat pt-5 k "init"))
      (send (hash k)))
    ((load lk-1 (cat pt-5 k "init")) (stor lk-1 (cat pt-6 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-7 k1 "wrap")) (send (enc k k1))))
  (label 899)
  (parent 334)
  (unrealized (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-1))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (4 2)) ((3 1) (7 2)) ((4 4) (0 0))
    ((5 1) (6 0)) ((5 2) (4 0)) ((6 1) (0 0)) ((7 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init")) (ch-msg lk-1 (cat pt-5 k "init"))
    (ch-msg lk-1 (cat pt-6 k "decrypt")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-1 (cat pt-4 old-1)) (stor lk-1 (cat pt-5 k "init"))
      (send (hash k)))
    ((load lk-1 (cat pt-5 k "init")) (stor lk-1 (cat pt-6 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-7 k1 "wrap")) (send (enc k k1))))
  (label 900)
  (parent 334)
  (unrealized (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-1))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (7 2)) ((2 1) (4 2)) ((3 1) (0 0))
    ((4 4) (0 0)) ((5 1) (6 0)) ((5 2) (4 0)) ((6 1) (0 0))
    ((7 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init")) (ch-msg lk-1 (cat pt-5 k "init"))
    (ch-msg lk-1 (cat pt-6 k "decrypt")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-1 (cat pt-4 old-1)) (stor lk-1 (cat pt-5 k "init"))
      (send (hash k)))
    ((load lk-1 (cat pt-5 k "init")) (stor lk-1 (cat pt-6 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-7 k1 "wrap")) (send (enc k k1))))
  (label 901)
  (parent 334)
  (unrealized (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-1))
  (defstrand decrypt 4 (x k) (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((2 1) (4 2)) ((3 1) (0 0)) ((4 4) (0 0))
    ((5 1) (6 0)) ((5 2) (4 0)) ((6 1) (7 2)) ((7 3) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-decrypt-0)
  (operation nonce-test (added-strand decrypt 4) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init")) (ch-msg lk-1 (cat pt-5 k "init"))
    (ch-msg lk-1 (cat pt-6 k "decrypt")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-1 (cat pt-4 old-1)) (stor lk-1 (cat pt-5 k "init"))
      (send (hash k)))
    ((load lk-1 (cat pt-5 k "init")) (stor lk-1 (cat pt-6 k "decrypt")))
    ((recv (enc k k)) (recv (hash k)) (load lk-1 (cat pt-6 k "decrypt"))
      (send k)))
  (label 902)
  (parent 334)
  (unrealized (7 0) (7 1))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-1))
  (defstrand set-decrypt 2 (cur "wrap") (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((2 1) (4 2)) ((3 1) (0 0)) ((4 4) (0 0))
    ((5 1) (6 0)) ((5 2) (4 0)) ((6 1) (7 0)) ((7 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "wrap" "decrypt") (neq "wrap" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-decrypt-at-1 fact-set-decrypt-neq0 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation nonce-test (added-strand set-decrypt 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init")) (ch-msg lk-1 (cat pt-5 k "init"))
    (ch-msg lk-1 (cat pt-6 k "wrap")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-1 (cat pt-4 old-1)) (stor lk-1 (cat pt-5 k "init"))
      (send (hash k)))
    ((load lk-1 (cat pt-5 k "init")) (stor lk-1 (cat pt-6 k "wrap")))
    ((load lk-1 (cat pt-6 k "wrap"))
      (stor lk-1 (cat pt-7 k "decrypt"))))
  (label 903)
  (parent 335)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-1))
  (defstrand set-wrap 2 (cur "wrap") (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((2 1) (4 2)) ((3 1) (0 0)) ((4 4) (0 0))
    ((5 1) (6 0)) ((5 2) (4 0)) ((6 1) (7 0)) ((7 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "wrap" "decrypt") (neq "wrap" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule fact-set-decrypt-neq0 trRl_set-wrap-at-1 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation nonce-test (added-strand set-wrap 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init")) (ch-msg lk-1 (cat pt-5 k "init"))
    (ch-msg lk-1 (cat pt-6 k "wrap")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-1 (cat pt-4 old-1)) (stor lk-1 (cat pt-5 k "init"))
      (send (hash k)))
    ((load lk-1 (cat pt-5 k "init")) (stor lk-1 (cat pt-6 k "wrap")))
    ((load lk-1 (cat pt-6 k "wrap")) (stor lk-1 (cat pt-7 k "wrap"))))
  (label 904)
  (parent 335)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-1))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (4 2)) ((3 1) (7 0)) ((4 4) (0 0))
    ((5 1) (6 0)) ((5 2) (4 0)) ((6 1) (0 0)) ((7 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-wrap-at-1 trRl_make-at-0)
  (operation nonce-test (added-strand set-wrap 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init")) (ch-msg lk-1 (cat pt-5 k "init"))
    (ch-msg lk-1 (cat pt-6 k "wrap")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-1 (cat pt-4 old-1)) (stor lk-1 (cat pt-5 k "init"))
      (send (hash k)))
    ((load lk-1 (cat pt-5 k "init")) (stor lk-1 (cat pt-6 k "wrap")))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-7 k "wrap"))))
  (label 905)
  (parent 335)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-1))
  (defstrand set-wrap 2 (cur "wrap") (k k) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (4 2)) ((2 1) (7 0)) ((3 1) (0 0))
    ((4 4) (0 0)) ((5 1) (6 0)) ((5 2) (4 0)) ((6 1) (0 0))
    ((7 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "wrap" "decrypt") (neq "wrap" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule fact-set-decrypt-neq0 trRl_set-wrap-at-1 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation nonce-test (added-strand set-wrap 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init")) (ch-msg lk-1 (cat pt-5 k "init"))
    (ch-msg lk-1 (cat pt-6 k "wrap")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-1 (cat pt-4 old-1)) (stor lk-1 (cat pt-5 k "init"))
      (send (hash k)))
    ((load lk-1 (cat pt-5 k "init")) (stor lk-1 (cat pt-6 k "wrap")))
    ((load lk (cat pt-1 k "wrap")) (stor lk (cat pt-7 k "wrap"))))
  (label 906)
  (parent 335)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-1))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk-1))
  (precedes ((1 1) (2 0)) ((2 1) (4 2)) ((3 1) (0 0)) ((4 4) (0 0))
    ((5 1) (6 0)) ((5 2) (4 0)) ((6 1) (7 2)) ((7 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init")) (ch-msg lk-1 (cat pt-5 k "init"))
    (ch-msg lk-1 (cat pt-6 k "wrap")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-1 (cat pt-4 old-1)) (stor lk-1 (cat pt-5 k "init"))
      (send (hash k)))
    ((load lk-1 (cat pt-5 k "init")) (stor lk-1 (cat pt-6 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk-1 (cat pt-6 k "wrap"))
      (load lk-1 (cat pt-7 k1 "wrap")) (send (enc k k1))))
  (label 907)
  (parent 335)
  (unrealized (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-1))
  (defstrand set-decrypt 2 (cur "wrap") (k k) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (4 2)) ((2 1) (7 0)) ((3 1) (0 0))
    ((4 4) (0 0)) ((5 1) (6 0)) ((5 2) (4 0)) ((6 1) (0 0))
    ((7 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "wrap" "decrypt") (neq "wrap" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-decrypt-at-1 fact-set-decrypt-neq0 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation nonce-test (added-strand set-decrypt 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init")) (ch-msg lk-1 (cat pt-5 k "init"))
    (ch-msg lk-1 (cat pt-6 k "wrap")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-1 (cat pt-4 old-1)) (stor lk-1 (cat pt-5 k "init"))
      (send (hash k)))
    ((load lk-1 (cat pt-5 k "init")) (stor lk-1 (cat pt-6 k "wrap")))
    ((load lk (cat pt-1 k "wrap")) (stor lk (cat pt-7 k "decrypt"))))
  (label 908)
  (parent 335)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-1))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (4 2)) ((3 1) (7 2)) ((4 4) (0 0))
    ((5 1) (6 0)) ((5 2) (4 0)) ((6 1) (0 0)) ((7 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init")) (ch-msg lk-1 (cat pt-5 k "init"))
    (ch-msg lk-1 (cat pt-6 k "wrap")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-1 (cat pt-4 old-1)) (stor lk-1 (cat pt-5 k "init"))
      (send (hash k)))
    ((load lk-1 (cat pt-5 k "init")) (stor lk-1 (cat pt-6 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-7 k1 "wrap")) (send (enc k k1))))
  (label 909)
  (parent 335)
  (unrealized (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-1))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-1))
  (precedes ((1 1) (2 0)) ((2 1) (4 2)) ((3 1) (0 0)) ((4 4) (0 0))
    ((5 1) (6 0)) ((5 1) (7 2)) ((5 2) (4 0)) ((6 1) (0 0))
    ((7 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init")) (ch-msg lk-1 (cat pt-5 k "init"))
    (ch-msg lk-1 (cat pt-6 k "wrap")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-1 (cat pt-4 old-1)) (stor lk-1 (cat pt-5 k "init"))
      (send (hash k)))
    ((load lk-1 (cat pt-5 k "init")) (stor lk-1 (cat pt-6 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk-1 (cat pt-5 k "init"))
      (load lk-1 (cat pt-7 k1 "wrap")) (send (enc k k1))))
  (label 910)
  (parent 335)
  (unrealized (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-1))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (4 2)) ((2 1) (7 2)) ((3 1) (0 0))
    ((4 4) (0 0)) ((5 1) (6 0)) ((5 2) (4 0)) ((6 1) (0 0))
    ((7 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init")) (ch-msg lk-1 (cat pt-5 k "init"))
    (ch-msg lk-1 (cat pt-6 k "wrap")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-1 (cat pt-4 old-1)) (stor lk-1 (cat pt-5 k "init"))
      (send (hash k)))
    ((load lk-1 (cat pt-5 k "init")) (stor lk-1 (cat pt-6 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-7 k1 "wrap")) (send (enc k k1))))
  (label 911)
  (parent 335)
  (unrealized (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-1))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (7 2)) ((2 1) (4 2)) ((3 1) (0 0))
    ((4 4) (0 0)) ((5 1) (6 0)) ((5 2) (4 0)) ((6 1) (0 0))
    ((7 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init")) (ch-msg lk-1 (cat pt-5 k "init"))
    (ch-msg lk-1 (cat pt-6 k "wrap")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-1 (cat pt-4 old-1)) (stor lk-1 (cat pt-5 k "init"))
      (send (hash k)))
    ((load lk-1 (cat pt-5 k "init")) (stor lk-1 (cat pt-6 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-7 k1 "wrap")) (send (enc k k1))))
  (label 912)
  (parent 335)
  (unrealized (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "wrap") (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (4 2)) ((3 1) (6 0)) ((4 4) (0 0))
    ((5 2) (4 0)) ((6 1) (7 0)) ((7 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "wrap" "decrypt") (neq "wrap" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-decrypt-at-1 fact-set-decrypt-neq0 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation nonce-test (added-strand set-decrypt 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init")) (ch-msg lk-0 (cat pt-6 k "wrap"))
    (ch-msg lk-1 (cat pt-5 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-1 (cat pt-4 old-1)) (stor lk-1 (cat pt-5 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-6 k "wrap")))
    ((load lk-0 (cat pt-6 k "wrap"))
      (stor lk-0 (cat pt-7 k "decrypt"))))
  (label 913)
  (parent 336)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "wrap") (k k) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (4 2)) ((2 1) (7 0)) ((3 1) (6 0))
    ((4 4) (0 0)) ((5 2) (4 0)) ((6 1) (0 0)) ((7 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "wrap" "decrypt") (neq "wrap" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule fact-set-decrypt-neq0 trRl_set-wrap-at-1 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation nonce-test (added-strand set-wrap 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init")) (ch-msg lk-0 (cat pt-6 k "wrap"))
    (ch-msg lk-1 (cat pt-5 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-1 (cat pt-4 old-1)) (stor lk-1 (cat pt-5 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-6 k "wrap")))
    ((load lk (cat pt-1 k "wrap")) (stor lk (cat pt-7 k "wrap"))))
  (label 914)
  (parent 336)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 old-2 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval) (k skey)
    (lk lk-0 lk-1 lk-2 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old-2) (k k) (lk lk-2))
  (precedes ((1 1) (2 0)) ((2 1) (4 2)) ((3 1) (6 0)) ((4 4) (0 0))
    ((5 2) (4 0)) ((6 1) (0 0)) ((7 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation nonce-test (added-strand make 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init")) (ch-msg lk-0 (cat pt-6 k "wrap"))
    (ch-msg lk-1 (cat pt-5 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-1 (cat pt-4 old-1)) (stor lk-1 (cat pt-5 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-6 k "wrap")))
    ((load lk-2 (cat pt-7 old-2)) (stor lk-2 (cat pt-8 k "init"))))
  (label 915)
  (parent 336)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "wrap") (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (4 2)) ((3 1) (6 0)) ((4 4) (0 0))
    ((5 2) (4 0)) ((6 1) (7 0)) ((7 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "wrap" "decrypt") (neq "wrap" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule fact-set-decrypt-neq0 trRl_set-wrap-at-1 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation nonce-test (added-strand set-wrap 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init")) (ch-msg lk-0 (cat pt-6 k "wrap"))
    (ch-msg lk-1 (cat pt-5 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-1 (cat pt-4 old-1)) (stor lk-1 (cat pt-5 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-6 k "wrap")))
    ((load lk-0 (cat pt-6 k "wrap")) (stor lk-0 (cat pt-7 k "wrap"))))
  (label 916)
  (parent 336)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-1))
  (precedes ((1 1) (2 0)) ((2 1) (4 2)) ((3 1) (6 0)) ((4 4) (0 0))
    ((5 1) (7 2)) ((5 2) (4 0)) ((6 1) (0 0)) ((7 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init")) (ch-msg lk-0 (cat pt-6 k "wrap"))
    (ch-msg lk-1 (cat pt-5 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-1 (cat pt-4 old-1)) (stor lk-1 (cat pt-5 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-6 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk-1 (cat pt-5 k "init"))
      (load lk-1 (cat pt-7 k1 "wrap")) (send (enc k k1))))
  (label 917)
  (parent 336)
  (unrealized (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "wrap") (k k) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (4 2)) ((2 1) (7 0)) ((3 1) (6 0))
    ((4 4) (0 0)) ((5 2) (4 0)) ((6 1) (0 0)) ((7 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "wrap" "decrypt") (neq "wrap" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-decrypt-at-1 fact-set-decrypt-neq0 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation nonce-test (added-strand set-decrypt 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init")) (ch-msg lk-0 (cat pt-6 k "wrap"))
    (ch-msg lk-1 (cat pt-5 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-1 (cat pt-4 old-1)) (stor lk-1 (cat pt-5 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-6 k "wrap")))
    ((load lk (cat pt-1 k "wrap")) (stor lk (cat pt-7 k "decrypt"))))
  (label 918)
  (parent 336)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (4 2)) ((3 1) (6 0)) ((3 1) (7 2))
    ((4 4) (0 0)) ((5 2) (4 0)) ((6 1) (0 0)) ((7 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init")) (ch-msg lk-0 (cat pt-6 k "wrap"))
    (ch-msg lk-1 (cat pt-5 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-1 (cat pt-4 old-1)) (stor lk-1 (cat pt-5 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-6 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-7 k1 "wrap")) (send (enc k k1))))
  (label 919)
  (parent 336)
  (unrealized (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (4 2)) ((3 1) (6 0)) ((4 4) (0 0))
    ((5 2) (4 0)) ((6 1) (7 2)) ((7 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init")) (ch-msg lk-0 (cat pt-6 k "wrap"))
    (ch-msg lk-1 (cat pt-5 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-1 (cat pt-4 old-1)) (stor lk-1 (cat pt-5 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-6 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-6 k "wrap"))
      (load lk-0 (cat pt-7 k1 "wrap")) (send (enc k k1))))
  (label 920)
  (parent 336)
  (unrealized (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (4 2)) ((2 1) (7 2)) ((3 1) (6 0))
    ((4 4) (0 0)) ((5 2) (4 0)) ((6 1) (0 0)) ((7 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init")) (ch-msg lk-0 (cat pt-6 k "wrap"))
    (ch-msg lk-1 (cat pt-5 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-1 (cat pt-4 old-1)) (stor lk-1 (cat pt-5 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-6 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-7 k1 "wrap")) (send (enc k k1))))
  (label 921)
  (parent 336)
  (unrealized (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (7 2)) ((2 1) (4 2)) ((3 1) (6 0))
    ((4 4) (0 0)) ((5 2) (4 0)) ((6 1) (0 0)) ((7 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init")) (ch-msg lk-0 (cat pt-6 k "wrap"))
    (ch-msg lk-1 (cat pt-5 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-1 (cat pt-4 old-1)) (stor lk-1 (cat pt-5 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-6 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-7 k1 "wrap")) (send (enc k k1))))
  (label 922)
  (parent 336)
  (unrealized (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 old-2 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval) (k k1 skey)
    (lk lk-0 lk-1 lk-2 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-1))
  (defstrand make 3 (old old-2) (k k) (lk lk-2))
  (precedes ((1 1) (2 0)) ((2 1) (4 2)) ((3 1) (0 0)) ((4 4) (0 0))
    ((5 1) (6 2)) ((5 2) (4 0)) ((6 4) (0 0)) ((7 2) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation encryption-test (added-strand make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-1 (cat pt-4 old-1)) (stor lk-1 (cat pt-5 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk-1 (cat pt-5 k "init"))
      (load lk-1 (cat pt-6 k1 "wrap")) (send (enc k k1)))
    ((load lk-2 (cat pt-7 old-2)) (stor lk-2 (cat pt-8 k "init"))
      (send (hash k))))
  (label 925)
  (parent 338)
  (unrealized (6 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-1))
  (deflistener k)
  (precedes ((1 1) (2 0)) ((2 1) (4 2)) ((3 1) (0 0)) ((4 4) (0 0))
    ((5 1) (6 2)) ((5 2) (4 0)) ((6 4) (0 0)) ((7 1) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (added-listener k) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-1 (cat pt-4 old-1)) (stor lk-1 (cat pt-5 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk-1 (cat pt-5 k "init"))
      (load lk-1 (cat pt-6 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k)))
  (label 926)
  (parent 338)
  (unrealized (6 3) (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 old-2 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval) (k k1 skey)
    (lk lk-0 lk-1 lk-2 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (defstrand make 3 (old old-2) (k k) (lk lk-2))
  (precedes ((1 1) (2 0)) ((2 1) (4 2)) ((3 1) (6 2)) ((4 4) (0 0))
    ((5 2) (4 0)) ((6 4) (0 0)) ((7 2) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation encryption-test (added-strand make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-1 (cat pt-4 old-1)) (stor lk-1 (cat pt-5 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-6 k1 "wrap")) (send (enc k k1)))
    ((load lk-2 (cat pt-7 old-2)) (stor lk-2 (cat pt-8 k "init"))
      (send (hash k))))
  (label 930)
  (parent 340)
  (unrealized (6 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (deflistener k)
  (precedes ((1 1) (2 0)) ((2 1) (4 2)) ((3 1) (6 2)) ((4 4) (0 0))
    ((5 2) (4 0)) ((6 4) (0 0)) ((7 1) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (added-listener k) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-1 (cat pt-4 old-1)) (stor lk-1 (cat pt-5 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-6 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k)))
  (label 931)
  (parent 340)
  (unrealized (6 3) (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 old-2 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval) (k k1 skey)
    (lk lk-0 lk-1 lk-2 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old-2) (k k) (lk lk-2))
  (precedes ((1 1) (2 0)) ((2 1) (4 2)) ((2 1) (6 2)) ((3 1) (0 0))
    ((4 4) (0 0)) ((5 2) (4 0)) ((6 4) (0 0)) ((7 2) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation encryption-test (added-strand make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-1 (cat pt-4 old-1)) (stor lk-1 (cat pt-5 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-6 k1 "wrap")) (send (enc k k1)))
    ((load lk-2 (cat pt-7 old-2)) (stor lk-2 (cat pt-8 k "init"))
      (send (hash k))))
  (label 934)
  (parent 341)
  (unrealized (6 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (precedes ((1 1) (2 0)) ((2 1) (4 2)) ((2 1) (6 2)) ((3 1) (0 0))
    ((4 4) (0 0)) ((5 2) (4 0)) ((6 4) (0 0)) ((7 1) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (added-listener k) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-1 (cat pt-4 old-1)) (stor lk-1 (cat pt-5 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-6 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k)))
  (label 935)
  (parent 341)
  (unrealized (6 3) (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 old-2 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval) (k k1 skey)
    (lk lk-0 lk-1 lk-2 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old-2) (k k) (lk lk-2))
  (precedes ((1 1) (2 0)) ((1 1) (6 2)) ((2 1) (4 2)) ((3 1) (0 0))
    ((4 4) (0 0)) ((5 2) (4 0)) ((6 4) (0 0)) ((7 2) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation encryption-test (added-strand make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-1 (cat pt-4 old-1)) (stor lk-1 (cat pt-5 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-6 k1 "wrap")) (send (enc k k1)))
    ((load lk-2 (cat pt-7 old-2)) (stor lk-2 (cat pt-8 k "init"))
      (send (hash k))))
  (label 938)
  (parent 342)
  (unrealized (6 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (precedes ((1 1) (2 0)) ((1 1) (6 2)) ((2 1) (4 2)) ((3 1) (0 0))
    ((4 4) (0 0)) ((5 2) (4 0)) ((6 4) (0 0)) ((7 1) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (added-listener k) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-1 (cat pt-4 old-1)) (stor lk-1 (cat pt-5 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-6 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k)))
  (label 939)
  (parent 342)
  (unrealized (6 3) (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "wrap") (k k) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (3 0)) ((2 1) (4 2)) ((3 1) (6 0))
    ((4 4) (0 0)) ((5 2) (4 0)) ((6 1) (4 3)) ((6 1) (7 0))
    ((7 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "wrap" "decrypt") (neq "wrap" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-decrypt-at-1 fact-set-decrypt-neq0 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation nonce-test (added-strand set-decrypt 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk (cat pt-3 k "init")) (ch-msg lk (cat pt-4 k "wrap"))
    (ch-msg lk-0 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-4 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-5 old-1)) (stor lk-0 (cat pt-6 k "init"))
      (send (hash k)))
    ((load lk (cat pt-3 k "init")) (stor lk (cat pt-4 k "wrap")))
    ((load lk (cat pt-4 k "wrap")) (stor lk (cat pt-7 k "decrypt"))))
  (label 940)
  (parent 343)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (3 0)) ((2 1) (4 2)) ((3 1) (6 0))
    ((4 4) (0 0)) ((5 1) (7 0)) ((5 2) (4 0)) ((6 1) (4 3))
    ((7 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-decrypt-at-1 trRl_make-at-0)
  (operation nonce-test (added-strand set-decrypt 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk (cat pt-3 k "init")) (ch-msg lk (cat pt-4 k "wrap"))
    (ch-msg lk-0 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-4 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-5 old-1)) (stor lk-0 (cat pt-6 k "init"))
      (send (hash k)))
    ((load lk (cat pt-3 k "init")) (stor lk (cat pt-4 k "wrap")))
    ((load lk-0 (cat pt-6 k "init"))
      (stor lk-0 (cat pt-7 k "decrypt"))))
  (label 941)
  (parent 343)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (3 0)) ((2 1) (4 2)) ((3 1) (6 0))
    ((4 4) (0 0)) ((5 1) (7 0)) ((5 2) (4 0)) ((6 1) (4 3))
    ((7 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-wrap-at-1 trRl_make-at-0)
  (operation nonce-test (added-strand set-wrap 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk (cat pt-3 k "init")) (ch-msg lk (cat pt-4 k "wrap"))
    (ch-msg lk-0 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-4 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-5 old-1)) (stor lk-0 (cat pt-6 k "init"))
      (send (hash k)))
    ((load lk (cat pt-3 k "init")) (stor lk (cat pt-4 k "wrap")))
    ((load lk-0 (cat pt-6 k "init")) (stor lk-0 (cat pt-7 k "wrap"))))
  (label 942)
  (parent 343)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 old-2 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval) (k skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-2) (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((2 1) (3 0)) ((2 1) (4 2)) ((3 1) (6 0))
    ((4 4) (0 0)) ((5 2) (4 0)) ((6 1) (4 3)) ((7 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation nonce-test (added-strand make 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk (cat pt-3 k "init")) (ch-msg lk (cat pt-4 k "wrap"))
    (ch-msg lk-0 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-4 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-5 old-1)) (stor lk-0 (cat pt-6 k "init"))
      (send (hash k)))
    ((load lk (cat pt-3 k "init")) (stor lk (cat pt-4 k "wrap")))
    ((load lk-1 (cat pt-7 old-2)) (stor lk-1 (cat pt-8 k "init"))))
  (label 943)
  (parent 343)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand set-wrap 2 (cur "wrap") (k k) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (3 0)) ((2 1) (4 2)) ((3 1) (6 0))
    ((4 4) (0 0)) ((5 2) (4 0)) ((6 1) (4 3)) ((6 1) (7 0))
    ((7 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "wrap" "decrypt") (neq "wrap" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule fact-set-decrypt-neq0 trRl_set-wrap-at-1 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation nonce-test (added-strand set-wrap 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk (cat pt-3 k "init")) (ch-msg lk (cat pt-4 k "wrap"))
    (ch-msg lk-0 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-4 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-5 old-1)) (stor lk-0 (cat pt-6 k "init"))
      (send (hash k)))
    ((load lk (cat pt-3 k "init")) (stor lk (cat pt-4 k "wrap")))
    ((load lk (cat pt-4 k "wrap")) (stor lk (cat pt-7 k "wrap"))))
  (label 944)
  (parent 343)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (3 0)) ((2 1) (4 2)) ((3 1) (6 0))
    ((4 4) (0 0)) ((5 1) (7 2)) ((5 2) (4 0)) ((6 1) (4 3))
    ((7 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk (cat pt-3 k "init")) (ch-msg lk (cat pt-4 k "wrap"))
    (ch-msg lk-0 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-4 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-5 old-1)) (stor lk-0 (cat pt-6 k "init"))
      (send (hash k)))
    ((load lk (cat pt-3 k "init")) (stor lk (cat pt-4 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-6 k "init"))
      (load lk-0 (cat pt-7 k1 "wrap")) (send (enc k k1))))
  (label 945)
  (parent 343)
  (unrealized (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (3 0)) ((2 1) (4 2)) ((3 1) (6 0))
    ((3 1) (7 2)) ((4 4) (0 0)) ((5 2) (4 0)) ((6 1) (4 3))
    ((7 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk (cat pt-3 k "init")) (ch-msg lk (cat pt-4 k "wrap"))
    (ch-msg lk-0 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-4 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-5 old-1)) (stor lk-0 (cat pt-6 k "init"))
      (send (hash k)))
    ((load lk (cat pt-3 k "init")) (stor lk (cat pt-4 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-3 k "init"))
      (load lk (cat pt-7 k1 "wrap")) (send (enc k k1))))
  (label 946)
  (parent 343)
  (unrealized (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (3 0)) ((2 1) (4 2)) ((3 1) (6 0))
    ((4 4) (0 0)) ((5 2) (4 0)) ((6 1) (4 3)) ((6 1) (7 2))
    ((7 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk (cat pt-3 k "init")) (ch-msg lk (cat pt-4 k "wrap"))
    (ch-msg lk-0 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-4 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-5 old-1)) (stor lk-0 (cat pt-6 k "init"))
      (send (hash k)))
    ((load lk (cat pt-3 k "init")) (stor lk (cat pt-4 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-4 k "wrap"))
      (load lk (cat pt-7 k1 "wrap")) (send (enc k k1))))
  (label 947)
  (parent 343)
  (unrealized (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (3 0)) ((2 1) (4 2)) ((2 1) (7 2))
    ((3 1) (6 0)) ((4 4) (0 0)) ((5 2) (4 0)) ((6 1) (4 3))
    ((7 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk (cat pt-3 k "init")) (ch-msg lk (cat pt-4 k "wrap"))
    (ch-msg lk-0 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-4 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-5 old-1)) (stor lk-0 (cat pt-6 k "init"))
      (send (hash k)))
    ((load lk (cat pt-3 k "init")) (stor lk (cat pt-4 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-7 k1 "wrap")) (send (enc k k1))))
  (label 948)
  (parent 343)
  (unrealized (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (7 2)) ((2 1) (3 0)) ((2 1) (4 2))
    ((3 1) (6 0)) ((4 4) (0 0)) ((5 2) (4 0)) ((6 1) (4 3))
    ((7 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk (cat pt-3 k "init")) (ch-msg lk (cat pt-4 k "wrap"))
    (ch-msg lk-0 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk (cat pt-2 old-0)) (stor lk (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-4 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-5 old-1)) (stor lk-0 (cat pt-6 k "init"))
      (send (hash k)))
    ((load lk (cat pt-3 k "init")) (stor lk (cat pt-4 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-7 k1 "wrap")) (send (enc k k1))))
  (label 949)
  (parent 343)
  (unrealized (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (4 2)) ((3 1) (0 0))
    ((4 4) (0 0)) ((5 1) (4 0)) ((6 1) (7 0)) ((7 1) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-wrap-at-1 trRl_make-at-0)
  (operation nonce-test (added-strand set-wrap 2) k (5 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk-1 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init")))
    ((load lk-1 (cat pt-6 k "init")) (stor lk-1 (cat pt-7 k "wrap"))))
  (label 953)
  (parent 346)
  (unrealized (4 3) (5 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (4 2)) ((3 1) (0 0))
    ((4 4) (0 0)) ((5 1) (4 0)) ((6 1) (7 0)) ((7 1) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-decrypt-at-1 trRl_make-at-0)
  (operation nonce-test (added-strand set-decrypt 2) k (5 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk-1 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init")))
    ((load lk-1 (cat pt-6 k "init"))
      (stor lk-1 (cat pt-7 k "decrypt"))))
  (label 954)
  (parent 346)
  (unrealized (4 3) (5 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 k1-0 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk-1))
  (precedes ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (4 2)) ((3 1) (0 0))
    ((4 4) (0 0)) ((5 1) (4 0)) ((6 1) (7 2)) ((7 4) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap")
    (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (5 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk-1 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-1 (cat pt-6 k "init"))
      (load lk-1 (cat pt-7 k1-0 "wrap")) (send (enc k k1-0))))
  (label 955)
  (parent 346)
  (unrealized (4 3) (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 k1-0 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (7 2)) ((2 1) (4 2)) ((3 1) (0 0))
    ((4 4) (0 0)) ((5 1) (4 0)) ((6 1) (5 0)) ((7 4) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap")
    (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (5 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk-1 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-7 k1-0 "wrap")) (send (enc k k1-0))))
  (label 956)
  (parent 346)
  (unrealized (4 3) (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 k1-0 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((1 1) (6 2)) ((2 1) (4 2)) ((3 1) (0 0))
    ((4 4) (0 0)) ((5 1) (4 0)) ((6 4) (5 0)) ((7 2) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap")
    (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation encryption-test (added-strand make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((recv (hash k)) (recv (hash k1-0)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-5 k1-0 "wrap")) (send (enc k k1-0)))
    ((load lk-1 (cat pt-6 old-1)) (stor lk-1 (cat pt-7 k "init"))
      (send (hash k))))
  (label 958)
  (parent 347)
  (unrealized (4 3) (6 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk))
  (deflistener k)
  (precedes ((1 1) (2 0)) ((1 1) (6 2)) ((2 1) (4 2)) ((3 1) (0 0))
    ((4 4) (0 0)) ((5 1) (4 0)) ((6 4) (5 0)) ((7 1) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap")
    (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (added-listener k) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((recv (hash k)) (recv (hash k1-0)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-5 k1-0 "wrap")) (send (enc k k1-0)))
    ((recv k) (send k)))
  (label 959)
  (parent 347)
  (unrealized (4 3) (6 3) (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-1))
  (defstrand set-decrypt 2 (cur "decrypt") (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((1 1) (4 2)) ((2 1) (4 3)) ((3 1) (0 0))
    ((4 4) (0 0)) ((5 1) (6 0)) ((5 2) (4 0)) ((6 1) (7 0))
    ((7 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k "wrap"))
  (facts (neq "decrypt" "decrypt") (neq "decrypt" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-decrypt-at-1 fact-set-decrypt-neq0 gen-st-set-wrap-0
    fact-set-wrap-neq0 trRl_make-at-0)
  (operation nonce-test (added-strand set-decrypt 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init")) (ch-msg lk-1 (cat pt-5 k "init"))
    (ch-msg lk-1 (cat pt-6 k "decrypt")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-1 (cat pt-4 old-1)) (stor lk-1 (cat pt-5 k "init"))
      (send (hash k)))
    ((load lk-1 (cat pt-5 k "init")) (stor lk-1 (cat pt-6 k "decrypt")))
    ((load lk-1 (cat pt-6 k "decrypt"))
      (stor lk-1 (cat pt-7 k "decrypt"))))
  (label 970)
  (parent 357)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-1))
  (defstrand set-wrap 2 (cur "wrap") (k k) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (4 2)) ((2 1) (4 3)) ((2 1) (7 0))
    ((3 1) (0 0)) ((4 4) (0 0)) ((5 1) (6 0)) ((5 2) (4 0))
    ((6 1) (0 0)) ((7 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "wrap" "decrypt") (neq "wrap" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule fact-set-decrypt-neq0 trRl_set-wrap-at-1 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation nonce-test (added-strand set-wrap 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init")) (ch-msg lk-1 (cat pt-5 k "init"))
    (ch-msg lk-1 (cat pt-6 k "decrypt")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-1 (cat pt-4 old-1)) (stor lk-1 (cat pt-5 k "init"))
      (send (hash k)))
    ((load lk-1 (cat pt-5 k "init")) (stor lk-1 (cat pt-6 k "decrypt")))
    ((load lk (cat pt-1 k "wrap")) (stor lk (cat pt-7 k "wrap"))))
  (label 971)
  (parent 357)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-1))
  (defstrand set-wrap 2 (cur "decrypt") (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((1 1) (4 2)) ((2 1) (4 3)) ((3 1) (0 0))
    ((4 4) (0 0)) ((5 1) (6 0)) ((5 2) (4 0)) ((6 1) (7 0))
    ((7 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k "wrap"))
  (facts (neq "decrypt" "decrypt") (neq "decrypt" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule fact-set-decrypt-neq0 trRl_set-wrap-at-1 gen-st-set-wrap-0
    fact-set-wrap-neq0 trRl_make-at-0)
  (operation nonce-test (added-strand set-wrap 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init")) (ch-msg lk-1 (cat pt-5 k "init"))
    (ch-msg lk-1 (cat pt-6 k "decrypt")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-1 (cat pt-4 old-1)) (stor lk-1 (cat pt-5 k "init"))
      (send (hash k)))
    ((load lk-1 (cat pt-5 k "init")) (stor lk-1 (cat pt-6 k "decrypt")))
    ((load lk-1 (cat pt-6 k "decrypt"))
      (stor lk-1 (cat pt-7 k "wrap"))))
  (label 972)
  (parent 357)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-1))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-1))
  (precedes ((1 1) (2 0)) ((1 1) (4 2)) ((2 1) (4 3)) ((3 1) (0 0))
    ((4 4) (0 0)) ((5 1) (6 0)) ((5 1) (7 2)) ((5 2) (4 0))
    ((6 1) (0 0)) ((7 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init")) (ch-msg lk-1 (cat pt-5 k "init"))
    (ch-msg lk-1 (cat pt-6 k "decrypt")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-1 (cat pt-4 old-1)) (stor lk-1 (cat pt-5 k "init"))
      (send (hash k)))
    ((load lk-1 (cat pt-5 k "init")) (stor lk-1 (cat pt-6 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk-1 (cat pt-5 k "init"))
      (load lk-1 (cat pt-7 k1 "wrap")) (send (enc k k1))))
  (label 973)
  (parent 357)
  (unrealized (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-1))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1) (lk lk-1))
  (precedes ((1 1) (2 0)) ((1 1) (4 2)) ((2 1) (4 3)) ((3 1) (0 0))
    ((4 4) (0 0)) ((5 1) (6 0)) ((5 2) (4 0)) ((6 1) (7 2))
    ((7 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k "wrap")
    (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0 gen-st-wrap-1)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init")) (ch-msg lk-1 (cat pt-5 k "init"))
    (ch-msg lk-1 (cat pt-6 k "decrypt")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-1 (cat pt-4 old-1)) (stor lk-1 (cat pt-5 k "init"))
      (send (hash k)))
    ((load lk-1 (cat pt-5 k "init")) (stor lk-1 (cat pt-6 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk-1 (cat pt-6 k "decrypt"))
      (load lk-1 (cat pt-7 k1 "wrap")) (send (enc k k1))))
  (label 974)
  (parent 357)
  (unrealized (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-1))
  (defstrand set-decrypt 2 (cur "wrap") (k k) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (4 2)) ((2 1) (4 3)) ((2 1) (7 0))
    ((3 1) (0 0)) ((4 4) (0 0)) ((5 1) (6 0)) ((5 2) (4 0))
    ((6 1) (0 0)) ((7 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "wrap" "decrypt") (neq "wrap" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-decrypt-at-1 fact-set-decrypt-neq0 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation nonce-test (added-strand set-decrypt 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init")) (ch-msg lk-1 (cat pt-5 k "init"))
    (ch-msg lk-1 (cat pt-6 k "decrypt")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-1 (cat pt-4 old-1)) (stor lk-1 (cat pt-5 k "init"))
      (send (hash k)))
    ((load lk-1 (cat pt-5 k "init")) (stor lk-1 (cat pt-6 k "decrypt")))
    ((load lk (cat pt-1 k "wrap")) (stor lk (cat pt-7 k "decrypt"))))
  (label 975)
  (parent 357)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-1))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (4 2)) ((2 1) (4 3)) ((2 1) (7 2))
    ((3 1) (0 0)) ((4 4) (0 0)) ((5 1) (6 0)) ((5 2) (4 0))
    ((6 1) (0 0)) ((7 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init")) (ch-msg lk-1 (cat pt-5 k "init"))
    (ch-msg lk-1 (cat pt-6 k "decrypt")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-1 (cat pt-4 old-1)) (stor lk-1 (cat pt-5 k "init"))
      (send (hash k)))
    ((load lk-1 (cat pt-5 k "init")) (stor lk-1 (cat pt-6 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-7 k1 "wrap")) (send (enc k k1))))
  (label 976)
  (parent 357)
  (unrealized (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-1))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (precedes ((1 1) (2 0)) ((1 1) (4 2)) ((2 1) (4 3)) ((3 1) (7 2))
    ((4 4) (0 0)) ((5 1) (6 0)) ((5 2) (4 0)) ((6 1) (0 0))
    ((7 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init")) (ch-msg lk-1 (cat pt-5 k "init"))
    (ch-msg lk-1 (cat pt-6 k "decrypt")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-1 (cat pt-4 old-1)) (stor lk-1 (cat pt-5 k "init"))
      (send (hash k)))
    ((load lk-1 (cat pt-5 k "init")) (stor lk-1 (cat pt-6 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-7 k1 "wrap")) (send (enc k k1))))
  (label 977)
  (parent 357)
  (unrealized (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-1))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (4 2)) ((1 1) (7 2)) ((2 1) (4 3))
    ((3 1) (0 0)) ((4 4) (0 0)) ((5 1) (6 0)) ((5 2) (4 0))
    ((6 1) (0 0)) ((7 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init")) (ch-msg lk-1 (cat pt-5 k "init"))
    (ch-msg lk-1 (cat pt-6 k "decrypt")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-1 (cat pt-4 old-1)) (stor lk-1 (cat pt-5 k "init"))
      (send (hash k)))
    ((load lk-1 (cat pt-5 k "init")) (stor lk-1 (cat pt-6 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-7 k1 "wrap")) (send (enc k k1))))
  (label 978)
  (parent 357)
  (unrealized (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-1))
  (defstrand decrypt 4 (x k) (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((1 1) (4 2)) ((2 1) (4 3)) ((3 1) (0 0))
    ((4 4) (0 0)) ((5 1) (6 0)) ((5 2) (4 0)) ((6 1) (7 2))
    ((7 3) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-decrypt-0)
  (operation nonce-test (added-strand decrypt 4) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init")) (ch-msg lk-1 (cat pt-5 k "init"))
    (ch-msg lk-1 (cat pt-6 k "decrypt")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-1 (cat pt-4 old-1)) (stor lk-1 (cat pt-5 k "init"))
      (send (hash k)))
    ((load lk-1 (cat pt-5 k "init")) (stor lk-1 (cat pt-6 k "decrypt")))
    ((recv (enc k k)) (recv (hash k)) (load lk-1 (cat pt-6 k "decrypt"))
      (send k)))
  (label 979)
  (parent 357)
  (unrealized (7 0) (7 1))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-1))
  (defstrand set-decrypt 2 (cur "wrap") (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((1 1) (4 2)) ((2 1) (4 3)) ((3 1) (0 0))
    ((4 4) (0 0)) ((5 1) (6 0)) ((5 2) (4 0)) ((6 1) (7 0))
    ((7 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "wrap" "decrypt") (neq "wrap" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-decrypt-at-1 fact-set-decrypt-neq0 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation nonce-test (added-strand set-decrypt 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init")) (ch-msg lk-1 (cat pt-5 k "init"))
    (ch-msg lk-1 (cat pt-6 k "wrap")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-1 (cat pt-4 old-1)) (stor lk-1 (cat pt-5 k "init"))
      (send (hash k)))
    ((load lk-1 (cat pt-5 k "init")) (stor lk-1 (cat pt-6 k "wrap")))
    ((load lk-1 (cat pt-6 k "wrap"))
      (stor lk-1 (cat pt-7 k "decrypt"))))
  (label 980)
  (parent 358)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-1))
  (defstrand set-wrap 2 (cur "wrap") (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((1 1) (4 2)) ((2 1) (4 3)) ((3 1) (0 0))
    ((4 4) (0 0)) ((5 1) (6 0)) ((5 2) (4 0)) ((6 1) (7 0))
    ((7 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "wrap" "decrypt") (neq "wrap" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule fact-set-decrypt-neq0 trRl_set-wrap-at-1 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation nonce-test (added-strand set-wrap 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init")) (ch-msg lk-1 (cat pt-5 k "init"))
    (ch-msg lk-1 (cat pt-6 k "wrap")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-1 (cat pt-4 old-1)) (stor lk-1 (cat pt-5 k "init"))
      (send (hash k)))
    ((load lk-1 (cat pt-5 k "init")) (stor lk-1 (cat pt-6 k "wrap")))
    ((load lk-1 (cat pt-6 k "wrap")) (stor lk-1 (cat pt-7 k "wrap"))))
  (label 981)
  (parent 358)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-1))
  (defstrand set-wrap 2 (cur "wrap") (k k) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (4 2)) ((2 1) (4 3)) ((2 1) (7 0))
    ((3 1) (0 0)) ((4 4) (0 0)) ((5 1) (6 0)) ((5 2) (4 0))
    ((6 1) (0 0)) ((7 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "wrap" "decrypt") (neq "wrap" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule fact-set-decrypt-neq0 trRl_set-wrap-at-1 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation nonce-test (added-strand set-wrap 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init")) (ch-msg lk-1 (cat pt-5 k "init"))
    (ch-msg lk-1 (cat pt-6 k "wrap")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-1 (cat pt-4 old-1)) (stor lk-1 (cat pt-5 k "init"))
      (send (hash k)))
    ((load lk-1 (cat pt-5 k "init")) (stor lk-1 (cat pt-6 k "wrap")))
    ((load lk (cat pt-1 k "wrap")) (stor lk (cat pt-7 k "wrap"))))
  (label 982)
  (parent 358)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-1))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk-1))
  (precedes ((1 1) (2 0)) ((1 1) (4 2)) ((2 1) (4 3)) ((3 1) (0 0))
    ((4 4) (0 0)) ((5 1) (6 0)) ((5 2) (4 0)) ((6 1) (7 2))
    ((7 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init")) (ch-msg lk-1 (cat pt-5 k "init"))
    (ch-msg lk-1 (cat pt-6 k "wrap")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-1 (cat pt-4 old-1)) (stor lk-1 (cat pt-5 k "init"))
      (send (hash k)))
    ((load lk-1 (cat pt-5 k "init")) (stor lk-1 (cat pt-6 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk-1 (cat pt-6 k "wrap"))
      (load lk-1 (cat pt-7 k1 "wrap")) (send (enc k k1))))
  (label 983)
  (parent 358)
  (unrealized (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-1))
  (defstrand set-decrypt 2 (cur "wrap") (k k) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (4 2)) ((2 1) (4 3)) ((2 1) (7 0))
    ((3 1) (0 0)) ((4 4) (0 0)) ((5 1) (6 0)) ((5 2) (4 0))
    ((6 1) (0 0)) ((7 1) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap"))
  (facts (neq "wrap" "decrypt") (neq "wrap" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-decrypt-at-1 fact-set-decrypt-neq0 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation nonce-test (added-strand set-decrypt 2) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init")) (ch-msg lk-1 (cat pt-5 k "init"))
    (ch-msg lk-1 (cat pt-6 k "wrap")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-1 (cat pt-4 old-1)) (stor lk-1 (cat pt-5 k "init"))
      (send (hash k)))
    ((load lk-1 (cat pt-5 k "init")) (stor lk-1 (cat pt-6 k "wrap")))
    ((load lk (cat pt-1 k "wrap")) (stor lk (cat pt-7 k "decrypt"))))
  (label 984)
  (parent 358)
  (unrealized (0 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-1))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (precedes ((1 1) (2 0)) ((1 1) (4 2)) ((2 1) (4 3)) ((3 1) (7 2))
    ((4 4) (0 0)) ((5 1) (6 0)) ((5 2) (4 0)) ((6 1) (0 0))
    ((7 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init")) (ch-msg lk-1 (cat pt-5 k "init"))
    (ch-msg lk-1 (cat pt-6 k "wrap")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-1 (cat pt-4 old-1)) (stor lk-1 (cat pt-5 k "init"))
      (send (hash k)))
    ((load lk-1 (cat pt-5 k "init")) (stor lk-1 (cat pt-6 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-7 k1 "wrap")) (send (enc k k1))))
  (label 985)
  (parent 358)
  (unrealized (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-1))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-1))
  (precedes ((1 1) (2 0)) ((1 1) (4 2)) ((2 1) (4 3)) ((3 1) (0 0))
    ((4 4) (0 0)) ((5 1) (6 0)) ((5 1) (7 2)) ((5 2) (4 0))
    ((6 1) (0 0)) ((7 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init")) (ch-msg lk-1 (cat pt-5 k "init"))
    (ch-msg lk-1 (cat pt-6 k "wrap")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-1 (cat pt-4 old-1)) (stor lk-1 (cat pt-5 k "init"))
      (send (hash k)))
    ((load lk-1 (cat pt-5 k "init")) (stor lk-1 (cat pt-6 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk-1 (cat pt-5 k "init"))
      (load lk-1 (cat pt-7 k1 "wrap")) (send (enc k k1))))
  (label 986)
  (parent 358)
  (unrealized (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-1))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (4 2)) ((2 1) (4 3)) ((2 1) (7 2))
    ((3 1) (0 0)) ((4 4) (0 0)) ((5 1) (6 0)) ((5 2) (4 0))
    ((6 1) (0 0)) ((7 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init")) (ch-msg lk-1 (cat pt-5 k "init"))
    (ch-msg lk-1 (cat pt-6 k "wrap")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-1 (cat pt-4 old-1)) (stor lk-1 (cat pt-5 k "init"))
      (send (hash k)))
    ((load lk-1 (cat pt-5 k "init")) (stor lk-1 (cat pt-6 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-7 k1 "wrap")) (send (enc k k1))))
  (label 987)
  (parent 358)
  (unrealized (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-1))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (4 2)) ((1 1) (7 2)) ((2 1) (4 3))
    ((3 1) (0 0)) ((4 4) (0 0)) ((5 1) (6 0)) ((5 2) (4 0))
    ((6 1) (0 0)) ((7 4) (0 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (0 0) (enc k k)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk (cat pt-1 k "wrap"))
    (ch-msg lk-0 (cat pt-3 k "init")) (ch-msg lk-1 (cat pt-5 k "init"))
    (ch-msg lk-1 (cat pt-6 k "wrap")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-1 (cat pt-4 old-1)) (stor lk-1 (cat pt-5 k "init"))
      (send (hash k)))
    ((load lk-1 (cat pt-5 k "init")) (stor lk-1 (cat pt-6 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-7 k1 "wrap")) (send (enc k k1))))
  (label 988)
  (parent 358)
  (unrealized (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 old-2 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval) (k k1 skey)
    (lk lk-0 lk-1 lk-2 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-1))
  (defstrand make 3 (old old-2) (k k) (lk lk-2))
  (precedes ((1 1) (2 0)) ((1 1) (4 2)) ((2 1) (4 3)) ((3 1) (0 0))
    ((4 4) (0 0)) ((5 1) (6 2)) ((5 2) (4 0)) ((6 4) (0 0))
    ((7 2) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation encryption-test (added-strand make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-1 (cat pt-4 old-1)) (stor lk-1 (cat pt-5 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk-1 (cat pt-5 k "init"))
      (load lk-1 (cat pt-6 k1 "wrap")) (send (enc k k1)))
    ((load lk-2 (cat pt-7 old-2)) (stor lk-2 (cat pt-8 k "init"))
      (send (hash k))))
  (label 991)
  (parent 360)
  (unrealized (6 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-1))
  (deflistener k)
  (precedes ((1 1) (2 0)) ((1 1) (4 2)) ((2 1) (4 3)) ((3 1) (0 0))
    ((4 4) (0 0)) ((5 1) (6 2)) ((5 2) (4 0)) ((6 4) (0 0))
    ((7 1) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (added-listener k) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-1 (cat pt-4 old-1)) (stor lk-1 (cat pt-5 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk-1 (cat pt-5 k "init"))
      (load lk-1 (cat pt-6 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k)))
  (label 992)
  (parent 360)
  (unrealized (6 3) (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 old-2 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval) (k k1 skey)
    (lk lk-0 lk-1 lk-2 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (defstrand make 3 (old old-2) (k k) (lk lk-2))
  (precedes ((1 1) (2 0)) ((1 1) (4 2)) ((2 1) (4 3)) ((3 1) (6 2))
    ((4 4) (0 0)) ((5 2) (4 0)) ((6 4) (0 0)) ((7 2) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation encryption-test (added-strand make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-1 (cat pt-4 old-1)) (stor lk-1 (cat pt-5 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-6 k1 "wrap")) (send (enc k k1)))
    ((load lk-2 (cat pt-7 old-2)) (stor lk-2 (cat pt-8 k "init"))
      (send (hash k))))
  (label 996)
  (parent 362)
  (unrealized (6 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (deflistener k)
  (precedes ((1 1) (2 0)) ((1 1) (4 2)) ((2 1) (4 3)) ((3 1) (6 2))
    ((4 4) (0 0)) ((5 2) (4 0)) ((6 4) (0 0)) ((7 1) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (added-listener k) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-1 (cat pt-4 old-1)) (stor lk-1 (cat pt-5 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-6 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k)))
  (label 997)
  (parent 362)
  (unrealized (6 3) (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 old-2 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval) (k k1 skey)
    (lk lk-0 lk-1 lk-2 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old-2) (k k) (lk lk-2))
  (precedes ((1 1) (2 0)) ((1 1) (4 2)) ((2 1) (4 3)) ((2 1) (6 2))
    ((3 1) (0 0)) ((4 4) (0 0)) ((5 2) (4 0)) ((6 4) (0 0))
    ((7 2) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation encryption-test (added-strand make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-1 (cat pt-4 old-1)) (stor lk-1 (cat pt-5 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-6 k1 "wrap")) (send (enc k k1)))
    ((load lk-2 (cat pt-7 old-2)) (stor lk-2 (cat pt-8 k "init"))
      (send (hash k))))
  (label 1000)
  (parent 363)
  (unrealized (6 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (precedes ((1 1) (2 0)) ((1 1) (4 2)) ((2 1) (4 3)) ((2 1) (6 2))
    ((3 1) (0 0)) ((4 4) (0 0)) ((5 2) (4 0)) ((6 4) (0 0))
    ((7 1) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (added-listener k) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-1 (cat pt-4 old-1)) (stor lk-1 (cat pt-5 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-6 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k)))
  (label 1001)
  (parent 363)
  (unrealized (6 3) (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 old-2 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval) (k k1 skey)
    (lk lk-0 lk-1 lk-2 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old-2) (k k) (lk lk-2))
  (precedes ((1 1) (2 0)) ((1 1) (4 2)) ((1 1) (6 2)) ((2 1) (4 3))
    ((3 1) (0 0)) ((4 4) (0 0)) ((5 2) (4 0)) ((6 4) (0 0))
    ((7 2) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation encryption-test (added-strand make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-1 (cat pt-4 old-1)) (stor lk-1 (cat pt-5 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-6 k1 "wrap")) (send (enc k k1)))
    ((load lk-2 (cat pt-7 old-2)) (stor lk-2 (cat pt-8 k "init"))
      (send (hash k))))
  (label 1004)
  (parent 364)
  (unrealized (6 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (precedes ((1 1) (2 0)) ((1 1) (4 2)) ((1 1) (6 2)) ((2 1) (4 3))
    ((3 1) (0 0)) ((4 4) (0 0)) ((5 2) (4 0)) ((6 4) (0 0))
    ((7 1) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (added-listener k) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-1 (cat pt-4 old-1)) (stor lk-1 (cat pt-5 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-6 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k)))
  (label 1005)
  (parent 364)
  (unrealized (6 3) (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (0 0)) ((3 1) (0 0))
    ((4 4) (0 0)) ((5 1) (4 0)) ((6 1) (7 0)) ((7 1) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-wrap-at-1 trRl_make-at-0)
  (operation nonce-test (added-strand set-wrap 2) k (5 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk-1 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init")))
    ((load lk-1 (cat pt-6 k "init")) (stor lk-1 (cat pt-7 k "wrap"))))
  (label 1006)
  (parent 366)
  (unrealized (4 3) (5 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (0 0)) ((3 1) (0 0))
    ((4 4) (0 0)) ((5 1) (4 0)) ((6 1) (7 0)) ((7 1) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-decrypt-at-1 trRl_make-at-0)
  (operation nonce-test (added-strand set-decrypt 2) k (5 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk-1 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init")))
    ((load lk-1 (cat pt-6 k "init"))
      (stor lk-1 (cat pt-7 k "decrypt"))))
  (label 1007)
  (parent 366)
  (unrealized (4 3) (5 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 k1-0 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk-1))
  (precedes ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (0 0)) ((3 1) (0 0))
    ((4 4) (0 0)) ((5 1) (4 0)) ((6 1) (7 2)) ((7 4) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (5 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk-1 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-1 (cat pt-6 k "init"))
      (load lk-1 (cat pt-7 k1-0 "wrap")) (send (enc k k1-0))))
  (label 1008)
  (parent 366)
  (unrealized (4 3) (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 k1-0 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (7 2)) ((2 1) (0 0)) ((3 1) (0 0))
    ((4 4) (0 0)) ((5 1) (4 0)) ((6 1) (5 0)) ((7 4) (5 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (5 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk-1 (cat pt-6 k "init")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-7 k1-0 "wrap")) (send (enc k k1-0))))
  (label 1009)
  (parent 366)
  (unrealized (4 3) (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 k1-0 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((1 1) (6 2)) ((2 1) (0 0)) ((3 1) (0 0))
    ((4 4) (0 0)) ((5 1) (4 0)) ((6 4) (5 0)) ((7 2) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation encryption-test (added-strand make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((recv (hash k)) (recv (hash k1-0)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-5 k1-0 "wrap")) (send (enc k k1-0)))
    ((load lk-1 (cat pt-6 old-1)) (stor lk-1 (cat pt-7 k "init"))
      (send (hash k))))
  (label 1011)
  (parent 367)
  (unrealized (4 3) (6 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk))
  (deflistener k)
  (precedes ((1 1) (2 0)) ((1 1) (6 2)) ((2 1) (0 0)) ((3 1) (0 0))
    ((4 4) (0 0)) ((5 1) (4 0)) ((6 4) (5 0)) ((7 1) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k1 "wrap") (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (added-listener k) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((recv (hash k)) (recv (hash k1-0)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-5 k1-0 "wrap")) (send (enc k k1-0)))
    ((recv k) (send k)))
  (label 1012)
  (parent 367)
  (unrealized (4 3) (6 3) (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1) (lk lk-0))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((2 1) (3 2)) ((3 4) (0 0)) ((4 1) (5 0))
    ((4 2) (3 0)) ((5 1) (6 2)) ((6 4) (0 0)) ((7 2) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k "wrap")
    (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation encryption-test (added-strand make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-4 k "decrypt"))
      (load lk-0 (cat pt-5 k1 "wrap")) (send (enc k k1)))
    ((load lk-1 (cat pt-6 old-1)) (stor lk-1 (cat pt-7 k "init"))
      (send (hash k))))
  (label 1025)
  (parent 380)
  (unrealized (6 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1) (lk lk-0))
  (deflistener k)
  (precedes ((1 1) (2 0)) ((2 1) (3 2)) ((3 4) (0 0)) ((4 1) (5 0))
    ((4 2) (3 0)) ((5 1) (6 2)) ((6 4) (0 0)) ((7 1) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k "wrap")
    (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (added-listener k) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-4 k "decrypt"))
      (load lk-0 (cat pt-5 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k)))
  (label 1026)
  (parent 380)
  (unrealized (6 3) (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((2 1) (3 2)) ((2 1) (6 2)) ((3 4) (0 0))
    ((4 1) (5 0)) ((4 2) (3 0)) ((5 1) (0 0)) ((6 4) (0 0))
    ((7 2) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation encryption-test (added-strand make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-5 k1 "wrap")) (send (enc k k1)))
    ((load lk-1 (cat pt-6 old-1)) (stor lk-1 (cat pt-7 k "init"))
      (send (hash k))))
  (label 1029)
  (parent 382)
  (unrealized (6 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (precedes ((1 1) (2 0)) ((2 1) (3 2)) ((2 1) (6 2)) ((3 4) (0 0))
    ((4 1) (5 0)) ((4 2) (3 0)) ((5 1) (0 0)) ((6 4) (0 0))
    ((7 1) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (added-listener k) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-5 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k)))
  (label 1030)
  (parent 382)
  (unrealized (6 3) (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((2 1) (3 2)) ((3 4) (0 0)) ((4 1) (5 0))
    ((4 1) (6 2)) ((4 2) (3 0)) ((5 1) (0 0)) ((6 4) (0 0))
    ((7 2) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation encryption-test (added-strand make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-5 k1 "wrap")) (send (enc k k1)))
    ((load lk-1 (cat pt-6 old-1)) (stor lk-1 (cat pt-7 k "init"))
      (send (hash k))))
  (label 1033)
  (parent 383)
  (unrealized (6 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (deflistener k)
  (precedes ((1 1) (2 0)) ((2 1) (3 2)) ((3 4) (0 0)) ((4 1) (5 0))
    ((4 1) (6 2)) ((4 2) (3 0)) ((5 1) (0 0)) ((6 4) (0 0))
    ((7 1) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (added-listener k) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-5 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k)))
  (label 1034)
  (parent 383)
  (unrealized (6 3) (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((1 1) (6 2)) ((2 1) (3 2)) ((3 4) (0 0))
    ((4 1) (5 0)) ((4 2) (3 0)) ((5 1) (0 0)) ((6 4) (0 0))
    ((7 2) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation encryption-test (added-strand make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-5 k1 "wrap")) (send (enc k k1)))
    ((load lk-1 (cat pt-6 old-1)) (stor lk-1 (cat pt-7 k "init"))
      (send (hash k))))
  (label 1037)
  (parent 384)
  (unrealized (6 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (precedes ((1 1) (2 0)) ((1 1) (6 2)) ((2 1) (3 2)) ((3 4) (0 0))
    ((4 1) (5 0)) ((4 2) (3 0)) ((5 1) (0 0)) ((6 4) (0 0))
    ((7 1) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (added-listener k) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-5 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k)))
  (label 1038)
  (parent 384)
  (unrealized (6 3) (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 cur mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand decrypt 4 (x k) (k k) (lk lk-0))
  (defstrand wrap 5 (cur cur) (k0 k) (k1 k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((2 1) (3 2)) ((3 4) (0 0)) ((4 1) (5 0))
    ((4 2) (3 0)) ((5 1) (6 2)) ((6 3) (0 0)) ((7 4) (6 0)))
  (pen-non-orig k)
  (genStV (cat k cur) (cat k "decrypt") (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-1)
  (operation encryption-test (added-strand wrap 5) (enc k k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((recv (enc k k)) (recv (hash k)) (load lk-0 (cat pt-4 k "decrypt"))
      (send k))
    ((recv (hash k)) (recv (hash k)) (load lk-1 (cat pt-5 k cur))
      (load lk-1 (cat pt-6 k "wrap")) (send (enc k k))))
  (label 1040)
  (parent 385)
  (unrealized (6 1) (7 0) (7 1) (7 2) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand decrypt 4 (x k) (k k) (lk lk-0))
  (deflistener k)
  (precedes ((1 1) (2 0)) ((2 1) (3 2)) ((3 4) (0 0)) ((4 1) (5 0))
    ((4 2) (3 0)) ((5 1) (6 2)) ((6 3) (0 0)) ((7 1) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (added-listener k) (enc k k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((recv (enc k k)) (recv (hash k)) (load lk-0 (cat pt-4 k "decrypt"))
      (send k)) ((recv k) (send k)))
  (label 1041)
  (parent 385)
  (unrealized (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk-0))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((2 1) (3 2)) ((3 4) (0 0)) ((4 1) (5 0))
    ((4 2) (3 0)) ((5 1) (6 2)) ((6 4) (0 0)) ((7 2) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation encryption-test (added-strand make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-4 k "wrap"))
      (load lk-0 (cat pt-5 k1 "wrap")) (send (enc k k1)))
    ((load lk-1 (cat pt-6 old-1)) (stor lk-1 (cat pt-7 k "init"))
      (send (hash k))))
  (label 1044)
  (parent 389)
  (unrealized (6 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk-0))
  (deflistener k)
  (precedes ((1 1) (2 0)) ((2 1) (3 2)) ((3 4) (0 0)) ((4 1) (5 0))
    ((4 2) (3 0)) ((5 1) (6 2)) ((6 4) (0 0)) ((7 1) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (added-listener k) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-4 k "wrap"))
      (load lk-0 (cat pt-5 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k)))
  (label 1045)
  (parent 389)
  (unrealized (6 3) (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((2 1) (3 2)) ((3 4) (0 0)) ((4 1) (5 0))
    ((4 1) (6 2)) ((4 2) (3 0)) ((5 1) (0 0)) ((6 4) (0 0))
    ((7 2) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation encryption-test (added-strand make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-5 k1 "wrap")) (send (enc k k1)))
    ((load lk-1 (cat pt-6 old-1)) (stor lk-1 (cat pt-7 k "init"))
      (send (hash k))))
  (label 1048)
  (parent 391)
  (unrealized (6 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (deflistener k)
  (precedes ((1 1) (2 0)) ((2 1) (3 2)) ((3 4) (0 0)) ((4 1) (5 0))
    ((4 1) (6 2)) ((4 2) (3 0)) ((5 1) (0 0)) ((6 4) (0 0))
    ((7 1) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (added-listener k) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-5 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k)))
  (label 1049)
  (parent 391)
  (unrealized (6 3) (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((2 1) (3 2)) ((2 1) (6 2)) ((3 4) (0 0))
    ((4 1) (5 0)) ((4 2) (3 0)) ((5 1) (0 0)) ((6 4) (0 0))
    ((7 2) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation encryption-test (added-strand make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-5 k1 "wrap")) (send (enc k k1)))
    ((load lk-1 (cat pt-6 old-1)) (stor lk-1 (cat pt-7 k "init"))
      (send (hash k))))
  (label 1052)
  (parent 392)
  (unrealized (6 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (precedes ((1 1) (2 0)) ((2 1) (3 2)) ((2 1) (6 2)) ((3 4) (0 0))
    ((4 1) (5 0)) ((4 2) (3 0)) ((5 1) (0 0)) ((6 4) (0 0))
    ((7 1) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (added-listener k) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-5 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k)))
  (label 1053)
  (parent 392)
  (unrealized (6 3) (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((1 1) (6 2)) ((2 1) (3 2)) ((3 4) (0 0))
    ((4 1) (5 0)) ((4 2) (3 0)) ((5 1) (0 0)) ((6 4) (0 0))
    ((7 2) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation encryption-test (added-strand make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-5 k1 "wrap")) (send (enc k k1)))
    ((load lk-1 (cat pt-6 old-1)) (stor lk-1 (cat pt-7 k "init"))
      (send (hash k))))
  (label 1056)
  (parent 393)
  (unrealized (6 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (precedes ((1 1) (2 0)) ((1 1) (6 2)) ((2 1) (3 2)) ((3 4) (0 0))
    ((4 1) (5 0)) ((4 2) (3 0)) ((5 1) (0 0)) ((6 4) (0 0))
    ((7 1) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (added-listener k) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-5 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k)))
  (label 1057)
  (parent 393)
  (unrealized (6 3) (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 cur mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand set-wrap 2 (cur cur) (k k1) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (3 2)) ((3 4) (0 0)) ((4 1) (5 2))
    ((4 1) (7 0)) ((4 2) (3 0)) ((5 4) (0 0)) ((6 2) (5 0))
    ((7 1) (5 3)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 cur) (cat k1 "wrap"))
  (facts (neq cur "decrypt") (neq cur "wrap") (neq "init" "decrypt")
    (neq "init" "wrap"))
  (rule invShearsRule fact-set-decrypt-neq0 trRl_set-wrap-at-1
    gen-st-set-wrap-0 fact-set-wrap-neq0 trRl_make-at-0)
  (operation channel-test (added-strand set-wrap 2)
    (ch-msg lk-0 (cat pt-4 k1 "wrap")) (5 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-7 k1 cur)) (stor lk-0 (cat pt-4 k1 "wrap"))))
  (label 1065)
  (parent 396)
  (unrealized (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (deflistener k)
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((2 1) (3 2)) ((3 4) (0 0)) ((4 1) (5 2))
    ((4 2) (3 0)) ((5 4) (0 0)) ((6 1) (5 0)) ((7 1) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation nonce-test (added-strand make 2) k (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init"))))
  (label 1068)
  (parent 397)
  (unrealized (5 3) (6 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 cur mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand set-wrap 2 (cur cur) (k k1) (lk lk))
  (precedes ((1 1) (2 0)) ((2 1) (3 2)) ((2 1) (5 2)) ((2 1) (7 0))
    ((3 4) (0 0)) ((4 2) (3 0)) ((5 4) (0 0)) ((6 2) (5 0))
    ((7 1) (5 3)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 cur) (cat k1 "wrap"))
  (facts (neq cur "decrypt") (neq cur "wrap") (neq "init" "decrypt")
    (neq "init" "wrap"))
  (rule invShearsRule fact-set-decrypt-neq0 trRl_set-wrap-at-1
    gen-st-set-wrap-0 fact-set-wrap-neq0 trRl_make-at-0)
  (operation channel-test (added-strand set-wrap 2)
    (ch-msg lk (cat pt-4 k1 "wrap")) (5 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init"))
      (send (hash k)))
    ((load lk (cat pt-7 k1 cur)) (stor lk (cat pt-4 k1 "wrap"))))
  (label 1070)
  (parent 400)
  (unrealized (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((2 1) (3 2)) ((2 1) (5 2)) ((3 4) (0 0))
    ((4 2) (3 0)) ((5 4) (0 0)) ((6 1) (5 0)) ((7 1) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation nonce-test (added-strand make 2) k (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init"))))
  (label 1073)
  (parent 401)
  (unrealized (5 3) (6 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 cur mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (defstrand set-wrap 2 (cur cur) (k k1) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (5 2)) ((2 1) (3 2)) ((2 1) (7 0))
    ((3 4) (0 0)) ((4 2) (3 0)) ((5 4) (0 0)) ((6 2) (5 0))
    ((7 1) (5 3)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 cur) (cat k1 "wrap"))
  (facts (neq cur "decrypt") (neq cur "wrap") (neq "init" "decrypt")
    (neq "init" "wrap"))
  (rule shearsRule invShearsRule fact-set-decrypt-neq0
    trRl_set-wrap-at-1 gen-st-set-wrap-0 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation channel-test (added-strand set-wrap 2)
    (ch-msg lk (cat pt-4 k1 "wrap")) (5 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init"))
      (send (hash k)))
    ((load lk (cat pt-7 k1 cur)) (stor lk (cat pt-4 k1 "wrap"))))
  (label 1079)
  (parent 404)
  (unrealized (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((1 1) (5 2)) ((2 1) (3 2)) ((3 4) (0 0))
    ((4 2) (3 0)) ((5 4) (0 0)) ((6 1) (5 0)) ((7 1) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation nonce-test (added-strand make 2) k (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-4 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-1 (cat pt-5 old-1)) (stor lk-1 (cat pt-6 k "init"))))
  (label 1082)
  (parent 405)
  (unrealized (5 3) (6 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "wrap") (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (3 2)) ((3 4) (0 0))
    ((4 1) (3 0)) ((5 1) (6 0)) ((6 1) (7 0)) ((7 1) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "wrap" "decrypt") (neq "wrap" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-decrypt-at-1 fact-set-decrypt-neq0 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation nonce-test (added-strand set-decrypt 2) k (4 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk-0 (cat pt-4 k "init"))
    (ch-msg lk-0 (cat pt-5 k "wrap")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt-5 k "wrap")))
    ((load lk-0 (cat pt-5 k "wrap"))
      (stor lk-0 (cat pt-6 k "decrypt"))))
  (label 1094)
  (parent 409)
  (unrealized (3 3) (4 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (3 2)) ((3 4) (0 0))
    ((4 1) (3 0)) ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation nonce-test (added-strand make 2) k (4 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk-0 (cat pt-4 k "init"))
    (ch-msg lk-0 (cat pt-5 k "wrap")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt-5 k "wrap")))
    ((load lk-1 (cat pt-6 old-1)) (stor lk-1 (cat pt-7 k "init"))))
  (label 1095)
  (parent 409)
  (unrealized (3 3) (4 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "wrap") (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (3 2)) ((3 4) (0 0))
    ((4 1) (3 0)) ((5 1) (6 0)) ((6 1) (7 0)) ((7 1) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "wrap" "decrypt") (neq "wrap" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule fact-set-decrypt-neq0 trRl_set-wrap-at-1 fact-set-wrap-neq0
    trRl_make-at-0)
  (operation nonce-test (added-strand set-wrap 2) k (4 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk-0 (cat pt-4 k "init"))
    (ch-msg lk-0 (cat pt-5 k "wrap")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt-5 k "wrap")))
    ((load lk-0 (cat pt-5 k "wrap")) (stor lk-0 (cat pt-6 k "wrap"))))
  (label 1096)
  (parent 409)
  (unrealized (3 3) (4 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1-0) (lk lk-0))
  (precedes ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (3 2)) ((3 4) (0 0))
    ((4 1) (3 0)) ((5 1) (6 0)) ((6 1) (7 2)) ((7 4) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap")
    (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (4 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk-0 (cat pt-4 k "init"))
    (ch-msg lk-0 (cat pt-5 k "wrap")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt-5 k "wrap")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-0 (cat pt-5 k "wrap"))
      (load lk-0 (cat pt-6 k1-0 "wrap")) (send (enc k k1-0))))
  (label 1097)
  (parent 409)
  (unrealized (3 3) (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk-0))
  (precedes ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (3 2)) ((3 4) (0 0))
    ((4 1) (3 0)) ((5 1) (6 0)) ((5 1) (7 2)) ((6 1) (4 0))
    ((7 4) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap")
    (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (4 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk-0 (cat pt-4 k "init"))
    (ch-msg lk-0 (cat pt-5 k "wrap")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt-5 k "wrap")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-0 (cat pt-4 k "init"))
      (load lk-0 (cat pt-6 k1-0 "wrap")) (send (enc k k1-0))))
  (label 1098)
  (parent 409)
  (unrealized (3 3) (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (7 2)) ((2 1) (3 2)) ((3 4) (0 0))
    ((4 1) (3 0)) ((5 1) (6 0)) ((6 1) (4 0)) ((7 4) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap")
    (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (4 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk-0 (cat pt-4 k "init"))
    (ch-msg lk-0 (cat pt-5 k "wrap")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt-5 k "wrap")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-6 k1-0 "wrap")) (send (enc k k1-0))))
  (label 1099)
  (parent 409)
  (unrealized (3 3) (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand make 2 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (3 2)) ((3 4) (0 0))
    ((4 1) (3 0)) ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation nonce-test (added-strand make 2) k (4 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk-0 (cat pt-4 k "init"))
    (ch-msg lk-0 (cat pt-5 k "decrypt")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt-5 k "decrypt")))
    ((load lk-1 (cat pt-6 old-1)) (stor lk-1 (cat pt-7 k "init"))))
  (label 1100)
  (parent 410)
  (unrealized (3 3) (4 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "decrypt") (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (3 2)) ((3 4) (0 0))
    ((4 1) (3 0)) ((5 1) (6 0)) ((6 1) (7 0)) ((7 1) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k "wrap")
    (cat k1 "wrap"))
  (facts (neq "decrypt" "decrypt") (neq "decrypt" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule fact-set-decrypt-neq0 trRl_set-wrap-at-1 gen-st-set-wrap-0
    fact-set-wrap-neq0 trRl_make-at-0)
  (operation nonce-test (added-strand set-wrap 2) k (4 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk-0 (cat pt-4 k "init"))
    (ch-msg lk-0 (cat pt-5 k "decrypt")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt-5 k "decrypt")))
    ((load lk-0 (cat pt-5 k "decrypt"))
      (stor lk-0 (cat pt-6 k "wrap"))))
  (label 1101)
  (parent 410)
  (unrealized (3 3) (4 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "decrypt") (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (3 2)) ((3 4) (0 0))
    ((4 1) (3 0)) ((5 1) (6 0)) ((6 1) (7 0)) ((7 1) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k "wrap")
    (cat k1 "wrap"))
  (facts (neq "decrypt" "decrypt") (neq "decrypt" "wrap")
    (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_set-decrypt-at-1 fact-set-decrypt-neq0 gen-st-set-wrap-0
    fact-set-wrap-neq0 trRl_make-at-0)
  (operation nonce-test (added-strand set-decrypt 2) k (4 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk-0 (cat pt-4 k "init"))
    (ch-msg lk-0 (cat pt-5 k "decrypt")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt-5 k "decrypt")))
    ((load lk-0 (cat pt-5 k "decrypt"))
      (stor lk-0 (cat pt-6 k "decrypt"))))
  (label 1102)
  (parent 410)
  (unrealized (3 3) (4 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1-0) (lk lk-0))
  (precedes ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (3 2)) ((3 4) (0 0))
    ((4 1) (3 0)) ((5 1) (6 0)) ((6 1) (7 2)) ((7 4) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k "wrap")
    (cat k1 "wrap") (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0 gen-st-wrap-1)
  (operation nonce-test (added-strand wrap 5) k (4 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk-0 (cat pt-4 k "init"))
    (ch-msg lk-0 (cat pt-5 k "decrypt")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt-5 k "decrypt")))
    ((recv (hash k)) (recv (hash k1-0))
      (load lk-0 (cat pt-5 k "decrypt"))
      (load lk-0 (cat pt-6 k1-0 "wrap")) (send (enc k k1-0))))
  (label 1103)
  (parent 410)
  (unrealized (3 3) (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk-0))
  (precedes ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (3 2)) ((3 4) (0 0))
    ((4 1) (3 0)) ((5 1) (6 0)) ((5 1) (7 2)) ((6 1) (4 0))
    ((7 4) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap")
    (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (4 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk-0 (cat pt-4 k "init"))
    (ch-msg lk-0 (cat pt-5 k "decrypt")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt-5 k "decrypt")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-0 (cat pt-4 k "init"))
      (load lk-0 (cat pt-6 k1-0 "wrap")) (send (enc k k1-0))))
  (label 1104)
  (parent 410)
  (unrealized (3 3) (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk))
  (precedes ((1 1) (2 0)) ((1 1) (7 2)) ((2 1) (3 2)) ((3 4) (0 0))
    ((4 1) (3 0)) ((5 1) (6 0)) ((6 1) (4 0)) ((7 4) (4 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap")
    (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-0)
  (operation nonce-test (added-strand wrap 5) k (4 0)
    (ch-msg lk (cat pt-0 k "init")) (ch-msg lk-0 (cat pt-4 k "init"))
    (ch-msg lk-0 (cat pt-5 k "decrypt")))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((load lk-0 (cat pt-4 k "init")) (stor lk-0 (cat pt-5 k "decrypt")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-6 k1-0 "wrap")) (send (enc k k1-0))))
  (label 1105)
  (parent 410)
  (unrealized (3 3) (7 0) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 k1-0 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk-0))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (3 2)) ((3 4) (0 0))
    ((4 1) (3 0)) ((5 1) (6 2)) ((6 4) (4 0)) ((7 2) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap")
    (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation encryption-test (added-strand make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-0 (cat pt-4 k "init"))
      (load lk-0 (cat pt-5 k1-0 "wrap")) (send (enc k k1-0)))
    ((load lk-1 (cat pt-6 old-1)) (stor lk-1 (cat pt-7 k "init"))
      (send (hash k))))
  (label 1107)
  (parent 411)
  (unrealized (3 3) (6 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk-0))
  (deflistener k)
  (precedes ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (3 2)) ((3 4) (0 0))
    ((4 1) (3 0)) ((5 1) (6 2)) ((6 4) (4 0)) ((7 1) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap")
    (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (added-listener k) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-0 (cat pt-4 k "init"))
      (load lk-0 (cat pt-5 k1-0 "wrap")) (send (enc k k1-0)))
    ((recv k) (send k)))
  (label 1108)
  (parent 411)
  (unrealized (3 3) (6 3) (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 k1-0 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((1 1) (6 2)) ((2 1) (3 2)) ((3 4) (0 0))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 4) (4 0)) ((7 2) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap")
    (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation encryption-test (added-strand make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-5 k1-0 "wrap")) (send (enc k k1-0)))
    ((load lk-1 (cat pt-6 old-1)) (stor lk-1 (cat pt-7 k "init"))
      (send (hash k))))
  (label 1110)
  (parent 412)
  (unrealized (3 3) (6 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk))
  (deflistener k)
  (precedes ((1 1) (2 0)) ((1 1) (6 2)) ((2 1) (3 2)) ((3 4) (0 0))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 4) (4 0)) ((7 1) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap")
    (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (added-listener k) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-3 old-0)) (stor lk-0 (cat pt-4 k "init")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-5 k1-0 "wrap")) (send (enc k k1-0)))
    ((recv k) (send k)))
  (label 1111)
  (parent 412)
  (unrealized (3 3) (6 3) (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 k1-0 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk))
  (deflistener k)
  (defstrand make 2 (old old-0) (k k) (lk lk-0))
  (precedes ((1 1) (2 0)) ((1 1) (5 2)) ((2 1) (3 2)) ((3 4) (0 0))
    ((4 1) (3 0)) ((5 4) (4 0)) ((6 1) (5 0)) ((7 1) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap")
    (cat k1-0 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation nonce-test (added-strand make 2) k (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((recv (hash k)) (recv (hash k1-0)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-3 k1-0 "wrap")) (send (enc k k1-0)))
    ((recv k) (send k))
    ((load lk-0 (cat pt-4 old-0)) (stor lk-0 (cat pt-5 k "init"))))
  (label 1115)
  (parent 415)
  (unrealized (3 3) (5 3) (6 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 cur mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 k1-0 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk-0))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur cur) (k k1-0) (lk lk-0))
  (precedes ((1 1) (2 0)) ((2 1) (3 2)) ((3 4) (0 0)) ((4 1) (3 0))
    ((5 4) (4 0)) ((6 1) (7 0)) ((6 2) (5 0)) ((7 1) (5 3)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap") (cat k1-0 cur)
    (cat k1-0 "wrap"))
  (facts (neq cur "decrypt") (neq cur "wrap") (neq "init" "decrypt")
    (neq "init" "wrap"))
  (rule invShearsRule fact-set-decrypt-neq0 trRl_set-wrap-at-1
    gen-st-set-wrap-0 fact-set-wrap-neq0 trRl_make-at-0)
  (operation channel-test (added-strand set-wrap 2)
    (ch-msg lk-0 (cat pt-3 k1-0 "wrap")) (5 3))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-2 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((recv (hash k)) (recv (hash k1-0)) (load lk-0 (cat pt-5 k "init"))
      (load lk-0 (cat pt-3 k1-0 "wrap")) (send (enc k k1-0)))
    ((load lk-0 (cat pt-4 old-0)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-6 k1-0 cur))
      (stor lk-0 (cat pt-3 k1-0 "wrap"))))
  (label 1117)
  (parent 426)
  (unrealized (3 3) (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 cur mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k k1 k1-0 skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk-0))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1-0) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur cur) (k k1-0) (lk lk))
  (precedes ((1 1) (2 2)) ((2 4) (0 0)) ((3 1) (2 0)) ((4 1) (5 2))
    ((4 1) (7 0)) ((5 4) (3 0)) ((6 1) (1 0)) ((6 2) (5 0))
    ((7 1) (5 3)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap") (cat k1-0 cur)
    (cat k1-0 "wrap"))
  (facts (neq cur "decrypt") (neq cur "wrap") (neq "init" "decrypt")
    (neq "init" "wrap"))
  (rule invShearsRule fact-set-decrypt-neq0 trRl_set-wrap-at-1
    gen-st-set-wrap-0 fact-set-wrap-neq0 trRl_make-at-0)
  (operation channel-test (added-strand set-wrap 2)
    (ch-msg lk (cat pt-3 k1-0 "wrap")) (5 3))
  (traces ((recv k) (send k))
    ((load lk-0 (cat pt-5 k "init")) (stor lk-0 (cat pt k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt k "wrap"))
      (load lk-0 (cat pt-0 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k))
    ((load lk (cat pt-1 old)) (stor lk (cat pt-2 k "init")))
    ((recv (hash k)) (recv (hash k1-0)) (load lk (cat pt-2 k "init"))
      (load lk (cat pt-3 k1-0 "wrap")) (send (enc k k1-0)))
    ((load lk-0 (cat pt-4 old-0)) (stor lk-0 (cat pt-5 k "init"))
      (send (hash k)))
    ((load lk (cat pt-6 k1-0 cur)) (stor lk (cat pt-3 k1-0 "wrap"))))
  (label 1118)
  (parent 427)
  (unrealized (2 3) (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1) (lk lk-0))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((1 1) (3 2)) ((2 1) (3 3)) ((3 4) (0 0))
    ((4 1) (5 0)) ((4 2) (3 0)) ((5 1) (6 2)) ((6 4) (0 0))
    ((7 2) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k "wrap")
    (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation encryption-test (added-strand make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-4 k "decrypt"))
      (load lk-0 (cat pt-5 k1 "wrap")) (send (enc k k1)))
    ((load lk-1 (cat pt-6 old-1)) (stor lk-1 (cat pt-7 k "init"))
      (send (hash k))))
  (label 1129)
  (parent 439)
  (unrealized (6 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "decrypt") (k0 k) (k1 k1) (lk lk-0))
  (deflistener k)
  (precedes ((1 1) (2 0)) ((1 1) (3 2)) ((2 1) (3 3)) ((3 4) (0 0))
    ((4 1) (5 0)) ((4 2) (3 0)) ((5 1) (6 2)) ((6 4) (0 0))
    ((7 1) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k "wrap")
    (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (added-listener k) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-4 k "decrypt"))
      (load lk-0 (cat pt-5 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k)))
  (label 1130)
  (parent 439)
  (unrealized (6 3) (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((1 1) (3 2)) ((2 1) (3 3)) ((2 1) (6 2))
    ((3 4) (0 0)) ((4 1) (5 0)) ((4 2) (3 0)) ((5 1) (0 0))
    ((6 4) (0 0)) ((7 2) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation encryption-test (added-strand make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-5 k1 "wrap")) (send (enc k k1)))
    ((load lk-1 (cat pt-6 old-1)) (stor lk-1 (cat pt-7 k "init"))
      (send (hash k))))
  (label 1133)
  (parent 441)
  (unrealized (6 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (precedes ((1 1) (2 0)) ((1 1) (3 2)) ((2 1) (3 3)) ((2 1) (6 2))
    ((3 4) (0 0)) ((4 1) (5 0)) ((4 2) (3 0)) ((5 1) (0 0))
    ((6 4) (0 0)) ((7 1) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (added-listener k) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-1 k "wrap"))
      (load lk (cat pt-5 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k)))
  (label 1134)
  (parent 441)
  (unrealized (6 3) (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((1 1) (3 2)) ((2 1) (3 3)) ((3 4) (0 0))
    ((4 1) (5 0)) ((4 1) (6 2)) ((4 2) (3 0)) ((5 1) (0 0))
    ((6 4) (0 0)) ((7 2) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation encryption-test (added-strand make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-5 k1 "wrap")) (send (enc k k1)))
    ((load lk-1 (cat pt-6 old-1)) (stor lk-1 (cat pt-7 k "init"))
      (send (hash k))))
  (label 1137)
  (parent 442)
  (unrealized (6 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk-0))
  (deflistener k)
  (precedes ((1 1) (2 0)) ((1 1) (3 2)) ((2 1) (3 3)) ((3 4) (0 0))
    ((4 1) (5 0)) ((4 1) (6 2)) ((4 2) (3 0)) ((5 1) (0 0))
    ((6 4) (0 0)) ((7 1) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (added-listener k) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-3 k "init"))
      (load lk-0 (cat pt-5 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k)))
  (label 1138)
  (parent 442)
  (unrealized (6 3) (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((1 1) (3 2)) ((1 1) (6 2)) ((2 1) (3 3))
    ((3 4) (0 0)) ((4 1) (5 0)) ((4 2) (3 0)) ((5 1) (0 0))
    ((6 4) (0 0)) ((7 2) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation encryption-test (added-strand make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-5 k1 "wrap")) (send (enc k k1)))
    ((load lk-1 (cat pt-6 old-1)) (stor lk-1 (cat pt-7 k "init"))
      (send (hash k))))
  (label 1141)
  (parent 443)
  (unrealized (6 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k1) (lk lk))
  (deflistener k)
  (precedes ((1 1) (2 0)) ((1 1) (3 2)) ((1 1) (6 2)) ((2 1) (3 3))
    ((3 4) (0 0)) ((4 1) (5 0)) ((4 2) (3 0)) ((5 1) (0 0))
    ((6 4) (0 0)) ((7 1) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (added-listener k) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((recv (hash k)) (recv (hash k1)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-5 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k)))
  (label 1142)
  (parent 443)
  (unrealized (6 3) (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 cur mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (k skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand decrypt 4 (x k) (k k) (lk lk-0))
  (defstrand wrap 5 (cur cur) (k0 k) (k1 k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((1 1) (3 2)) ((2 1) (3 3)) ((3 4) (0 0))
    ((4 1) (5 0)) ((4 2) (3 0)) ((5 1) (6 2)) ((6 3) (0 0))
    ((7 4) (6 0)))
  (pen-non-orig k)
  (genStV (cat k cur) (cat k "decrypt") (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule gen-st-wrap-1)
  (operation encryption-test (added-strand wrap 5) (enc k k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((recv (enc k k)) (recv (hash k)) (load lk-0 (cat pt-4 k "decrypt"))
      (send k))
    ((recv (hash k)) (recv (hash k)) (load lk-1 (cat pt-5 k cur))
      (load lk-1 (cat pt-6 k "wrap")) (send (enc k k))))
  (label 1144)
  (parent 444)
  (unrealized (6 1) (7 0) (7 1) (7 2) (7 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (k skey)
    (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-decrypt 2 (cur "init") (k k) (lk lk-0))
  (defstrand decrypt 4 (x k) (k k) (lk lk-0))
  (deflistener k)
  (precedes ((1 1) (2 0)) ((1 1) (3 2)) ((2 1) (3 3)) ((3 4) (0 0))
    ((4 1) (5 0)) ((4 2) (3 0)) ((5 1) (6 2)) ((6 3) (0 0))
    ((7 1) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "decrypt") (cat k "init") (cat k "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (added-listener k) (enc k k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "decrypt")))
    ((recv (enc k k)) (recv (hash k)) (load lk-0 (cat pt-4 k "decrypt"))
      (send k)) ((recv k) (send k)))
  (label 1145)
  (parent 444)
  (unrealized (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 old-1 mesg)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (k k1 skey)
    (lk lk-0 lk-1 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk-0))
  (defstrand make 3 (old old-1) (k k) (lk lk-1))
  (precedes ((1 1) (2 0)) ((1 1) (3 2)) ((2 1) (3 3)) ((3 4) (0 0))
    ((4 1) (5 0)) ((4 2) (3 0)) ((5 1) (6 2)) ((6 4) (0 0))
    ((7 2) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (rule trRl_make-at-0 trRl_make-at-1)
  (operation encryption-test (added-strand make 3) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-4 k "wrap"))
      (load lk-0 (cat pt-5 k1 "wrap")) (send (enc k k1)))
    ((load lk-1 (cat pt-6 old-1)) (stor lk-1 (cat pt-7 k "init"))
      (send (hash k))))
  (label 1148)
  (parent 448)
  (unrealized (6 3))
  (aborted)
  (comment "aborted"))

(defskeleton wrap-decrypt
  (vars (old old-0 mesg) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (k k1 skey) (lk lk-0 locn))
  (deflistener k)
  (defstrand make 2 (old old) (k k) (lk lk))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk))
  (defstrand wrap 5 (cur "init") (k0 k) (k1 k) (lk lk))
  (defstrand make 3 (old old-0) (k k) (lk lk-0))
  (defstrand set-wrap 2 (cur "init") (k k) (lk lk-0))
  (defstrand wrap 5 (cur "wrap") (k0 k) (k1 k1) (lk lk-0))
  (deflistener k)
  (precedes ((1 1) (2 0)) ((1 1) (3 2)) ((2 1) (3 3)) ((3 4) (0 0))
    ((4 1) (5 0)) ((4 2) (3 0)) ((5 1) (6 2)) ((6 4) (0 0))
    ((7 1) (6 0)))
  (pen-non-orig k)
  (genStV (cat k "init") (cat k "wrap") (cat k1 "wrap"))
  (facts (neq "init" "decrypt") (neq "init" "wrap"))
  (operation encryption-test (added-listener k) (hash k) (6 0))
  (traces ((recv k) (send k))
    ((load lk (cat pt old)) (stor lk (cat pt-0 k "init")))
    ((load lk (cat pt-0 k "init")) (stor lk (cat pt-1 k "wrap")))
    ((recv (hash k)) (recv (hash k)) (load lk (cat pt-0 k "init"))
      (load lk (cat pt-1 k "wrap")) (send (enc k k)))
    ((load lk-0 (cat pt-2 old-0)) (stor lk-0 (cat pt-3 k "init"))
      (send (hash k)))
    ((load lk-0 (cat pt-3 k "init")) (stor lk-0 (cat pt-4 k "wrap")))
    ((recv (hash k)) (recv (hash k1)) (load lk-0 (cat pt-4 k "wrap"))
      (load lk-0 (cat pt-5 k1 "wrap")) (send (enc k k1)))
    ((recv k) (send k)))
  (label 1149)
  (parent 448)
  (unrealized (6 3) (7 0))
  (aborted)
  (comment "aborted"))
